#include "stdafx.h"
#include "language.h"

extern "C"
{
#include "FileSystemTree.h"
#include "debug.h"
}
#include "dbtools.h"

#define MAX_KEY_BUFFER	80

extern "C" TCHAR	gWorkingDir[MAX_PATH_LENGTH];

TCHAR	*langIDStr[LANG_MAX_NO] = 
{
	LANG_ENG_ID_STR,
	LANG_CHT_ID_STR,
	LANG_CHS_ID_STR,
	LANG_JP_ID_STR,
	LANG_GM_ID_STR,
	LANG_FR_ID_STR,
	LANG_IT_ID_STR,
	LANG_SP_ID_STR,
	LANG_PO_ID_STR,
	LANG_DU_ID_STR,
	LANG_TU_ID_STR
};

LangMappingArray	langStringMap[LANG_MAX_NO];
int					currentLangID;
int					codepage;
int					parseLangConfigOK = FALSE;

static TCHAR			*startBuffer;

// This function detects a correct initial UI language for all
// platforms (Win9x, ME, NT4, Windows 2000, Windows XP)
LANGID GetLanguageID() {
	OSVERSIONINFO		VersionInfo;
	LANGID				uiLangID = 0;
	HKEY				hKey;
	DWORD				Type, BuffLen = MAX_KEY_BUFFER;
	TCHAR				LangKeyValue[MAX_KEY_BUFFER];

	VersionInfo.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
	if (!GetVersionEx(&VersionInfo))
	{
		return 0;
	}

	switch (VersionInfo.dwPlatformId) 
	{
	case VER_PLATFORM_WIN32_NT:
		// On Windows NT, Windows 2000 or higher
		if (VersionInfo.dwMajorVersion >= 5)   
		{
			// Windows 2000 or higher
			uiLangID = GetUserDefaultUILanguage();
		}
		else 
		{   
			// for NT4 check the language of ntdll.dll
			/*
			uiLangID = GetNTDLLNativeLangID();   
			if (uiLangID == 1033) {		// special processing for Honkong SAR version of NT4
				;
			if (IsHongKongVersion()) {
				uiLangID = 3076;
			}
			}
			*/
			return 0;
		}
		break;
	case VER_PLATFORM_WIN32_WINDOWS:
		// On Windows 95, Windows 98 or Windows ME
		// Open the registry key for the UI language
		if (RegOpenKeyEx(HKEY_CURRENT_USER,_T("Default\\Control Panel\\Desktop\\ResourceLocale"), 0,
			KEY_QUERY_VALUE, &hKey) == ERROR_SUCCESS) 
		{
			// Get the type of the default key
			if ((RegQueryValueEx(hKey, NULL, NULL, &Type, NULL, NULL) == ERROR_SUCCESS) && 
				(Type == REG_SZ)) 
			{
				// Read the key value
				if (RegQueryValueEx(hKey, NULL, NULL, &Type, (LPBYTE)LangKeyValue, &BuffLen) == ERROR_SUCCESS) 
				{
						uiLangID = _ttoi(LangKeyValue);
				}
			}
			RegCloseKey(hKey);
		}				
		break;
	}

	if (uiLangID == 0) {
		uiLangID = GetUserDefaultLangID();
	}

	return (uiLangID);
}

TCHAR	*GetNextLine(TCHAR	*buffer, TCHAR *end)
{
	while (1)
	{
		if (buffer >= end)
		{
			// end of file
			return NULL;
		}

		if (*buffer == _T('\n'))
		{
			break;
		}

		buffer++;
	}

	// skip white space
	while (1)
	{
		if (*buffer == _T(' ') || *buffer == _T('\t') || *buffer == _T('\n') || *buffer == _T('\r'))
		{
			buffer++;

			if (buffer >= end)
			{
				// end of file
				return NULL;
			}
		}
		else
		{
			break;
		}
	}

	return buffer;
}


TCHAR*	QueryLangStr(TCHAR	*key)
{
	if (currentLangID == -1)
	{
		return NULL;
	}

	for (int i = 0; i < langStringMap[currentLangID].strNo; i++)
	{
		if (_tcsicmp(key, langStringMap[currentLangID].pair[i].key) == 0)
		{
			return langStringMap[currentLangID].pair[i].stringValue;
		}
	}

	return NULL;
}

int	ParseLangConfig()
{
	CFile		langFile;
	CString		filePath;
	TCHAR		*buffer, *endBuffer;
	TCHAR		*key, *stringValue;
	LANGID		langID;	
	WORD		primaryLangID, subLangID;
	UINT		fileSize, fileReadCount; 
	int			testLangID = -1;
	//test
	TCHAR	queryResult[255];

	filePath = gWorkingDir;
	filePath +=	LANG_CONFIG_FILE;

	if (GetFileAttributes(filePath) == INVALID_FILE_ATTRIBUTES)
	{
		// Cannot find the language config file in 
		return FALSE;
	}

	startBuffer = NULL;
	try
	{
		if (!langFile.Open(filePath, CFile::modeRead | CFile::shareDenyNone))
			return FALSE;
	
		fileSize = (UINT) langFile.GetLength();
		startBuffer = (TCHAR *) MALLOC(fileSize);
		memset(startBuffer, 0, fileSize);
		fileReadCount = langFile.Read(startBuffer, fileSize);
		endBuffer = (TCHAR *) ((char *) startBuffer + fileReadCount);
	}
	catch (CFileException* pEx)
	{
	   pEx->Delete();
	   if (startBuffer != NULL)
	   {
		   FREE(startBuffer);
	   }
	   return FALSE;
	}

	for (int i = 0; i < LANG_MAX_NO; i++)
	{
		langStringMap[i].strNo = 0;
	}

	currentLangID = -1;
	buffer = startBuffer + 1;			// skip Unicode ID "0xFEFF
	do
	{
		if (*buffer == _T('['))
		{
			// A new language definition
			for (i = 0; i < LANG_MAX_NO; i++)
			{
				if ((_tcsnicmp(buffer, langIDStr[i], 4)) == 0)
				{
					currentLangID = i;
					break;
				}
			}
		}
		else if (*buffer == _T('@'))
		{
			// Test mode, set the language for testing
			buffer++;
			for (i = 0; i < LANG_MAX_NO; i++)
			{
				if ((_tcsnicmp(buffer, langIDStr[i], 4)) == 0)
				{
					testLangID = i;
					break;
				}
			}

		}
		else if ((currentLangID != -1) && (currentLangID < LANG_MAX_NO))
		{
			key = buffer;
			while (buffer <= endBuffer)
			{
				if (*buffer == _T('='))
				{
					break;
				}
				buffer++;
			}

			stringValue = buffer;
			if (stringValue < endBuffer)
			{
				*stringValue++ = _T('\0');	// skip '=' and set end of 'key'

				buffer = stringValue; 
				while (buffer <= endBuffer)
				{
					if (*buffer == _T('\r'))
					{
						*buffer = _T('\0');		// set end of "stringValue"
						break;
					}
					buffer++;
				}

				langStringMap[currentLangID].pair[langStringMap[currentLangID].strNo].key = key;
				langStringMap[currentLangID].pair[langStringMap[currentLangID].strNo].stringValue = stringValue;
				langStringMap[currentLangID].strNo++;
			}
		}

		buffer = GetNextLine(buffer, endBuffer);

	} while (buffer != NULL);

	if (testLangID != -1)
	{
		// Print all strings for selected testing language
		for (int i = 0; i < langStringMap[testLangID].strNo; i++)
		{
			DBGPrintf((_T("Current key value at %s: %s\n"), 
				      langStringMap[testLangID].pair[i].key, langStringMap[testLangID].pair[i].stringValue));
		}
	
		if (langStringMap[testLangID].strNo != 0)
		{
			codepage = 950;
			currentLangID = testLangID;
			return TRUE;
		}
	}		
	
	// We set the default code page is Traditional Chinese
	codepage = 950;
	langID = GetLanguageID();
	//primaryLangID = PRIMARYLANGID(langID);	
	if (db_getAVPT(_T("Language"), queryResult))
	{
		if (_tcsnicmp(queryResult,_T("0x04"), 4) == 0 )
		{
			primaryLangID = 0x04;
		}
		else if (_tcsnicmp(queryResult,_T("0x00"), 4) == 0 )
		{
			primaryLangID = 0x00;
		}
		else if (_tcsnicmp(queryResult,_T("0x0C"), 4) == 0 )
		{
			primaryLangID = 0x0C;
		}		
		else if (_tcsnicmp(queryResult,_T("0x07"), 4) == 0 )
		{
			primaryLangID = 0x07;
		}
		else if (_tcsnicmp(queryResult,_T("0x10"), 4) == 0 )
		{
			primaryLangID = 0x10;
		}
		else if (_tcsnicmp(queryResult,_T("0x11"), 4) == 0 )
		{
			primaryLangID = 0x11;
		}
		else if (_tcsnicmp(queryResult,_T("0x0A"), 4) == 0 )
		{
			primaryLangID = 0x0A;
		}
		else if (_tcsnicmp(queryResult,_T("0x16"), 4) == 0 )
		{
			primaryLangID = 0x16;
		}
		else if (_tcsnicmp(queryResult,_T("0x1f"), 4) == 0 )
		{
			primaryLangID = 0x1f;
		}
		else if (_tcsnicmp(queryResult,_T("0x13"), 4) == 0 )
		{
			primaryLangID = 0x13;
		}
		else
		{
			primaryLangID = PRIMARYLANGID(langID);
		}
	}
	subLangID = SUBLANGID(langID);
	switch (primaryLangID)
	{
	case 0x04:
		// Chinese
		if (subLangID == 0x02)
		{
			// Simplified Chinese
			currentLangID = LANG_CHS_ID;
			codepage = 936;
		}
		else
		{
			// Traditional Chinese
			currentLangID = LANG_CHT_ID;
			codepage = 950;
		}
		break;
	case 0x07:
		// German
		currentLangID = LANG_GM_ID;
		break;
	case 0x0A:
		// Spanish
		currentLangID = LANG_SP_ID;
		break;
	case 0x0C:
		// France
		currentLangID = LANG_FR_ID;
		break;
	case 0x10:
		// Italian 
		currentLangID = LANG_IT_ID;
		break;
	case 0x11:
		// Japanese
		currentLangID = LANG_JP_ID;
		codepage = 932;
		break;
	case 0x16:
		// Portuguese
		currentLangID = LANG_PO_ID;		
		break;
	case 0x1f:
		// Dutch
		currentLangID = LANG_DU_ID;		
		break;
	case 0x13:
		// Turkish
		currentLangID = LANG_TU_ID;		
		break;
	default:
		// others use English
		currentLangID = LANG_ENG_ID;
	}

	if (langStringMap[currentLangID].strNo == 0)
	{
		// Cannot find the language definition, use default UI
		currentLangID = -1;
		return FALSE;
	}
	
	return TRUE;
}

void FreeLangConfigResource()
{
	FREE(startBuffer);
}