#include "stdafx.h"
//******************************************************************************
//
// Microsoft Windows Media
// Copyright ( C) Microsoft Corporation. All rights reserved.
//
// FileName: main.cpp
//
// Abstract: Sample to edit and view metadata from a windows media file.
//
//******************************************************************************

#include <tchar.h>
#include <stdio.h>
#include <string.h>
#include <wmsdk.h>
#include <conio.h>

extern "C"
{
#include "FileSystemTree.h"
#include "mystring.h"
#include "VideoInfo.h"
}

extern int	codepage;

//------------------------------------------------------------------------------
// Macros
// ------------------------------------------------------------------------------
#ifndef SAFE_RELEASE

#define SAFE_RELEASE( x )       \
    if( NULL != x )             \
    {                           \
        x->Release( );          \
        x = NULL;               \
    }

#endif // SAFE_RELEASE

#ifndef SAFE_DELETE

#define SAFE_DELETE( x )        \
    if( NULL != x )             \
    {                           \
        delete x;               \
        x = NULL;               \
    }

#endif // SAFE_DELETE

#ifndef SAFE_ARRAYDELETE

#define SAFE_ARRAYDELETE( x )   \
    if( NULL != x )             \
    {                           \
        delete [] x;            \
        x = NULL;               \
    }

#endif // SAFE_ARRAYDELETE

//------------------------------------------------------------------------------
// Name: EditorOpenFile()
// Desc: Creates metadata editor, opens file, and gets header info interface.
//------------------------------------------------------------------------------
HRESULT EditorOpenFile( WCHAR* pwszInFile, 
                        IWMMetadataEditor ** ppEditor,
                        IWMHeaderInfo ** ppHeaderInfo, 
                        IWMHeaderInfo3 ** ppHeaderInfo3 )
{
    if( ( NULL == pwszInFile ) ||
        ( NULL == ppEditor ) )
    {
        return( E_INVALIDARG );
    }

    HRESULT hr = S_OK;

    do
    {
        hr = WMCreateEditor( ppEditor );
        if( FAILED( hr ) )
        {
            _tprintf( _T( "Could not create Metadata Editor ( hr=0x%08x ).\n" ), hr );
            break;
        }

        hr = ( *ppEditor )->Open( pwszInFile );
        if( FAILED ( hr ) )
        {
            _tprintf( _T( "Could not open the file %ws ( hr=0x%08x ).\n" ), 
                pwszInFile, hr );
            break;
        }

        if( NULL != ppHeaderInfo )
        {
            hr = ( *ppEditor )->QueryInterface( IID_IWMHeaderInfo, 
                                                (void **)ppHeaderInfo );
            if( FAILED( hr ) )
            {
                _tprintf( _T( "Could not QI for IWMHeaderInfo ( hr=0x%08x ).\n" ), hr );
                break;
            }
        }

        if( NULL != ppHeaderInfo3 )
        {
            hr = ( *ppEditor )->QueryInterface( IID_IWMHeaderInfo3, 
                                                (void **)ppHeaderInfo3 );
            if( FAILED( hr ) )
            {
                _tprintf( _T( "Could not QI for IWMHeaderInfo3 ( hr=0x%08x ).\n" ), hr );
                break;
            }
        }
    }
    while( FALSE );

    return( hr );
}

//------------------------------------------------------------------------------
// Name: ModifyAttrib()
// Desc: Changes the value of the specified attribute.
//------------------------------------------------------------------------------
HRESULT ModifyAttrib( WCHAR * pwszInFile, 
                      WORD wStreamNum, 
                      WORD wAttribIndex, 
                      WORD wAttribType, 
                      WCHAR * pwszAttribValue, 
                      WORD wLangIndex )
{
    if( ( NULL == pwszInFile ) ||
        ( NULL == pwszAttribValue ) )
    {
        return( E_INVALIDARG );
    }

    HRESULT hr = S_OK;

    IWMMetadataEditor   * pEditor       = NULL;
    IWMHeaderInfo3      * pHeaderInfo3  = NULL;
    BYTE*               pbAttribValue   = NULL;
    DWORD               dwAttribValueLen = 0;
    WMT_ATTR_DATATYPE   AttribDataType  = (WMT_ATTR_DATATYPE) wAttribType;
    DWORD               dwAttribValue   = 0;
    WORD                wAttribValue    = 0;
    QWORD               qwAttribValue   = 0;
    BOOL                fAttribValue    = 0;

    do
    {
        hr = EditorOpenFile( pwszInFile, &pEditor, NULL, &pHeaderInfo3 );
        if(FAILED( hr ) )
        {
            break;
        }

        switch(AttribDataType )
        {
        case WMT_TYPE_DWORD:
            dwAttribValue = _wtoi( pwszAttribValue );                               
            dwAttribValueLen = sizeof( DWORD );
            pbAttribValue = (BYTE *)&dwAttribValue;

            break;

        case WMT_TYPE_WORD:
            wAttribValue = (WORD)_wtoi( pwszAttribValue );                                
            dwAttribValueLen = sizeof(WORD);
            pbAttribValue = (BYTE *)&wAttribValue;

            break;

        case WMT_TYPE_QWORD:
            qwAttribValue = _wtoi64( pwszAttribValue );                             
            dwAttribValueLen = sizeof(QWORD );
            pbAttribValue = (BYTE *)&qwAttribValue;

            break;
                    
        case WMT_TYPE_STRING:
            dwAttribValueLen = ( wcslen( pwszAttribValue ) + 1 ) * sizeof( WCHAR );
            pbAttribValue = (BYTE *)pwszAttribValue;

            break;

        case WMT_TYPE_BOOL:
            fAttribValue = (BOOL)_wtoi( pwszAttribValue );                              
            dwAttribValueLen = sizeof(BOOL);
            pbAttribValue = (BYTE *)&fAttribValue;

            break;

        default:
            _tprintf( _T( "Unsupported data type for SetAttribute\n" ) );
            hr = E_INVALIDARG;
            break;
        }

        hr = pHeaderInfo3->ModifyAttribute( wStreamNum,
                                            wAttribIndex,
                                            AttribDataType, 
                                            wLangIndex,
                                            pbAttribValue,
                                            dwAttribValueLen );
        if( FAILED( hr ) )
        {
            _tprintf( _T( "ModifyAttribute failed for Attribute index = %d ( hr=0x%08x ).\n" ), 
                wAttribIndex, hr );
            break;
        }

        hr = pEditor->Flush();
        if( FAILED( hr ) )
        {
            _tprintf( _T( "Could not flush the file %ws ( hr=0x%08x ).\n" ), 
                pwszInFile, hr );
            break;
        }

        hr = pEditor->Close();
        if( FAILED( hr ) )
        {
            _tprintf( _T( "Could not close the file %ws ( hr=0x%08x ).\n" ), 
                pwszInFile, hr );
            break;
        }
    }
    while( FALSE );
    
    SAFE_RELEASE( pHeaderInfo3 );
    SAFE_RELEASE( pEditor );

    return( hr );
}

//------------------------------------------------------------------------------
// Name: AddAttrib()
// Desc: Adds an attribute by using IWMHeaderInfo3::AddAttribute.
//------------------------------------------------------------------------------
HRESULT AddAttrib( WCHAR * pwszInFile, 
                   WORD wStreamNum, 
                   WCHAR * pwszAttribName, 
                   WORD wAttribType, 
                   WCHAR * pwszAttribValue, 
                   WORD wLangIndex )
{
    if( ( NULL == pwszInFile ) ||
        ( NULL == pwszAttribName ) ||
        ( NULL == pwszAttribValue ) )
    {
        return( E_INVALIDARG );
    }

    HRESULT hr = S_OK;

    IWMMetadataEditor   * pEditor       = NULL;
    IWMHeaderInfo3      * pHeaderInfo3  = NULL;

    BYTE*               pbAttribValue   = NULL;
    DWORD               dwAttribValueLen = 0;

    WMT_ATTR_DATATYPE   AttribDataType  = ( WMT_ATTR_DATATYPE) wAttribType;

    DWORD               dwAttribValue   = 0;
    WORD                wAttribValue    = 0;
    QWORD               qwAttribValue   = 0;
    BOOL                fAttribValue    = 0;

    do
    {
        hr = EditorOpenFile( pwszInFile, &pEditor, NULL, &pHeaderInfo3 );
        if(FAILED( hr ) )
        {
            break;
        }

        switch( AttribDataType )
        {
        case WMT_TYPE_DWORD:
            dwAttribValue = _wtoi( pwszAttribValue );                               
            dwAttribValueLen = sizeof( DWORD );
            pbAttribValue = (BYTE *)&dwAttribValue;

            break;

        case WMT_TYPE_WORD:
            wAttribValue = (WORD)_wtoi( pwszAttribValue );                                
            dwAttribValueLen = sizeof(WORD);
            pbAttribValue = (BYTE *)&wAttribValue;

            break;

        case WMT_TYPE_QWORD:
            qwAttribValue = _wtoi64( pwszAttribValue );                             
            dwAttribValueLen = sizeof(QWORD );
            pbAttribValue = (BYTE *)&qwAttribValue;

            break;
                    
        case WMT_TYPE_STRING:
            dwAttribValueLen = ( wcslen( pwszAttribValue ) + 1 )* sizeof( WCHAR );
            pbAttribValue = (BYTE *)pwszAttribValue;

            break;

        case WMT_TYPE_BOOL:
            fAttribValue = (BOOL)_wtoi( pwszAttribValue );                              
            dwAttribValueLen = sizeof(BOOL);
            pbAttribValue = (BYTE *)&fAttribValue;

            break;

        default:
            _tprintf( _T( "Unsupported data type for SetAttribute\n" ) );
            hr = E_INVALIDARG;
            break;
        }

        hr = pHeaderInfo3->AddAttribute( wStreamNum,
                                         pwszAttribName,
                                         NULL,
                                         AttribDataType, 
                                         wLangIndex,
                                         pbAttribValue,
                                         dwAttribValueLen );
        if( FAILED( hr ) )
        {
            _tprintf( _T( "AddAttribute failed for Attribute name %ws ( hr=0x%08x ).\n" ), 
                pwszAttribName, hr );
            break;
        }

        hr = pEditor->Flush();
        if( FAILED( hr ) )
        {
            _tprintf( _T( "Could not flush the file %ws ( hr=0x%08x ).\n" ), 
                pwszInFile, hr );
            break;
        }

        hr = pEditor->Close();
        if( FAILED( hr ) )
        {
            _tprintf( _T( "Could not close the file %ws ( hr=0x%08x ).\n" ), 
                pwszInFile, hr );
            break;
        }
    }
    while( FALSE );
    
    SAFE_RELEASE( pHeaderInfo3 );
    SAFE_RELEASE( pEditor );

    return( hr );
}

//------------------------------------------------------------------------------
// Name: SetAttrib()
// Desc: Adds an attribute by using IWMHeaderInfo::SetAttribute.
//------------------------------------------------------------------------------
HRESULT SetAttrib( WCHAR * pwszInFile, 
                   WORD wStreamNum, 
                   WCHAR * pwszAttribName, 
                   WORD wAttribType, 
                   WCHAR * pwszAttribValue )
{
    if( ( NULL == pwszInFile ) ||
        ( NULL == pwszAttribName ) ||
        ( NULL == pwszAttribValue ) )
    {
        return( E_INVALIDARG );
    }

    HRESULT hr = S_OK;

    IWMMetadataEditor   * pEditor       = NULL;
    IWMHeaderInfo       * pHeaderInfo   = NULL;

    BYTE*               pbAttribValue   = NULL;
    WORD                wAttribValueLen = 0;

    WMT_ATTR_DATATYPE   AttribDataType  = ( WMT_ATTR_DATATYPE ) wAttribType;

    DWORD               dwAttribValue   = 0;
    WORD                wAttribValue    = 0;
    QWORD               qwAttribValue   = 0;
    BOOL                fAttribValue    = 0;

    do
    {
        hr = EditorOpenFile( pwszInFile, &pEditor, &pHeaderInfo, NULL);
        if(FAILED( hr ) )
        {
            break;
        }

        switch( AttribDataType )
        {
        case WMT_TYPE_DWORD:
            dwAttribValue = _wtoi( pwszAttribValue );                               
            wAttribValueLen = sizeof( DWORD );
            pbAttribValue = (BYTE *)&dwAttribValue;

            break;

        case WMT_TYPE_WORD:
            wAttribValue = (WORD)_wtoi( pwszAttribValue );                                
            wAttribValueLen = sizeof(WORD);
            pbAttribValue = (BYTE *)&wAttribValue;

            break;

        case WMT_TYPE_QWORD:
            qwAttribValue = _wtoi64( pwszAttribValue );                             
            wAttribValueLen = sizeof(QWORD );
            pbAttribValue = (BYTE *)&qwAttribValue;

            break;
                    
        case WMT_TYPE_STRING:
            wAttribValueLen = ( wcslen( pwszAttribValue ) + 1 )* sizeof( WCHAR );
            pbAttribValue = (BYTE *)pwszAttribValue;

            break;

        case WMT_TYPE_BOOL:
            fAttribValue = (BOOL)_wtoi( pwszAttribValue );                              
            wAttribValueLen = sizeof(BOOL);
            pbAttribValue = (BYTE *)&fAttribValue;

            break;

        default:
            _tprintf( _T( "Unsupported data type for SetAttribute\n" ) );
            hr = E_INVALIDARG;
            break;
        }

        hr = pHeaderInfo->SetAttribute( wStreamNum,
                                        pwszAttribName,
                                        AttribDataType,                                        
                                        pbAttribValue,
                                        wAttribValueLen );
        if( FAILED( hr ) )
        {
            _tprintf( _T( "SetAttribute failed for Attribute name %ws ( hr=0x%08x ).\n" ), 
                pwszAttribName, hr );
            break;
        }

        hr = pEditor->Flush();
        if( FAILED( hr ) )
        {
            _tprintf( _T( "Could not flush the file %ws ( hr=0x%08x ).\n" ), 
                pwszInFile, hr );
            break;
        }

        hr = pEditor->Close();
        if( FAILED( hr ) )
        {
            _tprintf( _T( "Could not close the file %ws ( hr=0x%08x ).\n" ), 
                pwszInFile, hr );
            break;
        }
    }
    while( FALSE );
    
    SAFE_RELEASE( pHeaderInfo );
    SAFE_RELEASE( pEditor );

    return( hr );
}

//------------------------------------------------------------------------------
// Name: DeleteAttrib()
// Desc: Delete the specified attribute.
//------------------------------------------------------------------------------
HRESULT DeleteAttrib( WCHAR * pwszInFile, 
                      WORD wStreamNum, 
                      WORD wAttribIndex )
{
    HRESULT hr = S_OK;
        
    IWMMetadataEditor   * pEditor       = NULL;
    IWMHeaderInfo3      * pHeaderInfo3  = NULL;

    do
    {
        hr = EditorOpenFile( pwszInFile, &pEditor, NULL, &pHeaderInfo3 );
        if(FAILED( hr ) )
        {
            break;
        }

        hr = pHeaderInfo3->DeleteAttribute( wStreamNum, wAttribIndex );
        if( FAILED( hr ) )
        {
            _tprintf( _T( "DeleteAttribute failed for stream = %d and index = %d ( hr=0x%08x ).\n" ), 
                wStreamNum, wAttribIndex, hr );
            break;
        }

        hr = pEditor->Flush();
        if( FAILED( hr ) )
        {
            _tprintf( _T( "Could not flush the file %ws ( hr=0x%08x ).\n" ), 
                pwszInFile, hr );
            break;
        }

        hr = pEditor->Close();
        if( FAILED( hr ) )
        {
            _tprintf( _T( "Could not close the file %ws ( hr=0x%08x ).\n" ), 
                pwszInFile, hr );
            break;
        }
    }
    while( FALSE );
    
    SAFE_RELEASE( pHeaderInfo3 );
    SAFE_RELEASE( pEditor );

    return( hr );
}

//------------------------------------------------------------------------------
// Name: ShowAttributes3()
// Desc: Display all attributes for the specified stream,
//       using IWMHeaderInfo3.
//------------------------------------------------------------------------------
HRESULT ShowAttributes3( WCHAR* pwszInFile, WORD wStreamNum )
{
    HRESULT             hr              = S_OK;

    IWMMetadataEditor   * pEditor       = NULL;
    IWMHeaderInfo3      * pHeaderInfo3  = NULL;

    WCHAR*             pwszAttribName   = NULL;
    WORD               wAttribNameLen   = 0;
    WMT_ATTR_DATATYPE  wAttribType;
    WORD               wLanguageIndex   = 0;
    BYTE*              pbAttribValue    = NULL;
    DWORD              dwAttribValueLen = 0;

    do
    {
        hr = EditorOpenFile( pwszInFile, &pEditor, NULL, &pHeaderInfo3 );
        if(FAILED( hr ) )
            break;

        WORD wAttributeCount = 0;

        hr = pHeaderInfo3->GetAttributeCountEx( wStreamNum, &wAttributeCount);
        if(FAILED( hr ) )
        {
            _tprintf( _T( "GetAttributeCount failed for stream = %d ( hr=0x%08x ).\n" ), wStreamNum, hr );
            break;
        }


        for( WORD wAttribIndex = 0; wAttribIndex < wAttributeCount; wAttribIndex++)
        {
            SAFE_ARRAYDELETE( pwszAttribName );
            SAFE_ARRAYDELETE( pbAttribValue );

            hr = pHeaderInfo3->GetAttributeByIndexEx( wStreamNum,
                                                      wAttribIndex,                                                
                                                      pwszAttribName,
                                                      &wAttribNameLen,
                                                      &wAttribType,
                                                      &wLanguageIndex,
                                                      pbAttribValue,
                                                      &dwAttribValueLen );
            if(FAILED( hr ) )
            {
                _tprintf( _T( "GetAttributeByIndexEx failed for index = %d ( hr=0x%08x ).\n" ), wAttribIndex, hr );
                break;
            }

            pwszAttribName = new WCHAR[wAttribNameLen];
            if( NULL == pwszAttribName )
            {
                hr = E_OUTOFMEMORY;
                break;
            }

            pbAttribValue = new BYTE[dwAttribValueLen];
            if( NULL == pbAttribValue )
            {
                hr = E_OUTOFMEMORY;
                break;
            }

            hr = pHeaderInfo3->GetAttributeByIndexEx( wStreamNum,
                                                      wAttribIndex,                                                
                                                      pwszAttribName,
                                                      &wAttribNameLen,
                                                      &wAttribType,
                                                      &wLanguageIndex,
                                                      pbAttribValue,
                                                      &dwAttribValueLen );
            if(FAILED( hr ) )
            {
                _tprintf( _T( "GetAttributeByIndexEx failed for index = %d ( hr=0x%08x ).\n" ), 
                    wAttribIndex, hr );
                break;
            }

/*
			hr = PrintAttribute( wAttribIndex, 
                                 wStreamNum, 
                                 pwszAttribName, 
                                 wAttribType, 
                                 wLanguageIndex, 
                                 pbAttribValue, 
                                 dwAttribValueLen );         
								 */
            if(FAILED( hr ) )
            {
                break;
            }
        }
        
        hr = pEditor->Close();
        if( FAILED( hr ) )
        {
            _tprintf( _T( "Could not close the file %ws ( hr=0x%08x ).\n" ), pwszInFile, hr );
            break;
        }
    }
    while( FALSE );
    
    SAFE_RELEASE( pHeaderInfo3 );
    SAFE_RELEASE( pEditor );

    SAFE_ARRAYDELETE( pwszAttribName );
    SAFE_ARRAYDELETE( pbAttribValue );

    return( hr );
}

//------------------------------------------------------------------------------
// Name: ShowAttributes()
// Desc: Display all attributes for the specified stream,
//       using IWMHeaderInfo.
//------------------------------------------------------------------------------
HRESULT ShowAttributes( WCHAR * pwszInFile, WORD wStreamNum )
{
    HRESULT             hr                  = S_OK;

    IWMMetadataEditor   * pEditor           = NULL;
    IWMHeaderInfo*      pHeaderInfo         = NULL;

    WCHAR               * pwszAttribName    = NULL;
    WORD                wAttribNameLen      = 0;
    WMT_ATTR_DATATYPE   wAttribType;
    BYTE                * pbAttribValue     = NULL;
    WORD                wAttribValueLen     = 0;

    do
    {
        hr = EditorOpenFile( pwszInFile, &pEditor, &pHeaderInfo, NULL );
        if(FAILED( hr ) )
        {
            break;
        }

        WORD wAttributeCount = 0;

        hr = pHeaderInfo->GetAttributeCount( wStreamNum, &wAttributeCount );
        if(FAILED( hr ) )
        {
            _tprintf( _T( "GetAttributeCount failed for stream = %d ( hr=0x%08x ).\n" ), 
                wStreamNum, hr );
            break;
        }

        for( WORD wAttribIndex = 0; wAttribIndex < wAttributeCount; wAttribIndex++ )
        {
            SAFE_ARRAYDELETE( pwszAttribName );
            SAFE_ARRAYDELETE( pbAttribValue );

            hr = pHeaderInfo->GetAttributeByIndex( wAttribIndex,
                                                   &wStreamNum,
                                                   pwszAttribName,
                                                   &wAttribNameLen,
                                                   &wAttribType,
                                                   pbAttribValue,
                                                   &wAttribValueLen );
            if( FAILED( hr ) )
            {
                _tprintf( _T( "GetAttributeByIndex failed for index = %d ( hr=0x%08x ).\n" ), 
                    wAttribIndex, hr );
                break;
            }

            pwszAttribName = new WCHAR[ wAttribNameLen ];
            if( NULL == pwszAttribName )
            {
                hr = E_OUTOFMEMORY;
                break;
            }

            pbAttribValue = new BYTE[ wAttribValueLen ];
            if( NULL == pbAttribValue )
            {
                hr = E_OUTOFMEMORY;
                break;
            }

            hr = pHeaderInfo->GetAttributeByIndex( wAttribIndex,
                                                   &wStreamNum,
                                                   pwszAttribName,
                                                   &wAttribNameLen,
                                                   &wAttribType,
                                                   pbAttribValue,
                                                   &wAttribValueLen );
            if( FAILED( hr ) )
            {
                _tprintf( _T( "GetAttributeByIndex failed for index = %d ( hr=0x%08x ).\n" ), 
                    wAttribIndex, hr );
                break;
            }

/*
			hr = PrintAttribute( wAttribIndex, 
                                 wStreamNum, 
                                 pwszAttribName, 
                                 wAttribType, 
                                 0, 
                                 pbAttribValue, 
                                 wAttribValueLen );
								 */
            if( FAILED( hr ) )
            {
                break;
            }
        }
        
        hr = pEditor->Close();
        if( FAILED( hr ) )
        {
            _tprintf( _T( "Could not close the file %ws ( hr=0x%08x ).\n" ), pwszInFile, hr );
            break;
        }
    }
    while( FALSE );
    
    SAFE_RELEASE( pHeaderInfo );
    SAFE_RELEASE( pEditor );

    SAFE_ARRAYDELETE( pwszAttribName );
    SAFE_ARRAYDELETE( pbAttribValue );

    return( hr );
}

//------------------------------------------------------------------------------
// Name: ConvertMBtoWC()
// Desc: Converts a string from multibyte to wide character.
//------------------------------------------------------------------------------
#ifndef UNICODE

extern "C" HRESULT ConvertMBtoWC( LPCTSTR ptszInString, LPWSTR *ppwszOutString )
{
    if( ptszInString == NULL || ppwszOutString == NULL )
    {
        return( E_INVALIDARG );
    }

    HRESULT hr          = S_OK;
    int     nSizeCount  = 0;

    *ppwszOutString     = NULL;

    do
    {
        //
        // Get the memory reqd for this string
        //
        nSizeCount = MultiByteToWideChar( CP_ACP, 0, ptszInString, -1, NULL, 0 );
        if( 0 ==  nSizeCount )
        {
            hr = HRESULT_FROM_WIN32( GetLastError() );
            break;
        }

		//WideCharToMultiByte();
        *ppwszOutString = new WCHAR[ nSizeCount ];
        if( NULL == *ppwszOutString )
        {
            hr = HRESULT_FROM_WIN32( GetLastError() );
            break;
        }

        if( 0 == MultiByteToWideChar( CP_ACP, 0, ptszInString, -1, *ppwszOutString, nSizeCount ) )
        {
            hr = HRESULT_FROM_WIN32( GetLastError() );
            break;
        }
    }
    while( FALSE );
    
    if( FAILED( hr ) )
    {
        SAFE_ARRAYDELETE( *ppwszOutString );
        _tprintf( _T( "Internal error ( hr=0x%08x )\n" ), hr );
    }

    return( hr );
}

#endif // UNICODE

extern int AviInfoStreamHeaderParser(AVISTREAMHEADER *aviH, DWORD *buf, unsigned long size);
extern int AviInfoStreamHeaderParser(AVISTREAMHEADER *aviH, DWORD *buf, unsigned long size);
extern int AviInfoWaveFormatMatExParser(WAVEFORMATEX *wfm, DWORD *buf, unsigned long size);
extern int AviInfoMainHeaderParser(AVIMAINHEADER *aviH, DWORD *buf, unsigned long size);

extern "C" static int CheckAVIPlayState(char *filename, struct MediaInfo_str *valueList)
{
	//unsigned long	vbuf[AVI_BUFFER_SIZE];	
	//unsigned long	*vbuf = (unsigned long*)malloc(4096*10);	
	unsigned long	*vbuf;
	unsigned long	headerSize = 0, writeDataSize = 0;
    int				byteRead, dwordRead, pos = 0 ,r = 0, headerCount = 0, i;
    CString         strFilename;
    AVIMAINHEADER	aviHeader;
    AVISTREAMHEADER vidsHeader, audsHeader;
    WAVEFORMATEX	waveFormatex;
	TCHAR	pwszInFile[MAX_PATH_LENGTH];

    CFile File;
#ifndef UNICODE
    WCHAR   * pwszInFile        = NULL;
#endif       

	Utf8ToStr(pwszInFile, filename, MAX_PATH_LENGTH);
#ifndef UNICODE

    hr = ConvertMBtoWC( ptszInFile, &pwszInFile );
    if( FAILED( hr ) )
    {
        return SFALSE;
    }
    strFilename = ptszInFile;
    SAFE_ARRAYDELETE( pwszInFile );
#else
    strFilename = pwszInFile;
#endif       

	vbuf = (unsigned long *)malloc(AVI_START_HEADER_SIZE);

    if (!File.Open(strFilename, CFile::modeRead | CFile::shareDenyNone))
	    return S_FALSE;

	File.Seek(0, CFile::begin);
	if (File.Read(vbuf, AVI_START_HEADER_SIZE) != AVI_START_HEADER_SIZE)
	{
		// Cannot get complete packet
        File.Close();
		return S_FALSE;
	}

    if(vbuf[0] != RIFFtag || vbuf[2] != AVI_tag || vbuf[3] != LISTtag || vbuf[5] != hdrltag)
    {
		// This file is not a valid AVI fornat
		return S_FALSE;
    }


    headerSize = vbuf[4];
    //realloc(vbuf, headerSize + 20);
	if( headerSize > AVI_START_HEADER_SIZE )
	{
		free(vbuf);
		vbuf = (unsigned long *)malloc(headerSize+100);

	}
	//vbuf = (unsigned long*)malloc(headerSize + 100);
		
	byteRead = File.Read(vbuf, headerSize + 20);
	if (byteRead != (headerSize + 20))
	{
		// Cannot get complete header
		return S_FALSE;
	}

	dwordRead = byteRead >> 2;
    while (((pos + 2) < dwordRead) && (headerCount < 3))
    {
        if (vbuf[pos] == avihtag)
        {
            r = AviInfoMainHeaderParser(&aviHeader, vbuf + pos, (dwordRead - pos) * 4);
            if (r < 1)
			{	
				break;
			}

            pos += r;
            headerCount++;
        }
        else if (vbuf[pos] == strhtag && vbuf[pos+2] == vidstag)
        {
            r = AviInfoStreamHeaderParser(&vidsHeader, vbuf + pos, (dwordRead - pos) * 4);

            if (r < 1)
			{
				break;
			}
			
			//2007/3/19 j令pg
			unsigned char *pp=(unsigned char *)&(vidsHeader.fccHandler);			

			for (int i = 0; i < 4; i++)
            {
                pp[i] = tolower(pp[i]);               // নpgr				
            }
				
			switch (vidsHeader.fccHandler)
            {
            case DIVXtag:
            case divxtag:
            case xvidtag:
            case div3tag:
            case DIV3tag:
            case div4tag:
            case DIV4tag:
            case div5tag:
            case DIV5tag:
                break;
            default:
                valueList->Flags |= FLAGS_FORCETRANSCODING;
                break;
            };

            pos += r;
            headerCount++;
        }
		else if (vbuf[pos] == strhtag && vbuf[pos+2] == audstag)
        {
            r = AviInfoStreamHeaderParser(&audsHeader, vbuf + pos, (dwordRead - pos) * 4);
            if (r < 1)
			{
				break;
			}
            pos += r;

            if (vbuf[pos] == strftag)
            {
                r = AviInfoWaveFormatMatExParser(&waveFormatex, vbuf + pos, (dwordRead - pos) * 4);
                pos += r;
            }
			headerCount++;
        }
        else
            pos++;
    }

    File.Close();
	free(vbuf);
    return S_OK;
};


//------------------------------------------------------------------------------
// Name: _tmain()
// Desc: Entry point for the application. Processes command-line arguments.
//------------------------------------------------------------------------------

//int __cdecl _tmain( int argc, TCHAR * argv[] )
extern "C" int getMediaInfo(char *filename, struct MediaInfo_str *valueList)
{
	HRESULT hr              = S_OK;
	//LPTSTR  ptszInFile    = (LPTSTR)_T("c:\\0-test\\00.mp3");
	//LPTSTR  ptszInFile      = _T("c:\\0-test\\03-Ǥf.MP3");
	//LPTSTR  ptszInFile      = _T(filename);
    WORD    wStreamNum      = 0;
    WORD    wAttribIndex    = 0;

    LPTSTR  ptszAttribName  = NULL;
    LPTSTR  ptszAttribValue = NULL;
    WORD    wLangIndex      = 0;
	int i=0, j=0 ;

    WCHAR   * pwszAttribName    = NULL;
    WCHAR   * pwszAttribValue   = NULL;
	TCHAR	pwszInFile[MAX_PATH_LENGTH];

	Utf8ToStr(pwszInFile, filename, MAX_PATH_LENGTH);

#ifndef UNICODE

    WCHAR   * pwszAttribName    = NULL;
    WCHAR   * pwszAttribValue   = NULL;
    WCHAR   * pwszInFile        = NULL;

#endif
	
        // Initialize COM
        hr = CoInitialize( NULL );
        if( FAILED( hr ) )
        {
            return FALSE;
        }

#ifndef UNICODE
        hr = ConvertMBtoWC( ptszInFile, &pwszInFile );
        if( FAILED( hr ) )
        {
            return FALSE;
        }
#endif       
			wStreamNum = 0 ;
			// get the mediainfo
			IWMMetadataEditor   * pEditor           = NULL;
			IWMHeaderInfo3*      pHeaderInfo3         = NULL;

			WORD               wAttribNameLen   = 0;
			WMT_ATTR_DATATYPE  wAttribType;
			WORD               wLanguageIndex   = 0;
			BYTE*              pbAttribValue    = NULL;
			DWORD              dwAttribValueLen = 0;

			WCHAR pwszValue[256];
			WCHAR wszNum[256];
			//
			// Make the data type string
			//
			WCHAR * pwszType = L"Unknown";
			WCHAR * pTypes[] = 
			{ 
				L"DWORD", 
					L"STRING", 
					L"BINARY", 
					L"BOOL", 
					L"QWORD", 
					L"WORD", 
					L"GUID" 
			};

			do
			{
				hr = EditorOpenFile( pwszInFile, &pEditor, NULL, &pHeaderInfo3 );
				if(FAILED( hr ) )
				{
                    int NameLen = strlen(filename);
                    if ((NameLen > 5) && (stricmp(&filename[NameLen - 4], ".avi") == 0))
                    {
                        hr = CheckAVIPlayState(filename, valueList);
                    };
					break;
				}

				WORD wAttributeCount = 0;

				hr = pHeaderInfo3->GetAttributeCountEx( wStreamNum, &wAttributeCount);
				if(FAILED( hr ) )
				{
					_tprintf( _T( "GetAttributeCount failed for stream = %d ( hr=0x%08x ).\n" ), wStreamNum, hr );
					break;
				}

				//
				//
				//
		

				// add by albert
				// hr = pHeaderInfo->GetAttributeByName(pwszAttribName,pbAttribValue);

				int listCounter =0 ;
				for( WORD wAttribIndex = 0; wAttribIndex < wAttributeCount; wAttribIndex++)
				{
					SAFE_ARRAYDELETE( pwszAttribName );
					SAFE_ARRAYDELETE( pbAttribValue );


					hr = pHeaderInfo3->GetAttributeByIndexEx( wStreamNum,
						wAttribIndex,                                                
						pwszAttribName,
						&wAttribNameLen,
						&wAttribType,
						&wLanguageIndex,
						pbAttribValue,
						&dwAttribValueLen );
					if(FAILED( hr ) )
					{
						_tprintf( _T( "GetAttributeByIndexEx failed for index = %d ( hr=0x%08x ).\n" ), wAttribIndex, hr );
						break;
					}

					pwszAttribName = new WCHAR[wAttribNameLen];
					if( NULL == pwszAttribName )
					{
						hr = E_OUTOFMEMORY;
						break;
					}

					pbAttribValue = new BYTE[dwAttribValueLen];
					if( NULL == pbAttribValue )
					{
						hr = E_OUTOFMEMORY;
						break;
					}

					hr = pHeaderInfo3->GetAttributeByIndexEx( wStreamNum,
						wAttribIndex,                                                
						pwszAttribName,
						&wAttribNameLen,
						&wAttribType,
						&wLanguageIndex,
						pbAttribValue,
						&dwAttribValueLen );
					if(FAILED( hr ) )
					{
						_tprintf( _T( "GetAttributeByIndexEx failed for index = %d ( hr=0x%08x ).\n" ), 
							wAttribIndex, hr );
						break;
					}

					ZeroMemory( pwszValue, sizeof( pwszValue ) );
					ZeroMemory( wszNum, sizeof( wszNum ) );

					if( ( sizeof( pTypes ) / sizeof( WCHAR ) ) > wAttribType )
					{
						pwszType = pTypes[wAttribType];
					}

					//
					// The attribute value.
					//

					switch ( wAttribType )
					{
						//
						// string
						//
					case WMT_TYPE_STRING:
						if( 0 == dwAttribValueLen )
						{
							wcscat( pwszValue, L"***** NULL *****" );
						}
						else
						{
							if( 0xFEFF == *(WCHAR*)pbAttribValue )
							{
								//wcscat( pwszValue, L"UTF-16LE BOM+" );
								wcscat( pwszValue, L"\"" );
								if( 4 <= dwAttribValueLen )
								{
									wcsncat( pwszValue, ( (WCHAR *)pbAttribValue ) + 1, 
										min( ( dwAttribValueLen / sizeof( WCHAR ) ) - 2, ( sizeof( pwszValue ) / sizeof( WCHAR ) ) - 20 ) );
								}
								//wcscat( pwszValue, L"\"" );
							}
							else if( 0xFFFE == *(WCHAR*)pbAttribValue )
							{
								wcscat( pwszValue, L"UTF-16BE BOM+" );
								//wcscat( pwszValue, L"\"" );
								if( 4 <= dwAttribValueLen )
								{
									wcsncat( pwszValue, ( (WCHAR *)pbAttribValue ) + 1, 
										min( ( dwAttribValueLen / sizeof( WCHAR ) ) - 2, ( sizeof( pwszValue ) / sizeof( WCHAR ) ) - 20 ) );
								}
								//wcscat( pwszValue, L"\"" );
							}
							else
							{
								//wcscat( pwszValue, L"\"" );
								if( 2 <= dwAttribValueLen )
								{
									wcsncat( pwszValue, (WCHAR *)pbAttribValue, 
										min( ( dwAttribValueLen / sizeof( WCHAR ) ) - 1, ( sizeof( pwszValue ) / sizeof( WCHAR ) ) - 10 ) );
								}
								//wcscat( pwszValue, L"\"" );
							}
						}
						break;

						//
						// Binary
						//
					case WMT_TYPE_BINARY:
						wcscat( pwszValue, L"[" );
						_itow( dwAttribValueLen, wszNum, 10 );
						wcscat( pwszValue, wszNum );
						wcscat( pwszValue, L" bytes]" );
						break;

						//
						// Boolean
						//
					case WMT_TYPE_BOOL:
						if( *( (BOOL* )pbAttribValue ) == FALSE )
						{
							wcscat( pwszValue, L"False" );
						}
						else
						{
							wcscat( pwszValue, L"True" );
						}
						break;

						//
						// DWORD
						//
					case WMT_TYPE_DWORD:
						//swprintf( wszNum, L"%ld, 0x%08lx", 
						swprintf( wszNum, L"%ld", 
							( (DWORD *)pbAttribValue )[0], ( (DWORD *)pbAttribValue )[0] );
						wcscat( pwszValue, wszNum );
						break;

						//
						// QWORD
						//
					case WMT_TYPE_QWORD:
						_ui64tow( *( (QWORD* )pbAttribValue ), wszNum, 10 );
						wcscat( pwszValue, wszNum );
						/*
						swprintf( wszNum, L", 0x%08lx%08lx", 
							( (DWORD *)pbAttribValue )[1], ( (DWORD *) pbAttribValue )[0] );
						wcscat( pwszValue, wszNum );
						*/
						break;

						//
						// WORD
						//
					case WMT_TYPE_WORD:
						swprintf( wszNum, L"%d, 0x%04x", 
							( ( WORD* )pbAttribValue )[0], ( ( WORD* ) pbAttribValue )[0] );
						wcscat( pwszValue, wszNum );
						break;

						//
						// GUID
						//
					case WMT_TYPE_GUID:
						if( !StringFromGUID2( *(GUID *)pbAttribValue, pwszValue, 256 ) )
						{
							wcscpy( pwszValue, L"ERROR" );
						}
						break;

					default:
						swprintf( pwszValue, L"Bad data type (%hu): value not displayed", 
							wAttribType );
						break;
					}

					// retrieve the attribute need to stroe in the database
					int iii=0;
					char	str[MAX_MEDIA_STR_LENGTH];	

					//if( wcscmp(pwszAttribName,L"Title")==0) //Title
					if( wcscmp(pwszAttribName,g_wszWMTitle)==0) //Title
					{
						iii=WideCharToMultiByte(CP_ACP,0,pwszValue,-1,str,1024,NULL,NULL);
						// SQL query string cannot contain /", so replace with a space
						strReplaceWith(str, '/"', ' ');
						MultiByteToWideChar(codepage, 0, str, -1, valueList->Title, MAX_MEDIA_STR_LENGTH);
					}

					//if( wcscmp(pwszAttribName,L"Duration")==0) //Duration
					if( wcscmp(pwszAttribName,g_wszWMDuration)==0) //Duration
						//iii=WideCharToMultiByte(CP_ACP,0,pwszValue,-1,valueList->Duration,1024,NULL,NULL);
						wcscpy(valueList->Duration, pwszValue);

					//if( wcscmp(pwszAttribName,L"Bitrate")==0) //Bitrate
					if( wcscmp(pwszAttribName,g_wszWMBitrate)==0) //Bitrate
						//iii=WideCharToMultiByte(CP_ACP,0,pwszValue,-1,valueList->Bitrate,1024,NULL,NULL);
						wcscpy(valueList->Bitrate, pwszValue);

					//if( wcscmp(pwszAttribName,L"WM/Track")==0) //WM/Track
					if( wcscmp(pwszAttribName,g_wszWMTrackNumber)==0) //WM/Track
						//iii=WideCharToMultiByte(CP_ACP,0,pwszValue,-1,valueList->Track,1024,NULL,NULL);
						wcscpy(valueList->Track, pwszValue);

					//if( wcscmp(pwszAttribName,L"WM/AlbumTitle")==0) //WM/AlbumTitle
					if( wcscmp(pwszAttribName,g_wszWMAlbumTitle)==0) //WM/AlbumTitle
					{
						iii=WideCharToMultiByte(CP_ACP,0,pwszValue,-1,str,1024,NULL,NULL);
						// SQL query string cannot contain /", so replace with a space
						strReplaceWith(str, '/"', ' ');
						MultiByteToWideChar(codepage, 0, str, -1, valueList->AlbumTitle, MAX_MEDIA_STR_LENGTH);
					}

					//if( wcscmp(pwszAttribName,L"Author")==0) //Author 
					if( wcscmp(pwszAttribName,g_wszWMAuthor)==0) //Author 
					{
						iii=WideCharToMultiByte(CP_ACP,0,pwszValue,-1,str,1024,NULL,NULL);
						// SQL query string cannot contain /", so replace with a space
						strReplaceWith(str, '/"', ' ');
						MultiByteToWideChar(codepage, 0, str, -1, valueList->Author, MAX_MEDIA_STR_LENGTH);
					}
					//if( wcscmp(pwszAttribName,L"WM/Genre")==0) //WM/Genre
					if( wcscmp(pwszAttribName,g_wszWMGenre)==0) //WM/Genre
					{
						iii=WideCharToMultiByte(CP_ACP,0,pwszValue,-1,str,1024,NULL,NULL);
						// SQL query string cannot contain /", so replace with a space
						strReplaceWith(str, '/"', ' ');
						MultiByteToWideChar(codepage, 0, str, -1, valueList->Genre, MAX_MEDIA_STR_LENGTH);
					}

					// albert add ,20050421
					
					if( wcscmp(pwszAttribName,g_wszWMStreamTypeInfo)==0) //WM/StreamTypeInfo
						iii=WideCharToMultiByte(CP_ACP,0,pwszValue,-1,valueList->StreamTypeInfo,1024,NULL,NULL);
					
				}

                {
                    DWORD CodecInfoCount;
                    WORD  cchName = MAX_PATH/2;
                    WCHAR wszName[MAX_PATH];
                    WORD  cchDescription = MAX_PATH/2;
                    WCHAR wszDescription[MAX_PATH];
                    WMT_CODEC_INFO_TYPE CodecType;
                    WORD cbCodecInfo = MAX_PATH;
                    BYTE bCodecInfo[MAX_PATH];

			        hr = pHeaderInfo3->GetCodecInfoCount(&CodecInfoCount);
                    if (hr == S_OK)
                    {
                        for (i = 0; i < CodecInfoCount; i++)
                        {
			                hr = pHeaderInfo3->GetCodecInfo(i, &cchName, wszName, &cchDescription, wszDescription, &CodecType, &cbCodecInfo, bCodecInfo);
                            if ((hr != S_OK) && (hr != 0xc00d07d1))
                                break;
			                hr = pHeaderInfo3->GetCodecInfo(i, &cchName, wszName, &cchDescription, wszDescription, &CodecType, &cbCodecInfo, bCodecInfo);
                            if ((hr != S_OK) && (hr != 0xc00d07d1))
                                break;
                            if ((CodecType == WMT_CODECINFO_VIDEO) && (cbCodecInfo == 0x04) && (strnicmp((const char *)bCodecInfo, "WMV3", 4) == 0) && 
                                (cchDescription == 0x0d) && (wcsnicmp(wszDescription, L"Professional", 0x0c) == 0))
                            {
                                int NameLen = strlen(filename);
                                if ((NameLen > 5) && (stricmp(&filename[NameLen - 4], ".wmv") == 0))
                                    valueList->Flags |= FLAGS_UNPLAYABLE;
                            }
                        }
                    };

                }

				hr = pEditor->Close();
				if( FAILED( hr ) )
				{
					_tprintf( _T( "Could not close the file %ws ( hr=0x%08x ).\n" ), pwszInFile, hr );
					break;
				}
			}
			while( FALSE );


			SAFE_RELEASE( pHeaderInfo3 );
			SAFE_RELEASE( pEditor );

			SAFE_ARRAYDELETE( pwszAttribName );
			SAFE_ARRAYDELETE( pbAttribValue );

            if(FAILED( hr ) )
            {
                return FALSE;
            }

#ifndef UNICODE

    SAFE_ARRAYDELETE( pwszInFile );
    SAFE_ARRAYDELETE( pwszAttribName );
    SAFE_ARRAYDELETE( pwszAttribValue );

#endif // UNICODE
    CoUninitialize();
   // return( hr );
	if(hr == S_OK)
		return(TRUE);
	else
		return(FALSE);
}


extern "C" int modifyMediaInfo(char *filename, struct MediaInfo_str *mediainfo)
{
	HRESULT hr              = S_OK;
	//LPTSTR  ptszInFile    = (LPTSTR)_T("c:\\0-test\\00.mp3");
	//LPTSTR  ptszInFile      = _T("c:\\0-test\\03-Ǥf.MP3");
	//LPTSTR  ptszInFile      = _T(filename);
	WORD    wStreamNum      = 0;
	WORD    wAttribIndex    = 0;

	LPTSTR  ptszAttribName  = NULL;
	LPTSTR  ptszAttribValue = NULL;
	WORD    wLangIndex      = 0;
	int i=0, j=0 ;
	int setTitle = 0 , setAlbum=0, setAuthor=0, setGenre=0;
	
	WCHAR   * pwszAttribName    = NULL;
	WCHAR   * pwszAttribValue   = NULL;
	TCHAR	pwszInFile[MAX_PATH_LENGTH];

	Utf8ToStr(pwszInFile, filename, MAX_PATH_LENGTH);

#ifndef UNICODE

	WCHAR   * pwszAttribName    = NULL;
	WCHAR   * pwszAttribValue   = NULL;
	WCHAR   * pwszInFile        = NULL;

#endif

	// Initialize COM
	hr = CoInitialize( NULL );
	if( FAILED( hr ) )
	{
		return FALSE;
	}

#ifndef UNICODE
	hr = ConvertMBtoWC( ptszInFile, &pwszInFile );
	if( FAILED( hr ) )
	{
		return FALSE;
	}
#endif          

	wStreamNum = 0 ;
	// get the mediainfo
	IWMMetadataEditor   * pEditor           = NULL;
	IWMHeaderInfo3*      pHeaderInfo3         = NULL;

	WORD               wAttribNameLen   = 0;
	WMT_ATTR_DATATYPE  wAttribType;
	WORD               wLanguageIndex   = 0;
	BYTE*              pbAttribValue    = NULL;
	DWORD              dwAttribValueLen = 0;

	WCHAR pwszValue[256];
	WCHAR wszNum[256];
	char str[MAX_MEDIA_STR_LENGTH];


	// for modify media info
	//WCHAR *tempAttribValue= NULL;
	DWORD tempAttribValueLen = 0;

	DWORD               dwAttribValue   = 0;
	WORD                wAttribValue    = 0;
	QWORD               qwAttribValue   = 0;
	BOOL                fAttribValue    = 0;



	//
	// Make the data type string
	//
	WCHAR * pwszType = L"Unknown";
	WCHAR * pTypes[] = 
	{ 
		L"DWORD", 
			L"STRING", 
			L"BINARY", 
			L"BOOL", 
			L"QWORD", 
			L"WORD", 
			L"GUID" 
	};

	do
	{
		hr = EditorOpenFile( pwszInFile, &pEditor, NULL, &pHeaderInfo3 );
		if(FAILED( hr ) )
		{
			break;
		}

		WORD wAttributeCount = 0;

		hr = pHeaderInfo3->GetAttributeCountEx( wStreamNum, &wAttributeCount);
		if(FAILED( hr ) )
		{
			_tprintf( _T( "GetAttributeCount failed for stream = %d ( hr=0x%08x ).\n" ), wStreamNum, hr );
			break;
		}


		int listCounter =0 ;
		for( WORD wAttribIndex = 0; wAttribIndex < wAttributeCount; wAttribIndex++)
		{
			SAFE_ARRAYDELETE( pwszAttribName );
			SAFE_ARRAYDELETE( pbAttribValue );

			hr = pHeaderInfo3->GetAttributeByIndexEx( wStreamNum,
				wAttribIndex,                                                
				pwszAttribName,
				&wAttribNameLen,
				&wAttribType,
				&wLanguageIndex,
				pbAttribValue,
				&dwAttribValueLen );
			if(FAILED( hr ) )
			{
				_tprintf( _T( "GetAttributeByIndexEx failed for index = %d ( hr=0x%08x ).\n" ), wAttribIndex, hr );
				break;
			}

			pwszAttribName = new WCHAR[wAttribNameLen];
			if( NULL == pwszAttribName )
			{
				hr = E_OUTOFMEMORY;
				break;
			}

			pbAttribValue = new BYTE[dwAttribValueLen];
			if( NULL == pbAttribValue )
			{
				hr = E_OUTOFMEMORY;
				break;
			}

			hr = pHeaderInfo3->GetAttributeByIndexEx( wStreamNum,
				wAttribIndex,                                                
				pwszAttribName,
				&wAttribNameLen,
				&wAttribType,
				&wLanguageIndex,
				pbAttribValue,
				&dwAttribValueLen );
			if(FAILED( hr ) )
			{
				_tprintf( _T( "GetAttributeByIndexEx failed for index = %d ( hr=0x%08x ).\n" ), 
					wAttribIndex, hr );
				break;
			}

			ZeroMemory( pwszValue, sizeof( pwszValue ) );
			ZeroMemory( wszNum, sizeof( wszNum ) );

			if( ( sizeof( pTypes ) / sizeof( WCHAR ) ) > wAttribType )
			{
				pwszType = pTypes[wAttribType];
			}

			//
			// modify the attribute store in the file
			//
			if( wcscmp(pwszAttribName,L"Title")==0) //Title
			{
				WideCharToMultiByte(codepage, 0, mediainfo->Title, -1, str, MAX_MEDIA_STR_LENGTH, NULL,NULL);
				tempAttribValueLen = (strlen(str) + 1) * sizeof(char);
				hr = pHeaderInfo3->ModifyAttribute( wStreamNum,
					wAttribIndex,
					wAttribType, 
					wLangIndex,
					//pbAttribValue,
					(BYTE *) str,
					tempAttribValueLen );

				if(FAILED( hr ) )
				{
					_tprintf( _T( "ModifyAttribute failed for index = %d ( hr=0x%08x ).\n" ), 
						wAttribIndex, hr );
					break;
				}
				setTitle = 1;
			}
			
			if( wcscmp(pwszAttribName,L"WM/AlbumTitle")==0) // WM/AlbumTitle
			{
				WideCharToMultiByte(codepage, 0, mediainfo->AlbumTitle, -1, str, MAX_MEDIA_STR_LENGTH, NULL,NULL);
				tempAttribValueLen = (strlen(str) + 1) * sizeof(char);
				hr = pHeaderInfo3->ModifyAttribute( wStreamNum,
					wAttribIndex,
					wAttribType, 
					wLangIndex,
					//pbAttribValue,
					(BYTE *) str,
					tempAttribValueLen );

				if(FAILED( hr ) )
				{
					_tprintf( _T( "ModifyAttribute failed for index = %d ( hr=0x%08x ).\n" ), 
						wAttribIndex, hr );
					break;
				}
				setAlbum = 1;
			}

			if( wcscmp(pwszAttribName,L"Author")==0) // Author
			{
				WideCharToMultiByte(codepage, 0, mediainfo->Author, -1, str, MAX_MEDIA_STR_LENGTH, NULL,NULL);
				tempAttribValueLen = (strlen(str) + 1) * sizeof(char);
				hr = pHeaderInfo3->ModifyAttribute( wStreamNum,
					wAttribIndex,
					wAttribType, 
					wLangIndex,
					//pbAttribValue,
					(BYTE *) str,
					tempAttribValueLen );

				if(FAILED( hr ) )
				{
					_tprintf( _T( "ModifyAttribute failed for index = %d ( hr=0x%08x ).\n" ), 
						wAttribIndex, hr );
					break;
				}
				setAuthor = 1;
			}

			if( wcscmp(pwszAttribName,L"WM/Genre")==0) // Genre
			{
				WideCharToMultiByte(codepage, 0, mediainfo->Genre, -1, str, MAX_MEDIA_STR_LENGTH, NULL,NULL);
				tempAttribValueLen = (strlen(str) + 1) * sizeof(char);
				hr = pHeaderInfo3->ModifyAttribute( wStreamNum,
					wAttribIndex,
					wAttribType, 
					wLangIndex,
					//pbAttribValue,
					(BYTE *) str,
					tempAttribValueLen );

				if(FAILED( hr ) )
				{
					_tprintf( _T( "ModifyAttribute failed for index = %d ( hr=0x%08x ).\n" ), 
						wAttribIndex, hr );
					break;
				}
				setGenre = 1;
			}

			hr = pEditor->Flush();
			if( FAILED( hr ) )
			{
				_tprintf( _T( "Could not flush the file %ws ( hr=0x%08x ).\n" ), 
					pwszInFile, hr );
				break;
			}

/*
			if( wcscmp(pwszAttribName,L"Duration")==0) //Duration
				iii=WideCharToMultiByte(CP_ACP,0,pwszValue,-1,valueList->Duration,1024,NULL,NULL);
			if( wcscmp(pwszAttribName,L"Bitrate")==0) //Bitrate
				iii=WideCharToMultiByte(CP_ACP,0,pwszValue,-1,valueList->Bitrate,1024,NULL,NULL);
			if( wcscmp(pwszAttribName,L"WM/Track")==0) //WM/Track
				iii=WideCharToMultiByte(CP_ACP,0,pwszValue,-1,valueList->Track,1024,NULL,NULL);
			
*/
			
		} // end of for loop

		if(setTitle == 0) // add attribute
		{
			WideCharToMultiByte(codepage, 0, mediainfo->Title, -1, str, MAX_MEDIA_STR_LENGTH, NULL,NULL);
			tempAttribValueLen = (strlen(str) + 1) * sizeof(char);
			hr = pHeaderInfo3->AddAttribute( wStreamNum,
				L"Title",
				NULL,
				wAttribType, 
				wLangIndex,
				//pbAttribValue,
				(BYTE *) str,
				tempAttribValueLen );

			if( FAILED( hr ) )
			{
				_tprintf( _T( "AddAttribute failed for Attribute name Title\n" ) );
				break;
			}
			setTitle = 1;
		}

		// if this file is a audio type , check album, author, gener update or not
		if(_tcscmp(mediainfo->FileType,_T("Audio"))==0)
		{
			if( setAlbum == 0) // Album
			{
			WideCharToMultiByte(codepage, 0, mediainfo->AlbumTitle, -1, str, MAX_MEDIA_STR_LENGTH, NULL,NULL);
			tempAttribValueLen = (strlen(str) + 1) * sizeof(char);
			hr = pHeaderInfo3->AddAttribute( wStreamNum,
				L"AlbumTitle",
				NULL,
				wAttribType, 
				wLangIndex,
				//pbAttribValue,
				(BYTE *) str, 
				tempAttribValueLen );

				if(FAILED( hr ) )
				{
					_tprintf( _T( "ModifyAttribute failed for name = AlbumTitle\n") );
					break;
				}
				setAlbum = 1;
			}
		
			if( setAuthor == 0) // Author
			{
				WideCharToMultiByte(codepage, 0, mediainfo->Author, -1, str, MAX_MEDIA_STR_LENGTH, NULL,NULL);
				tempAttribValueLen = (strlen(str) + 1) * sizeof(char);
				hr = pHeaderInfo3->AddAttribute( wStreamNum,
					L"Author",
					NULL,
					wAttribType, 
					wLangIndex,
					//pbAttribValue,
					(BYTE *) str, 
					tempAttribValueLen );

				if(FAILED( hr ) )
				{
					_tprintf( _T( "ModifyAttribute failed for name = Author\n" ));
					break;
				}
				setAuthor = 1;
			}

			if(setGenre == 0) // Genre
			{
				WideCharToMultiByte(codepage, 0, mediainfo->Genre, -1, str, MAX_MEDIA_STR_LENGTH, NULL,NULL);
				tempAttribValueLen = (strlen(str) + 1) * sizeof(char);
				hr = pHeaderInfo3->AddAttribute( wStreamNum,
					L"WM/Genre",
					NULL,
					wAttribType, 
					wLangIndex,
					//pbAttribValue,
					(BYTE *) str,
					tempAttribValueLen );

				if(FAILED( hr ) )
				{
					_tprintf( _T( "ModifyAttribute failed for name = WM/Genre\n" ) );
					break;
				}
				setGenre = 1;
			}
		}

		hr = pEditor->Flush();
		if( FAILED( hr ) )
		{
			_tprintf( _T( "Could not flush the file %ws ( hr=0x%08x ).\n" ), 
				pwszInFile, hr );
			break;
		}

		hr = pEditor->Close();
		if( FAILED( hr ) )
		{
			_tprintf( _T( "Could not close the file %ws ( hr=0x%08x ).\n" ), pwszInFile, hr );
			break;
		}
	}
	while( FALSE );

	SAFE_RELEASE( pHeaderInfo3 );
	SAFE_RELEASE( pEditor );

	SAFE_ARRAYDELETE( pwszAttribName );
	SAFE_ARRAYDELETE( pbAttribValue );

	if(FAILED( hr ) )
	{
		return FALSE;
	}

#ifndef UNICODE

	SAFE_ARRAYDELETE( pwszInFile );
	SAFE_ARRAYDELETE( pwszAttribName );
	SAFE_ARRAYDELETE( pwszAttribValue );

#endif // UNICODE
	CoUninitialize();
	// return( hr );
	if(hr == S_OK)
		return(TRUE);
	else
		return(FALSE);
}