#include "stdafx.h"
#include "TransStatus.h"

extern "C"
{
#include "mystring.h"
#include "debug.h"
}

// Use array instead of link-list to avoid sync between threads
static TransStatus		transStatus[TRANS_MAX_NO_PROGRESS_ITEMS];
static int		head = 0, tail= 0;	

extern "C" void AddNewTransStatus(char *filename, char *ip, unsigned long long fileSize, char *urlPath) 
{
	CString	strExt, str;
	int		i;

	str = filename;
	strExt = str.Right(str.GetLength() - str.ReverseFind(_T('.')));
		
	if ((strExt.CompareNoCase(_T(".hdx")) == 0) ||
		(strExt.CompareNoCase(_T(".idx")) == 0) ||
		(strExt.CompareNoCase(_T(".sub")) == 0) ||
		(strExt.CompareNoCase(_T(".saa")) == 0) ||
		(strExt.CompareNoCase(_T(".srt")) == 0) ||
		(strExt.CompareNoCase(_T(".ifo")) == 0))
	{
		// video index file (.hdx) and subtitle are used internally, don't show it
		return;
	}
	
	i = (head - 1 + TRANS_MAX_NO_PROGRESS_ITEMS) % TRANS_MAX_NO_PROGRESS_ITEMS;

	if ((stricmp(transStatus[i].urlPath, urlPath) == 0)	&& 
		(stricmp(transStatus[i].ip, ip) == 0))
	{
		// If this entry already in our buffer, don't add it
		return;
	}

	strcpy(transStatus[head].filename, filename);
	strcpy(transStatus[head].ip, ip);
	strcpy(transStatus[head].urlPath, urlPath);
	transStatus[head].fileSize = fileSize;
	transStatus[head].flag = TRANS_NEW_ITEM;
	
	// move head to next empty entry
	head = (head + 1) % TRANS_MAX_NO_PROGRESS_ITEMS;
	if (head == tail)
	{
		// the array is full, move tail to next item and let head point to an unued item
		tail = (tail + 1) % TRANS_MAX_NO_PROGRESS_ITEMS;
	}
}


extern "C" void ModifyTransStatus(char *urlPath, char *ip, off_t offset) 
{
	int		i, j;
	for (i = head; i != tail; i = j)
	{
		j = (i - 1 + TRANS_MAX_NO_PROGRESS_ITEMS) % TRANS_MAX_NO_PROGRESS_ITEMS;

		if ((stricmp(transStatus[j].urlPath, urlPath) == 0) &&
			(stricmp(transStatus[j].ip, ip) == 0))
		{
			// Find the item, update its progress
			transStatus[j].offset =  offset;
			if (transStatus[j].flag == TRANS_SHOWN_ITEM)
			{
				// This item has been shown, only need to update process
				transStatus[j].flag = TRANS_UPDATE_ITEM;
			}
			break;
		}
	}
}

// Copy the new and updated item to caller buffer
int	GetShowTransStatus(PTransStatus	showTransStatus)
{
	int		count = 0, i, j;

	for (i = head; i != tail; i = j)
	{
		j = (i - 1 + TRANS_MAX_NO_PROGRESS_ITEMS) % TRANS_MAX_NO_PROGRESS_ITEMS;

		if (transStatus[j].flag != TRANS_SHOWN_ITEM)
		{
			memcpy((showTransStatus + count), &transStatus[j], sizeof(TransStatus));
			transStatus[j].flag = TRANS_SHOWN_ITEM;
			count++;
		}
	}

	return count;
}