#include "stdafx.h"
#include "Utility.h"
#include "myGdiPlus.h"
#include "ScanFiles.h"

using namespace Gdiplus; 

extern "C"
{
	#include "FileIoAbstraction.h"
	#include "mystring.h"
	#include "debug.h"
}

extern "C" TCHAR	gWorkingDir[MAX_PATH_LENGTH];

int GetEncoderClsid(const WCHAR* format, CLSID* pClsid)
{
	UINT  num = 0;          // number of image encoders
	UINT  size = 0;         // size of the image encoder array in bytes

	ImageCodecInfo* pImageCodecInfo = NULL;

	GetImageEncodersSize(&num, &size);
	if(size == 0)
		return -1;  // Failure

	pImageCodecInfo = (ImageCodecInfo*)(MALLOC(size));
	if(pImageCodecInfo == NULL)
		return -1;  // Failure

	GetImageEncoders(num, size, pImageCodecInfo);

	for(UINT j = 0; j < num; ++j)
	{
		if( wcscmp(pImageCodecInfo[j].MimeType, format) == 0 )
		{
			*pClsid = pImageCodecInfo[j].Clsid;
			FREE(pImageCodecInfo);
			return j;  // Success
		}    
	}

	FREE(pImageCodecInfo);

	return -1;  // Failure
}


extern "C" void GenerateBMPWithSize(char *BMPFileName, char *outFileName, int width, int height)
{
	CString inFilePath, outFilePath; 
	WCHAR	wFileName[MAX_PATH_LENGTH];

	Utf8ToStr(wFileName, BMPFileName, MAX_PATH_LENGTH);
	inFilePath = wFileName;

	outFilePath = gWorkingDir;
	outFilePath += _T("\\thumbnail\\");
	outFilePath += outFileName;
	DeleteFile(outFilePath);

	if (IsFileTypeJ2K(inFilePath))
	{
		PROCESS_INFORMATION	pi;
		STARTUPINFO			si = {sizeof(si)};
		TCHAR				szCmdLine[MAX_PATH * 3];
		DWORD				dwExitCode;

		_stprintf(szCmdLine, _T("%s\\convert -resize %dx%d \"%s\" \"%s\""), gWorkingDir, 
				  width, height, inFilePath, outFilePath);

		BOOL ret = CreateProcess(NULL, szCmdLine, NULL, NULL, FALSE, CREATE_NO_WINDOW, NULL, NULL, &si, &pi);

		if (ret) 
		{
			CloseHandle(pi.hThread);
			WaitForSingleObject(pi.hProcess, INFINITE);
			GetExitCodeProcess(pi.hProcess, &dwExitCode);
			CloseHandle(pi.hProcess);

			inFilePath = outFilePath;
		}
		else
		{
			inFilePath = _T("");
		}
	}

	Bitmap image(inFilePath.AllocSysString());

	if (image.GetFlags() != ImageFlagsNone)
	{
		Bitmap  *bmPhoto;
		CBitmap bmpThumbnail;
		int		destX = 0, destY = 0; 

		int sourceWidth  = image.GetWidth();
		int sourceHeight = image.GetHeight();

		if ((sourceWidth == 0) || (sourceHeight == 0))
		{
			// empty image
			return;
		}

		if (width > sourceWidth)
		{
			width = sourceWidth;
		}

		if (height > sourceHeight)
		{
			height = sourceHeight;
		}

		// accounting the ratio to create the thumbnail
		float nPercent;
		float nPercentW = ((float)width / (float)sourceWidth);;
		float nPercentH = ((float)height / (float)sourceHeight);

		if(nPercentH < nPercentW)
		{
			nPercent = nPercentH;
			destX = (int)((width - (sourceWidth * nPercent))/2);
		}
		else
		{
			nPercent = nPercentW;
			destY    = (int)((height - (sourceHeight * nPercent))/2);
		}

		int destWidth  = (int)(sourceWidth * nPercent);
		int destHeight = (int)(sourceHeight * nPercent);

		bmPhoto = new Bitmap(width, height, PixelFormat24bppRGB);
		bmPhoto->SetResolution(image.GetHorizontalResolution(), image.GetVerticalResolution());

		Graphics *grPhoto = Graphics::FromImage(bmPhoto);
		Color colorW(0, 0, 0, 0);
		grPhoto->Clear(colorW);
		grPhoto->SetInterpolationMode(InterpolationModeHighQualityBicubic);
		grPhoto->DrawImage(&image, Rect(destX, destY, destWidth, destHeight));

		CLSID pngClsid;
		GetEncoderClsid(_T("image/bmp"), &pngClsid);
		bmPhoto->Save(outFilePath.AllocSysString(), &pngClsid, NULL);

		delete grPhoto;
		delete bmPhoto;
	}
}

extern "C" void GetImageInfo(char *fileName, int *sourceWidth, int *sourceHeight)
{
	CString filePath; 
	WCHAR	wFileName[MAX_PATH_LENGTH];

	Utf8ToStr(wFileName, fileName, MAX_PATH_LENGTH);
	filePath = wFileName;

	Bitmap image(filePath.AllocSysString());

	if (image.GetFlags()== ImageFlagsNone)
	{
		*sourceWidth  = 0;
		*sourceHeight = 0;
	}
	else
	{
		*sourceWidth  = image.GetWidth();
		*sourceHeight = image.GetHeight();

		if(*sourceWidth == 0)
		{
			*sourceWidth = 1;
		}

		if (*sourceHeight == 0)
		{
			*sourceHeight = 1;
		}
	}
}

extern "C" void GetFileCreationTime(char *filepath, char *time_string)
{
	struct _FILETIME filetime;
	struct _SYSTEMTIME systime_1;

	FIOA_GetFileCreationTime(filepath, &filetime);
	FileTimeToSystemTime(&filetime, &systime_1);
	sprintf(time_string, "%04d-%02d-%02d" ,systime_1.wYear ,systime_1.wMonth, systime_1.wDay);
}
