#include "stdafx.h"
#include "VideoInfo.h"
#include "dbtools.h"

extern "C"
{
#include "FileIoAbstraction.h"
#include "FileSystemTree.h"
#include "MyString.h"
#include "debug.h"
}

extern "C" TCHAR	gWorkingDir[MAX_PATH_LENGTH];

struct MpegInfo_Struct	mpegInfo;

HANDLE	hAddScanVideoFileMutex; 
HANDLE	ScanVideoFilesThreadHandle = 0; 
DWORD	ScanVideoFilesThreadId;
TCHAR	*scanVideoFileList = NULL;
int		fileNoInScanList;
int		scanListSize;
int		skipScanVideoFiles;
int		framerateTable[] = {24, 24, 24, 25, 30, 30, 50, 60, 60, 60, 60, 60, 60, 60, 60, 60};

AVIMAINHEADER	aviHeader;
AVISTREAMHEADER vidsHeader, audsHeader;
WAVEFORMATEX	waveFormatex;
AVIINFOSTRUCT	aviInfo;

int MpegInfo_Group_Of_Picture(unsigned char* buffer, int bufferSize)
{
	long cTEMP = 0;
	int retVal = 0;

	do {

		cTEMP = *((ULONG *)(buffer + retVal));

		if (cTEMP != GROUP_START_CODE)
		{
			retVal++;
		}
	}while (cTEMP != GROUP_START_CODE && retVal+3 < bufferSize);

	if(retVal + 3 >= bufferSize)  retVal = -1;

	return retVal;
}

int MpegInfo_Packet_Header(unsigned char* buffer, int bufferSize)
{
	ULONG	cTEMP = 0;
	int		retVal = 0;

	do {
		cTEMP = *((ULONG *)(buffer + retVal));

		if (cTEMP != PACK_START_CODE)
		{
			retVal++;
		}	
	} while (cTEMP != PACK_START_CODE && retVal+3 < bufferSize);
  
	if (retVal + 3 >= bufferSize)
		retVal = -1;

	return retVal;
}


int MpegInfo_CheckType(struct MpegInfo_Struct *mpegInfo, unsigned char *buffer, int bufferSize)
{
	int offset, next_offset,gop_offset;

	/* PreCondition Constraint */
	if (buffer == NULL || bufferSize <  MPEG1_PACKET_SIZE*2)
    {
		return FALSE;
    }

	offset = MpegInfo_Packet_Header(buffer,MPEG1_PACKET_SIZE);

	if(offset < 0) 
		return FALSE;
  
	next_offset = offset + MPEG2_PACKET_SIZE;
	next_offset += MpegInfo_Packet_Header(buffer+next_offset,bufferSize - next_offset);

	printf("\n\noffset:%d, next;%d\n",offset, next_offset);

	if(next_offset - offset == MPEG2_PACKET_SIZE)
    {
		mpegInfo->packetSize = MPEG2_PACKET_SIZE;
		mpegInfo->type = MPEGINFO_TYPE_MPEG2;
    }
	else if( mpegInfo->packetSize = MPEG1_PACKET_SIZE )
    {
		mpegInfo->packetSize = MPEG1_PACKET_SIZE;
		gop_offset = MpegInfo_Group_Of_Picture(buffer,bufferSize);
      
		if (gop_offset < 0) 
			return FALSE;

		//printf("gop_offset:%d  reminder:%d\n", gop_offset, gop_offset - offset - 134);
		if( (gop_offset - offset - 134) % MPEG1_PACKET_SIZE == 0 )
		{
			mpegInfo->type = MPEGINFO_TYPE_SVCD;
			mpegInfo->packetSize = MPEG1_PACKET_SIZE;
		}
		else
		{
			mpegInfo->type = MPEGINFO_TYPE_MPEG1;
			mpegInfo->packetSize = MPEG1_PACKET_SIZE;
		}
    }

	return TRUE;
}


int MpegInfo_GetGOP(long nCode)
{
	int second, minute, hour, frame;

	nCode >>= 7;
	frame = nCode & 0x3f;	// frame of pictures
	nCode >>= 6;			// - second of pictures
	second = nCode & 0x3f;
	nCode >>= (6+1);		// seconds + marker
	minute = nCode & 0x3f;
	nCode >>= 6;			// minutes
	hour = nCode & 0x1f;

	return 60*60*hour + 60*minute + second; 
}

int MpegInfo_GetPTS(unsigned char *buffer)
{
	unsigned long	time;

	// PTS have 33 bit and should be divided by 90k to get time (sec)
	// We only get 32 bit and divided by 45k to get the same result
	time = ((unsigned long) *buffer & 0xE) << 29;
	time |= ((unsigned long) *(buffer + 1)) << 21;
	time |= ((unsigned long) *(buffer + 2) & 0xFE) << (14 -1);
	time |= ((unsigned long) *(buffer + 3)) << 6;
	time |= ((unsigned long) *(buffer + 4) & 0xFE) >> 2;

	// Add 22500 for rounding
	time = (time + 22500) / 45000;
	
	return (int) time;
}


int CreateVideoIndexFileForMPG(CFile inFile, CFile outFile, BOOL ForceGOP)
{
	unsigned char	vbuf[MPEG1_PACKET_SIZE*2];
	ULONG			nCode, packetCount, packetNo;
	int				i, GOPTime, prevGOPTime = -1, GOPTimeDiff, findGOPStart, startVideoStraeam, usePTS = TRUE;
	unsigned char	PTSFlag, PTSType;

    if (ForceGOP == TRUE)
        usePTS = FALSE;
	inFile.Seek(0, CFile::begin);
	packetCount = (ULONG) ((inFile.GetLength() + MPEG2_PACKET_SIZE - 1) / MPEG2_PACKET_SIZE);
	findGOPStart = FALSE;
	for (packetNo = 0; packetNo < packetCount; packetNo++)
	{
		inFile.Read(vbuf, MPEG2_PACKET_SIZE);
		nCode = *((ULONG *)vbuf);

		if (nCode == PACK_START_CODE)
		{
			i = 14;
			nCode = *((ULONG *)(vbuf + i));

			if (nCode == SYSTEM_START_CODE)
			{
				// skip this system start header
				// skip the start code(4), the size of header(2) and header of this part
				WORD headerSize = *((BYTE *)(vbuf + i + 4)) * 256 + *((BYTE *)(vbuf + i + 5));
				i += (4 + 2 + headerSize);

				// Get next code
				nCode = *((ULONG *)(vbuf + i));
			}

			if (nCode == VIDEO_ELEMENTARY_STREAM)
			{
				// Keep this value if we want to use PTS
				startVideoStraeam = i;

				// skip the elementary code(4) and header of this part (5)
				i += (4 + 5);

				while (i < MPEG2_PACKET_SIZE)
				{
					nCode = *((ULONG *)(vbuf + i));

					if (nCode == GROUP_START_CODE)
					{
						if (findGOPStart)
						{
							// Doesn't find the next PICTURE_START_CODE, so we set the end no of packet here.
							outFile.Write(&packetNo, 4);
							findGOPStart = FALSE;
						}
						
						PTSFlag = *(vbuf + startVideoStraeam + 7) & 0xC0;
						PTSType = *(vbuf + startVideoStraeam + 9) & 0x30;

						if ((prevGOPTime == -1) && !((PTSFlag == 0x80) && (PTSType == 0x20)) &&
							!((PTSFlag == 0xC0) && (PTSType == 0x30)))
						{
							// The first I Frame, but don't have PTS
							usePTS = FALSE;
						}

						if (usePTS)
						{
							if (!((PTSFlag == 0x80) && (PTSType == 0x20)) &&
								!((PTSFlag == 0xC0) && (PTSType == 0x30)))
							{
								// Cannot get PTS in some frame, don't generate video index file
								return FALSE;
							}

							GOPTime = MpegInfo_GetPTS(vbuf + startVideoStraeam + 9);
						}
						else
						{
							nCode = *((ULONG *)(vbuf + i + 4));
							GOPTime = MpegInfo_GetGOP(ntohl(nCode));
						}

						if (prevGOPTime == -1)
						{
							prevGOPTime = GOPTime;
							GOPTimeDiff = GOPTime;
							GOPTime = 0;
						}
						else if (GOPTime == prevGOPTime)
						{
							// skip this frame becuase we only want one frame per second
							break;
						}
						else
						{
							if ((GOPTime > (prevGOPTime + 60)) || (GOPTime < prevGOPTime))
							{
								// If the time of two GOP have 60 second ellapse, we set this GOP time as the previous GOP time
								GOPTimeDiff = GOPTime - (prevGOPTime - GOPTimeDiff) - 1;
							}

							prevGOPTime = GOPTime;
							GOPTime -= GOPTimeDiff; 
						}

						// Set we have found GOP, then we need to find the next PICTURE_START_CODE
						findGOPStart = TRUE;
						outFile.Write(&packetNo, 4);
						outFile.Write(&GOPTime, 4);
						break;
					}
					else if (nCode == PICTURE_START_CODE && findGOPStart)
					{
						// find the end the I frame
						outFile.Write(&packetNo, 4);
						findGOPStart = FALSE;
						break;
					}
					i++;
				}
			}
		}
	}
	
	if (findGOPStart)
	{
		// Doesn't find the next PICTURE_START_CODE, so we set the end no of packet here.
		outFile.Write(&packetNo, 4);
		findGOPStart = FALSE;
	}

	return TRUE;
}


// Create index files for VOB, we can use navigation pack to create index quickly
int CreateVideoIndexFileForVOB(CFile inFile, CFile outFile)
{
	LONGLONG		lOffset;
	unsigned char	vbuf[MPEG1_PACKET_SIZE*2];
	ULONG			nCode, packetCount, packetNo, nextNaviPack, endOfIFrame;
	int				time, prevTime = -1, timeDiff, i;

	packetCount = (ULONG) ((inFile.GetLength() + MPEG2_PACKET_SIZE - 1) / MPEG2_PACKET_SIZE);

	for (packetNo = 0; packetNo < packetCount; packetNo = nextNaviPack)
	{
		lOffset = packetNo * MPEG2_PACKET_SIZE;
		inFile.Seek(lOffset, CFile::begin);
		if (inFile.Read(vbuf, MPEG2_PACKET_SIZE) != MPEG2_PACKET_SIZE)
		{
			// Cannot get complete packet
			return FALSE;
		}

		if ((*(ULONG *)vbuf == PACK_START_CODE) && ((*(ULONG *)(vbuf + 0x26)) == PRIVATE_STREAM_2))
		{
			// Get the VOBU presentation time
			nCode = ntohl(*(ULONG *)(vbuf + 0x39));

			// The VOBU presentation time use 90kHz clock, add 45000 for rounding
			time = (nCode + 45000) / 90000;

			// The packet of next navi pack
			nextNaviPack = packetNo + ntohl(*(ULONG *)(vbuf + 0x40f)) + 1;

			if (prevTime == -1)
			{
				prevTime = time;
				timeDiff = time;
				time = 0;
			}
			else if (time == prevTime)
			{
				// skip this frame becuase we only want one frame per second
				continue;
			}
			else
			{
				if ((time > (prevTime + 60)) || (time < prevTime))
				{
					// If the time of two VOBU have 60 second ellapse, we set this time as the time of previous frame
					timeDiff = time - (prevTime - timeDiff) - 1;
				}

				prevTime = time;
				time -= timeDiff; 
			}
			
			// The end packet of the fisrt I frame
			endOfIFrame = packetNo + ntohl(*(ULONG *)(vbuf + 0x413));

			outFile.Write(&packetNo, 4);
			outFile.Write(&time, 4);
			outFile.Write(&endOfIFrame, 4);
		}
		else 
		{
			// Cannot find navi pack in the start
			if (prevTime == -1)
			{
				nCode = *((ULONG *)vbuf);

				if (nCode == PACK_START_CODE)
				{
					i = 14;
					nCode = *((ULONG *)(vbuf + i));

					if (nCode == SYSTEM_START_CODE)
					{
						// skip this system start header
						// skip the start code(4), the size of header(2) and header of this part
						WORD headerSize = *((BYTE *)(vbuf + i + 4)) * 256 + *((BYTE *)(vbuf + i + 5));
						i += (4 + 2 + headerSize);

						// Get next code
						nCode = *((ULONG *)(vbuf + i));
					}

					if (nCode == VIDEO_ELEMENTARY_STREAM)
					{
						// skip the elementary code(4) and header of this part (5)
						i += (4 + 5);

						while (i < MPEG2_PACKET_SIZE)
						{
							nCode = *((ULONG *)(vbuf + i));

							if (nCode == GROUP_START_CODE)
							{
								// No navi pack, but we have found GOP
								return FALSE;
							}

							i++;
						}
					}
				}

				// Cannot find navi pack in the start, search the start navi pack
				nextNaviPack = packetNo + 1;
			}
			else
			{	
				// Cannot find navi pack in the middle, Cannot use this method
				return FALSE;
			}
		}
	}

	return TRUE;
}

int AviInfoMainHeaderParser(AVIMAINHEADER *aviH, DWORD *buf, unsigned long size)
{
    if (aviH == NULL || buf == NULL)
	{
		return -1;
	}

    if (buf[0] != avihtag ||  size < buf[1] + 8)
	{
		return -1;
	}

    memcpy(aviH, buf, sizeof(AVIMAINHEADER));

    return (buf[1] >> 2) + 2;
}

int AviInfoStreamHeaderParser(AVISTREAMHEADER *aviH, DWORD *buf, unsigned long size)
{
    int d = 0, pos = 0;

    if (buf[0] != strhtag ||  size < buf[1] + 8)
    {
        return -1;
    }

	memset(aviH, 0, sizeof(AVISTREAMHEADER));

    aviH->fcc = buf[pos++];
    aviH->cb = buf[pos++];
    aviH->fccType = buf[pos++];
    aviH->fccHandler = buf[pos++];
    aviH->dwFlags = buf[pos++];

	d = buf[pos++];
    aviH->wPriority = (d&0xFFFF);
    aviH->wLanguage = (d>>16);
    aviH->dwInitialFrames = buf[pos++];
    aviH->dwScale = buf[pos++];
    aviH->dwRate = buf[pos++];
    aviH->dwStart = buf[pos++];
    aviH->dwLength = buf[pos++];
    aviH->dwSuggestedBufferSize = buf[pos++];
    aviH->dwQuality = buf[pos++];
    aviH->dwSampleSize = buf[pos++];

	d = buf[pos++];
    (aviH->rcFrame).left = (d&0xFFFF);
    (aviH->rcFrame).top  = (d>>16);

    d = buf[pos++];
    (aviH->rcFrame).right = (d&0xFFFF);
    (aviH->rcFrame).bottom  = (d>>16);

    return (aviH->cb >> 2) + 2;
}

int AviInfoWaveFormatMatExParser(WAVEFORMATEX *wfm, DWORD *buf, unsigned long size)
{

    if (wfm == NULL || buf == NULL)
	{
		return -1;
	}

    if (buf[0] != strftag ||  size < buf[1] + 8)
	{
		return -1;
	}

    memcpy(wfm, &buf[2], sizeof(WAVEFORMATEX));

    return (buf[1]>>2)+2;
}

unsigned long  AviInfoGetIdx1Position(CFile inFile, unsigned long offset)
{
	DWORD		vbuf[AVI_IDX1_BUFFER_DW_SIZE];
	int			byteRead, retVal = 0;
	int			pos = 0;

    while (1)
    {
		inFile.Seek(offset, CFile::begin);
		byteRead = inFile.Read(vbuf, AVI_IDX1_BUFFER_SIZE);
		if (byteRead != AVI_IDX1_BUFFER_SIZE)
		{
			return 0;
		}

        if (vbuf[0] == LISTtag && vbuf[2] == movitag)
        {
            aviInfo.moviPos = offset + 8;
            retVal = vbuf[1] + aviInfo.moviPos;
            break;
        }
        else if(vbuf[0] == JUNKtag || vbuf[0] == LISTtag)
        {
            offset += (vbuf[pos+1] + 8);
        }
        else
        {
            // Cannot find idx1 position
            break;
        }
    }

    return retVal;
}


int CreateVideoIndexFileForAVI(CFile inFile, CFile outFile)
{
	unsigned long	vbuf[AVI_BUFFER_SIZE];
	unsigned long	idx1Pos, idx1Size, idxNo, idxTotalNo, writeIdxNo = 0;
	unsigned long	headerSize = 0, writeDataSize = 0;
    int				byteRead, dwordRead, pos = 0 ,r = 0, headerCount = 0, i;
	int		queryResult = 0;

	inFile.Seek(0, CFile::begin);
	if (inFile.Read(vbuf, AVI_START_HEADER_SIZE) != AVI_START_HEADER_SIZE)
	{
		// Cannot get complete packet
		return FALSE;
	}

    if(vbuf[0] != RIFFtag || vbuf[2] != AVI_tag || vbuf[3] != LISTtag || vbuf[5] != hdrltag)
    {
		// This file is not a valid AVI fornat
		return FALSE;
    }

    headerSize = vbuf[4];
	
	byteRead = inFile.Read(vbuf, headerSize + 20);
	if (byteRead != (headerSize + 20))
	{
		// Cannot get complete header
		return FALSE;
	}

	dwordRead = byteRead >> 2;

    while (((pos + 2) < dwordRead) && (headerCount < 3))
    {
        if (vbuf[pos] == avihtag)
        {
            r = AviInfoMainHeaderParser(&aviHeader, vbuf + pos, (dwordRead - pos) * 4);
            if (r < 1)
			{	
				break;
			}

            pos += r;
            headerCount++;
        }
        else if (vbuf[pos] == strhtag && vbuf[pos+2] == vidstag)
        {
            r = AviInfoStreamHeaderParser(&vidsHeader, vbuf + pos, (dwordRead - pos) * 4);

            if (r < 1)
			{
				break;
			}
            /*
            if (vidsHeader.fcc == mp42tag)
            {
                _stprintf(strSQL,_T("update FSE_QueryState SET where FilePath =\"%s\""), inFile.GetFilePath());
        	    queryResult = db_getOneRecordT(strSQL, &tmpFSE_QS);
            };*/

            pos += r;
            headerCount++;
        }
		else if (vbuf[pos] == strhtag && vbuf[pos+2] == audstag)
        {
            r = AviInfoStreamHeaderParser(&audsHeader, vbuf + pos, (dwordRead - pos) * 4);
            if (r < 1)
			{
				break;
			}
            pos += r;

            if (vbuf[pos] == strftag)
            {
                r = AviInfoWaveFormatMatExParser(&waveFormatex, vbuf + pos, (dwordRead - pos) * 4);
                pos += r;
            }
			headerCount++;
        }
        else
            pos++;
    }
	/*
    if (headerCount < 3)
	{
        // Ccould not parse header
		return FALSE;
	}
	*/
    if (aviHeader.dwTotalFrames <= 0)
	{
		// There is no any frame
		return FALSE;
	}

	// Skip start header and header 
    idx1Pos = AviInfoGetIdx1Position(inFile, headerSize + 20);
    if (idx1Pos == 0)
	{
		return FALSE;
	}

	inFile.Seek(idx1Pos , CFile::begin);
	byteRead = inFile.Read(vbuf, AVI_IDX1_SIZE_BUFFER_SIZE);
	if (byteRead != AVI_IDX1_SIZE_BUFFER_SIZE)
	{
		return FALSE;
	}

    if (vbuf[0] == idx1tag)
    {
        idx1Size = vbuf[1];
    }
	else
	{
		return FALSE;
	}

	// Reserve space for the data size
	outFile.Write(&writeDataSize, 4);

	// Each entry has 16 bytes
	idxTotalNo = idx1Size / AVI_IDX_ENTRY_SIZE;
	idxNo = 0; 
	while (1)
	{
		byteRead = inFile.Read(vbuf, AVI_BUFFER_SIZE);
		if (byteRead < AVI_IDX_ENTRY_SIZE)
		{
			// End of File
			goto Exit;
		}
		
		for (i = 0; i < (byteRead / AVI_IDX_ENTRY_DW_SIZE); i += AVI_IDX_ENTRY_DW_SIZE)
		{
			if ((vbuf[i] == Tag00dc) || (vbuf[i] == Tag00db))
			{
				// This is Video Frames. output these info
				outFile.Write(&vbuf[i], 16);
				writeIdxNo++;
			}

			idxNo++;
			if (idxNo >= idxTotalNo)
			{
				goto Exit;
			}
		}
	}

Exit:
	// Write data size to file
	writeDataSize = writeIdxNo * AVI_IDX_ENTRY_SIZE;
	outFile.Seek(4, CFile::begin);
	outFile.Write(&writeDataSize, 4);
	return TRUE;
}


void CreateVideoIndexFile(CString filepath)
{
	struct FSE_QueryState tmpFSE_QS;
	CString			outFilepath, hdxFilepath, strExt;
	CFile			inFile, outFile;
	CFileStatus		fileStatus ;
	unsigned char	vbuf[MPEG1_PACKET_SIZE*2];
	int				pos, scanOK, queryResult = 0;
	TCHAR			newFileName[MAX_PATH_LENGTH], strSQL[MAX_SQL_STR_LENGTH];

	// We use the first index in database as the name of video index file
	_stprintf(strSQL,_T("select * from FSE_QueryState where FilePath =\"%s\""), filepath.GetString());
	queryResult = db_getOneRecordT(strSQL, &tmpFSE_QS);

	if (!queryResult)
	{
		// Doesn't exist, skip it
		return;
	}

	// Get the working directory, we will store video index files in it
	_stprintf(newFileName, _T("%s\\cache\\%d%s"), gWorkingDir, tmpFSE_QS.Index, (TCHAR *) tmpFSE_QS.FileName);
	outFilepath = newFileName;
	hdxFilepath = outFilepath;
	outFilepath.Append(_T(".tmp"));
	hdxFilepath.Append(_T(".hdx"));

	pos = filepath.ReverseFind(_T('.'));
	strExt = filepath.Right(filepath.GetLength() - pos);

	// delete the exist index file (.hdx) in case that render access this index file.
	try
	{
		if (CFile::GetStatus(hdxFilepath, fileStatus))
		{
			fileStatus.m_attribute &= ~0x02;				// Remove hidden attribute;
			CFile::SetStatus(hdxFilepath, fileStatus);
		}

		if (outFile.Open(hdxFilepath, CFile::modeCreate | CFile::modeWrite))
		{
			outFile.Close();
			CFile::Remove(hdxFilepath);
		}
		else
		{
			return;
		}

		if (!inFile.Open(filepath, CFile::modeRead | CFile::shareDenyNone))
			return;
	
		inFile.Read(vbuf, MPEG1_PACKET_SIZE*2);
	
		memset(&mpegInfo, 0, sizeof(mpegInfo));	
		if (!(strExt.CompareNoCase(_T(".avi")) == 0) && 
			!MpegInfo_CheckType(&mpegInfo, vbuf, MPEG1_PACKET_SIZE*2) && (mpegInfo.type != MPEGINFO_TYPE_MPEG2))
		{
			// We only support MPEG2 and MPEG4 AVI in this stage,so skip other formats
			inFile.Close();
			return;
		}

		if (!outFile.Open(outFilepath, CFile::modeCreate | CFile::modeWrite))
		{
			inFile.Close();
			return;
		}

		// Write a tag to distiguish this with other files
		outFile.Write("hidx", 4);

		scanOK = FALSE;
		if (strExt.CompareNoCase(_T(".avi")) == 0)
		{
			scanOK = CreateVideoIndexFileForAVI(inFile, outFile);
		}
		else
		{
			//if (strExt.CompareNoCase(_T(".vob")) == 0) 
			if  ((strExt.CompareNoCase(_T(".vob")) == 0) ||
				(strExt.CompareNoCase(_T(".ts")) == 0) ||
				(strExt.CompareNoCase(_T(".m2ts")) == 0) ||
				(strExt.CompareNoCase(_T(".tp")) == 0) )
			{
				scanOK = CreateVideoIndexFileForVOB(inFile, outFile);
			}

			if (!scanOK)
			{
				// Reset the output file
				outFile.Seek(4, CFile::begin);

				// The file is not vob, or there is something wrong in this vob
				scanOK = CreateVideoIndexFileForMPG(inFile, outFile, FALSE);
                if (scanOK == FALSE)
                {
				    outFile.Seek(4, CFile::begin);
                    scanOK = CreateVideoIndexFileForMPG(inFile, outFile, TRUE);
                }
			}
		}
	
		if (outFile.GetLength() <= 8)
		{
			// It is not a valid index file
			scanOK = FALSE;
		}

		inFile.Close();
		outFile.Close();

		if (scanOK)
		{
			CFile::Rename(outFilepath, hdxFilepath);
			if (CFile::GetStatus(hdxFilepath, fileStatus))
			{
				fileStatus.m_attribute |= 0x02;				// Hidden;
				CFile::SetStatus(hdxFilepath, fileStatus);
			}
		}
		else
		{
			CFile::Remove(outFilepath);
		}
	}
	catch (CFileException* pEx)
	{
		pEx->Delete();
		return;
	}

	return;
}


int	AddVideoFileToScanList(CString filepath)
{
	if ((scanVideoFileList == NULL) || (hAddScanVideoFileMutex == NULL))
	{
		// Failed in initialization step, skip
		return FALSE;
	}

	while (WaitForSingleObject(hAddScanVideoFileMutex, 100L) != WAIT_OBJECT_0)
		Sleep(100);

	if (fileNoInScanList >= scanListSize)
	{
		TCHAR	*pNewBuffer = NULL;

		scanListSize += INC_SCAN_FILES_NO;
		pNewBuffer = (TCHAR *) realloc(scanVideoFileList, sizeof(TCHAR) * MAX_PATH_LENGTH * scanListSize);
		if (pNewBuffer == NULL)
		{
			ReleaseMutex(hAddScanVideoFileMutex);
			return FALSE;
		}

		scanVideoFileList = pNewBuffer;
	}

	_tcscpy((scanVideoFileList + (fileNoInScanList * MAX_PATH_LENGTH)), filepath.GetString());
	fileNoInScanList++;
	skipScanVideoFiles = FALSE;

	ReleaseMutex(hAddScanVideoFileMutex);

	return fileNoInScanList;
}

DWORD WINAPI ScanVideoFiles(LPVOID args)
{
	CString	filepath;
	int		i;

	while (1)
	{
		while (WaitForSingleObject(hAddScanVideoFileMutex, 100L) != WAIT_OBJECT_0)
			Sleep(1000);
	
		if ((fileNoInScanList == 0) || (skipScanVideoFiles))
		{
			fileNoInScanList = 0;
			ReleaseMutex(hAddScanVideoFileMutex);

			// There is no file need to scan, sleep 5 second
			Sleep(5000);
			continue;
		}

		ReleaseMutex(hAddScanVideoFileMutex);

		i = 0;
		while (!skipScanVideoFiles)
		{
			filepath = scanVideoFileList + (i * MAX_PATH_LENGTH);
			CreateVideoIndexFile(filepath);
			i++;
		
			while (WaitForSingleObject(hAddScanVideoFileMutex, 100L) != WAIT_OBJECT_0)
				Sleep(1000);
		
			if (i >= fileNoInScanList)
			{
				// We have parsed all video files
				fileNoInScanList = 0;
				ReleaseMutex(hAddScanVideoFileMutex);
				break;
			}

			ReleaseMutex(hAddScanVideoFileMutex);
		}

		if (skipScanVideoFiles)
		{
			while (WaitForSingleObject(hAddScanVideoFileMutex, 100L) != WAIT_OBJECT_0)
				Sleep(1000);
			
			// Clear all the files in scan list
			fileNoInScanList = 0;
			ReleaseMutex(hAddScanVideoFileMutex);
		}
	}
	
	return 1;
}


void DeleteVideoIndexFile(CString filepath)
{
	struct FSE_QueryState tmpFSE_QS;
	CString			hdxFilepath;
	CFile			outFile;
	CFileStatus		fileStatus ;
	int				queryResult = 0;
	TCHAR			newFileName[MAX_PATH_LENGTH], strSQL[MAX_SQL_STR_LENGTH];

	// We use the first index in database as the name of video index file
	_stprintf(strSQL,_T("select * from FSE_QueryState where FilePath =\"%s\""), filepath.GetString());
	queryResult = db_getOneRecordT(strSQL, &tmpFSE_QS);
	
	if (!queryResult)
	{
		// Doesn't exist, skip it
		return;
	}

	// Get the working directory, we will store video index files in it
	_stprintf(newFileName, _T("%s\\cache\\%d%s"), gWorkingDir, tmpFSE_QS.Index, (TCHAR *) tmpFSE_QS.FileName);
	hdxFilepath = newFileName;
	hdxFilepath.Append(_T(".hdx"));

	// delete the exist index file (.hdx) in case that render access this index file.
	try
	{
		if (CFile::GetStatus(hdxFilepath, fileStatus))
		{
			fileStatus.m_attribute &= ~0x02;				// Remove hidden attribute;
			CFile::SetStatus(hdxFilepath, fileStatus);
		}

		if (outFile.Open(hdxFilepath, CFile::modeCreate | CFile::modeWrite))
		{
			outFile.Close();
			CFile::Remove(hdxFilepath);
		}
		else
		{
			return;
		}
	}
	catch (CFileException* pEx)
	{
	   pEx->Delete();
	   return;
	}
}

int		StartScanVideoFilesThread()
{
	scanVideoFileList = (TCHAR *) MALLOC(sizeof(TCHAR) * MAX_PATH_LENGTH * INC_SCAN_FILES_NO);
	if (scanVideoFileList == NULL)
	{
		return FALSE;
	}

	scanListSize = INC_SCAN_FILES_NO;
	fileNoInScanList = 0;	
	
	// Create a mutex with no initial owner that will protect scanVideoFileList
	hAddScanVideoFileMutex = CreateMutex(
		NULL,                       // default security attributes
		FALSE,                      // initially not owned
		NULL);                      // unnamed mutex

	if (hAddScanVideoFileMutex == NULL)
	{
		return FALSE;
	}

	ScanVideoFilesThreadHandle = ::CreateThread(NULL, 0, ScanVideoFiles, NULL, 0, &ScanVideoFilesThreadId);
	if (ScanVideoFilesThreadHandle == NULL)
	{
		return FALSE;
	}

	// Lower the priority of this thread because we won't use too much system resource 
	SetThreadPriority(ScanVideoFilesThreadHandle, THREAD_PRIORITY_LOWEST);

	return TRUE;
}


void	TerminateScanVideoFilesThread(void)
{
	while (WaitForSingleObject(hAddScanVideoFileMutex, 100L) != WAIT_OBJECT_0)
		Sleep(100);

	if (ScanVideoFilesThreadHandle)
	{
		TerminateThread(ScanVideoFilesThreadHandle, 0);
		ScanVideoFilesThreadHandle = NULL;
	}

	if (scanVideoFileList != NULL)
	{
		FREE(scanVideoFileList);
		scanVideoFileList = NULL;
	}
	
	CloseHandle(hAddScanVideoFileMutex);
}