#ifndef __VIDEOINFO_H__
#define __VIDEOINFO_H__

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#define	INC_SCAN_FILES_NO			100

// Defines for MPEG1, MPEG2, MPEG4 files
/* code definition */
#define PICTURE_START_CODE			0x00010000
#define SLICE_START_CODE_MIN		0x01010000
#define SLICE_START_CODE_MAX		0xAF010000
#define USER_DATA_START_CODE		0xB2010000
#define SEQUENCE_HEADER_CODE		0xB3010000
#define EXTENSION_START_CODE		0xB5010000
#define SEQUENCE_END_CODE			0xB7010000
#define GROUP_START_CODE			0xB8010000

#define SYSTEM_END_CODE				0xB9010000
#define PACK_START_CODE				0xBA010000
#define SYSTEM_START_CODE			0xBB010000

#define VIDEO_ELEMENTARY_STREAM		0xE0010000

#define PRIVATE_STREAM_1			0xBD010000
#define PRIVATE_STREAM_2			0xBF010000
#define AUDIO_ELEMENTARY_STREAM_0	0xC0010000
#define AUDIO_ELEMENTARY_STREAM_1	0xC1010000
#define AUDIO_ELEMENTARY_STREAM_2	0xC2010000
#define AUDIO_ELEMENTARY_STREAM_3	0xC3010000
#define AUDIO_ELEMENTARY_STREAM_4	0xC4010000
#define AUDIO_ELEMENTARY_STREAM_5	0xC5010000
#define AUDIO_ELEMENTARY_STREAM_6	0xC6010000
#define AUDIO_ELEMENTARY_STREAM_7	0xC7010000

#define SUB_SUB						0x20
#define SUB_AC3						0x80
#define SUB_DTS						0x88
#define SUB_PCM						0xA0

#define MPEG1_PACKET_SIZE 2352
#define MPEG2_PACKET_SIZE 2048

// Defines for AVI files
#define AVI_START_HEADER_DW_SIZE		6
#define AVI_START_HEADER_SIZE			(AVI_START_HEADER_DW_SIZE * 4)
#define AVI_BUFFER_DW_SIZE				4096
#define AVI_BUFFER_SIZE					(AVI_BUFFER_DW_SIZE * 10)
#define AVI_IDX1_BUFFER_DW_SIZE			3
#define AVI_IDX1_BUFFER_SIZE			(AVI_IDX1_BUFFER_DW_SIZE * 4)
#define AVI_IDX1_SIZE_BUFFER_DW_SIZE	2
#define AVI_IDX1_SIZE_BUFFER_SIZE		(AVI_IDX1_SIZE_BUFFER_DW_SIZE * 4)
#define AVI_IDX_ENTRY_DW_SIZE			4
#define AVI_IDX_ENTRY_SIZE				(AVI_IDX_ENTRY_DW_SIZE * 4)
#define FOURCC							DWORD

#define MAKEFOURCC(a,b,c,d) ( ((int)a)      | (((int)b)<< 8) | \
                              (((int)c)<<16) | (((int)d)<<24)  )
#define RIFFtag MAKEFOURCC('R','I','F','F')
#define LISTtag MAKEFOURCC('L','I','S','T')
#define JUNKtag MAKEFOURCC('J','U','N','K')
#define AVI_tag MAKEFOURCC('A','V','I',' ')
#define hdrltag MAKEFOURCC('h','d','r','l')
#define avihtag MAKEFOURCC('a','v','i','h')
#define strltag MAKEFOURCC('s','t','r','l')
#define strhtag MAKEFOURCC('s','t','r','h')
#define strftag MAKEFOURCC('s','t','r','f')
#define vidstag MAKEFOURCC('v','i','d','s')
#define audstag MAKEFOURCC('a','u','d','s')
#define dmlhtag MAKEFOURCC('d','m','l','h')
#define movitag MAKEFOURCC('m','o','v','i')
#define idx1tag MAKEFOURCC('i','d','x','1')
#define Tag00db MAKEFOURCC('0','0','d','b')
#define Tag00dc MAKEFOURCC('0','0','d','c')
#define Tag00wb MAKEFOURCC('0','0','w','b')
#define Tag01wb MAKEFOURCC('0','1','w','b')
#define Tagwave MAKEFOURCC('f','m','t',' ')
#define Tagdata MAKEFOURCC('d','a','t','a')
#define TagDATA MAKEFOURCC('D','A','T','A')
#define Tag00__ MAKEFOURCC('0','0','_','_')

//for wave file
#define WAVEtag MAKEFOURCC('W','A','V','E')
#define fmt_tag MAKEFOURCC('f','m','t',' ')
#define Tagdata MAKEFOURCC('d','a','t','a')

#define mp42tag MAKEFOURCC('M','P','4','2')
#define DIVXtag MAKEFOURCC('D','I','V','X')
#define divxtag MAKEFOURCC('d','i','v','x')
#define XVIDtag MAKEFOURCC('X','V','I','D')
#define xvidtag MAKEFOURCC('x','v','i','d')
#define DIV3tag MAKEFOURCC('D','I','V','3')
#define div3tag MAKEFOURCC('d','i','v','3')
#define DIV4tag MAKEFOURCC('D','I','V','4')
#define div4tag MAKEFOURCC('d','i','v','4')
#define DIV5tag MAKEFOURCC('D','I','V','5')
#define div5tag MAKEFOURCC('d','i','v','5')

// structure for MPEG1, MPEG2, and MPEG4 files
enum MpegInfo_Type
{
  MPEGINFO_TYPE_UNKNOWN = 0,
  MPEGINFO_TYPE_MPEG1,
  MPEGINFO_TYPE_MPEG2,
  MPEGINFO_TYPE_SVCD
};

struct MpegInfo_Struct
{
  int GOPList[4096];
  int GOPIndex;
  int currentTime;
  int totalTime;
  int total_s;
  int total_m;
  int total_h;
  int checkVOBid;
  int avgKbps;
  int maxKbps;
  int kbps;
  int packetSize;
  int fps;
  enum MpegInfo_Type type;
};


// structure for AVI files
typedef struct _avioldindex_entry{
  DWORD dwChunkId;
  DWORD dwFlags;
  DWORD dwOffset;
  DWORD dwSize;
  DWORD dwTotalSize;
  WORD  wFrame;
}AVIOLDINDEX_ENTRY;

typedef struct _aviindex{
  DWORD pos;
  DWORD size;
  AVIOLDINDEX_ENTRY wb;
  WORD  iskey;
}AVIINDEX;

typedef struct _aviinfo{
  int moviPos;
  unsigned char *hdr; // 0xmoviPos(00xx ~xxx)00dc
  int hdr_len;
  int currIndex;
  int total_num;
  int total_num_wb;
  int playedTime;
  int totalTime;
  int totalTime_h;
  int totalTime_m;
  int totalTime_s;
  int kbps;
  int fps;
  int iframe_first;
  int iframe_last;
}AVIINFOSTRUCT;

typedef struct _avimainhader{
  FOURCC fcc;
  DWORD cb;
  DWORD dwMicroSecPerFrame;
  DWORD dwMaxBytesPerSec;
  DWORD dwPaddingGranularity;
  DWORD dwFlags;
  DWORD dwTotalFrames;
  DWORD dwInitialFrames;
  DWORD dwStreams;
  DWORD dwSuggestedBufferSize;
  DWORD dwWidth;
  DWORD dwHeight;
  DWORD dwReserved[4];
}AVIMAINHEADER;


typedef struct _avistreamheader{
  FOURCC fcc;
  DWORD  cb;
  FOURCC fccType;
  FOURCC fccHandler;
  DWORD  dwFlags;
  WORD   wPriority;
  WORD   wLanguage;
  DWORD  dwInitialFrames;
  DWORD  dwScale;
  DWORD  dwRate;
  DWORD  dwStart;
  DWORD  dwLength;
  DWORD  dwSuggestedBufferSize;
  DWORD  dwQuality;
  DWORD  dwSampleSize;
  struct{
    short int left;
    short int top;
    short int right;
    short int bottom;
  } rcFrame;
} AVISTREAMHEADER;

typedef struct _waveformatmatex{
  WORD  wFormatTag;
  WORD  nChannels;
  DWORD nSamplesPerSec;
  DWORD nAvgBytesPerSec;
  WORD  nBlockAlign;
  WORD  wBitsPerSample;
  WORD  cbSize;
} WAVEFORMATEX;

int				AddVideoFileToScanList(CString filepath);
void			DeleteVideoIndexFile(CString filepath);
int				StartScanVideoFilesThread();
void			TerminateScanVideoFilesThread(void);

extern int		skipScanVideoFiles;

#endif	// __VIDEOINFO_H__
