#include "dbtools.h"
#include <WINSOCK.h>
#include <windows.h>
#include <wmsdk.h>
#include "Registry.h"
#include "OEM.h"

extern "C" {
	#include "FileSystemTree.h"
	#include "mystring.h"
	#include "debug.h"
}

extern "C" TCHAR	gWorkingDir[MAX_PATH_LENGTH];

CADODatabase	*g_pDb;
HANDLE			hDBMutex; 
CString			databasePath;
CADORecordset	*pRsGetRecords;
CADORecordset	*pRsGetRecordsT;
char			prevSQLstringGetRecords[MAX_SQL_STR_LENGTH];
TCHAR			prevSQLstringGetRecordsT[MAX_SQL_STR_LENGTH];

static	int		flushDBQuery = FALSE;

void FlushCacheDBQuery()
{
	flushDBQuery = TRUE;
}

// Unicore Version
extern "C" int db_getCountT(const TCHAR *SQLstring)
{
	CADORecordset* pRs = new CADORecordset(g_pDb);
	int		count;

	while (WaitForSingleObject(hDBMutex, 100L) != WAIT_OBJECT_0)
		Sleep(100);

	if(pRs->Open(SQLstring)) 
	{
		count = pRs->GetRecordCount();
		pRs->Close();
	}
	else
	{
		AfxMessageBox(_T("database not exists,db_getCount()"));
		count =  0;
	}

	delete pRs;
	ReleaseMutex(hDBMutex);
	return count;
}


// Utf8 Version
extern "C" int db_getCount(const char *SQLstring)
{
	CADORecordset* pRs = new CADORecordset(g_pDb);
	TCHAR	strSQL[MAX_SQL_STR_LENGTH];
	int		count;

	while (WaitForSingleObject(hDBMutex, 100L) != WAIT_OBJECT_0)
		Sleep(100);

	Utf8ToStr(strSQL, SQLstring, MAX_SQL_STR_LENGTH);
	if(pRs->Open(strSQL)) 
	{
		count = pRs->GetRecordCount();
		pRs->Close();
	}
	else
	{
		AfxMessageBox(_T("database not exists,db_getCount()"));
		count =  0;
	}

	delete pRs;
	ReleaseMutex(hDBMutex);
	return count;
}


// Unicode version
extern "C" int db_getOneRecordT(const TCHAR *SQLstring, struct FSE_QueryState *tmpFSE_QS)
{
	CADORecordset*	pRs = new CADORecordset(g_pDb);
	CString TempSql;
	int		count = 0;

	while (WaitForSingleObject(hDBMutex, 100L) != WAIT_OBJECT_0)
		Sleep(100);


	if(pRs->Open(SQLstring))
	{
		count = pRs->GetRecordCount();

		if (count != 0)
		{
			// set the data in testFSE_QS , total 15 fields;
			pRs->GetFieldValue(_T("MyIndex"),TempSql);		
			tmpFSE_QS->Index = _ttoi(TempSql);

			pRs->GetFieldValue(_T("PIndex"),TempSql);
			tmpFSE_QS->pIndex = _ttoi(TempSql);

			pRs->GetFieldValue(_T("Flags"),TempSql);
			tmpFSE_QS->Flags = _ttoi(TempSql);

			pRs->GetFieldValue(_T("ObjectType"),TempSql);
			tmpFSE_QS->ObjectType = (FSE_ObjectType)_ttoi(TempSql);

			pRs->GetFieldValue(_T("FilePathLen"),TempSql);
			tmpFSE_QS->FilePathLen = _ttoi(TempSql);
			tmpFSE_QS->DirFilePathLen = tmpFSE_QS->FilePathLen;

			pRs->GetFieldValue(_T("FilePath"),TempSql);
			_tcscpy((TCHAR *) tmpFSE_QS->FilePath, TempSql.GetString());
			_tcscpy((TCHAR *) tmpFSE_QS->DirFilePath, TempSql.GetString());

			pRs->GetFieldValue(_T("FileName"),TempSql);
			_tcscpy((TCHAR *) tmpFSE_QS->FileName, TempSql.GetString());

			// set uri
			sprintf(tmpFSE_QS->ProcessPath,"\\%d\\%s", tmpFSE_QS->Index, tmpFSE_QS->FileName);

			pRs->GetFieldValue(_T("FileNameLen"),TempSql);
			tmpFSE_QS->FileNameLen = _ttoi(TempSql);

			pRs->GetFieldValue(_T("isSearchable"),TempSql);
			tmpFSE_QS->isSearchable = _ttoi(TempSql);

			pRs->GetFieldValue(_T("ProcessIndex"),TempSql);
			tmpFSE_QS->ProcessIndex = _ttoi(TempSql);

			pRs->GetFieldValue(_T("ProcessObjectType"),TempSql);
			tmpFSE_QS->ProcessObjectType = (FSE_ObjectType)_ttoi(TempSql);

			pRs->GetFieldValue(_T("ProcessUpdateID"),TempSql);
			tmpFSE_QS->ProcessUpdateID = _ttoi(TempSql);

			pRs->GetFieldValue(_T("Album"),TempSql);
			_tcscpy((TCHAR *) tmpFSE_QS->Album, TempSql.GetString());			

			pRs->GetFieldValue(_T("Title"),TempSql);
			_tcscpy((TCHAR *) tmpFSE_QS->Title, TempSql.GetString());

			pRs->GetFieldValue(_T("Author"),TempSql);
			_tcscpy((TCHAR *) tmpFSE_QS->Author, TempSql.GetString());

			pRs->GetFieldValue(_T("Genre"),TempSql);
			_tcscpy((TCHAR *) tmpFSE_QS->Genre, TempSql.GetString());

			pRs->GetFieldValue(_T("TrackNo"), TempSql);
			tmpFSE_QS->TrackNumber = _ttoi(TempSql);

			pRs->GetFieldValue(_T("Duration"),TempSql);
			tmpFSE_QS->Duration = _ttoi(TempSql);

			pRs->GetFieldValue(_T("Bitrate"),TempSql);
			tmpFSE_QS->Bitrate = _ttoi(TempSql);

			pRs->GetFieldValue(_T("FileSize"),TempSql);
			tmpFSE_QS->Size = _ttoi(TempSql);
		
			pRs->GetFieldValue(_T("FileType"),TempSql);
			_tcscpy((TCHAR *) tmpFSE_QS->FileType, TempSql.GetString());

			//birdy add for internetfile 2007/11/2
			pRs->GetFieldValue(_T("InternetFlags"),TempSql);
			tmpFSE_QS->InternetFlags = _ttoi(TempSql);

			//birdy add for CD_Cover  2007/12/17
			pRs->GetFieldValue(_T("CDCover"),TempSql);
			_tcscpy((TCHAR *) tmpFSE_QS->CDCover, TempSql.GetString());	
			
		}
		pRs->Close();
	}
	else
	{
		AfxMessageBox(_T("database no exists,db_getOneRecord()"));
		count = 0;
	}

	delete pRs;
	ReleaseMutex(hDBMutex);
	return count;
}


// Utf8 version
extern "C" int db_getOneRecord(const char *SQLstring, struct FSE_QueryState *tmpFSE_QS)
{
	CADORecordset*	pRs = new CADORecordset(g_pDb);
	CString TempSql;
	TCHAR	strSQL[MAX_SQL_STR_LENGTH];
	int		count = 0;

	Utf8ToStr(strSQL, SQLstring, MAX_SQL_STR_LENGTH);

	while (WaitForSingleObject(hDBMutex, 100L) != WAIT_OBJECT_0)
		Sleep(100);

	if(pRs->Open(strSQL))
	{
		count = pRs->GetRecordCount();

		if (count != 0)
		{
			// set the data in testFSE_QS , total 15 fields;
			pRs->GetFieldValue(_T("MyIndex"),TempSql);		
			tmpFSE_QS->Index = _ttoi(TempSql);

			pRs->GetFieldValue(_T("PIndex"),TempSql);
			tmpFSE_QS->pIndex = _ttoi(TempSql);

			pRs->GetFieldValue(_T("Flags"),TempSql);
			tmpFSE_QS->Flags = _ttoi(TempSql);

			pRs->GetFieldValue(_T("ObjectType"),TempSql);
			tmpFSE_QS->ObjectType = (FSE_ObjectType)_ttoi(TempSql);

			pRs->GetFieldValue(_T("FilePathLen"),TempSql);
			tmpFSE_QS->FilePathLen = _ttoi(TempSql);
			tmpFSE_QS->DirFilePathLen = tmpFSE_QS->FilePathLen;

			pRs->GetFieldValue(_T("FilePath"),TempSql);
			strToUtf8(tmpFSE_QS->FilePath, (char *) TempSql.GetString(), MAX_PATH_LENGTH, IS_WIDE, NULL);
			//birdy 2008/6/12 add for MKV file path
			_tcscpy((wchar_t*)tmpFSE_QS->MKVFilePath, TempSql);
			strToUtf8(tmpFSE_QS->DirFilePath, (char *) TempSql.GetString(), MAX_PATH_LENGTH, IS_WIDE, NULL);
			
			
			pRs->GetFieldValue(_T("FileName"),TempSql);
			strToUtf8(tmpFSE_QS->FileName, (char *) TempSql.GetString(), MAX_PATH_LENGTH, IS_WIDE, NULL);
			//birdy 2008/2/21 add for fix title show right in the Internet media page
			_tcscpy((wchar_t*)tmpFSE_QS->InternetFileName, TempSql);
						
			// set uri
			sprintf(tmpFSE_QS->ProcessPath,"\\%d\\%s\0", tmpFSE_QS->Index, tmpFSE_QS->FileName);

			pRs->GetFieldValue(_T("FileNameLen"),TempSql);
			tmpFSE_QS->FileNameLen = _ttoi(TempSql);

			pRs->GetFieldValue(_T("isSearchable"),TempSql);
			tmpFSE_QS->isSearchable = _ttoi(TempSql);

			pRs->GetFieldValue(_T("ProcessIndex"),TempSql);
			tmpFSE_QS->ProcessIndex = _ttoi(TempSql);

			pRs->GetFieldValue(_T("ProcessObjectType"),TempSql);
			tmpFSE_QS->ProcessObjectType = (FSE_ObjectType)_ttoi(TempSql);

			pRs->GetFieldValue(_T("ProcessUpdateID"),TempSql);
			tmpFSE_QS->ProcessUpdateID = _ttoi(TempSql);

			pRs->GetFieldValue(_T("Album"),TempSql);
			strToUtf8(tmpFSE_QS->Album, (char *) TempSql.GetString(), MAX_MEDIA_STR_LENGTH, IS_WIDE, NULL);

			pRs->GetFieldValue(_T("Title"),TempSql);
			strToUtf8(tmpFSE_QS->Title, (char *) TempSql.GetString(), MAX_MEDIA_STR_LENGTH, IS_WIDE, NULL);

			pRs->GetFieldValue(_T("Author"),TempSql);
			strToUtf8(tmpFSE_QS->Author, (char *) TempSql.GetString(), MAX_MEDIA_STR_LENGTH, IS_WIDE, NULL);

			pRs->GetFieldValue(_T("Genre"),TempSql);
			strToUtf8(tmpFSE_QS->Genre, (char *) TempSql.GetString(), MAX_MEDIA_STR_LENGTH, IS_WIDE, NULL);
			
			pRs->GetFieldValue(_T("TrackNo"), TempSql);
			tmpFSE_QS->TrackNumber = _ttoi(TempSql);

			pRs->GetFieldValue(_T("Duration"),TempSql);
			tmpFSE_QS->Duration = _ttoi(TempSql);

			pRs->GetFieldValue(_T("Bitrate"),TempSql);
			tmpFSE_QS->Bitrate = _ttoi(TempSql);

			pRs->GetFieldValue(_T("FileSize"),TempSql);
			tmpFSE_QS->Size = _ttoi(TempSql);
		
			pRs->GetFieldValue(_T("FileType"),TempSql);
			strToUtf8(tmpFSE_QS->FileType, (char *) TempSql.GetString(), MAX_EXT_STR_LENGTH, IS_WIDE, NULL);

			//birdy add for internetfile 2007/11/2
			pRs->GetFieldValue(_T("InternetFlags"),TempSql);
			tmpFSE_QS->InternetFlags = _ttoi(TempSql);

			//birdy add for CD_Cover  2007/12/17
			pRs->GetFieldValue(_T("CDCover"),TempSql);
			strToUtf8(tmpFSE_QS->CDCover, (char *) TempSql.GetString(), MAX_PATH_LENGTH, IS_WIDE, NULL);		
		}
		pRs->Close();
	}
	else
	{
		AfxMessageBox(_T("database no exists,db_getOneRecord()"));
		count = 0;
	}

	delete pRs;
	ReleaseMutex(hDBMutex);
	return count;
}

// Utf8 version
extern "C" int db_getOneRecordInIndex(const char *SQLstring, struct FSE_QueryState *tmpFSE_QS, int index)
{
	CADORecordset*	pRs = new CADORecordset(g_pDb);
	CString TempSql;
	TCHAR	strSQL[MAX_SQL_STR_LENGTH];
	int		result, count;

	Utf8ToStr(strSQL, SQLstring, MAX_SQL_STR_LENGTH);

	while (WaitForSingleObject(hDBMutex, 100L) != WAIT_OBJECT_0)
		Sleep(100);

	if(pRs->Open(strSQL))
	{
		count = pRs->GetRecordCount();
		if (index >= count)
		{
			result = FALSE;
		}
		else
		{
			pRs->SetSort(_T("ObjectType DESC, Album, FileName"));
			pRs->SetAbsolutePosition(index + 1);

			// set the data in testFSE_QS , total 15 fields;
			pRs->GetFieldValue(_T("MyIndex"),TempSql);		
			tmpFSE_QS->Index = _ttoi(TempSql);

			pRs->GetFieldValue(_T("PIndex"),TempSql);
			tmpFSE_QS->pIndex = _ttoi(TempSql);

			pRs->GetFieldValue(_T("Flags"),TempSql);
			tmpFSE_QS->Flags = _ttoi(TempSql);

			pRs->GetFieldValue(_T("ObjectType"),TempSql);
			tmpFSE_QS->ObjectType = (FSE_ObjectType)_ttoi(TempSql);

			pRs->GetFieldValue(_T("FilePathLen"),TempSql);
			tmpFSE_QS->FilePathLen = _ttoi(TempSql);
			tmpFSE_QS->DirFilePathLen = tmpFSE_QS->FilePathLen;

			pRs->GetFieldValue(_T("FilePath"),TempSql);
			strToUtf8(tmpFSE_QS->FilePath, (char *) TempSql.GetString(), MAX_PATH_LENGTH, IS_WIDE, NULL);
			//birdy 2008/6/12 add for MKV file path
			_tcscpy((wchar_t*)tmpFSE_QS->MKVFilePath, TempSql);
			strToUtf8(tmpFSE_QS->DirFilePath, (char *) TempSql.GetString(), MAX_PATH_LENGTH, IS_WIDE, NULL);
			

			pRs->GetFieldValue(_T("FileName"),TempSql);
			strToUtf8(tmpFSE_QS->FileName, (char *) TempSql.GetString(), MAX_PATH_LENGTH, IS_WIDE, NULL);
			//birdy 2008/2/21 add for fix title show right in the Internet media page
			_tcscpy((wchar_t*)tmpFSE_QS->InternetFileName, TempSql);
			

			// set uri
			sprintf(tmpFSE_QS->ProcessPath,"\\%d\\%s\0", tmpFSE_QS->Index, tmpFSE_QS->FileName);

			pRs->GetFieldValue(_T("FileNameLen"),TempSql);
			tmpFSE_QS->FileNameLen = _ttoi(TempSql);

			pRs->GetFieldValue(_T("isSearchable"),TempSql);
			tmpFSE_QS->isSearchable = _ttoi(TempSql);

			pRs->GetFieldValue(_T("ProcessIndex"),TempSql);
			tmpFSE_QS->ProcessIndex = _ttoi(TempSql);

			pRs->GetFieldValue(_T("ProcessObjectType"),TempSql);
			tmpFSE_QS->ProcessObjectType = (FSE_ObjectType)_ttoi(TempSql);

			pRs->GetFieldValue(_T("ProcessUpdateID"),TempSql);
			tmpFSE_QS->ProcessUpdateID = _ttoi(TempSql);

			pRs->GetFieldValue(_T("Album"),TempSql);
			strToUtf8(tmpFSE_QS->Album, (char *) TempSql.GetString(), MAX_MEDIA_STR_LENGTH, IS_WIDE, NULL);

			pRs->GetFieldValue(_T("Title"),TempSql);
			strToUtf8(tmpFSE_QS->Title, (char *) TempSql.GetString(), MAX_MEDIA_STR_LENGTH, IS_WIDE, NULL);

			pRs->GetFieldValue(_T("Author"),TempSql);
			strToUtf8(tmpFSE_QS->Author, (char *) TempSql.GetString(), MAX_MEDIA_STR_LENGTH, IS_WIDE, NULL);

			pRs->GetFieldValue(_T("Genre"),TempSql);
			strToUtf8(tmpFSE_QS->Genre, (char *) TempSql.GetString(), MAX_MEDIA_STR_LENGTH, IS_WIDE, NULL);
			
			pRs->GetFieldValue(_T("TrackNo"), TempSql);
			tmpFSE_QS->TrackNumber = _ttoi(TempSql);

			pRs->GetFieldValue(_T("Duration"),TempSql);
			tmpFSE_QS->Duration = _ttoi(TempSql);

			pRs->GetFieldValue(_T("Bitrate"),TempSql);
			tmpFSE_QS->Bitrate = _ttoi(TempSql);

			pRs->GetFieldValue(_T("FileSize"),TempSql);
			tmpFSE_QS->Size = _ttoi(TempSql);
		
			pRs->GetFieldValue(_T("FileType"),TempSql);
			strToUtf8(tmpFSE_QS->FileType, (char *) TempSql.GetString(), MAX_EXT_STR_LENGTH, IS_WIDE, NULL);

			//birdy add for internetfile 2007/11/2
			pRs->GetFieldValue(_T("InternetFlags"),TempSql);
			tmpFSE_QS->InternetFlags = _ttoi(TempSql);

			//birdy add for CD_Cover  2007/12/17
			pRs->GetFieldValue(_T("CDCover"),TempSql);
			strToUtf8(tmpFSE_QS->CDCover, (char *) TempSql.GetString(), MAX_PATH_LENGTH, IS_WIDE, NULL);
			
			result = TRUE;
		}

		pRs->Close();
	}

	delete pRs;
	ReleaseMutex(hDBMutex);
	return result;
}

// Unicode version
extern "C" int db_getRecordsT(const TCHAR *SQLstring, struct FSE_QueryState FSE_QSArray[], int startIndex, int readCount)
{
	CString TempSql;
	int		i, result;

	while (WaitForSingleObject(hDBMutex, 100L) != WAIT_OBJECT_0)
		Sleep(100);
	
	if (flushDBQuery || (_tcscmp(prevSQLstringGetRecordsT, SQLstring) != 0) || !pRsGetRecordsT->IsOpen())
	{
		// We haven't queried this query, use previous open query
		if (pRsGetRecordsT->IsOpen())
		{
			pRsGetRecordsT->Close();
		}

		if (pRsGetRecordsT->Open(SQLstring))
		{
			pRsGetRecordsT->SetSort(_T("ObjectType DESC, Album, FileName"));
		}

		_tcscpy(prevSQLstringGetRecordsT, SQLstring);
		flushDBQuery = FALSE;
	}

	if (startIndex >= (int) pRsGetRecordsT->GetRecordCount())
	{
		// the request start index exceed the total count
		ReleaseMutex(hDBMutex);
		return 0;
	}

	pRsGetRecordsT->SetAbsolutePosition(startIndex + 1);

	for (i = 0, result = 0; (i < readCount) && (!pRsGetRecordsT->IsEof()); i++, pRsGetRecordsT->MoveNext())
	{
		// set the data in testFSE_QS , total 15 fields;
		pRsGetRecordsT->GetFieldValue(_T("MyIndex"),TempSql);		
		FSE_QSArray[i].Index = _ttoi(TempSql);

		pRsGetRecordsT->GetFieldValue(_T("PIndex"),TempSql);
		FSE_QSArray[i].Index = _ttoi(TempSql);

		pRsGetRecordsT->GetFieldValue(_T("Flags"),TempSql);
		FSE_QSArray[i].Flags = _ttoi(TempSql);

		pRsGetRecordsT->GetFieldValue(_T("ObjectType"),TempSql);
		FSE_QSArray[i].ObjectType = (FSE_ObjectType)_ttoi(TempSql);

		pRsGetRecordsT->GetFieldValue(_T("FilePathLen"),TempSql);
		FSE_QSArray[i].FilePathLen = _ttoi(TempSql);
		FSE_QSArray[i].DirFilePathLen = FSE_QSArray[i].FilePathLen;

		pRsGetRecordsT->GetFieldValue(_T("FilePath"),TempSql);
		_tcscpy((TCHAR *) FSE_QSArray[i].FilePath, TempSql.GetString());
		_tcscpy((TCHAR *) FSE_QSArray[i].DirFilePath, TempSql.GetString());

		pRsGetRecordsT->GetFieldValue(_T("FileName"),TempSql);
		_tcscpy((TCHAR *) FSE_QSArray[i].FileName, TempSql.GetString());

		// set uri
		sprintf(FSE_QSArray->ProcessPath,"\\%d\\%s", FSE_QSArray[i].Index, FSE_QSArray[i].FileName);

		pRsGetRecordsT->GetFieldValue(_T("FileNameLen"),TempSql);
		FSE_QSArray[i].FileNameLen = _ttoi(TempSql);

		pRsGetRecordsT->GetFieldValue(_T("isSearchable"),TempSql);
		FSE_QSArray[i].isSearchable = _ttoi(TempSql);

		pRsGetRecordsT->GetFieldValue(_T("ProcessIndex"),TempSql);
		FSE_QSArray[i].ProcessIndex = _ttoi(TempSql);

		pRsGetRecordsT->GetFieldValue(_T("ProcessObjectType"),TempSql);
		FSE_QSArray[i].ProcessObjectType = (FSE_ObjectType)_ttoi(TempSql);
	
		pRsGetRecordsT->GetFieldValue(_T("ProcessUpdateID"),TempSql);
		FSE_QSArray[i].ProcessUpdateID = _ttoi(TempSql);

		pRsGetRecordsT->GetFieldValue(_T("Album"),TempSql);
		_tcscpy((TCHAR *) FSE_QSArray[i].Album, TempSql.GetString());			

		pRsGetRecordsT->GetFieldValue(_T("Title"),TempSql);
		_tcscpy((TCHAR *) FSE_QSArray[i].Title, TempSql.GetString());

		pRsGetRecordsT->GetFieldValue(_T("Author"),TempSql);
		_tcscpy((TCHAR *) FSE_QSArray[i].Author, TempSql.GetString());

		pRsGetRecordsT->GetFieldValue(_T("Genre"),TempSql);
		_tcscpy((TCHAR *) FSE_QSArray[i].Genre, TempSql.GetString());

		pRsGetRecordsT->GetFieldValue(_T("TrackNo"), TempSql);
		FSE_QSArray[i].TrackNumber = _ttoi(TempSql);

		pRsGetRecordsT->GetFieldValue(_T("Duration"),TempSql);
		FSE_QSArray[i].Duration = _ttoi(TempSql);

		pRsGetRecordsT->GetFieldValue(_T("Bitrate"),TempSql);
		FSE_QSArray[i].Bitrate = _ttoi(TempSql);

		pRsGetRecordsT->GetFieldValue(_T("FileSize"),TempSql);
		FSE_QSArray[i].Size = _ttoi(TempSql);
		
		pRsGetRecordsT->GetFieldValue(_T("FileType"),TempSql);
		_tcscpy((TCHAR *) FSE_QSArray[i].FileType, TempSql.GetString());

		//birdy add for internetfile 2007/11/2
		pRsGetRecordsT->GetFieldValue(_T("InternetFlags"),TempSql);
		FSE_QSArray[i].InternetFlags = _ttoi(TempSql);

		//birdy add for CD_Cover  2007/12/17
		pRsGetRecordsT->GetFieldValue(_T("CDCover"),TempSql);
		_tcscpy((TCHAR *) FSE_QSArray[i].CDCover, TempSql.GetString());
		
		result++;
	}

	ReleaseMutex(hDBMutex);
	return result;
}

// Utf8 version
extern "C" int db_getRecords(const char *SQLstring, struct FSE_QueryState FSE_QSArray[], int startIndex, int readCount)
{
	CString TempSql;
	TCHAR	strSQL[MAX_SQL_STR_LENGTH];
	int		result, i;
		
	while (WaitForSingleObject(hDBMutex, 100L) != WAIT_OBJECT_0)
		Sleep(100);

	if (flushDBQuery || (strcmp(prevSQLstringGetRecords, SQLstring) != 0) || !pRsGetRecords->IsOpen())
	{
		// We haven't queried this query, use previous open query
		if (pRsGetRecords->IsOpen())
		{
			pRsGetRecords->Close();
		}

		Utf8ToStr(strSQL, SQLstring, MAX_SQL_STR_LENGTH);

		//old
		/*if (pRsGetRecords->Open(strSQL))
		{
			pRsGetRecords->SetSort(_T("ObjectType DESC, Album, FileName"));
		}*/

		//2008/2/20 birdy check Youtube file not sotr		
		db_selectT(strSQL);	
		
		if (pRsGetRecords->Open(strSQL))
		{
			if (testFSE_QS[0].InternetFlags == 0)
			{
                pRsGetRecords->SetSort(_T("ObjectType DESC, Album, FileName"));
			}
		}
		
		strcpy(prevSQLstringGetRecords, SQLstring);
		flushDBQuery = FALSE;
	}

	if (startIndex >= (int) pRsGetRecords->GetRecordCount())
	{
		// the request start index exceed the total count
		ReleaseMutex(hDBMutex);
		return 0;
	}

	pRsGetRecords->SetAbsolutePosition(startIndex + 1);

	for (i = 0, result = 0; (i < readCount) && (!pRsGetRecords->IsEof()); i++, pRsGetRecords->MoveNext())
	{
		// set the data in testFSE_QS , total 15 fields;
		pRsGetRecords->GetFieldValue(_T("MyIndex"),TempSql);		
		FSE_QSArray[i].Index = _ttoi(TempSql);

		pRsGetRecords->GetFieldValue(_T("PIndex"),TempSql);
		FSE_QSArray[i].pIndex = _ttoi(TempSql);

		pRsGetRecords->GetFieldValue(_T("Flags"),TempSql);
		FSE_QSArray[i].Flags = _ttoi(TempSql);

		pRsGetRecords->GetFieldValue(_T("ObjectType"),TempSql);
		FSE_QSArray[i].ObjectType = (FSE_ObjectType)_ttoi(TempSql);

		pRsGetRecords->GetFieldValue(_T("FilePathLen"),TempSql);
		FSE_QSArray[i].FilePathLen = _ttoi(TempSql);
		FSE_QSArray[i].DirFilePathLen = FSE_QSArray[i].FilePathLen;

		pRsGetRecords->GetFieldValue(_T("FilePath"),TempSql);
		strToUtf8(FSE_QSArray[i].FilePath, (char *) TempSql.GetString(), MAX_PATH_LENGTH, IS_WIDE, NULL);
		//birdy 2008/6/12 add for MKV file path
		_tcscpy((wchar_t*)FSE_QSArray[i].MKVFilePath, TempSql);
		strToUtf8(FSE_QSArray[i].DirFilePath, (char *) TempSql.GetString(), MAX_PATH_LENGTH, IS_WIDE, NULL);
		

		pRsGetRecords->GetFieldValue(_T("FileName"),TempSql);
		strToUtf8(FSE_QSArray[i].FileName, (char *) TempSql.GetString(), MAX_PATH_LENGTH, IS_WIDE, NULL);
		//birdy 2008/2/21 add for fix title show right in the Internet media page
		_tcscpy((wchar_t*)FSE_QSArray[i].InternetFileName, TempSql);
		

		// set uri
		sprintf(FSE_QSArray[i].ProcessPath,"\\%d\\%s\0", FSE_QSArray[i].Index, FSE_QSArray[i].FileName);

		pRsGetRecords->GetFieldValue(_T("FileNameLen"),TempSql);
		FSE_QSArray[i].FileNameLen = _ttoi(TempSql);

		pRsGetRecords->GetFieldValue(_T("isSearchable"),TempSql);
		FSE_QSArray[i].isSearchable = _ttoi(TempSql);

		pRsGetRecords->GetFieldValue(_T("ProcessIndex"),TempSql);
		FSE_QSArray[i].ProcessIndex = _ttoi(TempSql);

		pRsGetRecords->GetFieldValue(_T("ProcessObjectType"),TempSql);
		FSE_QSArray[i].ProcessObjectType = (FSE_ObjectType)_ttoi(TempSql);

		pRsGetRecords->GetFieldValue(_T("ProcessUpdateID"),TempSql);
		FSE_QSArray[i].ProcessUpdateID = _ttoi(TempSql);

		pRsGetRecords->GetFieldValue(_T("Album"),TempSql);
		strToUtf8(FSE_QSArray[i].Album, (char *) TempSql.GetString(), MAX_MEDIA_STR_LENGTH, IS_WIDE, NULL);

		pRsGetRecords->GetFieldValue(_T("Title"),TempSql);
		strToUtf8(FSE_QSArray[i].Title, (char *) TempSql.GetString(), MAX_MEDIA_STR_LENGTH, IS_WIDE, NULL);

		pRsGetRecords->GetFieldValue(_T("Author"),TempSql);
		strToUtf8(FSE_QSArray[i].Author, (char *) TempSql.GetString(), MAX_MEDIA_STR_LENGTH, IS_WIDE, NULL);

		pRsGetRecords->GetFieldValue(_T("Genre"),TempSql);
		strToUtf8(FSE_QSArray[i].Genre, (char *) TempSql.GetString(), MAX_MEDIA_STR_LENGTH, IS_WIDE, NULL);
			
		pRsGetRecords->GetFieldValue(_T("TrackNo"), TempSql);
		FSE_QSArray[i].TrackNumber = _ttoi(TempSql);

		pRsGetRecords->GetFieldValue(_T("Duration"),TempSql);
		FSE_QSArray[i].Duration = _ttoi(TempSql);

		pRsGetRecords->GetFieldValue(_T("Bitrate"),TempSql);
		FSE_QSArray[i].Bitrate = _ttoi(TempSql);

		pRsGetRecords->GetFieldValue(_T("FileSize"),TempSql);
		FSE_QSArray[i].Size = _ttoi(TempSql);
		
		pRsGetRecords->GetFieldValue(_T("FileType"),TempSql);
		strToUtf8(FSE_QSArray[i].FileType, (char *) TempSql.GetString(), MAX_EXT_STR_LENGTH, IS_WIDE, NULL);

        //birdy add for internetfile 2007/11/2
		pRsGetRecords->GetFieldValue(_T("InternetFlags"),TempSql);
		FSE_QSArray[i].InternetFlags = _ttoi(TempSql);

		//birdy add for CD_Cover  2007/12/17
		pRsGetRecords->GetFieldValue(_T("CDCover"),TempSql);
		strToUtf8(FSE_QSArray[i].CDCover, (char *) TempSql.GetString(), MAX_PATH_LENGTH, IS_WIDE, NULL);
		
		result++;
	}

	ReleaseMutex(hDBMutex);
	return result;
}

// Unicode Version
extern "C" int db_setAVPT(const TCHAR *Attribute, const TCHAR *value)
{
	TCHAR	strSQL[MAX_SQL_STR_LENGTH];
	int		result;

	_stprintf(strSQL, _T("update AVP set my_value = \"%s\" where my_attribute =\"%s\""), value, Attribute);
	result = g_pDb->Execute(strSQL);

	return result;

}


// Utf8 Version
extern "C" int db_setAVP(const char *Attribute, const char *value)
{
	TCHAR	strAttr[MAX_SQL_STR_LENGTH], strValue[MAX_SQL_STR_LENGTH];
	
	Utf8ToStr(strAttr, Attribute, MAX_SQL_STR_LENGTH);
	Utf8ToStr(strValue, value, MAX_SQL_STR_LENGTH);
	return db_setAVPT(strAttr, strValue);
}

// Unicode Version
extern "C" int db_getAVPT(const TCHAR *Attribute, TCHAR* returnValue)
{
	CADORecordset* pRs = new CADORecordset(g_pDb);
	CString	tempData;
	TCHAR	strSQL[MAX_SQL_STR_LENGTH];
	int		result;

	_stprintf(strSQL, _T("Select * from AVP where my_attribute =\"%s\""), Attribute);

	if(pRs->Open(strSQL))
	{
		pRs->GetFieldValue(_T("my_value"), tempData);
		_tcscpy(returnValue, tempData);
		pRs->Close();
		result = 1;
	}
	else
	{
		AfxMessageBox(_T("database no exists, db_getAVP() error"));
		result = 0;
	}
	
	delete pRs;
	return result;
}

// Utf8 version
extern "C" int db_getAVP(const char *Attribute, char *returnValue)
{
	TCHAR	strAttr[MAX_SQL_STR_LENGTH], strResult[MAX_SQL_STR_LENGTH];
	int		ret;
	
	Utf8ToStr(strAttr, Attribute, MAX_SQL_STR_LENGTH);
	ret = db_getAVPT(strAttr, strResult);
	strToUtf8(returnValue, (char *) strResult, MAX_SQL_STR_LENGTH, IS_WIDE, NULL);
	
	return ret;
}


// Unicode Version
extern "C" int db_selectT(const TCHAR *SQLstring)
{
	CADORecordset* pRs;// = new CADORecordset(g_pDb);
	CString TempSql;
	int		i = 0, count;


	if( g_pDb == NULL )
	{
		return 0;
	}

	pRs = new CADORecordset(g_pDb);


	while (WaitForSingleObject(hDBMutex, 100L) != WAIT_OBJECT_0)
		Sleep(100);

	if (pRs->Open(SQLstring))
	{
		count = pRs->GetRecordCount();
		pRs->SetSort(_T("ObjectType DESC, Album, FileName"));

		if (count >= maxItemsInQS)
		{
			if (testFSE_QS)
                FREE(testFSE_QS);
			maxItemsInQS = count + NEW_ITEMS_NUM_IN_QS;
			testFSE_QS = (struct FSE_QueryState *) MALLOC(maxItemsInQS * sizeof(struct FSE_QueryState));
			if (testFSE_QS == NULL)
			{
				AfxMessageBox(_T("Cannot Get Enough Memory"));
				pRs->Close();
				delete pRs;
				ReleaseMutex(hDBMutex);
				return 0;
			}
		}

		while(!pRs->IsEof())
		{
			// set the data in testFSE_QS , total 15 fields;
			pRs->GetFieldValue(_T("MyIndex"), TempSql);		
			testFSE_QS[i].Index = _ttoi(TempSql);

			pRs->GetFieldValue(_T("PIndex"), TempSql);
			testFSE_QS[i].pIndex = _ttoi(TempSql);

			pRs->GetFieldValue(_T("Flags"), TempSql);
			testFSE_QS[i].Flags = _ttoi(TempSql);

			pRs->GetFieldValue(_T("ObjectType"), TempSql);
			testFSE_QS[i].ObjectType = (FSE_ObjectType)_ttoi(TempSql);

			pRs->GetFieldValue(_T("FilePathLen"), TempSql);
			testFSE_QS[i].FilePathLen = _ttoi(TempSql);
			testFSE_QS[i].DirFilePathLen = testFSE_QS[i].FilePathLen;
			
			pRs->GetFieldValue(_T("FileName"), TempSql);
			strToUtf8(testFSE_QS[i].FileName, (char *) TempSql.GetString(), MAX_PATH_LENGTH, IS_WIDE, NULL);
			//birdy 2008/2/21 add for fix title show right in the Internet media page
			_tcscpy((wchar_t*)testFSE_QS[i].InternetFileName, TempSql);
			

			//set uri
			sprintf(testFSE_QS[i].ProcessPath, "\\%d\\%s\0", testFSE_QS[i].Index, testFSE_QS[i].FileName);

			pRs->GetFieldValue(_T("FilePath"),TempSql);
			strToUtf8(testFSE_QS[i].FilePath, (char *) TempSql.GetString(), MAX_PATH_LENGTH, IS_WIDE, NULL);
			strToUtf8(testFSE_QS[i].DirFilePath, (char *) TempSql.GetString(), MAX_PATH_LENGTH, IS_WIDE, NULL);
			//birdy 2008/6/12 add for MKV file path
			_tcscpy((wchar_t*)testFSE_QS[i].MKVFilePath, TempSql);

			pRs->GetFieldValue(_T("FileNameLen"), TempSql);
			testFSE_QS[i].FileNameLen = _ttoi(TempSql);

			pRs->GetFieldValue(_T("isSearchable"), TempSql);
			testFSE_QS[i].isSearchable = _ttoi(TempSql);


			pRs->GetFieldValue(_T("ProcessIndex"), TempSql);
			testFSE_QS[i].ProcessIndex = _ttoi(TempSql);

			pRs->GetFieldValue(_T("ProcessObjectType"), TempSql);
			testFSE_QS[i].ProcessObjectType = (FSE_ObjectType)_ttoi(TempSql);

			pRs->GetFieldValue(_T("ProcessUpdateID"), TempSql);
			testFSE_QS[i].ProcessUpdateID = _ttoi(TempSql);

			pRs->GetFieldValue(_T("Album"), TempSql);
			strToUtf8(testFSE_QS[i].Album, (char *) TempSql.GetString(), MAX_MEDIA_STR_LENGTH, IS_WIDE, NULL);

			pRs->GetFieldValue(_T("Title"), TempSql);
			strToUtf8(testFSE_QS[i].Title, (char *) TempSql.GetString(), MAX_MEDIA_STR_LENGTH, IS_WIDE, NULL);

			pRs->GetFieldValue(_T("Author"), TempSql);
			strToUtf8(testFSE_QS[i].Author, (char *) TempSql.GetString(), MAX_MEDIA_STR_LENGTH, IS_WIDE, NULL);

			pRs->GetFieldValue(_T("Genre"), TempSql);
			strToUtf8(testFSE_QS[i].Genre, (char *) TempSql.GetString(), MAX_MEDIA_STR_LENGTH, IS_WIDE, NULL);

			pRs->GetFieldValue(_T("TrackNo"), TempSql);
			testFSE_QS[i].TrackNumber = _ttoi(TempSql);

			pRs->GetFieldValue(_T("Duration"), TempSql);
			testFSE_QS[i].Duration = _ttoi(TempSql);

			pRs->GetFieldValue(_T("Bitrate"), TempSql);
			testFSE_QS[i].Bitrate = _ttoi(TempSql);

			pRs->GetFieldValue(_T("FileSize"), TempSql);
			testFSE_QS[i].Size = _ttoi(TempSql);

			pRs->GetFieldValue(_T("FileType"),TempSql);
			strToUtf8(testFSE_QS[i].FileType, (char *) TempSql.GetString(), MAX_EXT_STR_LENGTH, IS_WIDE, NULL);
			
			testFSE_QS[i].ProcessDirHandle = NULL;

			//birdy add for internetfile 2007/11/2
		    pRs->GetFieldValue(_T("InternetFlags"),TempSql);
		    testFSE_QS[i].InternetFlags = _ttoi(TempSql);

			//birdy add for CD_Cover  2007/12/17
			pRs->GetFieldValue(_T("CDCover"),TempSql);
			strToUtf8(testFSE_QS[i].CDCover, (char *) TempSql.GetString(), MAX_PATH_LENGTH, IS_WIDE, NULL);
		    
			i++;
			//testFSE_QS[i]

			pRs->MoveNext();
		}
		pRs->Close();
	}
	else 
	{
		AfxMessageBox(_T("database no exists,db_selectT()"));
		count = 0;
	}

	delete pRs;
	ReleaseMutex(hDBMutex);
	return count;
}

extern "C" int db_select(const char *SQLstring)
{
	TCHAR	strSQL[MAX_SQL_STR_LENGTH];

	Utf8ToStr(strSQL, SQLstring, MAX_SQL_STR_LENGTH);
	
	return db_selectT(strSQL);
}

extern "C" int db_select_for_browseT(const TCHAR *SQLstring)
{
	CADORecordset* pRs = new CADORecordset(g_pDb);
	CString	TempSql;
	int		i = 0, count;

	while (WaitForSingleObject(hDBMutex, 100L) != WAIT_OBJECT_0)
		Sleep(100);

	if(pRs->Open(SQLstring))
	{
		count = pRs->GetRecordCount();
		pRs->SetSort(_T("ObjectType DESC, Album, FileName"));

		if (count >= maxItemsInQSForBrowse)
		{
			if (testFSE_QS_For_Browse)
                FREE(testFSE_QS_For_Browse);
			maxItemsInQSForBrowse = count + NEW_ITEMS_NUM_IN_QS;
			testFSE_QS_For_Browse = (struct FSE_QueryState *) MALLOC(maxItemsInQSForBrowse * sizeof(struct FSE_QueryState));
			if (testFSE_QS_For_Browse == NULL)
			{
				AfxMessageBox(_T("Cannot Get Enough Memory"));
				pRs->Close();
				delete pRs;
				ReleaseMutex(hDBMutex);
				return 0;
			}
		}

		while(!pRs->IsEof())
		{
			// set the data in testFSE_QS , total 15 fields;
			pRs->GetFieldValue(_T("MyIndex"), TempSql);		
			testFSE_QS_For_Browse[i].Index = _ttoi(TempSql);

			pRs->GetFieldValue(_T("PIndex"), TempSql);
			testFSE_QS_For_Browse[i].pIndex = _ttoi(TempSql);

			pRs->GetFieldValue(_T("Flags"), TempSql);
			testFSE_QS_For_Browse[i].Flags = _ttoi(TempSql);

			pRs->GetFieldValue(_T("ObjectType"), TempSql);
			testFSE_QS_For_Browse[i].ObjectType = (FSE_ObjectType)_ttoi(TempSql);

			pRs->GetFieldValue(_T("FilePathLen"), TempSql);
			testFSE_QS_For_Browse[i].FilePathLen = _ttoi(TempSql);
			testFSE_QS_For_Browse[i].DirFilePathLen = testFSE_QS_For_Browse[i].FilePathLen;


			pRs->GetFieldValue(_T("FileName"), TempSql);
			strToUtf8(testFSE_QS_For_Browse[i].FileName, (char *) TempSql.GetString(), MAX_PATH_LENGTH, IS_WIDE, NULL);
			_tcscpy((wchar_t*)testFSE_QS_For_Browse[i].FileName, TempSql);
			

			//set uri
			sprintf(testFSE_QS_For_Browse[i].ProcessPath, "\\%d\\%s\0", testFSE_QS_For_Browse[i].Index, testFSE_QS_For_Browse[i].FileName);

			pRs->GetFieldValue(_T("FilePath"),TempSql);
			strToUtf8(testFSE_QS_For_Browse[i].FilePath, (char *) TempSql.GetString(), MAX_PATH_LENGTH, IS_WIDE, NULL);
			strToUtf8(testFSE_QS_For_Browse[i].DirFilePath, (char *) TempSql.GetString(), MAX_PATH_LENGTH, IS_WIDE, NULL);
			
			pRs->GetFieldValue(_T("FileNameLen"), TempSql);
			testFSE_QS_For_Browse[i].FileNameLen = _ttoi(TempSql);

			pRs->GetFieldValue(_T("isSearchable"), TempSql);
			testFSE_QS_For_Browse[i].isSearchable = _ttoi(TempSql);


			pRs->GetFieldValue(_T("ProcessIndex"), TempSql);
			testFSE_QS_For_Browse[i].ProcessIndex = _ttoi(TempSql);

			pRs->GetFieldValue(_T("ProcessObjectType"), TempSql);
			testFSE_QS_For_Browse[i].ProcessObjectType = (FSE_ObjectType)_ttoi(TempSql);

			pRs->GetFieldValue(_T("ProcessUpdateID"), TempSql);
			testFSE_QS_For_Browse[i].ProcessUpdateID = _ttoi(TempSql);

			pRs->GetFieldValue(_T("Album"), TempSql);
			strToUtf8(testFSE_QS_For_Browse[i].Album, (char *) TempSql.GetString(), MAX_MEDIA_STR_LENGTH, IS_WIDE, NULL);

			pRs->GetFieldValue(_T("Title"), TempSql);
			strToUtf8(testFSE_QS_For_Browse[i].Title, (char *) TempSql.GetString(), MAX_MEDIA_STR_LENGTH, IS_WIDE, NULL);

			pRs->GetFieldValue(_T("Author"), TempSql);
			strToUtf8(testFSE_QS_For_Browse[i].Author, (char *) TempSql.GetString(), MAX_MEDIA_STR_LENGTH, IS_WIDE, NULL);
			
			pRs->GetFieldValue(_T("Genre"), TempSql);
			strToUtf8(testFSE_QS_For_Browse[i].Genre, (char *) TempSql.GetString(), MAX_MEDIA_STR_LENGTH, IS_WIDE, NULL);

			pRs->GetFieldValue(_T("TrackNo"), TempSql);
			testFSE_QS_For_Browse[i].TrackNumber = _ttoi(TempSql);

			pRs->GetFieldValue(_T("Duration"), TempSql);
			testFSE_QS_For_Browse[i].Duration = _ttoi(TempSql);

			pRs->GetFieldValue(_T("Bitrate"), TempSql);
			testFSE_QS_For_Browse[i].Bitrate = _ttoi(TempSql);

			pRs->GetFieldValue(_T("FileSize"), TempSql);
			testFSE_QS_For_Browse[i].Size = _ttoi(TempSql);

			pRs->GetFieldValue(_T("FileType"),TempSql);
			strToUtf8(testFSE_QS_For_Browse[i].FileType, (char *) TempSql.GetString(), MAX_EXT_STR_LENGTH, IS_WIDE, NULL);
			
			testFSE_QS_For_Browse[i].ProcessDirHandle = NULL;

            //birdy add for internetfile 2007/11/2
		    pRs->GetFieldValue(_T("InternetFlags"),TempSql);
		    testFSE_QS_For_Browse[i].InternetFlags = _ttoi(TempSql);

			//birdy add for CD_Cover  2007/12/17
			pRs->GetFieldValue(_T("CDCover"),TempSql);
			strToUtf8(testFSE_QS_For_Browse[i].CDCover, (char *) TempSql.GetString(), MAX_PATH_LENGTH, IS_WIDE, NULL);
		    
			i++;

			pRs->MoveNext();
		}
		pRs->Close();
	}
	else
	{
		AfxMessageBox(_T("database no exists,db_select_for_browseT()"));
		count = 0;
	}
	
	delete pRs;
	ReleaseMutex(hDBMutex);
	return count;
}

// Utf8 version
extern "C" int db_select_for_browse(const char *SQLstring)
{
	TCHAR	strSQL[MAX_SQL_STR_LENGTH];

	Utf8ToStr(strSQL, SQLstring, MAX_SQL_STR_LENGTH);
	
	return db_select_for_browseT(strSQL);
}

// Unicode version
extern "C" int db_executeT(const TCHAR *SQLstring)
{
	int		result;

	while (WaitForSingleObject(hDBMutex, 100L) != WAIT_OBJECT_0)
		Sleep(100);

	result = g_pDb->Execute(SQLstring);
	ReleaseMutex(hDBMutex);

	return result;

}

// Utf8 Version
extern "C" int db_execute(const char *SQLstring)
{
	TCHAR	strSQL[MAX_SQL_STR_LENGTH];
	int		result;

	Utf8ToStr(strSQL, SQLstring, MAX_SQL_STR_LENGTH);

	while (WaitForSingleObject(hDBMutex, 100L) != WAIT_OBJECT_0)
		Sleep(100);

	result = g_pDb->Execute(strSQL);
	ReleaseMutex(hDBMutex);

	return result;
}

void	CompactDatabase(CString srcDatabasePath)
{
	CJetEngine	jetEngine;
	CString		dstDatabasePath;

	dstDatabasePath.Append(gWorkingDir);
	dstDatabasePath.Append(_T("\\temp.mdb"));

	CString strConnectionSrc = _T("Provider=Microsoft.Jet.OLEDB.4.0;Data Source=");
	strConnectionSrc.Append(srcDatabasePath);

#ifdef Conceptronic
	//conceptronic => oh_oh_you_found_me
	strConnectionSrc.Append(_T(";Jet OLEDB:Database Password=oh_oh_you_found_me;"));
#else	
	strConnectionSrc.Append(_T(";Jet OLEDB:Database Password=honey_world;"));
#endif

	CString strConnectionDst = _T("Provider=Microsoft.Jet.OLEDB.4.0;Data Source=");
	strConnectionDst.Append(dstDatabasePath);

#ifdef Conceptronic
	strConnectionDst.Append(_T(";Jet OLEDB:Database Password=oh_oh_you_found_me;"));
#else
	strConnectionDst.Append(_T(";Jet OLEDB:Database Password=honey_world;"));
#endif

	if (jetEngine.CompactDatabase(strConnectionSrc, strConnectionDst))
	{
		try {
			// replace the original file with the compacted one.
			if (DeleteFile(srcDatabasePath))
			{
				MoveFile(dstDatabasePath, srcDatabasePath); 
			}
		}
		catch (CFileException* pEx)
		{
			DeleteFile(dstDatabasePath);
			pEx->Delete();
		}
	}
}


int	InitDatabase()
{
	CString		strConnection = _T("Provider=Microsoft.Jet.OLEDB.4.0;Data Source=");
	CRegistry	regMyReg;

	databasePath = gWorkingDir;
	databasePath += _T("\\");
	databasePath += _T(DATABASE_NAME);

	if (GetFileAttributes(databasePath) == INVALID_FILE_ATTRIBUTES)
	{
		// Cannot find the database
		return FALSE;
	}

	strConnection.Append(databasePath);

#ifdef Conceptronic
	strConnection.Append(_T(";Jet OLEDB:Database Password=oh_oh_you_found_me;"));
#else
	strConnection.Append(_T(";Jet OLEDB:Database Password=honey_world;"));
#endif

	g_pDb = new CADODatabase;

	if(!g_pDb->Open(strConnection))
	{
		// Cannot open the Database
		return FALSE;
	}

	// Create a Database mutex with no initial owner.
	hDBMutex = CreateMutex(
		NULL,                       // default security attributes
		FALSE,                      // initially not owned
		NULL);                      // unnamed mutex

	if (hDBMutex == NULL)
	{		
		// Cannot create the Database Mutex
		return FALSE;
	}

	// CADORecordset used in db_getRecords and db_getRecordsT, for speed up
	pRsGetRecords = new CADORecordset(g_pDb);
	pRsGetRecordsT = new CADORecordset(g_pDb);

	return TRUE;
}

void CloseDatabase()
{
	while (WaitForSingleObject(hDBMutex, 100L) != WAIT_OBJECT_0)
		Sleep(100);

	// Free the CADORecordset used in db_getRecords
	if (pRsGetRecords->IsOpen())
	{
		pRsGetRecords->Close();
	}

	delete pRsGetRecords;

	// Free the CADORecordset used in db_getRecordsT
	if (pRsGetRecordsT->IsOpen())
	{
		pRsGetRecordsT->Close();
	}

	delete pRsGetRecordsT;

	CloseHandle(hDBMutex);

	if (g_pDb)
	{
		g_pDb->Close();
		delete g_pDb;
		g_pDb = NULL;
	}

	if (testFSE_QS)
	{
		FREE(testFSE_QS);
		testFSE_QS = NULL;
		maxItemsInQS = 0;
	}

	if (testFSE_QS_For_Browse)
	{
		FREE(testFSE_QS_For_Browse);
		testFSE_QS_For_Browse = NULL;
		maxItemsInQSForBrowse = 0;
	}	

	CompactDatabase(databasePath);
}

int GetMaxMyIndexNoInDatabase()
{
	CADORecordset	*pRs = new CADORecordset(g_pDb);
	CString			TempSql;
	int				maxCount = 0;

	while (WaitForSingleObject(hDBMutex, 100L) != WAIT_OBJECT_0)
		Sleep(100);

	if (pRs->Open(_T("select * from FSE_QueryState")))
	{
		pRs->SetSort(_T("MyIndex DESC"));
		pRs->GetFieldValue(_T("MyIndex"), TempSql);		
		maxCount = _ttoi(TempSql);
	}

	ReleaseMutex(hDBMutex);

	return maxCount;
}

void	CleanQueueQueryInDatabase()
{
	strcpy(prevSQLstringGetRecords, "");
	_tcscpy(prevSQLstringGetRecordsT, _T(""));
}

// Description: Get the count of user shared directories in database
// Input: None
// Output:
//      Return: the count of user shared directories in database
int GetSharedDirCount()
{
	CADORecordset*		pRs;	
	int					count;

	while (WaitForSingleObject(hDBMutex, 100L) != WAIT_OBJECT_0)
		Sleep(100);

	pRs = new CADORecordset(g_pDb);

	if (pRs->Open(_T("select * from Shared_Folder")))
	{
		count = pRs->GetRecordCount();
		pRs->Close();
	}
	else
	{
		AfxMessageBox(_T("Database doesn't exist"));
		count = 0;
	}

	delete pRs;	
	ReleaseMutex(hDBMutex);

	return count;
}


// Description: Get the information of all user shared directories in database
// Input: None
// Output:
//      Return: the array of all user shared directories in database
PSharedDirStruct GetAllSharedDirs()
{
	CADORecordset*		pRs;	
	PSharedDirStruct	pSharedDir = NULL;
	CString				strQueryResult;
	int					count, i = 0;

	while (WaitForSingleObject(hDBMutex, 100L) != WAIT_OBJECT_0)
		Sleep(100);

	pRs = new CADORecordset(g_pDb);

	if (pRs->Open(_T("select * from Shared_Folder")))
	{
		count = pRs->GetRecordCount();

		pSharedDir = (PSharedDirStruct) MALLOC(count * sizeof(SharedDirStruct));

		if (pSharedDir == NULL)
		{
			// Cannot allocate enough memory
			AfxMessageBox(_T("Cannot Get Enough Memory"));
			pRs->Close();
			delete pRs;
			ReleaseMutex(hDBMutex);
			return NULL;
		}

		while(!pRs->IsEof())
		{
			pRs->GetFieldValue(_T("DirPath"), strQueryResult);

			if (!strQueryResult.IsEmpty())
			{
				_tcscpy(pSharedDir[i].DirPath, strQueryResult);

				pRs->GetFieldValue(_T("Flags"), strQueryResult);
				pSharedDir[i].Flags = _ttoi(strQueryResult);
               
				pRs->GetFieldValue(_T("VolumeName"), strQueryResult);
				_tcscpy(pSharedDir[i].VolumeName, strQueryResult);

				i++;
			}

			pRs->MoveNext();
		}

		pRs->Close();
	}
	else
	{
		AfxMessageBox(_T("Database doesn't exist"));
	}

	delete pRs;	
	ReleaseMutex(hDBMutex);

	return pSharedDir;
}

int	SelectSharedDir(const TCHAR *dirPath, PSharedDirStruct pSharedDir)
{
	CADORecordset*	pRs;
	CString			strQueryResult;
	TCHAR			strSQL[MAX_SQL_STR_LENGTH];
	int				result = TRUE;

	while (WaitForSingleObject(hDBMutex, 100L) != WAIT_OBJECT_0)
		Sleep(100);

	pRs = new CADORecordset(g_pDb);

	_stprintf(strSQL, _T("select * from Shared_Folder where DirPath =\"%s\""), dirPath);
	if (pRs->Open(strSQL))
	{
		if (pRs->GetRecordCount() != 0)
		{
			pRs->GetFieldValue(_T("DirPath"), strQueryResult);
			_tcscpy(pSharedDir->DirPath, strQueryResult);

			pRs->GetFieldValue(_T("Flags"), strQueryResult);
			pSharedDir->Flags = _ttoi(strQueryResult);
               
			pRs->GetFieldValue(_T("VolumeName"), strQueryResult);
			_tcscpy(pSharedDir->VolumeName, strQueryResult);
		}
		else
		{
			result = FALSE;
		}

		pRs->Close();
	}
	else
	{
		AfxMessageBox(_T("Database doesn't exist"));
		result = FALSE;
	}

	delete pRs;	
	ReleaseMutex(hDBMutex);

	return result;
}