/*
* INTEL CONFIDENTIAL
* Copyright (c) 2002 - 2005 Intel Corporation.  All rights reserved.
* 
* The source code contained or described herein and all documents
* related to the source code ("Material") are owned by Intel
* Corporation or its suppliers or licensors.  Title to the
* Material remains with Intel Corporation or its suppliers and
* licensors.  The Material contains trade secrets and proprietary
* and confidential information of Intel or its suppliers and
* licensors. The Material is protected by worldwide copyright and
* trade secret laws and treaty provisions.  No part of the Material
* may be used, copied, reproduced, modified, published, uploaded,
* posted, transmitted, distributed, or disclosed in any way without
* Intel's prior express written permission.

* No license under any patent, copyright, trade secret or other
* intellectual property right is granted to or conferred upon you
* by disclosure or delivery of the Materials, either expressly, by
* implication, inducement, estoppel or otherwise. Any license
* under such intellectual property rights must be express and
* approved by Intel in writing.
* 
* $Workfile: UpnpMicroStack.c
* $Revision: #1.0.1922.38347
* $Author:   Intel Corporation, Intel Device Builder
* $Date:     Wednesday, April 06, 2005
*
*
*
*/


#if defined(WIN32) || defined(_WIN32_WCE)
#	ifndef MICROSTACK_NO_STDAFX
#		include "stdafx.h"
#	endif
char* UpnpPLATFORM = "WINDOWS";
#else
char* UpnpPLATFORM = "POSIX";
#endif

#if defined(WIN32)
#define _CRTDBG_MAP_ALLOC
#endif

#if defined(WINSOCK2)
#	include <winsock2.h>
#	include <ws2tcpip.h>
#elif defined(WINSOCK1)
#	include <winsock.h>
#	include <wininet.h>
#endif

#include<stdio.h>
#include "ILibParsers.h"
#include "UpnpMicroStack.h"
#include "ILibWebServer.h"
#include "ILibWebClient.h"
#include "ILibAsyncSocket.h"

#include "mystring.h"
#include "debug.h"
#include "OEM.h"

#if defined(WIN32) && !defined(_WIN32_WCE)
#include <crtdbg.h>
#endif

// David Hu 2006/06/12 Support another port for break through firewall
#define USE_ALTERNATIVE_UPNP_PORT

#define UPNP_HTTP_MAXSOCKETS 5
#define UPNP_PORT 1900
#define UPNP_GROUP "239.255.255.250"
#define UpnpMIN(a,b) (((a)<(b))?(a):(b))

#ifdef USE_ALTERNATIVE_UPNP_PORT
#define ALTERNATIVE_UPNP_PORT	50000+1900
#endif


//phillo,2007/11/7
//
extern int ILibGetLocalIPInteraceList(INTERFACE_INFO ** pp_int);
#define LVL3DEBUG(x)
//#define __DEBUG_LOG__

FILE * logfile;

const char *UpnpDeviceDescription = UPNP_DEVICE_DESC;
const int UpnpDeviceDescriptionTemplateLengthUX = 1271;
const int UpnpDeviceDescriptionTemplateLength = 613;
const char UpnpDeviceDescriptionTemplate[613]={
	0x5A,0x3C,0x3F,0x78,0x6D,0x6C,0x20,0x76,0x65,0x72,0x73,0x69,0x6F,0x6E,0x3D,0x22,0x31,0x2E,0x30,0x22
	,0x20,0x65,0x6E,0x63,0x6F,0x64,0x69,0x6E,0x67,0x3D,0x22,0x75,0x74,0x66,0x2D,0x38,0x22,0x3F,0x3E,0x3C
	,0x72,0x6F,0x6F,0x74,0x20,0x78,0x6D,0x6C,0x6E,0x73,0x3D,0x22,0x75,0x72,0x6E,0x3A,0x73,0x63,0x68,0x65
	,0x6D,0x61,0x73,0x2D,0x75,0x70,0x6E,0x70,0x2D,0x6F,0x72,0x67,0x3A,0x64,0x65,0x76,0x69,0x63,0x65,0x2D
	,0x31,0x2D,0x30,0x22,0x3E,0x3C,0x73,0x70,0x65,0x63,0x56,0xC6,0x14,0x0B,0x3E,0x3C,0x6D,0x61,0x6A,0x6F
	,0x72,0x3E,0x31,0x3C,0x2F,0x46,0x02,0x0A,0x3C,0x6D,0x69,0x6E,0x6F,0x72,0x3E,0x30,0x3C,0x2F,0x46,0x02
	,0x02,0x3C,0x2F,0x8D,0x0B,0x00,0x06,0x12,0x00,0x08,0x02,0x05,0x54,0x79,0x70,0x65,0x3E,0x1B,0x1C,0x10
	,0x3A,0x4D,0x65,0x64,0x69,0x61,0x53,0x65,0x72,0x76,0x65,0x72,0x3A,0x31,0x3C,0x2F,0x8B,0x0D,0x0F,0x3C
	,0x64,0x6C,0x6E,0x61,0x3A,0x58,0x5F,0x44,0x4C,0x4E,0x41,0x44,0x4F,0x43,0x46,0x2F,0x01,0x3A,0x44,0x05
	,0x00,0x8E,0x30,0x00,0xC4,0x09,0x00,0x91,0x30,0x0A,0x44,0x4D,0x53,0x2D,0x31,0x2E,0x30,0x30,0x3C,0x2F
	,0x8E,0x11,0x13,0x3E,0x3C,0x66,0x72,0x69,0x65,0x6E,0x64,0x6C,0x79,0x4E,0x61,0x6D,0x65,0x3E,0x25,0x73
	,0x3C,0x2F,0x4D,0x04,0x35,0x3C,0x6D,0x61,0x6E,0x75,0x66,0x61,0x63,0x74,0x75,0x72,0x65,0x72,0x3E,0x49
	,0x6E,0x74,0x65,0x6C,0x27,0x73,0x20,0x43,0x6F,0x6E,0x6E,0x65,0x63,0x74,0x65,0x64,0x20,0x61,0x6E,0x64
	,0x20,0x45,0x78,0x74,0x65,0x6E,0x64,0x65,0x64,0x20,0x50,0x43,0x20,0x4C,0x61,0x62,0x3C,0x2F,0x0D,0x0D
	,0x00,0x8D,0x10,0x10,0x55,0x52,0x4C,0x3E,0x68,0x74,0x74,0x70,0x3A,0x2F,0x2F,0x77,0x77,0x77,0x2E,0x69
	,0x04,0x14,0x06,0x2E,0x63,0x6F,0x6D,0x3C,0x2F,0x90,0x09,0x0E,0x3C,0x6D,0x6F,0x64,0x65,0x6C,0x44,0x65
	,0x73,0x63,0x72,0x69,0x70,0x74,0x04,0x5E,0x16,0x55,0x50,0x6E,0x50,0x2F,0x41,0x56,0x20,0x31,0x2E,0x30
	,0x20,0x43,0x6F,0x6D,0x70,0x6C,0x69,0x61,0x6E,0x74,0x20,0x05,0x4C,0x01,0x20,0x46,0x4C,0x02,0x3C,0x2F
	,0x51,0x0D,0x00,0xC6,0x11,0x00,0x05,0x37,0x03,0x58,0x50,0x43,0x94,0x0B,0x00,0xC7,0x39,0x00,0xC5,0x09
	,0x0E,0x75,0x6D,0x62,0x65,0x72,0x20,0x2F,0x3E,0x3C,0x73,0x65,0x72,0x69,0x61,0x07,0x04,0x00,0x45,0x45
	,0x00,0x4D,0x04,0x0A,0x3C,0x55,0x44,0x4E,0x3E,0x75,0x75,0x69,0x64,0x3A,0x04,0x4C,0x03,0x55,0x44,0x4E
	,0x45,0x0C,0x00,0xC4,0x8B,0x04,0x4C,0x69,0x73,0x74,0x49,0x03,0x00,0x89,0x05,0x00,0x5A,0x7D,0x00,0xC7
	,0x0D,0x01,0x3A,0x47,0x54,0x08,0x69,0x6F,0x6E,0x4D,0x61,0x6E,0x61,0x67,0x06,0x7F,0x00,0x8C,0x0F,0x00
	,0x48,0x18,0x02,0x49,0x64,0x85,0x8F,0x00,0x50,0x10,0x02,0x49,0x64,0xD2,0x10,0x02,0x3C,0x2F,0x0A,0x0D
	,0x05,0x3C,0x53,0x43,0x50,0x44,0x44,0x5F,0x00,0x51,0x1A,0x0B,0x2F,0x73,0x63,0x70,0x64,0x2E,0x78,0x6D
	,0x6C,0x3C,0x2F,0x08,0x09,0x08,0x3C,0x63,0x6F,0x6E,0x74,0x72,0x6F,0x6C,0x16,0x0C,0x00,0x47,0x07,0x02
	,0x3C,0x2F,0x8B,0x09,0x09,0x3C,0x65,0x76,0x65,0x6E,0x74,0x53,0x75,0x62,0xD6,0x18,0x00,0x85,0x07,0x02
	,0x3C,0x2F,0x4C,0x09,0x02,0x3C,0x2F,0x50,0x49,0x00,0xAE,0x4B,0x0D,0x74,0x65,0x6E,0x74,0x44,0x69,0x72
	,0x65,0x63,0x74,0x6F,0x72,0x79,0x75,0x4B,0x00,0x8D,0x10,0x00,0x18,0x4B,0x00,0xCD,0x19,0x00,0xE2,0x4A
	,0x00,0xCE,0x0B,0x00,0xA4,0x4A,0x00,0x4E,0x18,0x00,0x5E,0x4A,0x01,0x2F,0x4D,0x99,0x03,0x2F,0x64,0x65
	,0xC7,0x05,0x02,0x72,0x6F,0x00,0x00,0x03,0x6F,0x74,0x3E,0x00,0x00};
/* ConnectionManager */
const int UpnpConnectionManagerDescriptionLengthUX = 3533;
const int UpnpConnectionManagerDescriptionLength = 948;
const char UpnpConnectionManagerDescription[948] = {
	0x50,0x48,0x54,0x54,0x50,0x2F,0x31,0x2E,0x31,0x20,0x32,0x30,0x30,0x20,0x20,0x4F,0x4B,0x0D,0x0A,0x43
	,0x4F,0x4E,0x54,0x45,0x4E,0x54,0x2D,0x54,0x59,0x50,0x45,0x3A,0x20,0x20,0x74,0x65,0x78,0x74,0x2F,0x78
	,0x6D,0x6C,0x3B,0x20,0x63,0x68,0x61,0x72,0x73,0x65,0x74,0x3D,0x22,0x75,0x74,0x66,0x2D,0x38,0x22,0x0D
	,0x0A,0x53,0x65,0x72,0x76,0x65,0x72,0x3A,0x20,0x57,0x49,0x4E,0x44,0x4F,0x57,0x53,0x2C,0x20,0x55,0x50
	,0x6E,0x44,0x13,0x13,0x30,0x2C,0x20,0x49,0x6E,0x74,0x65,0x6C,0x20,0x4D,0x69,0x63,0x72,0x6F,0x53,0x74
	,0x61,0x63,0x6B,0x84,0x05,0x3B,0x2E,0x31,0x39,0x32,0x32,0x0D,0x0A,0x43,0x6F,0x6E,0x74,0x65,0x6E,0x74
	,0x2D,0x4C,0x65,0x6E,0x67,0x74,0x68,0x3A,0x20,0x33,0x33,0x39,0x35,0x0D,0x0A,0x0D,0x0A,0x3C,0x3F,0x78
	,0x6D,0x6C,0x20,0x76,0x65,0x72,0x73,0x69,0x6F,0x6E,0x3D,0x22,0x31,0x2E,0x30,0x22,0x20,0x65,0x6E,0x63
	,0x6F,0x64,0x69,0x6E,0x67,0x08,0x1D,0x37,0x3F,0x3E,0x3C,0x73,0x63,0x70,0x64,0x20,0x78,0x6D,0x6C,0x6E
	,0x73,0x3D,0x22,0x75,0x72,0x6E,0x3A,0x73,0x63,0x68,0x65,0x6D,0x61,0x73,0x2D,0x75,0x70,0x6E,0x70,0x2D
	,0x6F,0x72,0x67,0x3A,0x73,0x65,0x72,0x76,0x69,0x63,0x65,0x2D,0x31,0x2D,0x30,0x22,0x3E,0x3C,0x73,0x70
	,0x65,0x63,0x56,0x06,0x15,0x0B,0x3E,0x3C,0x6D,0x61,0x6A,0x6F,0x72,0x3E,0x31,0x3C,0x2F,0x46,0x02,0x0A
	,0x3C,0x6D,0x69,0x6E,0x6F,0x72,0x3E,0x30,0x3C,0x2F,0x46,0x02,0x02,0x3C,0x2F,0x8D,0x0B,0x0A,0x61,0x63
	,0x74,0x69,0x6F,0x6E,0x4C,0x69,0x73,0x74,0x08,0x03,0x16,0x3E,0x3C,0x6E,0x61,0x6D,0x65,0x3E,0x47,0x65
	,0x74,0x43,0x75,0x72,0x72,0x65,0x6E,0x74,0x43,0x6F,0x6E,0x6E,0x65,0xC5,0x09,0x05,0x49,0x44,0x73,0x3C
	,0x2F,0x85,0x07,0x09,0x3C,0x61,0x72,0x67,0x75,0x6D,0x65,0x6E,0x74,0x87,0x0E,0x00,0x87,0x03,0x00,0x07
	,0x0F,0x00,0x95,0x0C,0x03,0x64,0x69,0x72,0x86,0x11,0x06,0x3E,0x6F,0x75,0x74,0x3C,0x2F,0xCA,0x03,0x16
	,0x3C,0x72,0x65,0x6C,0x61,0x74,0x65,0x64,0x53,0x74,0x61,0x74,0x65,0x56,0x61,0x72,0x69,0x61,0x62,0x6C
	,0x65,0x3E,0x56,0x1F,0x00,0xD5,0x0A,0x02,0x3C,0x2F,0x0A,0x20,0x01,0x2F,0x4E,0x26,0x04,0x2F,0x61,0x63
	,0x74,0x0B,0x3A,0x00,0x1C,0x37,0x03,0x6E,0x66,0x6F,0x71,0x37,0x00,0x12,0x37,0x02,0x69,0x6E,0xE2,0x36
	,0x06,0x41,0x5F,0x41,0x52,0x47,0x5F,0xC4,0x9E,0x01,0x5F,0x4E,0x13,0x00,0xA1,0x37,0x00,0x0F,0x5A,0x05
	,0x52,0x63,0x73,0x49,0x44,0x37,0x58,0x00,0x4B,0x21,0x03,0x52,0x63,0x73,0xB4,0x1F,0x0B,0x41,0x56,0x54
	,0x72,0x61,0x6E,0x73,0x70,0x6F,0x72,0x74,0xBF,0x21,0x00,0xC5,0x42,0x00,0xCF,0x13,0x00,0x30,0x43,0x08
	,0x50,0x72,0x6F,0x74,0x6F,0x63,0x6F,0x6C,0x0C,0x72,0x00,0xFA,0x44,0x00,0x8E,0x13,0x00,0x31,0x23,0x03
	,0x65,0x65,0x72,0x8A,0xCD,0x07,0x4D,0x61,0x6E,0x61,0x67,0x65,0x72,0x3F,0x6A,0x03,0x50,0x45,0x5F,0xD3
	,0x14,0x00,0xBE,0x26,0x00,0x7F,0x8F,0x00,0xBF,0xB0,0x00,0x84,0xE2,0x01,0x44,0x48,0xCE,0x00,0xBF,0xB1
	,0x03,0x50,0x45,0x5F,0xCB,0x12,0x00,0x30,0xD2,0x00,0xC4,0xE7,0x02,0x75,0x73,0x7F,0x68,0x00,0x8D,0xF3
	,0x00,0x88,0x14,0x00,0xA1,0xF4,0x00,0x49,0xF7,0x03,0x4C,0x69,0x73,0xC5,0x03,0x00,0x07,0xED,0x00,0x08
	,0x02,0x00,0x05,0xFA,0x03,0x47,0x65,0x74,0xD4,0xB7,0x00,0xCE,0x0E,0x00,0x10,0x35,0x05,0x6F,0x75,0x72
	,0x63,0x65,0xF7,0xE5,0x00,0x46,0x0F,0x00,0x7E,0xC1,0x04,0x53,0x69,0x6E,0x6B,0xF8,0x1F,0x03,0x69,0x6E
	,0x6B,0xAF,0xE0,0x00,0x18,0x52,0x00,0x47,0x54,0x00,0x86,0x57,0x07,0x73,0x65,0x72,0x76,0x69,0x63,0x65
	,0x85,0xFD,0x01,0x54,0x46,0xF1,0x01,0x73,0x0C,0xF5,0x10,0x20,0x73,0x65,0x6E,0x64,0x45,0x76,0x65,0x6E
	,0x74,0x73,0x3D,0x22,0x6E,0x6F,0x22,0xC7,0xF3,0x00,0xCB,0xE0,0x00,0xD4,0x61,0x11,0x64,0x61,0x74,0x61
	,0x54,0x79,0x70,0x65,0x3E,0x73,0x74,0x72,0x69,0x6E,0x67,0x3C,0x2F,0x49,0x04,0x03,0x3C,0x2F,0x73,0x4E
	,0xE6,0x00,0xAF,0x1B,0x00,0x12,0x94,0x00,0xA1,0x1C,0x0C,0x61,0x6C,0x6C,0x6F,0x77,0x65,0x64,0x56,0x61
	,0x6C,0x75,0x65,0x07,0x86,0x00,0x8B,0x04,0x05,0x3E,0x4F,0x4B,0x3C,0x2F,0x4D,0x04,0x00,0xCE,0x07,0x15
	,0x43,0x6F,0x6E,0x74,0x65,0x6E,0x74,0x46,0x6F,0x72,0x6D,0x61,0x74,0x4D,0x69,0x73,0x6D,0x61,0x74,0x63
	,0x68,0x9D,0x0C,0x14,0x49,0x6E,0x73,0x75,0x66,0x66,0x69,0x63,0x69,0x65,0x6E,0x74,0x42,0x61,0x6E,0x64
	,0x77,0x69,0x64,0x74,0x9E,0x0C,0x05,0x55,0x6E,0x72,0x65,0x6C,0x85,0xEF,0x07,0x43,0x68,0x61,0x6E,0x6E
	,0x65,0x6C,0x9F,0x0B,0x05,0x6B,0x6E,0x6F,0x77,0x6E,0x90,0x2D,0x01,0x2F,0x12,0x3A,0x00,0x3F,0x5B,0x0D
	,0x41,0x56,0x54,0x72,0x61,0x6E,0x73,0x70,0x6F,0x72,0x74,0x49,0x44,0xD1,0x76,0x02,0x69,0x34,0xFF,0x75
	,0x00,0x4C,0x91,0x03,0x52,0x63,0x73,0xFF,0x18,0x00,0xAB,0x8E,0x00,0xBF,0x1A,0x00,0x2B,0xA9,0x07,0x4D
	,0x61,0x6E,0x61,0x67,0x65,0x72,0xFF,0xC5,0x00,0x4E,0xE1,0x03,0x79,0x65,0x73,0x88,0xE1,0x06,0x53,0x6F
	,0x75,0x72,0x63,0x65,0x7F,0xE0,0x00,0xA6,0x1A,0x03,0x69,0x6E,0x6B,0x7F,0xFA,0x00,0x6F,0xFA,0x03,0x44
	,0x69,0x72,0x06,0xFA,0x00,0xBF,0xF8,0x00,0x45,0xE4,0x03,0x70,0x75,0x74,0x5D,0xF9,0x06,0x4F,0x75,0x74
	,0x70,0x75,0x74,0xBF,0xD4,0x00,0xD9,0x69,0x07,0x43,0x75,0x72,0x72,0x65,0x6E,0x74,0x4C,0xA0,0x01,0x73
	,0xF3,0x84,0x09,0x2F,0x73,0x65,0x72,0x76,0x69,0x63,0x65,0x53,0x44,0xF5,0x01,0x54,0x08,0x05,0x01,0x63
	,0x00,0x00,0x03,0x70,0x64,0x3E,0x00,0x00};
/* ContentDirectory */
const int UpnpContentDirectoryDescriptionLengthUX = 5254;
const int UpnpContentDirectoryDescriptionLength = 1220;
const char UpnpContentDirectoryDescription[1220] = {
	0x50,0x48,0x54,0x54,0x50,0x2F,0x31,0x2E,0x31,0x20,0x32,0x30,0x30,0x20,0x20,0x4F,0x4B,0x0D,0x0A,0x43
	,0x4F,0x4E,0x54,0x45,0x4E,0x54,0x2D,0x54,0x59,0x50,0x45,0x3A,0x20,0x20,0x74,0x65,0x78,0x74,0x2F,0x78
	,0x6D,0x6C,0x3B,0x20,0x63,0x68,0x61,0x72,0x73,0x65,0x74,0x3D,0x22,0x75,0x74,0x66,0x2D,0x38,0x22,0x0D
	,0x0A,0x53,0x65,0x72,0x76,0x65,0x72,0x3A,0x20,0x57,0x49,0x4E,0x44,0x4F,0x57,0x53,0x2C,0x20,0x55,0x50
	,0x6E,0x44,0x13,0x13,0x30,0x2C,0x20,0x49,0x6E,0x74,0x65,0x6C,0x20,0x4D,0x69,0x63,0x72,0x6F,0x53,0x74
	,0x61,0x63,0x6B,0x84,0x05,0x3B,0x2E,0x31,0x39,0x32,0x32,0x0D,0x0A,0x43,0x6F,0x6E,0x74,0x65,0x6E,0x74
	,0x2D,0x4C,0x65,0x6E,0x67,0x74,0x68,0x3A,0x20,0x35,0x31,0x31,0x36,0x0D,0x0A,0x0D,0x0A,0x3C,0x3F,0x78
	,0x6D,0x6C,0x20,0x76,0x65,0x72,0x73,0x69,0x6F,0x6E,0x3D,0x22,0x31,0x2E,0x30,0x22,0x20,0x65,0x6E,0x63
	,0x6F,0x64,0x69,0x6E,0x67,0x08,0x1D,0x37,0x3F,0x3E,0x3C,0x73,0x63,0x70,0x64,0x20,0x78,0x6D,0x6C,0x6E
	,0x73,0x3D,0x22,0x75,0x72,0x6E,0x3A,0x73,0x63,0x68,0x65,0x6D,0x61,0x73,0x2D,0x75,0x70,0x6E,0x70,0x2D
	,0x6F,0x72,0x67,0x3A,0x73,0x65,0x72,0x76,0x69,0x63,0x65,0x2D,0x31,0x2D,0x30,0x22,0x3E,0x3C,0x73,0x70
	,0x65,0x63,0x56,0x06,0x15,0x0B,0x3E,0x3C,0x6D,0x61,0x6A,0x6F,0x72,0x3E,0x31,0x3C,0x2F,0x46,0x02,0x0A
	,0x3C,0x6D,0x69,0x6E,0x6F,0x72,0x3E,0x30,0x3C,0x2F,0x46,0x02,0x02,0x3C,0x2F,0x8D,0x0B,0x0A,0x61,0x63
	,0x74,0x69,0x6F,0x6E,0x4C,0x69,0x73,0x74,0x08,0x03,0x0F,0x3E,0x3C,0x6E,0x61,0x6D,0x65,0x3E,0x42,0x72
	,0x6F,0x77,0x73,0x65,0x3C,0x2F,0x45,0x03,0x09,0x3C,0x61,0x72,0x67,0x75,0x6D,0x65,0x6E,0x74,0x47,0x0A
	,0x00,0x87,0x03,0x00,0xC7,0x0A,0x08,0x4F,0x62,0x6A,0x65,0x63,0x74,0x49,0x44,0x48,0x0B,0x04,0x64,0x69
	,0x72,0x65,0xC6,0x12,0x04,0x69,0x6E,0x3C,0x2F,0x8A,0x03,0x1C,0x3C,0x72,0x65,0x6C,0x61,0x74,0x65,0x64
	,0x53,0x74,0x61,0x74,0x65,0x56,0x61,0x72,0x69,0x61,0x62,0x6C,0x65,0x3E,0x41,0x5F,0x41,0x52,0x47,0x5F
	,0x44,0x62,0x01,0x5F,0x4A,0x12,0x00,0x95,0x0A,0x02,0x3C,0x2F,0x4A,0x1E,0x00,0xCF,0x20,0x00,0x86,0x2B
	,0x04,0x46,0x6C,0x61,0x67,0x7F,0x21,0x02,0x45,0x5F,0xCC,0x12,0x00,0xF0,0x21,0x06,0x46,0x69,0x6C,0x74
	,0x65,0x72,0x3F,0x42,0x02,0x45,0x5F,0xC8,0x11,0x00,0xB0,0x41,0x0D,0x53,0x74,0x61,0x72,0x74,0x69,0x6E
	,0x67,0x49,0x6E,0x64,0x65,0x78,0xBF,0x63,0x02,0x45,0x5F,0x87,0x11,0x00,0xF0,0x62,0x0E,0x52,0x65,0x71
	,0x75,0x65,0x73,0x74,0x65,0x64,0x43,0x6F,0x75,0x6E,0x74,0x3F,0x85,0x02,0x45,0x5F,0x87,0x11,0x00,0xF1
	,0x42,0x0B,0x6F,0x72,0x74,0x43,0x72,0x69,0x74,0x65,0x72,0x69,0x61,0x3F,0xA6,0x02,0x45,0x5F,0x4E,0x13
	,0x00,0x72,0x44,0x03,0x73,0x75,0x6C,0x53,0x42,0x03,0x6F,0x75,0x74,0xAD,0xC7,0x00,0x08,0x12,0x00,0x30
	,0xC7,0x0E,0x4E,0x75,0x6D,0x62,0x65,0x72,0x52,0x65,0x74,0x75,0x72,0x6E,0x65,0x64,0x3F,0x22,0x00,0xBA
	,0x64,0x0C,0x54,0x6F,0x74,0x61,0x6C,0x4D,0x61,0x74,0x63,0x68,0x65,0x73,0x7F,0x21,0x00,0xFA,0x85,0x08
	,0x55,0x70,0x64,0x61,0x74,0x65,0x49,0x44,0xBF,0x63,0x03,0x50,0x45,0x5F,0x8A,0x12,0x00,0xA1,0xE9,0x00
	,0x49,0xEC,0x03,0x4C,0x69,0x73,0xC5,0x03,0x00,0x47,0xE0,0x00,0x08,0x02,0x00,0x05,0xEF,0x13,0x47,0x65
	,0x74,0x53,0x65,0x61,0x72,0x63,0x68,0x43,0x61,0x70,0x61,0x62,0x69,0x6C,0x69,0x74,0x69,0x0A,0x4A,0x00
	,0x4E,0x10,0x00,0x90,0xFD,0x00,0xC8,0x0D,0x00,0xF8,0x55,0x00,0x14,0x1E,0x00,0x7F,0x35,0x00,0x4B,0x35
	,0x03,0x6F,0x72,0x74,0xF2,0x34,0x03,0x6F,0x72,0x74,0x7C,0x34,0x03,0x6F,0x72,0x74,0xFF,0x33,0x00,0x19
	,0x69,0x05,0x79,0x73,0x74,0x65,0x6D,0xD0,0x91,0x00,0x1D,0x68,0x01,0x49,0x38,0xDD,0x00,0x10,0x1B,0x00
	,0x7F,0x9A,0x00,0xCD,0x8B,0x00,0xA5,0x96,0x09,0x43,0x6F,0x6E,0x74,0x61,0x69,0x6E,0x65,0x72,0x54,0xCC
	,0x02,0x69,0x6E,0x6D,0xEC,0x08,0x4F,0x62,0x6A,0x65,0x63,0x74,0x49,0x44,0x32,0xED,0x00,0xC7,0xC5,0x07
	,0x72,0x69,0x74,0x65,0x72,0x69,0x61,0x7F,0x22,0x02,0x45,0x5F,0xD0,0x13,0x00,0xF0,0x23,0x06,0x46,0x69
	,0x6C,0x74,0x65,0x72,0x3F,0x44,0x02,0x45,0x5F,0xC8,0x11,0x00,0xB1,0x43,0x0C,0x74,0x61,0x72,0x74,0x69
	,0x6E,0x67,0x49,0x6E,0x64,0x65,0x78,0xBF,0x65,0x02,0x45,0x5F,0x87,0x11,0x00,0xF0,0x64,0x0E,0x52,0x65
	,0x71,0x75,0x65,0x73,0x74,0x65,0x64,0x43,0x6F,0x75,0x6E,0x74,0x3F,0x87,0x02,0x45,0x5F,0x87,0x11,0x00
	,0x71,0x86,0x03,0x6F,0x72,0x74,0xFF,0x85,0x00,0xCB,0x85,0x03,0x6F,0x72,0x74,0x7A,0x85,0x06,0x52,0x65
	,0x73,0x75,0x6C,0x74,0x37,0xFA,0x00,0x8B,0xC9,0x00,0x08,0x12,0x00,0x30,0xC9,0x0E,0x4E,0x75,0x6D,0x62
	,0x65,0x72,0x52,0x65,0x74,0x75,0x72,0x6E,0x65,0x64,0x3F,0x22,0x00,0xBA,0x64,0x0C,0x54,0x6F,0x74,0x61
	,0x6C,0x4D,0x61,0x74,0x63,0x68,0x65,0x73,0x7F,0x21,0x00,0xFA,0x85,0x08,0x55,0x70,0x64,0x61,0x74,0x65
	,0x49,0x44,0xBF,0x63,0x03,0x50,0x45,0x5F,0x8A,0x12,0x00,0xA1,0xE9,0x00,0x49,0xEC,0x03,0x4C,0x69,0x73
	,0xC5,0x03,0x00,0x47,0xE0,0x00,0x47,0x02,0x00,0x86,0x05,0x07,0x73,0x65,0x72,0x76,0x69,0x63,0x65,0x05
	,0xFB,0x01,0x54,0x46,0xFA,0x01,0x73,0x0C,0xFE,0x10,0x20,0x73,0x65,0x6E,0x64,0x45,0x76,0x65,0x6E,0x74
	,0x73,0x3D,0x22,0x6E,0x6F,0x22,0xC7,0xFC,0x00,0x0B,0xEC,0x0A,0x42,0x72,0x6F,0x77,0x73,0x65,0x46,0x6C
	,0x61,0x67,0xC9,0xFE,0x10,0x61,0x74,0x61,0x54,0x79,0x70,0x65,0x3E,0x73,0x74,0x72,0x69,0x6E,0x67,0x3C
	,0x2F,0x49,0x04,0x0D,0x3C,0x61,0x6C,0x6C,0x6F,0x77,0x65,0x64,0x56,0x61,0x6C,0x75,0x65,0xC6,0x25,0x00
	,0x8C,0x04,0x01,0x3E,0x06,0x13,0x04,0x4D,0x65,0x74,0x61,0x04,0x11,0x02,0x3C,0x2F,0x4D,0x07,0x00,0xD4
	,0x0A,0x01,0x44,0x05,0xF8,0x08,0x43,0x68,0x69,0x6C,0x64,0x72,0x65,0x6E,0x50,0x0C,0x01,0x2F,0xD2,0x1B
	,0x02,0x2F,0x73,0xCE,0xF2,0x00,0x5A,0x3B,0x03,0x79,0x65,0x73,0x88,0x3B,0x09,0x43,0x6F,0x6E,0x74,0x61
	,0x69,0x6E,0x65,0x72,0x08,0x72,0x01,0x73,0xE3,0x3A,0x00,0xB5,0x1A,0x06,0x53,0x79,0x73,0x74,0x65,0x6D
	,0xD1,0x8B,0x00,0x48,0x54,0x03,0x75,0x69,0x34,0xF6,0x18,0x00,0x95,0x6E,0x0C,0x53,0x6F,0x72,0x74,0x43
	,0x72,0x69,0x74,0x65,0x72,0x69,0x61,0x7F,0x34,0x00,0x18,0x8A,0x00,0xC5,0x18,0x0A,0x61,0x70,0x61,0x62
	,0x69,0x6C,0x69,0x74,0x69,0x65,0x3F,0x4E,0x00,0xE4,0xA3,0x00,0x3F,0x4F,0x00,0xA8,0xBD,0x05,0x49,0x6E
	,0x64,0x65,0x78,0x3F,0x68,0x00,0xA0,0xD6,0x08,0x4F,0x62,0x6A,0x65,0x63,0x74,0x49,0x44,0x7F,0x4D,0x00
	,0x23,0xF1,0x05,0x43,0x6F,0x75,0x6E,0x74,0xBF,0x9B,0x00,0xA0,0x9B,0x06,0x52,0x65,0x73,0x75,0x6C,0x74
	,0x3F,0x9A,0x00,0x19,0x9A,0x05,0x65,0x61,0x72,0x63,0x68,0xBF,0x9A,0x00,0xF0,0xCF,0x00,0x06,0x1D,0x00
	,0x7F,0xD0,0x00,0xEA,0xEB,0x06,0x46,0x69,0x6C,0x74,0x65,0x72,0x73,0xEA,0x09,0x2F,0x73,0x65,0x72,0x76
	,0x69,0x63,0x65,0x53,0x04,0xF0,0x01,0x54,0x08,0x05,0x01,0x63,0x00,0x00,0x03,0x70,0x64,0x3E,0x00,0x00
};



struct UpnpDataObject;

//
// It should not be necessary to expose/modify any of these structures. They
// are used by the internal stack
//

struct SubscriberInfo
{
	char* SID;		// Subscription ID
	int SIDLength;
	int SEQ;
	
	
	int Address;
	unsigned short Port;
	char* Path;
	int PathLength;
	int RefCount;
	int Disposing;
	
	#if defined(WIN32) || defined(_WIN32_WCE)
	unsigned int RenewByTime;
	#else
	struct timeval RenewByTime;
	#endif
	
	struct SubscriberInfo *Next;
	struct SubscriberInfo *Previous;
};
struct UpnpDataObject
{
	//
	// Absolutely DO NOT put anything above these 3 function pointers
	//
	ILibChain_PreSelect PreSelect;
	ILibChain_PostSelect PostSelect;
	ILibChain_Destroy Destroy;
	
	void *EventClient;
	void *Chain;
	int UpdateFlag;
	
	/* Network Poll */
	unsigned int NetworkPollTime;
	
	int ForceExit;
	char *UUID;
	char *UDN;
	char *Serial;
	
	void *WebServerTimer;
	void *HTTPServer;
	
	char *DeviceDescription;
	int DeviceDescriptionLength;
	int InitialNotify;
	
	char* ConnectionManager_SourceProtocolInfo;
	char* ConnectionManager_SinkProtocolInfo;
	char* ConnectionManager_CurrentConnectionIDs;
	char* ContentDirectory_ContainerUpdateIDs;
	char* ContentDirectory_SystemUpdateID;
	
	
	struct sockaddr_in addr;
	int addrlen;
	
	struct ip_mreq mreq;
	char message[4096];
	int *AddressList;
	int AddressListLength;
	
	//phillo,2007/11/7
	// add full interface information
	INTERFACE_INFO *InterfaceList;
	int InterfaceListNum;

	int _NumEmbeddedDevices;
	int WebSocketPortNumber;
	
	
	
	#if defined(WIN32) || defined(_WIN32_WCE)
	SOCKET *NOTIFY_SEND_socks;
	SOCKET NOTIFY_RECEIVE_sock;
#ifdef USE_ALTERNATIVE_UPNP_PORT
	SOCKET *NOTIFY_SEND_socks2;
	SOCKET NOTIFY_RECEIVE_sock2;
#endif
	SOCKET MSEARCH_sock;	
	unsigned int CurrentTime;
	unsigned int NotifyTime;
	#else
	int *NOTIFY_SEND_socks;
	int NOTIFY_RECEIVE_sock;
#ifdef USE_ALTERNATIVE_UPNP_PORT
	int *NOTIFY_SEND_socks2;
	int NOTIFY_RECEIVE_sock2;
#endif
	int MSEARCH_sock;
	struct timeval CurrentTime;
	struct timeval NotifyTime;
	#endif
	
	int SID;
	int NotifyCycleTime;
	
	
	sem_t EventLock;
	struct SubscriberInfo *HeadSubscriberPtr_ConnectionManager;
	int NumberOfSubscribers_ConnectionManager;
	struct SubscriberInfo *HeadSubscriberPtr_ContentDirectory;
	int NumberOfSubscribers_ContentDirectory;
	
};

struct MSEARCH_state
{
	char *ST;
	int STLength;
	void *upnp;
	struct sockaddr_in dest_addr;
};
struct UpnpFragmentNotifyStruct
{
	struct UpnpDataObject *upnp;
	int packetNumber;
};

/* Pre-declarations */
void UpnpFragmentedSendNotify(void *data);
void UpnpSendNotify(const struct UpnpDataObject *upnp);
void UpnpSendByeBye(const struct UpnpDataObject *upnp);
void UpnpMainInvokeSwitch();
void UpnpSendDataXmlEscaped(const void* UpnpToken, const char* Data, const int DataLength, const int Terminate);
void UpnpSendData(const void* UpnpToken, const char* Data, const int DataLength, const int Terminate);
int UpnpPeriodicNotify(struct UpnpDataObject *upnp);
void UpnpSendEvent_Body(void *upnptoken, char *body, int bodylength, struct SubscriberInfo *info);

/*! \fn UpnpGetWebServerToken(const UpnpMicroStackToken MicroStackToken)
\brief Converts a MicroStackToken to a WebServerToken
\para
\a MicroStackToken is the void* returned from a call to UpnpCreateMicroStack. The returned token, is the server token
not the session token.
\param MicroStackToken MicroStack Token
\returns WebServer Token
*/
void* UpnpGetWebServerToken(const UpnpMicroStackToken MicroStackToken)
{
	return(((struct UpnpDataObject*)MicroStackToken)->HTTPServer);
}


#define UpnpBuildSsdpResponsePacket(outpacket,outlength,ipaddr,port,EmbeddedDeviceNumber,USN,USNex,ST,NTex,NotifyTime)\
{\
	*outlength = sprintf(outpacket,"HTTP/1.1 200 OK\r\nLOCATION: http://%d.%d.%d.%d:%d/\r\nEXT:\r\nSERVER: %s, UPnP/1.0, Intel MicroStack/1.0.1922\r\nUSN: uuid:%s%s\r\nCACHE-CONTROL: max-age=%d\r\nST: %s%s\r\n\r\n" ,(ipaddr&0xFF),((ipaddr>>8)&0xFF),((ipaddr>>16)&0xFF),((ipaddr>>24)&0xFF),port,UpnpPLATFORM,USN,USNex,NotifyTime,ST,NTex);\
}
#define UpnpBuildSsdpNotifyPacket(outpacket,outlength,ipaddr,port,EmbeddedDeviceNumber,USN,USNex,NT,NTex,NotifyTime)\
{\
	*outlength = sprintf(outpacket,"NOTIFY * HTTP/1.1\r\nLOCATION: http://%d.%d.%d.%d:%d/\r\nHOST: 239.255.255.250:%d\r\nSERVER: %s, UPnP/1.0, Intel MicroStack/1.0.1922\r\nNTS: ssdp:alive\r\nUSN: uuid:%s%s\r\nCACHE-CONTROL: max-age=%d\r\nNT: %s%s\r\n\r\n",(ipaddr&0xFF),((ipaddr>>8)&0xFF),((ipaddr>>16)&0xFF),((ipaddr>>24)&0xFF),port,UPNP_PORT,UpnpPLATFORM,USN,USNex,NotifyTime,NT,NTex);\
}

#define UpnpBuildSsdpNotifyPacket2(outpacket,outlength,ipaddr,port,EmbeddedDeviceNumber,USN,USNex,NT,NTex,NotifyTime)\
{\
	*outlength = sprintf(outpacket,"NOTIFY * HTTP/1.1\r\nLOCATION: http://%d.%d.%d.%d:%d/\r\nHOST: 239.255.255.250:%d\r\nSERVER: %s, UPnP/1.0, Intel MicroStack/1.0.1922\r\nNTS: ssdp:alive\r\nUSN: uuid:%s%s\r\nCACHE-CONTROL: max-age=%d\r\nNT: %s%s\r\n\r\n",(ipaddr&0xFF),((ipaddr>>8)&0xFF),((ipaddr>>16)&0xFF),((ipaddr>>24)&0xFF),port,ALTERNATIVE_UPNP_PORT,UpnpPLATFORM,USN,USNex,NotifyTime,NT,NTex);\
}




void UpnpSetDisconnectFlag(UpnpSessionToken token,void *flag)
{
	((struct ILibWebServer_Session*)token)->Reserved10=flag;
}

/*! \defgroup FragmentResponse Fragmented Response System
\ingroup MicroStack
\brief Methods used by application to response to invocations in a fragmented manner.
\para
Typically an application will use one of the \a UpnpResponse_ methods to resond to an invocation
request. However, that requires that all of the arguments are known at the time that method is called.
There are times when the application may not be able to do that, such as when querying a back-end
server.<br><br>
In this case, the application can utilise the Fragmented Response System. The application would need
to call \a UpnpAsyncResponse_START exactly once to initialise the response. Then the application would
repeatedly call \a UpnpAsyncResponse_OUT for each argument in the response. Then finally, a call to
\a UpnpAsyncResponse_DONE to complete
\{
	*/
	/*! \fn UpnpAsyncResponse_START(const UpnpSessionToken UpnpToken, const char* actionName, const char* serviceUrnWithVersion)
	\brief Fragmented Response Initializer
	\param UpnpToken The UPnP token received in the invocation request
	\param actionName The name of the method this response is for
	\param serviceUrnWithVersion The full service type URN, this response is for.
	\returns Send Status
	*/
	int UpnpAsyncResponse_START(const UpnpSessionToken UpnpToken, const char* actionName, const char* serviceUrnWithVersion)
	{
		int RetVal=0;
		#if defined(WIN32) || defined(_WIN32_WCE)
		char* RESPONSE_HEADER = "\r\nEXT:\r\nCONTENT-TYPE: text/xml; charset=\"utf-8\"\r\nSERVER: WINDOWS, UPnP/1.0, Intel MicroStack/1.0.1922";
		#else
		char* RESPONSE_HEADER = "\r\nEXT:\r\nCONTENT-TYPE: text/xml; charset=\"utf-8\"\r\nSERVER: POSIX, UPnP/1.0, Intel MicroStack/1.0.1922";
		#endif
		char* RESPONSE_BODY = "<?xml version=\"1.0\" encoding=\"utf-8\"?>\r\n<s:Envelope s:encodingStyle=\"http://schemas.xmlsoap.org/soap/encoding/\" xmlns:s=\"http://schemas.xmlsoap.org/soap/envelope/\">\r\n<s:Body>\r\n<u:%sResponse xmlns:u=\"%s\">";
		struct ILibWebServer_Session *session = (struct ILibWebServer_Session*)UpnpToken;
		
		int headSize;
		char* head; 
		int headLength;
		
		if(session==NULL){return(1);}
		
		headSize = (int)strlen(RESPONSE_BODY) + (int)strlen(actionName) + (int)strlen(serviceUrnWithVersion) + 1;
		head = (char*) malloc (headSize);
		headLength = sprintf(head, RESPONSE_BODY, actionName, serviceUrnWithVersion);
		
		RetVal = ILibWebServer_StreamHeader_Raw(session,200,"OK",RESPONSE_HEADER,1);
		if(RetVal==0)
		{
			RetVal = ILibWebServer_StreamBody(session,head,headLength,0,0);
		}
		else
		{
			FREE(head);
		}
		return(RetVal);
	}
	/*! \fn UpnpAsyncResponse_DONE(const UpnpSessionToken UpnpToken, const char* actionName)
	\brief Fragmented Response Finalizer
	\param UpnpToken The UPnP token received in the invocation request
	\param actionName The name of the method this response is for
	\returns Send Status
	*/
	int UpnpAsyncResponse_DONE(const UpnpSessionToken UpnpToken, const char* actionName)
	{
		char* RESPONSE_FOOTER = "</u:%sResponse>\r\n   </s:Body>\r\n</s:Envelope>";
		
		int footSize = (int)strlen(RESPONSE_FOOTER) + (int)strlen(actionName);
		struct ILibWebServer_Session *session = (struct ILibWebServer_Session*)UpnpToken;
		char* footer;
		int footLength;
		
		footer = (char*) MALLOC(footSize);
		footLength = sprintf(footer, RESPONSE_FOOTER, actionName);
		
		return(ILibWebServer_StreamBody(session,footer,footLength,0,1));
	}
	/*! \fn UpnpAsyncResponse_OUT(const UpnpSessionToken UpnpToken, const char* outArgName, const char* bytes, const int byteLength, enum ILibAsyncSocket_MemoryOwnership bytesMemoryOwnership,const int startArg, const int endArg)
	\brief Fragmented Response Data
	\param UpnpToken The UPnP token received in the invocation request
	\param outArgName Variable Name
	\param bytes Variable Data \b Note: For string types, this MUST be escaped.
	\param byteLength Length of \a bytes
	\param bytesMemoryOwnership Memory Ownership flag for \a bytes
	\param startArg Boolean. 1 to start response, 0 to continue response
	\param endArg Boolean. 1 to finish response, 0 to continue response
	\returns Send Status
	*/
	int UpnpAsyncResponse_OUT(const UpnpSessionToken UpnpToken, const char* outArgName, const char* bytes, const int byteLength, enum ILibAsyncSocket_MemoryOwnership bytesMemoryOwnership,const int startArg, const int endArg)
	{
		struct ILibWebServer_Session *session = (struct ILibWebServer_Session*)UpnpToken;
		int RetVal=0;
		
		if (startArg != 0)
		{
			RetVal = ILibWebServer_StreamBody(session,"<",1,1,0);
			if(RetVal==0)
			{
				RetVal = ILibWebServer_StreamBody(session,(char*)outArgName,(int)strlen(outArgName),1,0);
			}
			if(RetVal==0)
			{
				RetVal = ILibWebServer_StreamBody(session,">",1,1,0);
			}
		}
		
		if(byteLength>0 && RetVal == 0)
		{
			RetVal = ILibWebServer_StreamBody(session,(char*)bytes,byteLength,bytesMemoryOwnership,0);
		}
		
		if (endArg != 0 && RetVal == 0)
		{
			RetVal = ILibWebServer_StreamBody(session,"</",2,1,0);
			if(RetVal==0)
			{
				ILibWebServer_StreamBody(session,(char*)outArgName,(int)strlen(outArgName),1,0);
			}
			if(RetVal==0)
			{
				ILibWebServer_StreamBody(session,">\r\n",3,1,0);
			}
		}
		return(RetVal);
	}
	/*! \} */


/*! \fn UpnpIPAddressListChanged(UpnpMicroStackToken MicroStackToken)
\brief Tell the underlying MicroStack that an IPAddress may have changed
\param MicroStackToken Microstack
*/
void UpnpIPAddressListChanged(UpnpMicroStackToken MicroStackToken)
{
	((struct UpnpDataObject*)MicroStackToken)->UpdateFlag = 1;
	ILibForceUnBlockChain(((struct UpnpDataObject*)MicroStackToken)->Chain);
}

//
//	Internal underlying Initialization, that shouldn't be called explicitely
// 
// <param name="state">State object</param>
// <param name="NotifyCycleSeconds">Cycle duration</param>
// <param name="PortNumber">Port Number</param>
void UpnpInit(struct UpnpDataObject *state,const int NotifyCycleSeconds,const unsigned short PortNumber)
{
	int ra = 1;
	int i;
	struct sockaddr_in addr;
	struct ip_mreq mreq;
	unsigned char TTL = 4;
	
	/* Complete State Reset */
	memset(state,0,sizeof(struct UpnpDataObject));
	
	/* Setup Notification Timer */
	state->NotifyCycleTime = NotifyCycleSeconds;
	
	#if defined(WIN32) || defined(_WIN32_WCE)
	state->CurrentTime = GetTickCount() / 1000;
	state->NotifyTime = state->CurrentTime  + (state->NotifyCycleTime/2);
	#else
	gettimeofday(&(state->CurrentTime),NULL);
	(state->NotifyTime).tv_sec = (state->CurrentTime).tv_sec  + (state->NotifyCycleTime/2);
	#endif
	
	memset((char *)&(state->addr), 0, sizeof(state->addr));
	state->addr.sin_family = AF_INET;
	state->addr.sin_addr.s_addr = htonl(INADDR_ANY);
	state->addr.sin_port = (unsigned short)htons(UPNP_PORT);
	state->addrlen = sizeof(state->addr);
	
	/* Set up socket */
	state->AddressListLength = ILibGetLocalIPAddressList(&(state->AddressList));

	//phillo,2007/11/7
	// get full interface information
	//
	state->InterfaceListNum = ILibGetLocalIPInteraceList(&(state->InterfaceList));
	if( state->AddressListLength != state->InterfaceListNum && state->AddressListLength != 0)
	{
		if( state->AddressList != 0 )
			free(state->AddressList);

		state->AddressListLength = ILibGetLocalIPAddressList(&(state->AddressList));

	}


	#if defined(WIN32) || defined(_WIN32_WCE)
	state->NOTIFY_SEND_socks = (SOCKET*)MALLOC(sizeof(int)*(state->AddressListLength));
	#else
	state->NOTIFY_SEND_socks = (int*)MALLOC(sizeof(int)*(state->AddressListLength));
	#endif

	state->NOTIFY_RECEIVE_sock = socket(AF_INET, SOCK_DGRAM, 0);
	memset((char *)&(addr), 0, sizeof(addr));
	addr.sin_family = AF_INET;
	addr.sin_addr.s_addr = htonl(INADDR_ANY);
	addr.sin_port = (unsigned short)htons(UPNP_PORT);
	if (setsockopt(state->NOTIFY_RECEIVE_sock, SOL_SOCKET, SO_REUSEADDR,(char*)&ra, sizeof(ra)) < 0)
	{
		printf("Setting SockOpt SO_REUSEADDR failed\r\n");
		exit(1);
	}
	if (bind(state->NOTIFY_RECEIVE_sock, (struct sockaddr *) &(addr), sizeof(addr)) < 0)
	{
		printf("Could not bind to UPnP Listen Port\r\n");
		exit(1);
	}

#ifdef USE_ALTERNATIVE_UPNP_PORT
	// Allocate memory if SEND socket for UPNP port 2
	#if defined(WIN32) || defined(_WIN32_WCE)
	state->NOTIFY_SEND_socks2 = (SOCKET*)MALLOC(sizeof(int)*(state->AddressListLength));
	#else
	state->NOTIFY_SEND_socks2 = (int*)MALLOC(sizeof(int)*(state->AddressListLength));
	#endif

	// Create the alternative UPNP port
	state->NOTIFY_RECEIVE_sock2 = socket(AF_INET, SOCK_DGRAM, 0);
	memset((char *)&(addr), 0, sizeof(addr));
	addr.sin_family = AF_INET;
	addr.sin_addr.s_addr = htonl(INADDR_ANY);
	addr.sin_port = (unsigned short)htons(ALTERNATIVE_UPNP_PORT);
	if (setsockopt(state->NOTIFY_RECEIVE_sock2, SOL_SOCKET, SO_REUSEADDR,(char*)&ra, sizeof(ra)) < 0)
	{
		printf("Setting SockOpt SO_REUSEADDR failed for port 2\r\n");
		exit(1);
	}
	if (bind(state->NOTIFY_RECEIVE_sock2, (struct sockaddr *) &(addr), sizeof(addr)) < 0)
	{
		printf("Could not bind to UPnP Listen Port 2\r\n");
		exit(1);
	}
#endif

	//
	// Iterate through all the current IP Addresses
	//
	for(i=0;i<state->AddressListLength;++i)
	{
		state->NOTIFY_SEND_socks[i] = socket(AF_INET, SOCK_DGRAM, 0);
		memset((char *)&(addr), 0, sizeof(addr));
		addr.sin_family = AF_INET;
		addr.sin_addr.s_addr = state->AddressList[i];
		addr.sin_port = (unsigned short)htons(UPNP_PORT);
		//
		// Set re-use address
		//
		if (setsockopt(state->NOTIFY_SEND_socks[i], SOL_SOCKET, SO_REUSEADDR,(char*)&ra, sizeof(ra)) == 0)
		{
			//
			// Set the Multicast TTL
			//
			if (setsockopt(state->NOTIFY_SEND_socks[i], IPPROTO_IP, IP_MULTICAST_TTL,(char*)&TTL, sizeof(TTL)) < 0)
			{
				/* Ignore this case */
			}
			
			//
			// Bind the socket
			//
			if (bind(state->NOTIFY_SEND_socks[i], (struct sockaddr *) &(addr), sizeof(addr)) == 0)
			{
				mreq.imr_multiaddr.s_addr = inet_addr(UPNP_GROUP);
				mreq.imr_interface.s_addr = state->AddressList[i];
				//
				// Join the multicast group
				//
				if (setsockopt(state->NOTIFY_RECEIVE_sock, IPPROTO_IP, IP_ADD_MEMBERSHIP,(char*)&mreq, sizeof(mreq)) < 0)
				{
					/* Does not matter */
				}
			}
		}
	}

#ifdef USE_ALTERNATIVE_UPNP_PORT
	// Create the send sockets for UPNP port 2
	for(i=0;i<state->AddressListLength;++i)
	{
		state->NOTIFY_SEND_socks2[i] = socket(AF_INET, SOCK_DGRAM, 0);
		memset((char *)&(addr), 0, sizeof(addr));
		addr.sin_family = AF_INET;
		addr.sin_addr.s_addr = state->AddressList[i];
		addr.sin_port = (unsigned short)htons(ALTERNATIVE_UPNP_PORT);
		//
		// Set re-use address
		//
		if (setsockopt(state->NOTIFY_SEND_socks2[i], SOL_SOCKET, SO_REUSEADDR,(char*)&ra, sizeof(ra)) == 0)
		{
			//
			// Set the Multicast TTL
			//
			if (setsockopt(state->NOTIFY_SEND_socks2[i], IPPROTO_IP, IP_MULTICAST_TTL,(char*)&TTL, sizeof(TTL)) < 0)
			{
				/* Ignore this case */
			}
			
			//
			// Bind the socket
			//
			if (bind(state->NOTIFY_SEND_socks2[i], (struct sockaddr *) &(addr), sizeof(addr)) == 0)
			{
				mreq.imr_multiaddr.s_addr = inet_addr(UPNP_GROUP);
				mreq.imr_interface.s_addr = state->AddressList[i];
				//
				// Join the multicast group
				//
				if (setsockopt(state->NOTIFY_RECEIVE_sock2, IPPROTO_IP, IP_ADD_MEMBERSHIP,(char*)&mreq, sizeof(mreq)) < 0)
				{
					/* Does not matter */
				}
			}
		}
	}
#endif
}
void UpnpPostMX_Destroy(void *object)
{
	struct MSEARCH_state *mss = (struct MSEARCH_state*)object;
	FREE(mss->ST);
	FREE(mss);
}
void UpnpPostMX_MSEARCH(void *object)
{
	struct MSEARCH_state *mss = (struct MSEARCH_state*)object;
	
	char *b = (char*)MALLOC(sizeof(char)*5000);
	int packetlength;
	struct sockaddr_in response_addr;
	int response_addrlen;
	#if defined(WIN32) || defined(_WIN32_WCE)
	SOCKET *response_socket;
	#else
	int *response_socket;
	#endif
	
	int cnt;
	int i;
	struct sockaddr_in dest_addr = mss->dest_addr;
	char *ST = mss->ST;
	int STLength = mss->STLength;
	struct UpnpDataObject *upnp = (struct UpnpDataObject*)mss->upnp;
	//phillo,2007/11/7
    struct sockaddr_in *pAddress,*pMask;
	//birdy	
	//INTERFACE_INFO InterfaceList;
//	struct UpnpDataObject *upnp_temp = malloc(sizeof(mss->upnp));
	//int j;
	
	#if defined(WIN32) || defined(_WIN32_WCE)
	response_socket = (SOCKET*)MALLOC(upnp->AddressListLength*sizeof(int));
	#else
	response_socket = (int*)MALLOC(upnp->AddressListLength*sizeof(int));
	#endif
	
	//
	// Iterate through all the current IP Addresses
	//
	for(i=0;i<upnp->AddressListLength;++i)
	{
		pAddress = (struct sockaddr_in *) & (upnp->InterfaceList[i].iiAddress);
		pMask = (struct sockaddr_in *) & (upnp->InterfaceList[i].iiNetmask);
		if( (mss->dest_addr.sin_addr.S_un.S_addr & pMask->sin_addr.S_un.S_addr) != (pAddress->sin_addr.S_un.S_addr & pMask->sin_addr.S_un.S_addr) )
		{
			response_socket[i] = INVALID_SOCKET;
			continue;
		}
		//
		// Create a socket to respond with
		//
		response_socket[i] = socket(AF_INET, SOCK_DGRAM, 0);
		if (response_socket[i]< 0)
		{
			/*
			printf("response socket");
				logfile = fopen("C:\\honeywld.txt","a+");
			fprintf(logfile,"UpnpMicroStack.c(674)\n");
			fclose(logfile);
			*/
			//exit(1);
			//tyhsu add
			continue;
		}

		//birdy 2008/3/26 fix vmware issue
		upnp->AddressList[i] = (((((*(upnp->InterfaceList)).iiAddress).AddressIn).sin_addr).S_un).S_addr;
		//
		memset((char *)&(response_addr), 0, sizeof(response_addr));
		response_addr.sin_family = AF_INET;
		response_addr.sin_addr.s_addr = upnp->AddressList[i];
		response_addr.sin_port = (unsigned short)htons(0);
		response_addrlen = sizeof(response_addr);	
		if (bind(response_socket[i], (struct sockaddr *) &(response_addr), sizeof(response_addr)) < 0)
		{
			/* Ignore if this happens */
		}
	}
	
	//
	// Search for root device
	//
	if(STLength==15 && memcmp(ST,"upnp:rootdevice",15)==0)
	{
		for(i=0;i<upnp->AddressListLength;++i)
		{
			//tyhsu add
			if (response_socket[i] == INVALID_SOCKET)
				continue;
			UpnpBuildSsdpResponsePacket(b,&packetlength,upnp->AddressList[i],(unsigned short)upnp->WebSocketPortNumber,0,upnp->UDN,"::upnp:rootdevice","upnp:rootdevice","",upnp->NotifyCycleTime);
			
			
			cnt = sendto(response_socket[i], b, packetlength, 0,(struct sockaddr *) &dest_addr, sizeof(dest_addr));
		}
	}
	//
	// Search for everything
	//
	else if(STLength==8 && memcmp(ST,"ssdp:all",8)==0)
	{
		for(i=0;i<upnp->AddressListLength;++i)
		{
			//tyhsu add
			if (response_socket[i] == INVALID_SOCKET)
				continue;

			UpnpBuildSsdpResponsePacket(b,&packetlength,upnp->AddressList[i],(unsigned short)upnp->WebSocketPortNumber,0,upnp->UDN,"::upnp:rootdevice","upnp:rootdevice","",upnp->NotifyCycleTime);
			cnt = sendto(response_socket[i], b, packetlength, 0,
			(struct sockaddr *) &dest_addr, sizeof(dest_addr));
			UpnpBuildSsdpResponsePacket(b,&packetlength,upnp->AddressList[i],(unsigned short)upnp->WebSocketPortNumber,0,upnp->UDN,"",upnp->UUID,"",upnp->NotifyCycleTime);
			cnt = sendto(response_socket[i], b, packetlength, 0,
			(struct sockaddr *) &dest_addr, sizeof(dest_addr));
			UpnpBuildSsdpResponsePacket(b,&packetlength,upnp->AddressList[i],(unsigned short)upnp->WebSocketPortNumber,0,upnp->UDN,"::urn:schemas-upnp-org:device:MediaServer:1","urn:schemas-upnp-org:device:MediaServer:1","",upnp->NotifyCycleTime);
			cnt = sendto(response_socket[i], b, packetlength, 0,
			(struct sockaddr *) &dest_addr, sizeof(dest_addr));
			UpnpBuildSsdpResponsePacket(b,&packetlength,upnp->AddressList[i],(unsigned short)upnp->WebSocketPortNumber,0,upnp->UDN,"::urn:schemas-upnp-org:service:ConnectionManager:1","urn:schemas-upnp-org:service:ConnectionManager:1","",upnp->NotifyCycleTime);
			cnt = sendto(response_socket[i], b, packetlength, 0,
			(struct sockaddr *) &dest_addr, sizeof(dest_addr));
			UpnpBuildSsdpResponsePacket(b,&packetlength,upnp->AddressList[i],(unsigned short)upnp->WebSocketPortNumber,0,upnp->UDN,"::urn:schemas-upnp-org:service:ContentDirectory:1","urn:schemas-upnp-org:service:ContentDirectory:1","",upnp->NotifyCycleTime);
			cnt = sendto(response_socket[i], b, packetlength, 0,
			(struct sockaddr *) &dest_addr, sizeof(dest_addr));
		}
		
	}
	if(STLength==(int)strlen(upnp->UUID) && memcmp(ST,upnp->UUID,(int)strlen(upnp->UUID))==0)
	{
		for(i=0;i<upnp->AddressListLength;++i)
		{
			//tyhsu add
			if (response_socket[i] == INVALID_SOCKET)
				continue;

			UpnpBuildSsdpResponsePacket(b,&packetlength,upnp->AddressList[i],(unsigned short)upnp->WebSocketPortNumber,0,upnp->UDN,"",upnp->UUID,"",upnp->NotifyCycleTime);
			cnt = sendto(response_socket[i], b, packetlength, 0,
			(struct sockaddr *) &dest_addr, sizeof(dest_addr));
		}
	}
	if(STLength>=40 && memcmp(ST,"urn:schemas-upnp-org:device:MediaServer:",40)==0 && atoi(ST+40)<=1)
	{
		for(i=0;i<upnp->AddressListLength;++i)
		{
			//tyhsu add
			if (response_socket[i] == INVALID_SOCKET)
				continue;

			UpnpBuildSsdpResponsePacket(b,&packetlength,upnp->AddressList[i],(unsigned short)upnp->WebSocketPortNumber,0,upnp->UDN,"::urn:schemas-upnp-org:device:MediaServer:1",ST,"",upnp->NotifyCycleTime);
			cnt = sendto(response_socket[i], b, packetlength, 0,
			(struct sockaddr *) &dest_addr, sizeof(dest_addr));
		}
	}
	if(STLength>=47 && memcmp(ST,"urn:schemas-upnp-org:service:ConnectionManager:",47)==0 && atoi(ST+47)<=1)
	{
		for(i=0;i<upnp->AddressListLength;++i)
		{
			//tyhsu add
			if (response_socket[i] == INVALID_SOCKET)
				continue;

			UpnpBuildSsdpResponsePacket(b,&packetlength,upnp->AddressList[i],(unsigned short)upnp->WebSocketPortNumber,0,upnp->UDN,"::urn:schemas-upnp-org:service:ConnectionManager:1",ST,"",upnp->NotifyCycleTime);
			cnt = sendto(response_socket[i], b, packetlength, 0,
			(struct sockaddr *) &dest_addr, sizeof(dest_addr));
		}
	}
	if(STLength>=46 && memcmp(ST,"urn:schemas-upnp-org:service:ContentDirectory:",46)==0 && atoi(ST+46)<=1)
	{
		for(i=0;i<upnp->AddressListLength;++i)
		{
			//tyhsu add
			if (response_socket[i] == INVALID_SOCKET)
				continue;

			UpnpBuildSsdpResponsePacket(b,&packetlength,upnp->AddressList[i],(unsigned short)upnp->WebSocketPortNumber,0,upnp->UDN,"::urn:schemas-upnp-org:service:ContentDirectory:1",ST,"",upnp->NotifyCycleTime);
			cnt = sendto(response_socket[i], b, packetlength, 0,
			(struct sockaddr *) &dest_addr, sizeof(dest_addr));
		}
	}
	
	
	for(i=0;i<upnp->AddressListLength;++i)
	{
		//tyhsu add
		if (response_socket[i] == INVALID_SOCKET)
			continue;

		#if defined(WIN32) || defined(_WIN32_WCE)
		closesocket(response_socket[i]);
		#else
		close(response_socket[i]);
		#endif
	}
	FREE(response_socket);
	FREE(mss->ST);
	FREE(mss);
	FREE(b);
}
void UpnpProcessMSEARCH(struct UpnpDataObject *upnp, struct packetheader *packet)
{
	char* ST = NULL;
	int STLength = 0;
	struct packetheader_field_node *node;
	int MANOK = 0;
	unsigned long MXVal;
	int MXOK = 0;
	int MX;
	struct MSEARCH_state *mss = NULL;
	
	if(memcmp(packet->DirectiveObj,"*",1)==0)
	{
		if(memcmp(packet->Version,"1.1",3)==0)
		{
			node = packet->FirstField;
			while(node!=NULL)
			{
				if(node->FieldLength==2 && strncasecmp(node->Field,"ST",2)==0)
				{
					//
					// This is what is being searched for
					//
					ST = (char*)MALLOC(1+node->FieldDataLength);
					memcpy(ST,node->FieldData,node->FieldDataLength);
					ST[node->FieldDataLength] = 0;
					STLength = node->FieldDataLength;
				}
				else if(node->FieldLength==3 && strncasecmp(node->Field,"MAN",3)==0 && memcmp(node->FieldData,"\"ssdp:discover\"",15)==0)
				{
					//
					// This is a required header field
					// 
					MANOK = 1;
				}
				else if(node->FieldLength==2 && strncasecmp(node->Field,"MX",2)==0 && ILibGetULong(node->FieldData,node->FieldDataLength,&MXVal)==0)
				{
					//
					// If the timeout value specified is greater than 10 seconds, just force it
					// down to 10 seconds
					//
					MXOK = 1;
					MXVal = MXVal>10?10:MXVal;
				}
				node = node->NextField;
			}
			if(MANOK!=0 && MXOK!=0)
			{
				if(MXVal==0)
				{
					MX = 0;
				}
				else
				{
					//
					// The timeout value should be a random number between 0 and the 
					// specified value
					//
					MX = (int)(0 + ((unsigned short)rand() % MXVal));
				}
				mss = (struct MSEARCH_state*)MALLOC(sizeof(struct MSEARCH_state));
				mss->ST = ST;
				mss->STLength = STLength;
				mss->upnp = upnp;
				memset((char *)&(mss->dest_addr), 0, sizeof(mss->dest_addr));
				mss->dest_addr.sin_family = AF_INET;
				mss->dest_addr.sin_addr = packet->Source->sin_addr;
				mss->dest_addr.sin_port = packet->Source->sin_port;
				
				//
				// Register for a timed callback, so we can respond later
				//
				ILibLifeTime_Add(upnp->WebServerTimer,mss,MX,&UpnpPostMX_MSEARCH,&UpnpPostMX_Destroy);
			}
			else
			{
				FREE(ST);
			}
		}
	}
}
#define UpnpDispatch_ConnectionManager_GetCurrentConnectionIDs(buffer,offset,bufferLength, session)\
{\
	UpnpConnectionManager_GetCurrentConnectionIDs((void*)session);\
}

void UpnpDispatch_ConnectionManager_GetCurrentConnectionInfo(char *buffer, int offset, int bufferLength, struct ILibWebServer_Session *ReaderObject)
{
	long TempLong;
	int OK = 0;
	char *p_ConnectionID = NULL;
	int p_ConnectionIDLength = 0;
	int _ConnectionID = 0;
	struct ILibXMLNode *xnode = ILibParseXML(buffer,offset,bufferLength);
	struct ILibXMLNode *root = xnode;
	if(ILibProcessXMLNodeList(root)!=0)
	{
		/* The XML is not well formed! */
		ILibDestructXMLNodeList(root);
		UpnpResponse_Error(ReaderObject,501,"Invalid XML");
		return;
	}
	while(xnode!=NULL)
	{
		if(xnode->StartTag!=0 && xnode->NameLength==8 && memcmp(xnode->Name,"Envelope",8)==0)
		{
			// Envelope
			xnode = xnode->Next;
			while(xnode!=NULL)
			{
				if(xnode->StartTag!=0 && xnode->NameLength==4 && memcmp(xnode->Name,"Body",4)==0)
				{
					// Body
					xnode = xnode->Next;
					while(xnode!=NULL)
					{
						if(xnode->StartTag!=0 && xnode->NameLength==24 && memcmp(xnode->Name,"GetCurrentConnectionInfo",24)==0)
						{
							// Inside the interesting part of the SOAP
							xnode = xnode->Next;
							while(xnode!=NULL)
							{
								if(xnode->NameLength==12 && memcmp(xnode->Name,"ConnectionID",12)==0)
								{
									p_ConnectionIDLength = ILibReadInnerXML(xnode,&p_ConnectionID);
									OK |= 1;
								}
								if(xnode->Peer==NULL)
								{
									xnode = xnode->Parent;
									break;
								}
								else
								{
									xnode = xnode->Peer;
								}
							}
						}
						if(xnode->Peer==NULL)
						{
							xnode = xnode->Parent;
							break;
						}
						else
						{
							xnode = xnode->Peer;
						}
					}
				}
				if(xnode->Peer==NULL)
				{
					xnode = xnode->Parent;
					break;
				}
				else
				{
					xnode = xnode->Peer;
				}
			}
		}
		xnode = xnode->Peer;
	}
	ILibDestructXMLNodeList(root);
	if (OK != 1)
	{
		UpnpResponse_Error(ReaderObject,402,"Incorrect Arguments");
		return;
	}
	
	/* Type Checking */
	OK = ILibGetLong(p_ConnectionID,p_ConnectionIDLength, &TempLong);
	if(OK!=0)
	{
		UpnpResponse_Error(ReaderObject,402,"Argument[ConnectionID] illegal value");
		return;
	}
	_ConnectionID = (int)TempLong;
	UpnpConnectionManager_GetCurrentConnectionInfo((void*)ReaderObject,_ConnectionID);
}

#define UpnpDispatch_ConnectionManager_GetProtocolInfo(buffer,offset,bufferLength, session)\
{\
	UpnpConnectionManager_GetProtocolInfo((void*)session);\
}

void UpnpDispatch_ContentDirectory_Browse(char *buffer, int offset, int bufferLength, struct ILibWebServer_Session *ReaderObject)
{
	unsigned long TempULong;
	int OK = 0;
	char *p_ObjectID = NULL;
	int p_ObjectIDLength = 0;
	char* _ObjectID = "";
	int _ObjectIDLength;
	char *p_BrowseFlag = NULL;
	int p_BrowseFlagLength = 0;
	char* _BrowseFlag = "";
	int _BrowseFlagLength;
	char *p_Filter = NULL;
	int p_FilterLength = 0;
	char* _Filter = "";
	int _FilterLength;
	char *p_StartingIndex = NULL;
	int p_StartingIndexLength = 0;
	unsigned int _StartingIndex = 0;
	char *p_RequestedCount = NULL;
	int p_RequestedCountLength = 0;
	unsigned int _RequestedCount = 0;
	char *p_SortCriteria = NULL;
	int p_SortCriteriaLength = 0;
	char* _SortCriteria = "";
	int _SortCriteriaLength;
	struct ILibXMLNode *xnode = ILibParseXML(buffer,offset,bufferLength);
	struct ILibXMLNode *root = xnode;
	if(ILibProcessXMLNodeList(root)!=0)
	{
		/* The XML is not well formed! */
		ILibDestructXMLNodeList(root);
		UpnpResponse_Error(ReaderObject,501,"Invalid XML");
		return;
	}
	while(xnode!=NULL)
	{
		if(xnode->StartTag!=0 && xnode->NameLength==8 && memcmp(xnode->Name,"Envelope",8)==0)
		{
			// Envelope
			xnode = xnode->Next;
			while(xnode!=NULL)
			{
				if(xnode->StartTag!=0 && xnode->NameLength==4 && memcmp(xnode->Name,"Body",4)==0)
				{
					// Body
					xnode = xnode->Next;
					while(xnode!=NULL)
					{
						if(xnode->StartTag!=0 && xnode->NameLength==6 && memcmp(xnode->Name,"Browse",6)==0)
						{
							// Inside the interesting part of the SOAP
							xnode = xnode->Next;
							while(xnode!=NULL)
							{
								if(xnode->NameLength==8 && memcmp(xnode->Name,"ObjectID",8)==0)
								{
									p_ObjectIDLength = ILibReadInnerXML(xnode,&p_ObjectID);
									p_ObjectID[p_ObjectIDLength]=0;
									OK |= 1;
								}
								else if(xnode->NameLength==10 && memcmp(xnode->Name,"BrowseFlag",10)==0)
								{
									p_BrowseFlagLength = ILibReadInnerXML(xnode,&p_BrowseFlag);
									p_BrowseFlag[p_BrowseFlagLength]=0;
									OK |= 2;
								}
								else if(xnode->NameLength==6 && memcmp(xnode->Name,"Filter",6)==0)
								{
									p_FilterLength = ILibReadInnerXML(xnode,&p_Filter);
									p_Filter[p_FilterLength]=0;
									OK |= 4;
								}
								else if(xnode->NameLength==13 && memcmp(xnode->Name,"StartingIndex",13)==0)
								{
									p_StartingIndexLength = ILibReadInnerXML(xnode,&p_StartingIndex);
									OK |= 8;
								}
								else if(xnode->NameLength==14 && memcmp(xnode->Name,"RequestedCount",14)==0)
								{
									p_RequestedCountLength = ILibReadInnerXML(xnode,&p_RequestedCount);
									OK |= 16;
								}
								else if(xnode->NameLength==12 && memcmp(xnode->Name,"SortCriteria",12)==0)
								{
									p_SortCriteriaLength = ILibReadInnerXML(xnode,&p_SortCriteria);
									p_SortCriteria[p_SortCriteriaLength]=0;
									OK |= 32;
								}
								if(xnode->Peer==NULL)
								{
									xnode = xnode->Parent;
									break;
								}
								else
								{
									xnode = xnode->Peer;
								}
							}
						}
						//phillo,2005/7/22
						// fix access violation
						if( xnode == NULL )
						{
							break;
						}

						if(xnode->Peer==NULL)
						{
							xnode = xnode->Parent;
							break;
						}
						else
						{
							xnode = xnode->Peer;
						}
					}
				}

				//phillo,2005/7/22
				// fix access violation
				if( xnode == NULL )
				{
					break;
				}

				if(xnode->Peer==NULL)
				{
					xnode = xnode->Parent;
					break;
				}
				else
				{
					xnode = xnode->Peer;
				}
			}
		}

		//phillo,2005/7/22
		// fix access violation
		if( xnode == NULL )
		{
			break;
		}

		xnode = xnode->Peer;
	}
	ILibDestructXMLNodeList(root);
	if (OK != 63)
	{
		UpnpResponse_Error(ReaderObject,402,"Incorrect Arguments");
		return;
	}
	
	/* Type Checking */
	_ObjectIDLength = ILibInPlaceXmlUnEscape(p_ObjectID);
	_ObjectID = p_ObjectID;
	_BrowseFlagLength = ILibInPlaceXmlUnEscape(p_BrowseFlag);
	_BrowseFlag = p_BrowseFlag;
	if(memcmp(_BrowseFlag, "BrowseMetadata\0",15) != 0
	&& memcmp(_BrowseFlag, "BrowseDirectChildren\0",21) != 0
	)
	{
		UpnpResponse_Error(ReaderObject,402,"Argument[BrowseFlag] contains a value that is not in AllowedValueList");
		return;
	}
	_FilterLength = ILibInPlaceXmlUnEscape(p_Filter);
	_Filter = p_Filter;
	OK = ILibGetULong(p_StartingIndex,p_StartingIndexLength, &TempULong);
	if(OK!=0)
	{
		UpnpResponse_Error(ReaderObject,402,"Argument[StartingIndex] illegal value");
		return;
	}
	_StartingIndex = (unsigned int)TempULong;
	OK = ILibGetULong(p_RequestedCount,p_RequestedCountLength, &TempULong);
	if(OK!=0)
	{
		UpnpResponse_Error(ReaderObject,402,"Argument[RequestedCount] illegal value");
		return;
	}
	_RequestedCount = (unsigned int)TempULong;
	_SortCriteriaLength = ILibInPlaceXmlUnEscape(p_SortCriteria);
	_SortCriteria = p_SortCriteria;
	UpnpContentDirectory_Browse((void*)ReaderObject,_ObjectID,_BrowseFlag,_Filter,_StartingIndex,_RequestedCount,_SortCriteria);
}

#define UpnpDispatch_ContentDirectory_GetSearchCapabilities(buffer,offset,bufferLength, session)\
{\
	UpnpContentDirectory_GetSearchCapabilities((void*)session);\
}

#define UpnpDispatch_ContentDirectory_GetSortCapabilities(buffer,offset,bufferLength, session)\
{\
	UpnpContentDirectory_GetSortCapabilities((void*)session);\
}

#define UpnpDispatch_ContentDirectory_GetSystemUpdateID(buffer,offset,bufferLength, session)\
{\
	UpnpContentDirectory_GetSystemUpdateID((void*)session);\
}

void UpnpDispatch_ContentDirectory_Search(char *buffer, int offset, int bufferLength, struct ILibWebServer_Session *ReaderObject)
{
	unsigned long TempULong;
	int OK = 0;
	char *p_ContainerID = NULL;
	int p_ContainerIDLength = 0;
	char* _ContainerID = "";
	int _ContainerIDLength;
	char *p_SearchCriteria = NULL;
	int p_SearchCriteriaLength = 0;
	char* _SearchCriteria = "";
	int _SearchCriteriaLength;
	char *p_Filter = NULL;
	int p_FilterLength = 0;
	char* _Filter = "";
	int _FilterLength;
	char *p_StartingIndex = NULL;
	int p_StartingIndexLength = 0;
	unsigned int _StartingIndex = 0;
	char *p_RequestedCount = NULL;
	int p_RequestedCountLength = 0;
	unsigned int _RequestedCount = 0;
	char *p_SortCriteria = NULL;
	int p_SortCriteriaLength = 0;
	char* _SortCriteria = "";
	int _SortCriteriaLength;
	struct ILibXMLNode *xnode = ILibParseXML(buffer,offset,bufferLength);
	struct ILibXMLNode *root = xnode;
	if(ILibProcessXMLNodeList(root)!=0)
	{
		/* The XML is not well formed! */
		ILibDestructXMLNodeList(root);
		UpnpResponse_Error(ReaderObject,501,"Invalid XML");
		return;
	}
	while(xnode!=NULL)
	{
		if(xnode->StartTag!=0 && xnode->NameLength==8 && memcmp(xnode->Name,"Envelope",8)==0)
		{
			// Envelope
			xnode = xnode->Next;
			while(xnode!=NULL)
			{
				if(xnode->StartTag!=0 && xnode->NameLength==4 && memcmp(xnode->Name,"Body",4)==0)
				{
					// Body
					xnode = xnode->Next;
					while(xnode!=NULL)
					{
						if(xnode->StartTag!=0 && xnode->NameLength==6 && memcmp(xnode->Name,"Search",6)==0)
						{
							// Inside the interesting part of the SOAP
							xnode = xnode->Next;
							while(xnode!=NULL)
							{
								if(xnode->NameLength==11 && memcmp(xnode->Name,"ContainerID",11)==0)
								{
									p_ContainerIDLength = ILibReadInnerXML(xnode,&p_ContainerID);
									p_ContainerID[p_ContainerIDLength]=0;
									OK |= 1;
								}
								else if(xnode->NameLength==14 && memcmp(xnode->Name,"SearchCriteria",14)==0)
								{
									p_SearchCriteriaLength = ILibReadInnerXML(xnode,&p_SearchCriteria);
									p_SearchCriteria[p_SearchCriteriaLength]=0;
									OK |= 2;
								}
								else if(xnode->NameLength==6 && memcmp(xnode->Name,"Filter",6)==0)
								{
									p_FilterLength = ILibReadInnerXML(xnode,&p_Filter);
									p_Filter[p_FilterLength]=0;
									OK |= 4;
								}
								else if(xnode->NameLength==13 && memcmp(xnode->Name,"StartingIndex",13)==0)
								{
									p_StartingIndexLength = ILibReadInnerXML(xnode,&p_StartingIndex);
									OK |= 8;
								}
								else if(xnode->NameLength==14 && memcmp(xnode->Name,"RequestedCount",14)==0)
								{
									p_RequestedCountLength = ILibReadInnerXML(xnode,&p_RequestedCount);
									OK |= 16;
								}
								else if(xnode->NameLength==12 && memcmp(xnode->Name,"SortCriteria",12)==0)
								{
									p_SortCriteriaLength = ILibReadInnerXML(xnode,&p_SortCriteria);
									p_SortCriteria[p_SortCriteriaLength]=0;
									OK |= 32;
								}
								if(xnode->Peer==NULL)
								{
									xnode = xnode->Parent;
									break;
								}
								else
								{
									xnode = xnode->Peer;
								}
							}
						}
						if(xnode->Peer==NULL)
						{
							xnode = xnode->Parent;
							break;
						}
						else
						{
							xnode = xnode->Peer;
						}
					}
				}
				if(xnode->Peer==NULL)
				{
					xnode = xnode->Parent;
					break;
				}
				else
				{
					xnode = xnode->Peer;
				}
			}
		}
		xnode = xnode->Peer;
	}
	ILibDestructXMLNodeList(root);
	if (OK != 63)
	{
		UpnpResponse_Error(ReaderObject,402,"Incorrect Arguments");
		return;
	}
	
	/* Type Checking */
	_ContainerIDLength = ILibInPlaceXmlUnEscape(p_ContainerID);
	_ContainerID = p_ContainerID;
	_SearchCriteriaLength = ILibInPlaceXmlUnEscape(p_SearchCriteria);
	_SearchCriteria = p_SearchCriteria;
	_FilterLength = ILibInPlaceXmlUnEscape(p_Filter);
	_Filter = p_Filter;
	OK = ILibGetULong(p_StartingIndex,p_StartingIndexLength, &TempULong);
	if(OK!=0)
	{
		UpnpResponse_Error(ReaderObject,402,"Argument[StartingIndex] illegal value");
		return;
	}
	_StartingIndex = (unsigned int)TempULong;
	OK = ILibGetULong(p_RequestedCount,p_RequestedCountLength, &TempULong);
	if(OK!=0)
	{
		UpnpResponse_Error(ReaderObject,402,"Argument[RequestedCount] illegal value");
		return;
	}
	_RequestedCount = (unsigned int)TempULong;
	_SortCriteriaLength = ILibInPlaceXmlUnEscape(p_SortCriteria);
	_SortCriteria = p_SortCriteria;
	UpnpContentDirectory_Search((void*)ReaderObject,_ContainerID,_SearchCriteria,_Filter,_StartingIndex,_RequestedCount,_SortCriteria);
}


int UpnpProcessPOST(struct ILibWebServer_Session *session, struct packetheader* header, char *bodyBuffer, int offset, int bodyBufferLength)
{
	struct packetheader_field_node *f = header->FirstField;
	char* HOST;
	char* SOAPACTION = NULL;
	int SOAPACTIONLength = 0;
	struct parser_result *r,*r2;
	struct parser_result_field *prf;
	
	int RetVal = 0;
	
	//
	// Iterate through all the HTTP Headers
	//
	while(f!=NULL)
	{
		if(f->FieldLength==4 && strncasecmp(f->Field,"HOST",4)==0)
		{
			HOST = f->FieldData;
		}
		else if(f->FieldLength==10 && strncasecmp(f->Field,"SOAPACTION",10)==0)
		{
			r = ILibParseString(f->FieldData,0,f->FieldDataLength,"#",1);
			SOAPACTION = r->LastResult->data;
			SOAPACTIONLength = r->LastResult->datalength-1;
			ILibDestructParserResults(r);
		}
		else if(f->FieldLength==10 && strncasecmp(f->Field,"USER-AGENT",10)==0)
		{
			// Check UPnP version of the Control Point which invoked us
			r = ILibParseString(f->FieldData,0,f->FieldDataLength," ",1);
			prf = r->FirstResult;
			while(prf!=NULL)
			{
				if(prf->datalength>5 && memcmp(prf->data,"UPnP/",5)==0)
				{
					r2 = ILibParseString(prf->data+5,0,prf->datalength-5,".",1);
					r2->FirstResult->data[r2->FirstResult->datalength]=0;
					r2->LastResult->data[r2->LastResult->datalength]=0;
					if(atoi(r2->FirstResult->data)==1 && atoi(r2->LastResult->data)>0)
					{
						session->Reserved9=1;
					}
					ILibDestructParserResults(r2);
				}
				prf = prf->NextResult;
			}
			ILibDestructParserResults(r);
		}
		f = f->NextField;
	}
	
	if(header->DirectiveObjLength==26 && memcmp((header->DirectiveObj)+1,"ConnectionManager/control",25)==0)
	{
		if(SOAPACTIONLength==23 && memcmp(SOAPACTION,"GetCurrentConnectionIDs",23)==0)
		{
			UpnpDispatch_ConnectionManager_GetCurrentConnectionIDs(bodyBuffer, offset, bodyBufferLength, session);
		}
		else if(SOAPACTIONLength==24 && memcmp(SOAPACTION,"GetCurrentConnectionInfo",24)==0)
		{
			UpnpDispatch_ConnectionManager_GetCurrentConnectionInfo(bodyBuffer, offset, bodyBufferLength, session);
		}
		else if(SOAPACTIONLength==15 && memcmp(SOAPACTION,"GetProtocolInfo",15)==0)
		{
			UpnpDispatch_ConnectionManager_GetProtocolInfo(bodyBuffer, offset, bodyBufferLength, session);
		}
		else
		{
			RetVal=1;
		}
	}
	else if(header->DirectiveObjLength==25 && memcmp((header->DirectiveObj)+1,"ContentDirectory/control",24)==0)
	{
		if(SOAPACTIONLength==6 && memcmp(SOAPACTION,"Browse",6)==0)
		{
			UpnpDispatch_ContentDirectory_Browse(bodyBuffer, offset, bodyBufferLength, session);
		}
		else if(SOAPACTIONLength==21 && memcmp(SOAPACTION,"GetSearchCapabilities",21)==0)
		{
			UpnpDispatch_ContentDirectory_GetSearchCapabilities(bodyBuffer, offset, bodyBufferLength, session);
		}
		else if(SOAPACTIONLength==19 && memcmp(SOAPACTION,"GetSortCapabilities",19)==0)
		{
			UpnpDispatch_ContentDirectory_GetSortCapabilities(bodyBuffer, offset, bodyBufferLength, session);
		}
		else if(SOAPACTIONLength==17 && memcmp(SOAPACTION,"GetSystemUpdateID",17)==0)
		{
			UpnpDispatch_ContentDirectory_GetSystemUpdateID(bodyBuffer, offset, bodyBufferLength, session);
		}
		else if(SOAPACTIONLength==6 && memcmp(SOAPACTION,"Search",6)==0)
		{
			UpnpDispatch_ContentDirectory_Search(bodyBuffer, offset, bodyBufferLength, session);
		}
		else
		{
			RetVal=1;
		}
	}
	else
	{
		RetVal=1;
	}
	
	
	return(RetVal);
}
struct SubscriberInfo* UpnpRemoveSubscriberInfo(struct SubscriberInfo **Head, int *TotalSubscribers,char* SID, int SIDLength)
{
	struct SubscriberInfo *info = *Head;
	while(info!=NULL)
	{
		if(info->SIDLength==SIDLength && memcmp(info->SID,SID,SIDLength)==0)
		{
			if ( info->Previous )
			info->Previous->Next = info->Next;
			else
			*Head = info->Next;
			if ( info->Next )
			info->Next->Previous = info->Previous;
			break;
		}
		info = info->Next;
		
	}
	if(info!=NULL)
	{
		info->Previous = NULL;
		info->Next = NULL;
		--(*TotalSubscribers);
	}
	return(info);
}

#define UpnpDestructSubscriberInfo(info)\
{\
	FREE(info->Path);\
	FREE(info->SID);\
	FREE(info);\
}

#define UpnpDestructEventObject(EvObject)\
{\
	FREE(EvObject->PacketBody);\
	FREE(EvObject);\
}

#define UpnpDestructEventDataObject(EvData)\
{\
	FREE(EvData);\
}
void UpnpExpireSubscriberInfo(struct UpnpDataObject *d, struct SubscriberInfo *info)
{
	struct SubscriberInfo *t = info;
	while(t->Previous!=NULL)
	{
		t = t->Previous;
	}
	if(d->HeadSubscriberPtr_ConnectionManager==t)
	{
		--(d->NumberOfSubscribers_ConnectionManager);
	}
	else if(d->HeadSubscriberPtr_ContentDirectory==t)
	{
		--(d->NumberOfSubscribers_ContentDirectory);
	}
	
	
	if(info->Previous!=NULL)
	{
		// This is not the Head
		info->Previous->Next = info->Next;
		if(info->Next!=NULL)
		{
			info->Next->Previous = info->Previous;
		}
	}
	else
	{
		// This is the Head
		if(d->HeadSubscriberPtr_ConnectionManager==info)
		{
			d->HeadSubscriberPtr_ConnectionManager = info->Next;
			if(info->Next!=NULL)
			{
				info->Next->Previous = NULL;
			}
		}
		else if(d->HeadSubscriberPtr_ContentDirectory==info)
		{
			d->HeadSubscriberPtr_ContentDirectory = info->Next;
			if(info->Next!=NULL)
			{
				info->Next->Previous = NULL;
			}
		}
		else 
		{
			// Error
			return;
		}
		
	}
	--info->RefCount;
	if(info->RefCount==0)
	{
		UpnpDestructSubscriberInfo(info);
	}
}

int UpnpSubscriptionExpired(struct SubscriberInfo *info)
{
	int RetVal = 0;
	#if defined(WIN32) || defined(_WIN32_WCE)
	if(info->RenewByTime < GetTickCount()/1000) {RetVal = -1;}
	#else
	struct timeval tv;
	gettimeofday(&tv,NULL);
	if((info->RenewByTime).tv_sec < tv.tv_sec) {RetVal = -1;}
	#endif
	return(RetVal);
}

void UpnpGetInitialEventBody_ConnectionManager(struct UpnpDataObject *UpnpObject,char ** body, int *bodylength)
{
	int TempLength;
	TempLength = (int)(177+(int)strlen(UpnpObject->ConnectionManager_SourceProtocolInfo)+(int)strlen(UpnpObject->ConnectionManager_SinkProtocolInfo)+(int)strlen(UpnpObject->ConnectionManager_CurrentConnectionIDs));
	*body = (char*)MALLOC(sizeof(char)*TempLength);
	*bodylength = sprintf(*body,"SourceProtocolInfo>%s</SourceProtocolInfo></e:property><e:property><SinkProtocolInfo>%s</SinkProtocolInfo></e:property><e:property><CurrentConnectionIDs>%s</CurrentConnectionIDs",UpnpObject->ConnectionManager_SourceProtocolInfo,UpnpObject->ConnectionManager_SinkProtocolInfo,UpnpObject->ConnectionManager_CurrentConnectionIDs);
}
void UpnpGetInitialEventBody_ContentDirectory(struct UpnpDataObject *UpnpObject,char ** body, int *bodylength)
{
	int TempLength;
	TempLength = (int)(101+(int)strlen(UpnpObject->ContentDirectory_ContainerUpdateIDs)+(int)strlen(UpnpObject->ContentDirectory_SystemUpdateID));
	*body = (char*)MALLOC(sizeof(char)*TempLength);
	*bodylength = sprintf(*body,"ContainerUpdateIDs>%s</ContainerUpdateIDs></e:property><e:property><SystemUpdateID>%s</SystemUpdateID",UpnpObject->ContentDirectory_ContainerUpdateIDs,UpnpObject->ContentDirectory_SystemUpdateID);
}


void UpnpProcessUNSUBSCRIBE(struct packetheader *header, struct ILibWebServer_Session *session)
{
	char* SID = NULL;
	int SIDLength = 0;
	struct SubscriberInfo *Info;
	struct packetheader_field_node *f;
	char* packet = (char*)MALLOC(sizeof(char)*50);
	int packetlength;
	
	//
	// Iterate through all the HTTP headers
	//
	f = header->FirstField;
	while(f!=NULL)
	{
		if(f->FieldLength==3)
		{
			if(strncasecmp(f->Field,"SID",3)==0)
			{
				//
				// Get the Subscription ID
				//
				SID = f->FieldData;
				SIDLength = f->FieldDataLength;
			}
		}
		f = f->NextField;
	}
	sem_wait(&(((struct UpnpDataObject*)session->User)->EventLock));
	if(header->DirectiveObjLength==24 && memcmp(header->DirectiveObj + 1,"ConnectionManager/event",23)==0)
	{
		Info = UpnpRemoveSubscriberInfo(&(((struct UpnpDataObject*)session->User)->HeadSubscriberPtr_ConnectionManager),&(((struct UpnpDataObject*)session->User)->NumberOfSubscribers_ConnectionManager),SID,SIDLength);
		if(Info!=NULL)
		{
			--Info->RefCount;
			if(Info->RefCount==0)
			{
				UpnpDestructSubscriberInfo(Info);
			}
			packetlength = sprintf(packet,"HTTP/1.1 %d %s\r\nContent-Length: 0\r\n\r\n",200,"OK");
			ILibWebServer_Send_Raw(session,packet,packetlength,0,1);
		}
		else
		{
			packetlength = sprintf(packet,"HTTP/1.1 %d %s\r\nContent-Length: 0\r\n\r\n",412,"Invalid SID");
			ILibWebServer_Send_Raw(session,packet,packetlength,0,1);
		}
	}
	else if(header->DirectiveObjLength==23 && memcmp(header->DirectiveObj + 1,"ContentDirectory/event",22)==0)
	{
		Info = UpnpRemoveSubscriberInfo(&(((struct UpnpDataObject*)session->User)->HeadSubscriberPtr_ContentDirectory),&(((struct UpnpDataObject*)session->User)->NumberOfSubscribers_ContentDirectory),SID,SIDLength);
		if(Info!=NULL)
		{
			--Info->RefCount;
			if(Info->RefCount==0)
			{
				UpnpDestructSubscriberInfo(Info);
			}
			packetlength = sprintf(packet,"HTTP/1.1 %d %s\r\nContent-Length: 0\r\n\r\n",200,"OK");
			ILibWebServer_Send_Raw(session,packet,packetlength,0,1);
		}
		else
		{
			packetlength = sprintf(packet,"HTTP/1.1 %d %s\r\nContent-Length: 0\r\n\r\n",412,"Invalid SID");
			ILibWebServer_Send_Raw(session,packet,packetlength,0,1);
		}
	}
	
	sem_post(&(((struct UpnpDataObject*)session->User)->EventLock));
}
void UpnpTryToSubscribe(char* ServiceName, long Timeout, char* URL, int URLLength,struct ILibWebServer_Session *session)
{
	int *TotalSubscribers = NULL;
	struct SubscriberInfo **HeadPtr = NULL;
	struct SubscriberInfo *NewSubscriber,*TempSubscriber;
	int SIDNumber,rnumber;
	char *SID;
	char *TempString;
	int TempStringLength;
	char *TempString2;
	long TempLong;
	char *packet;
	int packetlength;
	char* path;
	
	char* escapedURI;
	int escapedURILength;
	
	char *packetbody = NULL;
	int packetbodyLength;
	
	struct parser_result *p;
	struct parser_result *p2;
	
	struct UpnpDataObject *dataObject = (struct UpnpDataObject*)session->User;
	
	if(strncmp(ServiceName,"ConnectionManager",17)==0)
	{
		TotalSubscribers = &(dataObject->NumberOfSubscribers_ConnectionManager);
		HeadPtr = &(dataObject->HeadSubscriberPtr_ConnectionManager);
	}
	if(strncmp(ServiceName,"ContentDirectory",16)==0)
	{
		TotalSubscribers = &(dataObject->NumberOfSubscribers_ContentDirectory);
		HeadPtr = &(dataObject->HeadSubscriberPtr_ContentDirectory);
	}
	
	
	if(*HeadPtr!=NULL)
	{
		NewSubscriber = *HeadPtr;
		while(NewSubscriber!=NULL)
		{
			if(UpnpSubscriptionExpired(NewSubscriber)!=0)
			{
				TempSubscriber = NewSubscriber->Next;
				NewSubscriber = UpnpRemoveSubscriberInfo(HeadPtr,TotalSubscribers,NewSubscriber->SID,NewSubscriber->SIDLength);
				UpnpDestructSubscriberInfo(NewSubscriber);
				NewSubscriber = TempSubscriber;
			}
			else
			{
				NewSubscriber = NewSubscriber->Next;
			}
		}
	}
	//
	// The Maximum number of subscribers can be bounded
	//
	if(*TotalSubscribers<10)
	{
		NewSubscriber = (struct SubscriberInfo*)MALLOC(sizeof(struct SubscriberInfo));
		memset(NewSubscriber,0,sizeof(struct SubscriberInfo));
		
		
		//
		// The SID must be globally unique, so lets generate it using
		// a bunch of random hex characters
		//
		SID = (char*)MALLOC(43);
		memset(SID,0,38);
		sprintf(SID,"uuid:");
		for(SIDNumber=5;SIDNumber<=12;++SIDNumber)
		{
			rnumber = rand()%16;
			sprintf(SID+SIDNumber,"%x",rnumber);
		}
		sprintf(SID+SIDNumber,"-");
		for(SIDNumber=14;SIDNumber<=17;++SIDNumber)
		{
			rnumber = rand()%16;
			sprintf(SID+SIDNumber,"%x",rnumber);
		}
		sprintf(SID+SIDNumber,"-");
		for(SIDNumber=19;SIDNumber<=22;++SIDNumber)
		{
			rnumber = rand()%16;
			sprintf(SID+SIDNumber,"%x",rnumber);
		}
		sprintf(SID+SIDNumber,"-");
		for(SIDNumber=24;SIDNumber<=27;++SIDNumber)
		{
			rnumber = rand()%16;
			sprintf(SID+SIDNumber,"%x",rnumber);
		}
		sprintf(SID+SIDNumber,"-");
		for(SIDNumber=29;SIDNumber<=40;++SIDNumber)
		{
			rnumber = rand()%16;
			sprintf(SID+SIDNumber,"%x",rnumber);
		}
		
		p = ILibParseString(URL,0,URLLength,"://",3);
		if(p->NumResults==1)
		{
			ILibWebServer_Send_Raw(session,"HTTP/1.1 412 Precondition Failed\r\nContent-Length: 0\r\n\r\n",55,1,1);
			ILibDestructParserResults(p);
			return;
		}
		TempString = p->LastResult->data;
		TempStringLength = p->LastResult->datalength;
		ILibDestructParserResults(p);
		p = ILibParseString(TempString,0,TempStringLength,"/",1);
		p2 = ILibParseString(p->FirstResult->data,0,p->FirstResult->datalength,":",1);
		TempString2 = (char*)MALLOC(1+sizeof(char)*p2->FirstResult->datalength);
		memcpy(TempString2,p2->FirstResult->data,p2->FirstResult->datalength);
		TempString2[p2->FirstResult->datalength] = '\0';
		NewSubscriber->Address = inet_addr(TempString2);
		if(p2->NumResults==1)
		{
			NewSubscriber->Port = 80;
			path = (char*)MALLOC(1+TempStringLength - p2->FirstResult->datalength -1);
			memcpy(path,TempString + p2->FirstResult->datalength,TempStringLength - p2->FirstResult->datalength -1);
			path[TempStringLength - p2->FirstResult->datalength - 1] = '\0';
			NewSubscriber->Path = path;
			NewSubscriber->PathLength = (int)strlen(path);
		}
		else
		{
			ILibGetLong(p2->LastResult->data,p2->LastResult->datalength,&TempLong);
			NewSubscriber->Port = (unsigned short)TempLong;
			if(TempStringLength==p->FirstResult->datalength)
			{
				path = (char*)MALLOC(2);
				memcpy(path,"/",1);
				path[1] = '\0';
			}
			else
			{
				path = (char*)MALLOC(1+TempStringLength - p->FirstResult->datalength -1);
				memcpy(path,TempString + p->FirstResult->datalength,TempStringLength - p->FirstResult->datalength -1);
				path[TempStringLength - p->FirstResult->datalength -1] = '\0';
			}
			NewSubscriber->Path = path;
			NewSubscriber->PathLength = (int)strlen(path);
		}
		ILibDestructParserResults(p);
		ILibDestructParserResults(p2);
		FREE(TempString2);
		
		
		escapedURI = (char*)MALLOC(ILibHTTPEscapeLength(NewSubscriber->Path));
		escapedURILength = ILibHTTPEscape(escapedURI,NewSubscriber->Path);
		
		FREE(NewSubscriber->Path);
		NewSubscriber->Path = escapedURI;
		NewSubscriber->PathLength = escapedURILength;
		
		
		NewSubscriber->RefCount = 1;
		NewSubscriber->Disposing = 0;
		NewSubscriber->Previous = NULL;
		NewSubscriber->SID = SID;
		NewSubscriber->SIDLength = (int)strlen(SID);
		NewSubscriber->SEQ = 0;
		
		//
		// Determine what the subscription renewal cycle is
		//
		#if defined(WIN32) || defined(_WIN32_WCE)
		NewSubscriber->RenewByTime = (GetTickCount() / 1000) + Timeout;
		#else
		gettimeofday(&(NewSubscriber->RenewByTime),NULL);
		(NewSubscriber->RenewByTime).tv_sec += (int)Timeout;
		#endif
		NewSubscriber->Next = *HeadPtr;
		if(*HeadPtr!=NULL) {(*HeadPtr)->Previous = NewSubscriber;}
		*HeadPtr = NewSubscriber;
		++(*TotalSubscribers);
		LVL3DEBUG(printf("\r\n\r\nSubscribed [%s] %d.%d.%d.%d:%d FOR %d Duration\r\n",NewSubscriber->SID,(NewSubscriber->Address)&0xFF,(NewSubscriber->Address>>8)&0xFF,(NewSubscriber->Address>>16)&0xFF,(NewSubscriber->Address>>24)&0xFF,NewSubscriber->Port,Timeout);)
		#if defined(WIN32) || defined(_WIN32_WCE)	
		LVL3DEBUG(printf("TIMESTAMP: %d <%d>\r\n\r\n",(NewSubscriber->RenewByTime)-Timeout,NewSubscriber);)
		#else
		LVL3DEBUG(printf("TIMESTAMP: %d <%d>\r\n\r\n",(NewSubscriber->RenewByTime).tv_sec-Timeout,NewSubscriber);)
		#endif
		packet = (char*)MALLOC(134 + (int)strlen(SID) + (int)strlen(UpnpPLATFORM) + 4);
		packetlength = sprintf(packet,"HTTP/1.1 200 OK\r\nSERVER: %s, UPnP/1.0, Intel MicroStack/1.0.1922\r\nSID: %s\r\nTIMEOUT: Second-%ld\r\nContent-Length: 0\r\n\r\n",UpnpPLATFORM,SID,Timeout);
		if(strcmp(ServiceName,"ConnectionManager")==0)
		{
			UpnpGetInitialEventBody_ConnectionManager(dataObject,&packetbody,&packetbodyLength);
		}
		else if(strcmp(ServiceName,"ContentDirectory")==0)
		{
			UpnpGetInitialEventBody_ContentDirectory(dataObject,&packetbody,&packetbodyLength);
		}
		
		if (packetbody != NULL)	    {
			ILibWebServer_Send_Raw(session,packet,packetlength,0,1);
			
			UpnpSendEvent_Body(dataObject,packetbody,packetbodyLength,NewSubscriber);
			FREE(packetbody);
		} 
	}
	else
	{
		/* Too many subscribers */
		ILibWebServer_Send_Raw(session,"HTTP/1.1 412 Too Many Subscribers\r\nContent-Length: 0\r\n\r\n",56,1,1);
	}
}
void UpnpSubscribeEvents(char* path,int pathlength,char* Timeout,int TimeoutLength,char* URL,int URLLength,struct ILibWebServer_Session* session)
{
	long TimeoutVal;
	char* buffer = (char*)MALLOC(1+sizeof(char)*pathlength);
	
	ILibGetLong(Timeout,TimeoutLength,&TimeoutVal);
	memcpy(buffer,path,pathlength);
	buffer[pathlength] = '\0';
	FREE(buffer);
	if(TimeoutVal>7200) {TimeoutVal=7200;}
	
	if(pathlength==24 && memcmp(path+1,"ConnectionManager/event",23)==0)
	{
		UpnpTryToSubscribe("ConnectionManager",TimeoutVal,URL,URLLength,session);
	}
	else if(pathlength==23 && memcmp(path+1,"ContentDirectory/event",22)==0)
	{
		UpnpTryToSubscribe("ContentDirectory",TimeoutVal,URL,URLLength,session);
	}
	else
	{
		ILibWebServer_Send_Raw(session,"HTTP/1.1 412 Invalid Service Name\r\nContent-Length: 0\r\n\r\n",56,1,1);
	}
	
}
void UpnpRenewEvents(char* path,int pathlength,char *_SID,int SIDLength, char* Timeout, int TimeoutLength, struct ILibWebServer_Session *ReaderObject)
{
	struct SubscriberInfo *info = NULL;
	long TimeoutVal;
	#if !defined(WIN32) && !defined(_WIN32_WCE)
	struct timeval tv;
	#endif
	char* packet;
	int packetlength;
	char* SID = (char*)MALLOC(SIDLength+1);
	memcpy(SID,_SID,SIDLength);
	SID[SIDLength] ='\0';
	#if defined(WIN32) || defined(_WIN32_WCE)
	LVL3DEBUG(printf("\r\n\r\nTIMESTAMP: %d\r\n",GetTickCount()/1000);)
	#else
	LVL3DEBUG(gettimeofday(&tv,NULL);)
	LVL3DEBUG(printf("\r\n\r\nTIMESTAMP: %d\r\n",tv.tv_sec);)
	#endif
	LVL3DEBUG(printf("SUBSCRIBER [%s] attempting to Renew Events for %s Duration [",SID,Timeout);)
	
	if(pathlength==24 && memcmp(path+1,"ConnectionManager/event",23)==0)
	{
		info = ((struct UpnpDataObject*)ReaderObject->User)->HeadSubscriberPtr_ConnectionManager;
	}
	else if(pathlength==23 && memcmp(path+1,"ContentDirectory/event",22)==0)
	{
		info = ((struct UpnpDataObject*)ReaderObject->User)->HeadSubscriberPtr_ContentDirectory;
	}
	
	
	//
	// Find this SID in the subscriber list, and recalculate
	// the expiration timeout
	//
	while(info!=NULL && strcmp(info->SID,SID)!=0)
	{
		info = info->Next;
	}
	if(info!=NULL)
	{
		ILibGetLong(Timeout,TimeoutLength,&TimeoutVal);
		#if defined(WIN32) || defined(_WIN32_WCE)
		info->RenewByTime = TimeoutVal + (GetTickCount() / 1000);
		#else
		gettimeofday(&tv,NULL);
		(info->RenewByTime).tv_sec = tv.tv_sec + TimeoutVal;
		#endif
		packet = (char*)MALLOC(134 + (int)strlen(SID) + 4);
		packetlength = sprintf(packet,"HTTP/1.1 200 OK\r\nSERVER: %s, UPnP/1.0, Intel MicroStack/1.0.1922\r\nSID: %s\r\nTIMEOUT: Second-%ld\r\nContent-Length: 0\r\n\r\n",UpnpPLATFORM,SID,TimeoutVal);
		ILibWebServer_Send_Raw(ReaderObject,packet,packetlength,0,1);
		LVL3DEBUG(printf("OK] {%d} <%d>\r\n\r\n",TimeoutVal,info);)
	}
	else
	{
		LVL3DEBUG(printf("FAILED]\r\n\r\n");)
		ILibWebServer_Send_Raw(ReaderObject,"HTTP/1.1 412 Precondition Failed\r\nContent-Length: 0\r\n\r\n",55,1,1);
	}
	FREE(SID);
}
void UpnpProcessSUBSCRIBE(struct packetheader *header, struct ILibWebServer_Session *session)
{
	char* SID = NULL;
	int SIDLength = 0;
	char* Timeout = NULL;
	int TimeoutLength = 0;
	char* URL = NULL;
	int URLLength = 0;
	struct parser_result *p;
	
	struct packetheader_field_node *f;
	
	//
	// Iterate through all the HTTP Headers
	//
	f = header->FirstField;
	while(f!=NULL)
	{
		if(f->FieldLength==3 && strncasecmp(f->Field,"SID",3)==0)
		{
			//
			// Get the Subscription ID
			//
			SID = f->FieldData;
			SIDLength = f->FieldDataLength;
		}
		else if(f->FieldLength==8 && strncasecmp(f->Field,"Callback",8)==0)
		{
			//
			// Get the Callback URL
			//
			URL = f->FieldData;
			URLLength = f->FieldDataLength;
		}
		else if(f->FieldLength==7 && strncasecmp(f->Field,"Timeout",7)==0)
		{
			//
			// Get the requested timeout value
			//
			Timeout = f->FieldData;
			TimeoutLength = f->FieldDataLength;
		}
		
		f = f->NextField;
	}
	if(Timeout==NULL)
	{
		//
		// It a timeout wasn't specified, force it to a specific value
		//
		//Timeout = "7200";
		//TimeoutLength = 4;
		Timeout = "300";
		TimeoutLength = 3;
	}
	else
	{
		p = ILibParseString(Timeout,0,TimeoutLength,"-",1);
		if(p->NumResults==2)
		{
			Timeout = p->LastResult->data;
			TimeoutLength = p->LastResult->datalength;
			if(TimeoutLength==8 && strncasecmp(Timeout,"INFINITE",8)==0)
			{
				//
				// Infinite timeouts will cause problems, so we don't allow it
				//
				//Timeout = "7200";
				//TimeoutLength = 4;
				Timeout = "300";
				TimeoutLength = 3;
			}
		}
		else
		{
			//Timeout = "7200";
			//TimeoutLength = 4;
			Timeout = "300";
			TimeoutLength = 3;
		}
		ILibDestructParserResults(p);
	}
	if(SID==NULL)
	{
		//
		// If not SID was specified, this is a subscription request
		//
		
		/* Subscribe */
		UpnpSubscribeEvents(header->DirectiveObj,header->DirectiveObjLength,Timeout,TimeoutLength,URL,URLLength,session);
	}
	else
	{
		//
		// If a SID was specified, it is a renewal request for an existing subscription
		//
		
		/* Renew */
		UpnpRenewEvents(header->DirectiveObj,header->DirectiveObjLength,SID,SIDLength,Timeout,TimeoutLength,session);
	}
}
void UpnpProcessHTTPPacket(struct ILibWebServer_Session *session, struct packetheader* header, char *bodyBuffer, int offset, int bodyBufferLength)

{
	struct UpnpDataObject *dataObject = (struct UpnpDataObject*)session->User;
	#if defined(WIN32) || defined(_WIN32_WCE)
	char *responseHeader = "\r\nCONTENT-TYPE:  text/xml; charset=\"utf-8\"\r\nServer: WINDOWS, UPnP/1.0, Intel MicroStack/1.0.1922";
	#else
	char *responseHeader = "\r\nCONTENT-TYPE:  text/xml; charset=\"utf-8\"\r\nServer: POSIX, UPnP/1.0, Intel MicroStack/1.0.1922";
	#endif
	char *errorTemplate = "HTTP/1.1 %d %s\r\nServer: %s, UPnP/1.0, Intel MicroStack/1.0.1922\r\nContent-Length: 0\r\n\r\n";
	char *errorPacket;
	int errorPacketLength;
	char *buffer;
	
	LVL3DEBUG(errorPacketLength=ILibGetRawPacket(header,&errorPacket);)
	LVL3DEBUG(printf("%s\r\n",errorPacket);)
	LVL3DEBUG(FREE(errorPacket);)			
	
	
	if(header->DirectiveLength==4 && memcmp(header->Directive,"HEAD",4)==0)
	{
		if(header->DirectiveObjLength==1 && memcmp(header->DirectiveObj,"/",1)==0)
		{
			//
			// A HEAD request for the device description document.
			// We stream the document back, so we don't return content length or anything
			// because the actual response won't have it either
			//
			ILibWebServer_StreamHeader_Raw(session,200,"OK",responseHeader,1);
			ILibWebServer_StreamBody(session,NULL,0,ILibAsyncSocket_MemoryOwnership_STATIC,1);
		}
		else if(header->DirectiveObjLength==27 && memcmp((header->DirectiveObj)+1,"ConnectionManager/scpd.xml",26)==0)
		{
			ILibWebServer_StreamHeader_Raw(session,200,"OK",responseHeader,1);
			ILibWebServer_StreamBody(session,NULL,0,ILibAsyncSocket_MemoryOwnership_STATIC,1);
		}
		else if(header->DirectiveObjLength==26 && memcmp((header->DirectiveObj)+1,"ContentDirectory/scpd.xml",25)==0)
		{
			ILibWebServer_StreamHeader_Raw(session,200,"OK",responseHeader,1);
			ILibWebServer_StreamBody(session,NULL,0,ILibAsyncSocket_MemoryOwnership_STATIC,1);
		}
		
		else
		{
			//
			// A HEAD request for something we don't have
			//
			errorPacket = (char*)MALLOC(128);
			errorPacketLength = sprintf(errorPacket,errorTemplate,404,"File Not Found",UpnpPLATFORM);
			ILibWebServer_Send_Raw(session,errorPacket,errorPacketLength,0,1);
		}
	}
	else if(header->DirectiveLength==3 && memcmp(header->Directive,"GET",3)==0)
	{
		if(header->DirectiveObjLength==1 && memcmp(header->DirectiveObj,"/",1)==0)
		{
			//
			// A GET Request for the device description document, so lets stream
			// it back to the client
			//
			ILibWebServer_StreamHeader_Raw(session,200,"OK",responseHeader,1);
			ILibWebServer_StreamBody(session,dataObject->DeviceDescription,dataObject->DeviceDescriptionLength,1,1);
		}
		else if(header->DirectiveObjLength==27 && memcmp((header->DirectiveObj)+1,"ConnectionManager/scpd.xml",26)==0)
		{
			buffer = ILibDecompressString((char*)UpnpConnectionManagerDescription,UpnpConnectionManagerDescriptionLength,UpnpConnectionManagerDescriptionLengthUX);
			ILibWebServer_Send_Raw(session,buffer,UpnpConnectionManagerDescriptionLengthUX,0,1);
		}
		else if(header->DirectiveObjLength==26 && memcmp((header->DirectiveObj)+1,"ContentDirectory/scpd.xml",25)==0)
		{
			buffer = ILibDecompressString((char*)UpnpContentDirectoryDescription,UpnpContentDirectoryDescriptionLength,UpnpContentDirectoryDescriptionLengthUX);
			ILibWebServer_Send_Raw(session,buffer,UpnpContentDirectoryDescriptionLengthUX,0,1);
		}
		
		else
		{
			//
			// A GET Request for something we don't have
			//
			errorPacket = (char*)MALLOC(128);
			errorPacketLength = sprintf(errorPacket,errorTemplate,404,"File Not Found",UpnpPLATFORM);
			ILibWebServer_Send_Raw(session,errorPacket,errorPacketLength,0,1);
		}
	}
	else if(header->DirectiveLength==4 && memcmp(header->Directive,"POST",4)==0)
	{
		//
		// Defer Control to the POST Handler
		//
		if(UpnpProcessPOST(session,header,bodyBuffer,offset,bodyBufferLength)!=0)
		{
			//
			// A POST for an action that doesn't exist
			//
			UpnpResponse_Error(session,401,"Invalid Action");
		}
	}
	else if(header->DirectiveLength==9 && memcmp(header->Directive,"SUBSCRIBE",9)==0)
	{
		//
		// Subscription Handler
		//
		UpnpProcessSUBSCRIBE(header,session);
	}
	else if(header->DirectiveLength==11 && memcmp(header->Directive,"UNSUBSCRIBE",11)==0)
	{
		//
		// UnSubscribe Handler
		//
		UpnpProcessUNSUBSCRIBE(header,session);
	}
	else
	{
		//
		// The client tried something we didn't expect/support
		//
		errorPacket = (char*)MALLOC(128);
		errorPacketLength = sprintf(errorPacket,errorTemplate,400,"Bad Request",UpnpPLATFORM);
		ILibWebServer_Send_Raw(session,errorPacket,errorPacketLength,1,1);
	}
}
void UpnpFragmentedSendNotify_Destroy(void *data);
void UpnpMasterPreSelect(void* object,fd_set *socketset, fd_set *writeset, fd_set *errorset, int* blocktime)
{
	int i;
	struct UpnpDataObject *UpnpObject = (struct UpnpDataObject*)object;
	
	int ra = 1;
	struct sockaddr_in addr;
	struct ip_mreq mreq;
	unsigned char TTL = 4;
	struct UpnpFragmentNotifyStruct *f;
	int timeout;
	
	if(UpnpObject->InitialNotify==0)
	{
		//
		// The initial "HELLO" packets were not sent yet, so lets send them
		//
		UpnpObject->InitialNotify = -1;
		//
		// In case we were interrupted, we need to flush out the caches of
		// all the control points by sending a "byebye" first, to insure
		// control points don't ignore our "hello" packets thinking they are just
		// periodic re-advertisements.
		//
		UpnpSendByeBye(UpnpObject);
		
		//
		// PacketNumber 0 is the controller, for the rest of the packets. Send
		// one of these to send out an advertisement "group"
		//
		f = (struct UpnpFragmentNotifyStruct*)MALLOC(sizeof(struct UpnpFragmentNotifyStruct));
		f->packetNumber=0;
		f->upnp = UpnpObject;
		//
		// We need to inject some delay in these packets to space them out,
		// otherwise we could overflow the inbound buffer of the recipient, causing them
		// to lose packets. And UPnP/1.0 control points are not as robust as UPnP/1.1 control points,
		// so they need all the help they can get ;)
		//
		timeout = (int)(0 + ((unsigned short)rand() % (500)));
		do
		{
			f->upnp->InitialNotify = rand();
		}while(f->upnp->InitialNotify==0);
		//
		// Register for the timed callback, to actually send the packet
		//
		ILibLifeTime_AddEx(f->upnp->WebServerTimer,f,timeout,&UpnpFragmentedSendNotify,&UpnpFragmentedSendNotify_Destroy);
		
	}
	if(UpnpObject->UpdateFlag!=0)
	{
		//
		// Somebody told us that we should recheck our IP Address table,
		// as one of them may have changed
		//
		UpnpObject->UpdateFlag = 0;
		
		/* Clear Sockets */
		
		
		//
		// Iterate through all the currently bound IP addresses
		// and release the sockets
		//
		for(i=0;i<UpnpObject->AddressListLength;++i)
		{
			#if defined(WIN32) || defined(_WIN32_WCE)
			closesocket(UpnpObject->NOTIFY_SEND_socks[i]);
			#else
			close(UpnpObject->NOTIFY_SEND_socks[i]);
			#endif
		}
		FREE(UpnpObject->NOTIFY_SEND_socks);

#ifdef USE_ALTERNATIVE_UPNP_PORT	
		for(i=0;i<UpnpObject->AddressListLength;++i)
		{
			#if defined(WIN32) || defined(_WIN32_WCE)
			closesocket(UpnpObject->NOTIFY_SEND_socks2[i]);
			#else
			close(UpnpObject->NOTIFY_SEND_socks2[i]);
			#endif
		}
		FREE(UpnpObject->NOTIFY_SEND_socks2);
#endif

		/* Set up socket */
		FREE(UpnpObject->AddressList);
		//
		// Fetch a current list of ip addresses
		//
		UpnpObject->AddressListLength = ILibGetLocalIPAddressList(&(UpnpObject->AddressList));
		//phillo,2007/11/7
		// get full interface information
		//
		UpnpObject->InterfaceListNum = ILibGetLocalIPInteraceList(&(UpnpObject->InterfaceList));
		if( (UpnpObject->AddressListLength != UpnpObject->InterfaceListNum) && (UpnpObject->AddressListLength != 0) )
		{
			if( UpnpObject->AddressList != 0 )
				free(UpnpObject->AddressList);

			UpnpObject->AddressListLength = ILibGetLocalIPAddressList(&(UpnpObject->AddressList));
		}


		#if defined(WIN32) || defined(_WIN32_WCE)	
		UpnpObject->NOTIFY_SEND_socks = (SOCKET*)MALLOC(sizeof(int)*(UpnpObject->AddressListLength));
		#else
		UpnpObject->NOTIFY_SEND_socks = (int*)MALLOC(sizeof(int)*(UpnpObject->AddressListLength));
		#endif

#ifdef USE_ALTERNATIVE_UPNP_PORT
		#if defined(WIN32) || defined(_WIN32_WCE)	
		UpnpObject->NOTIFY_SEND_socks2 = (SOCKET*)MALLOC(sizeof(int)*(UpnpObject->AddressListLength));
		#else
		UpnpObject->NOTIFY_SEND_socks2 = (int*)MALLOC(sizeof(int)*(UpnpObject->AddressListLength));
		#endif
#endif
		//
		// Now that we have a new list of IP addresses, re-initialise everything
		//
		for(i=0;i<UpnpObject->AddressListLength;++i)
		{
			UpnpObject->NOTIFY_SEND_socks[i] = socket(AF_INET, SOCK_DGRAM, 0);
			memset((char *)&(addr), 0, sizeof(addr));
			addr.sin_family = AF_INET;
			addr.sin_addr.s_addr = UpnpObject->AddressList[i];
			addr.sin_port = (unsigned short)htons(UPNP_PORT);
			if (setsockopt(UpnpObject->NOTIFY_SEND_socks[i], SOL_SOCKET, SO_REUSEADDR,(char*)&ra, sizeof(ra)) == 0)
			{
				if (setsockopt(UpnpObject->NOTIFY_SEND_socks[i], IPPROTO_IP, IP_MULTICAST_TTL,(char*)&TTL, sizeof(TTL)) < 0)
				{
					// Ignore the case if setting the Multicast-TTL fails
				}
				if (bind(UpnpObject->NOTIFY_SEND_socks[i], (struct sockaddr *) &(addr), sizeof(addr)) == 0)
				{
					mreq.imr_multiaddr.s_addr = inet_addr(UPNP_GROUP);
					mreq.imr_interface.s_addr = UpnpObject->AddressList[i];
					if (setsockopt(UpnpObject->NOTIFY_RECEIVE_sock, IPPROTO_IP, IP_ADD_MEMBERSHIP,(char*)&mreq, sizeof(mreq)) < 0)
					{
						// Does not matter if it fails, just ignore
					}
				}
			}
		}

#ifdef USE_ALTERNATIVE_UPNP_PORT	
		for(i=0;i<UpnpObject->AddressListLength;++i)
		{
			UpnpObject->NOTIFY_SEND_socks2[i] = socket(AF_INET, SOCK_DGRAM, 0);
			memset((char *)&(addr), 0, sizeof(addr));
			addr.sin_family = AF_INET;
			addr.sin_addr.s_addr = UpnpObject->AddressList[i];
			addr.sin_port = (unsigned short)htons(ALTERNATIVE_UPNP_PORT);
			if (setsockopt(UpnpObject->NOTIFY_SEND_socks2[i], SOL_SOCKET, SO_REUSEADDR,(char*)&ra, sizeof(ra)) == 0)
			{
				if (setsockopt(UpnpObject->NOTIFY_SEND_socks2[i], IPPROTO_IP, IP_MULTICAST_TTL,(char*)&TTL, sizeof(TTL)) < 0)
				{
					// Ignore the case if setting the Multicast-TTL fails
				}
				if (bind(UpnpObject->NOTIFY_SEND_socks2[i], (struct sockaddr *) &(addr), sizeof(addr)) == 0)
				{
					mreq.imr_multiaddr.s_addr = inet_addr(UPNP_GROUP);
					mreq.imr_interface.s_addr = UpnpObject->AddressList[i];
					if (setsockopt(UpnpObject->NOTIFY_RECEIVE_sock2, IPPROTO_IP, IP_ADD_MEMBERSHIP,(char*)&mreq, sizeof(mreq)) < 0)
					{
						// Does not matter if it fails, just ignore
					}
				}
			}
		}
#endif
		//
		// Iterate through all the packet types, and re-broadcast
		//
		for(i=1;i<=5;++i)
		{
			f = (struct UpnpFragmentNotifyStruct*)MALLOC(sizeof(struct UpnpFragmentNotifyStruct));
			f->packetNumber=i;
			f->upnp = UpnpObject;
			//
			// Inject some random delay, to spread these packets out, to help prevent
			// the inbound buffer of the recipient from overflowing, causing dropped packets.
			//
			timeout = (int)(0 + ((unsigned short)rand() % (500)));
			ILibLifeTime_AddEx(f->upnp->WebServerTimer,f,timeout,&UpnpFragmentedSendNotify,&UpnpFragmentedSendNotify_Destroy);
		}
	}
	FD_SET(UpnpObject->NOTIFY_RECEIVE_sock,socketset);
#ifdef USE_ALTERNATIVE_UPNP_PORT	
	FD_SET(UpnpObject->NOTIFY_RECEIVE_sock2,socketset);
#endif
}

void UpnpMasterPostSelect(void* object,int slct, fd_set *socketset, fd_set *writeset, fd_set *errorset)
{
	#if defined(WIN32) || defined(_WIN32_WCE)
	unsigned long flags=0;
	#endif
	int cnt = 0;
	
	struct packetheader *packet;
	struct UpnpDataObject *UpnpObject = (struct UpnpDataObject*)object;
	
	if(slct>0)
	{
		
		
		//
		// Check to see if we got any Multicast SEARCH requests
		//
		if(FD_ISSET(UpnpObject->NOTIFY_RECEIVE_sock,socketset)!=0)
		{	
			cnt = recvfrom(UpnpObject->NOTIFY_RECEIVE_sock, UpnpObject->message, sizeof(UpnpObject->message), 0,
			(struct sockaddr *) &(UpnpObject->addr), &(UpnpObject->addrlen));
			if (cnt < 0)
			{
				printf("recvfrom");
				//pslin modify 0524
				//FILE * logfile = NULL;
#ifdef __DEBUG_LOG__
					logfile = fopen("C:\\honeywld.txt","a+");
					fprintf(logfile,"UpnpMicroStack.c(2302)\n");
					fclose(logfile);
#endif
				return;
				//exit(1);
			}
			else if (cnt == 0)
			{
				/* Socket Closed? */
			}
			packet = ILibParsePacketHeader(UpnpObject->message,0,cnt);
			if(packet!=NULL)
			{
				packet->Source = (struct sockaddr_in*)&(UpnpObject->addr);
				packet->ReceivingAddress = 0;
				if(packet->StatusCode==-1 && memcmp(packet->Directive,"M-SEARCH",8)==0)
				{
					//
					// Process the search request with our Multicast M-SEARCH Handler
					//
					UpnpProcessMSEARCH(UpnpObject, packet);
				}
				ILibDestructPacket(packet);
			}
		}

#ifdef USE_ALTERNATIVE_UPNP_PORT
		if(FD_ISSET(UpnpObject->NOTIFY_RECEIVE_sock2,socketset)!=0)
		{	
			cnt = recvfrom(UpnpObject->NOTIFY_RECEIVE_sock2, UpnpObject->message, sizeof(UpnpObject->message), 0,
			(struct sockaddr *) &(UpnpObject->addr), &(UpnpObject->addrlen));
			if (cnt < 0)
			{
				printf("recvfrom");
				//pslin modify 0524
				//FILE * logfile = NULL;
#ifdef __DEBUG_LOG__
					logfile = fopen("C:\\honeywld.txt","a+");
					fprintf(logfile,"UpnpMicroStack.c(2302)\n");
					fclose(logfile);
#endif
				return;
				//exit(1);
			}
			else if (cnt == 0)
			{
				/* Socket Closed? */
			}
			packet = ILibParsePacketHeader(UpnpObject->message,0,cnt);
			if(packet!=NULL)
			{
				packet->Source = (struct sockaddr_in*)&(UpnpObject->addr);
				packet->ReceivingAddress = 0;
				if(packet->StatusCode==-1 && memcmp(packet->Directive,"M-SEARCH",8)==0)
				{
					//
					// Process the search request with our Multicast M-SEARCH Handler
					//
					UpnpProcessMSEARCH(UpnpObject, packet);
				}
				ILibDestructPacket(packet);
			}
		}
#endif
	}
}
void UpnpFragmentedSendNotify_Destroy(void *data)
{
	FREE(data);
}
void UpnpFragmentedSendNotify(void *data)
{
	struct UpnpFragmentNotifyStruct *FNS = (struct UpnpFragmentNotifyStruct*)data;
	int timeout,timeout2;
	int subsetRange;
	int packetlength;
	char* packet = (char*)MALLOC(5000);
	int i,i2;
	struct sockaddr_in addr;
	int addrlen;
	struct in_addr interface_addr;
	struct UpnpFragmentNotifyStruct *f;
	
	memset((char *)&addr, 0, sizeof(addr));
	addr.sin_family = AF_INET;
	addr.sin_addr.s_addr = inet_addr(UPNP_GROUP);
	addr.sin_port = (unsigned short)htons(UPNP_PORT);
	addrlen = sizeof(addr);
	
	memset((char *)&interface_addr, 0, sizeof(interface_addr));
	
	if(FNS->packetNumber==0)
	{				
		subsetRange = 5000/5; // Make 
		/*
		f = (struct UpnpFragmentNotifyStruct*)MALLOC(sizeof(struct UpnpFragmentNotifyStruct));
		f->packetNumber=1;
		f->upnp = FNS->upnp;
		timeout2 = (rand() % subsetRange);
		ILibLifeTime_AddEx(FNS->upnp->WebServerTimer,f,timeout2,&UpnpFragmentedSendNotify,&UpnpFragmentedSendNotify_Destroy);
		*/
		// Send the first "group"
		for(i2=0;i2<5;++i2)
		{
			f = (struct UpnpFragmentNotifyStruct*)MALLOC(sizeof(struct UpnpFragmentNotifyStruct));
			f->packetNumber=i2+1;
			f->upnp = FNS->upnp;
			timeout2 = (rand() % subsetRange);
			ILibLifeTime_AddEx(FNS->upnp->WebServerTimer,f,timeout2,&UpnpFragmentedSendNotify,&UpnpFragmentedSendNotify_Destroy);
		}
		
		// Now Repeat this "group" after 7 seconds, to insure there is no overlap
		for(i2=0;i2<5;++i2)
		{
			f = (struct UpnpFragmentNotifyStruct*)MALLOC(sizeof(struct UpnpFragmentNotifyStruct));
			f->packetNumber=i2+1;
			f->upnp = FNS->upnp;
			timeout2 = 7000 + (rand() % subsetRange);
			ILibLifeTime_AddEx(FNS->upnp->WebServerTimer,f,timeout2,&UpnpFragmentedSendNotify,&UpnpFragmentedSendNotify_Destroy);
		}
		
		// Calculate the next transmission window and spread the packets
		timeout = (int)((FNS->upnp->NotifyCycleTime/4) + ((unsigned short)rand() % (FNS->upnp->NotifyCycleTime/2 - FNS->upnp->NotifyCycleTime/4)));
		ILibLifeTime_Add(FNS->upnp->WebServerTimer,FNS,timeout,&UpnpFragmentedSendNotify,&UpnpFragmentedSendNotify_Destroy);
	}
	
	for(i=0;i<FNS->upnp->AddressListLength;++i)
	{
		interface_addr.s_addr = FNS->upnp->AddressList[i];
		#if !defined(_WIN32_WCE) || (defined(_WIN32_WCE) && _WIN32_WCE>=400)
		if (setsockopt(FNS->upnp->NOTIFY_SEND_socks[i], IPPROTO_IP, IP_MULTICAST_IF,(char*)&interface_addr, sizeof(interface_addr)) == 0)
		{
			#endif
			
			switch(FNS->packetNumber)
			{
				case 1:
				UpnpBuildSsdpNotifyPacket(packet,&packetlength,FNS->upnp->AddressList[i],(unsigned short)FNS->upnp->WebSocketPortNumber,0,FNS->upnp->UDN,"::upnp:rootdevice","upnp:rootdevice","",FNS->upnp->NotifyCycleTime);
				sendto(FNS->upnp->NOTIFY_SEND_socks[i], packet, packetlength, 0, (struct sockaddr *) &addr, addrlen);
				break;
				case 2:
				UpnpBuildSsdpNotifyPacket(packet,&packetlength,FNS->upnp->AddressList[i],(unsigned short)FNS->upnp->WebSocketPortNumber,0,FNS->upnp->UDN,"","uuid:",FNS->upnp->UDN,FNS->upnp->NotifyCycleTime);
				sendto(FNS->upnp->NOTIFY_SEND_socks[i], packet, packetlength, 0, (struct sockaddr *) &addr, addrlen);
				break;
				case 3:
				UpnpBuildSsdpNotifyPacket(packet,&packetlength,FNS->upnp->AddressList[i],(unsigned short)FNS->upnp->WebSocketPortNumber,0,FNS->upnp->UDN,"::urn:schemas-upnp-org:device:MediaServer:1","urn:schemas-upnp-org:device:MediaServer:1","",FNS->upnp->NotifyCycleTime);
				sendto(FNS->upnp->NOTIFY_SEND_socks[i], packet, packetlength, 0, (struct sockaddr *) &addr, addrlen);
				break;
				case 4:
				UpnpBuildSsdpNotifyPacket(packet,&packetlength,FNS->upnp->AddressList[i],(unsigned short)FNS->upnp->WebSocketPortNumber,0,FNS->upnp->UDN,"::urn:schemas-upnp-org:service:ConnectionManager:1","urn:schemas-upnp-org:service:ConnectionManager:1","",FNS->upnp->NotifyCycleTime);
				sendto(FNS->upnp->NOTIFY_SEND_socks[i], packet, packetlength, 0, (struct sockaddr *) &addr, addrlen);
				break;
				case 5:
				UpnpBuildSsdpNotifyPacket(packet,&packetlength,FNS->upnp->AddressList[i],(unsigned short)FNS->upnp->WebSocketPortNumber,0,FNS->upnp->UDN,"::urn:schemas-upnp-org:service:ContentDirectory:1","urn:schemas-upnp-org:service:ContentDirectory:1","",FNS->upnp->NotifyCycleTime);
				sendto(FNS->upnp->NOTIFY_SEND_socks[i], packet, packetlength, 0, (struct sockaddr *) &addr, addrlen);
				break;
				
			}
			
			#if !defined(_WIN32_WCE) || (defined(_WIN32_WCE) && _WIN32_WCE>=400)
		}
		#endif
	}
#ifdef USE_ALTERNATIVE_UPNP_PORT
	addr.sin_port = (unsigned short)htons(ALTERNATIVE_UPNP_PORT);
	for(i=0;i<FNS->upnp->AddressListLength;++i)
	{
		interface_addr.s_addr = FNS->upnp->AddressList[i];
		#if !defined(_WIN32_WCE) || (defined(_WIN32_WCE) && _WIN32_WCE>=400)
		if (setsockopt(FNS->upnp->NOTIFY_SEND_socks2[i], IPPROTO_IP, IP_MULTICAST_IF,(char*)&interface_addr, sizeof(interface_addr)) == 0)
		{
			#endif
			
			switch(FNS->packetNumber)
			{
				case 1:
				UpnpBuildSsdpNotifyPacket2(packet,&packetlength,FNS->upnp->AddressList[i],(unsigned short)FNS->upnp->WebSocketPortNumber,0,FNS->upnp->UDN,"::upnp:rootdevice","upnp:rootdevice","",FNS->upnp->NotifyCycleTime);
				sendto(FNS->upnp->NOTIFY_SEND_socks2[i], packet, packetlength, 0, (struct sockaddr *) &addr, addrlen);
				break;
				case 2:
				UpnpBuildSsdpNotifyPacket2(packet,&packetlength,FNS->upnp->AddressList[i],(unsigned short)FNS->upnp->WebSocketPortNumber,0,FNS->upnp->UDN,"","uuid:",FNS->upnp->UDN,FNS->upnp->NotifyCycleTime);
				sendto(FNS->upnp->NOTIFY_SEND_socks2[i], packet, packetlength, 0, (struct sockaddr *) &addr, addrlen);
				break;
				case 3:
				UpnpBuildSsdpNotifyPacket2(packet,&packetlength,FNS->upnp->AddressList[i],(unsigned short)FNS->upnp->WebSocketPortNumber,0,FNS->upnp->UDN,"::urn:schemas-upnp-org:device:MediaServer:1","urn:schemas-upnp-org:device:MediaServer:1","",FNS->upnp->NotifyCycleTime);
				sendto(FNS->upnp->NOTIFY_SEND_socks2[i], packet, packetlength, 0, (struct sockaddr *) &addr, addrlen);
				break;
				case 4:
				UpnpBuildSsdpNotifyPacket2(packet,&packetlength,FNS->upnp->AddressList[i],(unsigned short)FNS->upnp->WebSocketPortNumber,0,FNS->upnp->UDN,"::urn:schemas-upnp-org:service:ConnectionManager:1","urn:schemas-upnp-org:service:ConnectionManager:1","",FNS->upnp->NotifyCycleTime);
				sendto(FNS->upnp->NOTIFY_SEND_socks2[i], packet, packetlength, 0, (struct sockaddr *) &addr, addrlen);
				break;
				case 5:
				UpnpBuildSsdpNotifyPacket2(packet,&packetlength,FNS->upnp->AddressList[i],(unsigned short)FNS->upnp->WebSocketPortNumber,0,FNS->upnp->UDN,"::urn:schemas-upnp-org:service:ContentDirectory:1","urn:schemas-upnp-org:service:ContentDirectory:1","",FNS->upnp->NotifyCycleTime);
				sendto(FNS->upnp->NOTIFY_SEND_socks2[i], packet, packetlength, 0, (struct sockaddr *) &addr, addrlen);
				break;
				
			}
			
			#if !defined(_WIN32_WCE) || (defined(_WIN32_WCE) && _WIN32_WCE>=400)
		}
		#endif
	}
#endif

	FREE(packet);
	if(FNS->packetNumber!=0)
	{
		FREE(FNS);
	}
}
void UpnpSendNotify(const struct UpnpDataObject *upnp)
{
	int packetlength;
	char* packet = (char*)MALLOC(5000);
	int i,i2;
	struct sockaddr_in addr;
	int addrlen;
	struct in_addr interface_addr;
	
	memset((char *)&addr, 0, sizeof(addr));
	addr.sin_family = AF_INET;
	addr.sin_addr.s_addr = inet_addr(UPNP_GROUP);
	addr.sin_port = (unsigned short)htons(UPNP_PORT);
	addrlen = sizeof(addr);
	
	memset((char *)&interface_addr, 0, sizeof(interface_addr));
	
	for(i=0;i<upnp->AddressListLength;++i)
	{
		interface_addr.s_addr = upnp->AddressList[i];
		#if !defined(_WIN32_WCE) || (defined(_WIN32_WCE) && _WIN32_WCE>=400)
		if (setsockopt(upnp->NOTIFY_SEND_socks[i], IPPROTO_IP, IP_MULTICAST_IF,(char*)&interface_addr, sizeof(interface_addr)) == 0)
		{
			#endif
			for (i2=0;i2<2;i2++)
			{
				UpnpBuildSsdpNotifyPacket(packet,&packetlength,upnp->AddressList[i],(unsigned short)upnp->WebSocketPortNumber,0,upnp->UDN,"::upnp:rootdevice","upnp:rootdevice","",upnp->NotifyCycleTime);
				if (sendto(upnp->NOTIFY_SEND_socks[i], packet, packetlength, 0, (struct sockaddr *) &addr, addrlen) < 0) {
					//FILE * logfile = NULL;
						logfile = fopen("C:\\honeywld.txt","a+");
					fprintf(logfile,"UpnpMicroStack.c(9999)\n");
					fclose(logfile);

					goto LERROR; 
				}//pslin modify 0524
				UpnpBuildSsdpNotifyPacket(packet,&packetlength,upnp->AddressList[i],(unsigned short)upnp->WebSocketPortNumber,0,upnp->UDN,"","uuid:",upnp->UDN,upnp->NotifyCycleTime);
				if (sendto(upnp->NOTIFY_SEND_socks[i], packet, packetlength, 0, (struct sockaddr *) &addr, addrlen) < 0) {
#ifdef __DEBUG_LOG__
						logfile = fopen("C:\\honeywld.txt","a+");
					fprintf(logfile,"UpnpMicroStack.c(9999a)\n");
					fclose(logfile);
#endif
					goto LERROR;
				}//pslin modify 0524
				UpnpBuildSsdpNotifyPacket(packet,&packetlength,upnp->AddressList[i],(unsigned short)upnp->WebSocketPortNumber,0,upnp->UDN,"::urn:schemas-upnp-org:device:MediaServer:1","urn:schemas-upnp-org:device:MediaServer:1","",upnp->NotifyCycleTime);
				if (sendto(upnp->NOTIFY_SEND_socks[i], packet, packetlength, 0, (struct sockaddr *) &addr, addrlen) < 0) {
					//FILE * logfile = NULL;
#ifdef __DEBUG_LOG__
						logfile = fopen("C:\\honeywld.txt","a+");
					fprintf(logfile,"UpnpMicroStack.c(9999b)\n");
					fclose(logfile);
#endif
					goto LERROR;}//pslin modify 0524
				UpnpBuildSsdpNotifyPacket(packet,&packetlength,upnp->AddressList[i],(unsigned short)upnp->WebSocketPortNumber,0,upnp->UDN,"::urn:schemas-upnp-org:service:ConnectionManager:1","urn:schemas-upnp-org:service:ConnectionManager:1","",upnp->NotifyCycleTime);
				if (sendto(upnp->NOTIFY_SEND_socks[i], packet, packetlength, 0, (struct sockaddr *) &addr, addrlen) < 0) {
#ifdef __DEBUG_LOG__
						logfile = fopen("C:\\honeywld.txt","a+");
					fprintf(logfile,"UpnpMicroStack.c(9999c)\n");
					fclose(logfile);
#endif
					goto LERROR;}//pslin modify 0524
				UpnpBuildSsdpNotifyPacket(packet,&packetlength,upnp->AddressList[i],(unsigned short)upnp->WebSocketPortNumber,0,upnp->UDN,"::urn:schemas-upnp-org:service:ContentDirectory:1","urn:schemas-upnp-org:service:ContentDirectory:1","",upnp->NotifyCycleTime);
				if (sendto(upnp->NOTIFY_SEND_socks[i], packet, packetlength, 0, (struct sockaddr *) &addr, addrlen) < 0) {
#ifdef __DEBUG_LOG__
						logfile = fopen("C:\\honeywld.txt","a+");
					fprintf(logfile,"UpnpMicroStack.c(9999d)\n");
					fclose(logfile);
#endif
					goto LERROR;}//pslin modify 0524
				
			}
			#if !defined(_WIN32_WCE) || (defined(_WIN32_WCE) && _WIN32_WCE>=4)
		}
		#endif
	}
#ifdef USE_ALTERNATIVE_UPNP_PORT
	addr.sin_port = (unsigned short)htons(ALTERNATIVE_UPNP_PORT);
	for(i=0;i<upnp->AddressListLength;++i)
	{
		interface_addr.s_addr = upnp->AddressList[i];
		#if !defined(_WIN32_WCE) || (defined(_WIN32_WCE) && _WIN32_WCE>=400)
		if (setsockopt(upnp->NOTIFY_SEND_socks2[i], IPPROTO_IP, IP_MULTICAST_IF,(char*)&interface_addr, sizeof(interface_addr)) == 0)
		{
			#endif
			for (i2=0;i2<2;i2++)
			{
				UpnpBuildSsdpNotifyPacket2(packet,&packetlength,upnp->AddressList[i],(unsigned short)upnp->WebSocketPortNumber,0,upnp->UDN,"::upnp:rootdevice","upnp:rootdevice","",upnp->NotifyCycleTime);
				if (sendto(upnp->NOTIFY_SEND_socks2[i], packet, packetlength, 0, (struct sockaddr *) &addr, addrlen) < 0) {
					//FILE * logfile = NULL;
						logfile = fopen("C:\\honeywld.txt","a+");
					fprintf(logfile,"UpnpMicroStack.c(9999)\n");
					fclose(logfile);

					goto LERROR; 
				}//pslin modify 0524
				UpnpBuildSsdpNotifyPacket2(packet,&packetlength,upnp->AddressList[i],(unsigned short)upnp->WebSocketPortNumber,0,upnp->UDN,"","uuid:",upnp->UDN,upnp->NotifyCycleTime);
				if (sendto(upnp->NOTIFY_SEND_socks2[i], packet, packetlength, 0, (struct sockaddr *) &addr, addrlen) < 0) {
#ifdef __DEBUG_LOG__
						logfile = fopen("C:\\honeywld.txt","a+");
					fprintf(logfile,"UpnpMicroStack.c(9999a)\n");
					fclose(logfile);
#endif
					goto LERROR;
				}//pslin modify 0524
				UpnpBuildSsdpNotifyPacket2(packet,&packetlength,upnp->AddressList[i],(unsigned short)upnp->WebSocketPortNumber,0,upnp->UDN,"::urn:schemas-upnp-org:device:MediaServer:1","urn:schemas-upnp-org:device:MediaServer:1","",upnp->NotifyCycleTime);
				if (sendto(upnp->NOTIFY_SEND_socks2[i], packet, packetlength, 0, (struct sockaddr *) &addr, addrlen) < 0) {
					//FILE * logfile = NULL;
#ifdef __DEBUG_LOG__
						logfile = fopen("C:\\honeywld.txt","a+");
					fprintf(logfile,"UpnpMicroStack.c(9999b)\n");
					fclose(logfile);
#endif
					goto LERROR;}//pslin modify 0524
				UpnpBuildSsdpNotifyPacket2(packet,&packetlength,upnp->AddressList[i],(unsigned short)upnp->WebSocketPortNumber,0,upnp->UDN,"::urn:schemas-upnp-org:service:ConnectionManager:1","urn:schemas-upnp-org:service:ConnectionManager:1","",upnp->NotifyCycleTime);
				if (sendto(upnp->NOTIFY_SEND_socks2[i], packet, packetlength, 0, (struct sockaddr *) &addr, addrlen) < 0) {
#ifdef __DEBUG_LOG__
						logfile = fopen("C:\\honeywld.txt","a+");
					fprintf(logfile,"UpnpMicroStack.c(9999c)\n");
					fclose(logfile);
#endif
					goto LERROR;}//pslin modify 0524
				UpnpBuildSsdpNotifyPacket2(packet,&packetlength,upnp->AddressList[i],(unsigned short)upnp->WebSocketPortNumber,0,upnp->UDN,"::urn:schemas-upnp-org:service:ContentDirectory:1","urn:schemas-upnp-org:service:ContentDirectory:1","",upnp->NotifyCycleTime);
				if (sendto(upnp->NOTIFY_SEND_socks2[i], packet, packetlength, 0, (struct sockaddr *) &addr, addrlen) < 0) {
#ifdef __DEBUG_LOG__
						logfile = fopen("C:\\honeywld.txt","a+");
					fprintf(logfile,"UpnpMicroStack.c(9999d)\n");
					fclose(logfile);
#endif
					goto LERROR;}//pslin modify 0524
				
			}
			#if !defined(_WIN32_WCE) || (defined(_WIN32_WCE) && _WIN32_WCE>=4)
		}
		#endif
	}
#endif

//pslin add 0524
LERROR:
	FREE(packet);
}

#define UpnpBuildSsdpByeByePacket(outpacket,outlength,USN,USNex,NT,NTex,DeviceID)\
{\
	if(DeviceID==0)\
	{\
		*outlength = sprintf(outpacket,"NOTIFY * HTTP/1.1\r\nHOST: 239.255.255.250:%d\r\nNTS: ssdp:byebye\r\nUSN: uuid:%s%s\r\nNT: %s%s\r\nContent-Length: 0\r\n\r\n",UPNP_PORT,USN,USNex,NT,NTex);\
	}\
	else\
	{\
		if(memcmp(NT,"uuid:",5)==0)\
		{\
			*outlength = sprintf(outpacket,"NOTIFY * HTTP/1.1\r\nHOST: 239.255.255.250:%d\r\nNTS: ssdp:byebye\r\nUSN: uuid:%s_%d%s\r\nNT: %s%s_%d\r\nContent-Length: 0\r\n\r\n",UPNP_PORT,USN,DeviceID,USNex,NT,NTex,DeviceID);\
		}\
		else\
		{\
			*outlength = sprintf(outpacket,"NOTIFY * HTTP/1.1\r\nHOST: 239.255.255.250:%d\r\nNTS: ssdp:byebye\r\nUSN: uuid:%s_%d%s\r\nNT: %s%s\r\nContent-Length: 0\r\n\r\n",UPNP_PORT,USN,DeviceID,USNex,NT,NTex);\
		}\
	}\
}

#define UpnpBuildSsdpByeByePacket2(outpacket,outlength,USN,USNex,NT,NTex,DeviceID)\
{\
	if(DeviceID==0)\
	{\
		*outlength = sprintf(outpacket,"NOTIFY * HTTP/1.1\r\nHOST: 239.255.255.250:%d\r\nNTS: ssdp:byebye\r\nUSN: uuid:%s%s\r\nNT: %s%s\r\nContent-Length: 0\r\n\r\n",ALTERNATIVE_UPNP_PORT,USN,USNex,NT,NTex);\
	}\
	else\
	{\
		if(memcmp(NT,"uuid:",5)==0)\
		{\
			*outlength = sprintf(outpacket,"NOTIFY * HTTP/1.1\r\nHOST: 239.255.255.250:%d\r\nNTS: ssdp:byebye\r\nUSN: uuid:%s_%d%s\r\nNT: %s%s_%d\r\nContent-Length: 0\r\n\r\n",ALTERNATIVE_UPNP_PORT,USN,DeviceID,USNex,NT,NTex,DeviceID);\
		}\
		else\
		{\
			*outlength = sprintf(outpacket,"NOTIFY * HTTP/1.1\r\nHOST: 239.255.255.250:%d\r\nNTS: ssdp:byebye\r\nUSN: uuid:%s_%d%s\r\nNT: %s%s\r\nContent-Length: 0\r\n\r\n",ALTERNATIVE_UPNP_PORT,USN,DeviceID,USNex,NT,NTex);\
		}\
	}\
}


void UpnpSendByeBye(const struct UpnpDataObject *upnp)
{
	int TempVal=0;
	int packetlength;
	char* packet = (char*)MALLOC(5000);
	int i, i2;
	struct sockaddr_in addr;
	int addrlen;
	struct in_addr interface_addr;
	
	memset((char *)&addr, 0, sizeof(addr));
	addr.sin_family = AF_INET;
	addr.sin_addr.s_addr = inet_addr(UPNP_GROUP);
	addr.sin_port = (unsigned short)htons(UPNP_PORT);
	addrlen = sizeof(addr);
	
	memset((char *)&interface_addr, 0, sizeof(interface_addr));
	
	for(i=0;i<upnp->AddressListLength;++i)
	{
		
		interface_addr.s_addr = upnp->AddressList[i];
		#if !defined(_WIN32_WCE) || (defined(_WIN32_WCE) && _WIN32_WCE>=400)
		if (setsockopt(upnp->NOTIFY_SEND_socks[i], IPPROTO_IP, IP_MULTICAST_IF,(char*)&interface_addr, sizeof(interface_addr)) == 0)
		{
			#endif		
			
			for (i2=0;i2<2;i2++)
			{
				UpnpBuildSsdpByeByePacket(packet,&packetlength,upnp->UDN,"::upnp:rootdevice","upnp:rootdevice","",0);
				if (sendto(upnp->NOTIFY_SEND_socks[i], packet, packetlength, 0, (struct sockaddr *) &addr, addrlen) < 0) continue;//exit(1)//pslin modify;
				UpnpBuildSsdpByeByePacket(packet,&packetlength,upnp->UDN,"","uuid:",upnp->UDN,0);
				if (sendto(upnp->NOTIFY_SEND_socks[i], packet, packetlength, 0, (struct sockaddr *) &addr, addrlen) < 0) continue;//exit(1)//pslin modify;
				UpnpBuildSsdpByeByePacket(packet,&packetlength,upnp->UDN,"::urn:schemas-upnp-org:device:MediaServer:1","urn:schemas-upnp-org:device:MediaServer:1","",0);
				if (sendto(upnp->NOTIFY_SEND_socks[i], packet, packetlength, 0, (struct sockaddr *) &addr, addrlen) < 0) continue;//exit(1)//pslin modify;
				UpnpBuildSsdpByeByePacket(packet,&packetlength,upnp->UDN,"::urn:schemas-upnp-org:service:ConnectionManager:1","urn:schemas-upnp-org:service:ConnectionManager:1","",0);
				if (sendto(upnp->NOTIFY_SEND_socks[i], packet, packetlength, 0, (struct sockaddr *) &addr, addrlen) < 0) continue;//exit(1)//pslin modify;
				UpnpBuildSsdpByeByePacket(packet,&packetlength,upnp->UDN,"::urn:schemas-upnp-org:service:ContentDirectory:1","urn:schemas-upnp-org:service:ContentDirectory:1","",0);
				if (sendto(upnp->NOTIFY_SEND_socks[i], packet, packetlength, 0, (struct sockaddr *) &addr, addrlen) < 0) continue;//exit(1)//pslin modify;
				
			}
			#if !defined(_WIN32_WCE) || (defined(_WIN32_WCE) && _WIN32_WCE>=400)
		}
		#endif
	}
#ifdef USE_ALTERNATIVE_UPNP_PORT
	addr.sin_port = (unsigned short)htons(ALTERNATIVE_UPNP_PORT);
	for(i=0;i<upnp->AddressListLength;++i)
	{
		
		interface_addr.s_addr = upnp->AddressList[i];
		#if !defined(_WIN32_WCE) || (defined(_WIN32_WCE) && _WIN32_WCE>=400)
		if (setsockopt(upnp->NOTIFY_SEND_socks2[i], IPPROTO_IP, IP_MULTICAST_IF,(char*)&interface_addr, sizeof(interface_addr)) == 0)
		{
			#endif		
			
			for (i2=0;i2<2;i2++)
			{
				UpnpBuildSsdpByeByePacket2(packet,&packetlength,upnp->UDN,"::upnp:rootdevice","upnp:rootdevice","",0);
				if (sendto(upnp->NOTIFY_SEND_socks2[i], packet, packetlength, 0, (struct sockaddr *) &addr, addrlen) < 0) continue;//exit(1)//pslin modify;
				UpnpBuildSsdpByeByePacket2(packet,&packetlength,upnp->UDN,"","uuid:",upnp->UDN,0);
				if (sendto(upnp->NOTIFY_SEND_socks2[i], packet, packetlength, 0, (struct sockaddr *) &addr, addrlen) < 0) continue;//exit(1)//pslin modify;
				UpnpBuildSsdpByeByePacket2(packet,&packetlength,upnp->UDN,"::urn:schemas-upnp-org:device:MediaServer:1","urn:schemas-upnp-org:device:MediaServer:1","",0);
				if (sendto(upnp->NOTIFY_SEND_socks2[i], packet, packetlength, 0, (struct sockaddr *) &addr, addrlen) < 0) continue;//exit(1)//pslin modify;
				UpnpBuildSsdpByeByePacket2(packet,&packetlength,upnp->UDN,"::urn:schemas-upnp-org:service:ConnectionManager:1","urn:schemas-upnp-org:service:ConnectionManager:1","",0);
				if (sendto(upnp->NOTIFY_SEND_socks2[i], packet, packetlength, 0, (struct sockaddr *) &addr, addrlen) < 0) continue;//exit(1)//pslin modify;
				UpnpBuildSsdpByeByePacket2(packet,&packetlength,upnp->UDN,"::urn:schemas-upnp-org:service:ContentDirectory:1","urn:schemas-upnp-org:service:ContentDirectory:1","",0);
				if (sendto(upnp->NOTIFY_SEND_socks2[i], packet, packetlength, 0, (struct sockaddr *) &addr, addrlen) < 0) continue;//exit(1)//pslin modify;
				
			}
			#if !defined(_WIN32_WCE) || (defined(_WIN32_WCE) && _WIN32_WCE>=400)
		}
		#endif
	}
#endif
	FREE(packet);
}

/*! \fn UpnpResponse_Error(const UpnpSessionToken UpnpToken, const int ErrorCode, const char* ErrorMsg)
\brief Responds to the client invocation with a SOAP Fault
\param UpnpToken UPnP token
\param ErrorCode Fault Code
\param ErrorMsg Error Detail
*/
void UpnpResponse_Error(const UpnpSessionToken UpnpToken, const int ErrorCode, const char* ErrorMsg)
{
	char* body;
	int bodylength;
	char* head;
	int headlength;
	body = (char*)MALLOC(395 + (int)strlen(ErrorMsg));
	bodylength = sprintf(body,"<s:Envelope\r\n xmlns:s=\"http://schemas.xmlsoap.org/soap/envelope/\" s:encodingStyle=\"http://schemas.xmlsoap.org/soap/encoding/\"><s:Body><s:Fault><faultcode>s:Client</faultcode><faultstring>UPnPError</faultstring><detail><UPnPError xmlns=\"urn:schemas-upnp-org:control-1-0\"><errorCode>%d</errorCode><errorDescription>%s</errorDescription></UPnPError></detail></s:Fault></s:Body></s:Envelope>",ErrorCode,ErrorMsg);
	head = (char*)MALLOC(59);
	headlength = sprintf(head,"HTTP/1.1 500 Internal\r\nContent-Length: %d\r\n\r\n",bodylength);
	ILibWebServer_Send_Raw((struct ILibWebServer_Session*)UpnpToken,head,headlength,0,0);
	ILibWebServer_Send_Raw((struct ILibWebServer_Session*)UpnpToken,body,bodylength,0,1);
}

/*! \fn UpnpGetLocalInterfaceToHost(const UpnpSessionToken UpnpToken)
\brief When a UPnP request is dispatched, this method determines which ip address actually received this request
\param UpnpToken UPnP token
\returns IP Address
*/
int UpnpGetLocalInterfaceToHost(const UpnpSessionToken UpnpToken)
{
	return(ILibWebServer_GetLocalInterface((struct ILibWebServer_Session*)UpnpToken));
}

void UpnpResponseGeneric(const UpnpMicroStackToken UpnpToken,const char* ServiceURI,const char* MethodName,const char* Params)
{
	char* packet;
	int packetlength;
	struct ILibWebServer_Session *session = (struct ILibWebServer_Session*)UpnpToken;
	int RVAL=0;
	
	packet = (char*)MALLOC(239+strlen(ServiceURI)+strlen(Params)+(strlen(MethodName)*2));
	packetlength = sprintf(packet,"<?xml version=\"1.0\" encoding=\"utf-8\"?>\r\n<s:Envelope s:encodingStyle=\"http://schemas.xmlsoap.org/soap/encoding/\" xmlns:s=\"http://schemas.xmlsoap.org/soap/envelope/\"><s:Body><u:%sResponse xmlns:u=\"%s\">%s</u:%sResponse></s:Body></s:Envelope>",MethodName,ServiceURI,Params,MethodName);
	LVL3DEBUG(printf("SendBody: %s\r\n",packet);)
	#if defined(WIN32) || defined(_WIN32_WCE)
	RVAL=ILibWebServer_StreamHeader_Raw(session,200,"OK","\r\nEXT:\r\nCONTENT-TYPE: text/xml; charset=\"utf-8\"\r\nSERVER: WINDOWS, UPnP/1.0, Intel MicroStack/1.0.1922",1);
	#else
	RVAL=ILibWebServer_StreamHeader_Raw(session,200,"OK","\r\nEXT:\r\nCONTENT-TYPE: text/xml; charset=\"utf-8\"\r\nSERVER: POSIX, UPnP/1.0, Intel MicroStack/1.0.1922",1);
	#endif
	if(RVAL!=ILibAsyncSocket_SEND_ON_CLOSED_SOCKET_ERROR && RVAL != ILibWebServer_SEND_RESULTED_IN_DISCONNECT)
	{
		RVAL=ILibWebServer_StreamBody(session,packet,packetlength,0,1);
	}
}

/*! \fn UpnpResponse_ConnectionManager_GetCurrentConnectionIDs(const UpnpSessionToken UpnpToken, const char* unescaped_ConnectionIDs)
\brief Response Method for ConnectionManager >> urn:schemas-upnp-org:service:ConnectionManager:1 >> GetCurrentConnectionIDs
\param UpnpToken MicroStack token
\param unescaped_ConnectionIDs Value of argument ConnectionIDs \b     Note: Automatically Escaped
*/
void UpnpResponse_ConnectionManager_GetCurrentConnectionIDs(const UpnpSessionToken UpnpToken, const char* unescaped_ConnectionIDs)
{
	char* body;
	char *ConnectionIDs = (char*)MALLOC(1+ILibXmlEscapeLength(unescaped_ConnectionIDs));
	
	ILibXmlEscape(ConnectionIDs,unescaped_ConnectionIDs);
	body = (char*)MALLOC(32+strlen(ConnectionIDs));
	sprintf(body,"<ConnectionIDs>%s</ConnectionIDs>",ConnectionIDs);
	UpnpResponseGeneric(UpnpToken,"urn:schemas-upnp-org:service:ConnectionManager:1","GetCurrentConnectionIDs",body);
	FREE(body);
	FREE(ConnectionIDs);
}

/*! \fn UpnpResponse_ConnectionManager_GetCurrentConnectionInfo(const UpnpSessionToken UpnpToken, const int RcsID, const int AVTransportID, const char* unescaped_ProtocolInfo, const char* unescaped_PeerConnectionManager, const int PeerConnectionID, const char* unescaped_Direction, const char* unescaped_Status)
\brief Response Method for ConnectionManager >> urn:schemas-upnp-org:service:ConnectionManager:1 >> GetCurrentConnectionInfo
\param UpnpToken MicroStack token
\param RcsID Value of argument RcsID
\param AVTransportID Value of argument AVTransportID
\param unescaped_ProtocolInfo Value of argument ProtocolInfo \b     Note: Automatically Escaped
\param unescaped_PeerConnectionManager Value of argument PeerConnectionManager \b     Note: Automatically Escaped
\param PeerConnectionID Value of argument PeerConnectionID
\param unescaped_Direction Value of argument Direction \b     Note: Automatically Escaped
\param unescaped_Status Value of argument Status \b     Note: Automatically Escaped
*/
void UpnpResponse_ConnectionManager_GetCurrentConnectionInfo(const UpnpSessionToken UpnpToken, const int RcsID, const int AVTransportID, const char* unescaped_ProtocolInfo, const char* unescaped_PeerConnectionManager, const int PeerConnectionID, const char* unescaped_Direction, const char* unescaped_Status)
{
	char* body;
	char *ProtocolInfo = (char*)MALLOC(1+ILibXmlEscapeLength(unescaped_ProtocolInfo));
	char *PeerConnectionManager = (char*)MALLOC(1+ILibXmlEscapeLength(unescaped_PeerConnectionManager));
	char *Direction = (char*)MALLOC(1+ILibXmlEscapeLength(unescaped_Direction));
	char *Status = (char*)MALLOC(1+ILibXmlEscapeLength(unescaped_Status));
	
	ILibXmlEscape(ProtocolInfo,unescaped_ProtocolInfo);
	ILibXmlEscape(PeerConnectionManager,unescaped_PeerConnectionManager);
	ILibXmlEscape(Direction,unescaped_Direction);
	ILibXmlEscape(Status,unescaped_Status);
	body = (char*)MALLOC(233+strlen(ProtocolInfo)+strlen(PeerConnectionManager)+strlen(Direction)+strlen(Status));
	sprintf(body,"<RcsID>%d</RcsID><AVTransportID>%d</AVTransportID><ProtocolInfo>%s</ProtocolInfo><PeerConnectionManager>%s</PeerConnectionManager><PeerConnectionID>%d</PeerConnectionID><Direction>%s</Direction><Status>%s</Status>",RcsID,AVTransportID,ProtocolInfo,PeerConnectionManager,PeerConnectionID,Direction,Status);
	UpnpResponseGeneric(UpnpToken,"urn:schemas-upnp-org:service:ConnectionManager:1","GetCurrentConnectionInfo",body);
	FREE(body);
	FREE(ProtocolInfo);
	FREE(PeerConnectionManager);
	FREE(Direction);
	FREE(Status);
}

/*! \fn UpnpResponse_ConnectionManager_GetProtocolInfo(const UpnpSessionToken UpnpToken, const char* unescaped_Source, const char* unescaped_Sink)
\brief Response Method for ConnectionManager >> urn:schemas-upnp-org:service:ConnectionManager:1 >> GetProtocolInfo
\param UpnpToken MicroStack token
\param unescaped_Source Value of argument Source \b     Note: Automatically Escaped
\param unescaped_Sink Value of argument Sink \b     Note: Automatically Escaped
*/
void UpnpResponse_ConnectionManager_GetProtocolInfo(const UpnpSessionToken UpnpToken, const char* unescaped_Source, const char* unescaped_Sink)
{
	char* body;
	char *Source = (char*)MALLOC(1+ILibXmlEscapeLength(unescaped_Source));
	char *Sink = (char*)MALLOC(1+ILibXmlEscapeLength(unescaped_Sink));
	
	ILibXmlEscape(Source,unescaped_Source);
	ILibXmlEscape(Sink,unescaped_Sink);
	body = (char*)MALLOC(31+strlen(Source)+strlen(Sink));
	sprintf(body,"<Source>%s</Source><Sink>%s</Sink>",Source,Sink);
	UpnpResponseGeneric(UpnpToken,"urn:schemas-upnp-org:service:ConnectionManager:1","GetProtocolInfo",body);
	FREE(body);
	FREE(Source);
	FREE(Sink);
}

/*! \fn UpnpResponse_ContentDirectory_Browse(const UpnpSessionToken UpnpToken, const char* Result, const unsigned int NumberReturned, const unsigned int TotalMatches, const unsigned int UpdateID)
\brief Response Method for ContentDirectory >> urn:schemas-upnp-org:service:ContentDirectory:1 >> Browse
\param UpnpToken MicroStack token
\param Result Value of argument Result \b     Note: Must be escaped
\param NumberReturned Value of argument NumberReturned
\param TotalMatches Value of argument TotalMatches
\param UpdateID Value of argument UpdateID
*/
void UpnpResponse_ContentDirectory_Browse(const UpnpSessionToken UpnpToken, const char* Result, const unsigned int NumberReturned, const unsigned int TotalMatches, const unsigned int UpdateID)
{
	char* body;
	
	body = (char*)MALLOC(134+strlen(Result));
	sprintf(body,"<Result>%s</Result><NumberReturned>%u</NumberReturned><TotalMatches>%u</TotalMatches><UpdateID>%u</UpdateID>",Result,NumberReturned,TotalMatches,UpdateID);
	UpnpResponseGeneric(UpnpToken,"urn:schemas-upnp-org:service:ContentDirectory:1","Browse",body);
	FREE(body);
}

/*! \fn UpnpResponse_ContentDirectory_GetSearchCapabilities(const UpnpSessionToken UpnpToken, const char* unescaped_SearchCaps)
\brief Response Method for ContentDirectory >> urn:schemas-upnp-org:service:ContentDirectory:1 >> GetSearchCapabilities
\param UpnpToken MicroStack token
\param unescaped_SearchCaps Value of argument SearchCaps \b     Note: Automatically Escaped
*/
void UpnpResponse_ContentDirectory_GetSearchCapabilities(const UpnpSessionToken UpnpToken, const char* unescaped_SearchCaps)
{
	char* body;
	char *SearchCaps = (char*)MALLOC(1+ILibXmlEscapeLength(unescaped_SearchCaps));
	
	ILibXmlEscape(SearchCaps,unescaped_SearchCaps);
	body = (char*)MALLOC(26+strlen(SearchCaps));
	sprintf(body,"<SearchCaps>%s</SearchCaps>",SearchCaps);
	UpnpResponseGeneric(UpnpToken,"urn:schemas-upnp-org:service:ContentDirectory:1","GetSearchCapabilities",body);
	FREE(body);
	FREE(SearchCaps);
}

/*! \fn UpnpResponse_ContentDirectory_GetSortCapabilities(const UpnpSessionToken UpnpToken, const char* unescaped_SortCaps)
\brief Response Method for ContentDirectory >> urn:schemas-upnp-org:service:ContentDirectory:1 >> GetSortCapabilities
\param UpnpToken MicroStack token
\param unescaped_SortCaps Value of argument SortCaps \b     Note: Automatically Escaped
*/
void UpnpResponse_ContentDirectory_GetSortCapabilities(const UpnpSessionToken UpnpToken, const char* unescaped_SortCaps)
{
	char* body;
	char *SortCaps = (char*)MALLOC(1+ILibXmlEscapeLength(unescaped_SortCaps));
	
	ILibXmlEscape(SortCaps,unescaped_SortCaps);
	body = (char*)MALLOC(22+strlen(SortCaps));
	sprintf(body,"<SortCaps>%s</SortCaps>",SortCaps);
	UpnpResponseGeneric(UpnpToken,"urn:schemas-upnp-org:service:ContentDirectory:1","GetSortCapabilities",body);
	FREE(body);
	FREE(SortCaps);
}

/*! \fn UpnpResponse_ContentDirectory_GetSystemUpdateID(const UpnpSessionToken UpnpToken, const unsigned int Id)
\brief Response Method for ContentDirectory >> urn:schemas-upnp-org:service:ContentDirectory:1 >> GetSystemUpdateID
\param UpnpToken MicroStack token
\param Id Value of argument Id
*/
void UpnpResponse_ContentDirectory_GetSystemUpdateID(const UpnpSessionToken UpnpToken, const unsigned int Id)
{
	char* body;
	
	body = (char*)MALLOC(21);
	sprintf(body,"<Id>%u</Id>",Id);
	UpnpResponseGeneric(UpnpToken,"urn:schemas-upnp-org:service:ContentDirectory:1","GetSystemUpdateID",body);
	FREE(body);
}

/*! \fn UpnpResponse_ContentDirectory_Search(const UpnpSessionToken UpnpToken, const char* Result, const unsigned int NumberReturned, const unsigned int TotalMatches, const unsigned int UpdateID)
\brief Response Method for ContentDirectory >> urn:schemas-upnp-org:service:ContentDirectory:1 >> Search
\param UpnpToken MicroStack token
\param Result Value of argument Result \b     Note: Must be escaped
\param NumberReturned Value of argument NumberReturned
\param TotalMatches Value of argument TotalMatches
\param UpdateID Value of argument UpdateID
*/
void UpnpResponse_ContentDirectory_Search(const UpnpSessionToken UpnpToken, const char* Result, const unsigned int NumberReturned, const unsigned int TotalMatches, const unsigned int UpdateID)
{
	char* body;
	
	body = (char*)MALLOC(134+strlen(Result));
	sprintf(body,"<Result>%s</Result><NumberReturned>%u</NumberReturned><TotalMatches>%u</TotalMatches><UpdateID>%u</UpdateID>",Result,NumberReturned,TotalMatches,UpdateID);
	UpnpResponseGeneric(UpnpToken,"urn:schemas-upnp-org:service:ContentDirectory:1","Search",body);
	FREE(body);
}



void UpnpSendEventSink(
void *WebReaderToken,
int IsInterrupt,
struct packetheader *header,
char *buffer,
int *p_BeginPointer,
int EndPointer,
int done,
void *subscriber,
void *upnp,
int *PAUSE)	
{
	if(done!=0 && ((struct SubscriberInfo*)subscriber)->Disposing==0)
	{
		sem_wait(&(((struct UpnpDataObject*)upnp)->EventLock));
		--((struct SubscriberInfo*)subscriber)->RefCount;
		if(((struct SubscriberInfo*)subscriber)->RefCount==0)
		{
			LVL3DEBUG(printf("\r\n\r\nSubscriber at [%s] %d.%d.%d.%d:%d was/did UNSUBSCRIBE while trying to send event\r\n\r\n",((struct SubscriberInfo*)subscriber)->SID,(((struct SubscriberInfo*)subscriber)->Address&0xFF),((((struct SubscriberInfo*)subscriber)->Address>>8)&0xFF),((((struct SubscriberInfo*)subscriber)->Address>>16)&0xFF),((((struct SubscriberInfo*)subscriber)->Address>>24)&0xFF),((struct SubscriberInfo*)subscriber)->Port);)
			UpnpDestructSubscriberInfo(((struct SubscriberInfo*)subscriber));
		}
		else if(header==NULL)
		{
			LVL3DEBUG(printf("\r\n\r\nCould not deliver event for [%s] %d.%d.%d.%d:%d UNSUBSCRIBING\r\n\r\n",((struct SubscriberInfo*)subscriber)->SID,(((struct SubscriberInfo*)subscriber)->Address&0xFF),((((struct SubscriberInfo*)subscriber)->Address>>8)&0xFF),((((struct SubscriberInfo*)subscriber)->Address>>16)&0xFF),((((struct SubscriberInfo*)subscriber)->Address>>24)&0xFF),((struct SubscriberInfo*)subscriber)->Port);)
			// Could not send Event, so unsubscribe the subscriber
			((struct SubscriberInfo*)subscriber)->Disposing = 1;
			UpnpExpireSubscriberInfo(upnp,subscriber);
		}
		sem_post(&(((struct UpnpDataObject*)upnp)->EventLock));
	}
}
void UpnpSendEvent_Body(void *upnptoken,char *body,int bodylength,struct SubscriberInfo *info)
{
	struct UpnpDataObject* UpnpObject = (struct UpnpDataObject*)upnptoken;
	struct sockaddr_in dest;
	int packetLength;
	char *packet;
	int ipaddr;
	
	memset(&dest,0,sizeof(dest));
	dest.sin_addr.s_addr = info->Address;
	dest.sin_port = htons(info->Port);
	dest.sin_family = AF_INET;
	ipaddr = info->Address;
	
	packet = (char*)MALLOC(info->PathLength + bodylength + 483);
	packetLength = sprintf(packet,"NOTIFY %s HTTP/1.1\r\nSERVER: %s, UPnP/1.0, Intel MicroStack/1.0.1922\r\nHOST: %d.%d.%d.%d:%d\r\nContent-Type: text/xml; charset=\"utf-8\"\r\nNT: upnp:event\r\nNTS: upnp:propchange\r\nSID: %s\r\nSEQ: %d\r\nContent-Length: %d\r\n\r\n<?xml version=\"1.0\" encoding=\"utf-8\"?><e:propertyset xmlns:e=\"urn:schemas-upnp-org:event-1-0\"><e:property><%s></e:property></e:propertyset>",info->Path,UpnpPLATFORM,(ipaddr&0xFF),((ipaddr>>8)&0xFF),((ipaddr>>16)&0xFF),((ipaddr>>24)&0xFF),info->Port,info->SID,info->SEQ,bodylength+137,body);
	++info->SEQ;
	
	++info->RefCount;
	ILibWebClient_PipelineRequestEx(UpnpObject->EventClient,&dest,packet,packetLength,0,NULL,0,0,&UpnpSendEventSink,info,upnptoken);
}
void UpnpSendEvent(void *upnptoken, char* body, const int bodylength, const char* eventname)
{
	struct SubscriberInfo *info = NULL;
	struct UpnpDataObject* UpnpObject = (struct UpnpDataObject*)upnptoken;
	struct sockaddr_in dest;
	LVL3DEBUG(struct timeval tv;)
	
	if(UpnpObject==NULL)
	{
		FREE(body);
		return;
	}
	sem_wait(&(UpnpObject->EventLock));
	if(strncmp(eventname,"ConnectionManager",17)==0)
	{
		info = UpnpObject->HeadSubscriberPtr_ConnectionManager;
	}
	if(strncmp(eventname,"ContentDirectory",16)==0)
	{
		info = UpnpObject->HeadSubscriberPtr_ContentDirectory;
	}
	
	memset(&dest,0,sizeof(dest));
	while(info!=NULL)
	{
		if(!UpnpSubscriptionExpired(info))
		{
			UpnpSendEvent_Body(upnptoken,body,bodylength,info);
		}
		else
		{
			//Remove Subscriber
			#if defined(WIN32) || defined(_WIN32_WCE)
			LVL3DEBUG(printf("\r\n\r\nTIMESTAMP: %d\r\n",GetTickCount()/1000);)
			#else
			LVL3DEBUG(gettimeofday(&tv,NULL);)
			LVL3DEBUG(printf("\r\n\r\nTIMESTAMP: %d\r\n",tv.tv_sec);)
			#endif
			LVL3DEBUG(printf("Did not renew [%s] %d.%d.%d.%d:%d UNSUBSCRIBING <%d>\r\n\r\n",((struct SubscriberInfo*)info)->SID,(((struct SubscriberInfo*)info)->Address&0xFF),((((struct SubscriberInfo*)info)->Address>>8)&0xFF),((((struct SubscriberInfo*)info)->Address>>16)&0xFF),((((struct SubscriberInfo*)info)->Address>>24)&0xFF),((struct SubscriberInfo*)info)->Port,info);)
		}
		
		info = info->Next;
	}
	
	sem_post(&(UpnpObject->EventLock));
}

/*! \fn UpnpSetState_ConnectionManager_SourceProtocolInfo(UpnpMicroStackToken upnptoken, char* val)
\brief Sets the state of SourceProtocolInfo << urn:schemas-upnp-org:service:ConnectionManager:1 << ConnectionManager \para
\b Note: Must be called at least once prior to start
\param upnptoken The MicroStack token
\param val The new value of the state variable
*/
void UpnpSetState_ConnectionManager_SourceProtocolInfo(UpnpMicroStackToken upnptoken, char* val)
{
	struct UpnpDataObject *UpnpObject = (struct UpnpDataObject*)upnptoken;
	char* body;
	int bodylength;
	char* valstr;
	valstr = (char*)MALLOC(ILibXmlEscapeLength(val)+1);
	ILibXmlEscape(valstr,val);
	if (UpnpObject->ConnectionManager_SourceProtocolInfo != NULL) FREE(UpnpObject->ConnectionManager_SourceProtocolInfo);
	UpnpObject->ConnectionManager_SourceProtocolInfo = valstr;
	body = (char*)MALLOC(46 + (int)strlen(valstr));
	bodylength = sprintf(body,"%s>%s</%s","SourceProtocolInfo",valstr,"SourceProtocolInfo");
	UpnpSendEvent(upnptoken,body,bodylength,"ConnectionManager");
	FREE(body);
}

/*! \fn UpnpSetState_ConnectionManager_SinkProtocolInfo(UpnpMicroStackToken upnptoken, char* val)
\brief Sets the state of SinkProtocolInfo << urn:schemas-upnp-org:service:ConnectionManager:1 << ConnectionManager \para
\b Note: Must be called at least once prior to start
\param upnptoken The MicroStack token
\param val The new value of the state variable
*/
void UpnpSetState_ConnectionManager_SinkProtocolInfo(UpnpMicroStackToken upnptoken, char* val)
{
	struct UpnpDataObject *UpnpObject = (struct UpnpDataObject*)upnptoken;
	char* body;
	int bodylength;
	char* valstr;
	valstr = (char*)MALLOC(ILibXmlEscapeLength(val)+1);
	ILibXmlEscape(valstr,val);
	if (UpnpObject->ConnectionManager_SinkProtocolInfo != NULL) FREE(UpnpObject->ConnectionManager_SinkProtocolInfo);
	UpnpObject->ConnectionManager_SinkProtocolInfo = valstr;
	body = (char*)MALLOC(42 + (int)strlen(valstr));
	bodylength = sprintf(body,"%s>%s</%s","SinkProtocolInfo",valstr,"SinkProtocolInfo");
	UpnpSendEvent(upnptoken,body,bodylength,"ConnectionManager");
	FREE(body);
}

/*! \fn UpnpSetState_ConnectionManager_CurrentConnectionIDs(UpnpMicroStackToken upnptoken, char* val)
\brief Sets the state of CurrentConnectionIDs << urn:schemas-upnp-org:service:ConnectionManager:1 << ConnectionManager \para
\b Note: Must be called at least once prior to start
\param upnptoken The MicroStack token
\param val The new value of the state variable
*/
void UpnpSetState_ConnectionManager_CurrentConnectionIDs(UpnpMicroStackToken upnptoken, char* val)
{
	struct UpnpDataObject *UpnpObject = (struct UpnpDataObject*)upnptoken;
	char* body;
	int bodylength;
	char* valstr;
	valstr = (char*)MALLOC(ILibXmlEscapeLength(val)+1);
	ILibXmlEscape(valstr,val);
	if (UpnpObject->ConnectionManager_CurrentConnectionIDs != NULL) FREE(UpnpObject->ConnectionManager_CurrentConnectionIDs);
	UpnpObject->ConnectionManager_CurrentConnectionIDs = valstr;
	body = (char*)MALLOC(50 + (int)strlen(valstr));
	bodylength = sprintf(body,"%s>%s</%s","CurrentConnectionIDs",valstr,"CurrentConnectionIDs");
	UpnpSendEvent(upnptoken,body,bodylength,"ConnectionManager");
	FREE(body);
}

/*! \fn UpnpSetState_ContentDirectory_ContainerUpdateIDs(UpnpMicroStackToken upnptoken, char* val)
\brief Sets the state of ContainerUpdateIDs << urn:schemas-upnp-org:service:ContentDirectory:1 << ContentDirectory \para
\b Note: Must be called at least once prior to start
\param upnptoken The MicroStack token
\param val The new value of the state variable
*/
void UpnpSetState_ContentDirectory_ContainerUpdateIDs(UpnpMicroStackToken upnptoken, char* val)
{
	struct UpnpDataObject *UpnpObject = (struct UpnpDataObject*)upnptoken;
	char* body;
	int bodylength;
	char* valstr;
	valstr = (char*)MALLOC(ILibXmlEscapeLength(val)+1);
	ILibXmlEscape(valstr,val);
	if (UpnpObject->ContentDirectory_ContainerUpdateIDs != NULL) FREE(UpnpObject->ContentDirectory_ContainerUpdateIDs);
	UpnpObject->ContentDirectory_ContainerUpdateIDs = valstr;
	body = (char*)MALLOC(46 + (int)strlen(valstr));
	bodylength = sprintf(body,"%s>%s</%s","ContainerUpdateIDs",valstr,"ContainerUpdateIDs");
	UpnpSendEvent(upnptoken,body,bodylength,"ContentDirectory");
	FREE(body);
}

/*! \fn UpnpSetState_ContentDirectory_SystemUpdateID(UpnpMicroStackToken upnptoken, unsigned int val)
\brief Sets the state of SystemUpdateID << urn:schemas-upnp-org:service:ContentDirectory:1 << ContentDirectory \para
\b Note: Must be called at least once prior to start
\param upnptoken The MicroStack token
\param val The new value of the state variable
*/
void UpnpSetState_ContentDirectory_SystemUpdateID(UpnpMicroStackToken upnptoken, unsigned int val)
{
	struct UpnpDataObject *UpnpObject = (struct UpnpDataObject*)upnptoken;
	char* body;
	int bodylength;
	char* valstr;
	valstr = (char*)MALLOC(10);
	sprintf(valstr,"%u",val);
	if (UpnpObject->ContentDirectory_SystemUpdateID != NULL) FREE(UpnpObject->ContentDirectory_SystemUpdateID);
	UpnpObject->ContentDirectory_SystemUpdateID = valstr;
	body = (char*)MALLOC(38 + (int)strlen(valstr));
	bodylength = sprintf(body,"%s>%s</%s","SystemUpdateID",valstr,"SystemUpdateID");
	UpnpSendEvent(upnptoken,body,bodylength,"ContentDirectory");
	FREE(body);
}



void UpnpDestroyMicroStack(void *object)
{
	struct UpnpDataObject *upnp = (struct UpnpDataObject*)object;
	struct SubscriberInfo  *sinfo,*sinfo2;
	UpnpSendByeBye(upnp);
	
	sem_destroy(&(upnp->EventLock));
	
	FREE(upnp->ConnectionManager_SourceProtocolInfo);
	FREE(upnp->ConnectionManager_SinkProtocolInfo);
	FREE(upnp->ConnectionManager_CurrentConnectionIDs);
	FREE(upnp->ContentDirectory_ContainerUpdateIDs);
	FREE(upnp->ContentDirectory_SystemUpdateID);
	
	
	FREE(upnp->AddressList);
	FREE(upnp->NOTIFY_SEND_socks);
	FREE(upnp->UUID);
	FREE(upnp->Serial);
	FREE(upnp->DeviceDescription);
	
	sinfo = upnp->HeadSubscriberPtr_ConnectionManager;
	while(sinfo!=NULL)
	{
		sinfo2 = sinfo->Next;
		UpnpDestructSubscriberInfo(sinfo);
		sinfo = sinfo2;
	}
	sinfo = upnp->HeadSubscriberPtr_ContentDirectory;
	while(sinfo!=NULL)
	{
		sinfo2 = sinfo->Next;
		UpnpDestructSubscriberInfo(sinfo);
		sinfo = sinfo2;
	}
	
	
	#if defined(WIN32) || defined(_WIN32_WCE)
	WSACleanup();
	#endif
}
int UpnpGetLocalPortNumber(UpnpSessionToken token)
{
	return(ILibWebServer_GetPortNumber(((struct ILibWebServer_Session*)token)->Parent));
}
void UpnpSessionReceiveSink(
struct ILibWebServer_Session *sender,
int InterruptFlag,
struct packetheader *header,
char *bodyBuffer,
int *beginPointer,
int endPointer,
int done)
{
	if(header!=NULL && done !=0 && InterruptFlag==0)
	{
		UpnpProcessHTTPPacket(sender,header,bodyBuffer,beginPointer==NULL?0:*beginPointer,endPointer);
		if(beginPointer!=NULL) {*beginPointer = endPointer;}
	}
}
void UpnpSessionSink(struct ILibWebServer_Session *SessionToken, void *user)
{
	SessionToken->OnReceive = &UpnpSessionReceiveSink;
	SessionToken->User = user;
}
UpnpMicroStackToken UpnpCreateMicroStack(void *Chain, const char* FriendlyName,const char* UDN, const char* SerialNumber, const int NotifyCycleSeconds, const unsigned short PortNum)

{
	struct UpnpDataObject* RetVal = (struct UpnpDataObject*)MALLOC(sizeof(struct UpnpDataObject));
	//char dest[1024];
	//char* DDT;
	
	#if defined(WIN32) || defined(_WIN32_WCE)
	WORD wVersionRequested;
	WSADATA wsaData;
	srand((int)GetTickCount());
	#ifdef WINSOCK1
	wVersionRequested = MAKEWORD( 1, 1 );	
	#elif WINSOCK2
	wVersionRequested = MAKEWORD( 2, 0 );
	#endif
	if (WSAStartup( wVersionRequested, &wsaData ) != 0) {exit(1);}
	#else
	struct timeval tv;
	gettimeofday(&tv,NULL);
	srand((int)tv.tv_sec);
	#endif
	
	UpnpInit(RetVal,NotifyCycleSeconds,PortNum);
	RetVal->ForceExit = 0;
	RetVal->PreSelect = &UpnpMasterPreSelect;
	RetVal->PostSelect = &UpnpMasterPostSelect;
	RetVal->Destroy = &UpnpDestroyMicroStack;
	RetVal->InitialNotify = 0;
	if (UDN != NULL)
	{
		RetVal->UUID = (char*)MALLOC((int)strlen(UDN)+6);
		sprintf(RetVal->UUID,"uuid:%s",UDN);
		RetVal->UDN = RetVal->UUID + 5;
	}
	if (SerialNumber != NULL)
	{
		RetVal->Serial = (char*)MALLOC((int)strlen(SerialNumber)+1);
		strcpy(RetVal->Serial,SerialNumber);
	}
	
	RetVal->DeviceDescription = (char*)MALLOC(10+UpnpDeviceDescriptionTemplateLengthUX+ (int)strlen(FriendlyName)  + (((int)strlen(RetVal->Serial) + (int)strlen(RetVal->UUID)) * 1));
	
	
	RetVal->WebServerTimer = ILibCreateLifeTime(Chain);
	
	RetVal->HTTPServer = ILibWebServer_Create(Chain,UPNP_HTTP_MAXSOCKETS,PortNum,&UpnpSessionSink,RetVal);
	RetVal->WebSocketPortNumber=(int)ILibWebServer_GetPortNumber(RetVal->HTTPServer);
	
	
	
	ILibAddToChain(Chain,RetVal);
	RetVal->EventClient = ILibCreateWebClient(5,Chain);
	RetVal->Chain = Chain;
	RetVal->UpdateFlag = 0;


	RetVal->DeviceDescriptionLength = sprintf(RetVal->DeviceDescription,UpnpDeviceDescription,FriendlyName,RetVal->Serial,RetVal->UDN);
	/*
	DDT = ILibDecompressString((char*)UpnpDeviceDescriptionTemplate,UpnpDeviceDescriptionTemplateLength,UpnpDeviceDescriptionTemplateLengthUX);

	// albert, bug #39, now server name can be chinese			
	//strToUtf8( dest, FriendlyName, 1024,0,NULL );
	RetVal->DeviceDescriptionLength = sprintf(RetVal->DeviceDescription,DDT,FriendlyName,RetVal->Serial,RetVal->UDN);
	//RetVal->DeviceDescriptionLength = sprintf(RetVal->DeviceDescription,DDT,dest,RetVal->Serial,RetVal->UDN);
	
	FREE(DDT);
	*/
	
	sem_init(&(RetVal->EventLock),0,1);

	return(RetVal);
}









