/*
 * INTEL CONFIDENTIAL
 * Copyright (c) 2002, 2003 Intel Corporation.  All rights reserved.
 * 
 * The source code contained or described herein and all documents
 * related to the source code ("Material") are owned by Intel
 * Corporation or its suppliers or licensors.  Title to the
 * Material remains with Intel Corporation or its suppliers and
 * licensors.  The Material contains trade secrets and proprietary
 * and confidential information of Intel or its suppliers and
 * licensors. The Material is protected by worldwide copyright and
 * trade secret laws and treaty provisions.  No part of the Material
 * may be used, copied, reproduced, modified, published, uploaded,
 * posted, transmitted, distributed, or disclosed in any way without
 * Intel's prior express written permission.
 
 * No license under any patent, copyright, trade secret or other
 * intellectual property right is granted to or conferred upon you
 * by disclosure or delivery of the Materials, either expressly, by
 * implication, inducement, estoppel or otherwise. Any license
 * under such intellectual property rights must be express and
 * approved by Intel in writing.
 * 
 * $Workfile: UpnpMicroStack.h
 * $Revision: #1.0.1868.18043
 * $Author:   Intel Corporation, Intel Device Builder
 * $Date:     Friday, February 11, 2005
 *
 *
 *
 */
#ifndef __UpnpMicrostack__
#define __UpnpMicrostack__


#include "ILibAsyncSocket.h"

/*! \file UpnpMicroStack.h 
	\brief MicroStack APIs for Device Implementation
*/

/*! \defgroup MicroStack MicroStack Module
	\{
*/

struct UpnpDataObject;
struct packetheader;

typedef void* UpnpMicroStackToken;
typedef void* UpnpSessionToken;


/* Complex Type Parsers */


/* Complex Type Serializers */



/* Upnp Stack Management */
UpnpMicroStackToken UpnpCreateMicroStack(void *Chain, const char* FriendlyName,const char* UDN, const char* SerialNumber, const int NotifyCycleSeconds, const unsigned short PortNum);


void UpnpIPAddressListChanged(UpnpMicroStackToken MicroStackToken);
int UpnpGetLocalPortNumber(UpnpSessionToken token);
int   UpnpGetLocalInterfaceToHost(const UpnpSessionToken UpnpToken);
void* UpnpGetWebServerToken(const UpnpMicroStackToken MicroStackToken);

extern void UpnpConnectionManager_GetCurrentConnectionIDs(void* upnptoken);
extern void UpnpConnectionManager_GetCurrentConnectionInfo(void* upnptoken,int ConnectionID);
extern void UpnpConnectionManager_GetProtocolInfo(void* upnptoken);
extern void UpnpContentDirectory_Browse(void* upnptoken,char* ObjectID,char* BrowseFlag,char* Filter,unsigned int StartingIndex,unsigned int RequestedCount,char* SortCriteria);
extern void UpnpContentDirectory_GetSearchCapabilities(void* upnptoken);
extern void UpnpContentDirectory_GetSortCapabilities(void* upnptoken);
extern void UpnpContentDirectory_GetSystemUpdateID(void* upnptoken);
extern void UpnpContentDirectory_Search(void* upnptoken,char* ContainerID,char* SearchCriteria,char* Filter,unsigned int StartingIndex,unsigned int RequestedCount,char* SortCriteria);
extern void UpnpPresentationRequest(void* upnptoken, struct packetheader *packet);

void UpnpSetDisconnectFlag(UpnpSessionToken token,void *flag);

/* Invocation Response Methods */
void UpnpResponse_Error(const UpnpSessionToken UpnpToken, const int ErrorCode, const char* ErrorMsg);
void UpnpResponseGeneric(const UpnpSessionToken UpnpToken,const char* ServiceURI,const char* MethodName,const char* Params);
int UpnpAsyncResponse_START(const UpnpSessionToken UpnpToken, const char* actionName, const char* serviceUrnWithVersion);
int UpnpAsyncResponse_DONE(const UpnpSessionToken UpnpToken, const char* actionName);
int UpnpAsyncResponse_OUT(const UpnpSessionToken UpnpToken, const char* outArgName, const char* bytes, const int byteLength, enum ILibAsyncSocket_MemoryOwnership bytesMemoryOwnership,const int startArg, const int endArg);
void UpnpResponse_ConnectionManager_GetCurrentConnectionIDs(const UpnpSessionToken UpnpToken, const char* ConnectionIDs);
void UpnpResponse_ConnectionManager_GetCurrentConnectionInfo(const UpnpSessionToken UpnpToken, const int RcsID, const int AVTransportID, const char* ProtocolInfo, const char* PeerConnectionManager, const int PeerConnectionID, const char* Direction, const char* Status);
void UpnpResponse_ConnectionManager_GetProtocolInfo(const UpnpSessionToken UpnpToken, const char* Source, const char* Sink);
void UpnpResponse_ContentDirectory_Browse(const UpnpSessionToken UpnpToken, const char* Result, const unsigned int NumberReturned, const unsigned int TotalMatches, const unsigned int UpdateID);
void UpnpResponse_ContentDirectory_GetSearchCapabilities(const UpnpSessionToken UpnpToken, const char* SearchCaps);
void UpnpResponse_ContentDirectory_GetSortCapabilities(const UpnpSessionToken UpnpToken, const char* SortCaps);
void UpnpResponse_ContentDirectory_GetSystemUpdateID(const UpnpSessionToken UpnpToken, const unsigned int Id);
void UpnpResponse_ContentDirectory_Search(const UpnpSessionToken UpnpToken, const char* Result, const unsigned int NumberReturned, const unsigned int TotalMatches, const unsigned int UpdateID);

/* The string parameters for the following response methods MUST be MANUALLY escaped */
/* void UpnpResponse_ContentDirectory_Browse */
/* void UpnpResponse_ContentDirectory_Search */



/* State Variable Eventing Methods */
void UpnpSetState_ConnectionManager_SourceProtocolInfo(UpnpMicroStackToken microstack,char* val);
void UpnpSetState_ConnectionManager_SinkProtocolInfo(UpnpMicroStackToken microstack,char* val);
void UpnpSetState_ConnectionManager_CurrentConnectionIDs(UpnpMicroStackToken microstack,char* val);
void UpnpSetState_ContentDirectory_ContainerUpdateIDs(UpnpMicroStackToken microstack,char* val);
void UpnpSetState_ContentDirectory_SystemUpdateID(UpnpMicroStackToken microstack,unsigned int val);


/*! \} */
#endif
