#ifdef WIN32
#define _CRTDBG_MAP_ALLOC
#include <stdlib.h>
#ifndef _WIN32_WCE
#include <crtdbg.h>
#endif
#else
#include <stdlib.h>
#endif

#include <string.h>
#include "CdsMediaObject.h"
#include "debug.h"

struct CdsMediaObject* CDS_AllocateObject()
{
	struct CdsMediaObject *cdsObj = (struct CdsMediaObject *) malloc (sizeof(struct CdsMediaObject));
	memset(cdsObj, 0, sizeof(struct CdsMediaObject));
	return cdsObj;
}

struct CdsMediaResource* CDS_AllocateResource()
{
	struct CdsMediaResource *res = (struct CdsMediaResource *) malloc (sizeof(struct CdsMediaResource));
	res->Allocated = 0;
	res->Next = NULL;
	res->Value = res->Protection = res->ProtocolInfo = res->CreationTime = NULL;
	res->Size = res->ColorDepth = res->Bitrate = res->Duration = res->ResolutionX = res->ResolutionY = res->BitsPerSample = res->SampleFrequency = res->NrAudioChannels = -1;
	return res;
}

void CDS_DestroyObjects(struct CdsMediaObject *cdsObjList)
{
	struct CdsMediaObject *cdsObj = cdsObjList, *nextCds;

	while (cdsObj != NULL)
	{
		nextCds = cdsObj->Next;

		if ((cdsObj->DeallocateThese & CDS_ALLOC_Creator)	&& (cdsObj->Creator != NULL))	{ FREE(cdsObj->Creator); }
		if ((cdsObj->DeallocateThese & CDS_ALLOC_ID)		&& (cdsObj->ID != NULL))		{ FREE(cdsObj->ID); }
		if ((cdsObj->DeallocateThese & CDS_ALLOC_ParentID)	&& (cdsObj->ParentID != NULL))	{ FREE(cdsObj->ParentID); }
		if ((cdsObj->DeallocateThese & CDS_ALLOC_RefID)		&& (cdsObj->RefID != NULL))		{ FREE(cdsObj->RefID); }
		if ((cdsObj->DeallocateThese & CDS_ALLOC_Title)		&& (cdsObj->Title != NULL))		{ FREE(cdsObj->Title); }
		if ((cdsObj->DeallocateThese & CDS_ALLOC_Album)		&& (cdsObj->Album != NULL))		{ FREE(cdsObj->Album); }
		if ((cdsObj->DeallocateThese & CDS_ALLOC_Genre)		&& (cdsObj->Genre != NULL))		{ FREE(cdsObj->Genre); }
		CDS_DestroyResources(cdsObj->Res);
		
		free (cdsObj);
		cdsObj = nextCds;
	}
}

void CDS_DestroyResources(struct CdsMediaResource *resList)
{
	struct CdsMediaResource *res = resList, *next;

	while (res != NULL)
	{
		next = res->Next;
		if (res->Value != NULL) { free (res->Value); }
		
		if ((res->Allocated & CDS_ALLOC_ProtInfo) && (res->ProtocolInfo != NULL)) 
		{
			free (res->ProtocolInfo); 
		}
		
		if ((res->Allocated & CDS_ALLOC_Protection) && (res->Protection != NULL)) 
		{
			free (res->Protection); 
		}

		free (res);
		res = next;
	}
}