#ifdef WIN32
#define _CRTDBG_MAP_ALLOC
#include <stdlib.h>
#ifndef _WIN32_WCE
#include <crtdbg.h>
#endif
#else
#include <stdlib.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include "CdsObjectToDidl.h"
#include "CdsStrings.h"
#include "CdsMediaClass.h"
#include "ILibParsers.h"

#ifdef WIN32
#define strncasecmp(x,y,z) _strnicmp(x,y,z)
#endif

#ifdef _WIN32_WCE
#define strncasecmp(x,y,z) _strnicmp(x,y,z)
#endif

#define MAX_TIME_STRING_SIZE 17


/*
 *	Helper function.
 *	Returns the length for the specified string in its doubly-escaped form.
 */
int CdsObjectToDidl_Helper_DoubleEscapeLength(const char* data)
{
	int i = 0, j = 0;
	while (data[i] != 0)
	{
		switch (data[i])
		{
			case '"':
			j += 10;
			break;
			case '\'':
			j += 10;
			break;
			case '<':
			j += 8;
			break;
			case '>':
			j += 8;
			break;
			case '&':
			j += 9;
			break;
			default:
			j++;
		}
		i++;
	}
	return j;
}

int CdsObjectToDidl_Helper_DoubleEscape(char* outdata, const char* indata)
{
	int i=0;
	int inlen;
	char* out;
	
	out = outdata;
	inlen = (int)strlen(indata);
	
	for (i=0; i < inlen; i++)
	{
		if (indata[i] == '"')
		{
			memcpy(out, "&amp;quot;", 10);
			out = out + 10;
		}
		else
		if (indata[i] == '\'')
		{
			memcpy(out, "&amp;apos;", 10);
			out = out + 10;
		}
		else
		if (indata[i] == '<')
		{
			memcpy(out, "&amp;lt;", 8);
			out = out + 8;
		}
		else
		if (indata[i] == '>')
		{
			memcpy(out, "&amp;gt;", 8);
			out = out + 8;
		}
		else
		if (indata[i] == '&')
		{
			memcpy(out, "&amp;amp;", 9);
			out = out + 9;
		}
		else
		{
			out[0] = indata[i];
			out++;
		}
	}
	
	out[0] = 0;
	
	return (int)(out - outdata);
}

/*
 *	Prints the number of seconds in hh:mm:ss format.
 *	Negative values do not write anything.
 */
void CdsObjectToDidl_Helper_WriteTimeString(char* str, int intTime)
{
	if (intTime > 0)
	{
		sprintf(str,"%02d:%02d:%02d",((intTime/3600)%60),((intTime/60)%60),(intTime%60));
	}
}


/* see header file */
unsigned int CdsToDidl_GetFilterBitString(const char *filter)
{
	unsigned int retVal = 0;
	int i=0;
	int nextComma = 0;
	int filterLen = 0;

	if (filter != NULL)
	{
		filterLen = (int) strlen(filter);
		if (filterLen > 0)
		{
			/*
 			 *	The filter string is comma-delimited.
			 *	Do a linear parse and set bits in the retVal
			 *	for supported fields.
			 *
			 *	All supported, filterable metadata fields begin with
			 *	dc:, upnp:, res, container@, or @.
			 */

			i = nextComma = 0;
			while (filter[i] != '\0')
			{
				if (nextComma == 0)
				{
					switch (tolower(filter[i]))
					{
					case 'a':
						nextComma = 1;
						/* only supported filterable field is ALLIP, which is a DHWG extension */
						if (strncasecmp(filter+i, CDS_STRING_ALLIP, CDS_STRING_ALLIP_LEN) == 0)
						{
							retVal |= CdsFilter_DhwgAllIp;
						}
						break;

					case '@':
						nextComma = 1;
						/* Only supported filterable field are container@childCount and container@searchable */
						if (strncasecmp(filter+i, CDS_STRING_CHILDCOUNT, CDS_STRING_CHILDCOUNT_LEN) == 0)
						{
							retVal |= CdsFilter_ChildCount;
						}
						else if (strncasecmp(filter+i, CDS_STRING_SEARCHABLE, CDS_STRING_SEARCHABLE_LEN) == 0)
						{
							retVal |= CdsFilter_Searchable;
						}
						break;

					case 'c':
						nextComma = 1;
						/* Only supported filterable field are container@childCount and container@searchable */
						if (strncasecmp(filter+i, CDS_STRING_CONTAINER_CHILDCOUNT, CDS_STRING_CONTAINER_CHILDCOUNT_LEN) == 0)
						{
							retVal |= CdsFilter_ChildCount;
						}
						else if (strncasecmp(filter+i, CDS_STRING_CONTAINER_SEARCHABLE, CDS_STRING_CONTAINER_SEARCHABLE_LEN) == 0)
						{
							retVal |= CdsFilter_Searchable;
						}
						break;

					case 'd':
						nextComma = 1;

						/* Only supported filterable field is dc:creator. (dc:title is always required.) */
						if (strncasecmp(filter+i, CDS_STRING_CREATOR, CDS_STRING_CREATOR_LEN) == 0)
						{
							retVal |= CdsFilter_Creator;
						}
						break;

					case 'u':
						/* Only supported filterable field is upnp:album and upnp:genre */
						if (strncasecmp(filter+i, CDS_STRING_ALBUM, CDS_STRING_ALBUM_LEN) == 0)
						{
							retVal |= CdsFilter_Album;
						}
						else if (strncasecmp(filter+i, CDS_STRING_GENRE, CDS_STRING_GENRE_LEN) == 0)
						{
							retVal |= CdsFilter_Genre;
						}
						nextComma = 1;
						break;

					case 'r':
						nextComma = 1;
						/* only supported fields are: res, resolution, duration, bitrate, colordepth, size */
						
						if (strncasecmp(filter+i, CDS_STRING_RES_BITRATE, CDS_STRING_RES_BITRATE_LEN) == 0)
						{
							retVal |= CdsFilter_Res;
							retVal |= CdsFilter_Bitrate;
						}
						else if (strncasecmp(filter+i, CDS_STRING_RES_BITSPERSAMPLE, CDS_STRING_RES_BITSPERSAMPLE_LEN) == 0)
						{
							retVal |= CdsFilter_Res;
							retVal |= CdsFilter_BitsPerSample;
						}
						else if (strncasecmp(filter+i, CDS_STRING_RES_COLORDEPTH, CDS_STRING_RES_COLORDEPTH_LEN) == 0)
						{
							retVal |= CdsFilter_Res;
							retVal |= CdsFilter_ColorDepth;
						}
						else if (strncasecmp(filter+i, CDS_STRING_RES_DURATION, CDS_STRING_RES_DURATION_LEN) == 0)
						{
							retVal |= CdsFilter_Res;
							retVal |= CdsFilter_Duration;
						}
						else if (strncasecmp(filter+i, CDS_STRING_RES_NRAUDIOCHANNELS, CDS_STRING_RES_NRAUDIOCHANNELS_LEN) == 0)
						{
							retVal |= CdsFilter_Res;
							retVal |= CdsFilter_nrAudioChannels;
						}
						else if (strncasecmp(filter+i, CDS_STRING_RES_PROTECTION, CDS_STRING_RES_PROTECTION_LEN) == 0)
						{
							retVal |= CdsFilter_Res;
							retVal |= CdsFilter_Protection;
						}
						else if (strncasecmp(filter+i, CDS_STRING_RES_RESOLUTION, CDS_STRING_RES_RESOLUTION_LEN) == 0)
						{
							retVal |= CdsFilter_Res;
							retVal |= CdsFilter_Resolution;
						}
						else if (strncasecmp(filter+i, CDS_STRING_RES_SAMPLEFREQUENCY, CDS_STRING_RES_SAMPLEFREQUENCY_LEN) == 0)
						{
							retVal |= CdsFilter_Res;
							retVal |= CdsFilter_SampleFrequency;
						}
						else if (strncasecmp(filter+i, CDS_STRING_RES_SIZE, CDS_STRING_RES_SIZE_LEN) == 0)
						{
							retVal |= CdsFilter_Res;
							retVal |= CdsFilter_Size;
						}
						else if (strncasecmp(filter+i, CDS_STRING_RES, CDS_STRING_RES_LEN) == 0)
						{
							/* "res" must be tested after all res attributes */
							retVal |= CdsFilter_Res;
						}
						break;

					case '*':
						/* asterik infers all except ALLIP */
						retVal |= (~CdsFilter_DhwgAllIp);
						break;

					default:
						/*
						 *	If the character is an alphabetic char, then go to the next comma
						 *	because we know that this character is not for something we support.
						 */
						if (isalpha(filter[i]))
						{
							nextComma = 1;
						}
					}
				}
				else
				{
					/* We're trying to find the next comma... so if we find it, set the nextComma flag to zero. */
					if (filter[i] == ',')
					{
						nextComma = 0;
					}
				}

				/* move to the next char */
				i++;
			}
		}
	}

	return retVal;
}

/* see header file */
int CdsToDidl_GetCsvLengthFromBitString(unsigned int bitstring)
{
	int length = 0;
	int numFields = 0;

	if (bitstring & CdsSearchSort_Title)		{ length += CDS_STRING_TITLE_LEN;			numFields++; }
	// albert add
	if (bitstring & CdsSearchSort_Class)		{ length += CDS_STRING_CLASS_LEN;		numFields++; }
	if (bitstring & CdsFilter_ProtocolInfo)		{ length += CDS_STRING_RES_PROTOCOLINFO_LEN;		numFields++; }


	if (bitstring & CdsFilter_Creator)			{ length += CDS_STRING_CREATOR_LEN;			numFields++; }
	
	if (bitstring & CdsFilter_ChildCount)		{ length += CDS_STRING_CHILDCOUNT_LEN;		numFields++; }
	if (bitstring & CdsFilter_Searchable)		{ length += CDS_STRING_SEARCHABLE_LEN;		numFields++; }
	
	if (bitstring & CdsFilter_Album)			{ length += CDS_STRING_ALBUM_LEN;			numFields++; }
	if (bitstring & CdsFilter_Genre)			{ length += CDS_STRING_GENRE_LEN;			numFields++; }

	if (bitstring & CdsFilter_Res)				{ length += CDS_STRING_RES_LEN;					numFields++; }
	if (bitstring & CdsFilter_Resolution)		{ length += CDS_STRING_RES_RESOLUTION_LEN;		numFields++; }
	if (bitstring & CdsFilter_Duration)			{ length += CDS_STRING_RES_DURATION_LEN;		numFields++; }
	if (bitstring & CdsFilter_Bitrate)			{ length += CDS_STRING_RES_BITRATE_LEN;			numFields++; }
	if (bitstring & CdsFilter_ColorDepth)		{ length += CDS_STRING_RES_COLORDEPTH_LEN;		numFields++; }
	if (bitstring & CdsFilter_Size)				{ length += CDS_STRING_RES_SIZE_LEN;			numFields++; }
	if (bitstring & CdsFilter_BitsPerSample)	{ length += CDS_STRING_RES_BITSPERSAMPLE_LEN;	numFields++; }
	if (bitstring & CdsFilter_SampleFrequency)	{ length += CDS_STRING_RES_SAMPLEFREQUENCY_LEN; numFields++; }
	if (bitstring & CdsFilter_nrAudioChannels)	{ length += CDS_STRING_RES_NRAUDIOCHANNELS_LEN; numFields++; }
	if (bitstring & CdsFilter_Protection)		{ length += CDS_STRING_RES_PROTECTION_LEN;		numFields++; }

	if (numFields > 1)
	{
		/*
		 *	if there is more than 1 field, we have to include 
		 *	bytes for the comma chacters between the fields
		 */
		length += numFields;
		length--;
	}

	return length;
}

/* see header file */
#define WRITE_CSV_VALUE(sh,st,n) \
	if (n>0) { sh += sprintf(sh,",%s\0", st); }\
	else { sh += sprintf(sh,"%s\0", st); }\
	n++;
char* CdsToDidl_WriteCsvStringFromBitString(char *csvString, unsigned int bitstring)
{
	int numElements = 0;
	char *csv = csvString;

	if (bitstring & CdsSearchSort_Title) { WRITE_CSV_VALUE(csv, CDS_STRING_TITLE, numElements); }
	if (bitstring & CdsFilter_Creator) { WRITE_CSV_VALUE(csv, CDS_STRING_CREATOR, numElements); }
	
	if (bitstring & CdsFilter_ChildCount) { WRITE_CSV_VALUE(csv, CDS_STRING_CHILDCOUNT, numElements); }
	if (bitstring & CdsFilter_Searchable) { WRITE_CSV_VALUE(csv, CDS_STRING_SEARCHABLE, numElements); }
	// albert add
	if (bitstring & CdsSearchSort_Class) { WRITE_CSV_VALUE(csv, CDS_STRING_CLASS, numElements); }
	if (bitstring & CdsFilter_ProtocolInfo) { WRITE_CSV_VALUE(csv, CDS_STRING_RES_PROTOCOLINFO, numElements); }
	

	if (bitstring & CdsFilter_Album) { WRITE_CSV_VALUE(csv, CDS_STRING_ALBUM, numElements); }
	if (bitstring & CdsFilter_Genre) { WRITE_CSV_VALUE(csv, CDS_STRING_GENRE, numElements); }

	if (bitstring & CdsFilter_Res) { WRITE_CSV_VALUE(csv, CDS_STRING_RES, numElements); }
	if (bitstring & CdsFilter_Resolution) { WRITE_CSV_VALUE(csv, CDS_STRING_RES_RESOLUTION, numElements); }
	if (bitstring & CdsFilter_Duration) { WRITE_CSV_VALUE(csv, CDS_STRING_RES_DURATION, numElements); }
	if (bitstring & CdsFilter_Bitrate) { WRITE_CSV_VALUE(csv, CDS_STRING_RES_BITRATE, numElements); }
	if (bitstring & CdsFilter_ColorDepth) { WRITE_CSV_VALUE(csv, CDS_STRING_RES_COLORDEPTH, numElements); }
	if (bitstring & CdsFilter_Size) { WRITE_CSV_VALUE(csv, CDS_STRING_RES_SIZE, numElements); }
	if (bitstring & CdsFilter_BitsPerSample) { WRITE_CSV_VALUE(csv, CDS_STRING_RES_BITSPERSAMPLE, numElements); }
	if (bitstring & CdsFilter_SampleFrequency) { WRITE_CSV_VALUE(csv, CDS_STRING_RES_SAMPLEFREQUENCY, numElements); }
	if (bitstring & CdsFilter_nrAudioChannels) { WRITE_CSV_VALUE(csv, CDS_STRING_RES_NRAUDIOCHANNELS, numElements); }
	if (bitstring & CdsFilter_Protection) { WRITE_CSV_VALUE(csv, CDS_STRING_RES_PROTECTION, numElements); }

	csv[0] = '\0';
	return csvString;
}

typedef int (*CdsToDidl_Fn_XmlEscapeLength) (const char* string);
typedef int (*CdsToDidl_Fn_XmlEscape) (char *dest, const char* src);

/* see header file */
char* CdsToDidl_GetMediaObjectDidlEscaped (struct CdsMediaObject *mediaObj, int metadataXmlEscaped, unsigned int filter, int includeHeaderFooter, int *outErrorOrDidlLen)
{
	char *retVal = NULL;	
	int size = 1;			/* allocation size of retVal: minimum of a null character needed; if value is ever negative, then there's an error. */
	int len = 0;			/* actual length of the string in retVal */
	int imc_Object = -1, imc_Major = -1, imc_Minor1 = -1, imc_Minor2 = -1; /* index into the various CDS_CLASS_xxx string arrays */
	struct CdsMediaResource *res;
	unsigned int printThese = 0;		/* similar to filter, except indicates which fields will actually get printed */
	char *cp = NULL;
	char timeString[MAX_TIME_STRING_SIZE];

	CdsToDidl_Fn_XmlEscapeLength fnEscapeLength;
	CdsToDidl_Fn_XmlEscape		 fnEscape;

	/*
	 *	Given the info about the metadata,
	 *	obtain function pointers to the appropriate
	 *	functions for calculating lengths and escaping strings.
	 */


	if (metadataXmlEscaped)
	{
		fnEscapeLength = ILibXmlEscapeLength;
		fnEscape = ILibXmlEscape;
	}
	else
	{
		fnEscapeLength = CdsObjectToDidl_Helper_DoubleEscapeLength;
		fnEscape = CdsObjectToDidl_Helper_DoubleEscape;
	}


	/*
	 *	Include length needed for metadata content.
	 *	We need to doubly-escape because the data is a value of an XML element or attribute,
	 *	but the DIDL-Lite XML itself needs to be escaped, so this means all of the
	 *	values of XML attributes and elements need escaping too.
	 *
	 *	If at any point we determine there's an error with the media object, we
	 *	set "size" to an appropriate negative error value.
	 */
	if ((mediaObj->ID != NULL) && ((int)strlen(mediaObj->ID) > 0))
	{
		size += fnEscapeLength(mediaObj->ID);

		if ((mediaObj->ParentID != NULL) && (strlen(mediaObj->ParentID) > 0))
		{

			size += fnEscapeLength(mediaObj->ParentID);	

			if ((mediaObj->Title != NULL) && (strlen(mediaObj->Title) > 0))
			{
				size += (CDS_DIDL_TITLE_ESCAPED_LEN + fnEscapeLength(mediaObj->Title));		
				// albert add , 20050411
				size += (CDS_DIDL_COMPANY_ESCAPED_LEN + fnEscapeLength("honeywld"));		

				/* ObjectID, ParentID, and Title are valid... */

				if (mediaObj->RefID != NULL)		
				{
					size += fnEscapeLength(mediaObj->RefID);		
				}
				
				/* do not add memory for creator unless requested by the metadata filter */
				if ((mediaObj->Creator != NULL) && (filter & CdsFilter_Creator))
				{
					// albert add
					//if(strlen(mediaObj->Creator) > 0)
					//{
						size += (CDS_DIDL_CREATOR_ESCAPED_LEN + fnEscapeLength(mediaObj->Creator));
						printThese |= CdsFilter_Creator;
					//}
				}
				else if(filter & CdsFilter_Creator)
				{
					size += (CDS_DIDL_CREATOR_ESCAPED_LEN + 8);
					//size += (CDS_DIDL_CREATOR_ESCAPED_LEN + fnEscapeLength("HoneyWld"));
					printThese |= CdsFilter_Creator;
				}
				


				/* do not add memory for album unless requested by the metadata filter */
				//if ((mediaObj->Album != NULL) && (filter & CdsFilter_Album))
				if ((mediaObj->Album != NULL) && (filter & CdsFilter_Album) )
				{
					//if(strlen(mediaObj->Album) > 0)
					//{
						size += (CDS_DIDL_ALBUM_ESCAPED_LEN + fnEscapeLength(mediaObj->Album));
						printThese |= CdsFilter_Album;
					//}
				}
				else if(filter & CdsFilter_Album)
				{
					size += (CDS_DIDL_ALBUM_ESCAPED_LEN + 4);
					//size += (CDS_DIDL_ALBUM_ESCAPED_LEN + fnEscapeLength("None"));
					printThese |= CdsFilter_Album;
				}
				


				/* do not add memory for genre unless requested by the metadata filter */
				if ((mediaObj->Genre != NULL) && (filter & CdsFilter_Genre)  )
				{
					//if(strlen(mediaObj->Genre) > 0)
					//{
						size += (CDS_DIDL_GENRE_ESCAPED_LEN + fnEscapeLength(mediaObj->Genre));
						printThese |= CdsFilter_Genre;
					//}
				}
				else if(filter & CdsFilter_Genre)
				{
					size += (CDS_DIDL_GENRE_ESCAPED_LEN + 7);
					//size += (CDS_DIDL_GENRE_ESCAPED_LEN + fnEscapeLength("Default"));
					printThese |= CdsFilter_Genre;
				}
				

				/* validate the media class */
				if ((mediaObj->MediaClass != 0) & (size > 0))
				{
					if (
						(mediaObj->RefID != NULL) &&
						((mediaObj->MediaClass & CDS_CLASS_MASK_CONTAINER) != 0)
						)
					{
						/* media class is a container of some sort, but we have refID value - this is an error */
						size = Error_CdsObjectToDidl_MismatchContainerRefID;
					}
					else
					{
						imc_Object = (mediaObj->MediaClass & CDS_CLASS_MASK_OBJECT_TYPE) >> CDS_SHIFT_OBJECT_TYPE;
						imc_Major  = (mediaObj->MediaClass & CDS_CLASS_MASK_MAJOR) >> CDS_SHIFT_MAJOR_TYPE;
						imc_Minor1 = (mediaObj->MediaClass & CDS_CLASS_MASK_MINOR1) >> CDS_SHIFT_MINOR1_TYPE;
						imc_Minor2 = (mediaObj->MediaClass & CDS_CLASS_MASK_MINOR2) >> CDS_SHIFT_MINOR2_TYPE;

						/* check for various error conditions with media class */
						if ((imc_Object >= CDS_CLASS_OBJECT_TYPE_LEN) || (imc_Object < 1))
						{
							size = Error_CdsObjectToDidl_UndefinedObjectType;
						}
						else if ((imc_Major >= CDS_CLASS_MAJOR_TYPE_LEN) || (imc_Major < 0))
						{
							size = Error_CdsObjectToDidl_UndefinedMajorType;
						}
						else if	((imc_Minor1 >= CDS_CLASS_MINOR1_TYPE_LEN) || (imc_Minor1 < 0))
						{
							size = Error_CdsObjectToDidl_UndefinedMinor1Type;
						}
						else if	((imc_Minor2 >= CDS_CLASS_MINOR2_TYPE_LEN) || (imc_Minor2 < 0))
						{

							/*
							 *	TODO: If you want to return an error for an invalid
							 *	index into CDS_CLASS_MINOR2_TYPE[], then use this
							 *	code instead.
							 *
							 *	size = Error_CdsObjectToDidl_UndefinedMinor2Type;
							 */

							/* ignore the MINOR2 field designator if it's out of range */
							imc_Minor2 = 0;
						}
						else
						{
							/* media class is valid - calculate length - assume no strings need escaping */
							size += (int) strlen(CDS_CLASS_OBJECT_TYPE[imc_Object]);
							size += (int) strlen(CDS_CLASS_MAJOR_TYPE[imc_Major]);
							size += (int) strlen(CDS_CLASS_MINOR1_TYPE[imc_Minor1]);
							size += (int) strlen(CDS_CLASS_MINOR2_TYPE[imc_Minor2]);
							size += CDS_DIDL_CLASS_ESCAPED_LEN + 4;	/*add 4 for additional . chars */

							/*
							*	Note the length needed for item/container element open and close tags.
							*	Also note additional length for restricted, searchable attributes.
							*/
							size++; /* restricted */
							if ((mediaObj->MediaClass & CDS_CLASS_MASK_CONTAINER) != 0)
							{
								size++; /* byte for searchable flag */
								size +=10; /* bytes for childCount attribute */
								size += CDS_DIDL_CONTAINER_START_ESCAPED_LEN;
								size += CDS_DIDL_CONTAINER_END_ESCAPED_LEN;
							}
							else if ((mediaObj->MediaClass & CDS_CLASS_MASK_ITEM) != 0)
							{
								if (mediaObj->RefID != NULL)
								{
									size += CDS_DIDL_REFITEM_START_ESCAPED_LEN;
								}
								else
								{
									size += CDS_DIDL_ITEM_START_ESCAPED_LEN;
								}

								size += CDS_DIDL_ITEM_END_ESCAPED_LEN;
							}

							/*
							 *	At this point, we add the amount of memory needed
							 *	for the resource elements to the size... if and only
							 *	if <res> elements were requsted.
							 */

							if (filter & CdsFilter_Res)
							{
								printThese |= CdsFilter_Res;

								res = mediaObj->Res;
								while (res != NULL)
								{
									/* add size for minimal <res> element and its metadata */
									size += (CDS_DIDL_RES_START_ESCAPED_LEN + CDS_DIDL_RES_VALUE_ESCAPED_LEN);

									size += fnEscapeLength(res->ProtocolInfo);

									if (res->Value != NULL)
									{
										size += fnEscapeLength(res->Value);
									}

									if ((res->Bitrate >= 0) && (filter & CdsFilter_Bitrate))
									{
										printThese |= CdsFilter_Bitrate;
										size += (CDS_DIDL_ATTRIB_BITRATE_ESCAPED_LEN + SIZE_INT32_AS_CHARS);
									}
									if ((res->BitsPerSample >= 0) && (filter & CdsFilter_BitsPerSample))
									{
										printThese |= CdsFilter_BitsPerSample;
										size += (CDS_DIDL_ATTRIB_BITSPERSAMPLE_ESCAPED_LEN + SIZE_INT32_AS_CHARS);
									}
									if ((res->ColorDepth >= 0) && (filter & CdsFilter_ColorDepth))
									{
										printThese |= CdsFilter_ColorDepth;
										size += (CDS_DIDL_ATTRIB_COLORDEPTH_ESCAPED_LEN	 + SIZE_INT32_AS_CHARS);
									}
									if ((res->Duration >= 0) && (filter & CdsFilter_Duration))
									{
										printThese |= CdsFilter_Duration;
										size += (CDS_DIDL_ATTRIB_DURATION_ESCAPED_LEN + MAX_TIME_STRING_SIZE);
									}
									if ((res->NrAudioChannels >= 0) && (filter & CdsFilter_nrAudioChannels))
									{
										printThese |= CdsFilter_nrAudioChannels;
										size += (CDS_DIDL_ATTRIB_NRAUDIOCHANNELS_ESCAPED_LEN + SIZE_INT32_AS_CHARS);
									}
									if ((res->Protection != NULL) && (filter & CdsFilter_Protection))
									{
										printThese |= CdsFilter_Protection;
										size += (CDS_DIDL_ATTRIB_PROTECTION_ESCAPED_LEN + fnEscapeLength(res->Protection) + 1);
									}
									//james 05.05.11 mark it
									if ((res->ResolutionX >= 0) && (res->ResolutionY > 0) /*&& (filter & CdsFilter_Resolution)*/)
									{
										printThese |= CdsFilter_Resolution;
										size += (CDS_DIDL_ATTRIB_RESOLUTION_ESCAPED_LEN + SIZE_INT32_AS_CHARS + SIZE_INT32_AS_CHARS);
									}
									if ((res->SampleFrequency >= 0) && (filter & CdsFilter_SampleFrequency))
									{
										printThese |= CdsFilter_SampleFrequency;
										size += (CDS_DIDL_ATTRIB_SAMPLEFREQUENCY_ESCAPED_LEN + SIZE_INT32_AS_CHARS);
									}
									if ((res->Size >= 0) && (filter & CdsFilter_Size))
									{
										printThese |= CdsFilter_Size;
										size += (CDS_DIDL_ATTRIB_SIZE_ESCAPED_LEN + SIZE_INT32_AS_CHARS);
									}

									res = res->Next;
								}
							}


							if (includeHeaderFooter != 0)
							{
								/* appropriately include length for header/footer, if requested */
								size += CDS_DIDL_HEADER_ESCAPED_LEN;
								size += CDS_DIDL_FOOTER_ESCAPED_LEN;
							}
						}
					}
				}
				else
				{
					/* invalid media class - this is an error */
					size = 	Error_CdsObjectToDidl_InvalidMediaClass;
				}
			}
			else
			{
				/* title cannot be empty or null */
				size = Error_CdsObjectToDidl_EmptyTitle;
			}
		}
		else
		{
			/* parent ID must not be empty or null */
			size = Error_CdsObjectToDidl_EmptyParentID;
		}
	}
	else
	{
		/* object ID cannot be empty or null */
		size = Error_CdsObjectToDidl_EmptyObjectID;
	}

	if (size > 0)
	{
		/*
		 *	If this code executes, then the media object can be serialized to DIDL-Lite
		 *	without any problems.
		 */
		// albert add ,20050410
		// David Hu 2007/04/21, fix SEGV for that size is too small 
		//size = size + 80;
		size = size + 255;

		cp = retVal = (char*) malloc (size+10);

		if (cp == NULL)
		{
			fprintf(stderr, "!!! malloc failed in CdsToDidl_GetmediaObjectDidlEscaped() !!!\r\n");
		}

		/* print DIDL-Lite element if requested */
		if (includeHeaderFooter != 0)
		{
			cp += sprintf(cp, CDS_DIDL_HEADER_ESCAPED);
		}

		/* print <item> or <container> start */
		if ((mediaObj->MediaClass & CDS_CLASS_MASK_CONTAINER) != 0)
		{
			cp += sprintf(cp, CDS_DIDL_CONTAINER_START1_ESCAPED);
			cp += fnEscape(cp, mediaObj->ID);
			
			cp += sprintf(cp, CDS_DIDL_CONTAINER_START2_ESCAPED);
			cp += fnEscape(cp, mediaObj->ParentID);
			
			cp += sprintf(cp, CDS_DIDL_CONTAINER_START3_ESCAPED);
			if (mediaObj->Flags & CDS_OBJPROP_FLAGS_Restricted)
			{
				cp[0] = '1';
			}
			else
			{
				cp[0] = '0';
			}
			cp++;
			
			if (filter & CdsFilter_Searchable)
			{
				cp += sprintf(cp, CDS_DIDL_CONTAINER_START4_ESCAPED);
				if (mediaObj->Flags & CDS_OBJPROP_FLAGS_Searchable )
				{
					cp[0] = '1';
				}
				else
				{
					cp[0] = '0';
				}
				cp++;
			}

			if (filter & CdsFilter_ChildCount)
			{
				cp += sprintf(cp, CDS_DIDL_CONTAINER_START5_ESCAPED, mediaObj->ChildCount);
			}

			cp += sprintf(cp, CDS_DIDL_CONTAINER_START6_ESCAPED);
		}
		else
		{
			cp += sprintf(cp, CDS_DIDL_ITEM_START1_ESCAPED);
			cp += fnEscape(cp, mediaObj->ID);
			
			cp += sprintf(cp, CDS_DIDL_ITEM_START2_ESCAPED);
			cp += fnEscape(cp, mediaObj->ParentID);
			
			cp += sprintf(cp, CDS_DIDL_ITEM_START3_ESCAPED);
			if (mediaObj->Flags & CDS_OBJPROP_FLAGS_Restricted)
			{
				cp[0] = '1';
			}
			else
			{
				cp[0] = '0';
			}
			cp++;

			if ((mediaObj->RefID != NULL) && (strlen(mediaObj->RefID) > 0))
			{
				cp += sprintf(cp, CDS_DIDL_REFITEM_START4_ESCAPED);
				cp += fnEscape(cp, mediaObj->RefID);
				cp += sprintf(cp, CDS_DIDL_REFITEM_START5_ESCAPED);
			}
			else
			{
				cp += sprintf(cp, CDS_DIDL_ITEM_START4_ESCAPED);
			}
		}

		/* print title */
		cp += sprintf(cp, CDS_DIDL_TITLE1_ESCAPED);
		cp += fnEscape(cp, mediaObj->Title);
		cp += sprintf(cp, CDS_DIDL_TITLE2_ESCAPED);

		// albert add , 20050411
		cp += sprintf(cp, CDS_DIDL_COMPANY1_ESCAPED);
		cp += fnEscape(cp, "honeywld");
		cp += sprintf(cp, CDS_DIDL_COMPANY2_ESCAPED);

		// TODO : need to modify
		cp += sprintf(cp, CDS_DIDL_DATE1_ESCAPED);
		cp += fnEscape(cp, mediaObj->CreationTime);
		cp += sprintf(cp, CDS_DIDL_DATE2_ESCAPED);

		/* print media class */
		cp += sprintf(cp, CDS_DIDL_CLASS1_ESCAPED);
		cp += sprintf(cp, CDS_CLASS_OBJECT_TYPE[imc_Object]);
		if (imc_Major > 0) { cp += sprintf(cp, ".%s", CDS_CLASS_MAJOR_TYPE[imc_Major]); }
		if (imc_Minor1 > 0) { cp += sprintf(cp, ".%s", CDS_CLASS_MINOR1_TYPE[imc_Minor1]); }
		if (imc_Minor2 > 0) { cp += sprintf(cp, ".%s", CDS_CLASS_MINOR2_TYPE[imc_Minor2]); }
		cp += sprintf(cp, CDS_DIDL_CLASS2_ESCAPED);

		// 20040514, albert modify, we need to print creator, album and genre, no matter there is av value or not
		/* print creator */
		if (printThese & CdsFilter_Creator)
		{
			if(mediaObj->Creator == NULL)
			{
				cp += sprintf(cp, CDS_DIDL_CREATOR1_ESCAPED);
				cp += sprintf(cp, "HoneyWld");
				cp += sprintf(cp, CDS_DIDL_CREATOR2_ESCAPED);
			}
			else
			{
				cp += sprintf(cp, CDS_DIDL_CREATOR1_ESCAPED);
				cp += fnEscape(cp, mediaObj->Creator);
				cp += sprintf(cp, CDS_DIDL_CREATOR2_ESCAPED);
			}
		}

		/* print genre */
		if (printThese & CdsFilter_Genre)
		{
			if(mediaObj->Genre == NULL)
			{
				cp += sprintf(cp, CDS_DIDL_GENRE1_ESCAPED);
				cp += sprintf(cp, "None");
				cp += sprintf(cp, CDS_DIDL_GENRE2_ESCAPED);
			}
			else
			{
				cp += sprintf(cp, CDS_DIDL_GENRE1_ESCAPED);
				cp += fnEscape(cp, mediaObj->Genre);
				cp += sprintf(cp, CDS_DIDL_GENRE2_ESCAPED);
			}
		}
		

		/* print album */
		if (printThese & CdsFilter_Album)
		{
			if(mediaObj->Album == NULL)
			{
				cp += sprintf(cp, CDS_DIDL_ALBUM1_ESCAPED);
				cp += sprintf(cp, "Default");
				cp += sprintf(cp, CDS_DIDL_ALBUM2_ESCAPED);
			}
			else
			{
				cp += sprintf(cp, CDS_DIDL_ALBUM1_ESCAPED);
				cp += fnEscape(cp, mediaObj->Album);
				cp += sprintf(cp, CDS_DIDL_ALBUM2_ESCAPED);
			}
		}
		

		/* print resource and appropriate fields */

		// 20040514, albert modify, we need to print creator, album and genre, no matter there is av value or not
		//printThese |= printThese | CdsFilter_Creator | CdsFilter_Album | CdsFilter_Genre ;
		if (printThese & CdsFilter_Res)
		{
			res = mediaObj->Res;

			while (res != NULL)
			{
				/*
				 *	Need to double escape because DIDL-Lite is XML and the DIDL-Lite is
				 *	going within a SOAP response message, which is also XML.
				 */
				cp += sprintf(cp, CDS_DIDL_RES_START1_ESCAPED);
				if (metadataXmlEscaped)
				{
					cp += ILibXmlEscape(cp, res->ProtocolInfo);
				}
				else
				{
					cp += CdsObjectToDidl_Helper_DoubleEscape(cp, res->ProtocolInfo);
				}
				cp += sprintf(cp, CDS_DIDL_RES_START2_ESCAPED);
				
				if ((res->Bitrate >= 0) && (printThese & CdsFilter_Bitrate))
				{
					/* CDS spec is strange in that bitRate is actually supposed to have the byte rate */
					cp += sprintf(cp, CDS_DIDL_ATTRIB_BITRATE_ESCAPED, (res->Bitrate / 8));
				}
				if ((res->BitsPerSample >= 0) && (printThese & CdsFilter_BitsPerSample))
				{
					cp += sprintf(cp, CDS_DIDL_ATTRIB_BITSPERSAMPLE_ESCAPED, res->BitsPerSample);
				}
				if ((res->ColorDepth >= 0) && (printThese & CdsFilter_ColorDepth))
				{
					cp += sprintf(cp, CDS_DIDL_ATTRIB_COLORDEPTH_ESCAPED, res->ColorDepth);
				}
				if ((res->Duration >= 0) && (printThese & CdsFilter_Duration))
				{
					CdsObjectToDidl_Helper_WriteTimeString(timeString, res->Duration);
					cp += sprintf(cp, CDS_DIDL_ATTRIB_DURATION_ESCAPED, timeString);
				}
				if ((res->NrAudioChannels >= 0) && (printThese & CdsFilter_nrAudioChannels))
				{
					cp += sprintf(cp, CDS_DIDL_ATTRIB_NRAUDIOCHANNELS_ESCAPED, res->NrAudioChannels);
				}
				if ((res->Protection != NULL) && (printThese & CdsFilter_Protection))
				{
					cp += sprintf(cp, CDS_DIDL_ATTRIB_PROTECTION_START);
					cp += fnEscape(cp, res->Protection);
					cp += sprintf(cp, CDS_DIDL_ATTRIB_PROTECTION_END);
				}
				//james 05.05.11 mark it
				if ((res->ResolutionX >= 0) && (res->ResolutionY >= 0)/* && (printThese & CdsFilter_Resolution)*/)
				{
					//cp += sprintf(cp, CDS_DIDL_ATTRIB_RESOLUTION_ESCAPED, res->ResolutionX, res->ResolutionY);
					cp += sprintf(cp, " resolution=&quot;%dx%d&quot;", res->ResolutionX, res->ResolutionY);
					 
				}
				if ((res->SampleFrequency >= 0) && (printThese & CdsFilter_SampleFrequency))
				{
					cp += sprintf(cp, CDS_DIDL_ATTRIB_SAMPLEFREQUENCY_ESCAPED, res->SampleFrequency);
				}
				if ((res->Size >= 0) && (printThese & CdsFilter_Size))
				{
					// albert modify
					//cp += sprintf(cp, CDS_DIDL_ATTRIB_SIZE_ESCAPED, res->Size);
					cp += sprintf(cp, " size=&quot;%ld&quot;", res->Size);
				}

				cp += sprintf(cp, CDS_DIDL_RES_VALUE1_ESCAPED);
				if (res->Value != NULL)
				{
					cp += fnEscape(cp, res->Value);
				}
				cp += sprintf(cp, CDS_DIDL_RES_VALUE2_ESCAPED);

				res = res->Next;
			}
		}

		if ((mediaObj->MediaClass & CDS_CLASS_MASK_CONTAINER) != 0)
		{
			cp += sprintf(cp, CDS_DIDL_CONTAINER_END_ESCAPED);
		}
		else
		{
			cp += sprintf(cp, CDS_DIDL_ITEM_END_ESCAPED);
		}

		if (includeHeaderFooter != 0)
		{
			sprintf(cp, CDS_DIDL_FOOTER_ESCAPED);
		}

		len = (int) (cp - retVal);
		if (len > size+5)
		{
			/* we overwrote memory - this is bad */
			free (retVal);
			retVal = NULL;
			size = Error_CdsObjectToDidl_CorruptedMemory;
		}
		if( len >= size )
		{
			retVal[size]= '\0';
		}
		else
		{
			size = len;
			retVal[size] = '\0';
		}
	}

	if (outErrorOrDidlLen != NULL)
	{
		*outErrorOrDidlLen = size;
	}

	return retVal;
}

