#ifdef WIN32
#define _CRTDBG_MAP_ALLOC
#include <stdlib.h>
#ifndef _WIN32_WCE
#include <crtdbg.h>
#endif
#else
#include <stdlib.h>
#endif

#include "FileSystemEnumerator.h"
#include "CdsErrors.h"
#include "ILibParsers.h"
#include "FileIoAbstraction.h"
#include "CdsMediaObject.h"
#include "MyString.h"
#include "debug.h"
#include "MimeTypes.h"
#include "FileSystemTree.h"
//#include <stdio.h>

extern int db_getOneRecord(const char *SQLstring,struct FSE_QueryState *tmpFSE_QS);
extern int db_getOneRecordInIndex(const char *SQLstring, struct FSE_QueryState *tmpFSE_QS, int index);
extern int db_getRecords(const char *SQLstring, struct FSE_QueryState FSE_QSArray[], int startIndex, int readCount);
extern void GetFileCreationTime(char *filepath, char *time_string);
extern int db_getCount(const char *SQLstring);

void _FSE_ParsePathInfo(const char* fullpath, const struct FSE_State *fse, const struct FSE_QueryState *qs, /*INOUT*/ struct FSE_PathInfo *pi);
void* _FSE_HandleQuery_Search(struct FSE_State *fse, struct FSE_QueryState *qs, struct MSA_CdsQuery *cdsQuery, unsigned int filter, /*OUT*/ enum Enum_CdsErrors *cdsErrorCode);


void _FSE_ParsePathInfo(const char* fullpath, const struct FSE_State *fse, const struct FSE_QueryState *qs, /*INOUT*/ struct FSE_PathInfo *pi)
{
	int len;
	int ddpos=0;
	int dotpos=0;

	/* free memory */
	if (pi->Ext != NULL) { free (pi->Ext); pi->Ext = NULL; pi->ExtLen = 0; }
	if (pi->FullPath != NULL) { free (pi->FullPath); pi->FullPath = NULL; pi->FullPathLen = 0; }
	if (pi->Name != NULL) { free (pi->Name); pi->Name = NULL; pi->NameLen = 0; }
	if (pi->NameNoExt != NULL) { free (pi->NameNoExt); pi->NameNoExt = NULL; pi->NameNoExtLen = 0; }
	if (pi->ParentPath != NULL) { free (pi->ParentPath); pi->ParentPath = NULL; pi->ParentPathLen = 0; }
	if (pi->RelativeFullPath != NULL) { free (pi->RelativeFullPath); pi->RelativeFullPath = NULL; pi->RelativeFullPathLen = 0; } 
	if (pi->RelativeParent != NULL) { free (pi->RelativeParent); pi->RelativeParent = NULL; pi->RelativeParentLen = 0; }

	len = (int) strlen(fullpath);
	pi->FullPath = (char*) MALLOC(len+2);
	memcpy(pi->FullPath, fullpath, len);
	pi->FullPath[len] = '\0';
	pi->FullPathLen = len;

	/*
	 *	If it's a directory, make sure it ends
	 *	with a trailing directory delimiter.
	 *	We do this because fse->RootPath will have
	 *	a trailing delimiter.
	 */
	if (
		(qs->ProcessObjectType == FSE_OT_Directory) &&
		(pi->FullPath[pi->FullPathLen-1] != fse->DirDelimiterChr)
		)
	{
		pi->FullPath[pi->FullPathLen] = fse->DirDelimiterChr;
		pi->FullPathLen++;
		pi->FullPath[pi->FullPathLen] = '\0';
	}

	/*
	 *	Get the relative full path.
	 */
	pi->RelativeFullPathLen = pi->FullPathLen - fse->RootLen;
	pi->RelativeFullPath = (char*) malloc (pi->RelativeFullPathLen + 1);
	memcpy(pi->RelativeFullPath, pi->FullPath + fse->RootLen, pi->RelativeFullPathLen);
	pi->RelativeFullPath[pi->RelativeFullPathLen] = '\0';

	/*
	 *	Find last directory delimiter.
	 *	If the fullpath is a directory, temporarily
	 *	terminate w/o a trailing delimiter.
	 */

	if (qs->ProcessObjectType == FSE_OT_Directory)
	{
		pi->FullPathLen--;
		pi->FullPath[pi->FullPathLen] = '\0';
	}
	ddpos = LastIndexOf(pi->FullPath, fse->DirDelimiter);
	if (qs->ProcessObjectType == FSE_OT_Directory)
	{
		pi->FullPath[pi->FullPathLen] = fse->DirDelimiterChr;
		pi->FullPathLen++;
	}

	/*
	 *	Get the name of the directory/file.
	 *	If no directory delimiter was found,
	 *	there is no name... which indicates
	 *	we should treat it as the root.
	 */
	if (ddpos >= 0)
	{
		pi->NameLen = pi->FullPathLen - ddpos - 1;
		pi->Name = (char*) MALLOC(pi->NameLen+1);
		memcpy(pi->Name, pi->FullPath + ddpos + 1, pi->NameLen+1);
	}
	else
	{
		pi->NameLen = 0;
		pi->Name = (char*) MALLOC(1);
		pi->Name[0] = '\0';
	}


	/*
	 *	Chop a trailing dir delimiter.
	 */
	if (
		(pi->NameLen > 0) && 
		(pi->Name[pi->NameLen-1] == fse->DirDelimiterChr)
		)
		{
			pi->NameLen--;
			pi->Name[pi->NameLen] = '\0';
		}

	dotpos = LastIndexOf(pi->Name, ".");
	if (dotpos >= 0 && qs->ObjectType != FSE_OT_Directory) //pslin add to avoid dir name cutting
	{
		/*
		 *	Get the name w/o the extension and save extension.
		 */
		pi->NameNoExtLen = dotpos;
		pi->NameNoExt = (char*) MALLOC(pi->NameNoExtLen + 1);
		memcpy(pi->NameNoExt, pi->Name, pi->NameNoExtLen);
		pi->NameNoExt[pi->NameNoExtLen] = '\0';

		pi->ExtLen = pi->NameLen - pi->NameNoExtLen;
		pi->Ext = (char*) malloc (pi->ExtLen + 1);
		memcpy(pi->Ext, pi->Name + dotpos, pi->ExtLen+1);
	}
	else
	{
		/*
		 *	Name w/o extension is the same as name.
		 *	Extension is blank.
		 */
		pi->NameNoExt = (char*) MALLOC(pi->NameLen+1);
		memcpy(pi->NameNoExt, pi->Name, pi->NameLen+1);
		pi->NameNoExtLen = pi->NameLen;

		pi->ExtLen = 0;
		pi->Ext = (char*) MALLOC(pi->ExtLen+1);
		pi->Ext[0] = '\0';
	}

	/*
	 *	Get the parent directory name
	 */
	if (ddpos >= 0)
	{
		pi->ParentPathLen = ddpos+1;
		pi->ParentPath = (char*) MALLOC(pi->ParentPathLen+1);
		memcpy(pi->ParentPath, pi->FullPath, pi->ParentPathLen);
		pi->ParentPath[pi->ParentPathLen] = '\0';
	}
	else
	{
		pi->ParentPathLen = 0;
		pi->ParentPath = (char*) MALLOC(1);
		pi->ParentPath[0] = '\0';
	}

	/* 
	 *	Get the relative parent dir.
	 *	rootlen includes a trailinger dir delimiter.
	 */
	if ((pi->ParentPathLen > 0) && (pi->ParentPathLen >= fse->RootLen))
	{
		pi->RelativeParentLen = pi->ParentPathLen - fse->RootLen;
		pi->RelativeParent = (char*) MALLOC(pi->RelativeParentLen + 1);
		memcpy(pi->RelativeParent, pi->ParentPath + fse->RootLen, pi->RelativeParentLen);
		pi->RelativeParent[pi->RelativeParentLen] = '\0';
	}
	else
	{
		pi->RelativeParentLen = 0;
		pi->RelativeParent = (char*) MALLOC(1);
		pi->RelativeParent[0] = '\0';
	}

}

void* _FSE_HandleQuery_Search(struct FSE_State *fse, struct FSE_QueryState *qs, struct MSA_CdsQuery *cdsQuery, unsigned int filter, /*OUT*/ enum Enum_CdsErrors *cdsErrorCode)
{
	/*
	 *	All containers are marked as non-searchable. (e.g. @searchable="0").
	 *	CDS spec says to reutn zero results. 
	 *	Furthermore, there is no error code defined for this case.
	 */

	return qs;
}

void* FSE_HandleQuery(void *fseObj, struct MSA_CdsQuery *cdsQuery, unsigned int filter, /*OUT*/ enum Enum_CdsErrors *cdsErrorCode)
{
	char strSQL[MAX_SQL_STR_LENGTH];
	struct FSE_State *fse = (struct FSE_State*) fseObj;
	struct FSE_QueryState *qs = NULL;
	
	*cdsErrorCode = CdsError_None;

	qs = (struct FSE_QueryState*) malloc (sizeof(struct FSE_QueryState));
	memset(qs, 0, sizeof(struct FSE_QueryState));

	if (strcmp(cdsQuery->ObjectID,"") == 0)
	{
		qs->ObjectType = FSE_OT_NoExist;
	}
	else
	{
		sprintf(strSQL,"select * from FSE_QueryState where MyIndex = %s", cdsQuery->ObjectID);
		if (db_getOneRecord(strSQL,qs) == 0)
		{
			qs->ObjectType = FSE_OT_NoExist;
		}
	}

	/*
	 *	Set qs->FilePath
	 */
	switch(qs->ObjectType)
	{
	case FSE_OT_NoExist:
		*cdsErrorCode = CDS_EC_OBJECT_ID_NO_EXIST;
		break;
	
	case FSE_OT_File:
		switch (cdsQuery->QueryType)
		{
		case MSA_Query_BrowseMetadata:
			qs->ProcessIndex = 0;
			qs->ProcessObjectType  = qs->ObjectType;
			// utf8
			strcpy(qs->ProcessPath, qs->FilePath);
			return qs;
		
		case MSA_Query_BrowseDirectChildren:
		case MSA_Query_Search:
			*cdsErrorCode = CDS_EC_NO_SUCH_CONTAINER;
			break;
		}
		break;

	case FSE_OT_Directory:
		/*
		 *	If it's a directory, ensure it has
		 *	a trailing delimiter.
		 */
		if (qs->FilePath[qs->FilePathLen-1] != fse->DirDelimiterChr)
		{
			qs->FilePath[qs->FilePathLen] = fse->DirDelimiterChr;
			qs->FilePathLen++;
			qs->FilePath[qs->FilePathLen] = '\0';
		}

		/*
		 *	If the query is on the rootpath,
		 *	allow the CDS to report "." entries.
		 */
		if ((cdsQuery->QueryType == MSA_Query_BrowseMetadata) &&
			(strcmp(qs->FilePath, fse->RootPath) == 0))
		{
			qs->Flags |= FSE_FLAG_ShowDot;
		}

		switch (cdsQuery->QueryType)
		{
		case MSA_Query_BrowseMetadata:
			qs->ProcessIndex = 0;
			qs->ProcessObjectType  = qs->ObjectType;
			strcpy(qs->ProcessPath, qs->FilePath);
			return qs;

		case MSA_Query_BrowseDirectChildren:
			qs->ProcessIndex = 0;
			qs->ProcessObjectType  = qs->ObjectType;
			qs->ProcessDirHandle = FIOA_GetDirFirstFile(qs->FilePath, qs->ProcessPath, MAX_PATH_LENGTH, &(qs->ProcessFileSize));
			return qs;

		case MSA_Query_Search:
			return _FSE_HandleQuery_Search(fse, qs, cdsQuery, filter, cdsErrorCode);
		}
		break;

	default:
		*cdsErrorCode = CDS_EC_ACTION_FAILED;
		break;
	}

	/* return NULL with the nonzero *cdsErrorCode */
	return NULL;
}

int FSE_HandleQueryCount(void *fseObj, struct MSA_CdsQuery *cdsQuery, void *onQueryObj, unsigned int filter)
{
	char	strSQL[MAX_SQL_STR_LENGTH];
	int		parent, count = 0;

	if (cdsQuery->QueryType == MSA_Query_BrowseDirectChildren)
	{
		parent = atoi(cdsQuery->ObjectID);

		sprintf(strSQL,"select * from FSE_QueryState where pIndex = %d ",parent);
		count = db_getCount(strSQL);
	}

	return count;
}


int FSE_HandleProcessObject(void *fseObj, struct MSA_CdsQuery *cdsQuery, void *onQueryObj, unsigned int filter, /*INOUT*/ struct CdsMediaObject *cdsObj)
{
	struct FSE_State *fse = (struct FSE_State*) fseObj;
	struct FSE_QueryState *qs = (struct FSE_QueryState*) onQueryObj;
	struct FSE_PathInfo pi;
	char	strSQL[MAX_SQL_STR_LENGTH],time_str[30];
	unsigned long long	len;
	int		parent;
	
	if ((qs == NULL) || (cdsObj == NULL))
	{
		return FALSE;
	}

	if (cdsQuery->QueryType == MSA_Query_BrowseMetadata)
	{
		sprintf(strSQL,"select * from FSE_QueryState where MyIndex = %d", atoi(cdsQuery->ObjectID));
		if (db_getOneRecord(strSQL, qs)) 
		{
			if (strcmp(cdsQuery->ObjectID,"0") == 0) 
			{
				// root item, there is no parent
				parent = -1;
			}
			else
			{
				parent =  qs->pIndex;
			}
		}
		else
		{
			// Cannot find this item in database
			return FALSE;
		}
	}
	else 
	{
		parent = atoi(cdsQuery->ObjectID);

		// retrive the items whose parent is pIndex
		sprintf(strSQL,"select * from FSE_QueryState where pIndex = %d ",parent);
		if (db_getOneRecordInIndex(strSQL, qs, cdsQuery->CurrentReadIndex))
		{
			// Cannot get this item pointed by the index
			return FALSE;
		}
	}

	/* get info about this path and determine if we're going to expose it */
	memset(&pi, 0, sizeof(struct FSE_PathInfo));
	_FSE_ParsePathInfo(qs->ProcessPath, fse, qs, &pi);

	cdsObj->ID = (char*) MALLOC(10);			
	cdsObj->ParentID = (char*) MALLOC(10);			
	sprintf(cdsObj->ID, "%d\0",qs->Index);
	sprintf(cdsObj->ParentID, "%d\0", parent);

	if (pi.NameNoExtLen > 0)
	{
		cdsObj->Title = (char*) MALLOC(pi.NameNoExtLen+1);
		strcpy(cdsObj->Title, pi.NameNoExt);
	}
	else
	{
		cdsObj->Title = (char*) MALLOC(qs->FilePathLen+1);
		strcpy(cdsObj->Title, qs->FileName);
	}

	if ((len = strlen(qs->Album)) > 0)
	{
		cdsObj->Album = MALLOC(len + 1);
		strcpy(cdsObj->Album, qs->Album);
		cdsObj->DeallocateThese |=	CDS_ALLOC_Album;
	}

	if ((len = strlen(qs->Author)) > 0)
	{
		cdsObj->Creator = MALLOC(len + 1);	
		strcpy(cdsObj->Creator, qs->Author);
		cdsObj->DeallocateThese |=	CDS_ALLOC_Creator;
	}

	if ((len = strlen(qs->Genre)) > 0)
	{
		cdsObj->Genre = MALLOC(len + 1);	
		strcpy(cdsObj->Genre, qs->Genre);
		cdsObj->DeallocateThese |=	CDS_ALLOC_Genre;
	}

	cdsObj->DeallocateThese |= CDS_ALLOC_ID | CDS_ALLOC_ParentID | CDS_ALLOC_Title;

	/*
	 *	Set the media class
	 */
	if (qs->ProcessObjectType == FSE_OT_File)
	{
		cdsObj->MediaClass = FileExtensionToClassCode(pi.Ext, 0);
		cdsObj->Flags |= CDS_OBJPROP_FLAGS_Restricted;

		GetFileCreationTime(qs->DirFilePath, time_str);
		sprintf(cdsObj->CreationTime, "%s\0", time_str);
	}
	else if (qs->ProcessObjectType == FSE_OT_Directory)
	{
		cdsObj->MediaClass = CDS_MEDIACLASS_STORAGEFOLDER;
		cdsObj->Flags |=	CDS_OBJPROP_FLAGS_Searchable;
	}
	else
	{
		/* 
		 *	TODO: If you have other types of classes
		 *	you'll need to modify this section.
		 */
		cdsObj->MediaClass = CDS_MEDIACLASS_CONTAINER;
	}

	/*
	 *	Containers are not searchable in this implementation.
	 */
	if (cdsObj->MediaClass & CDS_MEDIACLASS_CONTAINER)
	{
		/*
		 *	CUSTOMIZE: Set this if container is searchable.
		 *
		 *	cdsObj->Flags |= CDS_OBJPROP_FLAGS_Searchable;
		 */
	}

	/*
	 *	MUSTDO: Track the latest date as a means
	 *	for deriving the updateID for this entry
	 */

	/*
	 *	Track the number of entries we've processed.
	 */
	qs->ProcessIndex++;

	/*
	 *	free temp storage
	 */
	if (pi.Ext != NULL) { free (pi.Ext); pi.Ext = NULL; pi.ExtLen = 0; }
	if (pi.FullPath != NULL) { free (pi.FullPath); pi.FullPath = NULL; pi.FullPathLen = 0; }
	if (pi.Name != NULL) { free (pi.Name); pi.Name = NULL; pi.NameLen = 0; }
	if (pi.NameNoExt != NULL) { free (pi.NameNoExt); pi.NameNoExt = NULL; pi.NameNoExtLen = 0; }
	if (pi.ParentPath != NULL) { free (pi.ParentPath); pi.ParentPath = NULL; pi.ParentPathLen = 0; }
	if (pi.RelativeFullPath != NULL) { free (pi.RelativeFullPath); pi.RelativeFullPath = NULL; pi.RelativeFullPathLen = 0; } 
	if (pi.RelativeParent != NULL) { free (pi.RelativeParent); pi.RelativeParent = NULL; pi.RelativeParentLen = 0; }

	return TRUE;
}

int FSE_HandleProcessObjects(void *fseObj, struct MSA_CdsQuery *cdsQuery, void **onQueryObj, unsigned int filter, /*INOUT*/ struct CdsMediaObject **cdsObj, int count)
{
	struct FSE_State *fse = (struct FSE_State*) fseObj;
	struct FSE_QueryState *qsArray;
	struct FSE_PathInfo pi;
	char	strSQL[MAX_SQL_STR_LENGTH],time_str[30];
	unsigned long long	len;
	int		parent, readCount, i;
	
	if ((onQueryObj == NULL) || (cdsObj == NULL) || (cdsQuery->QueryType != MSA_Query_BrowseDirectChildren))
	{
		return 0;
	}
	
	qsArray = (struct FSE_QueryState*) MALLOC(sizeof(struct FSE_QueryState) * count);
	memset(qsArray, 0, sizeof(struct FSE_QueryState) * count);	
	for (i = 0; i < count; i++)
	{
		onQueryObj[i] = (void *) (qsArray + i);
	}

	parent = atoi(cdsQuery->ObjectID);

	// retrive the items whose parent is pIndex
	sprintf(strSQL,"select * from FSE_QueryState where pIndex = %d ",parent);
	readCount = db_getRecords(strSQL, qsArray, cdsQuery->CurrentReadIndex, count);
	if (readCount == 0)
	{
		// Cannot get this item under this parent index
		return 0;
	}

	for (i = 0; i < readCount; i++)
	{
		/* get info about this path and determine if we're going to expose it */
		memset(&pi, 0, sizeof(struct FSE_PathInfo));
		_FSE_ParsePathInfo(qsArray[i].ProcessPath, fse, &qsArray[i], &pi);

		cdsObj[i]->ID = (char*) MALLOC(10);			
		cdsObj[i]->ParentID = (char*) MALLOC(10);			
		sprintf(cdsObj[i]->ID, "%d\0",qsArray[i].Index);
		sprintf(cdsObj[i]->ParentID, "%d\0", parent);

		if (pi.NameNoExtLen > 0)
		{
			cdsObj[i]->Title = (char*) MALLOC(pi.NameNoExtLen+1);
			strcpy(cdsObj[i]->Title, pi.NameNoExt);
		}
		else
		{
			cdsObj[i]->Title = (char*) MALLOC(qsArray[i].FilePathLen+1);
			strcpy(cdsObj[i]->Title, qsArray[i].FileName);
		}

		if ((len = strlen(qsArray[i].Album)) > 0)
		{
			cdsObj[i]->Album = MALLOC(len + 1);
			strcpy(cdsObj[i]->Album, qsArray[i].Album);
			cdsObj[i]->DeallocateThese |=	CDS_ALLOC_Album;
		}

		if ((len = strlen(qsArray[i].Author)) > 0)
		{
			cdsObj[i]->Creator = MALLOC(len + 1);	
			strcpy(cdsObj[i]->Creator, qsArray[i].Author);
			cdsObj[i]->DeallocateThese |=	CDS_ALLOC_Creator;
		}

		if ((len = strlen(qsArray[i].Genre)) > 0)
		{
			cdsObj[i]->Genre = MALLOC(len + 1);	
			strcpy(cdsObj[i]->Genre, qsArray[i].Genre);
			cdsObj[i]->DeallocateThese |=	CDS_ALLOC_Genre;
		}

		cdsObj[i]->DeallocateThese |= CDS_ALLOC_ID | CDS_ALLOC_ParentID | CDS_ALLOC_Title;

		/*
		 *	Set the media class
		 */
		if (qsArray[i].ProcessObjectType == FSE_OT_File)
		{
			//phillo,2007/10/22
			// don't parse as directory if internet stream
			if( qsArray[i].pIndex == 17/*Internet audio*/ || qsArray[i].pIndex == 16 /*Inernet video*/ )
			{
				cdsObj[i]->MediaClass = CDS_MEDIACLASS_ITEM; // CDS_CLASS_MASK_MAJOR_PLAYLISTITEM;
			}
			else
			{
				cdsObj[i]->MediaClass = FileExtensionToClassCode(pi.Ext, 0);
			}
			cdsObj[i]->Flags |= CDS_OBJPROP_FLAGS_Restricted;

			GetFileCreationTime(qsArray[i].DirFilePath, time_str);
			sprintf(cdsObj[i]->CreationTime, "%s\0", time_str);
		}
		else if (qsArray[i].ProcessObjectType == FSE_OT_Directory)
		{
			cdsObj[i]->MediaClass = CDS_MEDIACLASS_STORAGEFOLDER;
			cdsObj[i]->Flags |=	CDS_OBJPROP_FLAGS_Searchable;
		}

		qsArray[i].ProcessIndex++;
	
		// free temp storage
		if (pi.Ext != NULL) { free (pi.Ext); pi.Ext = NULL; pi.ExtLen = 0; }
		if (pi.FullPath != NULL) { free (pi.FullPath); pi.FullPath = NULL; pi.FullPathLen = 0; }
		if (pi.Name != NULL) { free (pi.Name); pi.Name = NULL; pi.NameLen = 0; }
		if (pi.NameNoExt != NULL) { free (pi.NameNoExt); pi.NameNoExt = NULL; pi.NameNoExtLen = 0; }
		if (pi.ParentPath != NULL) { free (pi.ParentPath); pi.ParentPath = NULL; pi.ParentPathLen = 0; }
		if (pi.RelativeFullPath != NULL) { free (pi.RelativeFullPath); pi.RelativeFullPath = NULL; pi.RelativeFullPathLen = 0; } 
		if (pi.RelativeParent != NULL) { free (pi.RelativeParent); pi.RelativeParent = NULL; pi.RelativeParentLen = 0; }
	}

	return readCount;
}

void _FSE_DestroyFSE(void *fseObj)
{
	struct FSE_State* fse = (struct FSE_State*) fseObj;

	FREE(fse->RootPath);
}

void* FSE_InitFSEState(void *chain, const char *rootDir)
{
	struct FSE_State* fse = (struct FSE_State*) malloc (sizeof(struct FSE_State));
	int i;

	memset(fse, 0, sizeof(struct FSE_State));

	fse->Destroy = _FSE_DestroyFSE;
	fse->RootPath = (char*)MALLOC((int)strlen(rootDir) + 2);	/* add 2 extra bytes for delimiter and null */
	strcpy(fse->RootPath, rootDir);
	fse->RootLen = (int) strlen(fse->RootPath);
	fse->DirDelimiter = NULL;

	/*
	 *	Set fse->DirDelimiter
	 */
	for (i = 0; i < fse->RootLen; i++)
	{
		if (FSE_BACKSLASH_CHR == fse->RootPath[i])
		{
			fse->DirDelimiter = FSE_BACKSLASH_STR;
			fse->DirDelimiterChr = FSE_BACKSLASH_CHR;
			fse->BadDirDelimiterChr = FSE_FORSLASH_CHR;
		}
		else if (FSE_FORSLASH_CHR == fse->RootPath[i])
		{
			fse->DirDelimiter = FSE_FORSLASH_STR;
			fse->DirDelimiterChr = FSE_FORSLASH_CHR;
			fse->BadDirDelimiterChr = FSE_BACKSLASH_CHR;
		}
	}
	if (fse->DirDelimiter == NULL)
	{
		fse->DirDelimiter = FSE_FORSLASH_STR;
		fse->DirDelimiterChr = FSE_FORSLASH_CHR;
		fse->BadDirDelimiterChr = FSE_BACKSLASH_CHR;
	}

	#ifdef WIN32
	fse->DirDelimiter = FSE_BACKSLASH_STR;
	fse->DirDelimiterChr = FSE_BACKSLASH_CHR;
	fse->BadDirDelimiterChr = FSE_FORSLASH_CHR;
	#endif

	/*
	 *	Ensure all directory delimiters are consistent.
	 */
	strReplaceWith(fse->RootPath, fse->BadDirDelimiterChr, fse->DirDelimiterChr);

	/*
	 *	Appropriately acquire the root path with a trailing delimiter.
	 */
	if (fse->RootPath[fse->RootLen-1] != fse->DirDelimiterChr)
	{
		/* no traling delimiter, so go ahead and add one */
		fse->RootPath[fse->RootLen] = fse->DirDelimiterChr;
		fse->RootLen++;
		fse->RootPath[fse->RootLen] = '\0';
	}

	ILibAddToChain(chain, fse);

	return fse;
}

unsigned int FSE_HandleOnAcquireUpdateIDAndCleanup(void *fseObj, struct MSA_CdsQuery *cdsQuery, void *onQueryObj)
{
	struct FSE_QueryState *qs = (struct FSE_QueryState*) onQueryObj;
	unsigned int retVal = qs->ProcessUpdateID;

	/*
	 *	We're done, so clean up our qs object and return an updateID.
	 */

	if (qs->ProcessDirHandle != NULL)
	{
		// albert check, may has problem here , 20050329
		FIOA_CloseDir(qs->ProcessDirHandle);
	}

	FREE(qs);

	return retVal;
}

void FSE_GetResourceMetadata(void *fseObj, void *onQueryObj, /*INOUT*/ struct FSE_ResourceMetadata *res)
{
	struct FSE_State* fse = (struct FSE_State*) fseObj;
	struct FSE_QueryState *qs = (struct FSE_QueryState*) onQueryObj;
	int dotPos, slashPos;
	
	/*
	 *	Return the fullpath name and the file size.
	 */
	if (res != NULL)
	{                                                                                                             
		memset(res, 0, sizeof(struct FSE_ResourceMetadata));

		res->LocalFilePathLen = (int) strlen(qs->ProcessPath);
		res->LocalFilePath = (char*) malloc (res->LocalFilePathLen + 1);
		memcpy(res->LocalFilePath, qs->ProcessPath, res->LocalFilePathLen + 1);
		res->FileLength = qs->ProcessFileSize;

		slashPos = LastIndexOf(res->LocalFilePath, fse->DirDelimiter);
		dotPos = LastIndexOf(res->LocalFilePath, ".");

		if (qs->ProcessObjectType == FSE_OT_File)
		{
			res->Flags |= FSE_RMF_IsFile;
			if ((dotPos > 0) && (dotPos > slashPos))
			{
				res->FileExtensionLen = res->LocalFilePathLen - dotPos;
				res->FileExtension = (char*) MALLOC(res->FileExtensionLen + 1);
				memcpy(res->FileExtension, res->LocalFilePath + dotPos, res->FileExtensionLen + 1);
			}

			if (strcmpi(res->FileExtension, EXTENSION_PLAYLIST_M3U) == 0)
			{
				res->Flags |= FSE_RMF_IsPlaylist;
			}
		}
		else
		{
			res->Flags |= FSE_RMF_IsDirectory;
		}
	}
}

//birdy 2007/10/17 add for internet file
void FSE_GetResourceMetadata_InternetFile(void *fseObj, void *onQueryObj, /*INOUT*/ struct FSE_ResourceMetadata *res)
{
	struct FSE_State* fse = (struct FSE_State*) fseObj;
	struct FSE_QueryState *qs = (struct FSE_QueryState*) onQueryObj;
	int dotPos, slashPos;
	
	/*
	 *	Return the fullpath name and the file size.
	 */
	if (res != NULL)
	{
		memset(res, 0, sizeof(struct FSE_ResourceMetadata));

		res->LocalFilePathLen = (int) strlen(qs->ProcessPath);
		res->LocalFilePath = (char*) malloc (res->LocalFilePathLen + 1);
		memcpy(res->LocalFilePath, qs->ProcessPath, res->LocalFilePathLen + 1);
		res->FileLength = qs->ProcessFileSize;

		slashPos = LastIndexOf(res->LocalFilePath, fse->DirDelimiter);
		dotPos = LastIndexOf(res->LocalFilePath, ".");

		if (qs->ProcessObjectType == FSE_OT_File)
		{
			res->Flags |= FSE_RMF_IsFile;
			//if ((dotPos > 0) && (dotPos > slashPos))
			//{
				res->FileExtensionLen = 0;//res->LocalFilePathLen - dotPos;
				res->FileExtension = 0;//(char*) MALLOC(res->FileExtensionLen + 1);
				//memcpy(res->FileExtension, res->LocalFilePath + dotPos, res->FileExtensionLen + 1);
			//}

			/*if (strcmpi(res->FileExtension, EXTENSION_PLAYLIST_M3U) == 0)
			{
				res->Flags |= FSE_RMF_IsPlaylist;
			}*/
		}
		else
		{
			res->Flags |= FSE_RMF_IsDirectory;
		}
	}
}
