#ifndef _FILESYSTEM_TREE_H
#define _FILESYSTEM_TREE_H

#include	"FileIoAbstraction.h"

#define	MAX_MEDIA_STR_LENGTH	1024
#define	MAX_EXT_STR_LENGTH		10

/* slash and backslash */
#define FSE_BACKSLASH_CHR '\\'
#define FSE_BACKSLASH_STR "\\"
#define FSE_FORSLASH_CHR '/'
#define FSE_FORSLASH_STR "/"

#define FILE_TYPE_NONE				0x0000
#define FILE_TYPE_VIDEO				0x0001
#define FILE_TYPE_AUDIO				0x0002
#define FILE_TYPE_PHOTO				0x0004
#define FILE_TYPE_PLAYLIST			0x0008
#define FILE_TYPE_SUBTITLE			0x0010
#define FILE_TYPE_FILE_IN_PLAYLIST	0x0020
#define FILE_TYPE_ISO				0x0040

// Flags used in field "Flags"
#define FLAGS_FOLDER			0x0001
#define FLAGS_ALBUM				0x0002
#define FLAGS_ARTIST			0x0004
#define FLAGS_GENRE				0x0008
#define FLAGS_PLAYLIST			0x0010
#define FLAGS_FILE_IN_PLAYLIST	0x0020
#define FLAGS_VIDEOINDEX		0x0040
#define FLAGS_FORCETRANSCODING  0x0080
#define FLAGS_ISO				0x0100

#define FLAGS_UNPLAYABLE        0x0100

extern int qsCount;
extern int audioCount;
extern int videoCount;
extern int imageCount;
extern int filterCount;
extern int playlistCount;
extern int rejectCount;
extern int totalFileCount;

extern int scan_audioCount;
extern int scan_videoCount;
extern int scan_imageCount;
extern int scan_filterCount;
extern int scan_playlistCount;
extern int scan_rejectCount;
extern int scan_skipCount;
extern int scan_totalFileCount;

extern int addFSE(int parent, int FOType, const TCHAR *Path, const TCHAR *Filename, DWORD dwFileFilterTypes);
//extern int addFSE_FilesOnly(int parent, int FOType, const TCHAR *Path, const TCHAR *Filename, DWORD dwFileFilterTypes);
//birdy add fot internet file
extern int addFSE_FilesOnly(int parent, int FOType, const TCHAR *Path, const TCHAR *Filename, DWORD dwFileFilterTypes, int InternetFile);
extern void initFSE_QS();
extern struct FSE_QueryState *testFSE_QS;
extern struct FSE_QueryState *testFSE_QS_For_Browse;
extern int maxItemsInQS, maxItemsInQSForBrowse;

#define NEW_ITEMS_NUM_IN_QS		1000


enum FSE_Flags
{
	FSE_FLAG_Done			= 0x00000001,
	FSE_FLAG_ShowDot		= 0x00000002
};

/*
*	Must match values returned by FIOA_GetFileDirType()
*/
enum FSE_ObjectType
{
	FSE_OT_NoExist =		0x00,
	FSE_OT_File =			0x01,
	FSE_OT_Directory =		0x02,
	FSE_OT_Video_Stream =	0x04,
	FSE_OT_Audio_Stream =	0x08
};

#define FSE_OT_Directory_and_File_Mask (FSE_OT_File|FSE_OT_Directory)


//tyhsu modify
//struct FSE_QueryState
struct MediaInfo_str
{
	TCHAR	Title[MAX_MEDIA_STR_LENGTH];
	TCHAR	AlbumTitle[MAX_MEDIA_STR_LENGTH];
	TCHAR	Author[MAX_MEDIA_STR_LENGTH];
	TCHAR	Genre[MAX_MEDIA_STR_LENGTH];
	TCHAR	Duration[128];
	TCHAR	Bitrate[128];
	TCHAR	Track[128];
	TCHAR	FileType[MAX_EXT_STR_LENGTH];
	
	int		ResolutionX;
	int		ResolutionY;
	char	create_date[100];
	char	StreamTypeInfo[1024];
    unsigned long Flags;
};

struct FSE_QueryState
{
	int pIndex,Index;
	unsigned int Flags;
	unsigned int InternetFlags;   //birdy add 2007/11/2 
	char CDCover[MAX_PATH_LENGTH]; //birdy add for CD_Cover 2007/11/17
	enum FSE_ObjectType ObjectType;
	char FilePath[MAX_PATH_LENGTH];
	
	char DirFilePath[MAX_PATH_LENGTH];
	char MediaFilePath[MAX_PATH_LENGTH];
	char FileName[MAX_PATH_LENGTH]; //old
	//CString InternetFileName;
	TCHAR InternetFileName[MAX_PATH_LENGTH];  //birdy 2008/2/21 add for fix title show right in the Internet media page
	TCHAR MKVFilePath[MAX_PATH_LENGTH];  //birdy 2008/6/12 add for MKV file path
	
	unsigned int FilePathLen;
	unsigned int DirFilePathLen;
	unsigned int MediaFilePathLen;
	unsigned int FileNameLen;

	int isSearchable;

	/*
	*	TODO: Memory optimization can be achieved
	*	by using a char* instead of a large array.
	*/
	//MAX_PATH_LENGTH
	char ProcessPath[MAX_PATH_LENGTH];
	unsigned int ProcessIndex;
	enum FSE_ObjectType ProcessObjectType;
	void *ProcessDirHandle;
	int ProcessFileSize;
	unsigned int ProcessUpdateID;

	// media info , by albert
	char Album[MAX_MEDIA_STR_LENGTH];
	char Title[MAX_MEDIA_STR_LENGTH];
	char Author[MAX_MEDIA_STR_LENGTH];
	char Genre[MAX_MEDIA_STR_LENGTH];
	int TrackNumber;
	int Duration;				/* the duration, in number of seconds. Negative value means value is not set.*/
	int Bitrate;				/* the bitrate of the resource. If negative, treat as an unset value. */
	long Size;					/* the file size of the resource. If negative, treat as an unset value. */
	char FileType[MAX_EXT_STR_LENGTH];
};
typedef struct FSE_QueryState	DBQueryState;

//typedef struct FSE_QueryState _FSE_QueryState;

struct FSE_State
{
	/*
	*	The PreSelect, PostSelect, and Destroy fields must
	*	remain here as the object needs to be compatible with the
	*	thread-chaining framework.
	*/
	void (*PreSelect)(void* object,fd_set *readset, fd_set *writeset, fd_set *errorset, int* blocktime);
	void (*PostSelect)(void* object,int slct, fd_set *readset, fd_set *writeset, fd_set *errorset);
	void (*Destroy)(void* object);

	char *RootPath;
	int RootLen;
	char *DirDelimiter;
	char DirDelimiterChr;
	char BadDirDelimiterChr;
};

struct FSE_PathInfo
{
	char *FullPath;
	int FullPathLen;

	char *Name;
	int NameLen;

	char *NameNoExt;
	int NameNoExtLen;

	char *Ext;
	int ExtLen;

	char *ParentPath;
	int ParentPathLen;

	char *RelativeParent;
	int RelativeParentLen;

	char *RelativeFullPath;
	int RelativeFullPathLen;
};


//tyhsu add


#endif
/* declarations */