#ifdef WIN32
#define _CRTDBG_MAP_ALLOC
#include <stdlib.h>
#ifndef _WIN32_WCE
#include <crtdbg.h>
#endif
#else
#include <stdlib.h>
#endif

#include <string.h>
#include "TransferState.h"
#include "ILibAsyncSocket.h"
#include "CommonHttp.h"
#include "FileIoAbstraction.h"
#include "TransLibWarp.h"
#include "PreCache.h"
#include "MyString.h"
#include "debug.h"

#ifdef _POSIX
#define stricmp strcasecmp
#define strcmpi strcasecmp
#define strnicmp strncasecmp
#endif

struct TFS_Internal
{
	struct ILibWebServer_Session *Session;
};

void TS_AddRange(struct packetheader_field_node *phf, unsigned long long vrStart, unsigned long long vrLen, /*INOUT*/ struct TS_TransferState *ts)
{
	struct parser_result *pr,*pr2,*pr3;
	struct parser_result_field *prf;
	struct TS_Range *tsr;
	unsigned long long i;
	int j, check_format = 1;


	// albert add
	for(j=0;j<phf->FieldDataLength;j++)
	{
		if(phf->FieldData[j]=='-')
			check_format = 0;
	}

	// albert add
	if(check_format == 1)
	{
		ts->Flags |= TS_F_BadRequest ;//| TS_F_InvalidFormat ;
		//ts->Flags |= TS_F_InvalidFormat ;
		return;
	}


	/* parse byte ranges */
	pr = ILibParseString(phf->FieldData,0,phf->FieldDataLength,"=",1);
	pr2 = ILibParseString(pr->LastResult->data,0,pr->LastResult->datalength,",",1);
	prf = pr2->FirstResult;

	/* note whether this is a multi-part range request */
	ts->Flags |= (pr2->NumResults==1) ? (TS_F_Range) : (TS_F_Range | TS_F_MultipleRange);

	while(prf!=NULL)
	{
		/*
		 *	Allocate a TS_Range object to track the progress
		 *	when transmitting each range.
		 */
		tsr = (struct TS_Range*)MALLOC(sizeof(struct TS_Range));
		tsr->BytesSent = 0;
		pr3 = ILibParseString(prf->data,0,prf->datalength,"-",1);

		/*
		 *	Determine index of first requested byte, if possible.
		 */
		if(pr3->FirstResult->datalength==0)
		{
			/*
			 *	Not yet possible to determine first byte.
			 */
			tsr->StartIndex = -1;
		}
		else
		{
			/*
			 *	Parse the index of the first byte.
			 *	It's OK to null-terminate the string since
			 *	we'll destroy the parser results later.
			 */
			pr3->FirstResult->data[pr3->FirstResult->datalength] = '\0';
			tsr->StartIndex = atoi(pr3->FirstResult->data);
		}

		/*
		 *	parse the index of the last byte, if possible.
4		 */
		if(pr3->LastResult->datalength==0)
		{
			/*
			 *	Not yet possible to determine the last byte index.
			 */
			tsr->BytesLeft = vrLen - tsr->StartIndex;
		}
		else
		{
			/*
		 	 *	We're able to determine the index of the last byte.
			 *	Null terminate the string so we can do an atoi().
			 *	It's OK to do this because we will destroy the
			 *	parser results later.
			 */

			pr3->LastResult->data[pr3->LastResult->datalength] = '\0';

			if(tsr->StartIndex==-1)
			{
				/*
				 *	We don't know the index of the first byte
				 *	so we'll set it to be the zero-byte of the file.
				 *
				 *	Calculate the total number of bytes left
			 	 *	and then adjust the other values.
				 */

				tsr->BytesLeft = atoi(pr3->LastResult->data);

				if(tsr->BytesLeft >= vrLen) 
				{
					/* send the entire file */
					tsr->BytesLeft = vrLen;
					tsr->StartIndex = 0;
				}
				else
				{
					/* send the last 'tsr->BytesLeft' bytes */
					tsr->StartIndex = vrLen - tsr->BytesLeft;
				}
			}
			else
			{
				/*
				 *	Parse the last index and calculate
				 *	the number of bytes left.
				 */

				// 7.8.22.6
				// tsr->BytesLeft = atoi(pr3->LastResult->data) ;
				char tmpstr[36];

				if (tsr->StartIndex > (unsigned long long) atoi(pr3->LastResult->data))
				{
					/* invalid range */
					ts->Flags |= TS_F_InvalidRange;
					break;
				}

				sprintf(tmpstr,"%d", vrLen);
				if(strlen(pr3->LastResult->data) > strlen(tmpstr))
				{
					ts->Flags |= TS_F_BadRequest;

				}	
				else
				{
					
					tsr->BytesLeft = atoi(pr3->LastResult->data) - tsr->StartIndex + 1;
					i = (vrLen - tsr->StartIndex);
					if ((i >= 0) && (tsr->BytesLeft > (unsigned int)i))
					{
						tsr->BytesLeft = i;
					}
				}
			}
		}


		//albert modify , 
		if (tsr->StartIndex < vrStart)
		{
			/* invalid range */
			ts->Flags |= TS_F_InvalidRange;
			break;
		}
		else if (tsr->StartIndex > vrLen)
		{
			// for case 7.8.22.6
			ts->Flags |= TS_F_InvalidFormat;

			//TS_F_BadRequest ;//| TS_F_InvalidFormat ;
			break;
		}
		else
		{
			/*
			if (pr3->LastResult->data != NULL )
			{
				if( ((unsigned int)atoi(pr3->LastResult->data) < tsr->StartIndex) )
				{
					// invalid range 
					ts->Flags |= TS_F_InvalidRange;
					break;
				}
			}
			*/

			/*
			 *	We have the range, so put it in the queue 
			 *	and destroy the parser results.
			 */
			ts->TotalBytesToSend += tsr->BytesLeft;
			
			ILibQueue_Lock(ts->RangeQueue);
			ILibQueue_EnQueue(ts->RangeQueue, tsr);
			ILibQueue_UnLock(ts->RangeQueue);
			
			ILibDestructParserResults(pr3);
			prf = prf->NextResult;
		}
	}

	/* destroy the last of the parser results */
	ILibDestructParserResults(pr2);
	ILibDestructParserResults(pr);
}



void TS_DestroyTS(struct TS_TransferState *ts)
{
	struct TS_Range *tsr;

	if ((ts->Flags & TS_F_AllocContentFeatures) && (ts->ContentFeatures != NULL))
	{
		FREE(ts->ContentFeatures);
	}

	if ((ts->Flags & TS_F_AllocContentType) && (ts->ContentType != NULL))
	{
		FREE(ts->ContentType);
	}

	if ((ts->Flags & TS_F_OpenFdes_aio_fildes) && (ts->FileAiocb.aio_fildes != NULL))
	{
        if (ts->FileAiocb.isTranscoding)
        {
            CloseTranscodingFile(ts->FileAiocb.aio_fildes);
        }
		else
        {
            waio_close(ts->FileAiocb.aio_fildes);
        }

		if (ts->FileAiocb.aio_sync_fildes)
		{
			FIOA_FileClose(ts->FileAiocb.aio_sync_fildes);
		}
	}

	if ((ts->Flags & TS_F_AllocFdes_aio_buf) && (ts->FileAiocb.aio_buf != NULL))
	{
		free (ts->FileAiocb.aio_buf);
	}

	if (ts->RangeQueue != NULL)
	{
		/* deallocate everything in the queue */
		ILibQueue_Lock(ts->RangeQueue);
		while (! ILibQueue_IsEmpty(ts->RangeQueue))
		{
			tsr = ILibQueue_DeQueue(ts->RangeQueue);
			free (tsr);
		}
		ILibQueue_UnLock(ts->RangeQueue);
		ILibQueue_Destroy(ts->RangeQueue);
	}

	if ((ts->Flags & TS_F_AllocTarget) && (ts->Target != NULL))
	{
		FREE(ts->Target);
	}

	if (ts->MultipartSeparator != NULL)
	{
		FREE(ts->MultipartSeparator);
	}

	if (ts->Session)
	{
		//Birdy 2007/3/23 fix segmentation fault
		//ILibWebServer_Release(ts->Session);
	}

	//sem_wait(&(ts->RefCountLock));
	sem_post(&(ts->RefCountLock));
	sem_destroy(&(ts->RefCountLock));

	free (ts);

	UnInitPreCacheBuffer();
}

void TS_PrepareForHttp(struct ILibWebServer_Session *session, const struct packetheader *header, unsigned long long bufferSize, const char *contentType, const char *fourthField, TS_Callback_OnDestroy_TransferState onDestroyTs, /*INOUT*/struct TS_TransferState *ts, void* ignoreOrWAIO_Complete)
{
	memset(ts,0, sizeof(struct TS_TransferState));

	/* set callback */
	ts->Callback_OnDestroy = onDestroyTs;

	ts->ContentFeatures = MakeStrcpy(fourthField);
	ts->Flags |= TS_F_AllocContentFeatures;

	/* initialize ContentType */
	if (contentType != NULL)
	{
		ts->ContentTypeLen = (int) strlen(contentType);
		if (ts->ContentTypeLen >= 0)
		{
			ts->ContentType = (char*) malloc (ts->ContentTypeLen + 1);
			memcpy(ts->ContentType, contentType, ts->ContentTypeLen);
			ts->ContentType[ts->ContentTypeLen] = '\0';
			ts->Flags |= TS_F_AllocContentType;
		}
		else
		{
			ts->ContentTypeLen = 0;
		}
	}
	/* initialize file aiocb fields */
	if (bufferSize > 0)
	{
		ts->BufferSize = bufferSize;
		memset(&(ts->FileAiocb), 0, sizeof(struct aiocb));
		ts->FileAiocb.aio_buf = MALLOC(ts->BufferSize);
 
		/*
		 *	ignore the compiler warning because ts->BufferSize 
		 *	won't have a bit width greater than a DWORD
		 */
		ts->FileAiocb.aio_nbytes = (unsigned long long) ts->BufferSize;
		ts->Flags |= TS_F_AllocFdes_aio_buf;
	}
	else
	{
		ts->BufferSize = 0;
		memset(&(ts->FileAiocb), 0, sizeof(struct aiocb));
	}

	/* set up a callback */
	/* ignored but it aligns with some unix variants that support sigevent.sigev_notify */
	ts->FileAiocb.aio_sigevent.sigev_notify = SIGEV_CALLBACK;
	ts->FileAiocb.aio_sigevent.sigev_func = ignoreOrWAIO_Complete;
	ts->FileAiocb.aio_sigevent.sigev_value.sival_ptr = ts;

	//
	//	Included for posterity when I was trying to provide
	//	alignment between WinAio and Posix AIO.
	//
//#ifdef _POSIX
//	/* use a signal handler */
//	ts->FileAiocb.aio_sigevent.sigev_notify = SIGEV_SIGNAL;
//	ts->FileAiocb.aio_sigevent.sigev_signo = SIGUSR1;
//
//	/* set up signal handler */
//	sigemptyset(&(ts->SigAction.sa_mask));
//	ts->SigAction.sa_flags = SA_SIGINFO;
//	ts->SigAction.sa_handler = ignoreOrWAIO_Complete;
//	sigaction(SIGUSR1, &(ts->SigAction), 0);
//#endif

	/* create an empty queue and copy the session object */
	ts->RangeQueue = ILibQueue_Create();

	// Make sure that seesion won't be released before ts is released
	ILibWebServer_AddRef(session);
	ts->Session = session;
	
	/*
	 *	copy Target 
	 */
	if (header->DirectiveLength >= 0)
	{
		ts->TargetLen = header->DirectiveObjLength;
		ts->Target = (char*) MALLOC(ts->TargetLen+1);
		memcpy(ts->Target, header->DirectiveObj, ts->TargetLen);
		ts->Target[ts->TargetLen] = '\0';
		ts->Flags |= TS_F_AllocTarget;
	}

	/*
	 *	Determine method and set flags
	 */
	if (stricmp(header->Directive, HTTP_STR_GET) == 0)
	{
		ts->Flags |= TS_F_MethodGet;
	}
	else if (stricmp(header->Directive, HTTP_STR_HEAD) == 0)
	{
		ts->Flags |= TS_F_MethodHead;
	}
	else
	{
		ts->Flags |= TS_F_MethodNotImplemented;
	}

	sem_init(&(ts->RefCountLock), 0, 1);
}


// albert modify, set httpheader
void TS_ParseHttpHeaders(struct packetheader_field_node *phf, unsigned long long vrStart, unsigned long long vrLen, /*INOUT*/struct TS_TransferState *ts)
{
	struct TS_Range *tsr;

	while (phf != NULL)
	{
		if (
			(phf->FieldLength == HTTP_STR_RANGE_LEN) && 
			(stricmp(phf->Field, HTTP_STR_RANGE) == 0)
			)
		{
			/*
			 *	Don't worry about the signed/unsigned cast 
			 *	because we already did a non-negative check on fileSize.
			 */
			TS_AddRange(phf, vrStart, vrLen, ts);
		}
		else if (
			(phf->FieldLength == HTTP_STR_DLNA_GETCONTENTFEATURES_LEN) && 
			(stricmp(phf->Field, HTTP_STR_DLNA_GETCONTENTFEATURES) == 0)
			)
		{
			/*
			 *	TODO: You can optimize this by replacing this block
			 *	with ts->Flags |= TS_F_ContentFeatures. 
			 *	It's legal in DHWG to always return the header,
			 *	although some argue that it's best to only return
			 *	the HTTP header if the client asks for it.
			 */

			if (phf->FieldDataLength > 0)
			{
				if (memcmp(
					phf->FieldData, 
					HTTP_STR_DLNA_GETCONTENTFEATURES_VAL, 
					HTTP_STR_DLNA_GETCONTENTFEATURES_VAL_LEN
					) == 0)
				{
					ts->Flags |= TS_F_ContentFeatures;
				}
				else
				{
					ts->Flags |= TS_F_BadRequest;
					break;
				}
			}
			else
			{
				ts->Flags |= TS_F_BadRequest;
				break;
			}
		}

		/*
		 *	TODO: If you want to support additional features like 
		 *	trick modes, you'll want to parse for additional headers.
		 */

		phf = phf->NextField;
	}
 
	/*
	 *	Make sure we have a TS_Range.
	 */
	ILibQueue_Lock(ts->RangeQueue);
	if (ILibQueue_IsEmpty(ts->RangeQueue))
	{
		tsr = (struct TS_Range*)MALLOC(sizeof(struct TS_Range));
		tsr->BytesLeft = vrLen;
		tsr->BytesSent = tsr->StartIndex = vrStart;
		ILibQueue_EnQueue(ts->RangeQueue, tsr);
	}
	ILibQueue_UnLock(ts->RangeQueue);
}

void TS_RefInc(/*INOUT*/ struct TS_TransferState *ts)
{
	sem_wait(&(ts->RefCountLock));
	ts->RefCount++;
	sem_post(&(ts->RefCountLock));
}

void TS_RefDec(/*INOUT*/ struct TS_TransferState *ts)
{
	sem_wait(&(ts->RefCountLock));
	ts->RefCount--;
	if (ts->RefCount <= 0)
	{
		TS_DestroyTS(ts);
	}
	//Birdy 2007/3/22 fix segmentation fault
	else
	{
        sem_post(&(ts->RefCountLock));
	}
}

