#ifdef _POSIX
#define stricmp strcasecmp
#define strcmpi strcasecmp
#define strnicmp strncasecmp
#endif

#ifdef WIN32
#define _CRTDBG_MAP_ALLOC
#include <stdlib.h>
#ifndef _WIN32_WCE
#include <crtdbg.h>
#endif
#else
#include <stdlib.h>
#endif

#include <string.h>
#include <stdio.h>

#include "ICC_Integration.h"
#include "ILibAsyncSocket.h"
#include "ILibParsers.h"
#include "UpnpMicroStack.h"
#include "MediaServerAbstraction.h"
#include "MediaServerLogic.h"
#include "MyString.h"
#include "debug.h"
#include "MyThread.h"
#include "CdsObjectToDidl.h"
#include "FileSystemEnumerator.h"
#include "ResourceProvider_LocalFileHttp.h"
//#include "ResourceProvider_HttpPlaylist.h" /* DEPRECATED: DLNA will not be using M3U files. */
#include "MimeTypes.h"
#include "ILibWebServer.h"
#include "WinAio.h"
#include "TransferState.h"
#include "FileIoAbstraction.h"
#include "CommonHttp.h"

#ifdef ICC_DATABASE
//include .h files specific to ICC_DATABASE
#include "DatabaseIntegration.h"
#endif
#define ICCI_RP_LOCALFILE 0
#define ICCI_RP_LOCALPLAYLIST 1

#ifdef ICC_DATABASE
#ifdef ICC_FILESYSTEM
#error	"ICC_DATABASE and ICC_FILESYSTEM are mutually exclusive preprocessor definitions"
#endif
#endif

#ifdef OEM_ASUS
// In "DMSService.h", it is used to wait up ASUS NB
void	WaitUpSystem();
#endif

#include "FileSystemTree.h"
extern void GetMACaddress(char *xxx);

// In dbtools.h
extern int db_getAVP(const char *Attribute,char *returnValue);
extern int db_getOneRecord (const char *SQLstring,struct FSE_QueryState *tmpFSE_QS);

// In Utility.h
extern void GetImageInfo(char *BMP_filename, int *sourceWidth, int *sourceHeight);
extern void GetFileCreationTime(char *filepath, char *time_string);

char	globalServerName[255*3] = "";
char	utf8WorkingDir[MAX_PATH_LENGTH];
TCHAR	gWorkingDir[MAX_PATH_LENGTH];
extern int IsDMSRunning();

//Birdy 2007/3/6
void WAIO_StopLock(void);
void WAIO_StopUnlock(void);

#ifndef WINSOCK2
void _ICCI_IPAddressMonitor(void *icci_state)
{
	struct ICCI_State *state = (struct ICCI_State*) icci_state;
	int length;
	int *list;
	
	if (icci_state == NULL) return;

	sem_wait(&(state->IPAddressLock));

	// get the list of IP addresses 
	length = ILibGetLocalIPAddressList(&list);

	if(length!=state->IPAddressLength || memcmp((void*)list,(void*)state->IPAddressList,sizeof(int)*length)!=0)
	{
		// tells microstack to readvertise on new interfaces
		UpnpIPAddressListChanged(state->Upnp);
		
		// Get pointer to the the IP address list.
		// Be sure to delete the old memory.
		FREE(state->IPAddressList);
		state->IPAddressList = list;
		state->IPAddressLength = length;

		// Tell MSA to update its list of IP addresses
		MSA_UpdateIpInfo(state->Msa, state->IPAddressList, state->IPAddressLength);
	}
	else
	{
		FREE(list);
	}
	
	sem_post(&(state->IPAddressLock));

	// reregister for the event
	ILibLifeTime_Add(state->Ltm,NULL,4,_ICCI_IPAddressMonitor,NULL);
}
#endif

#ifdef WINSOCK2
void* TheIcciState = NULL;
void CALLBACK _ICCI_IPAddressMonitor
(
IN DWORD dwError, 
IN DWORD cbTransferred, 
IN LPWSAOVERLAPPED lpOverlapped, 
IN DWORD dwFlags 
)
{
	struct ICCI_State *state = (struct ICCI_State*) TheIcciState;
	int length;
	int *list;
	if (state == NULL) return;
	sem_wait(&(state->IPAddressLock));

	// tell the microstack to change its advertisements to match the new IP address list
	UpnpIPAddressListChanged(state->Upnp);

	// get the list of IP addresses
	length = ILibGetLocalIPAddressList(&list);
	
	// free the existing list
	// and assign the new list
	FREE(state->IPAddressList);
	state->IPAddressList = list;
	state->IPAddressLength = length;

	// Tell MediaServerLogic to update its list of IP addresses
	MSA_UpdateIpInfo(state->Msa, state->IPAddressList, state->IPAddressLength);
	
	sem_post(&(state->IPAddressLock));

	// reregister the event
	WSAIoctl(state->MonitorSocket,SIO_ADDRESS_LIST_CHANGE,NULL,0,NULL,0,&(state->MonitorSocketReserved),&(state->MonitorSocketStateObject),_ICCI_IPAddressMonitor);
}
#endif

void _ICCI_HandleTSCallback_OnDestroy(struct TS_TransferState *ts)
{
	/*
	 *	TODO: If you save TS_TransferState objects
	 *	to allow a GUI to report the progress of
	 *	a file transfer, then implement this method and
	 *	remove those TS_TransferState when they
	 *	are about to be deleted. It goes without
	 *	saying that you WILL need to synchronize
	 *	access to the list of TS_TransferState objects
	 *	as well as to individual items in the list.
	 */
	//phillo,2005/6/13
	// try to free ts to fix memory leak issue
	//
	if( ts == 0 )
	{
		return ;
	}

	if( ts->Target )
	{
		FREE(ts->Target);
		ts->Target = 0;
	}

	//Birdy 2007/3/7
	WAIO_StopLock();
	if( ts->FileAiocb.aio_buf )
	{
		FREE(ts->FileAiocb.aio_buf);
		ts->FileAiocb.aio_buf = 0;
	}
	//Birdy 2007/3/7
    WAIO_StopUnlock();

	// can't free ts, let TS_RefDec() to do it
	//FREE( ts );
}

/*
 *	Dispatch point for MSA callback.
 *	Method executes when MSA wants to handle a CDS query.
 */
void _ICCI_HandleMsaCallback_OnQuery (void *msaObj, struct MSA_CdsQuery *cdsQuery, void *icci_state)
{
	struct ICCI_State *state = (struct ICCI_State*) icci_state;

	MSL_HandleMsaCallback_OnQuery(state->Msl, cdsQuery);
}

/*
 *	Dispatch point for MSA callback.
 *	Method executes when MSA has statistical update.
 */
void _ICCI_HandleMsaCallback_OnStatsChanged (void *msaObj, struct MSA_Stats *stats, void *icci_state)
{
	struct ICCI_State *state = (struct ICCI_State*) icci_state;

	MSL_HandleMsaCallback_OnStatsChanged(state->Msl, stats);
}

/*
 *	Dispatch point for MSL callback.
 *	Method executes when MSL wants to initiate a query on the metadata store.
 *
 *	TODO: You can customize this method by calling methods on a custom
 *	metadata storage module. You can also customize by having this
 *	method abstract multiple metadata stores. This would enable "plug-in"
 *	CDS scenarios, where one MediaServer executable can respond to
 *	queries on multiple virtualized CDS hierarchies.
 */
void* _ICCI_HandleMslCallback_OnQuery(void *mslObj, struct MSA_CdsQuery *cdsQuery, unsigned int filter, void *icci_state, /*OUT*/ enum Enum_CdsErrors *cdsErrorCode)
{
	struct ICCI_State *state = (struct ICCI_State*) icci_state;

	#ifdef ICC_DATABASE
	//TODO: ICC_DATABASE -	Translate the CDS request and issue an appropriate database request. 
	//						Internal to the database implementation, have 'state->DbiObj' keep
	//						track of the results so that you can process them one at a time.
	//						Iterative processing is also recommended to reduce runtime memory consumption.
	return DBI_HandleOnQuery(state->DbiObj, cdsQuery, filter, cdsErrorCode);
	#elif ICC_FILESYSTEM
	return FSE_HandleQuery(state->FseObj, cdsQuery, filter, cdsErrorCode);
	#else
	#error "You must define either ICC_DATABASE or ICC_FILESYSTEM";
	#endif
}

/*
 *	Dispatch point for MSL callback.
 *	Method executes when MSL wants somebody to provide the metadata for a CDS object.
 */
int _ICCI_HandleMslCallback_OnProcessObject(void *mslObj, struct MSA_CdsQuery *cdsQuery, void *onQueryObj, unsigned int filter, void *icci_state, /*INOUT*/ struct CdsMediaObject *cdsObj)
{
	struct ICCI_State *state = (struct ICCI_State*) icci_state;

	#ifdef ICC_DATABASE
	//TODO: ICC_DATABASE -	Assuming 'state->DbiOjb' tracks the results of your database query,
	//						populate a single CDS object with the metadata for the 'next' object in the results.
	//						If you use MSL_RegisterResourceProviderCallbacks() to set up creation of <res>
	//						elements, then cdsObj->Res should be NULL when your return. Otherwise, you need
	//						to assign a linked list of CdsMediaResource objects to cdsObj->Res.
	return DBI_HandleProcessObject(state->DbiObj, cdsQuery, onQueryObj, filter, cdsObj);
	#elif ICC_FILESYSTEM
	return FSE_HandleProcessObject(state->FseObj, cdsQuery, onQueryObj, filter, cdsObj);
	#else
	#error "You must define either ICC_DATABASE or ICC_FILESYSTEM";
	#endif

	return 0;
}

int _ICCI_HandleMslCallback_OnProcessObjects(void *mslObj, struct MSA_CdsQuery *cdsQuery, void **onQueryObj, unsigned int filter, void *icci_state, /*INOUT*/ struct CdsMediaObject **cdsObj, int count)
{
	struct ICCI_State *state = (struct ICCI_State*) icci_state;

	#ifdef ICC_DATABASE
	//TODO: ICC_DATABASE -	Assuming 'state->DbiOjb' tracks the results of your database query,
	//						populate a single CDS object with the metadata for the 'next' object in the results.
	//						If you use MSL_RegisterResourceProviderCallbacks() to set up creation of <res>
	//						elements, then cdsObj->Res should be NULL when your return. Otherwise, you need
	//						to assign a linked list of CdsMediaResource objects to cdsObj->Res.
	return DBI_HandleProcessObjects(state->DbiObj, cdsQuery, onQueryObj, filter, cdsObj);
	#elif ICC_FILESYSTEM
	return FSE_HandleProcessObjects(state->FseObj, cdsQuery, onQueryObj, filter, cdsObj, count);
	#else
	#error "You must define either ICC_DATABASE or ICC_FILESYSTEM";
	#endif

	return 0;
}

int _ICCI_HandleMslCallback_OnQueryCount(void *mslObj, struct MSA_CdsQuery *cdsQuery, void *onQueryObj, unsigned int filter, void *icci_state)
{
	struct ICCI_State *state = (struct ICCI_State*) icci_state;

	#ifdef ICC_DATABASE
	//TODO: ICC_DATABASE -	Assuming 'state->DbiOjb' tracks the results of your database query,
	//						populate a single CDS object with the metadata for the 'next' object in the results.
	//						If you use MSL_RegisterResourceProviderCallbacks() to set up creation of <res>
	//						elements, then cdsObj->Res should be NULL when your return. Otherwise, you need
	//						to assign a linked list of CdsMediaResource objects to cdsObj->Res.
	return DBI_HandleOnQueryCount(state->DbiObj, cdsQuery, onQueryObj, filter);
	#elif ICC_FILESYSTEM
	return FSE_HandleQueryCount(state->FseObj, cdsQuery, onQueryObj, filter);
	#else
	#error "You must define either ICC_DATABASE or ICC_FILESYSTEM";
	#endif

	return 0;
}

/*
 *	Dispatch point for MSL callback.
 *	Method executes when MSL is done with a query on the metadata store.
 */
unsigned int _ICCI_HandleMslCallback_OnAcquireUpdateIDAndCleanup(void *mslObj, struct MSA_CdsQuery *cdsQuery, void *onQueryObj, void *icci_state)
{
	struct ICCI_State *state = (struct ICCI_State*) icci_state;

	#ifdef ICC_DATABASE
	//TODO: ICC_DATABASE -	Use this to clean up the results from the database query.
	//						Generally, this means deallocating anything from the _ICCI_HandleMslCallback_OnQuery.
	//						This is also where the UpdateID for the CDS container is acquired for the results.
	return DBI_HandleOnAcquireUpdateIDAndCleanup(state->DbiObj, cdsQuery, onQueryObj);
	#elif ICC_FILESYSTEM
	return FSE_HandleOnAcquireUpdateIDAndCleanup(state->FseObj, cdsQuery, onQueryObj);
	#else
	#error "You must define either ICC_DATABASE or ICC_FILESYSTEM";
	#endif

	return 0;
}

/* large enough for IPv4 address and port number */
#define MAX_ADDR_PORT_LEN 32

/* large enough for DHWG protocolInfo values */
#define MAX_PROTINFO_LEN 256

void _ICCI_HandleMslCallback_OnAcquireResources (void *mslObj, struct MSA_CdsQuery *cdsQuery, void *onQueryObj, unsigned int filter, void *icci_state, void *resProviderObj, /*INOUT*/ struct CdsMediaObject *cdsObj)
{
	struct ICCI_State *state = (struct ICCI_State*) icci_state;
	struct FSE_QueryState *qs = (struct FSE_QueryState*) onQueryObj;
	struct CdsMediaResource **c;
	char *uri, *tt, *pp;
	int addrListLen;		/* number of IP addresses for the machine */
	int *addrList;			/* array of desired IP addresses */
	int i,fileSize=0;
	char ipAddrAndPortStr[MAX_ADDR_PORT_LEN];
	int ipAddrAndPortStrLen;
	int sourceWidth=0, sourceHeight=0;
	char time_str[100];
	int tc;
	int webServerPort;

	//test
	char new_FileExtension[1000], picture_uri[1000];
	static char save_dirpath[1000], save_filepath[1000];
	int  j, k, dbCount;
	TCHAR	strQuery[MAX_SQL_STR_LENGTH];	
	int pos;	

	//birdy add for lighthttp
	int mkv_count = 0,mkv_path_number = 0;
	char mkv_database_number[100];
	FILE   *out;	
	TCHAR mkv_output_path[512];// = "c:\\database.txt\0";
	//char mkv_file_path[512];
	TCHAR mkv_file_path[512];
	int  mkv_path_size = 0, Count = 0, filesize = 0;
	int error = 0;
	char test[512];// = "c:\\database.txt\0";
	//

	/*
	struct _FILETIME filetime;
	struct _SYSTEMTIME*  systime_1;
	*/
	//memset(&filetime, 0 , sizeof(struct _FILETIME) );


	#ifdef ICC_FILESYSTEM
	struct FSE_ResourceMetadata fseRes;
	#elif ICC_DATABASE
	struct DBI_ResourceMetadata adiRes;
	#endif

	/*
	 *	Instead of registering multiple ResourceProvider callbacks with the 
	 *	MSL, we're going to register this one. This method will then
	 *	route to the correct ResourceProvider. Given that the ICCI
	 *	is the integration layer, this does make sense. (more below).
	 *
	 *	Creating a CdsMediaResource requires creating a URI value.
	 *	Creating a URI value will depend on a variety of things,
	 *	but generally streaming and metadata components (and
	 *	the architecture between the two) are the primary
	 *	things that determine what the 3 will look like
	 *	and how it will be created.
	 *
	 *	Because there's no way to anticipate what the end solution
	 *	will look like, we need to rely on the integration layer
	 *	to do a lot of "integration work"... meaning, this ICCI
	 *	layer will be the one to properly route requests for
	 *	CdsMediaResource objects to the correct components.
	 *
	 *	TODO: Customize _ICCI_HandleMslCallback_OnAcquireResources
	 *	so that a URI can be created given information from 
	 *	the metadata store.
	 */

	/*
	 *	Go the end of the resource list.
	 */
	if(!IsDMSRunning())
	{
		return;
	}
	c = &(cdsObj->Res);
	if (*c != NULL)
	{
		while ((*c)->Next != NULL)
		{
			c = &((*c)->Next);
		}
	}
	/* c points to the next place where we can allocate a CdsMediaResource */

	/*
	 *	Build a list of IP address + port combinations,
	 *	based on what the CdsFilter_DhwgAllIp bit.
	 */
	if (filter & CdsFilter_DhwgAllIp)
	{
		/*
		 *	All network addresses.
		 */
		addrListLen = cdsQuery->IpAddrListLen;
		i = sizeof(int) * addrListLen;
		addrList = (int*) MALLOC(i);
		memcpy(addrList, cdsQuery->IpAddrList, i);
	}
	else
	{
		/*
		 *	Only the IP address where the CDS request
		 *	was received.
		 */
		addrListLen = 1;
		addrList = (int*) MALLOC(sizeof(int));
		addrList[0] = cdsQuery->IpAddrList[0];
	}

	#ifdef ICC_DATABASE
	/*
	 *	TODO: ICC_DATABASE -	Get metadata needed for creating a <res> element.
	 *							Generally, it's best to have 'state-DbiObj' keep
	 *							track of the database results, which includes metadata
	 *							about the media format encoding. 
	 *
	 *							The goal of this part is to append 'CdsMediaResource' objects
	 *							to '*c', which represents the last <res> element in the
	 *							current linked list of cdsObj->Res. Keep in mind that you
	 *							will need to use multiple 'CdsMediaResource' objects for
	 *							different transports and media format profiles.
	 */

	DBI_GetResourceMetadata(state->DbiObj, onQueryObj, *c);

	#elif ICC_FILESYSTEM

	/*
	 *	MUSTDO: Build <res> elements and append them to *c.
	 */

	/*
	 *	TODO: Customize your solution to obtain the correct
	 *	metadata for <res> elements. In some case, you
	 *	may need to obtain the metadata from more than one
	 *	module (e.g., calls to FileExtensionToProtocolInfo()).
	 */
	
	//InternetFlags = 1,It is Video   //InternetFlags = 2,It is Audio
	if ( (qs->InternetFlags == 1) || (qs->InternetFlags == 2))
	{
		FSE_GetResourceMetadata_InternetFile(state->FseObj, onQueryObj, &fseRes);        
	}
	else
	{
		FSE_GetResourceMetadata(state->FseObj, onQueryObj, &fseRes);        
	}

	for(i=0;i<2;i++)
	{
		webServerPort = GetServerPort();
		if( webServerPort == -1 )
		{
			Sleep(1000);
		}

	}
	if( webServerPort == -1 )
	{
			webServerPort = cdsQuery->RequestedOnPort;
	}


	/*
	 *	TODO: If you want to expose things that are not files,
	 *	then you'll need to modify this line.
	 */
	if ((fseRes.LocalFilePath != NULL))
	{
		/* for each desired network address... */
		for (i=0; i < addrListLen; i++)
		{
			/* get IP and port in string form */
			/*ipAddrAndPortStrLen = 
				sprintf(ipAddrAndPortStr, "%d.%d.%d.%d:%d",
						(addrList[i])&0xFF,
						(addrList[i]>>8)&0xFF,
						(addrList[i]>>16)&0xFF,
						(addrList[i]>>24)&0xFF,
						cdsQuery->RequestedOnPort);
			ipAddrAndPortStr[ipAddrAndPortStrLen] = '\0';*/

			webServerPort = GetServerPort();

			//phillo,2007/10/22
			// let audio & video use the new http engine
			//
			//birdy 2008/6/17 add for lighthttpd
			if ( (fseRes.Flags & FSE_RMF_IsFile) && 
				 ( stricmp((char *)qs->FileType,"Video") ==0  || stricmp((char *)qs->FileType,"Audio") ==0) )
			{				
				if ( (fseRes.LocalFilePath[fseRes.LocalFilePathLen - 3] == 'm') && (fseRes.LocalFilePath[fseRes.LocalFilePathLen - 2] == 'k') && (fseRes.LocalFilePath[fseRes.LocalFilePathLen - 1] == 'v') )
				{
					ipAddrAndPortStrLen = 
	                    sprintf(ipAddrAndPortStr, "%d.%d.%d.%d:%d",
							(addrList[i])&0xFF,
							(addrList[i]>>8)&0xFF,
							(addrList[i]>>16)&0xFF,
							(addrList[i]>>24)&0xFF,
							3456);
					ipAddrAndPortStr[ipAddrAndPortStrLen] = '\0';
				}
				else
				{
					ipAddrAndPortStrLen = 
                    sprintf(ipAddrAndPortStr, "%d.%d.%d.%d:%d",
						(addrList[i])&0xFF,
						(addrList[i]>>8)&0xFF,
						(addrList[i]>>16)&0xFF,
						(addrList[i]>>24)&0xFF,
						webServerPort);
					ipAddrAndPortStr[ipAddrAndPortStrLen] = '\0';
				}
			}
			else if ( (qs->InternetFlags == 1) || (qs->InternetFlags == 2))
			{
				ipAddrAndPortStrLen = 
                    sprintf(ipAddrAndPortStr, "%d.%d.%d.%d:%d",
						(addrList[i])&0xFF,
						(addrList[i]>>8)&0xFF,
						(addrList[i]>>16)&0xFF,
						(addrList[i]>>24)&0xFF,
						webServerPort);
                ipAddrAndPortStr[ipAddrAndPortStrLen] = '\0';
			}
			else
			{
				ipAddrAndPortStrLen = 
                    sprintf(ipAddrAndPortStr, "%d.%d.%d.%d:%d",
						(addrList[i])&0xFF,
						(addrList[i]>>8)&0xFF,
						(addrList[i]>>16)&0xFF,
						(addrList[i]>>24)&0xFF,
						cdsQuery->RequestedOnPort);
                ipAddrAndPortStr[ipAddrAndPortStrLen] = '\0';
			}


			/*
			 *	We have a valid local file, so let's grab a URI
			 *	that will be handled by the
			 *	ResourceProvider_LocalFileHttp module.
			 *
			 *	TODO: Change 0 to use actual index if your solution
			 *	has more than one registered root path.
			 */

			//james 05.05.10
			/*
			if ((fseRes.Flags & FSE_RMF_IsFile) && 
				((fseRes.Flags & FSE_RMF_IsPlaylist) == 0)
				)
				*/
			if (fseRes.Flags & FSE_RMF_IsFile)
			{
				if (RP_LFHTTP_SetUriValue(
						state->UriProvider[0].ResUriProvider, 
						fseRes.LocalFilePath,
						0, 
						ipAddrAndPortStr, 
						ipAddrAndPortStrLen, 
						state->UriProvider[0].VirtualDirName,
						state->UriProvider[0].VirtualDirNameLen,
						&(uri), -1) != 0)
				{
					/* we got a URI, so we need to set the other metadata */

					// create a new resource 
					*c = CDS_AllocateResource();
					
					(*c)->Value = uri;

					//birdy 2008/6/17 add for lighthttpd
					if ( (fseRes.LocalFilePath[fseRes.LocalFilePathLen - 3] == 'm') && (fseRes.LocalFilePath[fseRes.LocalFilePathLen - 2] == 'k') && (fseRes.LocalFilePath[fseRes.LocalFilePathLen - 1] == 'v') )
					{
						mkv_count = 1;

						while (fseRes.LocalFilePath[mkv_count] != '\\')
						{
							mkv_database_number[mkv_count - 1] = fseRes.LocalFilePath[mkv_count];
							mkv_count++;
						}
						mkv_database_number[mkv_count] = '\0';
						mkv_path_number = atoi(mkv_database_number);

						//
						
						_stprintf(strQuery, _T("select * from FSE_QueryState where MyIndex = %d"), mkv_path_number);
						 dbCount = db_selectT(strQuery);
						
						_stprintf(mkv_output_path, _T("%s\\etc\\%d.txt"), gWorkingDir, mkv_path_number);
						
						mkv_count = strlen(testFSE_QS[0].FilePath);
						testFSE_QS[0].FilePath[mkv_count] = '\0';						
						mkv_count++;

						out = _wfopen(mkv_output_path, _T("w+b"));
						if (out != NULL)
						{
							fwrite(testFSE_QS[0].FilePath, sizeof(char), mkv_count, out);
							fclose(out);
						}
					}
					
					// albert modify
					 
					//if (qs->pIndex == 16) // internet video
					if (qs->InternetFlags == 1)
	                {
						RP_LFHTTP_SetProtocolInfo_InternetFile(
						state->UriProvider[1].ResUriProvider,
						qs->FilePath,
						fseRes.FileExtension,
						*c,1);
					}					
					else if(qs->InternetFlags == 2) // internet audio
					{
						RP_LFHTTP_SetProtocolInfo_InternetFile(
						state->UriProvider[1].ResUriProvider,
						qs->FilePath,
						fseRes.FileExtension,
						*c,2);
					}
					else
					{
						RP_LFHTTP_SetProtocolInfo(
						state->UriProvider[1].ResUriProvider,
						qs->FilePath,
						fseRes.FileExtension,
						*c,
						0);
					}
					//0 -> no transcode

					//phillo,2007/10/22
					// set internet stream as transcode type
					//if( qs->pIndex == 16 || qs->pIndex == 17 )
					if ( (qs->InternetFlags == 1) || (qs->InternetFlags == 2) )
					{

						tt = malloc(strlen(uri) + 9+2);
						strcpy(tt,uri);
						strcat(tt,"&TC&POWER");
						//birdy 2007/4/19 fixing memory leak
						pp = (*c)->Value;
						(*c)->Value = tt;	
						free(pp);
					}
					
					if( (stricmp((char *)qs->FileType,"Audio")==0) || (stricmp((char *)qs->FileType,"Video")==0)  )
					{
						// take care the part , albert 
						if(qs->ProcessObjectType == FSE_OT_File )
						{
							if(qs->Duration != 0)
								(*c)->Duration = qs->Duration;
						}
					}
					
					if(qs->Bitrate > 0)
						(*c)->Bitrate = qs->Bitrate;

					// TODO : set the real creation time here, by albert
					
					GetFileCreationTime(qs->FilePath, time_str);

					(*c)->Size = qs->Size;//fileSize;
					
					if(stricmp((char *)qs->FileType,"Image")==0)
					{
						GetImageInfo(qs->FilePath,  &sourceWidth,  &sourceHeight);
						(*c)->ResolutionX=(int)sourceWidth;
						(*c)->ResolutionY=(int)sourceHeight;
					}


//Birdy 2007/1/24  Add Resource
#if  0  //ndef 2_RESOURCE
					/* point to next res */
					c = &((*c)->Next);
#endif
//#else
//------------------

					//test
					
					if ( (fseRes.Flags & FSE_RMF_IsFile) && ( stricmp((char *)qs->FileType,"Video") ==0  || stricmp((char *)qs->FileType,"Audio") ==0) )
                    {
                        ipAddrAndPortStrLen = 
                            sprintf(ipAddrAndPortStr, "%d.%d.%d.%d:%d",
                            (addrList[i])&0xFF,
                            (addrList[i]>>8)&0xFF,
                            (addrList[i]>>16)&0xFF,
                            (addrList[i]>>24)&0xFF,
                            webServerPort);
                        ipAddrAndPortStr[ipAddrAndPortStrLen] = '\0';
					}
                    else if ( (qs->InternetFlags == 1) || (qs->InternetFlags == 2))
                    {
                        ipAddrAndPortStrLen = 
                            sprintf(ipAddrAndPortStr, "%d.%d.%d.%d:%d",
                            (addrList[i])&0xFF,
                            (addrList[i]>>8)&0xFF,
                            (addrList[i]>>16)&0xFF,
                            (addrList[i]>>24)&0xFF,
                            webServerPort);
                        ipAddrAndPortStr[ipAddrAndPortStrLen] = '\0';
                    }
                    else
                    {
                        ipAddrAndPortStrLen = 
                            sprintf(ipAddrAndPortStr, "%d.%d.%d.%d:%d",
                            (addrList[i])&0xFF,
                            (addrList[i]>>8)&0xFF,
                            (addrList[i]>>16)&0xFF,
                            (addrList[i]>>24)&0xFF,
                            cdsQuery->RequestedOnPort);
                        ipAddrAndPortStr[ipAddrAndPortStrLen] = '\0';
                    }	



					RP_LFHTTP_SetUriValue(
						state->UriProvider[0].ResUriProvider, 
						fseRes.LocalFilePath,
						0, 
						ipAddrAndPortStr, 
						ipAddrAndPortStrLen, 
						state->UriProvider[0].VirtualDirName,
						state->UriProvider[0].VirtualDirNameLen,
						&(uri), -1);
                    //
					
					(*c)->Next = CDS_AllocateResource();
					c = &((*c)->Next);

					(*c)->Value = (char *)malloc(strlen(uri)+10);
					strcpy((*c)->Value,uri);
					
                    /*tc = RP_LFHTTP_SetProtocolInfo(
						state->UriProvider[1].ResUriProvider,
						qs->FilePath,
						fseRes.FileExtension,
						*c,
						1);*/
					if ( (qs->InternetFlags == 1) || (qs->InternetFlags == 2) )
	                {
						/*tc = RP_LFHTTP_SetProtocolInfo_InternetFile(
						state->UriProvider[1].ResUriProvider,
						qs->FilePath,
						fseRes.FileExtension,
						*c,qs->Flags);*/
						tc = RP_LFHTTP_SetProtocolInfo_InternetFile(
						state->UriProvider[1].ResUriProvider,
						qs->FilePath,
						fseRes.FileExtension,
						*c,qs->InternetFlags);
					}					
					else
					{
						tc = RP_LFHTTP_SetProtocolInfo(
						state->UriProvider[1].ResUriProvider,
						qs->FilePath,
						fseRes.FileExtension,
						*c,
						1);
					}
					// 1 -> open transcode

					if (tc == 0)
					{
						//birdy	2007/12/05 add for transcode file,new server can not play transcode file
						if ( (qs->InternetFlags != 1) && (qs->InternetFlags != 2) )
	                    {
							ipAddrAndPortStrLen = 
                            sprintf(ipAddrAndPortStr, "%d.%d.%d.%d:%d",
                            (addrList[i])&0xFF,
                            (addrList[i]>>8)&0xFF,
                            (addrList[i]>>16)&0xFF,
                            (addrList[i]>>24)&0xFF,
                            cdsQuery->RequestedOnPort);
                            ipAddrAndPortStr[ipAddrAndPortStrLen] = '\0';
                    
					        RP_LFHTTP_SetUriValue(
                                state->UriProvider[0].ResUriProvider, 
                                fseRes.LocalFilePath,
                                0, 
                                ipAddrAndPortStr, 
                                ipAddrAndPortStrLen, 
                                state->UriProvider[0].VirtualDirName,
                                state->UriProvider[0].VirtualDirNameLen,
                                &(uri), -1);
						}						

						tt = malloc(strlen(uri) + 9+2);
						strcpy(tt,uri);
						strcat(tt,"&TC&POWER");
						//birdy 2007/4/19 fixing memory leak
						pp = (*c)->Value;
						(*c)->Value = tt;						
					}
					else if (tc == 1)
					{
						tt = malloc(strlen(uri) + 4+2);
						strcpy(tt,uri);
     					strcat(tt,"&TC");
						//birdy 2007/4/19 fixing memory leak
    					pp = (*c)->Value;
						(*c)->Value = tt;
					}

					
					if( (stricmp((char *)qs->FileType,"Audio")==0) || (stricmp((char *)qs->FileType,"Video")==0)  )
					{
						// take care the part , albert 
						if(qs->ProcessObjectType == FSE_OT_File )
						{
							if(qs->Duration != 0)
								(*c)->Duration = qs->Duration;
						}
					}
					
					if(qs->Bitrate > 0)
						(*c)->Bitrate = qs->Bitrate;

					// TODO : set the real creation time here, by albert
					
					GetFileCreationTime(qs->FilePath, time_str);

					(*c)->Size = qs->Size;//fileSize;
					
					if(stricmp((char *)qs->FileType,"Image")==0)
					{
						GetImageInfo(qs->FilePath,  &sourceWidth,  &sourceHeight);
						(*c)->ResolutionX=(int)sourceWidth;
						(*c)->ResolutionY=(int)sourceHeight;
					}
										
					}
					/* point to next res */
					//birdy 2007/12/6 for cd_cover
					if ( (stricmp((char *)qs->FileType,"Audio")==0) && (qs->ObjectType == 1)  && (qs->CDCover[0] != '\0') )
					{
						ipAddrAndPortStrLen = 
                            sprintf(ipAddrAndPortStr, "%d.%d.%d.%d:%d",
                            (addrList[i])&0xFF,
                            (addrList[i]>>8)&0xFF,
                            (addrList[i]>>16)&0xFF,
                            (addrList[i]>>24)&0xFF,
                            webServerPort);
                            ipAddrAndPortStr[ipAddrAndPortStrLen] = '\0';

							(*c)->Next = CDS_AllocateResource();
                            c = &((*c)->Next);
							pos = LastIndexOf(uri, "/");
							strncpy(picture_uri, uri, pos + 1);
							picture_uri[pos + 1] = '\0';
							strcat(picture_uri, qs->CDCover);
                            (*c)->Value = (char *)malloc(strlen(picture_uri)+10);
                            strcpy((*c)->Value,picture_uri);

							k = 0;
							pos = LastIndexOf(qs->CDCover, ".");							
							for ( j = pos; j < strlen(qs->CDCover); j++)
							{
								new_FileExtension[k] = qs->CDCover[j];
								k++;
							}
							new_FileExtension[k] = 0;

							RP_LFHTTP_SetProtocolInfo(
                                state->UriProvider[1].ResUriProvider,
                                qs->FilePath,
                                new_FileExtension,
                                *c,
                                1);

							/*tt = malloc(strlen(uri) + 9+2);
                            strcpy(tt,uri);
                            strcat(tt,"&CD_Cover");
                            //birdy 2007/4/19 fixing memory leak
                            pp = (*c)->Value;
                            (*c)->Value = tt;*/

					}

#if 0
					//birdy 2007/12/6 for cd_cover
					if ( (stricmp((char *)qs->FileType,"Audio")==0) && (qs->ObjectType == 1) )
					{						
						strcpy(ccc, fseRes.LocalFilePath);
                        
						k = 1; 
						while ( ccc[k] != 0x5c)
						{
							cc[k - 1] = ccc[k];
							k++;							
						}
						cc[k] = 0;
						pos = atol(cc);

						_stprintf(strQuery, _T("select * from FSE_QueryState where MyIndex =%d"), pos);
						dbCount = db_selectT(strQuery);
					
						strcpy(filepath, testFSE_QS[0].FilePath);
						strcpy(dirpath, testFSE_QS[0].FilePath);

						pos = LastIndexOf(dirpath, "\\");
						filepath[pos] = 0;
						dirpath[pos] = 0;		
						
						strcat(dirpath, "\\*.jpg");
                        Utf8ToStr(str, (char *) dirpath, MAX_PATH_LENGTH);
	                    FindFirstFile(str, &FileData);

						if ( FileData.cFileName[0] == 0)
						{
							dirpath[0] = 0;
							strcat(dirpath, testFSE_QS[0].FilePath);
							dirpath[pos] = 0;
							strcat(dirpath, "\\*.jpeg");
                            Utf8ToStr(str, (char *) dirpath, MAX_PATH_LENGTH);
                            FindFirstFile(str, &FileData);
							
							if ( FileData.cFileName[0] == 0)
							{
								dirpath[0] = 0;
								strcat(dirpath, testFSE_QS[0].FilePath);
								dirpath[pos] = 0;
                                strcat(dirpath, "\\*.bmp");
                                Utf8ToStr(str, (char *) dirpath, MAX_PATH_LENGTH);
                                FindFirstFile(str, &FileData);

								if ( FileData.cFileName[0] == 0)
                                {
									dirpath[0] = 0;
                                    strcat(dirpath, testFSE_QS[0].FilePath);
									dirpath[pos] = 0;
                                    strcat(dirpath, "\\*.png");
                                    Utf8ToStr(str, (char *) dirpath, MAX_PATH_LENGTH);
                                    FindFirstFile(str, &FileData);
                                }
                            }							
						}

						if ( FileData.cFileName[0] == 0)
						{
                            check_picture = 0;
						}
						else
						{
							check_picture = 1;
						}


						if ( check_picture == 1)
						{
                            Utf8ToStr(str1, (char *) filepath, MAX_PATH_LENGTH);
                            wcscat(str1, _T("\\"));
                            wcscat(str1, FileData.cFileName);

                            _stprintf(strQuery, _T("select * from FSE_QueryState where FilePath = \"%s\""), str1);
                            dbCount = db_selectT(strQuery);

                            fseRes.LocalFilePathLen = strlen(testFSE_QS[0].ProcessPath);
                            free(fseRes.LocalFilePath);
                            fseRes.LocalFilePath = malloc(fseRes.LocalFilePathLen);
                            memcpy(fseRes.LocalFilePath, testFSE_QS[0].ProcessPath, fseRes.LocalFilePathLen);	
							fseRes.LocalFilePath[fseRes.LocalFilePathLen] = 0;
							
                            for (j = 0; j < fseRes.LocalFilePathLen + 1; j++)
                            {
                                if ( fseRes.LocalFilePath[j] == '.' )
                                {
                                    number = j; 
                                }
                            }

                            for ( j = number; j <= fseRes.LocalFilePathLen + 1; j++)
                            {
                                test_FileExtension[j - number] = fseRes.LocalFilePath[j];
                            }

							test_FileExtension[j - number - 1] = 0;

                            fseRes.FileExtensionLen = strlen(fseRes.FileExtension);
                            free(fseRes.FileExtension);
                            fseRes.FileExtension = malloc(fseRes.FileExtensionLen + 1);
                            memcpy(fseRes.FileExtension, test_FileExtension, fseRes.FileExtensionLen + 1);								

							ipAddrAndPortStrLen = 
                            sprintf(ipAddrAndPortStr, "%d.%d.%d.%d:%d",
                            (addrList[i])&0xFF,
                            (addrList[i]>>8)&0xFF,
                            (addrList[i]>>16)&0xFF,
                            (addrList[i]>>24)&0xFF,
                            cdsQuery->RequestedOnPort);
                            ipAddrAndPortStr[ipAddrAndPortStrLen] = '\0';

                            RP_LFHTTP_SetUriValue(
                                state->UriProvider[0].ResUriProvider, 
                                fseRes.LocalFilePath,
                                0, 
                                ipAddrAndPortStr, 
                                ipAddrAndPortStrLen, 
                                state->UriProvider[0].VirtualDirName,
                                state->UriProvider[0].VirtualDirNameLen,
                                &(uri), -1);

                            (*c)->Next = CDS_AllocateResource();
                            c = &((*c)->Next);
                            (*c)->Value = (char *)malloc(strlen(uri)+10);
                            strcpy((*c)->Value,uri);

                            RP_LFHTTP_SetProtocolInfo(
                                state->UriProvider[1].ResUriProvider,
								testFSE_QS[0].FilePath,
                                fseRes.FileExtension,
                                *c,
                                0);

                            
                            GetImageInfo(str1,  &sourceWidth,  &sourceHeight);
                            (*c)->ResolutionX=(int)sourceWidth;
                            (*c)->ResolutionY=(int)sourceHeight;

                            c = &((*c)->Next);                   
						}
					}
					else
#endif
					{
                        c = &((*c)->Next);
                        free(pp);						
                        free(uri);
					}
				
//-------------

//#endif //2_RESOURCE
				
				// albert add, 20050402
				// if the file is a image, generate the second res for thumbnail image
				// Todo: change the url to meet the real location of the thumbnail image
				/*
				if(stricmp((char *)qs->FileType,"Image")==0)
				{
					
					/*
					//bmp
					if (RP_LFHTTP_SetUriValue(
						state->UriProvider[0].ResUriProvider, 
						fseRes.LocalFilePath,
						0, 
						ipAddrAndPortStr, 
						ipAddrAndPortStrLen, 
						state->UriProvider[0].VirtualDirName,
						state->UriProvider[0].VirtualDirNameLen,
						&(uri), -1) != 0)
					{
						

						
						*c = CDS_AllocateResource();
						
						uri[strlen(uri)-4]='.';
						uri[strlen(uri)-3]='b';
						uri[strlen(uri)-2]='m';
						uri[strlen(uri)-1]='p';

						(*c)->Value = uri;

						RP_LFHTTP_SetProtocolInfo(
							state->UriProvider[1].ResUriProvider,
							fseRes.LocalFilePath,
							fseRes.FileExtension,
							*c
							);

						c = &((*c)->Next);
					}
					//tb
					if (RP_LFHTTP_SetUriValue(
						state->UriProvider[0].ResUriProvider, 
						fseRes.LocalFilePath,
						0, 
						ipAddrAndPortStr, 
						ipAddrAndPortStrLen, 
						state->UriProvider[0].VirtualDirName,
						state->UriProvider[0].VirtualDirNameLen,
						&(uri), -1) != 0)
					{
						*c = CDS_AllocateResource();
												
						uri[strlen(uri)-4]='.';
						uri[strlen(uri)-3]='t';
						uri[strlen(uri)-2]='b';
						uri[strlen(uri)-1]='\0';
						
						(*c)->Value = uri;

						RP_LFHTTP_SetProtocolInfo(
							state->UriProvider[1].ResUriProvider,
							fseRes.LocalFilePath,
							fseRes.FileExtension,
							*c
							);

						
						c = &((*c)->Next);
					}
					
				}*/
				

			}
			else if (fseRes.Flags & FSE_RMF_IsPlaylist)
			{
				/*
				 *	This is a playlist, so build a URI
				 *	for a playlist
				 *
				 *	DEPRECATED: DLNA is not using M3U files, so
				 *	this code is no longer going to be developed.
				 */

				//if (RP_M3uHTTP_SetUriValue(
				//		state->UriProvider[1].ResUriProvider, 
				//		fseRes.LocalFilePath,
				//		0,
				//		0, 
				//		ipAddrAndPortStr, 
				//		ipAddrAndPortStrLen, 
				//		state->UriProvider[1].VirtualDirName,
				//		state->UriProvider[1].VirtualDirNameLen,
				//		&(uri), -1) != 0)
				//{
				//	/* create a new resource */
				//	*c = CDS_AllocateResource();
				//	(*c)->Value = uri;

				//	RP_LFHTTP_SetProtocolInfo(
				//		state->UriProvider[1].ResUriProvider,
				//		fseRes.LocalFilePath,
				//		fseRes.FileExtension,
				//		*c
				//		);

				//	/* point to next res */
				//	c = &((*c)->Next);
				//}
			}
			else if (fseRes.Flags & FSE_RMF_IsDirectory)
			{
				/*
				 *	TODO: If you want your solution to expose
				 *	<res> elements for CDS storage container objects
				 *	then modify the code below and modify
				 *	the implementation for RP_M3uHTTP_SetUriValue().
				 *
				 *	RP_M3uHTTP_SetUriValue(...)
				 */
				//birdy 2008/3/24 add for iso play dvd
								
				char str[100];
				int iso_myindex, VIDEO_TS_myindex = 0, nPos;

				ipAddrAndPortStrLen = 
                            sprintf(ipAddrAndPortStr, "%d.%d.%d.%d:%d",
                            (addrList[i])&0xFF,
                            (addrList[i]>>8)&0xFF,
                            (addrList[i]>>16)&0xFF,
                            (addrList[i]>>24)&0xFF,
                            webServerPort);
                            ipAddrAndPortStr[ipAddrAndPortStrLen] = '\0';


				if ( (fseRes.LocalFilePathLen - 5) < 0)
				{
					break;
				}
				
				k = 0;
				for (i = fseRes.LocalFilePathLen - 5 ; i < fseRes.LocalFilePathLen; i++)
				{
					str[k] = fseRes.LocalFilePath[i];					
					k++;
				}
				str[5] = 0;

				nPos = LastIndexOf(fseRes.LocalFilePath,"\\");

				if (strstr(str, "(iso)"))
				{
                    k = 0;

                    for (i = 1 ; i < nPos; i++)
                    {
                        str[k] = fseRes.LocalFilePath[i];					
                        k++;
					}
                }
                str[k] = 0;
					
				iso_myindex = atoi(str);

				_stprintf(strQuery, _T("select * from FSE_QueryState where pIndex = %d"), iso_myindex);
				dbCount = db_selectT(strQuery);

				if (dbCount == 0)
				{
					break;
				}

				if ( testFSE_QS[0].Flags == 256) 
				{
					
					
					for( i = 0; i < dbCount; i++)
					{
						if (strcmp(testFSE_QS[i].FileName , "VIDEO_TS") == 0)
						{
							VIDEO_TS_myindex = testFSE_QS[i].Index;
						}

					}

					_stprintf(strQuery, _T("select * from FSE_QueryState where pIndex = %d"), VIDEO_TS_myindex);
					dbCount = db_selectT(strQuery);

					//
					if (VIDEO_TS_myindex != 0)
					{
						for (i = 0; i < dbCount; i++)
						{
							RP_LFHTTP_SetUriValue(
							state->UriProvider[0].ResUriProvider, 
							//fseRes.LocalFilePath,
							testFSE_QS[i].ProcessPath,
							0, 
							ipAddrAndPortStr, 
							ipAddrAndPortStrLen, 
							state->UriProvider[0].VirtualDirName,
							state->UriProvider[0].VirtualDirNameLen,
							&(uri), -1);                    

							if (i == 0)
							{
			                    *c = CDS_AllocateResource();					
				                (*c)->Value = uri;					
							}
							else
							{
			                    (*c)->Next = CDS_AllocateResource();
				                c = &((*c)->Next);
							}

					
							(*c)->Value=uri;
							//(*c)->Value = (char *)malloc(strlen(uri)+10);
						
							//strcpy((*c)->Value,uri);
	
							nPos = LastIndexOf(testFSE_QS[i].FilePath,".");
	
							str[0] = testFSE_QS[i].FilePath[nPos];
							str[1] = tolower(testFSE_QS[i].FilePath[nPos+1]);
							str[2] = tolower(testFSE_QS[i].FilePath[nPos+2]);
							str[3] = tolower(testFSE_QS[i].FilePath[nPos+3]);
							str[4] = 0;

                    
							RP_LFHTTP_SetProtocolInfo(
							state->UriProvider[1].ResUriProvider,
							//qs->FilePath,
							testFSE_QS[i].FilePath,
							//fseRes.FileExtension,				
							str,
							*c,
							0);
                    						                    
							if( (stricmp((char *)qs->FileType,"Audio")==0) || (stricmp((char *)qs->FileType,"Video")==0)  )					
							{						
								// take care the part , albert 						
								if(qs->ProcessObjectType == FSE_OT_File )						
								{
									if(qs->Duration != 0)
										//(*c)->Duration = qs->Duration;
										(*c)->Duration = testFSE_QS[i].Duration;
								}
							}					
	
							if(qs->Bitrate > 0)
								//(*c)->Bitrate = qs->Bitrate;
								(*c)->Bitrate = testFSE_QS[i].Bitrate;

							// TODO : set the real creation time here, by albert
					

					
							(*c)->Size = testFSE_QS[i].Size;//fileSize;
					

							if(stricmp((char *)qs->FileType,"Image")==0)
							{
								GetImageInfo(qs->FilePath,  &sourceWidth,  &sourceHeight);
								(*c)->ResolutionX=(int)sourceWidth;
								(*c)->ResolutionY=(int)sourceHeight;
							}					
						}
						//
					}
				}
												
				
				//
			}

			/*
			 *	TODO: You can customize this to get more 
			 *	CdsMediaResource objects from other modules.
			 */
		}
		free (fseRes.LocalFilePath);
		free (fseRes.FileExtension);
		memset(&fseRes, 0, sizeof(struct FSE_ResourceMetadata));
	}
	
	#else
	#error "You must define either ICC_DATABASE or ICC_FILESYSTEM";
	#endif

	free (addrList);
}



/*
 *
 *	TODO: One way to guard against unauthorized access to local files
 *	is to change REQUIRE_REGISTERED_PATH to a nonzero value. This will
 *	require calls to RP_LFHTTP_RelativeUriToFullPath to match only
 *	those URI values that map to registered local paths.
 */
#define REQUIRE_REGISTERED_PATH 0

void _ICCI_HandleHttpContentRequest_LocalContent(struct ILibWebServer_Session *session, struct packetheader *header, struct ICCI_State *state, int isPlaylist)
{
	struct FSE_QueryState tmpFSE_QS;
	char fullpath[MAX_PATH_LENGTH];
	//tempstr[MAX_PATH_LENGTH];
	int len,resultSQL;
	char strSQL[MAX_SQL_STR_LENGTH];
	int fileNotFound = 0;
	struct TS_TransferState *ts;
	int directiveObjLen;
	
	//char ipAddrAndPortStr[MAX_ADDR_PORT_LEN];
	//int ipAddrAndPortStrLen;
	//int ipAddr;

	/*
	 *	BUGBUG: header->DirectiveObjLength has an off-by-one error.
	 *	Until then, use strlen.
	 */
	directiveObjLen = (int) strlen(header->DirectiveObj);

	/*
	 *	Handle HTTP content requests.
	 *
	 *	DEPRECATED: DLNA will not be using M3U files.
	 */
	isPlaylist = 0;
	len = 0;
	if (isPlaylist)
	{
		/*
		 *	DEPRECATED: DLNA will not be using M3U files.
		 */

		//ipAddr = ILibWebServer_GetLocalInterface(session);
		//ipAddrAndPortStrLen = 
		//	sprintf(ipAddrAndPortStr, "%d.%d.%d.%d:%d",
		//			(ipAddr)&0xFF,
		//			(ipAddr>>8)&0xFF,
		//			(ipAddr>>16)&0xFF,
		//			(ipAddr>>24)&0xFF,
		//			ILibWebServer_GetPortNumber(state->WebServerToken));
		//ipAddrAndPortStr[ipAddrAndPortStrLen] = '\0';

		//len = RP_M3uHTTP_RelativeUriToFullPath(
		//	state->UriProvider[ICCI_RP_LOCALPLAYLIST].ResUriProvider, 
		//	header->DirectiveObj, directiveObjLen, 
		//	REQUIRE_REGISTERED_PATH, 
		//	fullpath, MAX_PATH_LENGTH);
	}
	else
	{
		len = RP_LFHTTP_RelativeUriToFullPath(
			state->UriProvider[ICCI_RP_LOCALFILE].ResUriProvider, 
			header->DirectiveObj, directiveObjLen, 
			REQUIRE_REGISTERED_PATH, 
			fullpath, MAX_PATH_LENGTH);
	}

	if ((len > 0) && (len < MAX_PATH_LENGTH))
	{
		/*
		 *	We have a file path, but we do not know if the file exists.
		 *	More importantly, we do not know if the specified file can
		 *	be downloaded. 
		 *
		 *	Do a check to see if the file exists. Although this is an
		 *	I/O operation, checking for existance is just easier to do
		 *	in a synchronous operation.
		 *
		 *	TODO: Guard against unauthorized access to local files.
		 */

		// albert add,
		//if (FIOA_GetFileDirType(fullpath) == 1)

		sprintf(strSQL,"select * from FSE_QueryState where FilePath =\"%s\"",fullpath);
		resultSQL=db_getOneRecord(strSQL,&tmpFSE_QS);

		if ((resultSQL && (tmpFSE_QS.ObjectType == 1)) || (FIOA_GetFileDirType(fullpath) == 1))
		//if (FIOA_GetFileDirType(fullpath) == 1)
		{
			/*
			 *	We allow access only to files in this handler.
			 *	Now instantiate a TS_TransferState object
			 *	to keep state for this transfer.
			 *
			 *	TODO: You can implement a dynamic buffer size
			 *	based on server load or other things.
			 */
			
			if (isPlaylist)
			{
				/*
				 *	DEPRECATED: DLNA will not be using M3U files.
				 */

				//ts = RP_M3uHTTP_Respond(
				//	session, header, TS_BUFFER_SIZE, fullpath,
				//	FileExtensionToMimeType(fullpath + LastIndexOf(fullpath, "."),0), 
				//	_ICCI_HandleTSCallback_OnDestroy,
				//	state->UriProvider[ICCI_RP_LOCALFILE].ResUriProvider,
				//	0, 
				//	ipAddrAndPortStr, ipAddrAndPortStrLen,
				//	state->UriProvider[ICCI_RP_LOCALFILE].VirtualDirName,
				//	state->UriProvider[ICCI_RP_LOCALFILE].VirtualDirNameLen);
			}
			else
			{
				#ifdef _DEBUG
				printf("Webrequest: %s\r\n", fullpath);
				#endif

				ts = RP_LFHTTP_Respond(
					session, 
					header, 
					TS_BUFFER_SIZE, 
					fullpath,
					FileExtensionToMimeType(fullpath + LastIndexOf(fullpath, "."),0), 
					_ICCI_HandleTSCallback_OnDestroy);
			}

			/*
			 *	If ts is NULL, then a response has
			 *	been sent. If ts is non-NULL, then the response will
			 *	be sent in blocks asynchronously.
			 *
			 *	TODO: You can save 'ts' and use it to present
			 *	file transfer progress in a GUI.
			 */

			// Todo , albert add , set file transfer progress here

			/*
			 *	TODO: Applications that want to monitor the progress
			 *	of the transfer should save the ts pointer.
			 */
		}
		else
		{
			fileNotFound = 1;
		}
	}
	else
	{
		fileNotFound = 1;
	}

	if (fileNotFound)
	{
		/* send an HTTP error response */
		CHTTP_SendHttpError(session, TS_F_FileNotFound);
	}
}

void _ICCI_HandleHttpContentRequest_LocalFile(struct ILibWebServer_Session *session, struct packetheader *header, char *bodyBuffer, int *beginPointer, int endPointer, int done, void *user)
{
#ifdef OEM_ASUS
	// Call this function to wake up system if any clients try to download file from the server.
	WaitUpSystem();
#endif
	/*
	 *	There's always the chance that people will do something stupid like send
	 *	a message body on an HTTP request.
	 */
	*beginPointer = endPointer;

	_ICCI_HandleHttpContentRequest_LocalContent(
		session, 
		header, 
		(struct ICCI_State*)
		user, 
		0);
}

void _ICCI_HandleHttpContentRequest_Playlist(struct ILibWebServer_Session *session, struct packetheader *header, char *bodyBuffer, int *beginPointer, int endPointer, int done, void *user)
{
	/*
	 *	There's always the chance that people will do something stupid like send
	 *	a message body on an HTTP request.
	 */
	*beginPointer = endPointer;

	_ICCI_HandleHttpContentRequest_LocalContent(
		session, 
		header, 
		(struct ICCI_State*)
		user, 
		1);
}

void _ICCI_StartMSL (void *mslChain)
{
	#ifdef ICC_DATABASE
	//TODO: Initialize database and spawn thread for the database, if needed.
	DBI_InitThreadForCOM();
	#endif
	ILibStartChain(mslChain);
	#ifdef ICC_DATABASE
	//TODO: uninit database stuff done above
	DBI_UnInitThreadForCOM();
	#endif
	THRD_EndThisThread();
}

void ICCI_SetSortSearchFields(struct ICCI_State *state)
{
	/*
	 *	TODO: Set these fields to match the capabilities
	 *	of the back end service.
	 */

	#ifdef ICC_DATABASE
	//TODO: ICC_DATABASE
	state->SortFields = state->SearchFields = CdsSearchSort_ALL;

	#elif ICC_FILESYSTEM
	// albert modify , modify sort and search capabilities 20050325
	state->SortFields = 0;
	state->SearchFields = CdsSearchSort_Title | CdsSearchSort_Class | CdsFilter_ProtocolInfo ;
	#endif
}

void ICCI_SetUDN(struct ICCI_State *state)
{
	/*
	 *	If the UPnP device runs on a PC, this code is fine.
	 *
	 *	TODO:	If the UPnP device is not a PC-application, then
	 *				this value should be persisted between device or
	 *				reset/power cycles.
	 */
	state->UDN = (char*) MALLOC(STR_UDN_SIZE);
	//strMakeUDN(state->UDN);
	// albert add
	GetMACaddress(state->UDN);
}

void ICCI_SetSerialNum(struct ICCI_State *state)
{
	/*
	 *	TODO:	Each product instance should have a unique serial number
	 *				that is persisted between reboots.
	 */

	state->SerialNum = (char*) MALLOC(STR_UDN_SIZE);
	strMakeUDN(state->SerialNum);
}

void ICCI_SetupContentProviders(struct ICCI_State *state)
{
	#ifdef ICC_FILESYSTEM
	state->UriProviderLen = 2;
	state->UriProvider = (struct ICCI_ContentProvider*) MALLOC(state->UriProviderLen * sizeof(struct ICCI_ContentProvider));

	memset(&(state->UriProvider[ICCI_RP_LOCALFILE]), 0, sizeof(struct ICCI_ContentProvider));
	state->UriProvider[ICCI_RP_LOCALFILE].VirtualDirName = "0";
	state->UriProvider[ICCI_RP_LOCALFILE].VirtualDirNameLen = (int) strlen(state->UriProvider[0].VirtualDirName);

	memset(&(state->UriProvider[ICCI_RP_LOCALPLAYLIST]), 0, sizeof(struct ICCI_ContentProvider));
	state->UriProvider[ICCI_RP_LOCALPLAYLIST].VirtualDirName = "1";
	state->UriProvider[ICCI_RP_LOCALPLAYLIST].VirtualDirNameLen = (int) strlen(state->UriProvider[1].VirtualDirName);

	/*
	 *	We will initialize .ResUriProvider and .TransportHandler for
	 *	LocalFileHttp in ICCI_SetupComponentsAndStartUpnp().
	 */
	#endif
}

void ICCI_SetupComponentsAndStartUpnp(struct ICCI_State* icci_state)
{
	struct ICCI_State *state = (struct ICCI_State*) icci_state;
	char *searchFields, *sortFields;
	int size;
	
	#ifdef WINSOCK2
	/*
	 *	the winsock2 ioctl for monitoring ip addresses doesn't allow
	 *	me to send a void* that will be returned.
	 */
	TheIcciState = state;
	#endif

	waio_start();

	/*
	 *	Obtain the string forms of the search and sort fields.
	 */
	size = CdsToDidl_GetCsvLengthFromBitString(state->SearchFields) + 1;
	searchFields = (char*) MALLOC(size);
	CdsToDidl_WriteCsvStringFromBitString(searchFields, state->SearchFields);
	size = CdsToDidl_GetCsvLengthFromBitString(state->SortFields) + 1;
	sortFields = (char*) MALLOC(size);
	CdsToDidl_WriteCsvStringFromBitString(sortFields, state->SortFields);

	/*
	 *	Set up the upnp thread chain and components that will run on it.
	 */

	// by albert 
	state->UpnpChain = ILibCreateChain();
	state->Ltm =ILibCreateLifeTime(state->UpnpChain);
	#ifdef WITH_EMBEDDED_DMS
		state->Upnp = UpnpCreateMicroStack(state->UpnpChain, "Intel DMS at Root", "Intel Embedded DMS", state->UDN, state->SerialNum, 1800, 0);
	#else
	//albert 20050315 add
	db_getAVP("ServerName",globalServerName);
	if(globalServerName[0] != '\0')		
		state->Upnp = UpnpCreateMicroStack(state->UpnpChain, globalServerName, state->UDN, state->SerialNum, 1800, 0);	
	else		
#ifdef STOREX
		state->Upnp = UpnpCreateMicroStack(state->UpnpChain, "STOREX test Digital Media Server", state->UDN, state->SerialNum, 1800, 0);	
#elif  STOREX-Aivx370
		state->Upnp = UpnpCreateMicroStack(state->UpnpChain, "STOREX test Digital Media Server", state->UDN, state->SerialNum, 1800, 0);
#else
		state->Upnp = UpnpCreateMicroStack(state->UpnpChain, "HoneyWld test Digital Media Server", state->UDN, state->SerialNum, 1800, 0);	
#endif

	//state->Upnp = UpnpCreateMicroStack(state->UpnpChain, "Intel Digital Media Server", state->UDN, state->SerialNum, 1800, 0);
	#endif
	state->WebServerToken = UpnpGetWebServerToken(state->Upnp);

	#ifdef ICC_FILESYSTEM
	/*
	 *	Set up the virtual directory for handling HTTP requests for content.
	 *
	 *	TODO: We recommend that you register a virtual directory for each
	 *	module that will handle HTTP web requests. 
	 */
	
	/* register directory for local file HTTP handler */
	ILibWebServer_RegisterVirtualDirectory(
		UpnpGetWebServerToken(state->Upnp),
		state->UriProvider[ICCI_RP_LOCALFILE].VirtualDirName,
		state->UriProvider[ICCI_RP_LOCALFILE].VirtualDirNameLen,
		_ICCI_HandleHttpContentRequest_LocalFile,
		state);

	/* register for HTTP handling of playlists */
	ILibWebServer_RegisterVirtualDirectory(
		UpnpGetWebServerToken(state->Upnp),
		state->UriProvider[ICCI_RP_LOCALPLAYLIST].VirtualDirName,
		state->UriProvider[ICCI_RP_LOCALPLAYLIST].VirtualDirNameLen,
		_ICCI_HandleHttpContentRequest_Playlist,
		state);
	#endif

	/*
	 *	Initialize MSA.
	 */
	state->Msa = MSA_CreateMediaServer(
		state->UpnpChain, state->Upnp, state->Ltm, 
		"", "http-get:*:*:*", 
		sortFields, searchFields, 
		_ICCI_HandleMsaCallback_OnQuery, 
		_ICCI_HandleMsaCallback_OnStatsChanged, 
		state);

	/*
	 *	Obtain initial set of IP addresses and update MSA
	 */
	sem_init(&(state->IPAddressLock), 0, 1);
	sem_wait(&(state->IPAddressLock));
	state->IPAddressLength = ILibGetLocalIPAddressList(&(state->IPAddressList));
	MSA_UpdateIpInfo(state->Msa, state->IPAddressList, state->IPAddressLength);
	sem_post(&(state->IPAddressLock));

	/*
	 *	Start up the IP address monitoring mechanism.
	 */
	// albert check here, 20050401 , client ip
	#ifndef WINSOCK2
	/* set the IP address updater to poll every 4 seconds) */
	ILibLifeTime_Add(state->Ltm,NULL,4,_ICCI_IPAddressMonitor,NULL);
	#else
	state->MonitorSocket = socket(AF_INET,SOCK_DGRAM,0);
	WSAIoctl(state->MonitorSocket,SIO_ADDRESS_LIST_CHANGE,NULL,0,NULL,0,&(state->MonitorSocketReserved),&(state->MonitorSocketStateObject),_ICCI_IPAddressMonitor);
	#endif

	/*
	 *	Set up the MSL thread chain and components that will run on it.
	 */
	state->MslChain = ILibCreateChain();
	// albert del
	state->Msl = MSL_CreateMediaServer(state->MslChain, state->Upnp, state->Msa, "", "http-get:*:*:*", sortFields, 
									   searchFields, 
									   _ICCI_HandleMslCallback_OnQuery, 
									   _ICCI_HandleMslCallback_OnProcessObject, 
									   _ICCI_HandleMslCallback_OnAcquireUpdateIDAndCleanup,
									   _ICCI_HandleMslCallback_OnProcessObjects,
									   _ICCI_HandleMslCallback_OnQueryCount, 
									   state);

	
	/*
	 *	Register a single callback, which will route to zero or more
	 *	URI/resource providers. If desired, we can register multiple
	 *	callbacks with the MSL, but the ICCI is supposed to be the
	 *	integration point for everything.
	 */
	MSL_RegisterResourceProviderCallbacks(state->Msl, _ICCI_HandleMslCallback_OnAcquireResources, NULL);

	#ifdef ICC_FILESYSTEM
		/*
		 *	Initialize the resource providers.
		 */
		state->UriProvider[ICCI_RP_LOCALFILE].ResUriProvider = RP_LFHTTP_Create(state->MslChain);
		state->UriProvider[ICCI_RP_LOCALFILE].TransportHandler = NULL;

		state->UriProvider[ICCI_RP_LOCALPLAYLIST].ResUriProvider = state->UriProvider[0].ResUriProvider;
		state->UriProvider[ICCI_RP_LOCALPLAYLIST].TransportHandler = NULL;

		/*
		 *	TODO: You may need to register additional local paths
		 *	if your solution has more than one.
		 */
		RP_LFHTTP_RegisterStartPath(state->UriProvider[0].ResUriProvider, state->FseRootPath);
		state->FseObj = FSE_InitFSEState(state->MslChain, state->FseRootPath);
	#elif ICC_DATABASE
	//TODO: Initialize stuff specific to ICC_DATABASE
		/*
		 *	The database integration layer will be responsible for
		 *	providing all metadata, including generating relative URI
		 *	values for content.
		 */
		state->DbiObj = DBI_Init(state->AccessDbPath);
	#else
		#error "You must define either ICC_DATABASE or ICC_FILESYSTEM";
	#endif

	free (searchFields);
	free (sortFields);


	/*
	 *	Start the MSL thread chain.
	 */
	THRD_SpawnNormalThread(_ICCI_StartMSL, state->MslChain);

	/*
	 *	Everything is set up, so start the thread-chain.
	 *	Execution of the thread will remain here until instructed to stop.
	 */
	ILibStartChain(state->UpnpChain);

	#ifdef ICC_DATABASE
	//TODO: unitialize stuff specific to ICC_DATABASE
	DBI_UnInit(state->DbiObj);
	#endif

	/*
	 *	UPnP thread chain has been stopped. 
	 *	Stop the MSL thread-chain.
	 */
	MSL_Stop(state->Msl);
	ILibStopChain(state->MslChain);

	/*
	 *	The Upnp, Ltm, Msa, and Msl objects on the thread chain will 
	 *	automatically clean themselves up.
	 *	Clean up other things.
	 */

	/*
	 *	We can safely free this array because no fields on the array are 
	 *	allocated on the heap.
	 *
	 *	TODO: Properly deallocate memory allocated in ICCI_SetupContentProviders().
	 */
	free (state->UriProvider);
	sem_destroy(&(state->IPAddressLock));
	free (state->IPAddressList);
	free (state->UDN);
	free (state->SerialNum);

	// We should not free state here and it is better to let alloc function to free it
	//free (state);

	waio_stop();
}

void ICCI_Stop(struct ICCI_State *icci_state)
{
	if (icci_state != NULL)
	{
		if (icci_state->Msl != NULL)
		{
			// Why we set Msl to NULL so early, it will cause GPF in other threads. 
			//MSL_ShuttingDownMicrostack(icci_state->Msl);
		}
		ILibStopChain(icci_state->UpnpChain);
	}
}

