/*
 *	ICC_Integration
 *
 *	This module is the integration layer for the following functions.
 *		-UPnP communications
 *		-UPnP MediaServer logic (responding to UPnP actions)
 *		-HTTP content serving
 *		-infrastructure for querying a metadata store
 *
 *	The logic for getting stuff into the metadata store,
 *	is not included in this. That logic can be either be integrated
 *	at a higher level for this application, or can be part of a
 *	separate application.
 */

#ifndef _ICC_INTEGRATION_H
#define _ICC_INTEGRATION_H

#ifdef WINSOCK2
	#include <winsock2.h>
#endif


#ifdef _POSIX
#include <semaphore.h>
#else
#include "MyThread.h" /* for sem_t */
#endif

struct ICCI_ContentProvider
{
	char* VirtualDirName;
	int VirtualDirNameLen;
	void* ResUriProvider;
	void* TransportHandler;
};

struct ICCI_State
{
	/*
	 *	These are objects on a thread-chain.
	 *	They will automatically clean up when the
	 *	chain is destroyed.
	 */
	void *Upnp;
	void *Ltm;
	void *UpnpChain;
	void *MslChain;
	void *Msa;
	void *Msl;
	void *WebServerToken;

	struct ICCI_ContentProvider *UriProvider;
	int UriProviderLen;

	/*
	 *	These are not on a chain. They must be
	 *	deallocated appropriately.
	 */
	char *UDN;			/* should be a uuid value, persisted between reboots */
	char *SerialNum;	/* should be some kind of value, persisted between reboots */

	unsigned int SortFields;	/* set to CdsSearchSort_ALL by default */
	unsigned int SearchFields;	/* set to CdsSearchSort_ALL by default */

	#ifdef ICC_FILESYSTEM
	void *FseObj;
	char *FseRootPath;
	#endif

	#ifdef ICC_ACCESSDB
	//TODO: Declare stuff specific to database back-end
	void *DbiObj;
	char *DbPath;
	#endif
	sem_t IPAddressLock;		/* read/write lock for list of IP addresses */
	int IPAddressLength;		/* number of IP addresses for the machine */
	int *IPAddressList;		/* array of IP addresses for the machine */
	
	/* WINSOCK2 IP address monitoring fields */
	#ifdef WINSOCK2
	SOCKET MonitorSocket;
	DWORD MonitorSocketReserved;
	WSAOVERLAPPED MonitorSocketStateObject;
	#endif
};

/*
 *	Set the SortFields and SearchFields fields on 'state'.
 */
void ICCI_SetSortSearchFields(struct ICCI_State *state);

/*
 *	Set the UDN field on 'state'. state->UDN will be allocated with MALLOC().
 */
void ICCI_SetUDN(struct ICCI_State *state);

/*
 *	Set the SerialNum field on 'state'. state->SerialNum will be allocated with MALLOC().
 */
void ICCI_SetSerialNum(struct ICCI_State *state);

void ICCI_SetupContentProviders(struct ICCI_State *state);

/*
 *	Start the components that are integrated by this modules.
 */
void ICCI_SetupComponentsAndStartUpnp(struct ICCI_State* icci_state);

/*
 *	Tell things to stop.
 */
void ICCI_Stop(struct ICCI_State *icci_state);


#endif
