#ifdef _POSIX
#define stricmp strcasecmp
#define strcmpi strcasecmp
#define strnicmp strncasecmp
#endif

#ifdef WIN32
#define _CRTDBG_MAP_ALLOC
#include <stdlib.h>
#ifndef _WIN32_WCE
#include <crtdbg.h>
#endif
#else
#include <stdlib.h>
#endif

#include <stdio.h>
#include <string.h>
#include "ILibAsyncSocket.h"
#include "ILibWebServer.h"
#include "UpnpMicroStack.h"
#include "ILibParsers.h"
#include "MediaServerAbstraction.h"
#include "MyString.h"
#include "debug.h"
#include "MimeTypes.h"
#include "CdsStrings.h"
#include <assert.h>

#ifdef UNDER_CE
#define strnicmp _strnicmp
#define assert ASSERT
#define sem_t HANDLE
#define sem_init(x,y,z) *x=CreateSemaphore(NULL,z,FD_SETSIZE,NULL)
#define sem_destroy(x) (CloseHandle(*x)==0?1:0)
#define sem_wait(x) WaitForSingleObject(*x,INFINITE)
#define sem_trywait(x) ((WaitForSingleObject(*x,0)==WAIT_OBJECT_0)?0:1)
#define sem_post(x) ReleaseSemaphore(*x,1,NULL)
#endif

#ifdef WIN32
#ifndef UNDER_CE
#include "assert.h"
#include <crtdbg.h>
#define sem_t HANDLE
#define sem_init(x,y,z) *x=CreateSemaphore(NULL,z,FD_SETSIZE,NULL)
#define sem_destroy(x) (CloseHandle(*x)==0?1:0)
#define sem_wait(x) WaitForSingleObject(*x,INFINITE)
#define sem_trywait(x) ((WaitForSingleObject(*x,0)==WAIT_OBJECT_0)?0:1)
#define sem_post(x) ReleaseSemaphore(*x,1,NULL)
#endif
#endif

#ifdef _POSIX
#include "assert.h"
#include <semaphore.h>
#define strnicmp strncasecmp
#endif

/*
#ifdef _DEBUG
#define MSA_MALLOC msa_malloc
#define MSA_FREE msa_free
#define ASSERT(x) assert(x)
#else
*/
#define MSA_MALLOC malloc
#define MSA_FREE free
#define ASSERT(x)
//#endif

int msa_malloc_counter = 0;

/*
void* msa_malloc(int sz)
{
	++msa_malloc_counter;
	return((void*)MALLOC(sz));
}
void msa_free(void* ptr)
{
	--msa_malloc_counter;
	FREE(ptr);	
}
*/


/*
 *	BrowseFlag related error codes - this is a UPnP layer error code,
 *	and not really an AV error code.
 */
#define MSA_ERROR_CODE_INVALID_BROWSEFLAG	402
#define MSA_ERROR_MSG_INVALID_BROWSEFLAG	"Invalid value specified for BrowseFlag."

#define MSA_ERROR_CODE_INTERNAL				500
#define	MSA_ERROR_MSG_INTERNAL				"Unknown or internal error encountered."


struct MSA_ContainerUpdate
{
	char *ContainerID;
	unsigned int UpdateID;
	struct MSA_ContainerUpdate *Next;
};

struct MSA_MediaServerObject
{
	/*
	 *	The PreSelect, PostSelect, and Destroy fields must
	 *	remain here as the object needs to be compatible with the
	 *	thread-chaining framework.
	 */
	void (*PreSelect)(void* object,fd_set *readset, fd_set *writeset, fd_set *errorset, int* blocktime);
	void (*PostSelect)(void* object,int slct, fd_set *readset, fd_set *writeset, fd_set *errorset);
	void (*Destroy)(void* object);

	/*
	 *	Pointer to generated upnp microstack object.
	 */
	void*				UpnpStack;

	/*
	 *	Lock for thread-safety.
	 */
	sem_t				Lock;

	/*
	 *	SystemUpdateID value of the media server.
	 */
	unsigned int		SystemUpdateID;

	/*
	 *	Linked list of container update IDs to print.
	 */
	struct MSA_ContainerUpdate *ContainerUpdateID;

	/*
	 *	Moderation flag for eventing.
	 */
	unsigned short				ModerationFlag;

	/* lifetime monitor object used in moderation */
	void* LifeTimeMonitor;


	/*
	 *	Supported abilities for sorting browse/search results.
	 *	SortCapabilitiesString:		concatenation of strings in MSA_CONFIG_SORT_CAPABILITIES
	 */
	char	*SortCapabilitiesString;

	/*
	 *	Supported abilities for searching results.
	 *	SearchCapabilitiesString:	concatenation of strings in MSA_CONFIG_SEARCH_CAPABILITIES
	 */
	char	*SearchCapabilitiesString;

	/*
	 *	Contains the list of IP addresses for the media server.
	 */
	int		*IpAddrList;
	int		IpAddrListLen;

	/*
	 *	The list of protocolInfo for this device as a source.
	 */
	char	*SourceProtocolInfo;

	/*
	 *	The list of protocolInfo for this device as a sink.
	 */
	char	*SinkProtocolInfo;

	/*
	 *	Miscellaneous object that is provided in MSA_CreateMediaServer.
	 */
	void	*UserObject;

	/*
	 *	Callback pointers.
	 */
	MSA_Callback_OnQuery OnQuery;
	MSA_OnStatsChanged OnStats;
};

/*
 *	Because we use hard extern callbacks, instead of
 *	function pointers, we need to keep a globally, static
 *	variable for the mediaserver.
 */
static struct MSA_MediaServerObject *MSA_TheMsaObj = NULL;




/************************************************************************************/
/* START SECTION - helper functions */

/*
 *	This method release the resources occupied
 *		msaObj					: The object returned from CreateMediaServer
 */
void MSA_DestroyMediaServer(void *msaObj)
{
	struct MSA_MediaServerObject *msa;
	struct MSA_ContainerUpdate *cu, *nextCu;

	ASSERT(msaObj);

	msa = (struct MSA_MediaServerObject*) msaObj;

	/* properly deallocate the objects for the container UpdateIDs */
	sem_wait(&(msa->Lock));

	cu = msa->ContainerUpdateID;
	while (cu != NULL)
	{
		nextCu = cu->Next;
		FREE(cu);
		cu = nextCu;
	}

	MSA_FREE(msa->SinkProtocolInfo);
	MSA_FREE(msa->SourceProtocolInfo);
	MSA_FREE(msa->SearchCapabilitiesString);
	MSA_FREE(msa->SortCapabilitiesString);
	MSA_FREE(msa->IpAddrList);

	sem_post(&(msa->Lock));

	/* Framework will free the data at 'msaObj' */
	MSA_TheMsaObj = NULL;
	sem_destroy(&(msa->Lock));
	/*FREE(msaObj);*/
}


void MSA_Helper_PopulateIpInfo(struct MSA_MediaServerObject *msa, struct MSA_CdsQuery *cdsQuery)
{
	int size, i, swapValue;

	/*
	 *	Obtain the IP address and port that received this request
	 */
	cdsQuery->RequestedOnAddress = UpnpGetLocalInterfaceToHost(cdsQuery->UpnpToken);
	cdsQuery->RequestedOnPort = UpnpGetLocalPortNumber(cdsQuery->UpnpToken);

	/*
	 *	Obtain the list of active IP addresses for this machine.
 	 *	Microstack allows us to assume that the port number
	 *	will be the same for all IP addresses.
	 */
	sem_wait(&(msa->Lock));
	cdsQuery->IpAddrListLen = msa->IpAddrListLen;
	size = (int) (sizeof(int) * cdsQuery->IpAddrListLen);
	cdsQuery->IpAddrList = (int*) MALLOC(size);
	memcpy(cdsQuery->IpAddrList, msa->IpAddrList, size);
	sem_post(&(msa->Lock));

	/*
	 *	Reorder the list of active IP addresses so that the
	 *	IP address for the interface that received the request
	 *	is listed first.
	 */
	if (cdsQuery->IpAddrList[0] != cdsQuery->RequestedOnAddress)
	{
		swapValue = cdsQuery->IpAddrList[0];
		cdsQuery->IpAddrList[0] = cdsQuery->RequestedOnAddress;
		for (i=1; i < cdsQuery->IpAddrListLen; i++)
		{
			if (cdsQuery->IpAddrList[i] == cdsQuery->RequestedOnAddress)
			{
				cdsQuery->IpAddrList[i] = swapValue;
				break;
			}
		}
	}
}

/* number of bytes needed to represent a 32bit unsigned int as a string with a comma. */
#define MSA_MAX_BYTESIZE_UINT 13
void MSA_Helper_UpdateImmediate_ContainerUpdateID(struct MSA_MediaServerObject *msa)
{
	int size;
	struct MSA_ContainerUpdate *cu, *nextCu;
	char *sv, *var;
	int writecomma;

	/* don't bother updating if there's nothing to report */
	if (msa->ContainerUpdateID == NULL) return;

	size = 0;
	//sem_wait(&(msa->Lock));

	/* calculate size needed for state variable value */
	cu = msa->ContainerUpdateID;
	while (cu != NULL)
	{
		size += (int) (MSA_MAX_BYTESIZE_UINT + strlen(cu->ContainerID));
	}
	size++;

	/*
	 *	Acquire the value of the state variable by writing it to 'var.
	 *
	 *	We progressively write 'var' by writing to 'sv'. The format
	 *	of the state variable is a comma-delimited list of
	 *	containerID/UpdateID pairs... of course, the spec authors
	 *	also made the delimiter between ContainerID and UpdateID
	 *	into a comma... how silly.
	 */
	cu = msa->ContainerUpdateID;
	var = sv = (char*) MSA_MALLOC(size);
	writecomma = 0;
	while (cu != NULL)
	{
		nextCu = cu->Next;

		if (writecomma != 0) { sv += sprintf(sv, ","); }
		sv += sprintf(sv, "%s,%u", cu->ContainerID, cu->UpdateID);

		writecomma = 1;
		FREE(cu);
		cu = nextCu;
	}
	msa->ContainerUpdateID = NULL;

	ASSERT(sv <= var+size);
	MSA_FREE(var);

	//sem_post(&(msa->Lock));
	
}

void MSA_Helper_ModerationSink_ContainerUpdateID(void* data)
{
	struct MSA_MediaServerObject *msa;

	#ifdef _DEBUG
	printf("MSA_Helper_ModerationSink_ContainerUpdateID()\r\n");
	#endif

	ASSERT(data != NULL);

	msa = (struct MSA_MediaServerObject*) data;

	sem_wait(&(msa->Lock));

	MSA_Helper_UpdateImmediate_ContainerUpdateID(msa);
	msa->ModerationFlag = 0;

	sem_post(&(msa->Lock));
}

/*
 *	Assumes caller has locked msa->Lock.
 */
void MSA_Helper_UpdateModerated_ContainerUpdateID(struct MSA_MediaServerObject *msa)
{
	/* don't bother updating if there's nothing to report */
	if (msa->ContainerUpdateID == NULL) return;

	if (msa->ModerationFlag == 0)
	{
		msa->ModerationFlag = 1;
		MSA_Helper_UpdateImmediate_ContainerUpdateID(msa);
		ILibLifeTime_Add(msa->LifeTimeMonitor,msa,1,MSA_Helper_ModerationSink_ContainerUpdateID, NULL);
	}
}

/* END SECTION - helper functions */
/************************************************************************************/



/************************************************************************************/
/* START SECTION - Dispatch sinks generated in original main.c */

void UpnpContentDirectory_GetSearchCapabilities(void* upnptoken)
{
	/*
	 *	Reports the statically defined search capabilities of the MediaServer.
	 *	You can customize this value to the abilities of the backend database
	 *	by changing MSA_CONFIG_SEARCH_CAPABILITIES_xxx variables.
	 */

	#ifdef _DEBUG
	printf("UPnP Invoke: UpnpContentDirectory_GetSearchCapabilities();\r\n");
	#endif

	UpnpResponse_ContentDirectory_GetSearchCapabilities(upnptoken, MSA_TheMsaObj->SearchCapabilitiesString);
}

void UpnpContentDirectory_GetSortCapabilities(void* upnptoken)
{
	/*
	 *	Reports the statically defined sort capabilities of the MediaServer.
	 *	You can customize this value to the abilities of the backend database
	 *	by changing MSA_CONFIG_SORT_CAPABILITIES_xxx variables.
	 */

	#ifdef _DEBUG
	printf("UPnP Invoke: UpnpContentDirectory_GetSortCapabilities();\r\n");
	#endif

	UpnpResponse_ContentDirectory_GetSortCapabilities(upnptoken, MSA_TheMsaObj->SortCapabilitiesString);
}

void UpnpContentDirectory_GetSystemUpdateID(void* upnptoken)
{
	/*
	 *	Reports the known SystemUpdateID.
	 *	The SystemUpdateID is changed through MSA_IncrementSystemUpdateID().
	 */

	#ifdef _DEBUG
	printf("UPnP Invoke: UpnpContentDirectory_GetSystemUpdateID();\r\n");
	#endif

	UpnpResponse_ContentDirectory_GetSystemUpdateID(upnptoken, MSA_TheMsaObj->SystemUpdateID);
}

void UpnpConnectionManager_GetCurrentConnectionInfo(void* upnptoken,int ConnectionID)
{
	/*
	 *	HTTP connections are stateless, from the perspective of UPnP AV.
	 *	This is largely because we can't really monitor connection lifetime
	 *	of HTTP traffic in the UPnP AV sense, without risking memory leaks.
	 *
	 *	TODO: Low priority - Add support for connection-lifetime aware protocols.
	 */

	#ifdef _DEBUG
	printf("UPnP Invoke: UpnpConnectionManager_GetCurrentConnectionInfo(%u);\r\n",ConnectionID);
	#endif

	UpnpResponse_Error(upnptoken, (int) MSA_Error_ConnectionDoesNotExist, MSA_ErrorMsg_ConnectionDoesNotExist);
}

void UpnpConnectionManager_GetCurrentConnectionIDs(void* upnptoken)
{
	/*
	 *	HTTP connections are stateless, from the perspective of UPnP AV.
	 *	This is largely because we can't really monitor connection lifetime
	 *	of HTTP traffic in the UPnP AV sense, without risking memory leaks.
	 *
	 *	TODO: Low priority - Add support for connection-lifetime aware protocols.
	 */

	#ifdef _DEBUG
	printf("UPnP Invoke: UpnpConnectionManager_GetCurrentConnectionIDs();\r\n");
	#endif

	UpnpResponse_ConnectionManager_GetCurrentConnectionIDs(upnptoken, "0");
}

void UpnpConnectionManager_GetProtocolInfo(void* upnptoken)
{
	//	char str[12000];
	/*
	//char str1[]="http-get:*:image/jpeg:DLNA.ORG_PN=JPEG_SM,http-get:*:audio/L16:DLNA.ORG_PN=LPCM,http-get:*:video/mpeg:DLNA.ORG_PN=MPEG_PS_NTSC,http-get:*:video/mpeg:DLNA.ORG_PN=MPEG_PS_PAL,http-get:*:video/mpeg:DLNA.ORG_PN=MPEG_TS_SD_NA,http-get:*:video/mpeg:DLNA.ORG_PN=MPEG_TS_SD_NA_T,http-get:*:video/mpeg:DLNA.ORG_PN=MPEG_TS_HD_NA,http-get:*:video/mpeg:DLNA.ORG_PN=MPEG_TS_HD_NA_T,http-get:*:video/mpeg:DLNA.ORG_PN=MPEG_TS_SD_EU,http-get:*:video/mpeg:DLNA.ORG_PN=MPEG_TS_SD_EU_T,http-get:*:video/mpeg:DLNA.ORG_PN=MPEG_TS_SD_KO,http-get:*:video/mpeg:DLNA.ORG_PN=MPEG_TS_SD_KO_T,http-get:*:video/mpeg:DLNA.ORG_PN=MPEG_TS_HD_KO,http-get:*:video/mpeg:DLNA.ORG_PN=MPEG_TS_HD_KO_T,http-get:*:image/jpeg:DLNA.ORG_PN=JPEG_MED,http-get:*:image/jpeg:DLNA.ORG_PN=JPEG_LRG,http-get:*:image/jpeg:DLNA.ORG_PN=JPEG_TN,http-get:*:image/jpeg:DLNA.ORG_PN=JPEG_SM_ICO,http-get:*:image/jpeg:DLNA.ORG_PN=JPEG_LRG_ICO,http-get:*:image/png:DLNA.ORG_PN=PNG_TN,http-get:*:image/png:DLNA.ORG_PN=PNG_SM_ICO,http-get:*:image/png:DLNA.ORG_PN=PNG_LRG_ICO,http-get:*:audio/vnd.dlna.adts:DLNA.ORG_PN=AAC_ADTS,http-get:*:audio/mp4:DLNA.ORG_PN=AAC_ISO,http-get:*:audio/3gpp:DLNA.ORG_PN=AAC_ISO,http-get:*:audio/mp4:DLNA.ORG_PN=AAC_LPT_ISO,http-get:*:audio/3gpp:DLNA.ORG_PN=AAC_LPT_ISO,http-get:*:audio/vnd.dlna.adts:DLNA.ORG_PN=AAC_LPT_ISO,http-get:*:audio/mp4:DLNA.ORG_PN=AAC_LPT_MULT5_ISO,http-get:*:audio/3gpp:DLNA.ORG_PN=AAC_LPT_MULT5_ISO,http-get:*:audio/vnd.dlna.adts:DLNA.ORG_PN=AAC_LPT_MULT5_ISO,\0";
	char str1[]="http-get:*:image/jpeg:DLNA.ORG_PN=JPEG_SM,http-get:*:audio/L16:DLNA.ORG_PN=LPCM,http-get:*:video/mpeg:DLNA.ORG_PN=MPEG_PS_NTSC,http-get:*:video/mpeg:DLNA.ORG_PN=MPEG_PS_PAL,http-get:*:video/mpeg:DLNA.ORG_PN=MPEG_TS_SD_NA,http-get:*:video/mpeg:DLNA.ORG_PN=MPEG_TS_SD_NA_T,http-get:*:video/mpeg:DLNA.ORG_PN=MPEG_TS_HD_NA,http-get:*:video/mpeg:DLNA.ORG_PN=MPEG_TS_HD_NA_T,http-get:*:video/mpeg:DLNA.ORG_PN=MPEG_TS_SD_EU,http-get:*:video/mpeg:DLNA.ORG_PN=MPEG_TS_SD_EU_T,http-get:*:video/mpeg:DLNA.ORG_PN=MPEG_TS_SD_KO,http-get:*:video/mpeg:DLNA.ORG_PN=MPEG_TS_SD_KO_T,http-get:*:video/mpeg:DLNA.ORG_PN=MPEG_TS_HD_KO,http-get:*:video/mpeg:DLNA.ORG_PN=MPEG_TS_HD_KO_T,http-get:*:image/jpeg:DLNA.ORG_PN=JPEG_MED,http-get:*:image/jpeg:DLNA.ORG_PN=JPEG_LRG,http-get:*:image/jpeg:DLNA.ORG_PN=JPEG_TN,http-get:*:image/jpeg:DLNA.ORG_PN=JPEG_SM_ICO,http-get:*:image/jpeg:DLNA.ORG_PN=JPEG_LRG_ICO,http-get:*:image/png:DLNA.ORG_PN=PNG_TN,http-get:*:image/png:DLNA.ORG_PN=PNG_SM_ICO,http-get:*:image/png:DLNA.ORG_PN=PNG_LRG_ICO,http-get:*:audio/vnd.dlna.adts:DLNA.ORG_PN=AAC_ADTS,http-get:*:audio/mp4:DLNA.ORG_PN=AAC_ISO,http-get:*:audio/3gpp:DLNA.ORG_PN=AAC_ISO,http-get:*:audio/mp4:DLNA.ORG_PN=AAC_LPT_ISO,http-get:*:audio/3gpp:DLNA.ORG_PN=AAC_LPT_ISO,http-get:*:audio/vnd.dlna.adts:DLNA.ORG_PN=AAC_LPT_ISO,http-get:*:audio/mp4:DLNA.ORG_PN=AAC_LPT_MULT5_ISO,http-get:*:audio/3gpp:DLNA.ORG_PN=AAC_LPT_MULT5_ISO,http-get:*:audio/vnd.dlna.adts:DLNA.ORG_PN=AAC_LPT_MULT5_ISO,\0";
	char str2[]="http-get:*:audio/mp4:DLNA.ORG_PN=AAC_LPT_MULT7_ISO,http-get:*:audio/3gpp:DLNA.ORG_PN=AAC_LPT_MULT7_ISO,http-get:*:audio/vnd.dlna.adts:DLNA.ORG_PN=AAC_LPT_MULT7_ISO,http-get:*:audio/vnd.dlna.adts:DLNA.ORG_PN=AAC_MULT5_ADTS,http-get:*:audio/mp4:DLNA.ORG_PN=AAC_MULT5_ISO,http-get:*:audio/3gpp:DLNA.ORG_PN=AAC_MULT5_ISO,http-get:*:audio/vnd.dlna.adts:DLNA.ORG_PN=HEAAC_L2_ADTS,http-get:*:audio/mp4:DLNA.ORG_PN=HEAAC_L2_ISO,http-get:*:audio/3gpp:DLNA.ORG_PN=HEAAC_L2_ISO,http-get:*:audio/vnd.dlna.adts:DLNA.ORG_PN=HEAAC_L3_ADTS,http-get:*:audio/mp4:DLNA.ORG_PN=HEAAC_L3_ISO,http-get:*:audio/3gpp:DLNA.ORG_PN=HEAAC_L3_ISO,http-get:*:audio/vnd.dlna.adts:DLNA.ORG_PN=HEAAC_MULT5_ADTS,\0";
	char str3[]="http-get:*:audio/mp4:DLNA.ORG_PN=HEAAC_MULT5_ISO,http-get:*:audio/3gpp:DLNA.ORG_PN=HEAAC_MULT5_ISO,http-get:*:audio/vnd.dolby.dd-raw:DLNA.ORG_PN=AC3,http-get:*:audio/x-sony-oma:DLNA.ORG_PN=ATRAC3plus,http-get:*:audio/mpeg:DLNA.ORG_PN=MP3,http-get:*:video/mp4:DLNA.ORG_PN=MPEG4_P2_MP4_SP_AAC,http-get:*:video/mp4:DLNA.ORG_PN=MPEG4_P2_MP4_SP_HEAAC,http-get:*:video/mp4:DLNA.ORG_PN=MPEG4_P2_MP4_SP_ATRAC3plus,http-get:*:video/3gpp:DLNA.ORG_PN=MPEG4_P2_MP4_SP_AAC_LPT,http-get:*:video/vnd.dlna.mpeg-tts:DLNA.ORG_PN=MPEG4_P2_TS_SP_AAC,http-get:*:video/vnd.dlna.mpeg-tts:DLNA.ORG_PN=MPEG4_P2_TS_SP_AAC_T,http-get:*:video/vnd.dlna.mpeg-tts:DLNA.ORG_PN=MPEG4_P2_TS_SP_MPEG1_L3,http-get:*:video/vnd.dlna.mpeg-tts:DLNA.ORG_PN=MPEG4_P2_TS_SP_MPEG1_L3_T,http-get:*:video/vnd.dlna.mpeg-tts:DLNA.ORG_PN=MPEG4_P2_TS_SP_AC3,http-get:*:video/vnd.dlna.mpeg-tts:DLNA.ORG_PN=MPEG4_P2_TS_SP_AC3_T,http-get:*:video/vnd.dlna.mpeg-tts:DLNA.ORG_PN=MPEG4_P2_TS_SP_MPEG2_L2,http-get:*:video/vnd.dlna.mpeg-tts:DLNA.ORG_PN=MPEG4_P2_TS_SP_MPEG2_L2_T,http-get:*:video/x-ms-asf:DLNA.ORG_PN=MPEG4_P2_ASF_SP_G726,http-get:*:video/mp4:DLNA.ORG_PN=MPEG4_P2_MP4_ASP_AAC,http-get:*:video/mp4:DLNA.ORG_PN=MPEG4_P2_MP4_ASP_HEAAC,http-get:*:video/mp4:DLNA.ORG_PN=MPEG4_P2_MP4_ASP_HEAAC_MULT5,http-get:*:video/mp4:DLNA.ORG_PN=MPEG4_P2_MP4_ASP_ATRAC3plus,http-get:*:video/vnd.dlna.mpeg-tts:DLNA.ORG_PN=MPEG4_P2_TS_ASP_AAC,http-get:*:video/vnd.dlna.mpeg-tts:DLNA.ORG_PN=MPEG4_P2_TS_ASP_AAC_T,\0";
	char str4[]="http-get:*:video/vnd.dlna.mpeg-tts:DLNA.ORG_PN=MPEG4_P2_TS_ASP_MPEG1_L3,http-get:*:video/vnd.dlna.mpeg-tts:DLNA.ORG_PN=MPEG4_P2_TS_ASP_MPEG1_L3_T,http-get:*:video/vnd.dlna.mpeg-tts:DLNA.ORG_PN=MPEG4_P2_TS_ASP_AC3,http-get:*:video/vnd.dlna.mpeg-tts:DLNA.ORG_PN=MPEG4_P2_TS_ASP_AC3_T,http-get:*:video/mp4:DLNA.ORG_PN=MPEG4_P2_MP4_ASP_L5_SO_AAC,http-get:*:video/mp4:DLNA.ORG_PN=MPEG4_P2_MP4_ASP_L5_SO_HEAAC,http-get:*:video/mp4:DLNA.ORG_PN=MPEG4_P2_MP4_ASP_L5_SO_HEAAC_MULT5,http-get:*:video/x-ms-asf:DLNA.ORG_PN=MPEG4_P2_ASF_ASP_L5_SO_G726,http-get:*:video/mp4:DLNA.ORG_PN=MPEG4_P2_MP4_ASP_L4_SO_AAC,http-get:*:video/mp4:DLNA.ORG_PN=MPEG4_P2_MP4_ASP_L4_SO_HEAAC,http-get:*:video/mp4:DLNA.ORG_PN=MPEG4_P2_MP4_ASP_L4_SO_HEAAC_MULT5,http-get:*:video/x-ms-asf:DLNA.ORG_PN=MPEG4_P2_ASF_ASP_L4_SO_G726,http-get:*:video/3gpp:DLNA.ORG_PN=MPEG4_H263_MP4_P0_L10_AAC,http-get:*:video/3gpp:DLNA.ORG_PN=MPEG4_H263_MP4_P0_L10_AAC_LPT,http-get:*:video/vnd.dlna.mpeg-tts:DLNA.ORG_PN=MPEG4_P2_TS_CO_AC3,http-get:*:video/vnd.dlna.mpeg-tts:DLNA.ORG_PN=MPEG4_P2_TS_CO_AC3_T,http-get:*:video/vnd.dlna.mpeg-tts:DLNA.ORG_PN=MPEG4_P2_TS_CO_MPEG2_L2,http-get:*:video/vnd.dlna.mpeg-tts:DLNA.ORG_PN=MPEG4_P2_TS_CO_MPEG2_L2_T,http-get:*:video/vnd.dlna.mpeg-tts:DLNA.ORG_PN=AVC_TS_MP_SD_AAC_MULT5,http-get:*:video/vnd.dlna.mpeg-tts:DLNA.ORG_PN=AVC_TS_MP_SD_AAC_MULT5_T,http-get:*:video/vnd.dlna.mpeg-tts:DLNA.ORG_PN=AVC_TS_MP_SD_HEAAC_L2,http-get:*:video/vnd.dlna.mpeg-tts:DLNA.ORG_PN=AVC_TS_MP_SD_HEAAC_L2_T,http-get:*:video/vnd.dlna.mpeg-tts:DLNA.ORG_PN=AVC_TS_MP_SD_MPEG1_L3,http-get:*:video/vnd.dlna.mpeg-tts:DLNA.ORG_PN=AVC_TS_MP_SD_MPEG1_L3_T,http-get:*:video/vnd.dlna.mpeg-tts:DLNA.ORG_PN=AVC_TS_MP_SD_AC3,http-get:*:video/vnd.dlna.mpeg-tts:DLNA.ORG_PN=AVC_TS_MP_SD_AC3_T,http-get:*:video/vnd.dlna.mpeg-tts:DLNA.ORG_PN=AVC_TS_MP_SD_AAC_LTP,\0";
	char str5[]="http-get:*:video/vnd.dlna.mpeg-tts:DLNA.ORG_PN=AVC_TS_MP_SD_AAC_LTP_T,http-get:*:video/vnd.dlna.mpeg-tts:DLNA.ORG_PN=AVC_TS_MP_SD_AAC_LTP_MULT5,http-get:*:video/vnd.dlna.mpeg-tts:DLNA.ORG_PN=AVC_TS_MP_SD_AAC_LTP_MULT5_T,http-get:*:video/vnd.dlna.mpeg-tts:DLNA.ORG_PN=AVC_TS_MP_SD_AAC_LTP_MULT7,http-get:*:video/vnd.dlna.mpeg-tts:DLNA.ORG_PN=AVC_TS_MP_SD_AAC_LTP_MULT7_T,http-get:*:video/vnd.dlna.mpeg-tts:DLNA.ORG_PN=AVC_MP4_MP_SD_AAC_MULT5,http-get:*:video/mp4:DLNA.ORG_PN=AVC_MP4_MP_SD_HEAAC_L2,http-get:*:video/mp4:DLNA.ORG_PN=AVC_MP4_MP_SD_MPEG1_L3,http-get:*:video/mp4:DLNA.ORG_PN=AVC_MP4_MP_SD_AC3,http-get:*:video/mp4:DLNA.ORG_PN=AVC_MP4_MP_SD_AAC_LTP,http-get:*:video/mp4:DLNA.ORG_PN=AVC_MP4_MP_SD_AAC_LTP_MULT5,http-get:*:video/mp4:DLNA.ORG_PN=AVC_MP4_MP_SD_AAC_LTP_MULT7,http-get:*:video/mp4:DLNA.ORG_PN=AVC_MP4_MP_SD_ATRAC3plus,http-get:*:video/vnd.dlna.mpeg-tts:DLNA.ORG_PN=AVC_TS_BL_CIF30_AAC_MULT5,http-get:*:video/vnd.dlna.mpeg-tts:DLNA.ORG_PN=AVC_TS_BL_CIF30_AAC_MULT5_T,http-get:*:video/vnd.dlna.mpeg-tts:DLNA.ORG_PN=AVC_TS_BL_CIF30_HEAAC_L2,http-get:*:video/vnd.dlna.mpeg-tts:DLNA.ORG_PN=AVC_TS_BL_CIF30_HEAAC_L2_T,http-get:*:video/vnd.dlna.mpeg-tts:DLNA.ORG_PN=AVC_TS_BL_CIF30_MPEG_L3,http-get:*:video/vnd.dlna.mpeg-tts:DLNA.ORG_PN=AVC_TS_BL_CIF30_MPEG_L3_T,http-get:*:video/vnd.dlna.mpeg-tts:DLNA.ORG_PN=AVC_TS_BL_CIF30_AC3,http-get:*:video/vnd.dlna.mpeg-tts:DLNA.ORG_PN=AVC_TS_BL_CIF30_AC3_T,http-get:*:video/vnd.dlna.mpeg-tts:DLNA.ORG_PN=AVC_TS_BL_CIF30_AAC_LTP,http-get:*:video/vnd.dlna.mpeg-tts:DLNA.ORG_PN=AVC_TS_BL_CIF30_AAC_LTP_T,http-get:*:video/vnd.dlna.mpeg-tts:DLNA.ORG_PN=AVC_TS_BL_CIF30_AAC_LTP_MULT5,http-get:*:video/vnd.dlna.mpeg-tts:DLNA.ORG_PN=AVC_TS_BL_CIF30_AAC_LTP_MULT5_T,http-get:*:video/mp4:DLNA.ORG_PN=AVC_MP4_BL_CIF30_AAC_MULT5,\0";
	char str6[]="http-get:*:video/mp4:DLNA.ORG_PN=AVC_MP4_BL_CIF30_HEAAC_L2,http-get:*:video/mp4:DLNA.ORG_PN=AVC_MP4_BL_CIF30_MPEG1_L3,http-get:*:video/mp4:DLNA.ORG_PN=AVC_MP4_BL_CIF30_AC3,http-get:*:video/mp4:DLNA.ORG_PN=AVC_MP4_BL_CIF30_AAC_LTP,http-get:*:video/mp4:DLNA.ORG_PN=AVC_MP4_BL_CIF30_AAC_LTP_MULT5,http-get:*:video/vnd.dlna.mpeg-tts:DLNA.ORG_PN=AVC_TS_MP_HD_AAC_MULT5,http-get:*:video/vnd.dlna.mpeg-tts:DLNA.ORG_PN=AVC_TS_MP_HD_AAC_MULT5_T,http-get:*:video/vnd.dlna.mpeg-tts:DLNA.ORG_PN=AVC_TS_MP_HD_HEAAC_L2,http-get:*:video/vnd.dlna.mpeg-tts:DLNA.ORG_PN=AVC_TS_MP_HD_HEAAC_L2_T,http-get:*:video/vnd.dlna.mpeg-tts:DLNA.ORG_PN=AVC_TS_MP_HD_MPEG1_L3,http-get:*:video/vnd.dlna.mpeg-tts:DLNA.ORG_PN=AVC_TS_MP_HD_MPEG1_L3_T,http-get:*:video/vnd.dlna.mpeg-tts:DLNA.ORG_PN=AVC_TS_MP_HD_AC3,http-get:*:video/vnd.dlna.mpeg-tts:DLNA.ORG_PN=AVC_TS_MP_HD_AC3_T,http-get:*:video/vnd.dlna.mpeg-tts:DLNA.ORG_PN=AVC_TS_MP_HD_AAC_LTP,http-get:*:video/vnd.dlna.mpeg-tts:DLNA.ORG_PN=AVC_TS_MP_HD_AAC_LTP_T,http-get:*:video/vnd.dlna.mpeg-tts:DLNA.ORG_PN=AVC_TS_MP_HD_AAC_LTP_MULT5,http-get:*:video/vnd.dlna.mpeg-tts:DLNA.ORG_PN=AVC_TS_MP_HD_AAC_LTP_MULT5_T,http-get:*:video/vnd.dlna.mpeg-tts:DLNA.ORG_PN=AVC_TS_MP_HD_AAC_LTP_MULT7,http-get:*:video/vnd.dlna.mpeg-tts:DLNA.ORG_PN=AVC_TS_MP_HD_AAC_LTP_MULT7_T,http-get:*:video/vnd.dlna.mpeg-tts:DLNA.ORG_PN=AVC_TS_BL_CIF15_AAC,http-get:*:video/vnd.dlna.mpeg-tts:DLNA.ORG_PN=AVC_TS_BL_CIF15_AAC_T,\0";
	char str7[]="http-get:*:video/vnd.dlna.mpeg-tts:DLNA.ORG_PN=AVC_TS_BL_CIF15_AAC_LTP,http-get:*:video/vnd.dlna.mpeg-tts:DLNA.ORG_PN=AVC_TS_BL_CIF15_AAC_LTP_T,http-get:*:video/mp4:DLNA.ORG_PN=AVC_MP4_BL_CIF15_AAC,http-get:*:video/mp4:DLNA.ORG_PN=AVC_MP4_BL_CIF15_AAC_LTP,http-get:*:video/3gpp:DLNA.ORG_PN=AVC_3GPP_BL_QCIF15_AAC,http-get:*:video/3gpp:DLNA.ORG_PN=AVC_3GPP_BL_QCIF15_AAC_LTP,http-get:*:video/vnd.dlna.mpeg-tts:DLNA.ORG_PN=MPEG_TS_HD_KO_XAC3,http-get:*:video/vnd.dlna.mpeg-tts:DLNA.ORG_PN=MPEG_TS_HD_KO_XAC3_T,http-get:*:video/vnd.dlna.mpeg-tts:DLNA.ORG_PN=MPEG_TS_HD_NA_XAC3,http-get:*:video/vnd.dlna.mpeg-tts:DLNA.ORG_PN=MPEG_TS_HD_NA_XAC3_T,http-get:*:video/mpeg:DLNA.ORG_PN=MPEG_PS_NTSC_XAC3,http-get:*:video/mpeg:DLNA.ORG_PN=MPEG_PS_PAL_XAC3,http-get:*:video/vnd.dlna.mpeg-tts:DLNA.ORG_PN=MPEG_TS_SD_KO_XAC3,http-get:*:video/vnd.dlna.mpeg-tts:DLNA.ORG_PN=MPEG_TS_SD_KO_XAC3_T,http-get:*:video/vnd.dlna.mpeg-tts:DLNA.ORG_PN=MPEG_TS_SD_NA_XAC3,http-get:*:video/vnd.dlna.mpeg-tts:DLNA.ORG_PN=MPEG_TS_SD_NA_XAC3_T,http-get:*:audio/x-ms-wma:DLNA.ORG_PN=WMABASE,http-get:*:audio/x-ms-wma:DLNA.ORG_PN=WMAFULL,http-get:*:audio/x-ms-wma:DLNA.ORG_PN=WMAPRO,http-get:*:video/x-ms-wmv:DLNA.ORG_PN=WMVMED_BASE,http-get:*:video/x-ms-wmv:DLNA.ORG_PN=WMVMED_FULL,http-get:*:video/x-ms-wmv:DLNA.ORG_PN=WMVMED_PRO,http-get:*:video/x-ms-wmv:DLNA.ORG_PN=WMVHIGH_FULL,http-get:*:video/x-ms-wmv:DLNA.ORG_PN=WMVHIGH_PRO,http-get:*:video/x-ms-wmv:DLNA.ORG_PN=WMVVHM_BASE\0";
	sprintf(str,"%s%s%s%s%s%s%s",str1,str2,str3,str4,str5,str6,str7);
	*/

	//char str[] = "http-get:*:image/jpeg:DLNA.ORG_PN=JPEG_SM,http-get:*:image/jpeg:DLNA.ORG_PN=JPEG_MED,http-get:*:image/jpeg:DLNA.ORG_PN=JPEG_LRG,http-get:*:image/jpeg:DLNA.ORG_PN=JPEG_TN,http-get:*:image/jpeg:DLNA.ORG_PN=JPEG_SM_ICO,http-get:*:image/jpeg:DLNA.ORG_PN=JPEG_LRG_ICO,http-get:*:image/png:DLNA.ORG_PN=PNG_TN,http-get:*:image/png:DLNA.ORG_PN=PNG_SM_ICO,http-get:*:image/png:DLNA.ORG_PN=PNG_LRG_ICO,http-get:*:audio/L16;rate=44100;channels=1:DLNA.ORG_PN=LPCM,http-get:*:audio/L16;rate=44100;channels=2:DLNA.ORG_PN=LPCM,http-get:*:audio/L16;rate=48000;channels=1:DLNA.ORG_PN=LPCM,http-get:*:audio/L16;rate=48000;channels=2:DLNA.ORG_PN=LPCM,http-get:*:video/mpeg:DLNA.ORG_PN=MPEG_PS_NTSC,http-get:*:video/mpeg:DLNA.ORG_PN=MPEG_PS_PAL,http-get:*:video/vnd.dlna.mpeg-tts:DLNA.ORG_PN=MPEG_TS_SD_NA,http-get:*:video/vnd.dlna.mpeg-tts:DLNA.ORG_PN=MPEG_TS_SD_NA_T,http-get:*:video/vnd.dlna.mpeg-tts:DLNA.ORG_PN=MPEG_TS_HD_NA,http-get:*:video/vnd.dlna.mpeg-tts:DLNA.ORG_PN=MPEG_TS_HD_NA_T,http-get:*:video/vnd.dlna.mpeg-tts:DLNA.ORG_PN=MPEG_TS_SD_EU,http-get:*:video/vnd.dlna.mpeg-tts:DLNA.ORG_PN=MPEG_TS_SD_EU_T,http-get:*:video/vnd.dlna.mpeg-tts:DLNA.ORG_PN=MPEG_TS_SD_KO,http-get:*:video/vnd.dlna.mpeg-tts:DLNA.ORG_PN=MPEG_TS_SD_KO_T,http-get:*:video/vnd.dlna.mpeg-tts:DLNA.ORG_PN=MPEG_TS_HD_KO,http-get:*:video/vnd.dlna.mpeg-tts:DLNA.ORG_PN=MPEG_TS_HD_KO_T";
	char str[] = "http-get:*:image/jpeg:DLNA.ORG_PN=JPEG_SM,http-get:*:image/jpeg:DLNA.ORG_PN=JPEG_MED,http-get:*:image/jpeg:DLNA.ORG_PN=JPEG_LRG,http-get:*:image/jpeg:DLNA.ORG_PN=JPEG_TN,http-get:*:image/jpeg:DLNA.ORG_PN=JPEG_LRG_ICO,http-get:*:image/png:DLNA.ORG_PN=PNG_TN,http-get:*:image/png:DLNA.ORG_PN=PNG_SM_ICO,http-get:*:image/png:DLNA.ORG_PN=PNG_LRG_ICO,http-get:*:audio/L16;rate=44100;channels=1:DLNA.ORG_PN=LPCM,http-get:*:audio/L16;rate=44100;channels=2:DLNA.ORG_PN=LPCM,http-get:*:audio/L16;rate=48000;channels=1:DLNA.ORG_PN=LPCM,http-get:*:audio/L16;rate=48000;channels=2:DLNA.ORG_PN=LPCM,http-get:*:video/mpeg:DLNA.ORG_PN=MPEG_PS_NTSC,http-get:*:video/mpeg:DLNA.ORG_PN=MPEG_PS_PAL,http-get:*:video/vnd.dlna.mpeg-tts:DLNA.ORG_PN=MPEG_TS_SD_NA,http-get:*:video/vnd.dlna.mpeg-tts:DLNA.ORG_PN=MPEG_TS_SD_NA_T,http-get:*:video/vnd.dlna.mpeg-tts:DLNA.ORG_PN=MPEG_TS_HD_NA,http-get:*:video/vnd.dlna.mpeg-tts:DLNA.ORG_PN=MPEG_TS_HD_NA_T,http-get:*:video/vnd.dlna.mpeg-tts:DLNA.ORG_PN=MPEG_TS_SD_EU,http-get:*:video/vnd.dlna.mpeg-tts:DLNA.ORG_PN=MPEG_TS_SD_EU_T,http-get:*:video/vnd.dlna.mpeg-tts:DLNA.ORG_PN=MPEG_TS_SD_KO,http-get:*:video/vnd.dlna.mpeg-tts:DLNA.ORG_PN=MPEG_TS_SD_KO_T,http-get:*:video/vnd.dlna.mpeg-tts:DLNA.ORG_PN=MPEG_TS_HD_KO,http-get:*:video/vnd.dlna.mpeg-tts:DLNA.ORG_PN=MPEG_TS_HD_KO_T,http-get:*:image/jpeg:DLNA.ORG_PN=JPEG_SM_ICO";
	
	#ifdef _DEBUG
	printf("UPnP Invoke: UpnpConnectionManager_GetProtocolInfo();\r\n");
	#endif

	sem_wait(&(MSA_TheMsaObj->Lock));
	sem_post(&(MSA_TheMsaObj->Lock));

	// audio/L16:DLNA.ORG_PN=LPCM

	/*
	sprintf(str,"http-get:*:image/jpeg:DLNA.ORG_PN=JPEG_SM,\
				http-get:*:audio/L16:DLNA.ORG_PN=LPCM,\
				http-get:*:video/mpeg:DLNA.ORG_PN=MPEG_PS_NTSC,\
				http-get:*:video/mpeg:DLNA.ORG_PN=MPEG_PS_PAL,\
				http-get:*:video/mpeg:DLNA.ORG_PN=MPEG_TS_SD_NA,\
				http-get:*:video/mpeg:DLNA.ORG_PN=MPEG_TS_SD_NA_T,\
				http-get:*:video/mpeg:DLNA.ORG_PN=MPEG_TS_HD_NA,\
				http-get:*:video/mpeg:DLNA.ORG_PN=MPEG_TS_HD_NA_T,\
				http-get:*:video/mpeg:DLNA.ORG_PN=MPEG_TS_SD_EU,\
				http-get:*:video/mpeg:DLNA.ORG_PN=MPEG_TS_SD_EU_T,\
				http-get:*:video/mpeg:DLNA.ORG_PN=MPEG_TS_SD_KO,\
				http-get:*:video/mpeg:DLNA.ORG_PN=MPEG_TS_SD_KO_T,\
				http-get:*:video/mpeg:DLNA.ORG_PN=MPEG_TS_HD_KO,\
				http-get:*:video/mpeg:DLNA.ORG_PN=MPEG_TS_HD_KO_T,\
				http-get:*:image/jpeg:DLNA.ORG_PN=JPEG_MED,\
				http-get:*:image/jpeg:DLNA.ORG_PN=JPEG_LRG,\
				http-get:*:image/jpeg:DLNA.ORG_PN=JPEG_TN,\
				http-get:*:image/jpeg:DLNA.ORG_PN=JPEG_SM_ICO,\
				http-get:*:image/jpeg:DLNA.ORG_PN=JPEG_LRG_ICO,\
				http-get:*:image/png:DLNA.ORG_PN=PNG_TN,\
				http-get:*:image/png:DLNA.ORG_PN=PNG_SM_ICO,\
				http-get:*:image/png:DLNA.ORG_PN=PNG_LRG_ICO,\
				http-get:*:video/mpeg:DLNA.ORG_PN=MPEG_PS_NTSC_XAC3,\
				http-get:*:video/mpeg:DLNA.ORG_PN=MPEG_PS_PAL_XAC3,\
				http-get:*:audio/x-ms-wma:DLNA.ORG_PN=WMABASE,\
				http-get:*:audio/x-ms-wma:DLNA.ORG_PN=WMAFULL,\
				http-get:*:audio/x-ms-wma:DLNA.ORG_PN=WMAPRO,\
				http-get:*:video/x-ms-wmv:DLNA.ORG_PN=WMVMED_BASE,\
				http-get:*:video/x-ms-wmv:DLNA.ORG_PN=WMVMED_FULL,\
				http-get:*:video/x-ms-wmv:DLNA.ORG_PN=WMVMED_PRO,\
				http-get:*:video/x-ms-wmv:DLNA.ORG_PN=WMVHIGH_FULL,\
				http-get:*:video/x-ms-wmv:DLNA.ORG_PN=WMVHIGH_PRO,\
				http-get:*:video/x-ms-wmv:DLNA.ORG_PN=WMVVHM_BASE\0");
	*/

	UpnpResponse_ConnectionManager_GetProtocolInfo(upnptoken, str,"");

	/* TODO: modify these values to match capabilities of the server */
	/*
	UpnpResponse_ConnectionManager_GetProtocolInfo(upnptoken, 
		"http-get:*:audio/lpcm:*,
		http-get:*:audio/mpeg:*,
		http-get:*:audio/x-mpeg:*,
		http-get:*:audio/mpg:*,
		http-get:*:audio/mp3:*,
		http-get:*:video/mpeg:*,
		http-get:*:video/mpeg2:*,
		http-get:*:video/x-mpeg:*,
		http-get:*:video/mpg:*,
		http-get:*:video/MP1S:*,
		http-get:*:image/jpeg:*,
		http-get:*:image/jpg:*,
		http-get:*:image/bmp:*,
		http-get:*:image/x-ms-bmp:*,
		http-get:*:audio/mpegurl:*,
		http-get:*:audio/x-mpegurl:*", "");
		*/
}



void UpnpContentDirectory_Browse(void* upnptoken,char* ObjectID,char* BrowseFlag,char* Filter,unsigned int StartingIndex,unsigned int RequestedCount,char* SortCriteria)
{
	struct MSA_CdsQuery *browseArgs;
	int errorCode;
	const char *errorMsg;
	enum MSA_Enum_QueryTypes browseChildren;
	int size;

	#ifdef _DEBUG
	printf("UPnP Invoke: UpnpContentDirectory_Browse();\r\n");
	#endif

	/*
	 *	Validate arguments.
	 */

	errorCode = 0;
	errorMsg = NULL;
	if (strcmpi(BrowseFlag, CDS_STRING_BROWSE_DIRECT_CHILDREN) == 0)
	{
		browseChildren = MSA_Query_BrowseDirectChildren;
	}
	else if (strcmpi(BrowseFlag, CDS_STRING_BROWSE_METADATA) == 0)
	{
		browseChildren = MSA_Query_BrowseMetadata;
	}
	else
	{
		fprintf(stderr, "WARNING: UpnpContentDirectory_Browse(): Possible error with generated microstack. Encountered BrowseFlag='%s'\r\n", BrowseFlag);
		errorCode = MSA_ERROR_CODE_INVALID_BROWSEFLAG;
		errorMsg = MSA_ERROR_MSG_INVALID_BROWSEFLAG;
	}

	if ((errorCode != 0) || (errorMsg != NULL))
	{
		/* ensure that the error code and message map to something */
		if (errorCode == 0) { errorCode = MSA_ERROR_CODE_INTERNAL; }
		if (errorMsg == NULL) {	errorMsg = MSA_ERROR_MSG_INTERNAL; }

		UpnpResponse_Error(upnptoken, errorCode, errorMsg);
	}
	else
	{
		/*
		 *	Input arguments valid at UPnP layer.
		 *	Create an MSA_CdsQuery object and execute
		 *	the browse callback so that application can return results.
		 */

		browseArgs = (struct MSA_CdsQuery*) MSA_MALLOC (sizeof(struct MSA_CdsQuery));
		memset(browseArgs, 0, sizeof(struct MSA_CdsQuery));

		browseArgs->QueryType = browseChildren;
		
		size = (int) strlen(Filter)+1;
		browseArgs->Filter = (char*) MSA_MALLOC(size);
		memcpy(browseArgs->Filter, Filter, size);

		browseArgs->MediaServerObject = MSA_TheMsaObj;
		
		size = (int) strlen(ObjectID)+1;
		browseArgs->ObjectID = (char*) MSA_MALLOC(size);
		memcpy(browseArgs->ObjectID, ObjectID, size);
		/* Be sure to unescape it first. */
		ILibInPlaceXmlUnEscape(browseArgs->ObjectID);

		browseArgs->RequestedCount = RequestedCount;

		size = (int) strlen(SortCriteria)+1;
		browseArgs->SortCriteria = (char*) MSA_MALLOC(size);
		memcpy(browseArgs->SortCriteria, SortCriteria, size);

		browseArgs->StartingIndex = StartingIndex;

		browseArgs->UpnpToken = upnptoken;
		browseArgs->UserObject = NULL;

		browseArgs->IpAddrList = NULL;
		MSA_Helper_PopulateIpInfo(MSA_TheMsaObj, browseArgs);

		if (MSA_TheMsaObj->OnQuery!= NULL)
		{
			MSA_TheMsaObj->OnQuery(MSA_TheMsaObj, browseArgs, MSA_TheMsaObj->UserObject);
		}
	}
}

void UpnpContentDirectory_Search(void* upnptoken,char* ContainerID,char* SearchCriteria, char* Filter,unsigned int StartingIndex,unsigned int RequestedCount,char* SortCriteria)
{
	struct MSA_CdsQuery *searchArgs;
	int size;

	#ifdef _DEBUG
	printf("UPnP Invoke: UpnpContentDirectory_Search();\r\n");
	#endif

	/*
	 *	Validate arguments.
	 */

	/*
	 *	Input arguments valid at UPnP layer.
	 *	Create an MSA_CdsQuery object and execute
	 *	the browse callback so that application can return results.
	 */

	searchArgs = (struct MSA_CdsQuery*) MSA_MALLOC (sizeof(struct MSA_CdsQuery));
	memset(searchArgs, 0, sizeof(struct MSA_CdsQuery));

	searchArgs->QueryType = MSA_Query_Search;
	
	size = (int) strlen(Filter)+1;
	searchArgs->Filter = (char*) MSA_MALLOC(size);
	memcpy(searchArgs->Filter, Filter, size);

	searchArgs->MediaServerObject = MSA_TheMsaObj;
		
	size = (int) strlen(ContainerID)+1;
	searchArgs->ObjectID = (char*) MSA_MALLOC(size);
	memcpy(searchArgs->ObjectID, ContainerID, size);

	size = (int) strlen(SearchCriteria)+1;
	searchArgs->SearchCriteria = (char*) MSA_MALLOC(size);
	memcpy(searchArgs->SearchCriteria, SearchCriteria, size);

	searchArgs->RequestedCount = RequestedCount;

	size = (int) strlen(SortCriteria)+1;
	searchArgs->SortCriteria = (char*) MSA_MALLOC(size);
	memcpy(searchArgs->SortCriteria, SortCriteria, size);

	searchArgs->StartingIndex = StartingIndex;

	searchArgs->UpnpToken = upnptoken;
	searchArgs->UserObject = NULL;

	MSA_Helper_PopulateIpInfo(MSA_TheMsaObj, searchArgs);

	if (MSA_TheMsaObj->OnQuery != NULL)
	{
		MSA_TheMsaObj->OnQuery(MSA_TheMsaObj, searchArgs, MSA_TheMsaObj->UserObject);
	}
}


/* END SECTION - Dispatch sinks generated in original main.c */
/************************************************************************************/






/************************************************************************************/
/* START SECTION - public methods*/


/* see header file */
void *MSA_CreateMediaServer(void *chain, void *upnpStack, void* lifetimeMonitor, const char *sinkProtocolInfo, const char *sourceProtocolInfo, const char *sortFields, const char *searchFields, MSA_Callback_OnQuery onQueryCallback, MSA_OnStatsChanged onStatsCallback, void* userObj)
{
	struct MSA_MediaServerObject *msaObj;
	int size;

	msaObj = NULL;
	if (MSA_TheMsaObj == NULL)
	{
		msaObj = MSA_TheMsaObj = (struct MSA_MediaServerObject*) MSA_MALLOC(sizeof(struct MSA_MediaServerObject));
		memset(msaObj, 0, sizeof(struct MSA_MediaServerObject));
		
		msaObj->UpnpStack = upnpStack;
		msaObj->Destroy = MSA_DestroyMediaServer;
		msaObj->LifeTimeMonitor = lifetimeMonitor;
		msaObj->UserObject = userObj;
		msaObj->OnQuery = onQueryCallback;
		msaObj->OnStats = onStatsCallback;
		sem_init(&(msaObj->Lock), 0, 1);

		UpnpSetState_ContentDirectory_SystemUpdateID(upnpStack, msaObj->SystemUpdateID);
		UpnpSetState_ContentDirectory_ContainerUpdateIDs(upnpStack, "");

		/* set initial sourceProtocolInfo */
		size = (int) strlen(sourceProtocolInfo)+1;
		msaObj->SourceProtocolInfo = (char*) MALLOC(size);
		memcpy(msaObj->SourceProtocolInfo, sourceProtocolInfo, size);
		UpnpSetState_ConnectionManager_SourceProtocolInfo(upnpStack, msaObj->SourceProtocolInfo);

		/* set initial sinkProtocolInfo */
		size = (int) strlen(sinkProtocolInfo)+1;
		msaObj->SinkProtocolInfo = (char*) MALLOC(size);
		memcpy(msaObj->SinkProtocolInfo, sinkProtocolInfo, size);
		UpnpSetState_ConnectionManager_SinkProtocolInfo(upnpStack,msaObj->SinkProtocolInfo);

		/* no connections */
		UpnpSetState_ConnectionManager_CurrentConnectionIDs(upnpStack, "");

		#ifdef WITH_EMBEDDED_DMS
		/*
		 *	This code executes when we also have an embedded DMS.
		 */
		UpnpSetState_EmbeddedContentDirectory_SystemUpdateID(upnpStack, msaObj->SystemUpdateID);
		UpnpSetState_EmbeddedContentDirectory_ContainerUpdateIDs(upnpStack, "");

		/* set initial sourceProtocolInfo */
		size = (int) strlen(sourceProtocolInfo)+1;
		msaObj->SourceProtocolInfo = (char*) MALLOC(size);
		memcpy(msaObj->SourceProtocolInfo, sourceProtocolInfo, size);
		UpnpSetState_EmbeddedConnectionManager_SourceProtocolInfo(upnpStack, msaObj->SourceProtocolInfo);

		/* set initial sinkProtocolInfo */
		size = (int) strlen(sinkProtocolInfo)+1;
		msaObj->SinkProtocolInfo = (char*) MALLOC(size);
		memcpy(msaObj->SinkProtocolInfo, sinkProtocolInfo, size);
		UpnpSetState_EmbeddedConnectionManager_SinkProtocolInfo(upnpStack,msaObj->SinkProtocolInfo);

		/* no connections */
		UpnpSetState_EmbeddedConnectionManager_CurrentConnectionIDs(upnpStack, "");
		#endif

		/* set sort capabilities */
		size = (int) strlen(sortFields)+1;
		MSA_TheMsaObj->SortCapabilitiesString = (char*) MSA_MALLOC(size);
		memcpy(MSA_TheMsaObj->SortCapabilitiesString , sortFields, size);

		/* set search cabilities */
		size = (int) strlen(searchFields)+1;
		MSA_TheMsaObj->SearchCapabilitiesString = (char*) MSA_MALLOC(size);
		memcpy(MSA_TheMsaObj->SearchCapabilitiesString, searchFields, size);

		ILibAddToChain(chain, msaObj);
	}

	return msaObj;
}

/* see header file */
void MSA_DeallocateCdsQuery(struct MSA_CdsQuery *cdsQuery)
{
	if (cdsQuery->Filter != NULL) MSA_FREE (cdsQuery->Filter);
	if (cdsQuery->ObjectID != NULL) MSA_FREE (cdsQuery->ObjectID);
	if (cdsQuery->SortCriteria != NULL) MSA_FREE (cdsQuery->SortCriteria);
	if (cdsQuery->SearchCriteria != NULL) MSA_FREE (cdsQuery->SearchCriteria);
	if (cdsQuery->IpAddrList != NULL) MSA_FREE (cdsQuery->IpAddrList);
	MSA_FREE (cdsQuery);
}

/* see header file */
void MSA_ForResponse_RespondError(struct MSA_CdsQuery *cdsQuery, int errorCode, const char *errorMsg)
{
	ASSERT(cdsQuery != NULL);

	UpnpResponse_Error(cdsQuery->UpnpToken, errorCode, errorMsg);
}

/* see header file */
void MSA_ForQueryResponse_Start(struct MSA_CdsQuery *cdsQuery, int sendDidlHeader)
{
	ASSERT(cdsQuery != NULL);

	if (cdsQuery->QueryType == MSA_Query_Search)
	{
		UpnpAsyncResponse_START(cdsQuery->UpnpToken, CDS_STRING_SEARCH, CDS_STRING_URN_CDS);
	}
	else
	{
		UpnpAsyncResponse_START(cdsQuery->UpnpToken, CDS_STRING_BROWSE, CDS_STRING_URN_CDS);
	}

	if (sendDidlHeader != 0)
	{
		UpnpAsyncResponse_OUT(cdsQuery->UpnpToken, CDS_STRING_RESULT, CDS_DIDL_HEADER_ESCAPED, CDS_DIDL_HEADER_ESCAPED_LEN, ILibAsyncSocket_MemoryOwnership_STATIC, 1, 0);
	}
	else
	{
		UpnpAsyncResponse_OUT(cdsQuery->UpnpToken, CDS_STRING_RESULT, "", 0, ILibAsyncSocket_MemoryOwnership_STATIC, 1, 0);
	}
}

/* see header file */
void MSA_ForQueryResponse_ResultArgument(struct MSA_CdsQuery *cdsQuery, const char *xmlEscapedUtf8Didl, int didlSize)
{
	ASSERT(cdsQuery != NULL);

	UpnpAsyncResponse_OUT(cdsQuery->UpnpToken, CDS_STRING_RESULT, xmlEscapedUtf8Didl, didlSize, ILibAsyncSocket_MemoryOwnership_USER, 0, 0);
}

/* see header file */
void MSA_ForQueryResponse_FinishResponse(struct MSA_CdsQuery *cdsQuery, int sendDidlFooter, unsigned int numberReturned, unsigned int totalMatches, unsigned int updateID)
{
	char numResult[30];

	ASSERT(cdsQuery != NULL);

	//tyhsu add 0427
	//Sleep(100);
	if (sendDidlFooter != 0)
	{
		UpnpAsyncResponse_OUT(cdsQuery->UpnpToken, CDS_STRING_RESULT, CDS_DIDL_FOOTER_ESCAPED, CDS_DIDL_FOOTER_ESCAPED_LEN, ILibAsyncSocket_MemoryOwnership_STATIC, 0, 1);
	}
	else
	{
		UpnpAsyncResponse_OUT(cdsQuery->UpnpToken, CDS_STRING_RESULT, "", 0, ILibAsyncSocket_MemoryOwnership_STATIC, 0, 1);
	}

	/*
	 *	Instruct the generated microstack to send the data for the last
	 *	three out-arguments of the Browse request.
	 */

	sprintf(numResult, "%u", numberReturned);
	UpnpAsyncResponse_OUT(cdsQuery->UpnpToken, CDS_STRING_NUMBER_RETURNED, numResult, (int) strlen(numResult), ILibAsyncSocket_MemoryOwnership_USER, 1,1);

	sprintf(numResult, "%u", totalMatches);
	UpnpAsyncResponse_OUT(cdsQuery->UpnpToken, CDS_STRING_TOTAL_MATCHES, numResult, (int) strlen(numResult), ILibAsyncSocket_MemoryOwnership_USER, 1,1);

	sprintf(numResult, "%u", updateID);
	UpnpAsyncResponse_OUT(cdsQuery->UpnpToken, CDS_STRING_UPDATE_ID, numResult, (int) strlen(numResult), ILibAsyncSocket_MemoryOwnership_USER, 1,1);

	if (cdsQuery->QueryType == MSA_Query_Search)
	{
		UpnpAsyncResponse_DONE(cdsQuery->UpnpToken, CDS_STRING_SEARCH);
	}
	else
	{
		UpnpAsyncResponse_DONE(cdsQuery->UpnpToken, CDS_STRING_BROWSE);
	}
}

/* see header file */
void MSA_IncrementSystemUpdateID(void *msaObj)
{
	struct MSA_MediaServerObject *msa;

	ASSERT(msaObj != NULL);

	msa = (struct MSA_MediaServerObject*) msaObj;
	msa->SystemUpdateID++;
	UpnpSetState_ContentDirectory_SystemUpdateID(msa->UpnpStack, msa->SystemUpdateID);
}

/* see header file */
void MSA_UpdateContainerID(void *msaObj, const char *containerID, unsigned int containerUpdateID)
{
	struct MSA_MediaServerObject *msa;
	struct MSA_ContainerUpdate *cu;
	struct MSA_ContainerUpdate *fcu;
	struct MSA_ContainerUpdate *lcu;
	int size;

	ASSERT(msaObj != NULL);

	msa = (struct MSA_MediaServerObject*) msaObj;

	/* lock state */
	sem_wait(&(msa->Lock));

	/*
	 *	Attempt to find an existing ContainerUpdate
	 *	object for the specified containerID.
	 */
	cu = msa->ContainerUpdateID;
	lcu = fcu = NULL;
	while ((cu != NULL) && (fcu == NULL))
	{
		if (strcmp(cu->ContainerID, containerID) == 0)
		{
			fcu = cu;
		}

		lcu = cu;
		cu = cu->Next;
	}

	if (fcu == NULL)
	{
		/*
		 *	If fcu is NULL, then we need to add
		 *	a new MSA_ContainerUpdate to the object.
		 */
		if(lcu){
			fcu = lcu->Next = (struct MSA_ContainerUpdate*) MSA_MALLOC(sizeof(struct MSA_ContainerUpdate));
		}
		else{
			fcu = msa->ContainerUpdateID = (struct MSA_ContainerUpdate*) MSA_MALLOC(sizeof(struct MSA_ContainerUpdate));
			fcu->Next = NULL;
		}
		
		size = (int) strlen(containerID)+1;
		fcu->ContainerID = (char*) MSA_MALLOC(size);
		memcpy(fcu->ContainerID, containerID, size);
		fcu->Next = NULL;
	}

	ASSERT(fcu != NULL);

	/*
	 *	Assign a new UpdateID for the specified containerID.
	 */
	fcu->UpdateID = containerUpdateID;

	MSA_Helper_UpdateModerated_ContainerUpdateID(msa);

	/* unlock */
	sem_post(&(msa->Lock));
}

void MSA_UpdateIpInfo(void *msaObj, int *ipAddrList, int ipAddrListLen)
{
	struct MSA_MediaServerObject *msa;
	int size;

	ASSERT(msaObj != NULL);

	msa = (struct MSA_MediaServerObject*) msaObj;

	/* copy the ip addresses to the msa object */

	sem_wait(&(msa->Lock));

	if (msa->IpAddrList != NULL)
	{
		MSA_FREE(msa->IpAddrList);
	}
	size = (int) (ipAddrListLen * sizeof(int));
	msa->IpAddrList = (int*) MSA_MALLOC(size);
	memcpy(msa->IpAddrList, ipAddrList, size);
	msa->IpAddrListLen = ipAddrListLen;

	sem_post(&(msa->Lock));
}

/* END SECTION - public methods */
/************************************************************************************/


