/*
 * INTEL CONFIDENTIAL
 * Copyright (c) 2001, 2002 Intel Corporation.  All rights reserved.
 * 
 * The source code contained or described herein and all documents
 * related to the source code ("Material") are owned by Intel
 * Corporation or its suppliers or licensors.  Title to the
 * Material remains with Intel Corporation or its suppliers and
 * licensors.  The Material contains trade secrets and proprietary
 * and confidential information of Intel or its suppliers and
 * licensors. The Material is protected by worldwide copyright and
 * trade secret laws and treaty provisions.  No part of the Material
 * may be used, copied, reproduced, modified, published, uploaded,
 * posted, transmitted, distributed, or disclosed in any way without
 * Intel's prior express written permission.
 
 * No license under any patent, copyright, trade secret or other
 * intellectual property right is granted to or conferred upon you
 * by disclosure or delivery of the Materials, either expressly, by
 * implication, inducement, estoppel or otherwise. Any license
 * under such intellectual property rights must be express and
 * approved by Intel in writing.
 * 
 * $Workfile: FileIoAbstraction.c
 * $Revision:
 * $Author: Intel, DPA, Solution Architecture
 * $Date: 10/05/02
 * $Archive:
 *
 * This wraps a few simple tasks that are different between platforms. 
 * Most of it focuses on the file system calls that are completely 
 * different on POSIX, Windows and Pocket PC.
 * PocketPC is probably the most different since all of the calls have
 * to be made using wide character arrays (16 bit chars). So code was
 * added to convert multibyte (UTF-8) to wide.
 */
#ifdef WIN32
#define _CRTDBG_MAP_ALLOC
#include <stdlib.h>
#ifndef _WIN32_WCE
#include <crtdbg.h>
#endif
#else
#include <stdlib.h>
#endif

#include <stdio.h>
#include <string.h>

#include "MyString.h"
#include "FileIoAbstraction.h"

/* Windows 32 */
#ifdef WIN32
#include <windows.h>
#include <time.h>
#include <direct.h>
#endif

/* Win CE */
#ifdef UNDER_CE
#include <Winbase.h>
#endif

/* POSIX */
#ifdef _POSIX
#include <sys/stat.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <dirent.h>
#include <pthread.h>
#include <sys/time.h>
#include <unistd.h>
#endif

#include "debug.h"


// Windows Version
void FIOA_CloseDir(void* handle)
{
#ifdef WIN32
	FindClose(*((HANDLE*)handle));
	FREE(handle);
#endif

#ifdef UNDER_CE
	FindClose(*((HANDLE*)handle));
	FREE(handle);
#endif

#ifdef _POSIX
	DIR* dirObj = (DIR*) handle;
	closedir(dirObj);
#endif
}

int FIOA_FileClose(void *fileHandle)
{
	/* TODO: Modify to enable unicode char support */
	return fclose((FILE*)fileHandle);
}

void* FIOA_FileOpen(const char* fullPath, const TCHAR *mode)
{
	/* TODO: Modify to enable unicode char support */
	// We use UTF8 to store all path and file and need to transfer to Unicode firstly
	TCHAR	str[MAX_PATH_LENGTH];
	Utf8ToStr(str, fullPath, MAX_PATH_LENGTH);
	return (void*) _tfopen(str, mode);
}

int FIOA_FileRead(void*dest, int itemSize, int itemCount, void *fileHandle)
{
	/* TODO: Modify to enable unicode char support */
	return (int) fread(dest, itemSize, itemCount, (FILE*)fileHandle);
}

int FIOA_FileSeek(void *fileHandle, long offset, int origin)
{
	/* TODO: Modify to enable unicode char support */
	return fseek((FILE*)fileHandle, offset, origin);
}

int FIOA_FileTell(void *fileHandle)
{
	/* TODO: Modify to enable unicode char support */
	return ftell((FILE*)fileHandle);
}

void* FIOA_GetDirFirstFile(const char* directory, /*INOUT*/ char* filename, int filenamelength, /*INOUT*/ unsigned long long* filesize)
{
#ifdef WIN32
	WIN32_FIND_DATA FileData;
	HANDLE* hSearch;
	char* direx;
	TCHAR	str[MAX_PATH_LENGTH];

	hSearch = MALLOC(sizeof(HANDLE));
	direx = MALLOC(filenamelength + 5);

	if (directory[(int) strlen(directory) - 1] == '\\')
	{
		sprintf(direx,"%s*.*",directory);
	}
	else
	{
		sprintf(direx,"%s\\*.*",directory);
	}

	// We use UTF8 to store all path and file and need to transfer to Unicode firstly
	Utf8ToStr(str, direx, MAX_PATH_LENGTH);

	*hSearch = FindFirstFile(str, &FileData);
	FREE(direx);

	if (*hSearch == INVALID_HANDLE_VALUE)
	{
		FREE(hSearch);
		hSearch = NULL;
	}
	else
	{
		if (filename != NULL)
		{
			strToUtf8(filename,(char *) FileData.cFileName, filenamelength, IS_WIDE, NULL);
		}

		if (filesize != NULL)
		{
			*filesize = FileData.nFileSizeLow;
		}
	}

	return hSearch;
#endif


#ifdef UNDER_CE
	WIN32_FIND_DATA FileData;
	HANDLE* hSearch;			/* must MMS_FREE */
	wchar_t* wdirectory;		/* must MMS_FREE */
	int wDirLen;
	int wDirSize;
	int mbDirLen;
	int mbDirSize;
	char* direx;				/* must MMS_FREE */

	hSearch = MALLOC(sizeof(HANDLE));
	direx = MALLOC(filenamelength + 5);

	if (directory[(int) strlen(directory) - 1] == '\\')
	{
		sprintf(direx,"%s*.*",directory);
	}
	else
	{
		sprintf(direx,"%s\\*.*",directory);
	}

	mbDirLen = (int) strlen(direx);
	mbDirSize = mbDirLen+1;
	wDirLen = mbDirLen * 2;
	wDirSize = mbDirSize * 2;
	wdirectory = (wchar_t*)MALLOC(wDirSize);

	if (mbstowcs(wdirectory,direx,wDirSize) == -1)
	{
		FREE(hSearch);
		hSearch = NULL;
	}
	else
	{
		*hSearch = FindFirstFile(wdirectory, &FileData);
		if (*hSearch == INVALID_HANDLE_VALUE)
		{
			FREE(hSearch);
			hSearch = NULL;
		}
		else
		{
			if (filename != NULL)
			{
				if (strToUtf8(filename,(char*)FileData.cFileName,filenamelength, 1, NULL) == -1)
				{
					FindClose(*hSearch);
					FREE(hSearch);
					hSearch = NULL;
				}
				else
				{
					if (filesize != NULL)
					{
						*filesize = FileData.nFileSizeLow;
					}
				}
			}
		}
	}

	FREE(direx);
	FREE(wdirectory);

	return hSearch;
#endif

#ifdef _POSIX
	DIR* dirObj;
	struct dirent* dirEntry;	/* dirEntry is a pointer to static memory in the C runtime lib for readdir()*/
	struct stat _si;
	char fullPath[MAX_PATH_LENGTH];
	
	dirObj = opendir(directory);

	if (dirObj != NULL)
	{
		dirEntry = readdir(dirObj);

		if ((dirEntry != NULL) && ((int) strlen(dirEntry->d_name) < filenamelength))
		{
			if (filename != NULL)
			{
				strToUtf8(filename, dirEntry->d_name, filenamelength, 0, NULL);
				sprintf(fullPath, "%s%s", directory, dirEntry->d_name);

				if (filesize != NULL)
				{
					if (stat(fullPath, &_si) != -1)
					{
						if ((_si.st_mode & S_IFDIR) == S_IFDIR)
						{
							*filesize = 0;
						}
						else
						{
							*filesize = _si.st_size;
						}
					}
				}
			}
		}
	}

	return dirObj;
#endif
}


// Windows Version
// 0 = No More Files
// 1 = Next File
int FIOA_GetDirNextFile(void* handle, const char* dirName, char* filename, int filenamelength, int* filesize)
{
#ifdef WIN32
	WIN32_FIND_DATA FileData;
	
	if (FindNextFile(*((HANDLE*)handle), &FileData) == 0) {return 0;}
	strToUtf8(filename,(char *) FileData.cFileName,filenamelength,IS_WIDE, NULL);

	if (filesize != NULL)
	{
		*filesize = FileData.nFileSizeLow;
	}

	return 1;
#endif

#ifdef UNDER_CE
    WIN32_FIND_DATA FileData;
    int fnf = 0;
    int conv = -1;
    
    fnf = FindNextFile(*((HANDLE*)handle), &FileData);
    if (fnf == 0) {return 0;}

    conv = strToUtf8(filename, (char*)FileData.cFileName, filenamelength, 1, NULL);
    if (conv == -1) {return 0;}

	if (filesize != NULL)
	{
		*filesize = FileData.nFileSizeLow;
	}
    return 1;
#endif

#ifdef _POSIX
	DIR* dirObj;
	struct dirent* dirEntry;	/* dirEntry is a pointer to static memory in the C runtime lib for readdir()*/
	struct stat _si;
	char fullPath[MAX_PATH_LENGTH];

	dirObj = (DIR*) handle;
	dirEntry = readdir(dirObj);

	if ((dirEntry != NULL) && ((int) strlen(dirEntry->d_name) < filenamelength))
	{
		strToUtf8(filename, dirEntry->d_name, filenamelength, 0, NULL);
		sprintf(fullPath, "%s%s", dirName, dirEntry->d_name);

		if (filesize != NULL)
		{
			/* WTF? Cygwin has a memory leak with stat. */
			if (stat(fullPath, &_si) != -1)
			{
				if ((_si.st_mode & S_IFDIR) == S_IFDIR)
				{
					*filesize = 0;
				}
				else
				{
					*filesize = _si.st_size;
				}
			}
		}

		return 1;
	}

	return 0;
#endif
}

int FIOA_GetFileDirType(char* directory)
{
#ifdef WIN32
	DWORD _si;
	TCHAR str[MAX_PATH_LENGTH];

	// We use UTF8 to store all path and file and need to transfer to Unicode firstly
	Utf8ToStr(str, directory, MAX_PATH_LENGTH);

	_si = GetFileAttributes(str);
	
	if (_si == 0xFFFFFFFF)
	{
		return 0;
	}

	if ((_si & FILE_ATTRIBUTE_DIRECTORY) == 0)
	{
		return 1;
	}
	else 
	{
		return 2;
	}
#endif

#ifdef UNDER_CE
	wchar_t* wfullPath;
	DWORD _si;
	int mbDirSize;
	int wPathSize;
	int dirLen,dirSize;
	char *fullpath;
	int retVal = 0;

	dirLen = (int) strlen(directory);
	dirSize = dirLen+1;
	fullpath = (char*) MALLOC(dirSize);
	Utf8ToAnsi(fullpath, directory, dirSize);

	mbDirSize = (int) strlen(fullpath) + 1;
	wPathSize = mbDirSize * 2;

	wfullPath = (wchar_t*)MALLOC(wPathSize);
	if (mbstowcs(wfullPath,fullpath,wPathSize) == -1)
	{
		retVal = 0;
	}
	else
	{
		_si = GetFileAttributes(wfullPath);
		if (_si == 0xFFFFFFFF)
		{
			retVal = 0;
		}
		else
		{
			if ((_si & FILE_ATTRIBUTE_DIRECTORY) == 0)
			{
				retVal = 1;
			}
			else 
			{
				retVal = 2;
			}
		}
	}

	FREE(fullpath);
	FREE(wfullPath);

	return retVal;
#endif

#ifdef _POSIX
	struct stat _si;

	int dirLen,dirSize;
	char *fullpath;
	int pathExists;
	int retVal = 0;

	dirLen = (int) strlen(directory);
	dirSize = dirLen+1;
	fullpath = (char*) MALLOC(dirSize);
	Utf8ToAnsi(fullpath, directory, dirSize);

	pathExists = stat (fullpath, &_si);
	FREE(fullpath);

	if (pathExists != -1)
	{
		if ((_si.st_mode & S_IFDIR) == S_IFDIR)
		{
			retVal = 2;
		}
		else
		{
			retVal = 1;
		}
	}

	return retVal;
#endif
}

#ifdef _POSIX
/* only needed for posix because readdir returns statically allocated values */
pthread_mutex_t BrowseLock;
#endif


unsigned long long FIOA_GetFileSize(const char* fullPath)
{
	unsigned long long filesize = -1;	

#ifdef _POSIX
	struct stat _si;

	int pathLen,pathSize;
	char *fp;

	pathLen = (int) strlen(fullPath);
	pathSize = pathLen+1;
	fp = (char*) MALLOC(pathSize);
	Utf8ToAnsi(fp, fullPath, pathSize);

	if (stat(fp, &_si) != -1)
	{
		if (!((_si.st_mode & S_IFDIR) == S_IFDIR))
		{
			filesize = _si.st_size;
		}
	}

	FREE(fp);
#endif

#ifdef WIN32
	WIN32_FIND_DATA FileData;
	HANDLE* hSearch;			/* must MMS_FREE */
	TCHAR	str[MAX_PATH_LENGTH];

	// We use UTF8 to store all path and file and need to transfer to Unicode firstly
	Utf8ToStr(str, fullPath, MAX_PATH_LENGTH);

	hSearch = MALLOC(sizeof(HANDLE));
	*hSearch = FindFirstFile(str, &FileData);

	if (*hSearch == INVALID_HANDLE_VALUE)
	{
		filesize = -1;
	}
	else
	{
		//filesize = FileData.nFileSizeLow;
		filesize = (((unsigned long long)FileData.nFileSizeHigh) << 32) | FileData.nFileSizeLow;		
	}

	FindClose(*hSearch);
	FREE(hSearch);
#endif

#ifdef UNDER_CE
	WIN32_FIND_DATA FileData;
	HANDLE* hSearch;			/* must MMS_FREE */
	wchar_t* wdirectory;		/* must MMS_FREE */
	int wPathLen;
	int wPathSize;
	int fullPathLen;
	int fullPathSize;
	char* fp;

	fullPathLen = (int) strlen(fullPath);
	fullPathSize = fullPathLen + 1;
	fp = (char*) MALLOC(fullPathSize);
	Utf8ToAnsi(fp, fullPath, fullPathSize);

	hSearch = MALLOC(sizeof(HANDLE));

	wPathLen = fullPathLen * 2;
	wPathSize = fullPathSize * 2;
	wdirectory = (wchar_t*)MALLOC(wPathSize);

	if (mbstowcs(wdirectory,fp,wPathSize) == -1)
	{
		filesize = -1;
	}
	else
	{
		*hSearch = FindFirstFile(wdirectory, &FileData);
		if (*hSearch == INVALID_HANDLE_VALUE)
		{
			filesize = -1;
		}
		else
		{
			FindClose(*hSearch);
			filesize = FileData.nFileSizeLow;
		}
	}

	FREE(fp);
	FREE(wdirectory);
	FREE(hSearch);
#endif

	return filesize;
}

//albert add , for windows only
unsigned long long FIOA_GetFileCreationTime(const char* fullPath,FILETIME *filetime)
{
	unsigned long long filesize = -1;

	WIN32_FIND_DATA FileData;
	HANDLE* hSearch;			/* must MMS_FREE */
	TCHAR	str[MAX_PATH_LENGTH];

	// We use UTF8 to store all path and file and need to transfer to Unicode firstly
	Utf8ToStr(str, fullPath, MAX_PATH_LENGTH);

	hSearch = MALLOC(sizeof(HANDLE));
	*hSearch = FindFirstFile(str, &FileData);

	if (*hSearch == INVALID_HANDLE_VALUE)
	{
		filesize = -1;
	}
	else
	{
		//filesize = FileData.nFileSizeLow;
		filesize = (((unsigned long long)FileData.nFileSizeHigh) << 32) | FileData.nFileSizeLow;
		memcpy(filetime, &FileData.ftCreationTime,sizeof(FILETIME));
	}

	FindClose(*hSearch);
	FREE(hSearch);

	return filesize;
}


//albert add , for windows only
unsigned long long FIOA_GetFileLongSize(const char* fullPath)
{
	unsigned long long filesize = -1;

#ifdef _POSIX
	struct stat _si;

	int pathLen,pathSize;
	char *fp;

	pathLen = (int) strlen(fullPath);
	pathSize = pathLen+1;
	fp = (char*) MALLOC(pathSize);
	Utf8ToAnsi(fp, fullPath, pathSize);

	if (stat(fp, &_si) != -1)
	{
		if (!((_si.st_mode & S_IFDIR) == S_IFDIR))
		{
			filesize = _si.st_size;
		}
	}

	FREE(fp);
#endif

#ifdef WIN32
	WIN32_FIND_DATA FileData;
	HANDLE* hSearch;			/* must MMS_FREE */
	TCHAR	str[MAX_PATH_LENGTH];

	// We use UTF8 to store all path and file and need to transfer to Unicode firstly
	Utf8ToStr(str, fullPath, MAX_PATH_LENGTH);

	hSearch = MALLOC(sizeof(HANDLE));
	*hSearch = FindFirstFile(str, &FileData);

	if (*hSearch == INVALID_HANDLE_VALUE)
	{
		filesize = -1;
	}
	else
	{
		//filesize = FileData.nFileSizeLow;
		filesize = (((unsigned long long)FileData.nFileSizeHigh) << 32) | FileData.nFileSizeLow;
	}

	FindClose(*hSearch);
	FREE(hSearch);
#endif

#ifdef UNDER_CE
	WIN32_FIND_DATA FileData;
	HANDLE* hSearch;			/* must MMS_FREE */
	wchar_t* wdirectory;		/* must MMS_FREE */
	int wPathLen;
	int wPathSize;
	int fullPathLen;
	int fullPathSize;
	char* fp;

	fullPathLen = (int) strlen(fullPath);
	fullPathSize = fullPathLen + 1;
	fp = (char*) MALLOC(fullPathSize);
	Utf8ToAnsi(fp, fullPath, fullPathSize);

	hSearch = MALLOC(sizeof(HANDLE));

	wPathLen = fullPathLen * 2;
	wPathSize = fullPathSize * 2;
	wdirectory = (wchar_t*)MALLOC(wPathSize);

	if (mbstowcs(wdirectory,fp,wPathSize) == -1)
	{
		filesize = -1;
	}
	else
	{
		*hSearch = FindFirstFile(wdirectory, &FileData);
		if (*hSearch == INVALID_HANDLE_VALUE)
		{
			filesize = -1;
		}
		else
		{
			FindClose(*hSearch);
			filesize = FileData.nFileSizeLow;
		}
	}

	FREE(fp);
	FREE(wdirectory);
	FREE(hSearch);
#endif

	return filesize;
}


int FIOA_GetGetDirEntryCount(const char* fullPath, char *dirDelimiter)
{
	char fn[MAX_PATH_LENGTH];
	void *dirObj;
	int retVal = 0;
	int ewDD;
	int nextFile;

	dirObj = FIOA_GetDirFirstFile(fullPath, fn, MAX_PATH_LENGTH, NULL);

	if (dirObj != NULL)
	{
		ewDD = EndsWith(fullPath, dirDelimiter, 0);

		do
		{
			retVal++;
			nextFile = FIOA_GetDirNextFile(dirObj,fullPath,fn,MAX_PATH_LENGTH, NULL);
		}
		while (nextFile != 0);

		FIOA_CloseDir(dirObj);
	}

	return retVal;
}

char* FIOA_GetWorkingDir(char *path, unsigned long long sizeOfBuf)
{
#ifdef _POSIX
	return getcwd(path, sizeOfBuf);
#endif

#ifdef WIN32
#ifndef UNDER_CE
	return _getcwd(path, (int) sizeOfBuf);
#endif
#endif
}