/*
 * INTEL CONFIDENTIAL
 * Copyright (c) 2001, 2002 Intel Corporation.  All rights reserved.
 * 
 * The source code contained or described herein and all documents
 * related to the source code ("Material") are owned by Intel
 * Corporation or its suppliers or licensors.  Title to the
 * Material remains with Intel Corporation or its suppliers and
 * licensors.  The Material contains trade secrets and proprietary
 * and confidential information of Intel or its suppliers and
 * licensors. The Material is protected by worldwide copyright and
 * trade secret laws and treaty provisions.  No part of the Material
 * may be used, copied, reproduced, modified, published, uploaded,
 * posted, transmitted, distributed, or disclosed in any way without
 * Intel's prior express written permission.
 
 * No license under any patent, copyright, trade secret or other
 * intellectual property right is granted to or conferred upon you
 * by disclosure or delivery of the Materials, either expressly, by
 * implication, inducement, estoppel or otherwise. Any license
 * under such intellectual property rights must be express and
 * approved by Intel in writing.
 * 
 * $Workfile: FileIoAbstraction.h
 * $Revision:
 * $Author: Intel, DPA, Solution Architecture
 * $Date: 10/05/02
 * $Archive:
 *
 * This wraps a few simple tasks that are different between platforms. 
 * Most of it focuses on the file system calls that are completely 
 * different on POSIX, Windows and Pocket PC.
 * PocketPC is probably the most different since all of the calls have
 * to be made using wide character arrays (16 bit chars). So code was
 * added to convert multibyte (UTF-8) to wide.
 */

#ifndef _FILE_IO_ABSTRACTION_H
#define _FILE_IO_ABSTRACTION_H

// We translate the file name and path to UTF8, so the maximun lenght will be multiplied by 3
#define MAX_PATH_LENGTH MAX_PATH*3

//#define	MAX_SQL_STR_LENGTH	(2*MAX_PATH_LENGTH)+(4*MAX_MEDIA_STR_LENGTH)+512
#define	MAX_SQL_STR_LENGTH	(2*(MAX_PATH*3)+4*1024+512)

/*
 *	Use to a close a file after streaming is complete.
 *		fileHandle returned from FIOA_FileOpen. 
 */
int FIOA_FileClose(void *fileHandle);

/*
 *	Use to open a file for streaming.
 *		Returns a void* fileHandle for use with other FIOA_Filexxx methods.
 *		Arguments behave exactly like fopen().
 */
void* FIOA_FileOpen(const char* fullPath, const TCHAR *mode);

/*
 *	Use to read a file. 
 *		fileHandle returned from FIOA_FileOpen.
 *		Other arguments behave exactly like fread().
 */
int FIOA_FileRead(void *dest, int itemSize, int itemCount, void *fileHandle);

/*
 *	Use to move the file pointer to a specific offset.
 *		fileHandle returned from FIOA_FileOpen.
 *		Other arguments behave exactly like fread().
 */
int FIOA_FileSeek(void *fileHandle, long offset, int origin);

/*
 *	Use to obtain the current file pointer position.
 *		fileHandle returned from FIOA_FileOpen.
 */
int FIOA_FileTell(void *fileHandle);

/*
 *	Returns an integer describing the type of content at this path.
 *		0 = Does Not Exist
 *		1 = Is a File
 *		2 = Is a Directory
 *
 *	Provided path is a UTF8-encoded path into the file system.
 */
int FIOA_GetFileDirType(char* path);

/*
 *	Get the working directory.
 */
char* FIOA_GetWorkingDir(char *path, unsigned long long sizeOfBuf);

/*
 *	Closes a directory or specific path.
 *
 *	dirHandle returned from PCGetDirFirstFile.
 */
void FIOA_CloseDir(void* dirHandle);

/*
 *	Opens a directory for enumeration or a specific file.
 *		Returns void* dirHandle.
 *
 *		directory		is the directory portion of the path
 *		filename		allocated byte array, contains the first filename in that directory path upon return
 *		filenamelength	the number of bytes available in filename
 *		fileSize		the size of the file (specified by dirName+filename)
 */
void* FIOA_GetDirFirstFile(const char* directory, /*INOUT*/ char* filename, int filenamelength, /*INOUT*/ int* filesize);

/*
 *	Obtains the next entry in the directory.
 *		dirHandle		the void* returned in PCGetDirFirstFile
 *		dirName			the directory name
 *		filename		allocated byte array, contains the next filename in that directory path upon return
 *		filenamelength	the number of bytes available in filename
 *		fileSize		the size of the file (specified by dirName+filename)
 */
int FIOA_GetDirNextFile(void* dirHandle, const char* dirName, /*INOUT*/char* filename, int filenamelength, int* fileSize);

/*
 *	Obtains the filesize of the file specified by fullpath.
 */
unsigned long long FIOA_GetFileSize(const char* fullPath);

// By albert 20050127
unsigned long long FIOA_GetFileLongSize(const char* fullPath);

unsigned long long FIOA_GetFileCreationTime(const char* fullPath, struct _FILETIME *filetime);

/*
 *	Returns the number of entries specified in fullPath. 
 *
 *		dirDelimiter	single character to identify the character used to delimit directories within a path.
 *						Posix usually uses '/' whereas windows usually uses '\'
 */
int FIOA_GetGetDirEntryCount(const char* fullPath, char *dirDelimiter);

#endif
