/*
 * INTEL CONFIDENTIAL
 * Copyright (c) 2002, 2003 Intel Corporation.  All rights reserved.
 * 
 * The source code contained or described herein and all documents
 * related to the source code ("Material") are owned by Intel
 * Corporation or its suppliers or licensors.  Title to the
 * Material remains with Intel Corporation or its suppliers and
 * licensors.  The Material contains trade secrets and proprietary
 * and confidential information of Intel or its suppliers and
 * licensors. The Material is protected by worldwide copyright and
 * trade secret laws and treaty provisions.  No part of the Material
 * may be used, copied, reproduced, modified, published, uploaded,
 * posted, transmitted, distributed, or disclosed in any way without
 * Intel's prior express written permission. 
 *
 * No license under any patent, copyright, trade secret or other
 * intellectual property right is granted to or conferred upon you
 * by disclosure or delivery of the Materials, either expressly, by
 * implication, inducement, estoppel or otherwise. Any license
 * under such intellectual property rights must be express and
 * approved by Intel in writing.
 *  
 * $Workfile: MyString.h
 * $Revision:
 * $Author: Intel, DPA, Solution Architecture
 * $Date: 10/05/02
 * $Archive:
 */

/*
 * Implements additional string functionality.
 */

#ifndef MY_STRING_H

#include <ctype.h>
#include <stdlib.h>
#include <tchar.h>

// For Unicode support
#ifdef UNICODE
	#define	IS_WIDE			1
	#define	Utf8ToStr		Utf8ToWide
#else
	#define	IS_WDIE			0
	#define Utf8ToStr		Utf8ToAnsi
#endif


/*
 *	NOTE: Enabling for ISO 10646.
 *	You can enable MyString to support the full range of ISO 10646
 *	by changing the wchar_t references to a 32-bit character types.
 *	The wchar_t type is limited to 16-bit character types, so
 *	the compiler may report warnings for lines of code that work
 *	with characters wider than 16-bits.
 */

/*
 *	EndsWith()
 *		str					: the string to analyze
 *		endsWith			: the token to find at the end of str
 *		ignoreCase			: nonzero indicates case insensitive search
 *
 *	If "str" ends with "endsWith", then we return nonzero.
 */
int EndsWith(/*INOUT*/ const char* str, const char* endsWith, int ignoreCase);

/*
 *	IndexOfChr()
 *		str					: the string to analyze
 *		strLen				: length of 'str'
 *		findThis			: character to find
 *
 *	Returns index where 'findThis' was found. Otherwise, returns -1.
 */
int IndexOfChr(const char* str, int strLen, const char findThis);

/*
 *	IndexOf()
 *		str					: the string to analyze
 *		findThis			: the token to find 
 *
 *	Returns the first index where findThis can be found in str.
 *	Returns -1 if not found.
 */
int IndexOf(const char* str, const char* findThis);

/*
 *	IndexOf()
 *		str					: the string to analyze
 *		findThis			: the token to find 
 *
 *	Returns the last index where findThis can be found in str.
 *	Returns -1 if not found.
 */
int LastIndexOf(const char* str, const char* findThis);

/*
 *	IndexOf()
 *		str					: the string to analyze
 *		startsWith			: the token to find 
 *		ignoreCase			: nonzero indicates case insensitive search
 *
 *	Returns nonzero value if the string starts with 'startsWith'.
 */
int StartsWith(/*INOUT*/ const char* str, const char* startsWith, int ignoreCase);

int Utf8ToAnsi(/*INOUT*/ char *dest, const char *src, /*IN*/ int destLen);
int Utf8ToWide(/*INOUT*/wchar_t *dest, const char *src, /*IN*/ int destLen);
int strToUtf8(/*INOUT*/ char *dest, const char *src, int destSize, int isWide, /*INOUT*/ int *charactersConverted);
int strUtf8Len(/*IN*/ char *src, int isWide, int asEscapedUri);

/*
 *	Stores UTF8-compliant escaped URI in 'dest'.
 *	Returns number of bytes used in dest.
 */
int strToEscapedUri(/*INOUT*/ char *dest, const char *src, int destSize, int isWide, /*INOUT*/ int *charactersConverted);

/* makes a copy of the string by mallocing a new buffer and copying
 * the string into it 
 */
char *MakeStrcpy(const char *string);


/* Chomps off the trailing newline on a string much like Perl's chomp function
 */
void StrChomp(/*INOUT*/ char *s);

/*
 *	Returns the number of bytes needed to store an array of char* values
 *	as a comma-separated value list.
 *
 *		values			: The array of char* values to be changed into a CSV.
 *
 *		numValues		: The number of elements in 'values'.
 */
int strCSVSize(const char** values, unsigned int numValues);

/*
 *	Takes "values" and turns it into a comma-separated-value list in
 *	"storeHere".
 *
 *		storeHere		: The output value. Memory should be allocated already.
 *
 *		values			: The array of char* values to be changed into a CSV.
 *
 *		numValues		: The number of elements in 'values'.
 *
 *	Returns 'storeHere'.
 */
char* strMakeCSV(/*INOUT*/ char* storeHere, const char** values, unsigned int numValues);

/*
 *	Creates a UDN and stores it in 'storeHere'. Returns 'storehere'.
 *
 *		storeHere		: Allocated memory must be at least STR_UDN_SIZE bytes.
 */
#define STR_UDN_SIZE 80

char* strMakeUDN(/*INOUT*/ char* storeHere);

/*
 *	Replaces all occurences of 'findThis' with 'replaceWithThis'.
 *	Returns 'str'.
 */
char* strReplaceWith(/*INOUT*/char *str, const char findThis, const char replaceWithThis);

void SingleByteToWideChar(/*INOUT*/wchar_t *dest, const char *src, /*IN*/ int destLen);
void WideCharToSingleByte(/*INOUT*/char *dest, const wchar_t *src, /*IN*/ int destLen);

#endif
