#ifdef WIN32
#define _CRTDBG_MAP_ALLOC
#include <stdlib.h>
#ifndef _WIN32_WCE
#include <crtdbg.h>
#endif
#else
#include <stdlib.h>
#endif

#include <string.h>
#include <stdio.h>
#include <tchar.h>

#if defined(WIN32) && !defined(_WIN32_WCE)
	#ifndef WINVER				// Allow use of features specific to Windows 95 and Windows NT 4 or later.
		#define WINVER 0x0400		// Change this to the appropriate value to target Windows 98 and Windows 2000 or later.
	#endif

	#ifndef _WIN32_WINNT		// Allow use of features specific to Windows NT 4 or later.
		#define _WIN32_WINNT 0x0400	// Change this to the appropriate value to target Windows 2000 or later.
	#endif						

	#ifndef _WIN32_WINDOWS		// Allow use of features specific to Windows 98 or later.
		#define _WIN32_WINDOWS 0x0410 // Change this to the appropriate value to target Windows Me or later.
	#endif

#include <windows.h>
#include <winbase.h>
#endif

#include "MyThread.h"
#include "WinAio.h"
#include "PreCache.h"
#include "ILibParsers.h"
#include "debug.h"
#include "FileIoAbstraction.h"

extern struct WAIO_State WAIO_StateObj;

//Birdy 2007/3/6
void WAIO_StopLock(void);
void WAIO_StopUnlock(void);

static	PreCacheBuffer	preCacheBuffer;

static VOID CALLBACK CacheNextCacheLine_Sink(DWORD dwErrorCode, DWORD dwNumberOfBytesTransfered, LPOVERLAPPED lpOverlapped)
{
	PCacheLine	cacheLine;
	char		key[128];
	int			keySize;

	/*
	 *	Don't process the read operation if things are shutting down.
	 */

	//Birdy 2007/3/6
    WAIO_StopLock();

	if (WAIO_StateObj.StopFlag == 0)
	{		
		cacheLine = (PCacheLine) lpOverlapped->hEvent;

		// We must make sure the return value is not out of date
		if (preCacheBuffer.cacheValid && (cacheLine->offset == lpOverlapped->Offset) &&
			(preCacheBuffer.fildes == cacheLine->fildes))
		{
			// The asynchronous read operation has completed.
			cacheLine->size = dwNumberOfBytesTransfered; 
			cacheLine->valid = TRUE;
		}

		/*
		 *	Deallocate the data created in CacheNextCacheLine.
		 *	If waio_stop() is called before this executes, then
		 *	the StopFlag will be nonzero.
		 */
		#pragma warning( disable : 4311 )
		keySize = sprintf(key,"%ul", (DWORD)lpOverlapped);
		#pragma warning( default : 4311 )
		ILibHashTree_Lock(WAIO_StateObj.OverlappedTree);
		ILibDeleteEntry(WAIO_StateObj.OverlappedTree,key,keySize);
		FREE(lpOverlapped);
		ILibHashTree_UnLock(WAIO_StateObj.OverlappedTree);
	}

	//Birdy 2007/3/6
    WAIO_StopUnlock();
}

int InitPreCacheBuffer(_WAIO_FDES_TYPE fildes, off_t offset, unsigned long blockSize, _WAIO_FDES_TYPE sync_fildes)
{
	unsigned long long		fileSize;
	int			i, index;
	
	if (blockSize < PRECACHE_MIN_CACHELINE_SIZE || (sync_fildes == NULL))
	{	
		// The block size is too small or no sync file handle, don't cache it
		return FALSE;
	}

	if (preCacheBuffer.fildes != fildes)
	{
		fileSize = GetFileSize(fildes, NULL);
		if (fileSize < PRECACHE_MIN_FILE_SIZE)
		{
			// The file size is too small, don't cache it
			return FALSE;
		}
		preCacheBuffer.fileSize = fileSize;
	}

	FIOA_FileSeek(sync_fildes, offset, SEEK_SET);

	/* don't do anything if things are shutting down */

	//Birdy 2007/3/6
    WAIO_StopLock();

	if (WAIO_StateObj.StopFlag == 0)
	{		
		preCacheBuffer.bufferSize = FIOA_FileRead(preCacheBuffer.buffer, sizeof(char), PRECACHE_MAX_BUFFER_SIZE, sync_fildes);

		if (preCacheBuffer.bufferSize <= PRECACHE_MIN_READBACK_SIZE)
		{
			// Data readback is too small, don't cache it
			goto error;
		}
	}

	//Birdy 2007/3/6
    WAIO_StopUnlock();

	/* don't do anything if things are shutting down */
	if (WAIO_StateObj.StopFlag != 0)
	{
		goto error;
	}

	preCacheBuffer.cacheLineNo = preCacheBuffer.bufferSize / blockSize;
	if (preCacheBuffer.cacheLineNo > PRECACHE_MAX_CACHELINE_NO)
		preCacheBuffer.cacheLineNo = PRECACHE_MAX_CACHELINE_NO;

	preCacheBuffer.fildes = fildes;
	preCacheBuffer.cacheLineSize = blockSize;
	preCacheBuffer.rangeStart = offset;
	preCacheBuffer.rangeEnd = offset + preCacheBuffer.bufferSize;

	for (i = 0; i < preCacheBuffer.cacheLineNo; i++)
	{
		index = (offset >> PRECACHE_HASH_SHIFT) % preCacheBuffer.cacheLineNo;
		preCacheBuffer.cacheLine[index].fildes = fildes;
		preCacheBuffer.cacheLine[index].buffer = preCacheBuffer.buffer + i * preCacheBuffer.cacheLineSize;
		preCacheBuffer.cacheLine[index].offset = offset;
		preCacheBuffer.cacheLine[index].size = blockSize;
		preCacheBuffer.cacheLine[index].time = 0;
		preCacheBuffer.cacheLine[index].valid = TRUE;
		offset += preCacheBuffer.cacheLineSize;
	}
	preCacheBuffer.cacheValid = TRUE;

	return TRUE;

error:
	preCacheBuffer.fildes = fildes;
	preCacheBuffer.fileSize = 0;
	preCacheBuffer.rangeStart = 0;
	preCacheBuffer.rangeEnd = 0;
	preCacheBuffer.cacheLineNo  = 0;
	preCacheBuffer.cacheValid = FALSE;

	return FALSE;
}

void UnInitPreCacheBuffer()
{
	preCacheBuffer.fildes = 0;
	preCacheBuffer.fileSize = 0;
	preCacheBuffer.rangeStart = 0;
	preCacheBuffer.rangeEnd = 0;
	preCacheBuffer.cacheLineNo  = 0;
	preCacheBuffer.cacheValid = FALSE;
}

char* GetCacheLine(_WAIO_FDES_TYPE fildes, off_t offset, unsigned long long readSize, _WAIO_FDES_TYPE sync_fildes)
{
	int		index;

	if ((fildes == preCacheBuffer.fildes) && (preCacheBuffer.cacheValid == FALSE))
	{
		// We have tried to cache this file before, but it was rejected. Don't try again
		return NULL;
	}


	if ((fildes != preCacheBuffer.fildes) || (offset == 0) || 
		(offset >= preCacheBuffer.rangeEnd) || (offset < preCacheBuffer.rangeStart))

	{
		// Render try to read another file or jump to other position, we flush cache and cache new data
		if (!InitPreCacheBuffer(fildes, offset, readSize, sync_fildes))
		{
			return NULL;
		}
	}

	if ((fildes == preCacheBuffer.fildes) && (preCacheBuffer.cacheValid) &&
		(offset >= preCacheBuffer.rangeStart) && (offset < preCacheBuffer.rangeEnd))
	{
		// this data maybe in our cache
		index = (offset >> PRECACHE_HASH_SHIFT) % preCacheBuffer.cacheLineNo;
		if (preCacheBuffer.cacheLine[index].valid && (preCacheBuffer.cacheLine[index].offset == offset) &&
			(preCacheBuffer.cacheLine[index].size >= readSize))
		{
			// cache hit
			return preCacheBuffer.cacheLine[index].buffer;
		}
	}

	// Cache miss
	return NULL;
}

void CacheNextCacheLine(_WAIO_FDES_TYPE fildes, off_t offset)
{
	LPOVERLAPPED	lpOverlapped;
	char			key[128];
	int				index, keySize;
	off_t			newOffset;

	/* don't read if things are shutting down */
	if (WAIO_StateObj.StopFlag != 0) return;

	if ((preCacheBuffer.fildes != fildes) || (preCacheBuffer.cacheValid == FALSE))
	{
		// The cahce in not valid, or file has been changed
		return;
	}

	newOffset = offset + (preCacheBuffer.cacheLineNo * preCacheBuffer.cacheLineSize);
	if ((newOffset + preCacheBuffer.cacheLineSize) > preCacheBuffer.fileSize)
	{
		// exceed the file end, don't do anything
		return;
	}

	// offset is cache hit position, we will load a new value to replace it
	index = (offset >> PRECACHE_HASH_SHIFT) % preCacheBuffer.cacheLineNo;

	lpOverlapped = (LPOVERLAPPED)MALLOC(sizeof(OVERLAPPED));
	memset(lpOverlapped, 0, sizeof(OVERLAPPED));
	lpOverlapped->Offset =  newOffset;
	lpOverlapped->hEvent = (HANDLE) &preCacheBuffer.cacheLine[index];

	preCacheBuffer.rangeStart = offset + preCacheBuffer.cacheLineSize;
	preCacheBuffer.rangeEnd = lpOverlapped->Offset + preCacheBuffer.cacheLineSize;
	preCacheBuffer.cacheLine[index].fildes = fildes;
	preCacheBuffer.cacheLine[index].offset = newOffset;
	preCacheBuffer.cacheLine[index].valid = FALSE;
	
	/*
	 *	Make a note of the overlapped object 
	 *	so that we can destroy at shutdown if we
	 *	need to.
	 */
	#pragma warning( disable : 4311 )
	keySize = sprintf(key,"%ul", (DWORD)lpOverlapped);
	#pragma warning( default : 4311 )	
	ILibHashTree_Lock(WAIO_StateObj.OverlappedTree);
	ILibAddEntry(WAIO_StateObj.OverlappedTree, key, keySize, lpOverlapped);
	ILibHashTree_UnLock(WAIO_StateObj.OverlappedTree);

	ReadFileEx(preCacheBuffer.fildes, preCacheBuffer.cacheLine[index].buffer, preCacheBuffer.cacheLineSize, 
			   lpOverlapped, CacheNextCacheLine_Sink);
}