#ifdef WIN32
#define _CRTDBG_MAP_ALLOC
#include <stdlib.h>
#ifndef _WIN32_WCE
#include <crtdbg.h>
#endif
#else
#include <stdlib.h>
#endif

#ifdef _POSIX
// posix AIO and its use of signal handlers interrupts our calls to sem_wait, 
// we don't actually use it.
//#include <aio.h>
#endif

#include <string.h>
#include <stdio.h>
#include <tchar.h>

#if defined(WIN32) && !defined(_WIN32_WCE)
	#ifndef WINVER				// Allow use of features specific to Windows 95 and Windows NT 4 or later.
		#define WINVER 0x0400		// Change this to the appropriate value to target Windows 98 and Windows 2000 or later.
	#endif

	#ifndef _WIN32_WINNT		// Allow use of features specific to Windows NT 4 or later.
		#define _WIN32_WINNT 0x0400	// Change this to the appropriate value to target Windows 2000 or later.
	#endif						

	#ifndef _WIN32_WINDOWS		// Allow use of features specific to Windows 98 or later.
		#define _WIN32_WINDOWS 0x0410 // Change this to the appropriate value to target Windows Me or later.
	#endif

#include <windows.h>
#include <winbase.h>
#endif

#include "MyThread.h"
#include "WinAio.h"
#include "PreCache.h"
#include "ILibParsers.h"
#include "debug.h"
#include "TransLibWarp.h"
#include <assert.h>

#include "TransferState.h"

int		gPreCacheEnable = FALSE;
int		gTranscodingEnable = FALSE;
/*
 *	Global but not visible vars
 */

struct WAIO_State WAIO_StateObj;

static HANDLE hWAIO_StopMutex = -1;


void initWAIO_StopLock(void)
{
	hWAIO_StopMutex = CreateMutex( NULL, FALSE, NULL );
}

void WAIO_StopLock(void)
{
//Birdy 2007/3/7
    if( hWAIO_StopMutex != -1 )
    {
        WaitForSingleObject( hWAIO_StopMutex, INFINITE );
    }
    else
    {
        // should not occur
        assert(1);
    }
}

void WAIO_StopUnlock(void)
{
    if( hWAIO_StopMutex != -1 )
    {
        ReleaseMutex( hWAIO_StopMutex );
    }
    else
    {
        // should not occur
        assert(1);
    }
}

#if defined(WIN32) && !defined(_WIN32_WCE)
static VOID CALLBACK _waio_ReadFileEx_Sink(DWORD dwErrorCode, DWORD dwNumberOfBytesTransfered, LPOVERLAPPED lpOverlapped)
{
	char key[128];
	int keySize;
	struct aiocb *cb = NULL;
	struct TS_TransferState *ts = NULL;
	struct ILibWebServer_Session	*Session;

	/*
	 *	The asynchronous read operation has completed.
	 */

	/*
	 *	Don't process the read operation if things are shutting down.
	 */
	//phillo,test
	WAIO_StopLock();

	cb = (struct aiocb*)lpOverlapped->hEvent;
	ts = (struct TS_TransferState*) cb->aio_sigevent.sigev_value.sival_ptr;
	
	Session = ts->Session;

	if (WAIO_StateObj.StopFlag == 0)
	{
//		cb = (struct aiocb*)lpOverlapped->hEvent;

		/* set fields on cb */
		cb->Reserved_Status = dwNumberOfBytesTransfered;
		cb->Reserved_dwErrorCode = dwErrorCode;


		/* execute the callback */
		if (cb->aio_sigevent.sigev_func != NULL)
		{
			cb->aio_sigevent.sigev_func(cb);
		}



		/*
		 *	Deallocate the data created in _waio_QueueUserAPC_Sink.
		 *	If waio_stop() is called before this executes, then
		 *	the StopFlag will be nonzero.
		 */
		#pragma warning( disable : 4311 )
		keySize = sprintf(key,"%ul", (DWORD)lpOverlapped);
		#pragma warning( default : 4311 )
		ILibHashTree_Lock(WAIO_StateObj.OverlappedTree);
		ILibDeleteEntry(WAIO_StateObj.OverlappedTree,key,keySize);
		FREE(lpOverlapped);
		ILibHashTree_UnLock(WAIO_StateObj.OverlappedTree);
	}

	ILibWebServer_Release(Session);

	//phillo,test
	WAIO_StopUnlock();


	

}

static VOID CALLBACK _waio_QueueUserAPC_Sink (ULONG_PTR pdw)
{
	char key[128];
	int keySize;
	struct aiocb *cb = (struct aiocb*)pdw;
	struct TS_TransferState *ts = NULL;
	LPOVERLAPPED overlapped  = NULL;
	char	*cacheBuffer;
	int TS_TransferState_flag = 0;
	
	if (cb == NULL) return;

	/*
	 *	The system is going to process the asynchronous I/O request
	 *	on a different worker thread by calling this method.
	 */

	/* don't read if things are shutting down */
	if (WAIO_StateObj.StopFlag != 0) return;

    if (cb->isTranscoding)
    {
		/* don't do anything if things are shutting down */
		//Birdy 2007/3/7
		WAIO_StopLock();
		if (WAIO_StateObj.StopFlag == 0)
		{
			// this buffer has been freed, don't do anything
			if (cb->aio_buf != NULL) 
			{
				cb->Reserved_Status = ReadTranscodingFile(cb->aio_fildes, cb->aio_buf, cb->aio_nbytes);
				cb->Reserved_dwErrorCode = 0;

				/* execute the callback */
				if ((cb->aio_sigevent.sigev_func != NULL) && (WAIO_StateObj.StopFlag == 0))
				{					
					//phillo,2007/5/4
					struct TS_TransferState *ts = (struct TS_TransferState*) cb->aio_sigevent.sigev_value.sival_ptr;
					ILibWebServer_AddRef(ts->Session);
					cb->aio_sigevent.sigev_func(cb);	
					ILibWebServer_Release(ts->Session);
					TS_TransferState_flag = 1;
				}
			}
		}

		//Birdy 2007/3/7
		WAIO_StopUnlock();
		//if (TS_TransferState_flag == 1)
		{
            //ILibWebServer_Release(ts->Session);
		}
			
		return;
    }
	else if (gPreCacheEnable)
	{
		//Birdy 2007/3/7
		WAIO_StopLock();

		cacheBuffer = GetCacheLine(cb->aio_fildes, cb->aio_offset, cb->aio_nbytes, cb->aio_sync_fildes);
		if ((cacheBuffer) && (WAIO_StateObj.StopFlag == 0))
		{
			
#ifdef	PRECACHE_DEBUG
			DBGPrintf((_T("Hit %x: file=%d offset=%x nbytes=%x\n"), cb, cb->aio_fildes, cb->aio_offset, cb->aio_nbytes));
#endif // PRECACHE_DEBUG

			//Birdy 2007/3/7
			WAIO_StopLock();

			/* don't do anything if things are shutting down */
			if (WAIO_StateObj.StopFlag == 0)
			{
				// this buffer has been freed, don't do anything
				if (cb->aio_buf != NULL) 
				{
					memcpy(cb->aio_buf, cacheBuffer, cb->aio_nbytes);
					cb->Reserved_Status = cb->aio_nbytes;
					cb->Reserved_dwErrorCode = 0;

					// We have time to prepare reading advance block
					CacheNextCacheLine(cb->aio_fildes, cb->aio_offset);
			
					/* execute the callback */
					if ((cb->aio_sigevent.sigev_func != NULL) && (WAIO_StateObj.StopFlag == 0))
					{
						cb->aio_sigevent.sigev_func(cb);
					}
				}
			}

			//Birdy 2007/3/7
            WAIO_StopUnlock();
		
			return;
		}

		//Birdy 2007/3/7
		WAIO_StopUnlock();
	}

	/* don't do anything if things are shutting down */
	if (WAIO_StateObj.StopFlag != 0) return;

	/*
	 *	Actually read the file asynchronously.
	 */
	overlapped = (LPOVERLAPPED)MALLOC(sizeof(OVERLAPPED));
	memset(overlapped,0,sizeof(OVERLAPPED));

	//overlapped->Offset = cb->aio_offset;
	overlapped->Offset = (DWORD)(cb->aio_offset&0xffffffff);
	overlapped->OffsetHigh = (DWORD)(cb->aio_offset>>32);

	overlapped->hEvent = cb;

	/*
	 *	Make a note of the overlapped object 
	 *	so that we can destroy at shutdown if we
	 *	need to.
	 */
	#pragma warning( disable : 4311 )
	keySize = sprintf(key,"%ul", (DWORD)overlapped);
	#pragma warning( default : 4311 )	
	ILibHashTree_Lock(WAIO_StateObj.OverlappedTree);
	ILibAddEntry(WAIO_StateObj.OverlappedTree,key,keySize,overlapped);
	ILibHashTree_UnLock(WAIO_StateObj.OverlappedTree);
	
	{
		struct TS_TransferState *ts = (struct TS_TransferState*) cb->aio_sigevent.sigev_value.sival_ptr;
		ILibWebServer_AddRef(ts->Session);
	}

	ReadFileEx(cb->aio_fildes,cb->aio_buf,cb->aio_nbytes, overlapped, _waio_ReadFileEx_Sink);

}

static DWORD WINAPI _WAIO_Run(LPVOID lpP)
{
	void *en = NULL;
	void *data = NULL;
	struct WAIO_State* waio_state = (struct WAIO_State*)lpP;

	while(WaitForSingleObjectEx(waio_state->TerminateEvent,INFINITE,TRUE)!=WAIT_OBJECT_0);

	/*
	 *	Destroy any pending overlapped objects that
	 *	have not been destroyed.
	 */
	ILibHashTree_Lock(WAIO_StateObj.OverlappedTree);
	en = ILibHashTree_GetEnumerator(WAIO_StateObj.OverlappedTree);
	while(ILibHashTree_MoveNext(en)==0)
	{
		ILibHashTree_GetValue(en,NULL,NULL,&data);
		FREE(data);
	}
	ILibHashTree_UnLock(WAIO_StateObj.OverlappedTree);
	ILibHashTree_DestroyEnumerator(en);
	ILibDestroyHashTree(WAIO_StateObj.OverlappedTree);

	sem_post(&(WAIO_StateObj.WaitForWorkerThread));
	return(0);
}
#endif


#if defined(_POSIX) || defined(_WIN32_WCE)
static void* _WAIO_Run(void* arg)
{
	struct WAIO_State* waio_state = (struct WAIO_State*)arg;
	struct aiocb* cb;
	int status;

	while (waio_state->StopFlag == 0)
	{
		ILibQueue_Lock(waio_state->ApcQueue);
		cb = (struct aiocb*) ILibQueue_DeQueue(waio_state->ApcQueue);
		ILibQueue_UnLock(waio_state->ApcQueue);

		if (cb != NULL)
		{
			/*
			 *	Read the data from the file synchronously.
			 *	When finished, execute the callback to give the impression
			 *	of asynchronous IO.
			 */
			cb->Reserved_Status = FIOA_FileRead(cb->aio_buf, sizeof(char), cb->aio_nbytes, cb->aio_fildes);
			cb->Reserved_dwErrorCode = 0;
			if (cb->aio_sigevent.sigev_func != NULL)
			{
				cb->aio_sigevent.sigev_func(cb);
			}
		}
		else
		{
			/* block if there's nothing to do */
			sem_wait(&(waio_state->WaitForNextAPC));
		}
	}

	/* remove everything from the queue so that waio_stop() can destroy the queue */
	ILibQueue_Lock(waio_state->ApcQueue);
	while (ILibQueue_DeQueue(waio_state->ApcQueue) != NULL);
	ILibQueue_UnLock(waio_state->ApcQueue);

	sem_post(&(WAIO_StateObj.WaitForWorkerThread));
	THRD_EndThisThread();
	return(NULL);
}
#endif

int waio_read(struct aiocb *cb)
{
#if defined(WIN32) && !defined(_WIN32_WCE)
	/*
	 *	Tell the system to queue an asynchronous operation and to process the operation on
	 *	a different worker thread.
	 */
	return (int) QueueUserAPC(_waio_QueueUserAPC_Sink, WAIO_StateObj.WaioWorkerThread, (ULONG_PTR)cb);
#endif

#if defined(_POSIX) || defined(_WIN32_WCE)
	ILibQueue_Lock(WAIO_StateObj.ApcQueue);
	ILibQueue_EnQueue(WAIO_StateObj.ApcQueue, cb);
	ILibQueue_UnLock(WAIO_StateObj.ApcQueue);

	/* pulse _WAIO_Run */
	sem_post(&(WAIO_StateObj.WaitForNextAPC));
#endif
}

void waio_start()
{
	//Birdy 2007/3/7
    initWAIO_StopLock();

	WAIO_StateObj.StopFlag = 0;
	sem_init(&(WAIO_StateObj.WaitForWorkerThread),1,0);

#if defined(_POSIX) || defined(_WIN32_WCE)
	WAIO_StateObj.ApcQueue = ILibQueue_Create();
	sem_init(&(WAIO_StateObj.WaitForNextAPC), 0, 1);
	sem_init(&(WAIO_StateObj.BlockIndefinitely),1,0);
#endif

#if defined(WIN32) && !defined(_WIN32_WCE)
	WAIO_StateObj.TerminateEvent = CreateEvent(NULL,TRUE,FALSE,NULL);
	WAIO_StateObj.OverlappedTree = ILibInitHashTree();
#endif

	WAIO_StateObj.WaioWorkerThread = THRD_SpawnNormalThread(_WAIO_Run, &WAIO_StateObj);
}

void waio_stop()
{	
	WAIO_StateObj.StopFlag = 1;
	
#if defined(WIN32) && !defined(_WIN32_WCE)
	SetEvent(WAIO_StateObj.TerminateEvent);
#endif

#if defined(_POSIX) || defined(_WIN32_WCE)
	/* tell _WAIO_Run to unblock */
	sem_post(&(WAIO_StateObj.BlockIndefinitely));
#endif

	/* wait for the _WAIO_Run to finish */
	sem_wait(&(WAIO_StateObj.WaitForWorkerThread));
	sem_post(&(WAIO_StateObj.WaitForWorkerThread));

	/* destroy the sempahores */
	sem_destroy(&(WAIO_StateObj.WaitForWorkerThread));

#if defined(WIN32) && !defined(_WIN32_WCE)
	CloseHandle(WAIO_StateObj.TerminateEvent);
#endif

#if defined(_POSIX) || defined(_WIN32_WCE)
	ILibQueue_Destroy(WAIO_StateObj.ApcQueue);
	sem_destroy(&(WAIO_StateObj.WaitForNextAPC));
	sem_destroy(&(WAIO_StateObj.BlockIndefinitely));
#endif
}

