#ifndef _bk_h
#define _bk_h
#define NCHARS_SYMLINK_TARGET_MAX 251
#define MAX_NBYTES_HARDLINK_HEAD 32
#define NCHARS_FILE_ID_MAX_STORE 256
#define BK_WARNING_MAX_LEN 512
#define READ_WRITE_BUFFER_SIZE 102400
#define FNTYPE_9660 1
#define FNTYPE_ROCKRIDGE 2
#define FNTYPE_JOLIET 4

#define BOOT_MEDIA_NONE 0
#define BOOT_MEDIA_NO_EMULATION 1
#define BOOT_MEDIA_1_2_FLOPPY 2 /* 1228800 byte floppy disk image */
#define BOOT_MEDIA_1_44_FLOPPY 3 /* 1474560 byte floppy disk image */
#define BOOT_MEDIA_2_88_FLOPPY 4 /* 2949120 byte floppy disk image */
#define BOOT_MEDIA_HARD_DISK 5

#define VDTYPE_BOOT 0
#define VDTYPE_PRIMARY 1
#define VDTYPE_SUPPLEMENTARY 2
#define IS_DIR(posix)      ((posix & 0770000) == 0040000)
#define IS_REG_FILE(posix) ((posix & 0770000) == 0100000)
#define IS_SYMLINK(posix)  ((posix & 0770000) == 0120000)

#include <sys/timeb.h>


#define BK_BASE_PTR(item) ((BkFileBase*)(item))
#define BK_DIR_PTR(item) ((BkDir*)(item))
#define BK_FILE_PTR(item) ((BkFile*)(item))
#define BK_SYMLINK_PTR(item) ((BkSymLink*)(item))
/*******************************************************************************
* VolInfo
* Information about a volume (one image).
* Strings are '\0' terminated. */
typedef __int64 bk_off_t;

typedef struct BkFileBase
{
    char original9660name[15]; /* 8.3 + ";1" max */
    char name[NCHARS_FILE_ID_MAX_STORE]; /* '\0' terminated */
    unsigned posixFileMode; /* file type and permissions */
    
    struct BkFileBase* next;
    
} BkFileBase;

typedef struct BkHardLink
{
    int onImage;//bool onImage;
    bk_off_t position; /* if on image */
    char* pathAndName; /* if on filesystem, full path + filename
                       * is to be freed whenever the BkHardLink is freed */
    unsigned size; /* size of the file being pointed to */
    int headSize;
    unsigned char head[MAX_NBYTES_HARDLINK_HEAD];
    int alreadyCounted;//bool alreadyCounted; /* for estimateIsoSize() */
    
    unsigned extentNumberWrittenTo; /* only set once one file is written */
    
     struct BkHardLink* next;
    
} BkHardLink;

typedef struct BkDir
{
    BkFileBase base; /* intended to be accessed using a cast */
    
    BkFileBase* children; /* child directories, files, etc. */
    
} BkDir;

typedef struct BkFile
{
    BkFileBase base; 
    unsigned size; 
    BkHardLink* location;
    int onImage;
    bk_off_t position; 
    char* pathAndName;     
} BkFile;


typedef struct BkSymLink
{
    BkFileBase base; 
    
    char target[NCHARS_SYMLINK_TARGET_MAX];
    
} BkSymLink;


typedef struct VolInfo
{
    unsigned filenameTypes;
    bk_off_t pRootDrOffset; 
    bk_off_t sRootDrOffset; 
    bk_off_t bootRecordSectorNumberOffset;
    HANDLE imageForReading;
    int imageForReadingInode; 
    const BkFile* bootRecordOnImage; 
    char warningMessage[BK_WARNING_MAX_LEN];
    int rootRead;
    int stopOperation; 
    int imageForWriting;
    void(*progressFunction)(struct VolInfo*);
    void(*writeProgressFunction)(struct VolInfo*, double);
    struct timeb lastTimeCalledProgress;
    bk_off_t estimatedIsoSize;
    BkHardLink* fileLocations;
    char readWriteBuffer[READ_WRITE_BUFFER_SIZE];
    char readWriteBuffer2[READ_WRITE_BUFFER_SIZE];
 
    time_t creationTime;
    BkDir dirTree;
    unsigned char bootMediaType;
    unsigned bootRecordSize;       
    int bootRecordIsOnImage;
    bk_off_t bootRecordOffset;     
    char* bootRecordPathAndName;  
    int bootRecordIsVisible;
    int scanForDuplicateFiles;
    BYTE volId[33];
    BYTE publisher[129];
    BYTE dataPreparer[129];
    unsigned posixFileDefaults;    
    unsigned posixDirDefaults;     
    int (*warningCbk)(const char*);
    int followSymLinks;
} VolInfo;
void printNameAndContents(BkFileBase* base, int numSpaces);

#endif 