#include <stdafx.h>
#include <stdlib.h>
#include "bkDelete.hpp"

void deleteDirContents(VolInfo* volInfo, BkDir* dir)
{
    BkFileBase* child;
    BkFileBase* nextChild;
    
    child = dir->children;
    while(child != NULL)
    {
        nextChild = child->next;
        
        deleteNode(volInfo, dir, child->name);
        
        child = nextChild;
    }
}


void deleteNode(VolInfo* volInfo, BkDir* parentDir, char* nodeToDeleteName)
{
    BkFileBase** childPtr;
    BkFileBase* nodeToFree;
    
    childPtr = &(parentDir->children);
    while(*childPtr != NULL)
    {
        if( strcmp((*childPtr)->name, nodeToDeleteName) == 0 )
        {
            nodeToFree = *childPtr;
            
            *childPtr = (*childPtr)->next;
            
            if( IS_DIR(nodeToFree->posixFileMode) )
            {
                deleteDirContents(volInfo, BK_DIR_PTR(nodeToFree));
            }
            else if ( IS_REG_FILE(nodeToFree->posixFileMode) )
            {
                deleteRegFileContents(volInfo, BK_FILE_PTR(nodeToFree));
            }
            /* else the free below will be enough */
            
            free(nodeToFree);
            
            break;
        }
        
        childPtr = &((*childPtr)->next);
    }
}

void deleteRegFileContents(VolInfo* volInfo, BkFile* file)
{
    if( file->onImage )
        free( file->pathAndName );
    
    /* check whether file is being used as a boot record */
    if(volInfo->bootMediaType != BOOT_MEDIA_NONE &&
       volInfo->bootMediaType == BOOT_MEDIA_NO_EMULATION)
    {
        if(volInfo->bootRecordIsVisible && 
           volInfo->bootRecordOnImage == file)
        {
            /* and stop using it. perhaps insert a hook here one day to
            * let the user know the boot record has been/will be deleted */
            bk_delete_boot_record(volInfo);
        }
    }
}

void bk_delete_boot_record(VolInfo* volInfo)
{
    volInfo->bootMediaType = BOOT_MEDIA_NONE;
    
    if(volInfo->bootRecordPathAndName != NULL)
    {
        free(volInfo->bootRecordPathAndName);
        volInfo->bootRecordPathAndName = NULL;
    }
}