#include <stdafx.h>
#include <stdlib.h>
#include "bkLink.hpp"
#include "bkerror.hpp"
#include "bkIoWrappers.hpp"
/* returns 2 if found 1 if not found */
int findInHardLinkTable(VolInfo* volInfo, bk_off_t position, 
                        char* pathAndName, unsigned size,
                        int onImage, BkHardLink** foundLink)
{
    BkHardLink* currentLink;
    unsigned char head[MAX_NBYTES_HARDLINK_HEAD];
    int headSize;
    int rc;
    
    *foundLink = NULL;
    
    if(size < MAX_NBYTES_HARDLINK_HEAD)
        headSize = size;
    else
        headSize = MAX_NBYTES_HARDLINK_HEAD;
    
    rc = readFileHead(volInfo, position, pathAndName, onImage, head, headSize);
    if(rc <= 0)
        return rc;
    
    currentLink = volInfo->fileLocations;
    while(currentLink != NULL)
    {
        if(size == currentLink->size)
        {
            if( memcmp(head, currentLink->head, headSize) == 0 )
            {
                HANDLE origFile;
                int origFileWasOpened;
                bk_off_t origFileOffset;
                HANDLE newFile;
                HANDLE newFileWasOpened;
                bk_off_t newFileOffset;
                
                
                if(currentLink->onImage)
                {
                    origFile = volInfo->imageForReading;
                    origFileWasOpened = 0;
                    origFileOffset = currentLink->position;
                }
                else
                {
                    //origFile = (int)fopen(pathAndName, "r");
                    if(origFile == NULL)//-1)
                        return BKERROR_OPEN_READ_FAILED;
                    origFileWasOpened = 1;
                    origFileOffset = 0;
                }
                
                
                if(onImage)
                {
                    newFile = volInfo->imageForReading;
                    newFileWasOpened = 0;
                    newFileOffset = position;
                }
                else
                {
                   // newFile = (int)fopen(pathAndName, "r");
                    if(newFile == NULL)
                    {
                        if(origFileWasOpened)
                            bkClose((HANDLE)origFile);
                        return BKERROR_OPEN_READ_FAILED;
                    }
                    newFileWasOpened = (HANDLE)1;
                    newFileOffset = 0;
                }
                
                rc = filesAreSame(volInfo, origFile, origFileOffset, 
                                  newFile, newFileOffset, size);
                
                if(origFileWasOpened)
                    bkClose((HANDLE)origFile);
                if(newFileWasOpened)
                    bkClose((HANDLE)newFile);
                
                if(rc < 0)
                    return rc;
                
                if(rc == 2)
                {
                    *foundLink = currentLink;
                    return 2;
                }
            }
        }
        
        currentLink = currentLink->next;
    }
    
    return 1;
}


int addToHardLinkTable(VolInfo* volInfo, bk_off_t position, char* pathAndName, 
                       unsigned size, int onImage, BkHardLink** newLink)
{
    int rc;
    
    *newLink = (BkHardLink*)malloc(sizeof(BkHardLink));
    if(*newLink == NULL)
        return BKERROR_OUT_OF_MEMORY;
    
    memset(*newLink, 0, sizeof(BkHardLink));
    
    (*newLink)->onImage = onImage;
    (*newLink)->position = position;
    if(pathAndName != NULL)
    {
        (*newLink)->pathAndName = (char *)malloc(strlen(pathAndName) + 1);
        if((*newLink)->pathAndName == NULL)
            return BKERROR_OUT_OF_MEMORY;
        strcpy((*newLink)->pathAndName, pathAndName);
    }
    (*newLink)->size = size;
    (*newLink)->next = volInfo->fileLocations;
    
    if(size < MAX_NBYTES_HARDLINK_HEAD)
        (*newLink)->headSize = size;
    else
        (*newLink)->headSize = MAX_NBYTES_HARDLINK_HEAD;
    
    rc = readFileHead(volInfo, position, pathAndName, (*newLink)->onImage, 
                      (*newLink)->head, (*newLink)->headSize);
    if(rc <= 0)
        return rc;
    
    volInfo->fileLocations = *newLink;
    
    return 1;
}

int readFileHead(VolInfo* volInfo, bk_off_t position, char* pathAndName, 
                 int onImage, unsigned char* dest, int numBytes)
{
    HANDLE srcFile;
    int srcFileWasOpened;
    bk_off_t origPos;
    int rc;
    
    if(onImage)
    {
        srcFile = volInfo->imageForReading;
        origPos = bkSeekTell((HANDLE)volInfo->imageForReading);
        bkSeekSet(volInfo->imageForReading, position, FILE_BEGIN);
        srcFileWasOpened = 0;
    }
    else
    {
      //  srcFile = (int)fopen(pathAndName, "r");
        if(srcFile == (HANDLE)-1)
            return BKERROR_OPEN_READ_FAILED;
        srcFileWasOpened = 1;
    }
    
	LPBYTE ptr=(LPBYTE)dest;
    rc = bkRead(srcFile, ptr, numBytes);
    
    if(srcFileWasOpened)
        bkClose(srcFile);
    else
        bkSeekSet(volInfo->imageForReading, origPos, FILE_BEGIN);
    
    if(rc != numBytes)
        return BKERROR_READ_GENERIC;
    
    return 1;
}

int filesAreSame(VolInfo* volInfo, HANDLE file1, bk_off_t posFile1, 
                 HANDLE file2, bk_off_t posFile2, unsigned size)
{
    bk_off_t origPosFile1;
    bk_off_t origPosFile2;
    int numBlocks;
    int sizeLastBlock;
    int count;
    int rc;
    int sameSoFar;
    
    if(size == 0)
        return 2;
    
    origPosFile1 = bkSeekTell(file1);
    origPosFile2 = bkSeekTell(file2);
    
    numBlocks = size / READ_WRITE_BUFFER_SIZE;
    sizeLastBlock = size % READ_WRITE_BUFFER_SIZE;
    
    sameSoFar = 1;
    for(count = 0; count < numBlocks; count++)
    {
        bkSeekSet(file1, posFile1, FILE_BEGIN);
		LPBYTE ptr=(LPBYTE) volInfo->readWriteBuffer;
        rc = bkRead(file1, ptr, READ_WRITE_BUFFER_SIZE);
        if(rc != READ_WRITE_BUFFER_SIZE)
            return BKERROR_READ_GENERIC;
        posFile1 = bkSeekTell(file1);
        
        bkSeekSet(file2, posFile2, FILE_BEGIN);
		ptr= (LPBYTE)volInfo->readWriteBuffer2;
        rc = bkRead(file2, ptr, READ_WRITE_BUFFER_SIZE);
        if(rc != READ_WRITE_BUFFER_SIZE)
            return BKERROR_READ_GENERIC;
        posFile2 = bkSeekTell(file2);
        
        if( memcmp(volInfo->readWriteBuffer, volInfo->readWriteBuffer2, 
            READ_WRITE_BUFFER_SIZE) != 0 )
        {
            sameSoFar = 0;
            break;
        }
    }
    
    if(sameSoFar && sizeLastBlock > 0)
    {
        bkSeekSet(file1, posFile1, FILE_BEGIN);
		LPBYTE ptr= (LPBYTE)volInfo->readWriteBuffer;
        rc = bkRead(file1, ptr, sizeLastBlock);
        if(rc != sizeLastBlock)
            return BKERROR_READ_GENERIC;
        
        bkSeekSet(file2, posFile2, FILE_BEGIN);
		ptr=(LPBYTE)volInfo->readWriteBuffer2;
        rc = bkRead(file2,ptr , sizeLastBlock);
        if(rc != sizeLastBlock)
            return BKERROR_READ_GENERIC;
        
        if(memcmp(volInfo->readWriteBuffer, volInfo->readWriteBuffer2, sizeLastBlock) != 0)
            sameSoFar = 0;
    }
    
    bkSeekSet(file1, origPosFile1, FILE_BEGIN);
    bkSeekSet(file2, origPosFile2, FILE_BEGIN);
    
    if(sameSoFar)
        return 2;
    else
        return 1;
}