#include <stdafx.h>
#include <limits.h>
#include <stdlib.h>
#include "bkInternal.hpp"
#include "bkRead.hpp"
#include "bk.hpp"
#include "bkIoWrappers.hpp"
#include "bkRead7x.hpp"
#include "bkTime.hpp"
#include "bkMisc.hpp"
#include "bkLink.hpp"
int bk_open_image(VolInfo *volInfo,TCHAR *filename)
{
	//TCHAR *tmp;
	

    volInfo->imageForReading =(HANDLE) bkOpen(filename);
	
	if(volInfo->imageForReading == (HANDLE)-1)
    {
        volInfo->imageForReading = 0;
        return BKERROR_OPEN_READ_FAILED;
    };
    
	return 1;
}


/*******************************************************************************
* bk_read_vol_info()
* public function to read volume information
* assumes pvd is first descriptor in set
* */

int bk_read_vol_info(VolInfo* volInfo)
{
    DWORD rc=0;
	BYTE vdType;
    int haveMorePvd; 
    BYTE escapeSequence[3]; 
    BYTE timeString[17]; 
    bk_off_t locationOfNextDescriptor;
    BYTE bootCatalogLocation;
    char elToritoSig[24];
    BYTE bootMediaType;
    unsigned short bootRecordSize;
    unsigned bootRecordSectorNumber;
    
    
    unsigned realRootLoc; 
    BYTE recordLen; 
    BYTE sPsUentry[7]; 
    
    volInfo->filenameTypes = FNTYPE_9660;
   
    
    volInfo->sRootDrOffset = 0;
    
    
	
    readSeekSet(volInfo, NLS_SYSTEM_AREA * NBYTES_LOGICAL_BLOCK, FILE_BEGIN);

	rc=read711(volInfo,&vdType);
    if(rc != 1)
        return BKERROR_READ_GENERIC;
    

    if(vdType != VDTYPE_PRIMARY)
        return BKERROR_VD_NOT_PRIMARY;
  
    readSeekSet(volInfo, 39, FILE_CURRENT);
    
    rc = readRead(volInfo, volInfo->volId, 32);
    if(rc != 32)
        return BKERROR_READ_GENERIC;
       
	volInfo->volId[32] = '\0';

	stripSpacesFromEndOfString(volInfo->volId);
    
    readSeekSet(volInfo, 84, FILE_CURRENT);

    volInfo->pRootDrOffset = readSeekTell(volInfo);
    
   
    readSeekSet(volInfo, 2, FILE_CURRENT);
   
    rc = read733(volInfo, &realRootLoc);
    if(rc != 8)
        return BKERROR_READ_GENERIC;
    realRootLoc *= NBYTES_LOGICAL_BLOCK;

    readSeekSet(volInfo, realRootLoc, FILE_BEGIN);
    
    rc = read711(volInfo, &recordLen);
    if(rc != 1)
        return BKERROR_READ_GENERIC;

    if(recordLen >= 41)
    {
        readSeekSet(volInfo, 33, FILE_CURRENT);
        
          
        rc = readRead(volInfo, sPsUentry, 7);
        if(rc != 7)
            return BKERROR_READ_GENERIC;
        
        if( sPsUentry[0] == 0x53 && sPsUentry[1] == 0x50 &&
            sPsUentry[2] == 7 && 
            sPsUentry[4] == 0xBE && sPsUentry[5] == 0xEF )
        {
            volInfo->filenameTypes |= FNTYPE_ROCKRIDGE;
        }
    }

    readSeekSet(volInfo, volInfo->pRootDrOffset, FILE_BEGIN);
    
    
    readSeekSet(volInfo, 162, FILE_CURRENT);
    
    rc = readRead(volInfo, volInfo->publisher, 128);

    if(rc != 128)
        return BKERROR_READ_GENERIC;
    volInfo->publisher[128] = '\0';
    stripSpacesFromEndOfString(volInfo->publisher);

    rc = readRead(volInfo, volInfo->dataPreparer, 128);
    if(rc != 128)
        return BKERROR_READ_GENERIC;
    volInfo->dataPreparer[128] = '\0';
    stripSpacesFromEndOfString(volInfo->dataPreparer);

    readSeekSet(volInfo, 239, FILE_CURRENT);
    
    rc = readRead(volInfo, timeString, 17);
    if(rc != 17)
        return BKERROR_READ_GENERIC;

    longStringToEpoch(timeString, &(volInfo->creationTime));
    
    readSeekSet(volInfo, 1218, FILE_CURRENT);
    
	haveMorePvd = 1;
    while(haveMorePvd)
    {
        rc = read711(volInfo, &vdType);
        if(rc != 1)
            return BKERROR_READ_GENERIC;
        
       if(vdType == VDTYPE_PRIMARY)
        {
            readSeekSet(volInfo, 2047, FILE_CURRENT);
        }
        else
        {
			//fseek((FILE*)volInfo->imageForReading,-1,FILE_CURRENT);
            readSeekSet(volInfo, -1, FILE_CURRENT);//SEEK_CUR);
            haveMorePvd = 0;
        }
		
    }

    locationOfNextDescriptor = readSeekTell(volInfo) + 2048;
    
    rc = read711(volInfo, &vdType);
    if(rc != 1)
        return BKERROR_READ_GENERIC;

	if(vdType == VDTYPE_BOOT)
    {
        
        readSeekSet(volInfo, 6, SEEK_CUR);
        
		LPBYTE ptr=(LPBYTE)elToritoSig;
        rc = readRead(volInfo, ptr, 24);
        if(rc != 24)
            return BKERROR_READ_GENERIC;
        elToritoSig[23] = '\0';
        
        if(strcmp(elToritoSig, "EL TORITO SPECIFICATION") == 0)
        {
            readSeekSet(volInfo, 40, SEEK_CUR);
            
            rc = read731(volInfo, &bootCatalogLocation);
            if(rc != 4)
                return BKERROR_READ_GENERIC;
            
            readSeekSet(volInfo, bootCatalogLocation * NBYTES_LOGICAL_BLOCK, FILE_BEGIN);
            

            readSeekSet(volInfo, 32, SEEK_CUR);

            readSeekSet(volInfo, 1, SEEK_CUR);
            
            rc = readRead(volInfo, &bootMediaType, 1);
            if(rc != 1)
                return BKERROR_READ_GENERIC;
            if(bootMediaType == 0)
                volInfo->bootMediaType = BOOT_MEDIA_NO_EMULATION;
            else if(bootMediaType == 1)
                volInfo->bootMediaType = BOOT_MEDIA_1_2_FLOPPY;
            else if(bootMediaType == 2)
                volInfo->bootMediaType = BOOT_MEDIA_1_44_FLOPPY;
            else if(bootMediaType == 3)
                volInfo->bootMediaType = BOOT_MEDIA_2_88_FLOPPY;
            else if(bootMediaType == 4)
            {
                printf("hard disk boot emulation not supported\n");
                volInfo->bootMediaType = BOOT_MEDIA_NONE;
            }
            else
            {
                printf("unknown boot media type on iso\n");
                volInfo->bootMediaType = BOOT_MEDIA_NONE;
            }
            
            readSeekSet(volInfo, 4, SEEK_CUR);
            
            rc = read721(volInfo, &bootRecordSize);
            if(rc != 2)
                return BKERROR_READ_GENERIC;
            volInfo->bootRecordSize = bootRecordSize;
            
            if(volInfo->bootMediaType == BOOT_MEDIA_NO_EMULATION)
                volInfo->bootRecordSize *= NBYTES_VIRTUAL_SECTOR;
            else if(bootMediaType == BOOT_MEDIA_1_2_FLOPPY)
                volInfo->bootRecordSize = 1200 * 1024;
            else if(bootMediaType == BOOT_MEDIA_1_44_FLOPPY)
                volInfo->bootRecordSize = 1440 * 1024;
            else if(bootMediaType == BOOT_MEDIA_2_88_FLOPPY)
                volInfo->bootRecordSize = 2880 * 1024;
            
            volInfo->bootRecordIsOnImage = 1;
            
			LPBYTE ptrSect=(LPBYTE)&bootRecordSectorNumber;
            rc = read731(volInfo, ptrSect);
            if(rc != 4)
                return BKERROR_READ_GENERIC;
            volInfo->bootRecordOffset = bootRecordSectorNumber * 
                                        NBYTES_LOGICAL_BLOCK;
        }
        else
            printf("err, boot record not el torito\n");
        
        readSeekSet(volInfo, locationOfNextDescriptor, FILE_BEGIN);
    }
    else
    {
        readSeekSet(volInfo, -1, FILE_CURRENT);
    }

	
	rc = read711(volInfo, &vdType);
    if(rc != 1)
        return BKERROR_READ_GENERIC;
    
    if(vdType == VDTYPE_SUPPLEMENTARY)
    {
        readSeekSet(volInfo, 87, FILE_CURRENT);
        
        rc = readRead(volInfo, escapeSequence, 3);
        if(rc != 3)
            return BKERROR_READ_GENERIC;
        
        if( (escapeSequence[0] == 0x25 && escapeSequence[1] == 0x2F &&
             escapeSequence[2] == 0x40) ||
            (escapeSequence[0] == 0x25 && escapeSequence[1] == 0x2F &&
             escapeSequence[2] == 0x43) ||
            (escapeSequence[0] == 0x25 && escapeSequence[1] == 0x2F &&
             escapeSequence[2] == 0x45) )
        {
            readSeekSet(volInfo, 65, FILE_CURRENT);
            
            volInfo->sRootDrOffset = readSeekTell(volInfo);
            
            volInfo->filenameTypes |= FNTYPE_JOLIET;
        }
    }
    DWORD pos=SetFilePointer(volInfo->imageForReading,0x0,0x0,FILE_CURRENT);   
    return 1;
}

DWORD readRead(VolInfo* volInfo, LPBYTE dest, int numBytes)
{
	DWORD res;
	
	ReadFile((HANDLE)volInfo->imageForReading,dest,numBytes,&res,NULL);
    
	return res;
}


void stripSpacesFromEndOfString(unsigned char* str)
{
    size_t count;
    
    for(count = strlen((char*)str) - 1; str[count] == ' '; count--)
    {
        str[count] = '\0';

        if(count == 0) 
            break;
    }
}


int bk_read_dir_tree(VolInfo* volInfo, int filenameType, 
                     int keepPosixPermissions, 
                     void(*progressFunction)(VolInfo*))
{
    volInfo->progressFunction = progressFunction;
    
    if(filenameType == FNTYPE_ROCKRIDGE || filenameType == FNTYPE_9660)
        readSeekSet(volInfo, volInfo->pRootDrOffset, FILE_BEGIN);
    else /* if(filenameType == FNTYPE_JOLIET) */
        readSeekSet(volInfo, volInfo->sRootDrOffset, FILE_BEGIN);
    
    return readDir(volInfo, &(volInfo->dirTree), 
                   filenameType, keepPosixPermissions);
}

int readDir(VolInfo* volInfo, BkDir* dir, int filenameType, 
            int keepPosixPermissions)
{
    int rc;
    unsigned char recordLength;
    unsigned locExtent; 
    unsigned lenExtent; 
    unsigned char lenFileId9660; 
    int lenSU; 
    bk_off_t origPos;
    
   
    dir->children = NULL;
    
    if(volInfo->stopOperation)
        return BKERROR_OPER_CANCELED_BY_USER;
    
	maybeUpdateProgress(volInfo);
    
    rc = readRead(volInfo, &recordLength, 1);
    if(rc != 1)
        return BKERROR_READ_GENERIC;
    
    readSeekSet(volInfo, 1, FILE_CURRENT);
      
    rc = read733(volInfo, &locExtent);
    if(rc != 8)
        return BKERROR_READ_GENERIC;
    
    rc = read733(volInfo, &lenExtent);
    if(rc != 8)
        return BKERROR_READ_GENERIC;   
    readSeekSet(volInfo, 14, SEEK_CUR);
    
    rc = readRead(volInfo, &lenFileId9660, 1);
    if(rc != 1)
        return BKERROR_READ_GENERIC;
    
    lenSU = recordLength - 33 - lenFileId9660;
    if(lenFileId9660 % 2 == 0)
        lenSU -= 1;
    
    
    if(volInfo->rootRead)
    {
        bk_off_t posBeforeName = readSeekTell(volInfo);
        
		LPBYTE ptr=(LPBYTE)BK_BASE_PTR(dir)->name;
        rc = readRead(volInfo, ptr, lenFileId9660);
        if(rc != lenFileId9660)
            return BKERROR_READ_GENERIC;
        BK_BASE_PTR(dir)->name[lenFileId9660] = '\0';
        //BK_BASE_PTR(dir)->name[lenFileId9660+1] = '\0';
        wchar_t nameInAscii[UCHAR_MAX];
		wchar_t *src,*dst;
		wchar_t tmp_h,tmp_l,tmp;



		if(volInfo->filenameTypes & FNTYPE_JOLIET)
		{

			src = (wchar_t*) ptr;
			for( int i = 0 ; i <(lenFileId9660+1);i++)
			{
				tmp_h = src[i]&0xff;
				tmp_l = src[i]>>8;
				tmp = (tmp_h<<8 ) | tmp_l;
				nameInAscii[i]=tmp;
			}
			
			char *tmp_c;
			tmp_c = (char*)nameInAscii;
			wcsncpy(src,nameInAscii,lenFileId9660+1);
	
			src = (wchar_t*) ptr;
		
			ptr = (LPBYTE) (BK_BASE_PTR(dir)->original9660name);
			dst = (wchar_t*) ptr;
		
			int len = wcslen(src);

			wcsncpy(dst,src,len+1);

  		}else if(volInfo->filenameTypes ==FNTYPE_9660)
		{
			src = (wchar_t*) ptr;
			for( int i = 0 ; i <(lenFileId9660+1);i++)
			{
				tmp_h = src[i]&0xff;
				tmp_l = src[i]>>8;
				//tmp = (tmp_h<<8 ) | tmp_l;
				nameInAscii[2*i]=tmp_h;
				nameInAscii[2*i+1]=tmp_l;
			//	nameInAscii[2*i]=tmp;
			}
			
			int len = wcslen(nameInAscii);
			wcsncpy((wchar_t*)ptr,nameInAscii,len+1);
		}
		
		if(lenFileId9660 % 2 == 0)
				readSeekSet(volInfo, 1, SEEK_CUR);		
        
       if(filenameType != FNTYPE_9660)
         readSeekSet(volInfo, posBeforeName, SEEK_SET);
    }
    
    if(filenameType == FNTYPE_JOLIET)
    {
        if(volInfo->rootRead)
        {
            char nameAsOnDisk[UCHAR_MAX];
            
            wchar_t nameInAscii[UCHAR_MAX];
            //int ucsCount, byteCount;
            
            
            if(lenFileId9660 % 2 != 0)
                return BKERROR_INVALID_UCS2;
            
			LPBYTE ptr=(LPBYTE)nameAsOnDisk;
            rc = readRead(volInfo, ptr, lenFileId9660);
            if(rc != lenFileId9660)
                return BKERROR_READ_GENERIC;
            
			ptr[lenFileId9660]='\0';
			ptr[lenFileId9660+1]='\0';

			wchar_t *src, *dst,tmp_h,tmp_l,tmp;
			src = (wchar_t*) ptr;
			for( int i = 0 ; i <(lenFileId9660+1);i++)
			{
				tmp_h = src[i]&0xff;
				tmp_l = src[i]>>8;
				tmp = (tmp_h<<8 ) | tmp_l;
				nameInAscii[i]=tmp;
			}
			


			src = (wchar_t*) nameInAscii;

			ptr = (LPBYTE)(BK_BASE_PTR(dir)->name);
			dst = (wchar_t*) ptr;

			int len = wcslen(src);

			wcsncpy(dst,src,len+1);

			len = wcslen(dst);
            //strncpy(BK_BASE_PTR(dir)->name, nameInAscii, lenFileId9660);
            //BK_BASE_PTR(dir)->name[lenFileId9660] = '\0';
            
            
            if(lenFileId9660 % 2 == 0)
                readSeekSet(volInfo, 1, SEEK_CUR);
        }
    }
    else if(filenameType == FNTYPE_ROCKRIDGE)
    {
        if(volInfo->rootRead)
        {
            
            readSeekSet(volInfo, lenFileId9660, SEEK_CUR);
            
            if(lenFileId9660 % 2 == 0)
                readSeekSet(volInfo, 1, SEEK_CUR);
            
            rc = readRockridgeFilename(volInfo, BK_BASE_PTR(dir)->name, lenSU, 0);
            if(rc < 0)
                return rc;
        }
    }
    else if(filenameType != FNTYPE_9660)
        return BKERROR_UNKNOWN_FILENAME_TYPE;
    
    
    if(keepPosixPermissions)
    {
        if( !(volInfo->rootRead) )
        {
            unsigned char realRootRecordLen;
            
            origPos = readSeekTell(volInfo);
            
            
            readSeekSet(volInfo, locExtent * NBYTES_LOGICAL_BLOCK, SEEK_SET);
            
            
            readRead(volInfo, &realRootRecordLen, 1);
            if(rc != 1)
                return BKERROR_READ_GENERIC;
            
            
            readSeekSet(volInfo, 33, SEEK_CUR);
            
            rc = readPosixFileMode(volInfo, &(BK_BASE_PTR(dir)->posixFileMode), realRootRecordLen - 34);
            if(rc <= 0)
                return rc;
            
            
            readSeekSet(volInfo, origPos, SEEK_SET);
        }
        else
        {
            rc = readPosixFileMode(volInfo, &(BK_BASE_PTR(dir)->posixFileMode), lenSU);
            if(rc <= 0)
                return rc;
        }
    }
    else
    {
        
        BK_BASE_PTR(dir)->posixFileMode = volInfo->posixDirDefaults;
    }
   
    readSeekSet(volInfo, lenSU, SEEK_CUR);
    
    origPos = readSeekTell(volInfo);
    
    readSeekSet(volInfo, locExtent * NBYTES_LOGICAL_BLOCK, SEEK_SET);
    
    volInfo->rootRead = 1;
    
    rc = readDirContents(volInfo, dir, lenExtent, filenameType, keepPosixPermissions);
    if(rc < 0)
        return rc;
    
    readSeekSet(volInfo, origPos, SEEK_SET);
    
    return recordLength;
}

int readRockridgeFilename(VolInfo* volInfo, char* dest, int lenSU, 
                          unsigned numCharsReadAlready)
{
    bk_off_t origPos;
    unsigned char* suFields;
    int rc;
    int count;
    int lengthThisNM;
    int usableLenThisNM;
    int foundName;
    int nameContinues; /* in another NM entry */
    int foundCE;
    unsigned logicalBlockOfCE;
    unsigned offsetInLogicalBlockOfCE;
    unsigned lengthOfCE; /* in bytes */
    
   suFields =(unsigned char*) malloc(lenSU);
    if(suFields == NULL)
        return BKERROR_OUT_OF_MEMORY;
    
    origPos = readSeekTell(volInfo);
    
    rc = readRead(volInfo, suFields, lenSU);
    if(rc != lenSU)
    {
        free(suFields);
        return BKERROR_READ_GENERIC;
    }
    
    count = 0;
    foundName = 0;
    nameContinues = 0;
    foundCE = 0;
    while(count < lenSU)
    {
        if(suFields[count] == 0)

            break;
        
        if(suFields[count] == 'N' && suFields[count + 1] == 'M')
        {
            lengthThisNM = suFields[count + 2] - 5;
            
            
            if(lengthThisNM + numCharsReadAlready > NCHARS_FILE_ID_MAX_STORE - 1)
                usableLenThisNM = NCHARS_FILE_ID_MAX_STORE - numCharsReadAlready - 1;
            else
                usableLenThisNM = lengthThisNM;
            
            strncpy(dest + numCharsReadAlready, (char*)suFields + count + 5, usableLenThisNM);
            dest[usableLenThisNM + numCharsReadAlready] = '\0';
            
            numCharsReadAlready += usableLenThisNM;
            
            foundName = 1;
            nameContinues = suFields[count + 4] & 0x01; 
        }
        else if(suFields[count] == 'C' && suFields[count + 1] == 'E')
        {
            foundCE = 1;
            read733FromCharArray(suFields + count + 4, &logicalBlockOfCE);
            read733FromCharArray(suFields + count + 12, &offsetInLogicalBlockOfCE);
            read733FromCharArray(suFields + count + 20, &lengthOfCE);
        }
        
       
        count += suFields[count + 2];
    }
    
    free(suFields);
    readSeekSet(volInfo, origPos, SEEK_SET);
    
    if( !foundName || (foundName && nameContinues) )
    {
        if(!foundCE)
            return BKERROR_RR_FILENAME_MISSING;
        else
        {
            readSeekSet(volInfo, 
                        logicalBlockOfCE * NBYTES_LOGICAL_BLOCK + offsetInLogicalBlockOfCE, 
                        SEEK_SET);
            rc = readRockridgeFilename(volInfo, dest, lengthOfCE, numCharsReadAlready);
            
            readSeekSet(volInfo, origPos, SEEK_SET);
            
            return rc;
        }
    }
    else
		
        return 1;
}


int readPosixFileMode(VolInfo* volInfo, unsigned* posixFileMode, int lenSU)
{
    bk_off_t origPos;
    unsigned char* suFields;
    int rc;
    int foundPosix;
    int foundCE;
    int count;
    unsigned logicalBlockOfCE;
    unsigned offsetInLogicalBlockOfCE;
    unsigned lengthOfCE; /* in bytes */
    
    suFields = (unsigned char*)malloc(lenSU);
    if(suFields == NULL)
        return BKERROR_OUT_OF_MEMORY;
    
    origPos = readSeekTell(volInfo);
    
    rc = readRead(volInfo, suFields, lenSU);
    
    if(rc != lenSU)
        return BKERROR_READ_GENERIC;
    count = 0;
    foundPosix = 0;
    foundCE = 0;
    while(count < lenSU && !foundPosix)
    {
        if(suFields[count] == 0)
        /* not an SU field, mkisofs sometimes has a trailing 0 in the directory
        * record and this is the easiest way to ignore it */
            break;
        
        if(suFields[count] == 'P' && suFields[count + 1] == 'X')
        {
            read733FromCharArray(suFields + count + 4, posixFileMode);
            
            /* not interested in anything else from this field */
            
            foundPosix = 1;
        }
        else if(suFields[count] == 'C' && suFields[count + 1] == 'E')
        {
            foundCE = 1;
            read733FromCharArray(suFields + count + 4, &logicalBlockOfCE);
            read733FromCharArray(suFields + count + 12, &offsetInLogicalBlockOfCE);
            read733FromCharArray(suFields + count + 20, &lengthOfCE);
        }
        
        /* skip su record */
        count += suFields[count + 2];
    }
    
    free(suFields);
    readSeekSet(volInfo, origPos, SEEK_SET);
    
    if(!foundPosix)
    {
        if(!foundCE)
            return BKERROR_NO_POSIX_PRESENT;
        else
        {
            readSeekSet(volInfo, logicalBlockOfCE * NBYTES_LOGICAL_BLOCK + 
                        offsetInLogicalBlockOfCE, SEEK_SET);
            rc = readPosixFileMode(volInfo, posixFileMode, lengthOfCE);
            
            readSeekSet(volInfo, origPos, SEEK_SET);
            
            return rc;
        }
    }
    
    return 1;
}


int readDirContents(VolInfo* volInfo, BkDir* dir, unsigned size, 
                    int filenameType, int keepPosixPermissions)
{
    int rc;
    unsigned bytesRead = 0;
    unsigned childrenBytesRead;
    BkFileBase** nextChild; 
    
    
    rc = skipDR(volInfo);
    if(rc <= 0)
        return rc;
    bytesRead += rc;
    rc = skipDR(volInfo);
    if(rc <= 0)
        return rc;
    bytesRead += rc;
    
    nextChild = &(dir->children);
    childrenBytesRead = 0;
    while(childrenBytesRead + bytesRead < size)
    {
        if(haveNextRecordInSector(volInfo))
        
        {
            int recordLength;
            
            if( dirDrFollows(volInfo) )
            
            {
                *nextChild = (BkFileBase*)malloc(sizeof(BkDir));
                if(*nextChild == NULL)
                    return BKERROR_OUT_OF_MEMORY;
                
                memset(*nextChild, 0, sizeof(BkDir));
                
                recordLength = readDir(volInfo, BK_DIR_PTR(*nextChild), 
                                       filenameType, keepPosixPermissions);
                if(recordLength < 0)
                    return recordLength;
            }
            else
            
            {
                BkFileBase* specialFile;
                
                
                *nextChild = (BkFileBase*)malloc(sizeof(BkFile));
                if(*nextChild == NULL)
                    return BKERROR_OUT_OF_MEMORY;
                
                memset(*nextChild, 0, sizeof(BkFile));
                
                recordLength = readFileInfo(volInfo, BK_FILE_PTR(*nextChild), 
                                            filenameType, keepPosixPermissions, 
                                            &specialFile);
                if(recordLength < 0)
                    return recordLength;
                
                if(specialFile != NULL)
                
                {
                    free(*nextChild);
                    *nextChild = specialFile;
                }
            }
            
            childrenBytesRead += recordLength;
            
            nextChild = &((*nextChild)->next);
            *nextChild = NULL;
        }
        else
      
        {
            BYTE testByte;
            bk_off_t origPos;
            
            do
            {
                origPos = readSeekTell(volInfo);
                
                rc = readRead(volInfo, &testByte, 1);
                if(rc != 1)
                    return BKERROR_READ_GENERIC;
                
                if(testByte != 0)
                {
                    readSeekSet(volInfo, origPos, SEEK_SET);
                    break;
                }
                
                childrenBytesRead += 1;
                
            } while (childrenBytesRead + bytesRead < size);
        }
    }
    
    return bytesRead;
}

int skipDR(VolInfo* volInfo)
{
    BYTE dRLen;
    int rc;
    
    rc = read711(volInfo, &dRLen);
    if(rc <= 0)
        return BKERROR_READ_GENERIC;
    
    readSeekSet(volInfo, dRLen - 1, SEEK_CUR);
    
    return dRLen;
}


int haveNextRecordInSector(VolInfo* volInfo)
{
    bk_off_t origPos;
    BYTE testByte;
    int rc;
    
    origPos = readSeekTell(volInfo);
    
    rc = readRead(volInfo, &testByte, 1);
    if(rc != 1)
        return 0;
    
    readSeekSet(volInfo, origPos, SEEK_SET);
    
    return (testByte == 0) ? 0 : 1;
}


int dirDrFollows(VolInfo* volInfo)
{
    BYTE fileFlags;
    bk_off_t origPos;
    int rc;
    
    origPos = readSeekTell(volInfo);
    
    readSeekSet(volInfo, 25, SEEK_CUR);
    
    rc = read711(volInfo, &fileFlags);
    if(rc != 1)
        return BKERROR_READ_GENERIC;
    
    readSeekSet(volInfo, origPos, SEEK_SET);
    
    if((fileFlags >> 1 & 1) == 1)
        return 1;
    else
        return 0;
}



int readFileInfo(VolInfo* volInfo, BkFile* file, int filenameType, 
                 int keepPosixPermissions, BkFileBase** specialFile)
{
    int rc;
    BYTE recordLength;
    unsigned locExtent; /* block num where the file is */
    unsigned lenExtent; /* in bytes */
    unsigned char lenFileId9660; /* also len joliet fileid (bytes) */
    int lenSU; /* calculated as recordLength - 33 - lenFileId9660 */
    bk_off_t posBeforeName;
    char nameAsOnDisk[UCHAR_MAX + 1];
    
    
    file->pathAndName = NULL;
    
    if(volInfo->stopOperation)
        return BKERROR_OPER_CANCELED_BY_USER;
    
    maybeUpdateProgress(volInfo);
    
    *specialFile = NULL;
    
    rc = readRead(volInfo, &recordLength, 1);
    if(rc != 1)
        return BKERROR_READ_GENERIC;
    
    readSeekSet(volInfo, 1, SEEK_CUR);
    
    rc = read733(volInfo, &locExtent);
    if(rc != 8)
        return BKERROR_READ_GENERIC;
    
    rc = read733(volInfo, &lenExtent);
    if(rc != 8)
        return BKERROR_READ_GENERIC;
    
  
    if(volInfo->bootMediaType == BOOT_MEDIA_NO_EMULATION && 
       (signed )locExtent == volInfo->bootRecordOffset / NBYTES_LOGICAL_BLOCK)
    {
        volInfo->bootRecordSize = lenExtent;
        
        volInfo->bootRecordIsVisible = 1;
        volInfo->bootRecordOnImage = file;
    }
    
    readSeekSet(volInfo, 14, SEEK_CUR);
    
    rc = readRead(volInfo, &lenFileId9660, 1);
    if(rc != 1)
        return BKERROR_READ_GENERIC;
    
    lenSU = recordLength - 33 - lenFileId9660;
    if(lenFileId9660 % 2 == 0)
        lenSU -= 1;
    
   
    posBeforeName = readSeekTell(volInfo);
    
	LPBYTE ptr=(LPBYTE)nameAsOnDisk;
    rc = readRead(volInfo, ptr, lenFileId9660);
    if(rc != lenFileId9660)
        return BKERROR_READ_GENERIC;
    nameAsOnDisk[lenFileId9660] = '\0';
	nameAsOnDisk[lenFileId9660+1]= '\0'; 
    wchar_t nameInAscii[UCHAR_MAX + 1];
	wchar_t tmp_h,tmp_l,tmp;
    wchar_t *src,*dst;

	src = (wchar_t*)ptr;
	
	//birdy 2008/8/7 fix crash and can not play some iso file
	//if(volInfo->filenameTypes == FNTYPE_9660)
	{
		for(int i = 0 ; i < lenFileId9660/2+1; i++)
		{
			tmp_h = src[i]& 0xff;
			tmp_l = src[i]>>8;
			nameInAscii[2*i]=tmp_h;
			nameInAscii[2*i+1]=tmp_l;
		}		
	}
	/*else if ( (volInfo->filenameTypes & FNTYPE_JOLIET) || (volInfo->filenameTypes & FNTYPE_ROCKRIDGE) )
	{
		for( int i = 0 ; i <(lenFileId9660+1);i++)
		{
			tmp_h = src[i]&0xff;
			tmp_l = src[i]>>8;
			tmp = (tmp_h<<8 ) | tmp_l;
			nameInAscii[i]=tmp;
		}
	}
	else
	{
		for(int i = 0 ; i < lenFileId9660/2+1; i++)
		{
			tmp_h = src[i]& 0xff;
			tmp_l = src[i]>>8;
			nameInAscii[2*i]=tmp_h;
			nameInAscii[2*i+1]=tmp_l;
		}
	}*/
			
	src = nameInAscii;	
//	src = (wchar_t*)ptr;

	ptr = (LPBYTE) (BK_BASE_PTR(file)->name);
	dst = (wchar_t*)ptr;
	int len = wcslen(src);
	wcsncpy(dst,src,len+1);
//    strncpy(BK_BASE_PTR(file)->name, nameAsOnDisk, NCHARS_FILE_ID_MAX_STORE - 1);
    BK_BASE_PTR(file)->name[NCHARS_FILE_ID_MAX_STORE - 1] = '\0';
    
	src = dst;
#if 0
	ptr = (LPBYTE) (BK_BASE_PTR(file)->original9660name);
	dst = (wchar_t*) ptr; 
	len = wcslen(src);

	wcsncpy(dst,nameInAscii,len+1);

	len = wcslen(dst);
//    strncpy(BK_BASE_PTR(file)->original9660name, BK_BASE_PTR(file)->name, 14);
//    BK_BASE_PTR(file)->original9660name[14] = '\0';
#endif
    
    if(lenFileId9660 % 2 == 0)
        readSeekSet(volInfo, 1, SEEK_CUR);
    
    if(filenameType != FNTYPE_9660)
            readSeekSet(volInfo, posBeforeName, SEEK_SET);
    
    
    if(filenameType == FNTYPE_JOLIET)
    {
        char nameAsOnDisk[UCHAR_MAX];
        
        wchar_t nameInAscii[UCHAR_MAX];
        //int ucsCount, byteCount;
        
        if(lenFileId9660 % 2 != 0)
            return BKERROR_INVALID_UCS2;
        LPBYTE ptr=(LPBYTE)nameAsOnDisk;
        rc = readRead(volInfo, ptr, lenFileId9660);
        if(rc != lenFileId9660)
            return BKERROR_READ_GENERIC;
		ptr[lenFileId9660]='\0';
		ptr[lenFileId9660+1]='\0';

		wchar_t *src,*dst;
		
		src = (wchar_t*) ptr;
		for( int i = 0 ; i <(lenFileId9660+1);i++)
		{
			tmp_h = src[i]&0xff;
			tmp_l = src[i]>>8;
			tmp = (tmp_h<<8 ) | tmp_l;
			nameInAscii[i]=tmp;
		}
		
        src = nameInAscii; 
       // removeCrapFromFilename(nameInAscii, lenFileId9660 / 2);
        
		if(wcslen(src) > NCHARS_FILE_ID_MAX_STORE -1 )
       // if( strlen(nameInAscii) > NCHARS_FILE_ID_MAX_STORE - 1 )
            return BKERROR_MAX_NAME_LENGTH_EXCEEDED;
        
		ptr = (LPBYTE) (BK_BASE_PTR(file)->name);

		dst = (wchar_t*) ptr;
		int len = wcslen(src);
		wcsncpy(dst,src,len+1);
		len = wcslen(dst);
        //strncpy(BK_BASE_PTR(file)->name, nameInAscii, NCHARS_FILE_ID_MAX_STORE - 1);
        //BK_BASE_PTR(file)->name[NCHARS_FILE_ID_MAX_STORE - 1] = '\0';
        
        
        if(lenFileId9660 % 2 == 0)
            readSeekSet(volInfo, 1, SEEK_CUR);
    }
    else if(filenameType == FNTYPE_ROCKRIDGE)
    {
        
        readSeekSet(volInfo, lenFileId9660, SEEK_CUR);
        
        if(lenFileId9660 % 2 == 0)
            readSeekSet(volInfo, 1, SEEK_CUR);
        
        rc = readRockridgeFilename(volInfo, BK_BASE_PTR(file)->name, lenSU, 0);
        if(rc < 0)
            return rc;
    }
    else if(filenameType != FNTYPE_9660)
        return BKERROR_UNKNOWN_FILENAME_TYPE;
    
    if(keepPosixPermissions)
    {
        rc = readPosixFileMode(volInfo, &(BK_BASE_PTR(file)->posixFileMode), lenSU);
        if(rc < 0)
            return rc;
    }
    else
    {
        BK_BASE_PTR(file)->posixFileMode = volInfo->posixFileDefaults;
    }
    
    rc = readRockridgeSymlink(volInfo, (BkSymLink**)specialFile, lenSU);
    if(rc < 0)
        return rc;
    
    if(*specialFile != NULL)
    
    {
        strcpy((*specialFile)->name, BK_BASE_PTR(file)->name);
        strcpy((*specialFile)->original9660name, BK_BASE_PTR(file)->original9660name);
        
        (*specialFile)->posixFileMode = 0120777;
    }
    
    if(volInfo->scanForDuplicateFiles)
    {
        BkHardLink* newLink;
        
        rc = findInHardLinkTable(volInfo, locExtent * NBYTES_LOGICAL_BLOCK, NULL,
                                 lenExtent, 1, &newLink);
        if(rc < 0)
            return rc;
        
        if(newLink == NULL)
        
        {
            rc = addToHardLinkTable(volInfo, locExtent * NBYTES_LOGICAL_BLOCK, 
                                    NULL, lenExtent, 1, &newLink);
            if(rc < 0)
                return rc;
        }
        
        file->location = newLink;
    }
    
    readSeekSet(volInfo, lenSU, SEEK_CUR);
    
    file->onImage = 1;
    //file->position = locExtent * NBYTES_LOGICAL_BLOCK;
	//birdy
	file->position = ((bk_off_t)locExtent) * ((bk_off_t)NBYTES_LOGICAL_BLOCK);
    file->size = lenExtent;
    
    return recordLength;
}


void removeCrapFromFilename(char* filename, int length)
{
    int count;
    int stop = 0;
    
    for(count = 0; count < length && !stop; count++)
    {
        if(filename[count] == ';')
        {
            filename[count] = '\0';
            stop = 1;
        }
    }
    
    /* if did not get a ';' terminate string anyway */
    filename[count] = '\0';
}



int readRockridgeSymlink(VolInfo* volInfo, BkSymLink** dest, int lenSU)
{
    bk_off_t origPos;
    unsigned char* suFields;
    int rc;
    int count;
    int count2;
  
    suFields =(unsigned char*) malloc(lenSU);
    if(suFields == NULL)
        return BKERROR_OUT_OF_MEMORY;
    
    origPos = readSeekTell(volInfo);
    
    rc = readRead(volInfo, suFields, lenSU);
    if(rc != lenSU)
    {
        free(suFields);
        return BKERROR_READ_GENERIC;
    }
    
    count = 0;
    while(count < lenSU)
    {
        if(suFields[count] == 0)

            break;
        
        if(suFields[count] == 'S' && suFields[count + 1] == 'L')
        {
            size_t numCharsUsed; 
            
            *dest = (BkSymLink*)malloc(sizeof(BkSymLink));
            if(*dest == NULL)
                return BKERROR_OUT_OF_MEMORY;
            
            memset(*dest, 0, sizeof(BkSymLink));
            
            numCharsUsed = 0;
            (*dest)->target[0] = '\0';
            
            count2 = count + 5;
            while(count2 < count + suFields[count + 2])
            {
                if(suFields[count2] & 0x02)
                {
                    numCharsUsed += appendStringIfHaveRoom((*dest)->target, 
                                            ".", NCHARS_SYMLINK_TARGET_MAX - 1, 
                                            numCharsUsed, -1);
                }
                else if(suFields[count2] & 0x04)
                {
                    numCharsUsed += appendStringIfHaveRoom((*dest)->target, 
                                            "..", NCHARS_SYMLINK_TARGET_MAX - 1, 
                                            numCharsUsed, -1);
                }
                else if(suFields[count2] & 0x08)
                {
                    strcpy((*dest)->target, "/");
                    numCharsUsed = 1;
                }
                
                
                if( !(suFields[count2] & 0x3E) )
                {
                    numCharsUsed += appendStringIfHaveRoom((*dest)->target, 
                                            (char*)(suFields + count2 + 2), 
                                            NCHARS_SYMLINK_TARGET_MAX - 1, 
                                            numCharsUsed, suFields[count2 + 1]);
                }
                
                
                count2 += suFields[count2 + 1] + 2;
                
                if(count2 < count + suFields[count + 2])
                
                {
                    numCharsUsed += appendStringIfHaveRoom((*dest)->target, 
                                            "/", NCHARS_SYMLINK_TARGET_MAX - 1, 
                                            numCharsUsed, -1);
               }
            }
            
            
            break;
        }
        
        
        count += 1;
    }
    
	
    free(suFields);
    readSeekSet(volInfo, origPos, SEEK_SET);
    
    return 1;
}


size_t appendStringIfHaveRoom(char* dest, const char* src, size_t destMaxLen, 
                              size_t destCharsAlreadyUsed, int maxSrcLen)
{
    size_t srcLen;
    
    if(maxSrcLen == -1)
        srcLen = strlen(src);
    else
        srcLen = maxSrcLen;
    
    if(destCharsAlreadyUsed + srcLen > destMaxLen)
        return 0;
    
    strncat(dest, src, srcLen);
    
    return srcLen;
}