/***********************************************************************
 *    msdl.c:  main entry point, and download management functions
 ***********************************************************************
 * Copyright (C) 2007 metro <me_t_ro@yahoo.com>
 *
 * This file is part of msdl, media stream downloader
 * See README for program usage and information.
 * See COPYING for license information.
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
 *
 ***********************************************************************/



#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <ctype.h>
//#include <unistd.h>
#include "getopt_long.h"
#include <time.h>

#include<winsock.h>

#include "msdl.h"
#include "msdllib.h"
#include "network.h"
#include "mmst.h"
#include "mmsh.h"
#include "http.h"
#include "rtsp.h"

static struct target_t *new_target_t(char *name,int meta_f);
static void free_target_t(struct target_t *t);
static struct options_t *new_options_t(void);
void free_options_t(struct options_t *opt);
static struct download_opts_t *new_download_opts_t(void);
static void free_download_opts_t(struct download_opts_t *dlopts);
static struct dlresult_t *new_dlresult_t(void);
static void free_dlresult_t(struct dlresult_t *dlr);


static void print_usage(void);
static struct p_list *set_options(struct options_t *options,int argc,char **argv);
static int msdl(struct target_t *target,struct options_t *options,
		struct dlresult_t *result);
static int streaming_download(FILE *fp,struct URL_t *url,
			      struct download_opts_t *dlopts);

static int same_file(void *a,void *b);
static int is_metafile(char *name);
static struct p_list *get_url_list_from_file(char *filename);




/* GLOBAL VARIABLE */
struct options_t *options;


/*
 * msdl -- media stream downloader ---
 * main entry point
 */

#if 0
int main(int argc,char **argv)
{
    struct p_list *targets = NULL,*target;
    struct dlresult_t *result;
  
    struct p_list *p;
    int success;
    int failed;
  

    /* for displaying finished time */
    time_t start_time,fin_time,diffsec;

  //phillo
  	WORD wVersionRequested;
	WSADATA wsaData;
	
		wVersionRequested = MAKEWORD( 2, 0 );
	if (WSAStartup( wVersionRequested, &wsaData ) != 0) {
printf("WSAStartup() fail\n");
		exit(1);
		}
  
    start_time = time(NULL);

    options = new_options_t();
    result  = new_dlresult_t();

    targets = set_options(options,argc,argv);

    for(target = targets; target ; target = target->next) {
  
	msdl(target->p,options,result);

    }
  
    fin_time = time(NULL);
  
    success = p_list_num(result->success_list);
    failed = p_list_num(result->failed_list);

    diffsec = fin_time - start_time;

    display(MSDL_NOR,"FINISHED --%02d:%02d:%02d--\n",
	    diffsec/3600,(diffsec%3600)/60,diffsec % 60);


    /* don't display anything if only one file target */
    if(success > 1) {
	display(MSDL_NOR,"%d files downloaded\n",success);
    }
    else if(success == 0) {
	display(MSDL_NOR,"ALL download failed\n");
    }


    /* print failed log so that later user can copy-and-paste :) */
    if(failed > 0) {
	display(MSDL_NOR,"fialed: %d files\n",p_list_num(result->failed_list));
	for(p = result->failed_list ; p ; p = p->next) {
	    display(MSDL_NOR,"%s\n",p->p);
	}
    }
  
  
    free_options_t(options);
    free_dlresult_t(result);

      WSACleanup( );

    return 0;
}
#endif


/*
 * meta_f ... 1 --> always metafile
 *            0 --> don't know, judge from name
 *           -1 --> NEVER.
 */
static struct target_t *new_target_t(char *name,int meta_f)
{
    struct target_t *t = xmalloc(sizeof(struct target_t));
    t->target_name = strdup(name);
    t->metafile_f = meta_f;
    return t;
}



static void free_target_t(struct target_t *t)
{
    free(t->target_name);
    free(t);
}



struct options_t *new_options_t(void)
{
    struct options_t *opt = xmalloc(sizeof(struct options_t));
    memset(opt,0,sizeof(struct options_t));
    return opt;
}

void free_options_t(struct options_t *opt)
{
  
    if(opt->protocol)       free(opt->protocol);
    if(opt->local_filename) free(opt->local_filename);
    if(opt->username)       free(opt->username);
    if(opt->password)       free(opt->password);
    if(opt->http_proxy)     free(opt->http_proxy);

    free_p_list(opt->targets,(void (*)(void *))free_target_t);
  
    free(opt);
}


static struct download_opts_t *new_download_opts_t(void)
{
    struct download_opts_t *dlopts = xmalloc(sizeof(struct download_opts_t));
    memset(dlopts,0,sizeof(struct download_opts_t));
    return dlopts;
}



static void free_download_opts_t(struct download_opts_t *dlopts)
{
    free(dlopts);
}



static struct dlresult_t *new_dlresult_t(void)
{
    struct dlresult_t *dlr = xmalloc(sizeof(struct dlresult_t));
    memset(dlr,0,sizeof(struct dlresult_t));
    return dlr;
}



static void free_dlresult_t(struct dlresult_t *dlr)
{
    if(dlr->success_list) free_p_list(dlr->success_list,free);
    if(dlr->failed_list)  free_p_list(dlr->failed_list,free);
  
    free(dlr);
}




/*
 * take 2 urls, a and b. only use filename at the end of url.
 *           return value:   0   ... same file
 *                       : non 0 ... not same file
 */
static int same_file(void *a,void *b)
{
    char *s = (char *)a,*t = (char *)b;
    s = strrchr(s,'/') + 1;
    t = strrchr(t,'/') + 1;
  
    return strcmp(s,t);
}



/*
 * files with these extensions are regarded as metafiles.
 */
char *metafile_prefixes[] = {
    /* Windows Media metafiles */
    "asx",
    "wvx",
    "wax",
    /* Real Audio metafiles */
    "ram",
    "rpm",
    "smi",
};

char *metafile_starts[] = {
    "meta",
};



/*
 * judge if url 'name' is windows/real media metafile or not, from 
 * metafile_prefixes described above
 *            return value:   0  ... not
 *                            1  ... yes it is.
 */
static int is_metafile(char *name)
{
    int i;
    char *p;
  
    if((p = strrchr(name,'.'))) {
	p++;
	for(i = 0; i < sizeof(metafile_prefixes)/sizeof(char *) ; i++) {
	    if(!strcmp(metafile_prefixes[i],p)) return 1;
	}
    }
  
    if((p = strrchr(name,'/'))) {
	p++;
	for(i = 0; i < sizeof(metafile_starts)/sizeof(char *) ; i++) {
	    if(!strncmp(metafile_starts[i],p,strlen(metafile_starts[i]))) return 1;
	}
    }

    return 0;
}



/*
 * get string(url) list from file 'filename'
 * for not downloading same URLs again and again,
 * this function returns list via "uniq" filter.
 *
 *            return value:   string list ... success
 *                            NULL        ... no url in file
 */
static struct p_list *get_url_list_from_file(char *filename)
{
    FILE *localfp = NULL;
    struct p_list *url_list = NULL;
    int linesize = BUFSIZE_1K;
    char *line = xmalloc(linesize); /* 1024 byte */
    char *p;
    char *sep;
    int url_len;

    /* 
       try to download first url in the list
       --> maybe playlist contain same url twice
    */
    if((localfp = fopen(filename,"r")) == NULL) {
	display(MSDL_ERR,"cannot open file %s\n",filename);
	goto failed;
    }
    while(fgets(line,linesize,localfp)) {
	while((line[strlen(line) - 1] != '\n')) { /* didn't read the whole line */

	    linesize += BUFSIZE_1K;
	    line = xrealloc(line,linesize);

	    if(fgets(line + strlen(line),BUFSIZE_1K,localfp) == NULL) {
		/* error or EOF */
		break;
	    }
	}
    
	p = line;
	while((sep = strstr(p,"://"))) { /* contains url string */
	    /* protocol is alphabet */
	    for(sep--; sep >= p && isalpha(*sep) ; sep--); 
	    sep++;
	    for(url_len = 0; url_valid_char(sep[url_len]); url_len++);
	    sep[url_len] = '\0';
      
	    /* for not downloading same file again and again */
	    if(search_p_list(url_list,sep,same_file) == NULL) {
		p_list_append(&url_list,strdup(sep));
	    }
	    else {
		display(MSDL_VER,"file: <%s> is not uniq, ignore this\n",
			strrchr(sep,'/') + 1);
	    }
      
	    p = &(sep[url_len]) + 1;
	}
    }
  
    free(line);
    fclose(localfp);
    return url_list;

  failed:
    free(line);
    return NULL;
}



/*
 * force using 'protocol'. members in 'url' will be changed
 * to match the protocol specified.
 */
void force_protocol(char *protocol, struct URL_t *url)
{
    char *new_url;
    int new_url_len;

    if(protocol == NULL) return;
  
    new_url_len = strlen(protocol) + strlen("://") + strlen(url->file);
    new_url = xmalloc(new_url_len + 1);
    sprintf(new_url,"%s://%s",protocol,url->file);

    free(url->url);
    free(url->protocol);
    /* file and filepath are not malloc()ed, they points to part of url->url */
  

    url->url = new_url;
    url->protocol = strdup(protocol);
  
    url->file = strstr(new_url,"://") + 3;
    if(strchr(url->file,'/')) {
	url->filepath = strchr(url->file,'/');
    }
    else {
	url->filepath = url->file + strlen(url->file);
    }
  
    /* set protocol type */
    set_protocol_type(protocol,url);
  
}





/*
 * main download manager function 
 *
 * 'target' is url or local file which contains url(s) to download
 * 
 * return value:       number of files which successfully donwloaded
 *
 */
int msdl(struct target_t *target,struct options_t *options,struct dlresult_t *result)
{
    FILE *fp;
  
    struct p_list *target_list = NULL;        /* string                */
    struct p_list *p;                         /* url target list       */
    struct URL_t *url;                        /* for each target url   */
    struct download_opts_t *dlopts = NULL;    /* download options      */
  
    struct target_t *metafile = NULL;         /* save name of metafile */
  
    int downloaded_files_count = 0;
    int ret;
  
  
    target_list = NULL;
  
    /*
      set urls to download.
    */
    if(strstr(target->target_name,"://")) {
    
	/* we will get metafile from network */
	if(!(target->metafile_f == -1) && /* -1 is no-metafile flag */
	   (is_metafile(target->target_name) || target->metafile_f)) {
	    
	    /* set metafile name */
	    if(options->local_filename) { /* change name */
		metafile = new_target_t(options->local_filename,0);
	    }
	    else {
		metafile = new_target_t((strrchr(target->target_name,'/') + 1),0);
	    }
	}
	
	p_list_append(&target_list,strdup(target->target_name));
    }
    else { /* probablly local file (metafile or playlist)*/
    
	target_list = get_url_list_from_file(target->target_name);
    }
  
    if(target_list == NULL) {
	display(MSDL_ERR,"please specify url or file contains url\n");
	if(metafile) free_target_t(metafile);
	return 0;
    }
  
  
    /*
      set download options, such as bandwidth
      !!!! CAUTION !!!!
      these options are applied to all download in target list.
    */
    dlopts = new_download_opts_t();
    if(options->bandwidth) {
	dlopts->bandwidth = options->bandwidth;
    }
  
  
    /*
      try to download files from urls one by one
    */
    for(p = target_list ; p ; p = p->next) {
    
	char *new_file; /* file name to be created */

	/*
	  check if target is really a file
	*/
	if(!strcmp((strrchr(p->p,'/') + 1),"")) { /* no file */
	    continue;
	}
    
	url = new_url_t(p->p);
	if(url == NULL) continue;
    
	/*
	  add username and password, and proxy settings
	*/
	if(options->username) url->username = strdup(options->username);
	if(options->password) url->password = strdup(options->password);
	if(options->http_proxy) url->http_proxy = strdup(options->http_proxy);
    
    
	/*
	  create file which contains downloaded data.
	*/
	if(options->local_filename && strcmp(options->local_filename,"")) {
	    /* change name */
	    new_file = strdup(options->local_filename);
	    strcpy(options->local_filename,""); /* don't use local_filename again */
	}
	else {
	    new_file = strdup(strrchr(url->file,'/') + 1);
	}
    
	if((fp = fopen(new_file,"wb")) ==  NULL) {
	    display(MSDL_ERR,"cannot create file \"%s\"\n",new_file);
      
	    if(url)      free_url_t(url);
	    if(new_file) free(new_file);
	    continue;
	}
    
    
	/*
	  display download file name
	*/
	display(MSDL_NOR,"download [ %s ]\n",new_file);
	display(MSDL_VER,"%s\n",url->url);	

	/*
	  if url starts with "http", try MMSH first, and fallback on HTTP.
	  also, if MMST and HTTP_PROXY specified, use http.
	*/
	if(url->protocol_type == HTTP ||
	   (url->protocol_type == MMST && url->http_proxy)) {
	    url->protocol_type = MMSH;
	}
	/*
	  if url starts with "mms", try rtsp because it's the standard.
	  pure mmst protocol is obsolate.
	*/
	if(url->protocol_type == MMST) {
	    force_protocol("rtsp",url); /* change url */
	    url->protocol_type = RTSP_WMS;
	}
    
	/*
	  choose protocol if specified
	*/
	if(options->protocol) {
    
	    force_protocol(options->protocol,url);
      
	    if(url->protocol_type == UNKNOWN_PROTOCOL) {
		display(MSDL_ERR,"protocol [ %s ] not supported: fallback to http\n",
			options->protocol);
		url->protocol_type = HTTP;
	    }
	}
    

	/*
	  download file
	*/
	ret = streaming_download(fp,url,dlopts);
	if(ret > 0) {
	    downloaded_files_count++;
	    p_list_append(&result->success_list,strdup(url->url));
	}
	else { /* failed */
	    p_list_append(&result->failed_list,strdup(url->url));
	}
    
    
	/*
	  cleanup operations
	*/
	fclose(fp);
    
	if(url)      free_url_t(url);
	if(new_file) free(new_file);
    }

    free_p_list(target_list,free);

    /*
      download what's written inside the file if we...

      1. got file from network (see at the top of this function)
      2. it was metafile       (see at the top of this function)
      3. succeeeded downloading metafile itself
    */
    if(metafile && downloaded_files_count) {
	/*
	  this will not be infinite recursion because
	  'metafile' will not contain "://"
	*/
	downloaded_files_count += msdl(metafile,options,result);
    }
  
  
    if(dlopts)   free_download_opts_t(dlopts);
    if(metafile) free_target_t(metafile);

    return downloaded_files_count;
  
}





static void fill_buffer_with_ws(char *buffer,int buflen)
{
    if(strlen(buffer) < buflen) {
	memset(buffer + strlen(buffer),' ',buflen - strlen(buffer) - 1);
    }
    buffer[buflen - 1] = '\0';
}



/*
 * display download progress.
 */
void display_progress_info(size_t downloaded,size_t filesize)
{
    const int width = 48;
    //char buffer[width + 1]; /* 1 for '\0' */
	char buffer[48 + 1]; /* 1 for '\0' */
  
    if(filesize == 0) {
	_snprintf(buffer,width,"DL:  %lld B",(long long)downloaded);
    }
    else {
	_snprintf(buffer,width,"DL:  %lld/%lld B -- %3d %%",
		 (long long)downloaded,
		 (long long)filesize,
		 (int)(((double)downloaded/(double)filesize) * 100));
    }
  
    fill_buffer_with_ws(buffer,width + 1);
  
    display(MSDL_NOR,"%s",buffer);
  
}



/*
 * display download speed.
 */
void display_dl_speed(int speed)
{
    const int width = 12;
    //char buffer[width + 1];
	char buffer[12 + 1];

    if(speed > 1000000000) {
	_snprintf(buffer,width,"%.1f gB/s",(double)speed / 1000000000);
    }
    else if(speed > 1000000) {
	_snprintf(buffer,width,"%.1f mB/s",(double)speed / 1000000);
    }
    else if(speed > 1000) {
	_snprintf(buffer,width,"%.1f kB/s",(double)speed / 1000);
    }
    else {
	_snprintf(buffer,width,"%.1d B/s",speed);
    }

    fill_buffer_with_ws(buffer,width + 1); /* fill str with ' ' */
  
    display(MSDL_NOR,"%s\r",buffer); /* set cursor to beginning of line */
}



void display_remain_time(size_t remain_size,int speed)
{
    const int width = 12;
    //char buffer[width + 1];
	char buffer[12 + 1];
    size_t etl;

    /* remain size / speed is estimated second left */
    if(speed) {
	etl = remain_size / speed;
    }
    else {
	etl = 100 * 3600;
    }
  
    if((etl / 3600) > 100) {
	_snprintf(buffer,width,"--:--:--");
    }
    else {
	_snprintf(buffer,width,"%02d:%02d:%02d",
		 (int)(etl / 3600),(int)((etl % 3600 / 60)),(int)(etl % 60));
    }
  
    fill_buffer_with_ws(buffer,width + 1);
  
    display(MSDL_NOR,"%s",buffer);
}






/*
 * print protocol name to screen.
 */
void display_protocol(struct stream_t *stream)
{
    display(MSDL_VER,"download protocol: ");
    switch (stream->stream_ctrl->protocol) {
    case MMST:
	display(MSDL_VER,"mmst");
	break;
    case MMSH:
	display(MSDL_VER,"mmsh");
	break;
    case RTSP_REAL:
	display(MSDL_VER,"rtsp - real");
	break;
    case RTSP_WMS:
	display(MSDL_VER,"rtsp - wms");
	break;
    case HTTP:
	display(MSDL_VER,"http");
	break;
    default:
	display(MSDL_ERR," :) ...bug");
	break;
    }
  
    display(MSDL_VER,"\n\n");
}




/*
 * main downloading function.
 * select DLing function according to url->protocol.
 * download 'url' to 'localfile'
 *
 *      return value;   1 ... succeeded
 *                      0 ... failed
 */
int streaming_download(FILE *localfile,struct URL_t *url,
		       struct download_opts_t *dlopts)
{
    int ret;
    struct stream_t *stream = NULL;
    unsigned char *buffer = NULL;
    int initial_port = url->port; /* port specified in URL, foobar:8080 etc..*/
    int retries;

    size_t size_written = 0;   /* size written to file */

    /* for displaying dl speed */
    time_t cur_time,last_time;
    size_t last_size_written;
    int speed;

    /*
      download header.
    */
    for(retries = 2 ; retries > 0 ; retries--) {
    
	switch(url->protocol_type) {
	case MMST:
	    stream = mmst_streaming_init(url);
	    break;
	case HTTP:
	    stream = http_streaming_init(url);
	    break;
	case MMSH:
	    stream = mmsh_streaming_init(url);
	    break;
	case RTSP:
	case RTSP_WMS:
	case RTSP_REAL:
	    stream = rtsp_streaming_init(url);
	    break;
      
	default:
	    display(MSDL_ERR,"protocol [ %s:// ] not supported\n",url->protocol);
	    goto failed;
	}
    
	/*
	  set options
	*/
	stream->stream_ctrl->bandwidth = dlopts->bandwidth; /* default 0 */
    
	ret = stream->start(stream);
    
	if(ret > 0) {
	    break;
	}
	else if(stream->stream_ctrl->status == streaming_other_protocol) {
	    url->protocol_type = stream->stream_ctrl->retry_protocol;
	    url->port = initial_port;
      
	    stream->close(stream); /* close stream for this protocol */
	    stream = NULL;
	    continue;
	}
	else {
	    display(MSDL_ERR,"cannot establish stream\n");
	    goto failed;
	}
    }
  
    if(retries <= 0) {
	display(MSDL_ERR,"no more retry chance\n");
	goto failed;
    }
  
    display_protocol(stream);
  
    /*
      download media.
    */
    buffer = (unsigned char *)xmalloc(BUFSIZE_4_DL);  

    last_time = time(NULL);
    last_size_written = 0;
    speed = 0;

    for(;;) {
    
	if(stream_check_data(stream,STREAM_CHECK_DATA_TIME) <= 0) {
	    /* some error or data didn't come within few(default 5) seconds*/
	    display_progress_info(size_written,stream->stream_ctrl->file_size);
	    display_dl_speed(0);
	}
    
    
	ret = stream->read(stream,buffer,BUFSIZE_4_DL);

	/* 
	   write received packet directory to file.
	   everything, such as padding has been done alrady, just write it.
	*/
	if(ret > 0) { /* success */
      
	    if(stream->stream_ctrl->status == streaming_rewind) {
		display(MSDL_VER,"rewind file!!\n");
		rewind(localfile);
		stream->stream_ctrl->packet_count = 0;
		size_written = 0;
		last_size_written = 0;
		speed = 0;
		stream->stream_ctrl->status = streaming_downloading;
	    }
      
	    size_written += ret;

	    display_progress_info(size_written,stream->stream_ctrl->file_size);
      
	    cur_time = time(NULL);
	    if(cur_time - last_time >= 1) { /* 1 is display update second */
		/* update speed */
		speed = (size_written - last_size_written) / (cur_time - last_time);
		/* update last_* info */
		last_time = cur_time;
		last_size_written = size_written;
	    }
	    display_dl_speed(speed);
	    
	    display(MSDL_DBG,"\npacket count: %d\n",stream->stream_ctrl->packet_count);
      
	    fwrite(buffer,sizeof(unsigned char),ret,localfile);
	
	}
	else if(stream->stream_ctrl->status == streaming_finished) {

	    /*
	      show 100%, because 99% sucks ...
	      have to do this because stream->stream_ctrl->file_size
	      may show wrogn file size.
	    */
      
	    display_progress_info(size_written,size_written);
	    display(MSDL_NOR,"\nfinished!!\n");
	    break;
	}
	/*
	  return value is 0 if failure
	*/
	else { /* failure */
	    display(MSDL_ERR,"!!! aborted by error !!!\n");
	    goto failed;
	}
    }  

    display(MSDL_NOR,"\n");
    if(stream) stream->close(stream);
    if(buffer) free(buffer);
    return 1;


  failed:
  
    display(MSDL_NOR,"\n");
    if(stream) stream->close(stream);
    if(buffer) free(buffer);
    return 0;
}



char *usage =
    "media stream downloader\n"
    "Usage: msdl [options] targets\n"
    "  -o, --output <localfile>    specify output file name.\n"
    "  -v, --verbose               show verbose messages.\n"
    "  -q, --quiet                 do quietly. no output.\n"
    "  -b, --bandwidth <bandwidth> set bandwidth.\n"
    "  -m, --metafile <url>        treat url as metafile.\n"
    "  -n, --no-metafile <url>     DO NOT treat url as metafile\n"
    "  -h, --help                  display this help.\n"
    "  -p, --protocol <protocol>   specify download protocol.\n"
    "      --username <username>   user name for basic authentication.\n"
    "      --password <password>   password for basic authentication.\n"
    "      --no-proxy              DO NOT use proxy, even if HTTP_PROXY set.\n"
    "      --debug                 show debug message (super verbose).\n"
    "Supported protocols: mms(mmst) mmsh rtsp http\n"
    "To use proxy for mms,mmsh, and http, set HTTP_PROXY variable\n";

/*
 * print usage and exit
 */
static void print_usage(void)
{
    display(MSDL_ERR,"%s",usage);
}


/*
 * read arguments. returns target url
 */
static struct p_list *set_options(struct options_t *options,int argc,char **argv)
{
  
    int ch;
    int option_index = 0;
    int target_count = 0; /* argument count */
    struct p_list *targets = NULL;
    struct target_t *t = NULL;

	//birdy add
	const char * optarg;


    struct option long_options[] = {
	{"output",1,0,'o'},     /* equal to '-o' */
	{"protocol",1,0,'p'},   /*          '-p' */
	{"quiet",0,0,'q'},      /*          '-q' */
	{"bandwidth",1,0,'b'},  /*          '-b' */
	{"metafile",1,0,'m'},   /*          '-m' */
	{"no-metafile",1,0,'n'},/*          '-n' */
	{"help",0,0,'h'},       /*          '-h' */
	{"verbose",0,0,'v'},    /*          '-v' */
	{"username",1,0,0},
	{"password",1,0,0},
	{"no-proxy",0,0,0},     /* turn off proxy, no matter HTTP_PROXY set */
	{"debug",0,0,0},        /* debug messages, super verbose mode */
	{NULL,0,0,0}
    };
    char *env;
  
    /*
      get environment variable
    */
    if((env = getenv("HTTP_PROXY"))) {
	options->http_proxy = strdup(env);
    }
  
  
    /*
      set command line options
    */
    while(1) {

	ch = getopt_long(argc,argv,"o:p:b:m:n:hvq",long_options,&option_index);
    
	if(ch == -1) /* end of options */
	    break;
    
	switch(ch) {
      
	case 0:   /* long options, such as --help */

	    if(!strcmp(long_options[option_index].name,"username")) {
		options->username = strdup(optarg);
	    }
	    else if(!strcmp(long_options[option_index].name,"password")) {
		options->password = strdup(optarg);
	    }
	    else if(!strcmp(long_options[option_index].name,"no-proxy")) {
		if(options->http_proxy) free(options->http_proxy);
		options->http_proxy = NULL;
	    }
	    else if(!strcmp(long_options[option_index].name,"debug")) {
		options->debug_f = 1;
	    }
      
	    break;
      
	case 'p': /* specify protocol */
	    options->protocol = strdup(optarg);
	    break;
      
	case 'o': /* output file name specification */
	    options->local_filename = strdup(optarg);
	    break;
      
	case 'b':
	    options->bandwidth = atoi(optarg);
	    break;
    
	case 'm':
	    t = new_target_t(optarg,1);
	    p_list_append(&targets,t);
	    target_count++;
	    break;
	    
	case 'n':
	    t = new_target_t(optarg,-1);
	    p_list_append(&targets,t);
	    target_count++;
	    break;
	    
	case 'h': /* help */
	    print_usage();
	    exit(1);
	    break;

	case 'v': /* verbose */
	    options->verbose_f = 1;
	    break;

	case 'q': /* quiet */
	    options->quiet_f = 1;
	    break;
      
	default:
	    display(MSDL_ERR,"cannot recognize option '%c'\n",ch);
	    print_usage();
	    exit(1);
	}
    }
  
    if(optind < argc) {
    
	while(optind < argc) {
	    t = new_target_t(argv[optind++],0);
	    p_list_append(&targets,t);
	    target_count++;
	}
    }
  
    options->targets = targets;
    
  
    if(target_count == 0) {
	display(MSDL_ERR,"no target\n");
	print_usage();
	exit(1);
    }
  
    /*
      show warnings
    */
    if(options->local_filename && target_count > 1) {
	display(MSDL_ERR,
		"warning: output filename specified but trying to\n"
		"         download more than 1 files. name which\n"
		"         you specfified will be applied to only\n"
		"         first file downloaded\n");
    }
  
    if(options->protocol && target_count > 1) {
	display(MSDL_ERR,
		"warning: protocol specified but trying to \n"
		"         download more than 1 files. keep \n"
		"         in mind that protocol you specified\n"
		"         will be applied to all targets, thus\n"
		"         some files can not be downloaded.\n");
    }

    if(options->bandwidth) {
	display(MSDL_ERR,
		"warning: you have set bandwidth to %d\n"
		"         for this setting, you might not receive\n"
		"         best quality stream.\n",options->bandwidth);
    }
  

    return targets;
}


