/***********************************************************************
 *    msdllib.h:  utility functions
 ***********************************************************************
 * Copyright (C) 2007 metro <me_t_ro@yahoo.com>
 *
 * This file is part of msdl, media stream downloader
 *
 * utility functions and screen display functions. 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
 *
 ***********************************************************************/




#ifndef __MSDLLIB_H__
#define __MSDLLIB_H__



/*
 * display flags (bitfield)
 */
enum {
    MSDL_NOR = 1,       /* stdout  */
    MSDL_VER = 1 << 1,  /* verbose */
    MSDL_DBG = 1 << 2,  /* debug   */
    MSDL_ERR = 1 << 3,  /* stderr  */
} display_flags;



/*
 * functions to handle bitorder
 * 
 * bswap_* : swap upper and lower bits
 * put*_*  : put number to p in specified bitorder
 * get*_*  : get number from p in specified bitorder
 */
#define inline

static inline unsigned int bswap_16(unsigned int x)
{
    return (x>>8) | (x<<8);
}

static inline unsigned long bswap_32(unsigned long x)
{
    x= ((x<<8)&0xFF00FF00) | ((x>>8)&0x00FF00FF);
    return (x>>16) | (x<<16);
}

static inline size_t bswap_64(size_t x)
{
    x= ((x<< 8)&0xFF00FF00FF00FF00ULL) | ((x>> 8)&0x00FF00FF00FF00FFULL);
    x= ((x<<16)&0xFFFF0000FFFF0000ULL) | ((x>>16)&0x0000FFFF0000FFFFULL);
    return (x>>32) | (x<<32);
}

static inline void put32_be(void *p,unsigned long x)
{
    *((unsigned char *)p    ) = x >> 24 & 0xff;
    *((unsigned char *)p + 1) = x >> 16 & 0xff;
    *((unsigned char *)p + 2) = x >>  8 & 0xff;
    *((unsigned char *)p + 3) = x       & 0xff;
}

static inline void put32_le(void *p,unsigned long x)
{
    *((unsigned char *)p    ) = x       & 0xff;
    *((unsigned char *)p + 1) = x >>  8 & 0xff;
    *((unsigned char *)p + 2) = x >> 16 & 0xff;
    *((unsigned char *)p + 3) = x >> 24 & 0xff;
}

static inline void put16_be(void *p,unsigned int x)
{
    *((unsigned char *)p    ) = x >>8   & 0xff;
    *((unsigned char *)p + 1) = x       & 0xff;
}

static inline void put16_le(void *p,unsigned int x)
{
    *((unsigned char *)p    ) = x       & 0xff;
    *((unsigned char *)p + 1) = x >>8   & 0xff;
}

static inline void put8(void *p,unsigned char x)
{
    *(unsigned char *)p       = x       & 0xff;
}

static inline unsigned long get32_be(void *p)
{
    return *(unsigned char *)p << 24 | *((unsigned char *)p + 1) << 16 |
	*((unsigned char *)p + 2) << 8 | *((unsigned char *)p + 3);
}

static inline unsigned long get32_le(void *p)
{
    return *(unsigned char *)p | *((unsigned char *)p + 1) << 8 |
	*((unsigned char *)p + 2) << 16 | *((unsigned char *)p + 3) << 24;
}

static inline unsigned int get16_be(void *p)
{
    return *(unsigned char *)p << 8 | *((unsigned char *)p + 1);
}

static inline unsigned int get16_le(void *p)
{
    return *(unsigned char *)p | *((unsigned char *)p + 1) << 8;
}

static inline unsigned char get8(void *p)
{
    return *(unsigned char *)p;
}


/*
 * Big/Little Endian to Machine Endian
 */
#ifdef WORDS_BIGENDIAN
#define be2me_16(x) (x)
#define be2me_32(x) (x)
#define be2me_64(x) (x)
#define le2me_16(x) bswap_16(x)
#define le2me_32(x) bswap_32(x)
#define le2me_64(x) bswap_64(x)
#else
#define be2me_16(x) bswap_16(x)
#define be2me_32(x) bswap_32(x)
#define be2me_64(x) bswap_64(x)
#define le2me_16(x) (x)
#define le2me_32(x) (x)
#define le2me_64(x) (x)
#endif


struct p_list {
    void *p;
    struct p_list *next;
};


#define free(p) do {free((p));(p) = NULL;} while(0)
void *xmalloc(const size_t size);
void *xrealloc(void *p,const size_t size);
void display(const int flag,const char *fmt, ...);
void fatal_error(const char *fmt, ...);

struct p_list *p_list_append(struct p_list **list,void *p);
struct p_list *search_p_list(struct p_list *list,void *p,
			     int (*comp)(void *,void *));
int p_list_num(struct p_list *list);
struct p_list *new_p_list(void *p);
void free_p_list(struct p_list *list,void (*free_er)(void *));
  

#endif /* __MSDLLIB_H__ */
