/***********************************************************************
 *    real.c : for downloading from real server using rtsp
 ***********************************************************************
 * Copyright (C) 2007 metro <me_t_ro@yahoo.com>
 *
 * This file is part of msdl, media stream downloader
 *
 * This file is based on rtsp (real) implementation of 
 * mplayer and xine, and includes code from both projects.
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
 *
 ***********************************************************************/



#include <stdio.h>
#include <stdlib.h>
#include <string.h>
//#include <unistd.h>

//#include <inttypes.h>
#include <sys/types.h>
#include<winsock2.h>

#include "msdl.h"
#include "msdllib.h"
#include "network.h"
#include "rtsp.h"
#include "real.h"
#include "rmff.h"
#include "sdpreal.h"
#include "asmrule.h"


enum {
    XOR_TABLE_LEN = 37,
};



/*
 * This data is taken from xine-lib program.
 * used to calculate clientchallenge2.
 */
static const unsigned char xor_table[] = {
    0x05, 0x18, 0x74, 0xd0, 0x0d, 0x09, 0x02, 0x53,
    0xc0, 0x01, 0x05, 0x05, 0x67, 0x03, 0x19, 0x70,
    0x08, 0x27, 0x66, 0x10, 0x10, 0x72, 0x08, 0x09,
    0x63, 0x11, 0x03, 0x71, 0x08, 0x08, 0x70, 0x02,
    0x10, 0x57, 0x05, 0x18, 0x54, 0x00, 0x00, 0x00 };


/*
 * This function is taken from xine-lib program.
 * used to calculate clientchallenge2.
 */
static void hash(char *field, char *param)
{

    unsigned long a, b, c, d;
 

    /* fill variables */
    a = get32_le(field);
    b = get32_le(field+4);
    c = get32_le(field+8);
    d = get32_le(field+12);
  
  
    a = ((b & c) | (~b & d)) + get32_le((param+0x00)) + a - 0x28955B88;
    a = ((a << 0x07) | (a >> 0x19)) + b;
    d = ((a & b) | (~a & c)) + get32_le((param+0x04)) + d - 0x173848AA;
    d = ((d << 0x0c) | (d >> 0x14)) + a;
    c = ((d & a) | (~d & b)) + get32_le((param+0x08)) + c + 0x242070DB;
    c = ((c << 0x11) | (c >> 0x0f)) + d;
    b = ((c & d) | (~c & a)) + get32_le((param+0x0c)) + b - 0x3E423112;
    b = ((b << 0x16) | (b >> 0x0a)) + c;
    a = ((b & c) | (~b & d)) + get32_le((param+0x10)) + a - 0x0A83F051;
    a = ((a << 0x07) | (a >> 0x19)) + b;
    d = ((a & b) | (~a & c)) + get32_le((param+0x14)) + d + 0x4787C62A;
    d = ((d << 0x0c) | (d >> 0x14)) + a;
    c = ((d & a) | (~d & b)) + get32_le((param+0x18)) + c - 0x57CFB9ED;
    c = ((c << 0x11) | (c >> 0x0f)) + d;
    b = ((c & d) | (~c & a)) + get32_le((param+0x1c)) + b - 0x02B96AFF;
    b = ((b << 0x16) | (b >> 0x0a)) + c;
    a = ((b & c) | (~b & d)) + get32_le((param+0x20)) + a + 0x698098D8;
    a = ((a << 0x07) | (a >> 0x19)) + b;
    d = ((a & b) | (~a & c)) + get32_le((param+0x24)) + d - 0x74BB0851;
    d = ((d << 0x0c) | (d >> 0x14)) + a;
    c = ((d & a) | (~d & b)) + get32_le((param+0x28)) + c - 0x0000A44F;
    c = ((c << 0x11) | (c >> 0x0f)) + d;
    b = ((c & d) | (~c & a)) + get32_le((param+0x2C)) + b - 0x76A32842;
    b = ((b << 0x16) | (b >> 0x0a)) + c;
    a = ((b & c) | (~b & d)) + get32_le((param+0x30)) + a + 0x6B901122;
    a = ((a << 0x07) | (a >> 0x19)) + b;
    d = ((a & b) | (~a & c)) + get32_le((param+0x34)) + d - 0x02678E6D;
    d = ((d << 0x0c) | (d >> 0x14)) + a;
    c = ((d & a) | (~d & b)) + get32_le((param+0x38)) + c - 0x5986BC72;
    c = ((c << 0x11) | (c >> 0x0f)) + d;
    b = ((c & d) | (~c & a)) + get32_le((param+0x3c)) + b + 0x49B40821;
    b = ((b << 0x16) | (b >> 0x0a)) + c;
  
    a = ((b & d) | (~d & c)) + get32_le((param+0x04)) + a - 0x09E1DA9E;
    a = ((a << 0x05) | (a >> 0x1b)) + b;
    d = ((a & c) | (~c & b)) + get32_le((param+0x18)) + d - 0x3FBF4CC0;
    d = ((d << 0x09) | (d >> 0x17)) + a;
    c = ((d & b) | (~b & a)) + get32_le((param+0x2c)) + c + 0x265E5A51;
    c = ((c << 0x0e) | (c >> 0x12)) + d;
    b = ((c & a) | (~a & d)) + get32_le((param+0x00)) + b - 0x16493856;
    b = ((b << 0x14) | (b >> 0x0c)) + c;
    a = ((b & d) | (~d & c)) + get32_le((param+0x14)) + a - 0x29D0EFA3;
    a = ((a << 0x05) | (a >> 0x1b)) + b;
    d = ((a & c) | (~c & b)) + get32_le((param+0x28)) + d + 0x02441453;
    d = ((d << 0x09) | (d >> 0x17)) + a;
    c = ((d & b) | (~b & a)) + get32_le((param+0x3c)) + c - 0x275E197F;
    c = ((c << 0x0e) | (c >> 0x12)) + d;
    b = ((c & a) | (~a & d)) + get32_le((param+0x10)) + b - 0x182C0438;
    b = ((b << 0x14) | (b >> 0x0c)) + c;
    a = ((b & d) | (~d & c)) + get32_le((param+0x24)) + a + 0x21E1CDE6;
    a = ((a << 0x05) | (a >> 0x1b)) + b;
    d = ((a & c) | (~c & b)) + get32_le((param+0x38)) + d - 0x3CC8F82A;
    d = ((d << 0x09) | (d >> 0x17)) + a;
    c = ((d & b) | (~b & a)) + get32_le((param+0x0c)) + c - 0x0B2AF279;
    c = ((c << 0x0e) | (c >> 0x12)) + d;
    b = ((c & a) | (~a & d)) + get32_le((param+0x20)) + b + 0x455A14ED;
    b = ((b << 0x14) | (b >> 0x0c)) + c;
    a = ((b & d) | (~d & c)) + get32_le((param+0x34)) + a - 0x561C16FB;
    a = ((a << 0x05) | (a >> 0x1b)) + b;
    d = ((a & c) | (~c & b)) + get32_le((param+0x08)) + d - 0x03105C08;
    d = ((d << 0x09) | (d >> 0x17)) + a;
    c = ((d & b) | (~b & a)) + get32_le((param+0x1c)) + c + 0x676F02D9;
    c = ((c << 0x0e) | (c >> 0x12)) + d;
    b = ((c & a) | (~a & d)) + get32_le((param+0x30)) + b - 0x72D5B376;
    b = ((b << 0x14) | (b >> 0x0c)) + c;
  
    a = (b ^ c ^ d) + get32_le((param+0x14)) + a - 0x0005C6BE;
    a = ((a << 0x04) | (a >> 0x1c)) + b;
    d = (a ^ b ^ c) + get32_le((param+0x20)) + d - 0x788E097F;
    d = ((d << 0x0b) | (d >> 0x15)) + a;
    c = (d ^ a ^ b) + get32_le((param+0x2c)) + c + 0x6D9D6122;
    c = ((c << 0x10) | (c >> 0x10)) + d;
    b = (c ^ d ^ a) + get32_le((param+0x38)) + b - 0x021AC7F4;
    b = ((b << 0x17) | (b >> 0x09)) + c;
    a = (b ^ c ^ d) + get32_le((param+0x04)) + a - 0x5B4115BC;
    a = ((a << 0x04) | (a >> 0x1c)) + b;
    d = (a ^ b ^ c) + get32_le((param+0x10)) + d + 0x4BDECFA9;
    d = ((d << 0x0b) | (d >> 0x15)) + a;
    c = (d ^ a ^ b) + get32_le((param+0x1c)) + c - 0x0944B4A0;
    c = ((c << 0x10) | (c >> 0x10)) + d;
    b = (c ^ d ^ a) + get32_le((param+0x28)) + b - 0x41404390;
    b = ((b << 0x17) | (b >> 0x09)) + c;
    a = (b ^ c ^ d) + get32_le((param+0x34)) + a + 0x289B7EC6;
    a = ((a << 0x04) | (a >> 0x1c)) + b;
    d = (a ^ b ^ c) + get32_le((param+0x00)) + d - 0x155ED806;
    d = ((d << 0x0b) | (d >> 0x15)) + a;
    c = (d ^ a ^ b) + get32_le((param+0x0c)) + c - 0x2B10CF7B;
    c = ((c << 0x10) | (c >> 0x10)) + d;
    b = (c ^ d ^ a) + get32_le((param+0x18)) + b + 0x04881D05;
    b = ((b << 0x17) | (b >> 0x09)) + c;
    a = (b ^ c ^ d) + get32_le((param+0x24)) + a - 0x262B2FC7;
    a = ((a << 0x04) | (a >> 0x1c)) + b;
    d = (a ^ b ^ c) + get32_le((param+0x30)) + d - 0x1924661B;
    d = ((d << 0x0b) | (d >> 0x15)) + a;
    c = (d ^ a ^ b) + get32_le((param+0x3c)) + c + 0x1fa27cf8;
    c = ((c << 0x10) | (c >> 0x10)) + d;
    b = (c ^ d ^ a) + get32_le((param+0x08)) + b - 0x3B53A99B;
    b = ((b << 0x17) | (b >> 0x09)) + c;
  
    a = ((~d | b) ^ c)  + get32_le((param+0x00)) + a - 0x0BD6DDBC;
    a = ((a << 0x06) | (a >> 0x1a)) + b; 
    d = ((~c | a) ^ b)  + get32_le((param+0x1c)) + d + 0x432AFF97;
    d = ((d << 0x0a) | (d >> 0x16)) + a; 
    c = ((~b | d) ^ a)  + get32_le((param+0x38)) + c - 0x546BDC59;
    c = ((c << 0x0f) | (c >> 0x11)) + d; 
    b = ((~a | c) ^ d)  + get32_le((param+0x14)) + b - 0x036C5FC7;
    b = ((b << 0x15) | (b >> 0x0b)) + c; 
    a = ((~d | b) ^ c)  + get32_le((param+0x30)) + a + 0x655B59C3;
    a = ((a << 0x06) | (a >> 0x1a)) + b; 
    d = ((~c | a) ^ b)  + get32_le((param+0x0C)) + d - 0x70F3336E;
    d = ((d << 0x0a) | (d >> 0x16)) + a; 
    c = ((~b | d) ^ a)  + get32_le((param+0x28)) + c - 0x00100B83;
    c = ((c << 0x0f) | (c >> 0x11)) + d; 
    b = ((~a | c) ^ d)  + get32_le((param+0x04)) + b - 0x7A7BA22F;
    b = ((b << 0x15) | (b >> 0x0b)) + c; 
    a = ((~d | b) ^ c)  + get32_le((param+0x20)) + a + 0x6FA87E4F;
    a = ((a << 0x06) | (a >> 0x1a)) + b; 
    d = ((~c | a) ^ b)  + get32_le((param+0x3c)) + d - 0x01D31920;
    d = ((d << 0x0a) | (d >> 0x16)) + a; 
    c = ((~b | d) ^ a)  + get32_le((param+0x18)) + c - 0x5CFEBCEC;
    c = ((c << 0x0f) | (c >> 0x11)) + d; 
    b = ((~a | c) ^ d)  + get32_le((param+0x34)) + b + 0x4E0811A1;
    b = ((b << 0x15) | (b >> 0x0b)) + c; 
    a = ((~d | b) ^ c)  + get32_le((param+0x10)) + a - 0x08AC817E;
    a = ((a << 0x06) | (a >> 0x1a)) + b; 
    d = ((~c | a) ^ b)  + get32_le((param+0x2c)) + d - 0x42C50DCB;
    d = ((d << 0x0a) | (d >> 0x16)) + a; 
    c = ((~b | d) ^ a)  + get32_le((param+0x08)) + c + 0x2AD7D2BB;
    c = ((c << 0x0f) | (c >> 0x11)) + d; 
    b = ((~a | c) ^ d)  + get32_le((param+0x24)) + b - 0x14792C6F;
    b = ((b << 0x15) | (b >> 0x0b)) + c; 
  
  
    a += get32_le(field);
    b += get32_le(field+4);
    c += get32_le(field+8);
    d += get32_le(field+12);

    put32_le(field,    a);
    put32_le(field+4,  b);
    put32_le(field+8,  c);
    put32_le(field+12, d);
}



/*
 * This function is taken from xine-lib program.
 * used to calculate clientchallenge2.
 */
static void call_hash (char *key, char *challenge, unsigned int len) {

    char *ptr1, *ptr2;
    unsigned long a, b, c, d, tmp;
  
    ptr1=(key+16);
    ptr2=(key+20);
  
    a = get32_le(ptr1);
    b = (a >> 3) & 0x3f;
    a += len * 8;
    put32_le(ptr1, a);
  
    if (a < (len << 3))
	{
	    ptr2 += 4;
	}

    tmp = get32_le(ptr2) + (len >> 0x1d);
    put32_le(ptr2, tmp);
    a = 64 - b;
    c = 0;  
    if (a <= len)
	{
    
	    memcpy(key+b+24, challenge, a);
	    hash(key, key+24);
	    c = a;
	    d = c + 0x3f;
    
	    while ( d < len ) {
		hash(key, challenge+d-0x3f);
		d += 64;
		c += 64;
	    }
	    b = 0;
	}
  
    memcpy(key+b+24, challenge+c, len-c);
}



/*
 * This function is taken from xine-lib program.
 * used to calculate clientchallenge2.
 */
static void calc_response (char *result, char *field) {
  
    char buf1[128];
    char buf2[128];
    unsigned int i;
  
    memset(buf1, 0, 64);
    *buf1 = 128;
  
    memcpy(buf2, field+16, 8);
  
    i = (get32_le((buf2)) >> 3) & 0x3f;
 
    if (i < 56) {
	i = 56 - i;
    } else {
	i = 120 - i;
    }

    call_hash(field, buf1, i);
    call_hash(field, buf2, 8);

    memcpy(result, field, 16);

}



/*
 * This function is taken from xine-lib program.
 * used to calculate clientchallenge2.
 */
static void calc_response_string (char *result, char *challenge) {
 
    char field[128];
    char zres[20];
    int  i;
  
    /* initialize our field */
    put32_be(field,      0x01234567);
    put32_be(field + 4,  0x89abcdef);
    put32_be(field + 8,  0xfedcba98);
    put32_be(field + 12, 0x76543210);
    put32_be(field + 16, 0x00000000);
    put32_be(field + 20, 0x00000000);
  
    /* calculate response */
    call_hash(field, challenge, 64);
    calc_response(zres,field);
 
    /* convert zres to ascii string */
    for (i=0; i<16; i++ ) {
	char a, b;
    
	a = (zres[i] >> 4) & 0x0f;
	b = zres[i] & 0x0f;

	result[i*2]   = ((a<10) ? (a+48) : (a+87)) & 0xff;
	result[i*2+1] = ((b<10) ? (b+48) : (b+87)) & 0xff;
    }
}



/*
 * This function is taken from xine-lib program.
 * used to calculate clientchallenge2.
 */
void real_calc_challenge2_and_checksum (char *response, char *chksum, /* outs */
					char *challenge) /* in */
{

    int   ch_len, resp_len;
    int   i;
    char *ptr;
    char  buf[128];

    /* initialize return values */
    memset(response, 0, 64);
    memset(chksum, 0, 34);
  
    /* initialize buffer */
    memset(buf, 0, 128);
    ptr=buf;
    put32_be(ptr, 0xa1e9149d);
    ptr+=4;
    put32_be(ptr, 0x0e6b3b59);
    ptr+=4;

    /* some (length) checks */
    if (challenge != NULL)
	{
	    ch_len = strlen (challenge);

	    if (ch_len == 40) /* what a hack... */
		{
		    challenge[32]=0;
		    ch_len=32;
		}
	    if ( ch_len > 56 ) ch_len=56;
    
	    /* copy challenge to buf */
	    memcpy(ptr, challenge, ch_len);
	}
  
    /* xor challenge bytewise with xor_table */
    for (i=0; i<XOR_TABLE_LEN; i++)
	ptr[i] = ptr[i] ^ xor_table[i];
  
    calc_response_string (response, buf);

    /* add tail */
    resp_len = strlen (response);
    strcpy (&response[resp_len], "01d0a8e3");

    /* calculate checksum */
    for (i=0; i<resp_len/4; i++)
	chksum[i] = response[i*4];
}




/*
 * count rules 
 *     >>>> CAUTION <<<<
 * starting '#' is not necessary, but ending ';' is necessary, so 
 * couting ; is correct.
 */
int count_rules(char *asmrule)
{
    int count = 0;
    while(*asmrule)
	if(*(asmrule++) == ';') count++;
  
    return count;
}

static char *next_rule(char *data)
{
    data = strchr(data,';');
    return data + 1;
}



/*
 * finds rule which matches bandwidth.
 *          return value:    number of matched rules
 * rulearray contains its rule numbers.
 */
int find_asmrule_match(char *asmrule,int **matched,int bw)
{
    int i;
    int rules;
    int matchedrules = 0;

    rules = count_rules(asmrule);
  
    *matched = (int *)xmalloc(sizeof(int) * rules);
  
    for(i = 0; i < rules ;i++) {
	/* copy one rule to string*/
	if(asmrule_match(asmrule,bw)) { /* match!! */
	    display(MSDL_DBG,"rule[%d] = %d matched\n",matchedrules,i);
	    (*matched)[matchedrules] = i;
	    matchedrules++;
	}
	asmrule = next_rule(asmrule);
    }
  
    return matchedrules;
}



/*
 * copy mlti_data('type specific data') to it's buffer.
 *
 * *buf is malloc()ed in this function. don't forget to free later
 */
static int select_mlti_data(unsigned char *mlti_data,int mlti_data_size,
			    int match,unsigned char **buf)
{

    int numrules,codec,size;
    int i;
  
    if(memcmp(mlti_data,"MLTI",4)) {
	display(MSDL_ERR | MSDL_VER,
		"'MLTI' not found on OpaqueData base64 decoded string\n");
	*buf = (unsigned char *)xmalloc(mlti_data_size);
	memcpy(*buf,mlti_data,mlti_data_size);
	return mlti_data_size;
    }
  
    mlti_data += 4;
  
    numrules = get16_be(mlti_data);
    if(match >= numrules) {
	display(MSDL_ERR,"matched rule >= number of rules .. strange ...\n");
	return 0;
    }
  
    mlti_data += (match + 1) * 2;
  
    codec = get16_be(mlti_data);
  
    mlti_data += (numrules - match) * 2;

    numrules = get16_be(mlti_data);
  
    if(codec >= numrules) {
	display(MSDL_ERR,"number of codecs >= number of codecs .. strange ...\n");
	return 0;
    }
  
    mlti_data += 2;
  
    for(i = 0; i < codec ; i++) {
	size = get32_be(mlti_data);
	mlti_data += size + 4;
    }

    size = get32_be(mlti_data);
  
    *buf = (unsigned char *)xmalloc(size);
    memcpy(*buf,mlti_data + 4,size);
  
    return size;
}



/*
 * parse sdp(stream description) lines.
 * 
 * subscribe is "Subscribe: " which is send with SET_PARAMETER request.
 *
 */
struct rmff_header_t *real_parse_sdp(char *data,char **subscribe,int bw)
{
  
    struct sdpreal_t *desc;
    struct p_list *p;
    struct rmff_header_t *rmff_header;
    unsigned char *buf;
    int *matched; /* array of matched rules. malloced in find_asmrule_match */
    int matched_rules;
    int i,j;
    int mlti_data_len;
  
    int subscribe_len = BUFSIZE_1K;
    char subbuf[32];
  
    int duration =0;
    int max_bit_rate = 0;
    int avg_bit_rate = 0;
    int max_packet_size = 0;
    int avg_packet_size = 0;

    desc = sdpreal_parse(data);
  
    rmff_header = new_rmff_header_t();
    rmff_header->fileheader = new_rmff_fileheader(4 + desc->stream_count);
    rmff_header->cont = new_rmff_cont(desc->title,desc->author,
				      desc->copyright,desc->abstract);
    rmff_header->data = new_rmff_dataheader(0,0);
    rmff_header->streams =
	xmalloc(sizeof(struct rmff_mdpr_t*) * (desc->stream_count + 1));
    display(MSDL_VER,"number of streams: %u\n",desc->stream_count);
  
    *subscribe = (char *)xmalloc(subscribe_len);
    strcpy(*subscribe,"Subscribe: ");
  
    for(i = 0,p = desc->streams; i < desc->stream_count; i++,p = p->next) {
    
	struct sdpreal_stream_t *stream = p->p;
  
	/*  Subscribe: stream=0;rule=16,stream=0;rule=17 */
    
	matched_rules = find_asmrule_match(stream->asm_rule_book,
					   &matched,bw);
	if((strlen(*subscribe) + 32 * matched_rules) > subscribe_len) {
	    /* subscribe should be longer */
	    subscribe_len = (subscribe_len * 2 >
			     strlen(*subscribe) + 32 * matched_rules) ?
		subscribe_len * 2 : strlen(*subscribe) + 32 * matched_rules;
	    *subscribe = (char *)xrealloc(*subscribe,subscribe_len);
	}
    
	for(j = 0; j < matched_rules ; j++) {
	    _snprintf(subbuf,sizeof(subbuf),"stream=%u;rule=%u,",i,matched[j]);
	    strcat(*subscribe,subbuf);
	}
    
	buf = NULL;
	if(!stream->mlti_data) {
	    mlti_data_len = 0;
	}
	else {
	    /* buf is malloc()ed inside select_mlti_data */
	    mlti_data_len = select_mlti_data(stream->mlti_data,
					     stream->mlti_data_size,
					     matched[0],&buf);
	}
    
	rmff_header->streams[i] =
	    new_rmff_mdpr(stream->stream_id,
			  stream->max_bit_rate,
			  stream->avg_bit_rate,
			  stream->max_packet_size,
			  stream->avg_packet_size,
			  stream->start_time,
			  stream->preroll,
			  stream->duration,
			  stream->stream_name,
			  stream->mime_type,
			  mlti_data_len,
			  buf);
    
	duration = (duration > stream->duration) ? 
	    duration : stream->duration;
	max_bit_rate += stream->max_bit_rate;
	avg_bit_rate += stream->avg_bit_rate;
	max_packet_size = (max_packet_size > stream->max_packet_size) ?
	    max_packet_size : stream->max_packet_size;
    
	if(avg_packet_size) {
	    avg_packet_size = (avg_packet_size + stream->avg_packet_size) / 2;
	}
	else {
	    avg_packet_size = stream->avg_packet_size;
	}
    
	if(matched) {
	    free(matched);
	    matched = NULL;
	}
	if(buf) {
	    free(buf);
	    buf = NULL;
	}
    
    }
  
    if((*subscribe)[strlen(*subscribe) -1 ] == ',') {
	(*subscribe)[strlen(*subscribe) -1 ] = '\0'; /* delete last comma */
    }
  
    rmff_header->prop =
	new_rmff_prop(max_bit_rate,
		      avg_bit_rate,
		      max_packet_size,
		      avg_packet_size,
		      0,
		      duration,
		      0,0,0,
		      desc->stream_count,
		      desc->flags);

    rmff_fix_header(rmff_header);
  
    free_sdpreal_t(desc);

    rmff_print_header(rmff_header);
  
    return rmff_header;
}




/*
 * send SETUP,SEND_REQUEST,PLAY requests.
 *
 *      return value     rmff_header: success
 *                                -1: error.
 */
struct rmff_header_t *real_setup_and_get_header(struct stream_t *stream)
{
    struct stream_ctrl_t *stream_ctrl = stream->stream_ctrl;
    struct rtsp_ctrl_t *rtsp_ctrl = stream_ctrl->rtsp_ctrl;
    struct rtsp_header_t *rtsp_hdr;
    struct rmff_header_t *rmff_header;

    int status_code = 0; /* return value, status code of last PLAY request */
    int maxbw;
    int len;
    char *field;        /* temporary pointer */
    char *session_id = NULL;
    char *description = NULL;
    char *subscribe = NULL;
	char *buffer = NULL; /* temp buffer */
    char challenge2[64];
    char checksum[34];

	buffer = xmalloc(BUFSIZE_1K);
    /*
      !!!CAUTION!!!
      setting maxbw too hight value may cause download failure!!
    */
    maxbw = stream_ctrl->bandwidth;
  
    /* don't use this value, its just for choosing highest rate stream */
    if(!stream_ctrl->bandwidth) stream_ctrl->bandwidth = 10485800;  
  
    /*
      buffering policy:
      USE buffer for temp buffer
    */
    rtsp_hdr = new_rtsp_header();
    rtsp_set_standard_fields(rtsp_ctrl,rtsp_hdr);
    rtsp_set_field(rtsp_hdr, "Accept: application/sdp");
    rtsp_set_field(rtsp_hdr, real_useragent);
    rtsp_set_field(rtsp_hdr, "Require: com.real.retain-entity-for-setup");
  
    _snprintf(buffer,BUFSIZE_1K - 1,"Bandwidth: %u",stream_ctrl->bandwidth);
    rtsp_set_field(rtsp_hdr,buffer); 
  
    rtsp_set_field(rtsp_hdr, "Language: en-US");
    rtsp_set_field(rtsp_hdr, "RegionData: 0");
    rtsp_set_field(rtsp_hdr, real_clientid);
    rtsp_set_field(rtsp_hdr, "GUID: 00000000-0000-0000-0000-000000000000");
    rtsp_set_field(rtsp_hdr, "SupportsMaximumASMBandwidth: 1");
  
  
    rtsp_request_describe(rtsp_hdr,stream_ctrl->url,NULL);
    rtsp_send_request(stream,rtsp_hdr);
    free_rtsp_header(rtsp_hdr);

    rtsp_hdr = new_rtsp_header();
    rtsp_recv_header(stream,rtsp_hdr);
  
    /* NOT OK */
    if(rtsp_hdr->status_code < 200 || 299 < rtsp_hdr->status_code) { 
	display(MSDL_ERR,"status code %d (%s)\n",
		rtsp_hdr->status_code,rtsp_hdr->reason_phrase);

	field = rtsp_get_field(rtsp_hdr,"Alert");
	if(field) {
	    while(*field == ' ') field++;
	    display(MSDL_ERR,"message from server --> %s\n",field);
	}
	goto failed;
    }

    len = 0;
  
    /* Content-length must be present */
    if((field = rtsp_get_field(rtsp_hdr,"Content-length")) != NULL) {
	while(*field == ' ') field++;
	len = atoi(field);
    }
    else { /* no Content-length */
	display(MSDL_ERR,"warning: No Content-length in fields!!\n");
    }
  
    if((field = rtsp_get_field(rtsp_hdr,"ETag")) != NULL) {
	while(*field == ' ') field++;
	session_id = strdup(field);
    }
    else {
	display(MSDL_ERR,"warning: No ETag!!\n");
    }
  

    /*
      copy description (sdp)
    */
    description = (char *)xmalloc(len + 1);
    read_data(stream,(unsigned char *)description,len);
    description[len] = '\0';
  
    display(MSDL_DBG,"==================\n%s\n=(%d bytes)========\n",
	    description,(int)strlen(description));
  
  
    /* parse sdp and get information about file to download */
    rmff_header = real_parse_sdp(description,&subscribe,stream_ctrl->bandwidth);
  
  
    display(MSDL_VER,"sdp parse done.\n");
    /* make new rtsp_hdr 2 send */
    free_rtsp_header(rtsp_hdr);
  

    /* send SETUP request */
    rtsp_hdr = new_rtsp_header();
    rtsp_set_standard_fields(rtsp_ctrl,rtsp_hdr);
  
    real_calc_challenge2_and_checksum(challenge2,checksum,rtsp_ctrl->challenge);
  
    _snprintf(buffer,BUFSIZE_1K - 1,"RealChallenge2: %s, sd=%s",
	     challenge2,checksum);
    rtsp_set_field(rtsp_hdr,buffer);
  
  
    _snprintf(buffer,BUFSIZE_1K - 1,"If-Match: %s",session_id);
    rtsp_set_field(rtsp_hdr,buffer);
    //rtsp_set_field(rtsp_hdr,"RDTFeatureLevel: 2");
    rtsp_set_field(rtsp_hdr,real_transport);
    _snprintf(buffer,BUFSIZE_1K - 1,"%s/streamid=0",rtsp_ctrl->mrl);
    rtsp_request_setup(rtsp_hdr,stream_ctrl->url,buffer);
  
    rtsp_send_request(stream,rtsp_hdr);
    free_rtsp_header(rtsp_hdr);
  
    /* receive message for SETUP */
    rtsp_hdr = new_rtsp_header();
    rtsp_recv_header(stream,rtsp_hdr);
    free_rtsp_header(rtsp_hdr);
  
    /*
      send messages if multiple stream exists (max is 2 streams anyways...)
    */
    if(rmff_header->prop->num_streams > 1) {
	rtsp_hdr = new_rtsp_header();
	rtsp_set_standard_fields(rtsp_ctrl,rtsp_hdr);
	rtsp_set_field(rtsp_hdr,real_transport);
	_snprintf(buffer,BUFSIZE_1K - 1,"If-Match: %s",session_id);
	rtsp_set_field(rtsp_hdr,buffer);
    
	_snprintf(buffer,BUFSIZE_1K - 1,"%s/streamid=1",rtsp_ctrl->mrl);
	rtsp_request_setup(rtsp_hdr,stream_ctrl->url,buffer);
    
	rtsp_send_request(stream,rtsp_hdr);
	free_rtsp_header(rtsp_hdr);
    
	/* receive message for SETUP */
	rtsp_hdr = new_rtsp_header();
	rtsp_recv_header(stream,rtsp_hdr);
	free_rtsp_header(rtsp_hdr);
    }
  

    /* send SET_PARAMETER request to download stream */
    rtsp_hdr = new_rtsp_header();
    rtsp_set_standard_fields(rtsp_ctrl,rtsp_hdr);
    rtsp_set_field(rtsp_hdr,subscribe);
    rtsp_request_set_parameter(rtsp_hdr,stream_ctrl->url,NULL);
    rtsp_send_request(stream,rtsp_hdr);
    free_rtsp_header(rtsp_hdr);
  
  
    /* receive message for SET_PARAMETER*/
    rtsp_hdr = new_rtsp_header();
    rtsp_recv_header(stream,rtsp_hdr);
    /* do something here  */
    free_rtsp_header(rtsp_hdr);
  

    if(maxbw) { /* 0, if no bandwidth limit */
    
	/* send SET_PARAMETER(2nd) request to download stream */
	rtsp_hdr = new_rtsp_header();
	rtsp_set_standard_fields(rtsp_ctrl,rtsp_hdr);
	_snprintf(buffer,BUFSIZE_1K - 1,
		 "SetDeliveryBandwidth: Bandwidth=%d;BackOff=0",maxbw);
    
	rtsp_set_field(rtsp_hdr,buffer);
	rtsp_request_set_parameter(rtsp_hdr,stream_ctrl->url,NULL);
	rtsp_send_request(stream,rtsp_hdr);
	free_rtsp_header(rtsp_hdr);
    
	/*
	  receive message for SET_PARAMETER
	  (!!caution!! the answer will arrive among rdt datas. how dumb!!)
	*/

	/*
	  rtsp_hdr = new_rtsp_header();
	  rtsp_recv_header(stream,rtsp_hdr);
	  free_rtsp_header(rtsp_hdr);
	*/
    }
    

    /*  send PLAY request to download stream */
    rtsp_hdr = new_rtsp_header();
    rtsp_set_standard_fields(rtsp_ctrl,rtsp_hdr);

    //rtsp_set_field(rtsp_hdr,useragent);
    rtsp_set_field(rtsp_hdr,"Speed: 1.000");
    rtsp_set_field(rtsp_hdr,"Range: npt=0-");
    rtsp_request_play(rtsp_hdr,stream_ctrl->url,NULL);
  
    rtsp_send_request(stream,rtsp_hdr);
    free_rtsp_header(rtsp_hdr);
  
    /* receive message for PLAY request */
    rtsp_hdr = new_rtsp_header();
    status_code = rtsp_recv_header(stream,rtsp_hdr);
    free_rtsp_header(rtsp_hdr);
  
  
  

    free(subscribe);
    free(session_id);
    free(description);
 	free(buffer); 
    return rmff_header;
  
  failed:
    if(subscribe) free(subscribe);
    if(session_id) free(session_id);
    if(description) free(description);
	if(buffer) free(buffer);
    return NULL;
}



/*
 * read from media stream. (real rdp packet)
 * 
 * fill 'buffer' with data, and buffer size is 'max_size'.
 * receive one chunk from stream->sock, and fills 'max_size' bytes to buffer.
 * and if there are more bytes in the chunk, the data is stored to
 * stream->stream_ctrl->buffer.
 *
 *    return value: positive value ... bytes written to buffer.
 *                  0              ... END OF STREAM
 *                 -1              ... ERROR
 */
int real_rdt_get_media_packet(struct stream_t *stream,
			      unsigned char *buffer,size_t max_size)
{
    int n = 1;
    struct stream_ctrl_t *stream_ctrl = stream->stream_ctrl;
    unsigned char header[8];
    struct rmff_pheader_t ph;
    int size;
    int flags1,flags2;
    int unknown1;
    unsigned long ts;
  
  
    n = rtsp_real_get_data(stream,header,8); /* through away all non-rdt packets*/
    if(n < 8) {
	if(stream_ctrl->status == streaming_finished){
	    display(MSDL_VER,"got end of stream message packet\n");
	    return 0;
	}
	else {
	    goto failed;
	}
    }
  
    if(header[0] != 0x24) {
	int i;
	display(MSDL_ERR,"wrong rdt magic : [0x%02x]\n",header[0]);
	/* dump header*/
	for( i = 0; i < 8; i++) display(MSDL_VER,"[%02x] ",header[i]);
	display(MSDL_ERR,"\n");
	display(MSDL_ERR,"(");
	for( i = 0; i < 8; i++) display(MSDL_VER,"%c",header[i]);
	display(MSDL_ERR,")\n");
	goto failed;
    }
    size = (header[1] << 16) + (header[2] << 8) + header[3];
    flags1 = header[4];
  
    if((flags1!=0x40) && (flags1!=0x42)) {
	display(MSDL_DBG | MSDL_ERR,"wrong rdt flags1 : [0x%02x]\n",flags1);
	if(header[6] == 0x06) {
	    display(MSDL_VER,"got end of stream packet\n");
	    stream_ctrl->status = streaming_finished;
	    return 0;
	}
	header[0] = header[5];
	header[1] = header[6];
	header[2] = header[7];
    
	n = read_data(stream,header + 3,5);
	if(n < 5) goto failed;
    
	n = read_data(stream,header + 4,4);
	if(n < 4) goto failed;
    
	flags1 = header[4];
	size -= 9;
    
    }
    flags2 = header[7];
    unknown1 = (header[5] << 16) + (header[6] << 8) + header[7];
    n = read_data(stream,header,6);
    if(n < 6) goto failed;
  
    ts = get32_be(header);
    stream_ctrl->packet_count++;
    display(MSDL_DBG,"ts: %u size: %u, flags:0x%02x, unknown:%u  0x%02x 0x%02x, flags2 %d\n",
	    ts,size,flags1,unknown1,header[4],header[5],flags2);
  
    size += 2;
  
    ph.object_version = 0;
    ph.length = size;
    ph.stream_number = (flags1>>1) & 1;
    ph.timestamp = ts;
    ph.reserved = 0;
  
    if((flags2 & 1) == 0 &&
       (stream_ctrl->rtsp_ctrl->prev_ts != ts ||
	stream_ctrl->rtsp_ctrl->prev_stream_number != ph.stream_number)) {
	stream_ctrl->rtsp_ctrl->prev_ts = ts;
	stream_ctrl->rtsp_ctrl->prev_stream_number = ph.stream_number;
	ph.flags = 2;
    }
  
  
    else {
	ph.flags = 0;
    }
  
    if(max_size > size) {
	/* all data can go to buffer --> just do it!! */
	rmff_dump_pheader(&ph,buffer);
	size -= 12;
	n = read_data(stream,buffer + 12,size);
	if(n <= 0) goto failed;
    
	return n + 12;
    }
    else {
	/*
	  buffer is not enough.. copy max_size data to 
	  buffer and the rest goes to stream_ctrl->buffer.
	*/
	rmff_dump_pheader(&ph,stream_ctrl->write_buffer);
    
	n = read_data(stream,stream_ctrl->write_buffer + 12,size - 12);
	if(n <= 0) goto failed;
    
	/*
	  it is guranteed that stream_ctrl->buffer is
	  practically empty when it comes here!
	  --> reset stream_ctrl->write_pos and so on...
	*/
    
	memcpy(buffer,stream_ctrl->write_buffer,max_size);
	stream_ctrl->write_data_len = size - max_size;
	stream_ctrl->write_pos = max_size;
    
	return max_size;
    }
  
  
  failed:
    return -1;
}



/*
 * get_data function for RTSP-REAL protocol.
 * buffer is guranteed to be bigger than size,
 * if not, this function may cause segfault.
 * all data goes to 'buffer'.
 *        return value     bytes read form sock : success
 *                             0                : EOF (retval of read_data)
 *                            -1                : error
 *
 */
int rtsp_real_get_data(struct stream_t *stream,unsigned char *buffer,size_t size)
{
    int i;
    struct stream_ctrl_t *stream_ctrl = stream->stream_ctrl;
    struct rtsp_ctrl_t *rtsp_ctrl = stream_ctrl->rtsp_ctrl;


    if(size > 4) {

	i = read_data(stream,buffer,4);
	if(i < 4) { /* failed (0 or -1)*/
	    return i;
	}

	while(!memcmp(buffer,"SET_",4) ||
	      !memcmp(buffer,"OPTI",4) ||
	      !memcmp(buffer,"RTSP",4) ||
	      !memcmp(buffer,"ANNO",4)) {

	    struct rtsp_header_t *rtsp_hdr;
	    int status_code;
	    int bufsize;
	    int len;
	    unsigned char *body = NULL;
	    char *field;
	    int got_cseq;
      
	    buffer[4] = '\0'; /* insert \0 byte (buffer is bigger than 4 bytes)*/
      
	    bufsize = 0;
	    /* push back 4 bytes */
	    stream_data_push_back(stream,buffer,4);
      
	    rtsp_hdr = new_rtsp_header();
	    rtsp_recv_header(stream,rtsp_hdr);
      
      
	    /*
	      get necessary information from rtsp header.
	    */
	    status_code = rtsp_hdr->status_code;
      
	    /* get content length data */
	    if((field = rtsp_get_field(rtsp_hdr,"Content-length")) != NULL) {
		while(*field == ' ') field++;
		len = atoi(field);	
		body = xmalloc(len);
		len = read_data(stream,body,len);
	    }
	    /* get Cseq, for sending OK back */
	    if((field = rtsp_get_field(rtsp_hdr,"Cseq")) != NULL) {
		while(*field == ' ') field++;
		got_cseq = atoi(field);	
	    }

	    free_rtsp_header(rtsp_hdr);

      
	    if(!memcmp(buffer,"RTSP",4)) { /* RTSP/1.0 200 OK , ignore this */
		/*
		  somehow RTSP message reply can be found among rdt messages.
		  send nothing.
		*/
		if(!got_cseq == rtsp_ctrl->cseq) {
		    display(MSDL_DBG | MSDL_ERR,"CSeq mismatch: expected %d, got %d",
			    rtsp_ctrl->cseq,got_cseq);
		}
		/* DO NOT send RTSP OK */
	    }
	    else { /* not RTSP, whatever it is, send OK would be just fine  */
		rtsp_200ok(stream,got_cseq,rtsp_ctrl->session);
	    }
      
	    free(body);
	    i = read_data(stream,buffer,4);
	}
   
	/* normal case, probably rdp pacekt */
	i = read_data(stream,buffer + 4,size - 4) + 4;
    
    } /* if(size > 4) */
    else { /* size is smaller than 4 bytes */
	i = read_data(stream,buffer,size);
    }
  
  
    if(i != size) {
	display(MSDL_ERR,"rtsp_get_data: got %i of %u bytes\n",
		i,(unsigned int)size);
    }
    return i;
}
