/***********************************************************************
 *    rmff.h: for using rmff header (real media file format)
 ***********************************************************************
 * Copyright (C) 2007 metro <me_t_ro@yahoo.com>
 *
 * This file is part of msdl, media stream downloader
 *
 * This file is mainly(totally) based on rmff.c in xine-lib,
 * and include code from xine.
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
 *
 ***********************************************************************/ 



#ifndef __RMFF_H__
#define __RMFF_H__

//#include <inttypes.h>


enum {
    RMF_TAG  = ('.'<<24) | ('R'<<16) | ('M'<<8) | 'F',
    PROP_TAG = ('P'<<24) | ('R'<<16) | ('O'<<8) | 'P',
    MDPR_TAG = ('M'<<24) | ('D'<<16) | ('P'<<8) | 'R',
    CONT_TAG = ('C'<<24) | ('O'<<16) | ('N'<<8) | 'T',
    DATA_TAG = ('D'<<24) | ('A'<<16) | ('T'<<8) | 'A',
    INDX_TAG = ('I'<<24) | ('N'<<16) | ('D'<<8) | 'X',
    PNA_TAG  = ('P'<<24) | ('N'<<16) | ('A'<<8) |  0 ,
    MLTI_TAG = ('M'<<24) | ('L'<<16) | ('T'<<8) | 'I',
};

enum {
    PN_SAVE_ENABLED         = 0x01,
    PN_PERFECT_PLAY_ENABLED = 0x02,
    PN_LIVE_BROADCAST       = 0x04,
} prop_flags;



/*
 * rm header data structs (taken from xine project, thank you so much :))
 */
struct rmff_fileheader_t {

    unsigned long object_id;
    unsigned long size;
    unsigned int object_version;

    unsigned long file_version;
    unsigned long num_headers;
};

struct rmff_prop_t {

    unsigned long object_id;
    unsigned long size;
    unsigned int object_version;
  
    unsigned long max_bit_rate;
    unsigned long avg_bit_rate;
    unsigned long max_packet_size;
    unsigned long avg_packet_size;
    unsigned long num_packets;
    unsigned long duration;
    unsigned long preroll;
    unsigned long index_offset;
    unsigned long data_offset;
    unsigned int num_streams;
    unsigned int flags;
};

struct rmff_mdpr_t {
  
    unsigned long  object_id;
    unsigned long  size;
    unsigned int  object_version;

    unsigned int  stream_number;
    unsigned long  max_bit_rate;
    unsigned long  avg_bit_rate;
    unsigned long  max_packet_size;
    unsigned long  avg_packet_size;
    unsigned long  start_time;
    unsigned long  preroll;
    unsigned long  duration;
    unsigned char   stream_name_size;
    char      *stream_name;
    unsigned char   mime_type_size;
    char      *mime_type;
    unsigned long  type_specific_len;
    char      *type_specific_data;

    int       mlti_data_size;
    char      *mlti_data;
};

struct rmff_cont_t {

    unsigned long  object_id;
    unsigned long  size;
    unsigned int  object_version;

    unsigned int  title_len;
    char      *title;
    unsigned int  author_len;
    char      *author;
    unsigned int  copyright_len;
    char      *copyright;
    unsigned int  comment_len;
    char      *comment;
};

struct rmff_data_t {
  
    unsigned long object_id;
    unsigned long size;
    unsigned int object_version;

    unsigned long num_packets;
    unsigned long next_data_header; /* rarely used */
};

struct rmff_header_t {

    struct rmff_fileheader_t *fileheader;
    struct rmff_prop_t *prop;
    struct rmff_mdpr_t **streams;
    struct rmff_cont_t *cont;
    struct rmff_data_t *data;
}; 

struct rmff_pheader_t {

    unsigned int object_version;

    unsigned int length;
    unsigned int stream_number;
    unsigned long timestamp;
    unsigned char reserved;
    unsigned char flags;
};

struct rmff_header_t *new_rmff_header_t(void);
void free_rmff_header_t(struct rmff_header_t *h);

int rmff_dump_header(struct rmff_header_t *h, unsigned char *buffer);
void rmff_dump_pheader(struct rmff_pheader_t *ph,unsigned char *data);

void rmff_fix_header(struct rmff_header_t *h);

struct rmff_fileheader_t *new_rmff_fileheader(unsigned long num_headers);


struct rmff_data_t *new_rmff_dataheader(unsigned long num_packets,
					unsigned long next_data_header);

struct rmff_cont_t *new_rmff_cont(const char *title,
				  const char *author,
				  const char *copyright,
				  const char *comment);


struct rmff_prop_t *new_rmff_prop (unsigned long max_bit_rate,
				   unsigned long avg_bit_rate,
				   unsigned long max_packet_size,
				   unsigned long avg_packet_size,
				   unsigned long num_packets,
				   unsigned long duration,
				   unsigned long preroll,
				   unsigned long index_offset,
				   unsigned long data_offset,
				   unsigned int num_streams,
				   unsigned int flags );

struct rmff_mdpr_t *new_rmff_mdpr(unsigned int   stream_number,
				  unsigned long   max_bit_rate,
				  unsigned long   avg_bit_rate,
				  unsigned long   max_packet_size,
				  unsigned long   avg_packet_size,
				  unsigned long   start_time,
				  unsigned long   preroll,
				  unsigned long   duration,
				  const char *stream_name,
				  const char *mime_type,
				  unsigned long   type_specific_len,
				  const unsigned char *type_specific_data);
  

void rmff_print_header(struct rmff_header_t *h);

#endif /* __RMFF_H__ */

