/***********************************************************************
 *   wmserver.c : for downloading from WMServer using rtsp
 ***********************************************************************
 * Copyright (C) 2007 metro <me_t_ro@yahoo.com>
 *
 * This file is part of msdl, media stream downloader
 *
 * This file is based on information shown on wireshark,
 * analyzing transaction of Windows media player.
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
 *
 ***********************************************************************/ 



#include <stdio.h>
#include <stdlib.h>
#include <string.h>
//#include <unistd.h>

//#include <inttypes.h>
#include <sys/types.h>
#include<winsock2.h>

#include "msdl.h"
#include "msdllib.h"
#include "network.h"
#include "rtsp.h"
#include "asf.h"
#include "sdpcommon.h"
#include "sdpwms.h"
//#include <sys/syscall.h>


static const char wms_useragent[] =  "User-Agent: WMPlayer/10.0.0.380 guid/3300AD50-2C39-46C0-AE0A-2062C3818CAD";

static const char wms_supported[] = "Supported: com.microsoft.wm.srvppair, com.microsoft.wm.sswitch, com.microsoft.wm.eosmsg, com.microsoft.wm.fastcache, com.microsoft.wm.packetpairssrc, com.microsoft.wm.startupprofile";

static const char wms_authentication[]  = "X-Accept-Authentication: Negotiate, NTLM, Digest, Basic";

static const char wms_transport[] = "Transport: RTP/AVP/TCP;unicast;interleaved=0-1;ssrc=75afe1e1;mode=PLAY";




/*
 * send SETUP,SEND_REQUEST,PLAY requests.
 *
 *      return value   asf_headerinfo: success
 *                                -1 : error.
 */
struct asf_headerinfo_t *wmserver_setup_and_get_header(struct stream_t *stream)
{
    struct stream_ctrl_t *stream_ctrl = stream->stream_ctrl;
    struct rtsp_ctrl_t *rtsp_ctrl = stream_ctrl->rtsp_ctrl;
    struct rtsp_header_t *rtsp_hdr;

    struct asf_headerinfo_t *asf_headerinfo = NULL;
    struct sdpwms_t *sdpwms = NULL;

    int status_code = 0; /* return value, status code of last PLAY request */
    int maxbw;
    int len;
    char *field;         /* temporary pointer            */
    char *description = NULL;
    char *buffer = NULL; /* temporary buffer for request */
  
    buffer = xmalloc(BUFSIZE_1K);

    /*
      use "-b" option 4 low bandwidth..
    */
    maxbw = stream_ctrl->bandwidth;

    rtsp_ctrl->cseq = 1;
    /*
      buffering policy:
      USE buffer
    */
    rtsp_hdr = new_rtsp_header();
    rtsp_set_standard_fields(rtsp_ctrl,rtsp_hdr);
    rtsp_set_field(rtsp_hdr, wms_useragent);
    rtsp_set_field(rtsp_hdr, wms_supported);

    rtsp_request_describe(rtsp_hdr,stream_ctrl->url,NULL);
    rtsp_send_request(stream,rtsp_hdr);
    free_rtsp_header(rtsp_hdr);

    rtsp_hdr = new_rtsp_header();
    rtsp_recv_header(stream,rtsp_hdr);
  
    /* NOT OK */
    if(rtsp_hdr->status_code < 200 || 299 < rtsp_hdr->status_code) { 
	display(MSDL_ERR,"status code %d (%s)\n",
		rtsp_hdr->status_code,rtsp_hdr->reason_phrase);
    
	field = rtsp_get_field(rtsp_hdr,"Alert");
	if(field) {
	    while(*field == ' ') field++;
	    display(MSDL_ERR,"message from server --> %s\n",field);
	}
	goto failed;
    }
  
    len = 0;

    /* Content-length must be present */
    if((field = rtsp_get_field(rtsp_hdr,"Content-Length")) != NULL) {
	while(*field == ' ') field++;
	len = atoi(field);
    }
    else { /* no Content-length */
	display(MSDL_ERR,"warning: No Content-Length in fields!!\n");
    }
  
    if((field = rtsp_get_field(rtsp_hdr,"ETag")) == NULL) {
	display(MSDL_ERR,"warning: No ETag!!\n");
    }

  
    /*
      copy description (sdp)
    */
    description = (char *)xmalloc(len + 1);
  
    read_data(stream,(unsigned char *)description,len);
    description[len] = '\0';
  
  
    display(MSDL_DBG,"=desc=================\n%s\n=(%d bytes)========desc=\n",
	    description,(int)strlen(description));
  
  
    /* parse sdp and get information about file to download */
    sdpwms = wmserver_parse_sdp(description);
    //if(sdpwms->asf_header_len <= sizeof(struct asf_header_t)) {
	if(sdpwms->asf_header_len <= sizeof(struct asf_header_t)) {
	display(MSDL_ERR,"asf header smaller than asf_header_t\n");
	goto failed;
    }
  
    asf_headerinfo = new_asf_headerinfo_t();
    asf_interpret_header(asf_headerinfo,maxbw,
			 sdpwms->asf_header,sdpwms->asf_header_len);
    rtsp_ctrl->asf_headerinfo = asf_headerinfo;
  
    /* set asf header len */
    asf_headerinfo->asf_header_len = sdpwms->asf_header_len;

    /* copy raw header */
    asf_headerinfo->asf_header = xmalloc(sdpwms->asf_header_len);
    memcpy(asf_headerinfo->asf_header,sdpwms->asf_header,sdpwms->asf_header_len);
  
    /* set infomation */
    stream_ctrl->packet_length = asf_headerinfo->fileh->max_packet_size;
    stream_ctrl->file_size = asf_headerinfo->fileh->file_size;
    stream_ctrl->total_packets = asf_headerinfo->fileh->num_packets;
  
    free_rtsp_header(rtsp_hdr);
  
    /* copy to write buffer */
    stream_ctrl->write_pos = 0;
    memcpy(stream_ctrl->write_buffer, sdpwms->asf_header, sdpwms->asf_header_len);
    stream_ctrl->write_data_len = sdpwms->asf_header_len;
  
  
    /* SETUP requests (for audio and video) */
    if(!(asf_headerinfo->streams->n_audio + asf_headerinfo->streams->n_video)) {
	goto failed;
    }
  
    /* SETUP for audio */
    if(asf_headerinfo->streams->n_audio) {
	struct p_list *p;
	struct sdpwms_stream_t *sdpwmsstream;
	char *controlstr = NULL;

	rtsp_hdr = new_rtsp_header();
	rtsp_set_standard_fields(rtsp_ctrl,rtsp_hdr);
	rtsp_set_field(rtsp_hdr,wms_supported);
	rtsp_set_field(rtsp_hdr,wms_transport);
    
    
	/* find control for audio_id */
	controlstr = NULL;
	for(p = sdpwms->streams ; p ; p = p->next) {
	    sdpwmsstream = p->p;
	    if(sdpwmsstream->streamnum == asf_headerinfo->streams->audio_id) {
		controlstr = sdpwmsstream->control;
	    }
	}
    
	if(!controlstr) {
	    controlstr = "audio"; /* default */
	}

	/* stream number */
	_snprintf(buffer,BUFSIZE_1K - 1,"%s/%s",rtsp_ctrl->mrl,controlstr);

	rtsp_request_setup(rtsp_hdr,stream_ctrl->url,buffer);
	rtsp_send_request(stream,rtsp_hdr);
	free_rtsp_header(rtsp_hdr);

	/* receive message for SETUP */
	rtsp_hdr = new_rtsp_header();
	rtsp_recv_header(stream,rtsp_hdr);
	free_rtsp_header(rtsp_hdr);
    }
    /* SETUP for video */
    if(asf_headerinfo->streams->n_video) {
	struct p_list *p;
	struct sdpwms_stream_t *sdpwmsstream;
	char *controlstr = NULL;
    
	rtsp_hdr = new_rtsp_header();
	rtsp_set_standard_fields(rtsp_ctrl,rtsp_hdr);
	rtsp_set_field(rtsp_hdr,wms_supported);
	rtsp_set_field(rtsp_hdr,wms_transport);
    

	/* find control for audio_id */
	controlstr = NULL;
	for(p = sdpwms->streams ; p ; p = p->next) {
	    sdpwmsstream = p->p;
	    if(sdpwmsstream->streamnum == asf_headerinfo->streams->video_id) {
		controlstr = sdpwmsstream->control;
	    }
	}
    
	if(!controlstr) {
	    controlstr = "video"; /* default */
	}
    
	_snprintf(buffer,BUFSIZE_1K - 1,"%s/%s",rtsp_ctrl->mrl,controlstr);

	rtsp_request_setup(rtsp_hdr,stream_ctrl->url,buffer);
	rtsp_send_request(stream,rtsp_hdr);
	free_rtsp_header(rtsp_hdr);
    
	/* receive message for SETUP */
	rtsp_hdr = new_rtsp_header();
	rtsp_recv_header(stream,rtsp_hdr);
	free_rtsp_header(rtsp_hdr);
    }
 

    /* send SET_PARAMETER request to download stream */
    rtsp_hdr = new_rtsp_header();
    rtsp_set_standard_fields(rtsp_ctrl,rtsp_hdr);
    rtsp_set_field(rtsp_hdr,"Speed: 1.000");
    rtsp_request_set_parameter(rtsp_hdr,stream_ctrl->url,NULL);
    rtsp_send_request(stream,rtsp_hdr);
    free_rtsp_header(rtsp_hdr);
  
    /* receive message for SET_PARAMETER*/
    rtsp_hdr = new_rtsp_header();
    rtsp_recv_header(stream,rtsp_hdr);
    /* do something here  */
    free_rtsp_header(rtsp_hdr);
  
  
    /*  send PLAY request to download stream */
    rtsp_hdr = new_rtsp_header();
    rtsp_set_standard_fields(rtsp_ctrl,rtsp_hdr);
  
    //rtsp_set_field(rtsp_hdr,useragent);
    rtsp_set_field(rtsp_hdr,"Range: npt=0-");
    rtsp_request_play(rtsp_hdr,stream_ctrl->url,NULL);
  

    rtsp_send_request(stream,rtsp_hdr);
    free_rtsp_header(rtsp_hdr);
  
    /* receive message for PLAY request */
    rtsp_hdr = new_rtsp_header();
    status_code = rtsp_recv_header(stream,rtsp_hdr);
    free_rtsp_header(rtsp_hdr);
    

    free_sdpwms_t(sdpwms);
    free(description);
    free(buffer);
  
    return asf_headerinfo;
  
  failed:
    if(asf_headerinfo) free_asf_headerinfo_t(asf_headerinfo);
    if(sdpwms) free_sdpwms_t(sdpwms);
    if(description) free(description);
    if(buffer) free(buffer);
    return NULL;
}



/*
 * After reveiving EOF packet, ANNOUNCE may come when
 * streaming still continues. (this is most likely to be a download protection)
 * 
 *
 *
 */
struct asf_headerinfo_t *wmserver_announce_continue(struct stream_t *stream)
{
    struct stream_ctrl_t *stream_ctrl = stream->stream_ctrl;
    struct rtsp_ctrl_t *rtsp_ctrl = stream_ctrl->rtsp_ctrl;
  
    int i;                          /* for various purposes                */
    char checkbuf[10];              /* to check if the message is ANNOUCNE */
    struct rtsp_header_t *rtsp_hdr;
    int got_cseq;                   /* cseq received                */
    int status_code;                /* rtsp_hdr status code         */
    char *field;                    /* filed pointer for rtsp_hdr   */
    int len;                        /* len of body (content lencth) */
    unsigned char *body = NULL;           /* body, data of rtsp packet    */
    char *buffer = NULL;            /* temp buffer for building req */

    int maxbw = stream_ctrl->bandwidth;
    struct sdpwms_t *sdpwms = NULL;
    struct asf_headerinfo_t* asf_headerinfo = NULL;

    
    buffer = xmalloc(BUFSIZE_1K);

    i = recv_data(stream,checkbuf,8);

    if(i < 8 || memcmp(checkbuf,"ANNOUNCE",8)) {
	goto failed;
    }
  
    stream_data_push_back(stream,checkbuf,i); /* push back "ANNOUNCE" */
  
    rtsp_hdr = new_rtsp_header();
    rtsp_recv_header(stream,rtsp_hdr);
  
    status_code = rtsp_hdr->status_code;
    /* get content length data */
    if((field = rtsp_get_field(rtsp_hdr,"Content-Length")) != NULL) {
	while(*field == ' ') field++;
	len = atoi(field);	
	body = xmalloc(len);
	len = read_data(stream,body,len);
    }
  
    if((field = rtsp_get_field(rtsp_hdr,"Cseq")) != NULL) {
	while(*field == ' ') field++;
	got_cseq = atoi(field);	
    }
  
    free_rtsp_header(rtsp_hdr);
  
    if(status_code != RTSP_STATUS_ANNOUNCE) {
	goto failed; /* not likely but have to check this */
    }
  
    /*
      got annouce packet, and data is in body
    */
  
    rtsp_200ok(stream,got_cseq,rtsp_ctrl->session); /* OK (required, according to RFC )*/
  
  
    sdpwms = wmserver_parse_sdp((char *)body);
    if(sdpwms->asf_header_len <= sizeof(struct asf_header_t)) {	
	display(MSDL_ERR,"asf header smaller than asf_header_t %d.%d\n",
		len,sdpwms->asf_header_len);
	goto failed;
    }
	
    asf_headerinfo = new_asf_headerinfo_t();
    asf_interpret_header(asf_headerinfo,maxbw,
			 (unsigned char *)sdpwms->asf_header,sdpwms->asf_header_len);

    /* set header length */
    asf_headerinfo->asf_header_len = sdpwms->asf_header_len;

    /* copy raw header */
    asf_headerinfo->asf_header = xmalloc(sdpwms->asf_header_len);
    memcpy(asf_headerinfo->asf_header,sdpwms->asf_header,sdpwms->asf_header_len);

    /* free last one */
    free_asf_headerinfo_t(rtsp_ctrl->asf_headerinfo);
    rtsp_ctrl->asf_headerinfo = asf_headerinfo;
  
    /* set informations */
    stream_ctrl->packet_length = asf_headerinfo->fileh->max_packet_size;
    stream_ctrl->file_size = asf_headerinfo->fileh->file_size;
    stream_ctrl->total_packets = asf_headerinfo->fileh->num_packets;

    /* Be careful with Cseq: */
  
    /* SETUP for audio */
    if(asf_headerinfo->streams->n_audio) {
	struct p_list *p;
	struct sdpwms_stream_t *sdpwmsstream;
	char *controlstr = NULL;

	rtsp_hdr = new_rtsp_header();
	rtsp_set_standard_fields(rtsp_ctrl,rtsp_hdr);
	rtsp_set_field(rtsp_hdr,wms_supported);
	rtsp_set_field(rtsp_hdr,wms_transport);
    
    
	/* find control for audio_id */
	controlstr = NULL;
	for(p = sdpwms->streams ; p ; p = p->next) {
	    sdpwmsstream = p->p;
	    if(sdpwmsstream->streamnum == asf_headerinfo->streams->audio_id) {
		controlstr = sdpwmsstream->control;
	    }
	}
    
	if(!controlstr) {
	    controlstr = "audio"; /* default */
	}
	
	_snprintf(buffer,BUFSIZE_1K - 1,"%s/%s",rtsp_ctrl->mrl,controlstr);

	rtsp_request_setup(rtsp_hdr,stream_ctrl->url,buffer);
	rtsp_send_request(stream,rtsp_hdr);
	free_rtsp_header(rtsp_hdr);

	/* receive message for SETUP */
	rtsp_hdr = new_rtsp_header();
	rtsp_recv_header(stream,rtsp_hdr);
	free_rtsp_header(rtsp_hdr);
    }
    /* SETUP for video */
    if(asf_headerinfo->streams->n_video) {
	struct p_list *p;
	struct sdpwms_stream_t *sdpwmsstream;
	char *controlstr = NULL;
    

	rtsp_hdr = new_rtsp_header();
	rtsp_set_standard_fields(rtsp_ctrl,rtsp_hdr);
	rtsp_set_field(rtsp_hdr,wms_supported);
	rtsp_set_field(rtsp_hdr,wms_transport);
    

	/* find control for audio_id */
	controlstr = NULL;
	for(p = sdpwms->streams ; p ; p = p->next) {
	    sdpwmsstream = p->p;
	    if(sdpwmsstream->streamnum == asf_headerinfo->streams->video_id) {
		controlstr = sdpwmsstream->control;
	    }
	}
    
	if(!controlstr) {
	    controlstr = "video"; /* default */
	}
    
    
	_snprintf(buffer,BUFSIZE_1K - 1,"%s/%s",rtsp_ctrl->mrl,controlstr);
    
	rtsp_request_setup(rtsp_hdr,stream_ctrl->url,buffer);
	rtsp_send_request(stream,rtsp_hdr);
	free_rtsp_header(rtsp_hdr);
    
	/* receive message for SETUP */
	rtsp_hdr = new_rtsp_header();
	rtsp_recv_header(stream,rtsp_hdr);
	free_rtsp_header(rtsp_hdr);
    }
	

	
    /* send SET_PARAMETER request to download stream */
    rtsp_hdr = new_rtsp_header();
    rtsp_set_standard_fields(rtsp_ctrl,rtsp_hdr);
    rtsp_set_field(rtsp_hdr,"Speed: 1.000");
    rtsp_request_set_parameter(rtsp_hdr,stream_ctrl->url,NULL);
    rtsp_send_request(stream,rtsp_hdr);
    free_rtsp_header(rtsp_hdr);
  
    /* receive message for SET_PARAMETER*/
    rtsp_hdr = new_rtsp_header();
    rtsp_recv_header(stream,rtsp_hdr);
    /* do something here  */
    free_rtsp_header(rtsp_hdr);
  
  
	
    /*  send PLAY request to download stream */
    rtsp_hdr = new_rtsp_header();
    rtsp_set_standard_fields(rtsp_ctrl,rtsp_hdr);
  
    //rtsp_set_field(rtsp_hdr,useragent);
    rtsp_set_field(rtsp_hdr,"Range: npt=0-");
    rtsp_request_play(rtsp_hdr,stream_ctrl->url,NULL);
    rtsp_send_request(stream,rtsp_hdr);
    free_rtsp_header(rtsp_hdr);
	
    free(body);
    free_sdpwms_t(sdpwms);
    free(buffer);
    return asf_headerinfo;
  

  failed:
    if(body) free(body);
    if(sdpwms) free_sdpwms_t(sdpwms);
    if(asf_headerinfo) free_asf_headerinfo_t(asf_headerinfo);
    if(buffer) free(buffer);
    return NULL;
}


/*
 * get_data function for RTSP-WMS protocol.
 * buffer is guranteed to be bigger than size,
 * if not, this function may cause segfault.
 * all data goes to 'buffer'.
 *        return value     bytes read form sock : success
 *                             0                : EOF (retval of read_data)
 *                            -1                : error
 *
 */
int rtsp_wmserver_get_data(struct stream_t *stream,unsigned char *buffer,size_t size)
{
    int i;

    struct stream_ctrl_t *stream_ctrl = stream->stream_ctrl;
    struct rtsp_ctrl_t *rtsp_ctrl = stream_ctrl->rtsp_ctrl;


    if(size > 4) {
    
	i = read_data(stream,buffer,4);
	if(i < 4) { /* failed (0 or -1)*/
	    return i;
	}

    
	while(!memcmp(buffer,"SET_",4) ||
	      !memcmp(buffer,"OPTI",4) ||
	      !memcmp(buffer,"RTSP",4) ||
	      !memcmp(buffer,"ANNO",4)) {
      
	    struct rtsp_header_t *rtsp_hdr;
	    int status_code;
	    int bufsize;
	    int len;
	    unsigned char *body = NULL;
	    char *field;
	    int got_cseq;
      
	    buffer[4] = '\0'; /* insert \0 byte (buffer is bigger than 4 bytes)*/
      
	    bufsize = 0;
	    /* push back 4 bytes */
	    stream_data_push_back(stream,buffer,4);
      
	    rtsp_hdr = new_rtsp_header();
	    rtsp_recv_header(stream,rtsp_hdr);
      
      
	    /*
	      get necessary information from rtsp header.
	    */
	    status_code = rtsp_hdr->status_code;
      
	    /* get content length data */
	    if((field = rtsp_get_field(rtsp_hdr,"Content-Length")) != NULL) {
		while(*field == ' ') field++;
		len = atoi(field);
		body = xmalloc(len);
		len = read_data(stream,body,len);
	    }
      
	    if((field = rtsp_get_field(rtsp_hdr,"Cseq")) != NULL) {
		while(*field == ' ') field++;
		got_cseq = atoi(field);	
	    }
      
	    free_rtsp_header(rtsp_hdr);

      
	    if(!memcmp(buffer,"RTSP",4)) { /* RTSP/1.0 200 OK ... ignore this */
	
		if(!got_cseq == rtsp_ctrl->cseq) {
		    display(MSDL_DBG | MSDL_ERR,"CSeq mismatch: expected %d, got %d",
			    rtsp_ctrl->cseq,got_cseq);
		}
		/* DO NOT send OK back to this */
	    }
	    else if(status_code == RTSP_STATUS_SET_PARAMETER) { /* SET_PARAMETER */
		char *p;

		/* EOF message comes here */
		if((p = strstr((char *)body,"EOF:"))) {
		    p += 4;
		    while(*p == ' ') p++; 
		    if((!strnicmp(p,"true",4))) { /* EOF */	    
			display(MSDL_VER,"End of file packet received\n");
			stream_ctrl->status = streaming_finished;
			free(body);
			return 0;
		    }
		}
		rtsp_200ok(stream,got_cseq,rtsp_ctrl->session);	
	    }      
	    else {
		rtsp_200ok(stream,got_cseq,rtsp_ctrl->session);
	    }
      
	    free(body);
	    i = read_data(stream,buffer,4);
	}
    
	i = read_data(stream,buffer + 4,size - 4) + 4;
    
    } /* if(size > 4) */
    else { /* size is smaller than 4 bytes */
	i = read_data(stream,buffer,size);
    }
  
  
    if(i != size) {
	display(MSDL_ERR,"rtsp_get_data: got %i of %u bytes\n",
		i,(unsigned int)size);
    }
    return i;
}








/*
 * read from media stream. (wmserver rdp packet)
 * 
 * fill 'buffer' with data, and buffer size is 'max_size'.
 * receive one chunk from stream->sock, and fills 'max_size' bytes to buffer.
 * and if there are more bytes in the chunk, the data is stored to
 * stream->stream_ctrl->buffer.
 *
 *    return value: positive value ... bytes written to buffer.
 *                  0              ... END OF STREAM
 *                 -1              ... ERROR
 */
int wmserver_rtp_get_media_packet(struct stream_t *stream,
				  unsigned char *buffer,size_t max_size)
{
    int i;
    int n = 1;
    struct stream_ctrl_t *stream_ctrl = stream->stream_ctrl;
    unsigned char header[8];
    unsigned char header2[8];
  
    int size;    /* size of =in-wma-usable part of= packet */
    int flags1;
    int flags2;
  
    int cc; /* number of csrc */
    unsigned int seq;
    unsigned long ts;
    unsigned long ssrc; /* syncronization source */
    unsigned long csrc = 0; /* contributing source   */
    unsigned char magicheader[4];
  
    n = rtsp_wmserver_get_data(stream,header,8);
  
    if(n < 8) {
	if(stream_ctrl->status == streaming_finished) {
      
	    struct asf_headerinfo_t *asf_headerinfo;
      
	    /*
	      wait for 2 seconds.. this is necessary becasue
	      otherwise it would block when waiting "ANNOUNCE"
	    */
	    if(stream_check_data(stream,2) > 0) {
	
		if((asf_headerinfo = wmserver_announce_continue(stream))) {
		    int ret = 0;
		    display(MSDL_VER,"streaming still continues!!\n");

	  
		    /* write header to write buffer and return... */
		    if(asf_headerinfo->asf_header_len <= max_size) { /* everything can go... */
			memcpy(buffer,asf_headerinfo->asf_header,
			       asf_headerinfo->asf_header_len);
			ret = asf_headerinfo->asf_header_len;
		    }
		    else {
			memcpy(buffer,asf_headerinfo->asf_header,max_size);
			memcpy(stream_ctrl->write_buffer, asf_headerinfo->asf_header + max_size,
			       asf_headerinfo->asf_header_len - max_size);
			stream_ctrl->write_data_len = asf_headerinfo->asf_header_len - max_size;
			ret = max_size;
		    }
	  
		    /*free_asf_headerinfo_t(asf_headerinfo); this is done in free_rtsp_ctrl_t */
		    stream_ctrl->status = streaming_rewind;
	  
		    return ret; /* write asf header */
		}
	    }
      
	    /* if no ANNOUCNE, finish streaming */
	    display(MSDL_DBG,"finish rtsp streaming\n");
	    return 0; /* streaming really finished */
	}
	else {
	    goto failed;
	}
    }
  
  
    if(header[0] != 0x24) {
	int i;
	display(MSDL_ERR,"wrong rdt/rtp magic : [0x%02x]\n",header[0]);
	/* dump header*/
	for( i = 0; i < 8; i++) display(MSDL_VER,"[%02x] ",header[i]);
	display(MSDL_ERR,"\n");
	display(MSDL_ERR,"(");
	for( i = 0; i < 8; i++) display(MSDL_VER,"%c",header[i]);
	display(MSDL_ERR,")\n");
	goto failed;
    }
    size = (header[1] << 16) + (header[2] << 8) + header[3];

    flags1 = header[4];
    flags2 = header[5];
    cc = (flags1 & 0x0f);
  
    seq = get16_be((unsigned char *)&header + 2);
  
    read_data(stream,&header2,8);
    ts = get32_be((unsigned char *)&header2);
    ssrc = get32_be((unsigned char *)&header2 + 4);
    size -= 12;  
  
    for(i = 0 ; i < cc ; i++ ) {
	read_data(stream,(unsigned char *)&csrc,4);
	csrc = be2me_32(csrc);
	size -= 4;
    }
  

    /*
      DIRTY, but this seems to be the only way to donwload :D
    */
    for(;size > 0; size -= 4) { /* skip until magic(0x82) appears */
	read_data(stream,(unsigned char *)&magicheader,4);
	if(magicheader[0] == 0x82) break; /* magiq */
    }

    display(MSDL_DBG,"f1: %x, f2: %x, cc: %d, size: %d, ts: %u,"
	    "ssrc: %x, csrc :%x\n",flags1,flags2,cc,size,ts,ssrc,csrc);
    display(MSDL_DBG,"magic: [");
    for(i = 0 ; i < 4 ; i++) {
	display(MSDL_DBG,"%02x ",magicheader[i]);
    }
    display(MSDL_DBG,"]\n");
  
  
    stream_ctrl->packet_count++;
  
    /*
      we have to do this to show rtsp server that
      this client is still alive, otherwise server sends
      EOS pacekt...
    */ 
    if(!(stream_ctrl->packet_count % 200)) {
	struct rtsp_header_t *rtsp_hdr;
	rtsp_hdr = new_rtsp_header();
	rtsp_set_standard_fields(stream_ctrl->rtsp_ctrl,rtsp_hdr);
    
	rtsp_request_get_parameter(rtsp_hdr,stream_ctrl->url,NULL);
	rtsp_send_request(stream,rtsp_hdr);
	free_rtsp_header(rtsp_hdr);
    }
  

    /* store to buffer, or write_buffer */
    if(max_size > stream_ctrl->packet_length) {
	/* all data can go to buffer --> just do it!! */
	memcpy(buffer,&magicheader,4);
	n = read_data(stream,buffer + 4,size - 4);
	if(n <= 0) goto failed;
    
	memset(buffer + size,0,stream_ctrl->packet_length - size); /* padding */
	return stream_ctrl->packet_length;
    }
    else {
	/*
	  buffer is not enough.. copy max_size data to 
	  buffer and the rest goes to stream_ctrl->write_buffer.
	*/
	memcpy(stream_ctrl->write_buffer,&magicheader,4);
	n = read_data(stream,stream_ctrl->write_buffer + 4,size - 4);
	if(n <= 0) goto failed;
    
	/*
	  it is guranteed that stream_ctrl->write_buffer is
	  practically empty when it comes here!
	  --> reset stream_ctrl->write_pos and so on...
	*/
	if(stream_ctrl->packet_length < size) {
	    display(MSDL_ERR,"packet_length(%d) is smaller than size(%d)\n",
		    stream_ctrl->packet_length,size);
	    goto failed; /* this should not happen*/
	}
	memset(stream_ctrl->write_buffer + size,0,
	       stream_ctrl->packet_length - size); /* padding */
	memcpy(buffer,stream_ctrl->write_buffer,max_size);
    
	stream_ctrl->write_data_len = stream_ctrl->packet_length - max_size;
	stream_ctrl->write_pos += max_size;
    
	return max_size;
    }
  
  
  failed:
    return -1;
}

