/*******************************************************************************
**
** conn.c
**
** This file is part of the ABYSS Web server project.
**
** Copyright (C) 2000 by Moez Mahfoudh <mmoez@bigfoot.com>.
** All rights reserved.
**
** Redistribution and use in source and binary forms, with or without
** modification, are permitted provided that the following conditions
** are met:
** 1. Redistributions of source code must retain the above copyright
**    notice, this list of conditions and the following disclaimer.
** 2. Redistributions in binary form must reproduce the above copyright
**    notice, this list of conditions and the following disclaimer in the
**    documentation and/or other materials provided with the distribution.
** 3. The name of the author may not be used to endorse or promote products
**    derived from this software without specific prior written permission.
** 
** THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
** ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
** IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
** ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
** FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
** DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
** OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
** HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
** LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
** OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
** SUCH DAMAGE.
**
*******************************************************************************/

#include <time.h>
#include <string.h>
#include "abyss.h"
/*********************************************************************
** Conn
*********************************************************************/

TConn *ConnAlloc()
{
	return (TConn *)malloc(sizeof(TConn));
}

void ConnFree(TConn *c)
{
	free(c);
}

uint32 ConnJob(TConn *c)
{
	c->connected=TRUE;
	(c->job)(c);
	c->connected=FALSE;
	return 0;
}

int ConnCreate(TConn *c, TSocket *s, void (*func)(TConn *))
{
	//phillo
	//TConn c is initialized body at server.c/ServerRun()

	c->socket=*s;
	c->buffersize=c->bufferpos=0;
	c->connected=TRUE;
	c->job=func;
	c->inbytes=c->outbytes=0;

	//phillo,2007/11/2
	c->thread_dobulebuffer = 0;
	c->buffer_doublebuffer=0;
	c->write_pointer_doublebuffer=0;
	c->read_pointer_doublebuffer=0;
	c->lock=0;//CreateSemaphore(NULL,1,FD_SETSIZE,NULL);
	c->stop_thread = 0;
	c->thread_stat = STREAM_THREAD_FLAG_FIRST;

	return ThreadCreate(&(c->thread),(TThreadProc)ConnJob,c);
}

int ConnProcess(TConn *c)
{
/******* Must check this undef *****/
#ifndef _WIN32
	c->connected=FALSE;
#endif
	return ThreadRun(&(c->thread));
}

int ConnKill(TConn *c)
{
	c->connected=FALSE;
	return ThreadKill(&(c->thread));
}

void ConnReadInit(TConn *c)
{
	if (c->buffersize>c->bufferpos)
	{
		c->buffersize-=c->bufferpos;
		memmove(c->buffer,c->buffer+c->bufferpos,c->buffersize);
		c->bufferpos=0;
	}
	else
		c->buffersize=c->bufferpos=0;

	c->inbytes=c->outbytes=0;
}

int ConnRead(TConn *c,uint32 timeout)
{
	while (SocketWait(&(c->socket),TRUE,FALSE,timeout*1000)==1)
	{
		uint32 y,x=SocketAvailableReadBytes(&(c->socket));
		
		/* Avoid lost connections */
		if (x<=0)
			break;
		
		/* Avoid Buffer overflow and lost Connections */
		if (x+c->buffersize>=BUFFER_SIZE)
			x=BUFFER_SIZE-c->buffersize-1;
		
		if ((y=SocketRead(&(c->socket),c->buffer+c->buffersize,x))>0)
			{
				/*int i;
				char d;

				for (i=0;i<y;i++)
				{
					d=*(c->buffer+c->buffersize+i);
					if (d<32)
						printf("[%d]",d);
					else
						printf("%c",d);
				};
				printf("\n");*/

				c->inbytes+=y;
				c->buffersize+=y;
				c->buffer[c->buffersize]='\0';
				return TRUE;
			};

		break;
	};

	return FALSE;
}
			
int ConnWrite(TConn *c,void *buffer,uint32 size)
{
	c->outbytes+=size;
	return SocketWrite(&(c->socket),buffer,size);
}

//philllo,2007/10/21
int ConnWriteFromStream(TConn *c,HANDLE *h,uint64 start,uint64 end,
			void *buffer,uint32 buffersize,uint32 rate,char *filepath)
{	
	uint64 y;
	static uint64 bytesread=0;
	uint32 waittime;
	int r;
	static char s[100];
	char ipAddress[1000];
	int test_add = 0;

	if (rate>0)
	{
		if (buffersize>rate)
			buffersize=rate;

		waittime=(1000*buffersize)/rate;
	}
	else
		waittime=0;

//	if (!FileSeek(h,start,SEEK_SET))
//		return FALSE;

	while (bytesread<=end-start)
	{
		//phillo,2007/10/22
		//y=(end-start+1)-bytesread;

		//phillo,2007/10/22
		//if (y>buffersize)
			y=buffersize;

		y=StreamRead(c,h,buffer,y);
		bytesread+=y;

		if (test_add == 0)
		{					        
            sprintf(ipAddress,"%d.%d.%d.%d",c->peerip.S_un.S_un_b.s_b1,c->peerip.S_un.S_un_b.s_b2,c->peerip.S_un.S_un_b.s_b3,c->peerip.S_un.S_un_b.s_b4);
            AddNewTransStatus(filepath, ipAddress, end, filepath);
            test_add = 1;
        }
        ModifyTransStatus(filepath, ipAddress, start);

		if (y>0)
		{
			r = ConnWrite(c,buffer,y);
			
			if( r == -1 )
			{
				//phillo
				//sprintf(s,"send fail :  socket= %d\n",c->socket);
				//OutputDebugString(s);
				//SocketClose(&(c->socket));
				break;
			}
		}
		else
			break;

		if (waittime)
			ThreadWait(waittime);
	};

	test_add = 1;
	return (bytesread>end-start);

	

}

#include <FileSystemTree.h>
int ConnWriteFromFile(TConn *c,TFile *file,uint64 start,uint64 end,
			void *buffer,uint32 buffersize,uint32 rate,char *filepath)
{
	uint64 y,bytesread=0;
	uint32 waittime;
	int r;
	static char s[100];
	char ipAddress[1000];
	int test_add = 0;

    wchar_t tempStr[1000];
    char filename[1000];
    struct FSE_QueryState tmpFSE_QS;
    char *ptr, *ptr2;

	if (rate>0)
	{
		if (buffersize>rate)
			buffersize=rate;

		waittime=(1000*buffersize)/rate;
	}
	else
		waittime=0;

	if (!FileSeek(file,start,SEEK_SET))
		return FALSE;

	while (bytesread<=end-start)
	{
		y=(end-start+1)-bytesread;

		if (y>buffersize)
			y=buffersize;

		y=FileRead(file,buffer,y);
		bytesread+=y;

		//if (test_add == 0)
		{	
			//birdy 200711/20 ADD for connection status can show all right in the systempage

            ptr = filepath+7;
            strcpy(tempStr,ptr);			
            ptr2 = strchr(tempStr,'/');
            strcpy(filename,ptr2+1); 
			*ptr2 = NULL;
            

            sprintf(ipAddress,"%d.%d.%d.%d",c->peerip.S_un.S_un_b.s_b1,c->peerip.S_un.S_un_b.s_b2,c->peerip.S_un.S_un_b.s_b3,c->peerip.S_un.S_un_b.s_b4);
            //AddNewTransStatus(filepath, ipAddress, end, filepath);
			AddNewTransStatus(filename, ipAddress, end, filepath);
            test_add = 1;
        }
        ModifyTransStatus(filepath, ipAddress, bytesread + start);
		

		if (y>0)
		{
			r = ConnWrite(c,buffer,y);
			
			if( r == -1 )
			{
				//phillo
				//sprintf(s,"send fail :  socket= %d\n",c->socket);
				//OutputDebugString(s);
				//SocketClose(&(c->socket));
				break;
			}
		}
		else
			break;

		if (waittime)
			ThreadWait(waittime);
	};

	test_add = 1;
	return (bytesread>end-start);
}
#include <io.h>
int ConnWriteFromIsoFile(TConn *c,TFile *file,uint64 start,uint64 end,
			void *buffer,uint32 buffersize,uint32 rate,char *filepath,__int64 offset)
{
	uint64 y,bytesread=0;
	uint32 waittime;
	int r;
	static char s[100];
	char ipAddress[1000];
	int test_add = 0;

    wchar_t tempStr[1000];
    char filename[1000];
    struct FSE_QueryState tmpFSE_QS;
    char *ptr, *ptr2;
	long pos;
	if (rate>0)
	{
		if (buffersize>rate)
			buffersize=rate;

		waittime=(1000*buffersize)/rate;
	}
	else
		waittime=0;
	


	if(!FileSeek(file,offset,SEEK_SET))
		return FALSE;
	if(!FileSeek(file,start,SEEK_CUR))
		return FALSE;



	
	while (bytesread<=end-start)
	{
		y=(end-start+1)-bytesread;

		if (y>buffersize)
			y=buffersize;

		
		y=FileRead(file,buffer,y);
		bytesread+=y;

		//if (test_add == 0)
		{	
			//birdy 200711/20 ADD for connection status can show all right in the systempage

            ptr = filepath+7;
            strcpy(tempStr,ptr);			
            ptr2 = strchr(tempStr,'/');
            strcpy(filename,ptr2+1); 
			*ptr2 = NULL;
            

            sprintf(ipAddress,"%d.%d.%d.%d",c->peerip.S_un.S_un_b.s_b1,c->peerip.S_un.S_un_b.s_b2,c->peerip.S_un.S_un_b.s_b3,c->peerip.S_un.S_un_b.s_b4);
            //AddNewTransStatus(filepath, ipAddress, end, filepath);
			AddNewTransStatus(filename, ipAddress, end, filepath);
            test_add = 1;
        }
		//IsoFileSeek(file,-offset,SEEK_CUR,0);
        ModifyTransStatus(filepath, ipAddress, bytesread + start);
		

		if (y>0)
		{
			r = ConnWrite(c,buffer,y);
			
			if( r == -1 )
			{
				//phillo
				//sprintf(s,"send fail :  socket= %d\n",c->socket);
				//OutputDebugString(s);
				//SocketClose(&(c->socket));
				break;
			}
		}
		else
			break;

		if (waittime)
			ThreadWait(waittime);
	};

	test_add = 1;
	return (bytesread>end-start);
}
int ConnReadLine(TConn *c,char **z,uint32 timeout)
{
	char *p,*t;
	int first=TRUE;
	uint32 to,start;

	p=*z=c->buffer+c->bufferpos;
	start=clock();

	for (;;)
	{
		to=(clock()-start)/CLOCKS_PER_SEC;
		if (to>timeout)
			break;

		if (first)
		{
			if (c->bufferpos>=c->buffersize)
				if (!ConnRead(c,timeout-to))
					break;

			first=FALSE;
		}
		else
		{
			if (!ConnRead(c,timeout-to))
				break;
		};

		if (t=strchr(p,LF))
		{
			if ((*p!=LF) && (*p!=CR))
			{
				if (!*(t+1))
					continue;

				p=t;

				if ((*(p+1)==' ') || (*(p+1)=='\t'))
				{
					if (p>*z)
						if (*(p-1)==CR)
							*(p-1)=' ';

					*(p++)=' ';
					continue;
				};
			};

			c->bufferpos+=p+1-*z;

			if (p>*z)
				if (*(p-1)==CR)
					p--;

			*p='\0';
			return TRUE;
		}
	};

	return FALSE;
}
