/*******************************************************************************
**
** server.c
**
** This file is part of the ABYSS Web server project.
**
** Copyright (C) 2000 by Moez Mahfoudh <mmoez@bigfoot.com>.
** All rights reserved.
**
** Redistribution and use in source and binary forms, with or without
** modification, are permitted provided that the following conditions
** are met:
** 1. Redistributions of source code must retain the above copyright
**    notice, this list of conditions and the following disclaimer.
** 2. Redistributions in binary form must reproduce the above copyright
**    notice, this list of conditions and the following disclaimer in the
**    documentation and/or other materials provided with the distribution.
** 3. The name of the author may not be used to endorse or promote products
**    derived from this software without specific prior written permission.
** 
** THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
** ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
** IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
** ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
** FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
** DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
** OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
** HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
** LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
** OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
** SUCH DAMAGE.
**
*******************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#ifdef _WIN32
#include <io.h>
#else
/* Check this
#include <sys/io.h>
*/
#endif	/* _WIN32 */
#include <fcntl.h>
#include "abyss.h"

#include "mystring.h"
#include "MimeTypes.h"

#define BOUNDARY	"##123456789###BOUNDARY"

#define TX_BUFFER_SIZE (64*1024)

#include <FileSystemTree.h>
typedef int (*TQSortProc)(const void *, const void *);
//birdy 20074/12/24 add for Youtube
//extern int HTTP_download(char *url,char **buf, int *size);
int ConnWriteFromIsoFile(TConn *c,TFile *file,uint64 start,uint64 end,
			void *buffer,uint32 buffersize,uint32 rate,char *filepath,__int64 offset);
char *MIMETypeGuessFromIsoFile(char *filename);

int cmpfilenames(const TFileInfo **f1,const TFileInfo **f2)
{
	if (((*f1)->attrib & A_SUBDIR) && !((*f2)->attrib & A_SUBDIR))
		return (-1);
	if (!((*f1)->attrib & A_SUBDIR) && ((*f2)->attrib & A_SUBDIR))
		return 1;

	return strcmp((*f1)->name,(*f2)->name);
}

int cmpfiledates(const TFileInfo **f1,const TFileInfo **f2)
{
	if (((*f1)->attrib & A_SUBDIR) && !((*f2)->attrib & A_SUBDIR))
		return (-1);
	if (!((*f1)->attrib & A_SUBDIR) && ((*f2)->attrib & A_SUBDIR))
		return 1;

	return ((*f1)->time_write-(*f2)->time_write);
}

int ServerDirectoryHandler(TSession *r,char *z,TDate *dirdate)
{
	TFileInfo fileinfo,*fi;
	TFileFind findhandle;
	char *p,z1[26],z2[20],z3[9],u,*z4;
	TList list;
	int16 i;
	uint32 k;
	int text=FALSE;
	int ascending=TRUE;
	uint16 sort=1;	/* 1=by name, 2=by date */
	struct tm ftm;
	TPool pool;
	TDate date;

	if (r->query)
		if (strcmp(r->query,"plain")==0)
			text=TRUE;
		else if (strcmp(r->query,"name-up")==0)
		{
			sort=1;
			ascending=TRUE;
		}
		else if (strcmp(r->query,"name-down")==0)
		{
			sort=1;
			ascending=FALSE;
		}
		else if (strcmp(r->query,"date-up")==0)
		{
			sort=2;
			ascending=TRUE;
		}
		else if (strcmp(r->query,"date-down")==0)
		{
			sort=2;
			ascending=FALSE;
		}
		else 
		{
			ResponseStatus(r,400);
			return TRUE;
		};

	if (DateCompare(&r->date,dirdate)<0)
		dirdate=&r->date;

	if (p=RequestHeaderValue(r,"If-Modified-Since"))
		if (DateDecode(p,&date))
			if (DateCompare(dirdate,&date)<=0)
			{
				ResponseStatus(r,304);
				ResponseWrite(r);
				return TRUE;
			};

	if (!FileFindFirst(&findhandle,z,&fileinfo))
	{
		ResponseStatusErrno(r);
		return TRUE;
	};

	WebListInit(&list);

	if (!PoolCreate(&pool,1024))
	{
		ResponseStatus(r,500);
		return TRUE;
	};

	do
	{
		/* Files which names start with a dot are ignored */
		/* This includes implicitly the ./ and ../ */
		if (*fileinfo.name=='.')
			if (strcmp(fileinfo.name,"..")==0)
			{
				if (strcmp(r->uri,"/")==0)
					continue;
			}
			else
				continue;

		if (fi=(TFileInfo *)PoolAlloc(&pool,sizeof(fileinfo)))
		{
			memcpy(fi,&fileinfo,sizeof(fileinfo));
			if (ListAdd(&list,fi))
				continue;
		};

		ResponseStatus(r,500);
		FileFindClose(&findhandle);
		ListFree(&list);
		PoolFree(&pool);
		return TRUE;

	} while (FileFindNext(&findhandle,&fileinfo));

	FileFindClose(&findhandle);

	/* Send something to the user to show that we are still alive */
	ResponseStatus(r,200);
	ResponseContentType(r,(text?"text/plain":"text/html"));

	if (DateToString(dirdate,z))
		ResponseAddField(r,"Last-Modified",z);
	
	ResponseChunked(r);
	ResponseWrite(r);

	if (r->method!=m_head)
	{
		if (text)
		{
			sprintf(z,"Index of %s" CRLF,r->uri);
			i=strlen(z)-2;
			p=z+i+2;

			while (i>0)
			{
				*(p++)='-';
				i--;
			};

			*p='\0';
			strcat(z,CRLF CRLF
				"Name                      Size      Date-Time             Type" CRLF
				"--------------------------------------------------------------------------------"CRLF);
		}
		else
		{
			sprintf(z,"<HTML><HEAD><TITLE>Index of %s</TITLE></HEAD><BODY>"
					"<H1>Index of %s</H1><PRE>",r->uri,r->uri);
			strcat(z,"Name                      Size      Date-Time             Type<HR WIDTH=100%>"CRLF);
 		};

		HTTPWrite(r,z,strlen(z));

		/* Sort the files */
			qsort(list.item,list.size,sizeof(void *),(TQSortProc)(sort==1?cmpfilenames:cmpfiledates));
		
		/* Write the listing */
		if (ascending)
			i=0;
		else
			i=list.size-1;

		while ((i<list.size) && (i>=0))
		{
			fi=list.item[i];

			if (ascending)
				i++;
			else
				i--;
			
			strcpy(z,fi->name);

			k=strlen(z);

			if (fi->attrib & A_SUBDIR)
			{
				z[k++]='/';
				z[k]='\0';
			};

			if (k>24)
			{
				z[10]='\0';
				strcpy(z1,z);
				strcat(z1,"...");
				strcat(z1,z+k-11);
				k=24;
				p=z1+24;
			}
			else
			{
				strcpy(z1,z);
				
				k++;
				p=z1+k;
				while (k<25)
					z1[k++]=' ';

				z1[25]='\0';
			};

			ftm=*(gmtime(&fi->time_write));
			sprintf(z2,"%02u/%02u/%04u %02u:%02u:%02u",ftm.tm_mday,ftm.tm_mon,
				ftm.tm_year+1900,ftm.tm_hour,ftm.tm_min,ftm.tm_sec);

			if (fi->attrib & A_SUBDIR)
			{
				strcpy(z3,"   --  ");
				z4="Directory";
			}
			else
			{
				if (fi->size<9999)
					u='b';
				else 
				{
					fi->size/=1024;
					if (fi->size<9999)
						u='K';
					else
					{
						fi->size/=1024;
						if (fi->size<9999)
							u='M';
						else
							u='G';
					};
				};

				sprintf(z3,"%5lu %c",fi->size,u);

				if (strcmp(fi->name,"..")==0)
					z4="";
				else
					z4=MIMETypeFromFileName(fi->name);

				if (!z4)
					z4="Unknown";
			};

			if (text)
				sprintf(z,"%s%s %s    %s   %s"CRLF,
					z1,p,z3,z2,z4);
			else
				sprintf(z,"<A HREF=\"%s%s\">%s</A>%s %s    %s   %s"CRLF,
					fi->name,(fi->attrib & A_SUBDIR?"/":""),z1,p,z3,z2,z4);

			HTTPWrite(r,z,strlen(z));
		};
		
		/* Write the tail of the file */
		if (text)
			strcpy(z,SERVER_PLAIN_INFO);
		else
			strcpy(z,"</PRE>" SERVER_HTML_INFO "</BODY></HTML>" CRLF CRLF);

		HTTPWrite(r,z,strlen(z));
	};
	
	HTTPWriteEnd(r);
	/* Free memory and exit */
	ListFree(&list);
	PoolFree(&pool);

	return TRUE;
}

//phillo,2007/10/21
int ServerStreamHandler(TSession *r,char *z)
{
	char *mediatype;
	uint64 filesize = 0,start=0,end=0;			
	int rr ;
	HANDLE h;
	

	rr = StreamOpen(&h,r,z);

	if( rr != TRUE )
	{
		//ResponseStatusErrno(r);
		//ResponseStatusErrno(r,404);
		ResponseStatus(r,500);
		//ResponseError(r);
		ResponseWrite(r);

		//phillo,2008/1/17
		return FALSE;
	};


	switch (r->ranges.size)
	{
	case 0:
		ResponseStatus(r,200);
		break;

	case 1:
		if (!RangeDecode((char *)(r->ranges.item[0]),filesize,&start,&end))
		{
			ListFree(&(r->ranges));
			ResponseStatus(r,200);
			break;
		}
		
		if( start == 0 )
		{
			//phillo
			//sprintf(z,"bytes %ld-%ld/%ld",start,end,filesize);
			sprintf(z,"bytes %I64d-%I64d/%I64d",start,end,filesize);

			ResponseAddField(r,"Content-range",z);
			ResponseContentLength(r,end-start+1);
			ResponseStatus(r,206);
		}
		else
		{
			ResponseStatus(r,501); 

		}
		break;

	default:
		ResponseContentType(r,"multipart/ranges; boundary=" BOUNDARY);
		ResponseStatus(r,501); //206);
		break;
	};


	//ResponseStatus(r,200);

			//only for free box
		if( strncmp(z,"rtsp://mafreebox",16 ) == 0 )
		{
				mediatype = MIME_TYPE_VIDEO_MPEG;
	
		}
		else
		{
			mediatype = MIME_TYPE_VIDEO_AVI; 
		}
		ResponseContentType(r,mediatype);


	ResponseWrite(r);


	ConnWriteFromStream(r->conn,&h,0,filesize-1,z,TX_BUFFER_SIZE,0,r->uri);


	StreamClose(r->conn,&h);


	return TRUE;
}


int ServerFileHandler(TSession *r,char *z,TDate *filedate)
{
	char *mediatype;
	TFile file;
	uint64 filesize,start=0,end=0;
	uint16 i;
	TDate date;
	char *p;
	int rr ;
	
	mediatype=MIMETypeGuessFromFile(z);

	rr = FileOpenUTF8(&file,z,O_BINARY | O_RDONLY);

	if( rr != TRUE )
	{
		//ResponseStatusErrno(r);
		ResponseStatusErrno(r,404);
		ResponseError(r);
		return TRUE;
	};

	if (DateCompare(&r->date,filedate)<0)
		filedate=&r->date;

	if (p=RequestHeaderValue(r,"if-modified-since"))
		if (DateDecode(p,&date))
			if (DateCompare(filedate,&date)<=0)
			{
				ResponseStatus(r,304);
				ResponseWrite(r);
				return TRUE;
			}
			else
				r->ranges.size=0;

	filesize=FileSize(&file);

	switch (r->ranges.size)
	{
	case 0:
		ResponseStatus(r,200);
		break;

	case 1:
		if (!RangeDecode((char *)(r->ranges.item[0]),filesize,&start,&end))
		{
			ListFree(&(r->ranges));
			ResponseStatus(r,200);
			break;
		}
		
		//phillo
		//sprintf(z,"bytes %ld-%ld/%ld",start,end,filesize);
		sprintf(z,"bytes %I64d-%I64d/%I64d",start,end,filesize);

		ResponseAddField(r,"Content-range",z);
		ResponseContentLength(r,end-start+1);
		ResponseStatus(r,206);
		break;

	default:
		ResponseContentType(r,"multipart/ranges; boundary=" BOUNDARY);
		ResponseStatus(r,206);
		break;
	};
	
	if (r->ranges.size==0)
	{
		ResponseContentLength(r,filesize);
		ResponseContentType(r,mediatype);
	};
	
	if (DateToString(filedate,z))
		ResponseAddField(r,"Last-Modified",z);

	ResponseWrite(r);

	if (r->method!=m_head)
	{
		if (r->ranges.size==0)
			ConnWriteFromFile(r->conn,&file,0,filesize-1,z,TX_BUFFER_SIZE,0,r->uri);
		else if (r->ranges.size==1)
			ConnWriteFromFile(r->conn,&file,start,end,z,TX_BUFFER_SIZE,0,r->uri);
		else
			for (i=0;i<=r->ranges.size;i++)
			{
				ConnWrite(r->conn,"--",2);
				ConnWrite(r->conn,BOUNDARY,strlen(BOUNDARY));
				ConnWrite(r->conn,CRLF,2);

				if (i<r->ranges.size)
					if (RangeDecode((char *)(r->ranges.item[i]),filesize,&start,&end))
					{

						//phillo
						/* Entity header, not response header */
//						sprintf(z,"Content-type: %s" CRLF
//									"Content-range: bytes %ld-%ld/%ld" CRLF
//									"Content-length: %lu" CRLF
//									CRLF,mediatype,start,end,filesize,end-start+1);


						sprintf(z,"Content-type: %s" CRLF
									"Content-range: bytes %I64d-%I64d/%I64d" CRLF
									"Content-length: %I64u" CRLF
									CRLF,mediatype,start,end,filesize,end-start+1);


						ConnWrite(r->conn,z,strlen(z));

						ConnWriteFromFile(r->conn,&file,start,end,z,TX_BUFFER_SIZE,0,r->uri);
					};
			};
	};

	FileClose(&file);

	return TRUE;
}


int ServerIsoFileHandler(TSession *r,char *z,TDate *filedate,long filesize,__int64 *offset)
{
	char *mediatype;
	TFile file;
	uint64 /*filesize,*/start=0,end=0;
	uint16 i;
	TDate date;
	char *p;
	int rr ;
	
	mediatype=MIMETypeGuessFromIsoFile(z);

	rr = IsoFileOpenUTF8(&file,z,O_BINARY | O_RDONLY,offset);

	if( rr != TRUE )
	{
		//ResponseStatusErrno(r);
		ResponseStatusErrno(r,404);
		ResponseError(r);
		return TRUE;
	};

	if (DateCompare(&r->date,filedate)<0)
		filedate=&r->date;

	if (p=RequestHeaderValue(r,"if-modified-since"))
		if (DateDecode(p,&date))
			if (DateCompare(filedate,&date)<=0)
			{
				ResponseStatus(r,304);
				ResponseWrite(r);
				return TRUE;
			}
			else
				r->ranges.size=0;

//	filesize=IsoFileSize(&file);
//	filesize=423198720;
	switch (r->ranges.size)
	{
	case 0:
		ResponseStatus(r,200);
		break;

	case 1:
		if (!RangeDecode((char *)(r->ranges.item[0]),filesize,&start,&end))
		{
			ListFree(&(r->ranges));
			ResponseStatus(r,200);
			break;
		}
		
		//phillo
		//sprintf(z,"bytes %ld-%ld/%ld",start,end,filesize);
		sprintf(z,"bytes %I64d-%I64d/%I64d",start,end,filesize);

		ResponseAddField(r,"Content-range",z);
		if(start >=*offset+filesize)
		{
//			start-=*offset;

			start = start;
		}

		ResponseContentLength(r,end-start+1);
		ResponseStatus(r,206);
		break;

	default:
		ResponseContentType(r,"multipart/ranges; boundary=" BOUNDARY);
		ResponseStatus(r,206);
		break;
	};
	
	if (r->ranges.size==0)
	{
		ResponseContentLength(r,filesize);
		ResponseContentType(r,mediatype);
	};
	
	if (DateToString(filedate,z))
		ResponseAddField(r,"Last-Modified",z);

	ResponseWrite(r);

	if (r->method!=m_head)
	{
		if (r->ranges.size==0)
			ConnWriteFromIsoFile(r->conn,&file,0,filesize-1,z,TX_BUFFER_SIZE,0,r->uri,*offset);
		else if (r->ranges.size==1)
			ConnWriteFromIsoFile(r->conn,&file,start,end,z,TX_BUFFER_SIZE,0,r->uri,*offset);
		else
			for (i=0;i<=r->ranges.size;i++)
			{
				ConnWrite(r->conn,"--",2);
				ConnWrite(r->conn,BOUNDARY,strlen(BOUNDARY));
				ConnWrite(r->conn,CRLF,2);

				if (i<r->ranges.size)
					if (RangeDecode((char *)(r->ranges.item[i]),filesize,&start,&end))
					{

						//phillo
						/* Entity header, not response header */
//						sprintf(z,"Content-type: %s" CRLF
//									"Content-range: bytes %ld-%ld/%ld" CRLF
//									"Content-length: %lu" CRLF
//									CRLF,mediatype,start,end,filesize,end-start+1);


						sprintf(z,"Content-type: %s" CRLF
									"Content-range: bytes %I64d-%I64d/%I64d" CRLF
									"Content-length: %I64u" CRLF
									CRLF,mediatype,start,end,filesize,end-start+1);


						ConnWrite(r->conn,z,strlen(z));

						ConnWriteFromIsoFile(r->conn,&file,start,end,z,TX_BUFFER_SIZE,0,r->uri,*offset);
					};
			};
	};

	FileClose(&file);

	return TRUE;
}
//robert,2008/02/19
int IsIsoStream(char *s,char *dst,__int64 *filesize)
{
			char *ptr,*ptr2,tempStr[1000],strSQL[1000], filename[1000];
			struct FSE_QueryState tmpFSE_QS;
			int findex;
			ptr = s+7; // skip /0/___/
			//fPos=LastIndexOf(r.uri,"/");
			strcpy(tempStr,ptr);
			ptr2 = strchr(tempStr,'/');
			strcpy(filename,ptr2+1); // skip 1st /
			//tempStr[fPos]='\0';
			*ptr2 = NULL;
			//fPos=LastIndexOf(tempStr,"/");
			//findex=atoi(tempStr+fPos+1);
			findex = atoi(tempStr);

			//Utf8ToStr(FilePath, (char *) r.uri, 1000);
			sprintf(strSQL,"select * from FSE_QueryState where MyIndex = %d",findex);
            db_getOneRecord(strSQL,&tmpFSE_QS);
			if(tmpFSE_QS.Flags && 0x100)
			{
				wchar_t *ptr=(wchar_t*)tmpFSE_QS.FilePath;
				*filesize = tmpFSE_QS.Size;
				
				//TCHAR dst;
				//Utf8ToStr(dst,ptr,1000);
				strcpy(dst,tmpFSE_QS.FilePath);

				
				return 1;
			}else
				return 0;
}
//phillo,2007/10/21
int IsInternetStream(char *s)
{
	if( !strnicmp(s,"http",4))
		return TRUE;

	if( !strnicmp(s,"mms",3))
		return TRUE;

	if( !strnicmp(s,"mmsh",4))
		return TRUE;
	
	if( !strnicmp(s,"rtsp",4))
		return TRUE;

	return FALSE;
}


int ServerDefaultHandlerFunc(TSession *r)
{
	//phillo,2008/1/16
	char *p = NULL,*z;//[TX_BUFFER_SIZE];
	TFileStat fs;
	uint16 i;
	int endingslash=FALSE;
	TDate objdate;
	int mIsInternetStream=FALSE;
	int mIsIsoStream=FALSE;
	int rr;
	__int64 filesize,fileoffset;
	z = (char *)malloc(TX_BUFFER_SIZE);
	if( z == NULL )
	{
		return FALSE;
	}

	z[0] = 0;

	//phillo,2007/11/6 , we don't need to check through UPnP Protocol
/*
	if (!RequestValidURIPath(r))
	{
		ResponseStatus(r,400);
		return TRUE;
	};
*/
	 /*Must check for * (asterisk uri) in the future */
	if (r->method==m_options)
	{
		ResponseAddField(r,"Allow","GET, HEAD");
		ResponseContentLength(r,0);
		ResponseStatus(r,200);
		//phillo,2008/1/16
		if(z)
		{
			free(z);
		}
		return TRUE;
	};

	if ((r->method!=m_get) && (r->method!=m_head))
	{
		ResponseAddField(r,"Allow","GET, HEAD");
		ResponseStatus(r,405);
		//phillo,2008/1/16
		if(z)
		{
			free(z);
		}
		return TRUE;
	};

	strcpy(z,r->server->filespath);
	//strcat(z,r->uri);

	/*p=z+strlen(z)-1;
	if (*p=='/')
	{
		endingslash=TRUE;
		*p='\0';
	};

#ifdef _WIN32
	p=z;
	while (*p)
	{
		if ((*p)=='/')
			*p='\\';

		p++;
	};
#endif	/* _WIN32 */


	mIsIsoStream = IsIsoStream(r->uri,z,&filesize);

	if(mIsIsoStream)
	{

		ServerIsoFileHandler(r,z,&objdate,filesize,&fileoffset);
		if(z)
		{
			free(z);
		}
		return TRUE;
	}

	mIsInternetStream = IsInternetStream(z);

	if( mIsInternetStream )
	{
		// do something
		memset(&fs,0,sizeof(fs));
	}
	else
	{
		if (!FileStat(z,&fs))
		{
			ResponseStatusErrno(r);
			ResponseError(r);
			//phillo,2008/1/16
			if(z)
			{
				free(z);
			}
			return TRUE;
		};
	}

	if (fs.st_mode & S_IFDIR)
	{
		/* Redirect to the same directory but with the ending slash
		** to avoid problems with some browsers (IE for examples) when
		** they generate relative urls */
		if (!endingslash)
		{
			strcpy(z,r->uri);
			p=z+strlen(z);
			*p='/';
			*(p+1)='\0';
			ResponseAddField(r,"Location",z);
			ResponseStatus(r,302);
			ResponseWrite(r);
			//phillo,2008/1/16
			if(z)
			{
				free(z);
			}
			return TRUE;
		};

#ifdef _WIN32
		*p='\\';
#else
		*p='/';
#endif	/* _WIN32 */
		p++;

		i=r->server->defaultfilenames.size;
		while (i-->0)
		{
  			*p='\0';		
  			strcat(z,(char *)(r->server->defaultfilenames.item[i]));
  			if (FileStat(z,&fs))
  				if (!(fs.st_mode & S_IFDIR))
					if (DateFromLocal(&objdate,fs.st_mtime))
					{
						rr = ServerFileHandler(r,z,&objdate);
						//phillo,2008/1/16
						if(z)
						{
							free(z);
						}

						return rr;
					}
					else
					{
						rr =  ServerFileHandler(r,z,NULL);
						//phillo,2008/1/16
						if(z)
						{
							free(z);
						}

						return rr;
					}
		};

		*(p-1)='\0';
		
		if (!FileStat(z,&fs))
		{
			ResponseStatusErrno(r);
			//phillo,2008/1/16
			if(z)
			{
				free(z);
			}
			return TRUE;
		};

		if (DateFromLocal(&objdate,fs.st_mtime))
		{
			rr = ServerDirectoryHandler(r,z,&objdate);
			//phillo,2008/1/16
			if(z)
			{
				free(z);
			}

			return rr;
		}
		else
		{
			rr = ServerDirectoryHandler(r,z,NULL);
			//phillo,2008/1/16
			if(z)
			{
				free(z);
			}

			return rr;
		}

	}
	else
	{
		if( mIsInternetStream )
		{
			//phillo,2008/1/17
			rr = ServerStreamHandler(r,z);
			//phillo,2008/1/16
			if(z)
			{
				free(z);
			}

			return rr;
		}
		else
		{
			if (DateFromLocal(&objdate,fs.st_mtime))
			{
				rr  =  ServerFileHandler(r,z,&objdate);
				//phillo,2008/1/16
				if(z)
				{
					free(z);
				}

				return rr;
			}
			else
			{
				rr = ServerFileHandler(r,z,NULL);
				//phillo,2008/1/16
				if(z)
				{
					free(z);
				}

				return rr;
			}
		}
	}

	//phillo,2008/1/16
	if(z)
	{
		free(z);
	}

	//phillo,2008/1/17
	return FALSE;
}

int ServerCreate(TServer *srv,char *name,uint16 port,char *filespath,
				  char *logfilename)
{
	srv->name=strdup(name);
	srv->port=port;
	srv->defaulthandler=ServerDefaultHandlerFunc;
	srv->filespath=strdup(filespath);
	srv->keepalivetimeout=15;
	srv->keepalivemaxconn=30;
	srv->timeout=15;
	srv->advertise=TRUE;
#ifdef _UNIX
	srv->pidfile=srv->uid=srv->gid=(-1);
#endif	/* _UNIX */

	WebListInit(&srv->handlers);
	WebListInitAutoFree(&srv->defaultfilenames);

	if (logfilename)
		return LogOpen(srv,logfilename);
	else
	{
		srv->logfile=(-1);
		return TRUE;
	};
}

void ServerFree(TServer *srv)
{
	free(srv->name);
	free(srv->filespath);
	ListFree(&srv->handlers);
	WebListInitAutoFree(&srv->defaultfilenames);
	LogClose(srv);
}
#include <FileSystemTree.h>
void ServerFunc(TConn *c)
{	
	TSession r;
	uint32 i = 0,ka;
	int treated;
	URIHandler *hl=(URIHandler *)(c->server)->handlers.item;

	char strSQL[1000];
	char tempStr[1000], filename[1000], filetype[1000], new_filepath[1000], picture_name[1000]; 
	struct FSE_QueryState tmpFSE_QS;
	int findex, j = 0 , k = 0, pos;

	int phase=0;
	int lasti = 0;
	char c2[1000], str1[1000];
	char *buf;
	int bufsize, r1;
	char internet_str[1000] = "http://www.youtube.com/get_video?video_id=";
	int rr = FALSE;
    
	ka=c->server->keepalivemaxconn;

	while (ka--)
	{
		RequestInit(&r,c);

		/* Wait to read until timeout */
		if (!ConnRead(c,c->server->keepalivetimeout))
			break;

		if (RequestRead(&r))
		{
			char *ptr,*ptr2;
			ptr = r.uri+7; // skip /0/___/
			//fPos=LastIndexOf(r.uri,"/");
			strcpy(tempStr,ptr);
			ptr2 = strchr(tempStr,'/');
			strcpy(filename,ptr2+1); // skip 1st /
			//tempStr[fPos]='\0';
			*ptr2 = NULL;
			//fPos=LastIndexOf(tempStr,"/");
			//findex=atoi(tempStr+fPos+1);
			findex = atoi(tempStr);

			//Utf8ToStr(FilePath, (char *) r.uri, 1000);
			sprintf(strSQL,"select * from FSE_QueryState where MyIndex = %d",findex);
            db_getOneRecord(strSQL,&tmpFSE_QS);
			
			//phillo,2007/10/22
			memcpy(r.MediaFileType ,  tmpFSE_QS.FileType ,10 );
			
			r.server->filespath = tmpFSE_QS.FilePath; // Video, Music , Image

            //birdy 2007/12/24 add for Youtube
			//if ( (tmpFSE_QS.pIndex >= 26) && (tmpFSE_QS.pIndex <= 32) )
			if ( ((tmpFSE_QS.FilePath[11] == 'y') && (tmpFSE_QS.FilePath[12] == 'o') && (tmpFSE_QS.FilePath[13] == 'u') && (tmpFSE_QS.FilePath[14] == 't') && (tmpFSE_QS.FilePath[15] == 'u') && (tmpFSE_QS.FilePath[16] == 'b') && (tmpFSE_QS.FilePath[17] == 'e')) || ((tmpFSE_QS.FilePath[7] == 'y') && (tmpFSE_QS.FilePath[8] == 'o') && (tmpFSE_QS.FilePath[9] == 'u') && (tmpFSE_QS.FilePath[10] == 't') && (tmpFSE_QS.FilePath[11] == 'u') && (tmpFSE_QS.FilePath[12] == 'b') && (tmpFSE_QS.FilePath[13] == 'e')) )
			{
				r1 = HTTP_download_1(r.server->filespath, &buf, &bufsize);

				if (r1 == 0)
				{
                    while( i < bufsize)
                    {
                        //player2.swf?v=1
						//if ( (buf[i] == 'p') && ( buf[i + 1] == 'l') && ( buf[i + 2] == 'a') && ( buf[i + 3] == 'y') && ( buf[i + 4] == 'e') && ( buf[i + 5] == 'r') && ( buf[i + 6] == '2') && ( buf[i + 7] == '.') && ( buf[i + 8] == 's') && ( buf[i + 9] == 'w') && ( buf[i + 10] == 'f') && ( buf[i + 11] == '?') )
						//var swfArgs
                        if ( (buf[i] == 'v') && ( buf[i + 1] == 'a') && ( buf[i + 2] == 'r') && ( buf[i + 3] == ' ') && ( buf[i + 4] == 's') && ( buf[i + 5] == 'w') && ( buf[i + 6] == 'f') && ( buf[i + 7] == 'A') && ( buf[i + 8] == 'r') && ( buf[i + 9] == 'g') && ( buf[i + 10] == 's') )
                        {
                            i = i + 58;
                            j = 0;
                            phase  = 0;            
                            //video_id=
							//"video_id": "
                            while ( phase == 0)
                            {
                                //if ( (buf[i] == 'v') && ( buf[i + 1] == 'i') && ( buf[i + 2] == 'd') && ( buf[i + 3] == 'e') && ( buf[i + 4] == 'o') && ( buf[i + 5] == '_') && ( buf[i + 6] == 'i') && ( buf[i + 7] == 'd') && ( buf[i + 8] == '=') )
								if ( (buf[i] == '"') && ( buf[i + 1] == 'v') && ( buf[i + 2] == 'i') && ( buf[i + 3] == 'd') && ( buf[i + 4] == 'e') && ( buf[i + 5] == 'o') )//&& ( buf[i + 6] == '_') && ( buf[i + 7] == 'i') && ( buf[i + 8] == 'd') && ( buf[i + 9] == '"') && ( buf[i + 10] == ':') && ( buf[i + 11] == ' ') && ( buf[i + 12] == '"'))
                                {
                                    i = i + 13;
                                    				
                                    while( 1 )
                                    {                                        
										//", "l": 
                                        //if ( ( buf[i] == '&') && ( buf[i + 1] == 's') && ( buf[i + 2] == 'k') )
										if ( ( buf[i] == '"') && ( buf[i + 1] == ',') )//&& ( buf[i + 2] == ' ') && ( buf[i + 3] == '"') && ( buf[i + 4] == 'l') && ( buf[i + 5] == '"') && ( buf[i + 6] == ' '))
                                        {
											i = i + 7;
                                            break;
                                        }
										if ( i >= bufsize)
                                        {
                                            break;
                                        }
										c2[j] = buf[i];
                                        j++;
                                        i++;
                                    }
									c2[j] = '&';
									j++;
									c2[j] = 'l';
									j++;
									c2[j] = '=';
									j++;
									//", "l": 
									i = i + 1;
									while ( buf[i] != ',')
									{
										c2[j] = buf[i];
                                        j++;
										i++;
									}

									//"t"
									while( 1 )
									{
										if ( ( buf[i] == '"') && ( buf[i + 1] == 't') && ( buf[i + 2] == '"') )
										{
											i = i + 6;
											break;
										}
										i++;
									}
									
									c2[j] = '&';
									j++;
									c2[j] = 't';
									j++;
									c2[j] = '=';
									j++;

									while ( buf[i] != '"')
									{
										c2[j] = buf[i];
                                        j++;
										i++;
									}
									
                                    c2[j] = 0;
                                    strcpy(str1, internet_str);
                                    strcat(str1, c2);
                                    r.server->filespath = str1;
                                    phase = 1;
                                }
                                i++;

								if ( i >= bufsize)
								{
									break;
								}
                            }
                            i++;
                            if (phase == 1)
                            {
                                break;
                            }
                        }
                        i++;
                    }
				}
				free(buf);
			}

			//
			//birdy 2007/12/18 add for CD_Cover
			pos = LastIndexOf(r.uri, ".");
			for ( j = pos; j < strlen(r.uri); j++)
            {
				filetype[k] = r.uri[j];
                k++;
            }
			filetype[k] = 0;

			k = 0;
			pos = LastIndexOf(r.uri, "/");
			for ( j = pos + 1; j < strlen(r.uri); j++)
            {
				picture_name[k] = r.uri[j];
                k++;
            }
			picture_name[k] = 0;

			for (j = 0; j < strlen(filetype); j++)
            {
                filetype[j] = tolower(filetype[j]);
            }
			
			if ( (strcmp(filetype, ".jpg") == 0) || (strcmp(filetype, ".jpeg") == 0) || (strcmp(filetype, ".bmp") == 0) || (strcmp(filetype, ".png") == 0) )
			{
                pos = LastIndexOf(tmpFSE_QS.FilePath, "\\");
				strncpy(new_filepath, tmpFSE_QS.FilePath, pos);
				new_filepath[pos] = 0;
				strcat(new_filepath, "\\");				
				strcat(new_filepath, picture_name);
				r.server->filespath = new_filepath;
			}

			//

			/* Check if it is the last keepalive */
			if (ka==1)
				r.keepalive=FALSE;

			r.cankeepalive=r.keepalive;
			r.cankeepalive=1;

			if (r.status==0)
				if (r.versionmajor>=2)
					ResponseStatus(&r,505);
				else if (!RequestValidURI(&r))
					ResponseStatus(&r,400);
				else
				{
					i=c->server->handlers.size;
					treated=FALSE;

					while (i>0)
					{
						i--;
						if ((hl[i])(&r))
						{
							treated=TRUE;
							break;
						}
					}

					if (!treated)
					{
						//phillo,2008/1/17
						rr = ((URIHandler)(c->server->defaulthandler))(&r);
						if( rr == FALSE )
						{
							break;
						}
					}
			    }
		}
       		
		HTTPWriteEnd(&r);

		/*if (!r.done)
			ResponseError(&r);*/

		SessionLog(&r);

		if (!(r.keepalive && r.cankeepalive))
			break;

		/**************** Must adjust the read buffer *****************/
		ConnReadInit(c);		
	};

	RequestFree(&r);

	SocketClose(&(c->socket));
}

void ServerInit(TServer *srv)
{
	/********* Must check errors from these functions *************/
	if (!SocketInit())
		TraceExit("Can't initialize TCP sockets\n");;

	if (!SocketCreate(&srv->listensock))
		TraceExit("Can't create a socket\n");;

	if (!SocketBind(&srv->listensock,NULL,srv->port))
		TraceExit("Can't bind\n");

	if (!SocketListen(&srv->listensock,MAX_CONN))
		TraceExit("Can't listen\n");
};


#ifdef _FORK
void ServerRun(TServer *srv)
{
	TSocket s,ns;
	TConn c;
	pid_t r;
	TIPAddr ip;

	s=srv->listensock;

	while (1)
	{
		if (SocketAccept(&s,&ns,&ip))
		{
			switch (fork())
			{
			case 0:
				SocketClose(&s);
				c.socket=ns;
				c.buffersize=c.bufferpos=0;
				c.inbytes=c.outbytes=0;
				c.server=srv;
				c.peerip=ip;
				ServerFunc(&c);
			 	exit(0);
			case (-1):
				TraceExit("Fork Error");
			};
			SocketClose(&ns);
		}
		else
			TraceMsg("Socket Error=%d\n", SocketError());
	};
}
#else
void ServerRun(TServer *srv)
{
	uint32 i;
	TSocket s,ns;
	TIPAddr ip;
	TConn c[MAX_CONN];

	for (i=0;i<MAX_CONN;i++)
	{
		c[i].connected=FALSE;
		c[i].server=srv;
	};

	s=srv->listensock;

	while (1)
	{
		for (i=0;i<MAX_CONN;i++)
			if (!c[i].connected)
				break;

		if (i==MAX_CONN)
		{
			ThreadWait(2000);
			continue;
		};

		if (SocketAccept(&s,&ns,&ip))
		{
			if (ConnCreate(c+i,&ns,&ServerFunc))
			{
				c[i].peerip=ip;
				c[i].connected=TRUE;
				ConnProcess(c+i);
			}
			else
				SocketClose(&ns);
		}
		else
			TraceMsg("Socket Error=%d\n", SocketError());
	};
}
#endif	/* _FORK */




int ServerAddHandler(TServer *srv,URIHandler handler)
{
	return ListAdd(&srv->handlers,handler);
}

void ServerDefaultHandler(TServer *srv,URIHandler handler)
{
	srv->defaulthandler=handler;
}

int LogOpen(TServer *srv, char *filename)
{
	if (FileOpenCreate(&(srv->logfile),filename,O_WRONLY | O_APPEND))
		if (MutexCreate(&(srv->logmutex)))
			return TRUE;
		else
		{
			FileClose(&(srv->logfile));
			srv->logfile=(-1);
		};

	TraceMsg("Can't open log file '%s'",filename);
	return FALSE;
}

void LogWrite(TServer *srv,char *c)
{
	if ((srv->logfile)==(-1))
		return;

	if (!MutexLock(&(srv->logmutex)))
		return;

	FileWrite(&(srv->logfile),c,strlen(c));
	FileWrite(&(srv->logfile),LBR,strlen(LBR));

	MutexUnlock(&(srv->logmutex));
}

void LogClose(TServer *srv)
{
	if ((srv->logfile)==(-1))
		return;

	FileClose(&(srv->logfile));
	MutexFree(&(srv->logmutex));
}

int SessionLog(TSession *s)
{
	char z[1024];
	uint32 n;

	if (strlen(s->requestline)>1024-26-50)
		s->requestline[1024-26-50]='\0';

	n=sprintf(z,"%d.%d.%d.%d - %s - [",s->conn->peerip.S_un.S_un_b.s_b1,
		s->conn->peerip.S_un.S_un_b.s_b2,s->conn->peerip.S_un.S_un_b.s_b3,
		s->conn->peerip.S_un.S_un_b.s_b4,(s->user?s->user:""));

	DateToLogString(&s->date,z+n);

	sprintf(z+n+26,"] \"%s\" %d %d",s->requestline,s->status,s->conn->outbytes);

	LogWrite(s->server,z);
	return TRUE;
}
